// BehaviorModule.cpp: class implementation
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "ClientOrServer.h"
#if ( ROBOT_SERVER == 1 )	// This module used for Robot Server only

#include <math.h>
//#include <MMSystem.h>	// For Sound functions
#include "Globals.h"
#include "module.h"
#include "thread.h"
#include "HardwareConfig.h"

//#include "LaserScannerBehavior.h"


#define DEBUG_ARM_MOVEMENT		0
// for testing
#define HUMAN_FOLLOW_DISTANCE_TARGET_TENTH_INCHES	( 36 * 10 ) //TenthInches
#define HUMAN_FOLLOW_DISTANCE_MIN_TENTH_INCHES		( 24 * 10 ) //TenthInches
//#define HUMAN_FOLLOW_DISTANCE_MAX_TENTH_INCHES		( 36 * 10 ) //TenthInches

//#define HUMAN_FOLLOW_DISTANCE_TARGET_TENTH_INCHES	( 36 * 10 ) //TenthInches
//#define HUMAN_FOLLOW_DISTANCE_MIN_TENTH_INCHES		( 30 * 10 ) //TenthInches
//#define HUMAN_FOLLOW_DISTANCE_MAX_TENTH_INCHES		( 48 * 10 ) //TenthInches
///////////////////////////////////////////////////////////////////////////////
// Macros and defines

#define BLINK_CHECK_TIME 8.0 //how often to blink, in number of Sensor updates from PIC

__itt_string_handle* pshKinectPickupObjectStart = __itt_string_handle_create("KinectPickupObjectStart"); // marker
__itt_string_handle* pshKinectPickupObjectSearchComplete = __itt_string_handle_create("KinectPickupObjectSearchComplete"); // marker
__itt_string_handle* pshKinectPickupObjectTrackingStart = __itt_string_handle_create("KinectPickupObjectTrackingStart"); // marker
__itt_string_handle* pshKinectPickupNoObject = __itt_string_handle_create("KinectPickupNoObject"); // marker
__itt_string_handle* pshKinectPickupObjectLost = __itt_string_handle_create("KinectPickupObjectLost"); // marker
__itt_string_handle* pshKinectArrivedAtObject = __itt_string_handle_create("KinectArrivedAtObject"); // marker
__itt_string_handle* pshKinectMoveToObject = __itt_string_handle_create("KinectMoveToObject"); // marker



///////////////////////////////////////////////////////////////////////////////
//	MODULE: BehaviorModule
///////////////////////////////////////////////////////////////////////////////


CBehaviorModule::CBehaviorModule( CDriveControlModule *pDriveControlModule )
{
	m_pDriveCtrl = pDriveControlModule;
	m_CurrentActionMode = ACTION_MODE_NONE;
	m_CurrentTask = TASK_NONE;
	m_TaskState = 0;
	m_TurnHandleState = 0;
	m_NextShoulderPosition = 0;
	m_nObjectsPickedUp = 0;
	m_nHumanIDToTrack = 0;

	m_LedEyeMode = LED_EYES_OFF;
	m_LedEyeBlinkTimer = 0;
	m_LedEyeBrightness = LED_EYE_BRIGHTNESS_MAX;
	m_ArmMovementRight = 0;
	m_ArmMovementLeft = 0;
	m_ArmMovementBoth = 0;
	m_ArmMovementStateRight = 0;
	m_ArmMovementStateLeft = 0;
	gArmTimerRight = 0;
	gArmTimerLeft = 0;
	m_ObjectDetectCount = 0; // Used by ArmBehavior
	nKinectObjectsDetected = 0;	// Used for counting number of objects detected by Kinect
	m_KinectRetries = 0;		// Number of times to retry to find object
	m_ArmSpeedRight = SERVO_SPEED_MED;		// Used for saving and restoring prior arm speed
	m_ArmSpeedLeft = SERVO_SPEED_MED;		// Used for saving and restoring prior arm speed
	m_pArmControlRight = new ArmControl( RIGHT_ARM );
	m_pArmControlLeft = new ArmControl( LEFT_ARM );
	m_pHeadControl = new HeadControl();
	m_pKinectServoControl = new KinectServoControl;

	m_MaxMoveTimeRight = 0;
	m_MaxMoveTimeLeft = 0;
	m_ArmWaitForMoveToCompleteRight = TRUE;
	m_ArmWaitForMoveToCompleteLeft = TRUE;
	m_KinectSearchComplete = FALSE;
	m_ObjectPickupComplete = FALSE;
	m_PutObjectInCarryBasket = FALSE;

	m_HeadMovement = 0;
	m_HeadMovementState = 0;
	m_HeadSpeed = SERVO_SPEED_SLOW;
	m_HeadWaitForMoveToComplete = TRUE;

	m_ObjectX = 0;
	m_ObjectY = 0;
	m_PriorObjectX = 0;
	m_PriorObjectY = 0;
	m_ObjectDirectionDegrees = 0;
	m_ObjectDirectionDegreesPeak = 0;

//	m_pKinectTempObjects3D = new TEMP_OBJECT_3D_ARRAY_T;
//	memset( m_pKinectTempObjects3D, 0x00, sizeof( TEMP_OBJECT_3D_ARRAY_T ) );


}


CBehaviorModule::~CBehaviorModule()
{
	SAFE_DELETE( m_pArmControlRight );
	SAFE_DELETE( m_pArmControlLeft );
	SAFE_DELETE( m_pHeadControl );
	SAFE_DELETE( m_pKinectServoControl );
}



/////////////////////////////////////////////////////////////////////////////
//void CBehaviorModule::Init()
//{
//}

__itt_string_handle* pshRightArmHome = __itt_string_handle_create("RightArmHome");
__itt_string_handle* pshLeftArmHome = __itt_string_handle_create("LeftArmHome");
__itt_string_handle* pshHeadCenter = __itt_string_handle_create("HeadCenter");


///////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::RightArmHome()
{
	// Move all right arm seros to home position, and reset to default speed
	__itt_marker(pDomainControlThread, __itt_null, pshRightArmHome, __itt_marker_scope_task);
	__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshRightArmHome);
	m_pArmControlRight->MoveArmHome( m_ArmSpeedRight );
	m_pArmControlRight->EnableIdleArmMovement(TRUE);
	gHeadIdle = TRUE;
	m_ArmMovementStateRight = 0;	// back to Idle state
	m_ArmMovementRight = ARM_MOVEMENT_NONE; // back to Idle state
	__itt_task_end(pDomainControlThread);
}
///////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::LeftArmHome()
{
	// Move all right arm seros to home position, and reset to default speed
	__itt_marker(pDomainControlThread, __itt_null, pshLeftArmHome, __itt_marker_scope_task);
	__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshLeftArmHome);
	m_pArmControlLeft->MoveArmHome( m_ArmSpeedLeft );
	m_pArmControlLeft->EnableIdleArmMovement(TRUE);
	gHeadIdle = TRUE;
	m_ArmMovementStateLeft = 0;	// back to Idle state
	m_ArmMovementLeft = ARM_MOVEMENT_NONE; // back to Idle state
	__itt_task_end(pDomainControlThread);
}

///////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::HeadCenter()
{
	// Move head to home position, and reset to default speed
	// TODO:  m_pHeadControl->SetHeadSpeed( HEAD_OWNER_BEHAVIOR_P1, SERVO_SPEED_SLOW, SERVO_SPEED_SLOW, SERVO_SPEED_SLOW );
	__itt_marker(pDomainControlThread, __itt_null, pshHeadCenter, __itt_marker_scope_task);
	__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshHeadCenter);
	m_pHeadControl->SetHeadPositionCenter( HEAD_OWNER_BEHAVIOR_P1 );
	m_pHeadControl->ExecutePosition( HEAD_OWNER_BEHAVIOR_P1 );
	__itt_task_end(pDomainControlThread);
}

///////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::EndActionMode()
{
	// Cleanup at the end of each action.
	// Move head and Kinect to home position, reset to default speeds
	// Reset state machine (what about Arms - move to home?)

	m_pKinectServoControl->SetTiltPosition( KINECT_TILT_OWNER_TRACK_OBJECT, KINECT_HUMAN_DETECT_START_POSITION );	// By default, leave Kinect in a position where it can find humans
	m_pKinectServoControl->ReleaseOwner(KINECT_TILT_OWNER_TRACK_OBJECT);
	g_pKinectObjects3D->nObjectsDetected = 0;
	g_pKinectDebugSliceArray->nSlices = 0;
	HeadCenter();
	m_CurrentActionMode = 0;
	m_CurrentTask = 0;
	m_TaskState = 0;
}

///////////////////////////////////////////////////////////////////////////////

void CBehaviorModule::ProcessMessage(
		UINT uMsg, WPARAM wParam, LPARAM lParam )
{

	// Process Behavior commands from the GUI
	// and status updates from Servos and Sensors
	CString MsgString;
	UINT ActionRequested;
	//static int foo;

	switch( uMsg )
	{
		// Behavior modes - what to do while standing around
		// Some of these are mutually exclusive!

		case WM_ROBOT_SET_BEHAVIOR_CMD:
		{
			g_bCmdRecognized = TRUE;
			if( BEHAVIOR_FRIENDLY == wParam )  // wParam is the Behavior requested
			{
				// Enable random Head and Arm movements
				gArmIdleLeft = TRUE;
				gArmIdleRight = TRUE;
				gHeadIdle = TRUE;
			}
			else
			{
				// Enable random Head and Arm movements
				gArmIdleLeft = FALSE;
				gArmIdleRight = FALSE;
				gHeadIdle = FALSE;
				if( m_pHeadControl->IsOwner(HEAD_OWNER_RANDOM_MOVEMENT) )
				{
					m_pHeadControl->ReleaseOwner(HEAD_OWNER_RANDOM_MOVEMENT);
				}

				// Now, do any additional specific processing needed

				switch( wParam )  // wParam is the Behavior requested
				{
					case BEHAVIOR_MANUAL_CONTROL:
					{
					}
					break;

					case BEHAVIOR_WAIT_FOR_VERBAL_CMD:
					{
					}
					break;

					case BEHAVIOR_GUARD:
					{
					}
					break;

					case BEHAVIOR_PERSONAL_SPACE:
					{
						// If enabled, robot will move whenever someone violates
						// it's personal space (as detected by sensors) when standing still
						if( 0 == wParam )
						{
							// Disable Personal space mode
						}
						else
						{
							// Enable Personal space mode
						}
						//SendCommand( WM_ROBOT_CAMERA_BEHAVIOR_MODE, BEHAVIOR_GUARD, 0 );
					}
					break;

					default:
					{
						ROBOT_LOG( TRUE,"ERROR - Unhandled BEHAVIOR command (%02X)!\n", lParam)
					}
				}
			}
		}
		break;

		// Action modes - what to do right now
		case WM_ROBOT_SET_ACTION_CMD:
		{
			ROBOT_LOG( TRUE,"BehaviorModule: WM_ROBOT_SET_ACTION_CMD \n")
			g_bCmdRecognized = TRUE;
			ActionRequested = wParam;
			// lParam optional, depending upon the task

			/* Defined in RobotSharedParams.h:
			// Action Modes - Random ideas for now, of things robot could do
			// Set by WM_ROBOT_SET_ACTION_CMD
			ACTION_MODE_NONE = 0,					// No Action mode pending
			ACTION_MODE_FOLLOW_PERSON,				//
			ACTION_MODE_KARATE_DEMO,				//	
			ACTION_MODE_PICKUP_OBJECTS,				// Look for and pickup object anywhere in front of robot
			ACTION_MODE_PICKUP_CLOSE_OBJECT,		// Look for and pickup nearest object that is within reach of the robot
			ACTION_MODE_GO_TO_LOCATION,				// Go to named location on map (example "master bedroom")?
			ACTION_MODE_FOLLOW_OBJECT,				//
			ACTION_MODE_FIND_OBJECT, 				//
			ACTION_MODE_DELIVER_RECORDED_MSG,		//
			ACTION_MODE_RETREIVE_INTERNET_DATA,		//
			ACTION_MODE_START_PHONE_CALL,			//	Call someone using Skype?
			ACTION_MODE_GET_BEER,					//	Every robot should be able to get Beer from the Fridge!
			ACTION_MODE_OPEN_DOOR,					//	Just to demo door opening ability
			ACTION_MODE_DARTH_VADER					//  Do a Darth impression
			*/

			switch( ActionRequested )  // wParam is the Action requested
			{
				case ACTION_MODE_NONE:
				{
					if( ACTION_MODE_NONE != m_CurrentActionMode )
					{
						ROBOT_LOG( TRUE,"BehaviorModule: CURRENT ACTION CANCELLED!\n")
					}
					else
					{
						ROBOT_LOG( TRUE,"BehaviorModule: ACTION_MODE_NONE requested. NOP\n")
					}
					EndActionMode();
				}
				break;

				case ACTION_MODE_FOLLOW_PERSON:
				{
					// Follow the person currently in front of the robot
					m_CurrentActionMode = ActionRequested;
					m_CurrentTask = 1;
					m_TaskState = 1;	// go to first state
				}
				break;

				case ACTION_MODE_KARATE_DEMO:
				{
					// Demonstrate the Karate Task
					m_CurrentActionMode = ActionRequested;
					m_CurrentTask = 1;
					m_TaskState = 1;	// go to first state
				}
				break;

				case ACTION_MODE_GET_BEER:
				{
					// Begin task to get beer!
					// First Task: Go to Fridge
					m_CurrentActionMode = ActionRequested;
					m_CurrentTask = TASK_GO_TO_LOCATION;
					m_TaskState = 1;	// go to first state
				}
				break;

				case ACTION_MODE_OPEN_DOOR:
				{
					// Begin task to open a door in front of the robot
					// First Task: locate the door handle
					m_CurrentActionMode = ActionRequested;
					m_CurrentTask = TASK_OPEN_DOOR;
					m_TaskState = 1;	// go to first state
				}
				break;

				case ACTION_MODE_PICKUP_OBJECTS:
				case ACTION_MODE_PICKUP_CLOSE_OBJECT:
				{
					/*if( m_CurrentActionMode == ActionRequested )
					{
						// Repeat command for DEBUG -continue to next step!
						if( OBJECT_TASK_MOVE_TO_OBJECT == m_CurrentTask )
						{
							if( 1 == m_TaskState )
							{
								// ROBOT_LOG( TRUE,"CONTINUING To next Action Mode Step\n")
								m_TaskState++;
							}
						}
					}
					else*/
					{
						ROBOT_LOG( TRUE,"BehaviorModule: ACTION_MODE_PICKUP_OBJECT (Close or Far) requested\n")
						if( lParam )
						{
							if( CAMERA_STATE_INITIALIZED != g_Camera[KINECT_DEPTH].State )
							{
								ROBOT_DISPLAY( TRUE, "ACTION_MODE_PICKUP_OBJECT: KINECT Not Enabled! ABORTING" )
								g_ClientTextToSend = _T( "Bummer. My Kinect module is not enabled" );
								SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
								break;
							}
							m_CurrentActionMode = ActionRequested;
							m_CurrentTask = OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT;	// Start by looking for any objects nearby
							m_TaskState = 1;	// go to first state
							m_nObjectsPickedUp = 0; // keep track of number of objects picked up to measure success
						}
						else
						{
							///m_PickupObjectsEnabled = FALSE;
							ROBOT_LOG( TRUE,"BehaviorModule: ACTION_MODE_PICKUP_OBJECTS Cancelled\n")
							EndActionMode();
						}
					}
				}
				break;

				default:
				{
					ROBOT_LOG( TRUE,"ERROR - Unhandled ACTION command (%02X)!\n", ActionRequested)
				}

			}	// ActionRequested

		}	// WM_ROBOT_SET_ACTION_CMD
		break;


		case WM_ROBOT_SET_LED_EYES_CMD:
		{
			g_bCmdRecognized = TRUE;
			m_LedEyeMode = wParam;	// Used for blink only
			m_LedEyeBlinkTimer = 0;

			SendHardwareCmd( HW_SET_LED_EYES, wParam, lParam  );	// Mode, Brightness(not used)
		}
		break;


		/* todo
		#define ARM_MOVEMENT_OPEN_HATCH				0x08	// 
		#define ARM_MOVEMENT_CLOSE_HATCH			0x09	// 
		#define ARM_MOVEMENT_PICKUP_OBJECT			0x0A	// 
		#define ARM_MOVEMENT_LIFT_OBJECT			0x12	// */


		// Arm Movements
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		case WM_ROBOT_SET_ARM_DEFAULT_SPEED:
		{
			g_bCmdRecognized = TRUE;
			if( RIGHT_ARM == wParam )	// wParam is Right or Left Arm
			{
				m_ArmSpeedRight = lParam;
			}
			else if( LEFT_ARM == wParam )	// wParam is Right or Left Arm
			{
				m_ArmSpeedLeft = lParam;
			}
			else
			{
				ROBOT_ASSERT(0);	// bad state
			}
		}
		break;

		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		case WM_ROBOT_SET_ARM_MOVEMENT:
		{
			g_bCmdRecognized = TRUE;
			// Process Arm Behavior commands and do inital movemment for arms (move into first position)

			HandleArmMovementRequest( wParam, lParam );
		}
		break;

		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		case WM_ROBOT_CAMERA_MATCH_RESPONSE:
		{
			g_bCmdRecognized = TRUE;
			// Update from the vision system.  Response to WM_ROBOT_CAMERA_MATCH_OBJECT
			// wParam: ObjectPos, lParam: Location in Frame(TODO)
			HandleCameraMatchResponse( wParam, lParam );
		}
		break;

		case WM_ROBOT_FIND_OBJECT_AT_XYZ_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Tell camera to find object at X,Y,Z in space
			ROBOT_ASSERT(0);
			///m_pLaserScannerBehavior->HandleFindObjectAtXYZ( wParam, lParam );		
		}
		break;

		case WM_ROBOT_DO_LASER_SCANS:
		{
			g_bCmdRecognized = TRUE;
			// Do multi-line laser scans at location specified
			ROBOT_ASSERT(0);
			///m_pLaserScannerBehavior->DoLaserScans( wParam, lParam );		
		}
		break;


		case WM_ROBOT_SLAM_CHECK_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Put head in position and do a location check
			ROBOT_ASSERT(0);
			///m_pLaserScannerBehavior->DoLocationCheck();		
		}

		case WM_ROBOT_USER_CAMERA_PAN_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to tell camera to move Pan and Tilt
			// This command controls both Pan and Tilt
			HandleUserHeadMovementCmd(wParam, lParam );
		}
		break;

		case WM_ROBOT_USER_CAMERA_PAN_TILT_SPEED_CMD:
		{
			g_bCmdRecognized = TRUE;
			UINT Speed = lParam;
			m_pHeadControl->SetUserHeadSpeed( Speed ); // Save, so it can be restored as needed later
			m_pHeadControl->SetHeadSpeed( HEAD_OWNER_USER_CONTROL, Speed, Speed, Speed );
			m_pHeadControl->ExecutePositionAndSpeed( HEAD_OWNER_USER_CONTROL );
			return;
		}

		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// ============================================================================================================
		case WM_ROBOT_KINECT_SEARCH_COMPLETE:
		case WM_ROBOT_SERVO_STATUS_READY:
		case WM_ROBOT_SENSOR_STATUS_READY:
		// ============================================================================================================
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		{
			// All these events are handled here to prevent stalling if any one event or subsystem fails!
			// such as the PIC being offline - the Heartbeat continues!
			g_bCmdRecognized = TRUE;

			if( WM_ROBOT_KINECT_SEARCH_COMPLETE == uMsg )
			{
				m_KinectSearchComplete = TRUE;
				nKinectObjectsDetected = wParam;
				m_ObjectX = (int)(signed short)(LOWORD(lParam));
				m_ObjectY = (int)(signed short)(HIWORD(lParam));
				ROBOT_LOG( TRUE,"DEBUG - WM_ROBOT_KINECT_SEARCH_COMPLETE: m_ObjectX = %d, m_ObjectY = %d\n", m_ObjectX, m_ObjectY)
			}

			if( WM_ROBOT_SERVO_STATUS_READY == uMsg )
			{
				// Handle Arm Servo updates
				// Check Arm Movement State Machine.  Only used with movements that have multiple steps
				// Uses m_ArmMovementStateRight and m_ArmMovementStateLeft to track progress
				//m_pArmControlLeft->CheckArmPosition(DEBUG_ARM_MOVEMENT); // DEBUG
				HandleArmServoStatusUpdate( wParam, lParam );
			}

			/////////////////////////////////////////////////////////////////////////////////////////
			// Check status of Behavior Tasks
			//
			// Action Mode
			//   Task
			//     Task Step
			//       Step State
				//	m_CurrentActionMode = ACTION_MODE_NONE;
				//	m_CurrentTask = TASK_NONE;
				//	m_TaskState = 0;

			if( (ACTION_MODE_NONE != m_CurrentActionMode) && (TASK_NONE != m_CurrentTask) ) 
			{
				// Something to do.  See if we are ready to do the next step
				if( 0 != gArmTimerLeft)
				{
					ROBOT_LOG( TRUE,"WAITING FOR ARM TIMER LEFT = %d\n", gArmTimerLeft)
					break;
				}

				if( 0 != gArmTimerRight)
				{
					ROBOT_LOG( TRUE,"WAITING FOR ARM TIMER RIGHT = %d\n", gArmTimerRight)
					break;
				}

				switch( m_CurrentActionMode )
				{
					case ACTION_MODE_NONE:
					{
						break;	// Nothing to do
					}

					case ACTION_MODE_FOLLOW_PERSON:
					{	
						ActionFollowPerson();
						break;
					}	// case ACTION_MODE_FOLLOW_PERSON

					case ACTION_MODE_KARATE_DEMO:
					{	
						ActionKarate();
						break;
					}	// case ACTION_MODE_KARATE_DEMO

					case ACTION_MODE_PICKUP_OBJECTS:
					{	
						ActionPickupObjects();
						break;
					}	// case ACTION_MODE_PICKUP_OBJECTS

					case ACTION_MODE_PICKUP_CLOSE_OBJECT:
					{	
						ActionPickupCloseObject();
						break;
					}	// case ACTION_MODE_PICKUP_CLOSE_OBJECT

					case ACTION_MODE_OPEN_DOOR:
					{	
						ActionOpenDoor();
						break;
					}	// case ACTION_MODE_OPEN_DOOR

					case ACTION_MODE_GET_BEER:
					{	
						ActionGetBeer();
						break;
					}	// case ACTION_MODE_GET_BEER

				}	// switch m_CurrentActionMode
			} // If !ACTION_MODE_NONE
			else
			{
				// Do Idle processing
				if( (gHeadIdle) && (0 == gHeadIdleTimer) )
				{
					m_pHeadControl->DoIdleMovement( HEAD_OWNER_RANDOM_MOVEMENT );
					gHeadIdleTimer = 4;	// 1/10 second per count!
				}
				if( m_pArmControlRight->IdleArmMovementEnabled() )
				{
					m_pArmControlRight->DoIdleArmMovement();
				}
				if( m_pArmControlLeft->IdleArmMovementEnabled() )
				{
					m_pArmControlLeft->DoIdleArmMovement();
				}
			}

			if( (0 != m_CurrentTask) || (0 != m_TaskState) )
			{
				//ROBOT_LOG( TRUE,"BEHAVIOR MODULE: TASK = %d, TASKSTATE = %d\n", m_CurrentTask, m_TaskState)
			}

			break;
		}	//	case WM_ROBOT_SENSOR_STATUS_READY:
		break;
	}	// Switch uMsg
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Behavioral Tasks
// These can be called as subroutines to some larger task
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::TaskDoKarate()
{
	// See if we are ready to do the next step
	if( (0 != gArmTimerLeft) || (0 != gArmTimerRight) )
	{
		ROBOT_LOG( TRUE,"KARATE WAITING FOR TIMER TimerL = %d, TimerR = %d m_TaskState = %d\n", gArmTimerLeft, gArmTimerRight, m_TaskState)
	}

	if( !m_pArmControlLeft->CheckArmPosition(FALSE) ) // Verbose?
	{
		ROBOT_LOG( TRUE,"KARATE WAITING FOR LEFT ARM\n")
	}
	if( !m_pArmControlRight->CheckArmPosition(FALSE) )
	{
		ROBOT_LOG( TRUE,"KARATE WAITING FOR RIGHT ARM\n")
	}

	if( (0 == gArmTimerLeft) && 
		((!m_ArmWaitForMoveToCompleteLeft) || (m_pArmControlLeft->CheckArmPosition(DEBUG_ARM_MOVEMENT))) )	//	TRUE = Verbose
	{
		if( (0 == gArmTimerRight) && 
			((!m_ArmWaitForMoveToCompleteRight) || (m_pArmControlRight->CheckArmPosition(DEBUG_ARM_MOVEMENT))) )	//	TRUE = Verbose
		{
			// Both arms in position, and timer(if any) expired.  Ready for next move
			m_ArmWaitForMoveToCompleteRight = TRUE;	// Default, can be overridden below
			m_ArmWaitForMoveToCompleteLeft = TRUE;

			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case 2:
				case 4:
				case 6:
				case 8:
				case 10:
				case 12:
				case 14:
				case 16:
				case 18:
				case 20:
				{
					// Wait between each move
					gArmTimerLeft = 5;	// 1/10 second per count!
					m_TaskState++;
					ROBOT_LOG( TRUE,"m_TaskState = %d\n",  m_TaskState)
					break;

				}
				case 1: // Move to first Karate state
				{
					m_pArmControlRight->SetArmSpeed( m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight );	
					m_pArmControlRight->ExecutePositionAndSpeed();
					m_pArmControlRight->SetArmPosition( 80, 0, 90, 5, 20 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmSpeed( m_ArmSpeedLeft, m_ArmSpeedLeft, m_ArmSpeedLeft, m_ArmSpeedLeft, m_ArmSpeedLeft );	
					m_pArmControlLeft->ExecutePositionAndSpeed();
					m_pArmControlLeft->SetArmPosition( 80, 0, 90, 5, 20 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 3: // Arms in front
				{
					m_pArmControlRight->SetArmPosition( 90, -30, 90, 5, 30 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmPosition( 40, -60, 90, 5, 10 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 5: // Arm above head
				{
					// Disabled for now!
					//m_pArmControlRight->SetArmPosition( 180, -90, 85, 140, 30 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					//m_pArmControlRight->ExecutePosition();
					//m_pArmControlRight->SetArmPosition( -30, 0, 130, 0, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					//m_pArmControlRight->ExecutePosition();

					//m_pArmControlLeft->SetArmPosition( 90, -90, 80, 0, 10 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					//m_pArmControlLeft->ExecutePosition();
					m_TaskState = 7;	// SKIP TO NEXT STATE!
					break;
				}
				case 7: // Back in Front
				{
					m_pArmControlRight->SetArmPosition( 90, -30, 90, 5, 30 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmPosition( 40, -60, 90, 5, 10 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 9: // Punch Home Position
				{
					m_pArmControlRight->SetArmPosition( -30, 0, 130, 0, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmPosition( -40, 2, 130, 0, 10 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 11: // Punch Right
				{
 					//m_pArmControlRight->SetArmSpeed( SERVO_SPEED_MAX, SERVO_SPEED_MED_FAST, SERVO_SPEED_MED_FAST, SERVO_SPEED_MED_FAST, SERVO_SPEED_MED_FAST );	
					//m_pArmControlRight->ExecutePositionAndSpeed();
					m_pArmControlRight->SetArmPosition( 110, 5, 45, 0, 1 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
				case 13: // Punch Home
				{
					m_pArmControlRight->SetArmPosition( -40, 0, 130, 0, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 15: // Extend slightly before next move
				{
					//m_pArmControlRight->SetArmSpeed( m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight );	
					//m_pArmControlLeft->SetArmSpeed( m_ArmSpeedLeft, m_ArmSpeedLeft, m_ArmSpeedLeft, m_ArmSpeedLeft, m_ArmSpeedLeft );	
					//m_pArmControlRight->ExecutePositionAndSpeed();
					m_pArmControlRight->SetArmPosition( 30, 0, 90, 120, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmPosition( 30, 0, 90, 120, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 17: // Arms Crossed
				{
					m_pArmControlRight->SetArmPosition( 30, -65, 90, 120, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmPosition( 50, -65, 90, 120, 15 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 19: // Straghten arm before next move
				{
					m_pArmControlRight->SetArmPosition( 30, 0, NOP, NOP, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_pArmControlLeft->SetArmPosition( 50, 0, NOP, NOP, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_TaskState++;
					break;
				}

				case 21: // Arms Home
				{
					RightArmHome();
					LeftArmHome();
					m_TaskState = 0; // Done
					break;
				}
/*
				case 10: // 
				{
					m_pArmControlRight->SetArmPosition( 0, 0, 0, 0, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();
					m_ArmMovementStateRight++;
					m_ArmWaitForMoveToCompleteRight = TRUE;

					m_pArmControlLeft->SetArmPosition( 0, 0, 0, 0, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_ArmMovementStateLeft++;
					m_ArmWaitForMoveToCompleteLeft = TRUE;
					break;
				}
*/
				default:
				{
					ROBOT_ASSERT(0);
				}
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::TaskOpenDoor()
{
	// Called upon sensor updates.
	// See if we are ready to do the next step
	if( (0 != gArmTimerLeft) || (0 != gArmTimerRight) )
	{
		ROBOT_LOG( TRUE,"TimerL = %d, TimerR = %d m_TaskState = %d\n", gArmTimerLeft, gArmTimerRight, m_TaskState)
	}

	if( (0 == gArmTimerLeft) && 
		((!m_ArmWaitForMoveToCompleteLeft) || (m_pArmControlLeft->CheckArmPosition(DEBUG_ARM_MOVEMENT))) )	//	TRUE = Verbose
	{
		if( (0 == gArmTimerRight) && 
			((!m_ArmWaitForMoveToCompleteRight) || (m_pArmControlRight->CheckArmPosition(DEBUG_ARM_MOVEMENT))) )	//	TRUE = Verbose
		{
			// Both arms in position, and timer(if any) expired.  Ready for next move
			m_ArmWaitForMoveToCompleteRight = TRUE;	// Default, can be overridden below
			m_ArmWaitForMoveToCompleteLeft = TRUE;

			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case 2:
				case 4:
				case 6:
				case 8:
				case 10:
				case 12:
				case 14:
				{
					// Wait between each move
					gArmTimerLeft = 2;	// 1/10 second per count!
					m_TaskState++;
					ROBOT_LOG( TRUE,"m_TaskState = %d\n",  m_TaskState)
					break;

				}
				case 1: // Move to first Door state
				{
					//m_pArmControlRight->SetArmSpeed( m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight, m_ArmSpeedRight );	
					//m_pArmControlRight->ExecutePositionAndSpeed();
					m_pArmControlRight->SetArmPosition( 126, 0, -30, -90, 85 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
				case 3: // find door handle
				{
					m_pArmControlRight->SetArmPosition( 119, 0, -30, -90, 85  ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
				case 5: // close claw
				{
					m_pArmControlRight->SetArmPosition( NOP, NOP, NOP, NOP, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
				case 7: // rotate wrist
				{
					m_pArmControlRight->SetArmPosition( NOP, NOP, NOP, -40, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
				case 9: //
				{
					m_pArmControlRight->SetArmSpeed( SERVO_SPEED_MAX, SERVO_SPEED_MED_FAST, SERVO_SPEED_MED_FAST, SERVO_SPEED_MED_FAST, SERVO_SPEED_MED_FAST );	
					m_pArmControlRight->ExecutePositionAndSpeed();
					m_pArmControlRight->SetArmPosition( 100, 0, -40, -140, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
//BACK UP and then
				case 11: // 
				{
					m_pArmControlRight->SetArmPosition( 124, 0, -35, -120, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();
					m_TaskState++;
					break;
				}
				case 13: // Let loose of handle, just one finger to pull
				{
					m_pArmControlRight->SetArmPosition( 122, NOP, NOP, -95 , 85 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
// Backup more and then release completely
				case 15: //
				{
					m_pArmControlRight->SetArmPosition( 130, NOP, NOP, NOP, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
// retract arm a bit
				case 17: //
				{
					m_pArmControlRight->SetArmPosition( 130, NOP, NOP, NOP, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
// turn right and then exten arm down
				case 19: //
				{
					m_pArmControlRight->SetArmPosition( 90, NOP, 0, NOP, 10 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					// Keep left arm out of the way
					m_pArmControlRight->SetArmPosition( -30, -4, 120, NOP, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
// Move forward again, close to the door and then
				case 21: //
				{
					m_pArmControlRight->SetArmPosition( 30, -30, 85, 0, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();

					m_TaskState++;
					break;
				}
// Rotate robot to open door fully, then rotate back, and put arms in normal position

				case 23: // Arms Home
				{
					RightArmHome();
					LeftArmHome();
					m_TaskState = 0; // Done
					break;
				}
				// Ready to go through the doorway!
/*
				case 10: // 
				{
					m_pArmControlRight->SetArmPosition( 0, 0, 0, 0, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlRight->ExecutePosition();
					m_ArmMovementStateRight++;
					m_ArmWaitForMoveToCompleteRight = TRUE;

					m_pArmControlLeft->SetArmPosition( 0, 0, 0, 0, 0 ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
					m_pArmControlLeft->ExecutePosition();
					m_ArmMovementStateLeft++;
					m_ArmWaitForMoveToCompleteLeft = TRUE;
					break;
				}
*/
				default:
				{
					ROBOT_ASSERT(0);
				}
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Returns Distance to move and Degrees to turn to reach object
// NOTE: Supports LEFT ARM Pickup ONLY!

void CBehaviorModule::CalculatePathToObjectPickup( int ObjectX, int ObjectY, int &DistanceTenthInches, double &DirectionDegrees )
{
	// Offset for Right or Left arm pickup
	double OffsetDistanceTenthInches = (double)( HALF_ROBOT_BODY_WIDTH_TENTH_INCHES + HALF_ROBOT_CLAW_WIDTH_TENTH_INCHES  ); // add fudge for claw size

	double X = ObjectX; // Tenthinches
	double Y = ObjectY; // 

	// translate X so robot end up with Left arm over the object
	X += OffsetDistanceTenthInches;

	// calculate direction and distance to object pickup location
	DistanceTenthInches = (int)((sqrt((X*X) + (Y*Y))) );
	double AngleRadians = atan2( X, Y );	// X,Y backward from classical math
	DirectionDegrees = AngleRadians * RADIANS_TO_DEGREES;

	ROBOT_LOG( TRUE,"------------------------------------------------------\n")
	ROBOT_LOG( TRUE,"ACTION_MODE_PICKUP_OBJECTS: For X=%4.1f, Y=%4.1f: Move Distance = %4d  Move Angle = %4.1f (all in inches)\n\n", 
		(X/10.0), (Y/10.0), (DistanceTenthInches/10), DirectionDegrees)

}



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// High level Actions
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::ActionKarate()
{
	// Called upon sensor updates.

	switch( m_CurrentTask )
	{
		case 0:
		{
			break;	// Nothing to do
		}
		case 1:	// Karate Demo only has one task (since it's just a demo), but several Task States
		{
			//m_CurrentTask++; 
			TaskDoKarate();
			break;
		}
		default:
		{
			ROBOT_ASSERT(0);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::ActionFollowPerson()
{
	// Confirm that Kinect is working, and I have a lock on the person in front of me
	// Confirm that Object Avoidance is enabled (highly recommended)
	// Remember the Human Number.  If we lose lock, ask user if it's OK to continue?
	// Turn robot toward person as needed
	// Maintain proper following distance 
	// When person says "Stop" Cancel this action.
	// Santity check
	if( g_nHumansTracked > KINECT_MAX_HUMANS_TO_TRACK )
	{
		ROBOT_LOG( TRUE, "*********** FATAL ERROR!!! g_nHumansTracked > KINECT_MAX_HUMANS_TO_TRACK ***********" )
		m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, SPEED_STOP, TURN_CENTER );
		m_nHumanIDToTrack = 0;
		m_CurrentActionMode = m_CurrentTask = m_TaskState = 0; // Clear Action Mode
		return;
	}

	int HumanPosX = 0; 
	int HumanPosY = 0; 
	int HumanPosZ = 0;

	switch( m_CurrentTask )
	{
		case 0:
		{
			break;	// Nothing to do
		}
		case 1:	//
		{
			// Check to see if we have a person to track, and save the HumanID
			m_nHumanIDToTrack = 0;
			//int nHumanToTrackIndex = -1; 
			__itt_task_begin(pDomainModuleThread, __itt_null, __itt_null, psh_csKinectHumanTrackingLock);
			EnterCriticalSection(&g_csKinectHumanTrackingLock);
				for( int nHuman=0; nHuman < g_nHumansTracked; nHuman++ )
				{
					if( (0 != g_HumanLocationTracking[nHuman].nHumanID) &&
						(0 != g_HumanLocationTracking[nHuman].CenterOfMass.Y) )
					{
						// found someone!  - TODO: Look for person in center of screen(compare .x values)
						m_nHumanIDToTrack = g_HumanLocationTracking[nHuman].nHumanID;
						break;
					}
				}
			LeaveCriticalSection(&g_csKinectHumanTrackingLock);
			__itt_task_end(pDomainModuleThread);

			if( 0 == m_nHumanIDToTrack )
			{
				// HANDLE ERROR HERE
				///g_ClientTextToSend = _T( "I am having a problem seeing you. Please move and try again." );
				///SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
				ROBOT_LOG( TRUE, "ACTION_MODE_FOLLOW_PERSON: Can't lock on to human" )
				/// wait until we find someone --> m_CurrentActionMode = m_CurrentTask = m_TaskState = 0; // Clear Action Mode
			}
			else
			{
				// Found someone
				g_ClientTextToSend = _T( "OK, I will follow." );
				SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
				ROBOT_LOG( TRUE, "ACTION_MODE_FOLLOW_PERSON: Lock on to human ID %d successful",  m_nHumanIDToTrack )

				// Optional: Point Eyes at human for realism.
			///	m_pHeadControl->SetHeadSpeed( HEAD_OWNER_KINECT_HUMAN, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW );
			///	m_pHeadControl->LookAtXYZ( HEAD_OWNER_KINECT_HUMAN, g_HumanLocationTracking[m_nHumanIDToTrack].HeadLocation.x, 
			///		g_HumanLocationTracking[m_nHumanIDToTrack].HeadLocation.y, (g_HumanLocationTracking[m_nHumanIDToTrack].HeadLocation.z - 60) ); //  Look at Human eyes, not top of head
				//gKinectDelayTimer = 5; // tenth-seconds - Don't jerk around (or overload the CPU)

				m_CurrentTask++; 
			}
			break;
		}
		case 2:	// Following Human!
		{	
			// Get human position info
			HumanPosX = HumanPosY = HumanPosZ = 0;
			__itt_task_begin(pDomainKinectThread, __itt_null, __itt_null, psh_csKinectHumanTrackingLock);
			EnterCriticalSection(&g_csKinectHumanTrackingLock);
				for( int nHuman=0; nHuman < g_nHumansTracked; nHuman++ )
				{
					if( g_HumanLocationTracking[nHuman].nHumanID == m_nHumanIDToTrack )
					{
						// found the human we were tracking
						HumanPosX = g_HumanLocationTracking[nHuman].CenterOfMass.X;
						HumanPosY = g_HumanLocationTracking[nHuman].CenterOfMass.Y; 
						HumanPosZ = g_HumanLocationTracking[nHuman].CenterOfMass.Z; 
						//int HumanPosZ = g_HumanLocationTracking[nHuman].HeadLocation.z;
						break;
					}
				}
			LeaveCriticalSection(&g_csKinectHumanTrackingLock);
			__itt_task_end(pDomainModuleThread);

			// Make sure we have not lost the person we were tracking
			if( 0 == HumanPosY )
			{
				// Lost person we were tracking!
				g_ClientTextToSend = _T( "Oh No!  I lost track of you.  Stopping" );
				SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
				ROBOT_LOG( TRUE, "ACTION_MODE_FOLLOW_PERSON: Lost Lock on to human!  Stopping." )
				m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, SPEED_STOP, TURN_CENTER );
				m_nHumanIDToTrack = 0;
				m_CurrentTask = 1; // RETRY!!! (DEBUG)
				//m_CurrentActionMode = m_CurrentTask = m_TaskState = 0; // Clear Action Mode
				break;
			}

			// Looks like we found the Human.  Let's follow!
			ROBOT_LOG( TRUE, "ACTION_MODE_FOLLOW_PERSON: Tracking Human ID %d ",  m_nHumanIDToTrack )

			// Turn and move as needed
			int NewSpeed = 0;
			int NewTurn = 0;
			int CurrentSpeed = m_pDriveCtrl->GetCurrentSpeed();
			int CurrentTurn = m_pDriveCtrl->GetCurrentTurn();
			//if( m_pDriveCtrl->RobotStopped() )

			///////////////////////////////////////
			// Calculate Turn needed
			/// FYI, TURN_LEFT_MAX	== -127
			int CalculatedTurn =  HumanPosX / 4; // TODO - TUNE THIS
			NewTurn = CalculatedTurn;

			if( (NewTurn > -20) && (NewTurn < 20) )
			{
				ROBOT_LOG( TRUE,"DEBUG TURN: IN DEADBAND %d\n", NewTurn)
				NewTurn = 0; // Don't move in the deadband
			}
			else if( NewTurn < TURN_LEFT_MED )	
			{
				NewTurn = TURN_LEFT_MED;
			}
			else if( NewTurn > TURN_RIGHT_MED )	
			{
				NewTurn = TURN_RIGHT_MED;
			}


			///////////////////////////////////////
			// Calculate Speed needed
			int DistanceToHuman = HumanPosY;

			NewSpeed = (DistanceToHuman - HUMAN_FOLLOW_DISTANCE_TARGET_TENTH_INCHES) / 5;
			if( (NewSpeed > -25) && (NewSpeed < 25) )
			{
				ROBOT_LOG( TRUE,"DEBUG SPEED: IN DEADBAND %d\n", NewSpeed)
				NewSpeed = 0; // Don't move in the deadband
			}

			// Limit Check
			if( NewSpeed < SPEED_REV_MED_SLOW )	 // SPEED_REV_MED
			{
				NewSpeed = SPEED_REV_MED_SLOW;	// SPEED_REV_MED
			}
			else if( NewSpeed > SPEED_FWD_MED ) // SPEED_FWD_MED_FAST
			{
				NewSpeed = SPEED_FWD_MED; // SPEED_FWD_MED_FAST
			}

			// Emergeny override								 
			if( DistanceToHuman < HUMAN_FOLLOW_DISTANCE_MIN_TENTH_INCHES )
			{
				// Way too close!
				ROBOT_LOG( TRUE,"DEBUG SPEED: EMERGENCY STOP %d\n", NewSpeed)
				NewSpeed = SPEED_STOP; // Stop fast!
			}

			m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, NewSpeed, NewTurn );
					
			//m_CurrentTask++; // STAY IN THIS TASK
			break;
		}
		default:
		{
			ROBOT_ASSERT(0);
		}
	}
	//break;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::ActionPickupObjects()
{
	// Picks up multiple objects

	// Look for nearest object
	// if object found, determine if best to pick on on left or right side
	// plot course and distance
	// move near object, tracking Object position the whole time
	// pick up
	// look for next one
	switch( m_CurrentTask )
	{
		case OBJECT_TASK_NONE:
		{
			break;	// Nothing to do
		}

		case OBJECT_TASK_DONE: // Done looking!
		{
			__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupNoObject, __itt_marker_scope_task);
			if( 0 != m_TaskState )
			{	// Note: 0 indicates "don't say anything, just quit"
				if( m_nObjectsPickedUp > 0 )
				{
					ROBOT_DISPLAY( TRUE, "BEHAVIOR MODULE:ActionPickupObjects: I don't see any more objects to pick up.  DONE" )
					g_ClientTextToSend.Format( "My work here is finished." );
					SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
					SendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, 180, TURN_RIGHT_MED );	// wParam = distance in degrees, lParam = direction and speed

				}
				else
				{
					ROBOT_DISPLAY( TRUE, "BEHAVIOR MODULE: No Object found to Pick up.  ABORTING" )
					g_ClientTextToSend.Format( "I don't see anything to pick up" );
					SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
				}
			}
			EndActionMode();
			break;
		}

		case OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT:	// First Step.  Robot not moving yet
		{	
			__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectStart, __itt_marker_scope_task);

			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case KINECT_FLOOR_SCAN_MOVE_ARMS_STATE:
				{	
					// Make sure arms are not in the way of the Kinect View
					ROBOT_LOG( TRUE,"ACTION_MODE_PICKUP_OBJECTS Moving Arms Home\n")
					m_pArmControlLeft->SetArmPosition( LEFT_ARM_SHOULDER_HOME1, LEFT_ARM_ELBOW_ROTATE_HOME1, 
						LEFT_ARM_ELBOW_BEND_HOME1, LEFT_ARM_WRIST_ROTATE_HOME1, LEFT_ARM_CLAW_HOME1 );  // Explicitly close the claw
					m_pArmControlLeft->ExecutePositionAndSpeed();

					m_pArmControlRight->SetArmPosition( LEFT_ARM_SHOULDER_HOME1, LEFT_ARM_ELBOW_ROTATE_HOME1, 
						LEFT_ARM_ELBOW_BEND_HOME1, LEFT_ARM_WRIST_ROTATE_HOME1, LEFT_ARM_CLAW_HOME1 );  // Explicitly close the claw
					m_pArmControlRight->ExecutePositionAndSpeed();
					m_ObjectDirectionDegreesPeak = 0;
					m_TaskState++;
				}
				case KINECT_FLOOR_SCAN_START_SEARCH_STATE:	// Request a scan by the Kinect Module
				{
					ROBOT_LOG( TRUE,"Behavior: KINECT_FLOOR_SCAN_START_SEARCH_STATE Sending WM_ROBOT_KINECT_SEARCH_FLOOR_CMD\n")
					m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
					nKinectObjectsDetected = 0;
					SendCommand( WM_ROBOT_KINECT_SEARCH_FLOOR_CMD, (DWORD)FALSE, (DWORD)0 ); // FALSE = Find Close or Far Objects
					// Point the head looking at the floor too (just for affect)
					gHeadIdle = FALSE;
					m_pHeadControl->SetHeadSpeed( HEAD_OWNER_BEHAVIOR_P1, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW );
					m_pHeadControl->SetHeadPosition( HEAD_OWNER_BEHAVIOR_P1, CAMERA_PAN_CENTER, CAMERA_TILT_CENTER-300, CAMERA_SIDETILT_CENTER );
					m_pHeadControl->ExecutePositionAndSpeed( HEAD_OWNER_BEHAVIOR_P1 );
					m_TaskState++;
					break;
				}
				case KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE:	// See if we are done waiting
				{	
					if( !m_KinectSearchComplete )
					{
						//ROBOT_LOG( TRUE,"KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE: Waiting for m_KinectSearchComplete 2\n")
						break;	// continue waiting
					}
					else
					{
						__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectSearchComplete, __itt_marker_scope_task);

						// Done waiting for the search to complete.								
						if( nKinectObjectsDetected > 0 )
						{	
							// Object found.																								
							m_TaskState = KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE;
						}
						else
						{	// No object found.
							m_CurrentTask = OBJECT_TASK_DONE;
							m_TaskState = 1;
						}
					}
					break;
				}
				case KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE:	// Object found!
				{
					// Objects found.  Calculate path to the nearest one
					if( g_pKinectObjects3D->nObjectsDetected > 1 )
					{
						g_ClientTextToSend.Format( "I see %d objects.  The closest is %4.1f inches in front of me", 
								g_pKinectObjects3D->nObjectsDetected, (float)g_pKinectObjects3D->nClosestObjectDistance / 10.0 );
						SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
					}
					else if( g_pKinectObjects3D->nObjectsDetected > 0 )
					{
						g_ClientTextToSend.Format( "I see one object %4.1f inches in front of me", (float)g_pKinectObjects3D->nClosestObjectDistance / 10.0 );
						SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
					}

					CString MsgString;
					MsgString.Format( "BEHAVIOR MODULE: Found Object to Pick up at Y =%4.1f, X =%4.1f\n",
						(double)m_ObjectY/10.0, (double)m_ObjectX/10.0 );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					if( (m_ObjectX > KINECT_RANGE_TENTH_INCHES_MAX) || (m_ObjectY > KINECT_RANGE_TENTH_INCHES_MAX) )
					{
						ROBOT_ASSERT(0);
					}

					if( m_pArmControlLeft->IsObjectInPickupZone(m_ObjectY, m_ObjectX) )
					{
						// Object within range, just pick it up.  -- TODO-MUST, what if Object not near Left arm?  Back up?
						__itt_marker(pDomainControlThread, __itt_null, pshKinectArrivedAtObject, __itt_marker_scope_task);
						ROBOT_DISPLAY( TRUE, "OBJECT_TASK_MOVE_TO_OBJECT: NO NEED TO MOVE, JUST PICK UP!\n" )
						m_CurrentTask = OBJECT_TASK_PICK_UP_OBJECT;
						m_TaskState = 1;
					}
					else
					{
						// Move to the object.
						// Track the object and do course corrections as needed
						__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectTrackingStart, __itt_marker_scope_task);

						// Point head at object for realism.  Note Z assumed to be 1 inch above the floor (doesn't really matter)
						m_pHeadControl->SetHeadSpeed( HEAD_OWNER_BEHAVIOR_P1, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW );
						m_pHeadControl->LookAtXYZ( HEAD_OWNER_BEHAVIOR_P1, m_ObjectX,(m_ObjectY+100), 10 ); // Pad Y so camera does not bottom out

						m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
						nKinectObjectsDetected = 0;
						m_KinectRetries = 3;
						SendCommand( WM_ROBOT_KINECT_TRACK_OBJECT_CMD, (DWORD)1, (DWORD)0 );	// Enable Tracking
						m_CurrentTask = OBJECT_TASK_MOVE_TO_OBJECT;
						m_TaskState = 1;
					}
					break;
				}

				default: ROBOT_ASSERT(0);
			}
			break;

		}	// OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT

		case OBJECT_TASK_MOVE_TO_OBJECT:
		{
			/***
				// DEBUG DEBUG DEBUG!!!  NORMAL OPERATION DISABLED - FOR TESTING, JUST TRACK THE OBJECT
				ROBOT_DISPLAY( TRUE, "OBJECT_TASK_MOVE_TO_OBJECT: Pick up disabled in code!" )
				SendCommand( WM_ROBOT_KINECT_TRACK_OBJECT_CMD, (DWORD)1, (DWORD)0 );	// Enable Tracking -  Just loop for now
				m_CurrentTask = 0;
				m_TaskState = 0;
				break;
			***/

			if( !m_KinectSearchComplete )
			{
				//ROBOT_LOG( TRUE,"KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE: Waiting for m_KinectSearchComplete 3\n")
				break;	// continue waiting
			}
			/*else if( 1 == m_TaskState )
			{
					//DEBUG!  Pause until go ahead from GUI
				break; // Continue waiting
			}*/
			else
			{
				// Done waiting for the search to complete.	
				if( 0 == nKinectObjectsDetected )
				{	
					// No object found.
					__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectLost, __itt_marker_scope_task);
					if( 0 == m_KinectRetries )
					{
						ROBOT_DISPLAY( TRUE, "OBJECT_TASK_MOVE_TO_OBJECT: LOST OBJECT to Pick up!  Abort" )
						m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, SPEED_STOP, TURN_CENTER );
						m_CurrentTask = OBJECT_TASK_DONE;
						m_TaskState = 1;
						break;
					}
					else
					{
						ROBOT_DISPLAY( TRUE, "OBJECT_TASK_MOVE_TO_OBJECT: LOST OBJECT to Pick up!  See if we can find it again!" )
						ROBOT_LOG( TRUE,"%d Retries remaining/n", m_KinectRetries)
						m_KinectRetries--;

						m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
						nKinectObjectsDetected = 0;
						SendCommand( WM_ROBOT_KINECT_TRACK_OBJECT_CMD, (DWORD)1, (DWORD)0 );	// Enable Tracking
						m_TaskState = 1;
						break;
					}
				}
				else
				{
					/////////////////////////////////////////////////
					// Object found.																								
					m_KinectRetries = 3;

					// Point head at object for realism.  Note Z assumed to be 1 inch above the floor (doesn't really matter)
					//m_pHeadControl->SetHeadSpeed( HEAD_OWNER_BEHAVIOR_P1, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW );
					//m_pHeadControl->LookAtXYZ( HEAD_OWNER_BEHAVIOR_P1, m_ObjectX,(m_ObjectY+100), 10 ); // Pad Y so camera does not bottom out

					// Calculate direction and distance to nearest object (to align with left or right arm)
					CalculatePathToObjectPickup( m_ObjectX, m_ObjectY, m_ObjectDistanceTenthInches, m_ObjectDirectionDegrees ); // IN,IN,OUT,OUT

					// NOTE-  IGNORE m_ObjectDistanceTenthInches!  just use Y
					m_ObjectDistanceTenthInches = m_ObjectY;
					if ( abs(m_ObjectDirectionDegrees) > abs(m_ObjectDirectionDegreesPeak) )
					{
						// Keep track of how much we turned to get to this object
						m_ObjectDirectionDegreesPeak = m_ObjectDirectionDegrees;
					}

					// Do checks for Errors such as object tracking lost or object too close or behind / under robot
					// ****** TODO: Figure out what to do when this happens!  For now, just abort.

					if( (m_ObjectDirectionDegrees > 180) || (m_ObjectDirectionDegrees < -180) )
					{
						ROBOT_DISPLAY( TRUE, "OBJECT_TASK_MOVE_TO_OBJECT: ERROR! m_ObjectDirectionDegrees > 180!  ABORT PICKUP OBJ\n" )
						m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, SPEED_STOP, TURN_CENTER );
						m_CurrentTask = OBJECT_TASK_DONE;
						m_TaskState = 1;
						break;
					}

					if( m_pArmControlLeft->IsObjectInPickupZone(m_ObjectY, m_ObjectX) )
					{
						__itt_marker(pDomainControlThread, __itt_null, pshKinectArrivedAtObject, __itt_marker_scope_task);
						ROBOT_DISPLAY( TRUE, "OBJECT_TASK_MOVE_TO_OBJECT: WE HAVE ARRIVED!\n" );
						m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, SPEED_STOP, TURN_CENTER );
						/////////////////////////////////////////////////////////////////////////////////
						// OK, now switch to Pickup behavior!
						// Start with a final close scan.  Set flag to indicate close search only
						ROBOT_LOG( TRUE,"Behavior: OBJECT_TASK_MOVE_TO_OBJECT Sending WM_ROBOT_KINECT_SEARCH_FLOOR_CMD\n")
						m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
						nKinectObjectsDetected = 0;
						SendCommand( WM_ROBOT_KINECT_SEARCH_FLOOR_CMD, (DWORD)TRUE, (DWORD)0 ); // TRUE = Find Close Objects Only
						m_CurrentTask = OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT;
						m_TaskState = KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE;
						/////////////////////////////////////////////////////////////////////////////////
						break;
					}

					int Speed = SPEED_FWD_MED_SLOW;

					if( m_ObjectDistanceTenthInches < (OBJECT_PICKUP_LEFT_MAX_Y+10) )  // add fudge for stopping time
					{
						ROBOT_LOG( TRUE,"Object close, but not in Pickup Zone!\n")

						if ( m_ObjectDistanceTenthInches < OBJECT_PICKUP_LEFT_MAX_Y )
						{
							// do a turn in place, pivoting around the left wheel to bring the left arm into position
							Speed = SPEED_REV_MED_SLOW;
							ROBOT_LOG( TRUE,"Pickup Object Backing up while turning - Object too close\n")
						}
						else if( m_pDriveCtrl->RobotStopped() )
						{
							// Stopped just short of the target.  Move forward
							Speed = SPEED_FWD_MED_SLOW;
						}
					}
					else
					{
						// Still heading toward the object
						if( !m_pDriveCtrl->RobotStopped() )
						{
							// Don't slow when first starting - might stall
							if( m_ObjectDistanceTenthInches < 180 )
							{
								Speed = SPEED_FWD_SLOW;
							}
						}
					}

					int Turn = 0;
					if( m_ObjectDirectionDegrees > 40 )
					{
						Turn = TURN_RIGHT_MED_SLOW;
					}
					else if( m_ObjectDirectionDegrees > 15 )
					{
						Turn = TURN_RIGHT_SLOW;
					}
					else if( m_ObjectDirectionDegrees > 5 )
					{
						Turn = TURN_RIGHT_VERY_SLOW;
					}
					else if( m_ObjectDirectionDegrees > -5 )
					{
						Turn = TURN_CENTER;
					}
					else if( m_ObjectDirectionDegrees > -15 )
					{
						Turn = TURN_LEFT_VERY_SLOW;
					}
					else if( m_ObjectDirectionDegrees > -40 )
					{
						Turn = TURN_LEFT_SLOW;
					}
					else
					{
						Turn = TURN_LEFT_MED_SLOW;
					}

					m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, Speed, Turn );
					ROBOT_LOG( TRUE,"OBJECT_TASK_MOVE_TO_OBJECT: Speed = %d, Turn = %d\n", Speed, Turn)

					m_KinectSearchComplete = FALSE;	// Reset flag for the next scan
					nKinectObjectsDetected = 0;
					SendCommand( WM_ROBOT_KINECT_TRACK_OBJECT_CMD, (DWORD)1, (DWORD)0 ); 	// Enable Tracking
					__itt_marker(pDomainControlThread, __itt_null, pshKinectMoveToObject, __itt_marker_scope_task);

				}
			}
			break;
		}	// OBJECT_TASK_MOVE_TO_OBJECT

		case OBJECT_TASK_PICK_UP_OBJECT:	// Now, Pick up the object
		{	
			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case 1:	
				{
					// Object found.  Tell Arm Behavior to pickup the nearest one
					m_ObjectPickupComplete = FALSE;
					m_PutObjectInCarryBasket = TRUE; // And, tell the arm to toss it in the basket!!!
					CString MsgString;
					MsgString.Format( "OBJECT_TASK_PICK_UP_OBJECT: Found Object to Pick up at Y =%4.1f, X =%4.1f\n    Picking up... \n",
						(double)m_ObjectY/10.0, (double)m_ObjectX/10.0 );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					if( (m_ObjectX > KINECT_RANGE_TENTH_INCHES_MAX) || (m_ObjectY > KINECT_RANGE_TENTH_INCHES_MAX) ) 
					{
						ROBOT_ASSERT(0);
					}
					//__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectTrackingStart, __itt_marker_scope_task);

									
					// Tell Arm Behavior to pickup the object
					/*if( m_ObjectX >= 0 )
					{	// Object on Right side
						ROBOT_ASSERT(0); // TODO-MUST! not implemented!
						SendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_PICKUP_OBJECT_XYZ );
					}
					else
					*/
					// Object on Left side
					SendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_PICKUP_OBJECT_XYZ );
					m_TaskState = 2;								
					break;
				}
				case 2:	
				{	
					// Wait for the Arm Behavior to pick up the object
					if( !m_ObjectPickupComplete )
					{
						break;	// continue waiting
					}

					// Pickup Done!  Now look for the next object.
					// m_nObjectsPickedUp++; - Done in LEFT ARM control (TODO - need to do Right also)

					//m_pKinectServoControl->SetTiltPosition( 0 );	// Center Kinect
					//m_pKinectServoControl->SetTiltPosition( KINECT_TILT_OWNER_TRACK_OBJECT, 0 );	// Center Kinect
					//m_pKinectServoControl->ReleaseOwner(KINECT_TILT_OWNER_TRACK_OBJECT);

					//__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupNoObject, __itt_marker_scope_task);

					ROBOT_DISPLAY( TRUE, "OBJECT_TASK_PICK_UP_OBJECT: DONE PICKING UP OBJECT!  Looking for next object." )

					/* TODO - Needs DEBUG!
					// Note:  if fails to pickup object, don't do this!!!:
					// Turn back to direction we started in
					if( m_ObjectDirectionDegreesPeak > 0 )
					{	// Turned right to get to object, so turn Left to look back
						SendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, (DWORD)(abs(m_ObjectDirectionDegreesPeak)), TURN_LEFT_MED );	// wParam = distance in degrees, lParam = direction and speed
					}
					else
					{
						SendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, (DWORD)(abs(m_ObjectDirectionDegreesPeak)), TURN_RIGHT_MED );	// wParam = distance in degrees, lParam = direction and speed
					}
					*/
					m_CurrentTask = OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT;	// Start again looking for any objects nearby
					m_TaskState = 1;	// go to first state
					break;
				}
				default: ROBOT_ASSERT(0);
			}  // m_TaskState
			break;
		}	// OBJECT_TASK_PICK_UP_OBJECT

		case OBJECT_TASK_LOOK_WHILE_MOVING:
		{
			// DEBUG - TODO:  For now, just ABORT
			ROBOT_DISPLAY( TRUE, "*** ERROR: OBJECT_TASK_LOOK_WHILE_MOVING:  NOT ENABLED! ***" )
			m_pDriveCtrl->SetSpeedAndTurn( BEHAVIOR_GOAL_MODULE, SPEED_STOP, TURN_CENTER );
			m_CurrentTask = OBJECT_TASK_DONE;
			m_TaskState = 1;
			break;


			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case 1:	// Wait for Servo to reach commanded position
				{	
					//if ( m_pKinectServoControl->CheckServoPosition(FALSE) )
					{
						// Kinect in position, go to next state (Kinect should have done a scan by then)
						m_TaskState++;	
					}
					// else, keep waiting
					break;
				}
				case 2:	// Get result at current Kinect Tilt Position
				{
					// Start robot moving forward, will look while moving
					// TODO - get this working after basic object search working!
					// For now, just ABORT
					m_CurrentTask = 0;
					m_TaskState = 0;

					//m_TaskState++;
					break;
				}
				case 3:	// Get result at current Kinect Tilt Position
				{	

					//int nObjectsFound = FindObjectsOnFloor();

					// if objects found, go to next state, else keep on trying
					//if( nObjectsFound > 0 )
					{
						m_TaskState = KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE; // Object found
					}
					break;
				}
			}

			// check every update for an object
			//if( m_pLaserScannerBehavior->LookForObjectsWhileRobotIsMoving() )
			//{
			//	m_CurrentTask = OBJECT_TASK_MOVE_TO_OBJECT;
			//}
			break;

		}	// OBJECT_TASK_LOOK_WHILE_MOVING

		default: ROBOT_ASSERT(0);
	}	// OBJECT_TASK_PICKUP_OBJECT
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::ActionPickupCloseObject()
{
	// Look for nearest object
	// if object found, determine if best to pick on on left or right side
	// pick up
	switch( m_CurrentTask )
	{
		case OBJECT_TASK_NONE:
		{
			break;	// Nothing to do
		}

		case OBJECT_TASK_DONE: // Done looking!
		{
			__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupNoObject, __itt_marker_scope_task);
			if( 0 != m_TaskState )
			{	// Note: 0 indicates "don't say anything, just quit"
				if( m_nObjectsPickedUp > 0 )
				{
					ROBOT_DISPLAY( TRUE, "BEHAVIOR MODULE:ActionPickupCloseObject: I don't see any more objects to pick up.  DONE" )
					g_ClientTextToSend.Format( "All done.  My work here is finished." );
					SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
				}
				else
				{
					ROBOT_DISPLAY( TRUE, "BEHAVIOR MODULE: No Object found to Pick up.  ABORTING" )
					g_ClientTextToSend.Format( "I don't see anything to pick up" );
					SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
				}
			}
			EndActionMode();
			break;
		}

		case OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT:	// Look for the nearest object
		{	
			__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectStart, __itt_marker_scope_task);

			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case KINECT_FLOOR_SCAN_MOVE_ARMS_STATE:
				{	
					// Make sure arms are not in the way of the Kinect View
					ROBOT_LOG( TRUE,"ACTION_MODE_PICKUP_OBJECTS Moving Arms Home\n")
					m_pArmControlLeft->SetArmPosition( LEFT_ARM_SHOULDER_HOME1, LEFT_ARM_ELBOW_ROTATE_HOME1, 
						LEFT_ARM_ELBOW_BEND_HOME1, LEFT_ARM_WRIST_ROTATE_HOME1, LEFT_ARM_CLAW_HOME1 );  // Explicitly close the claw
					m_pArmControlLeft->ExecutePositionAndSpeed();

					m_pArmControlRight->SetArmPosition( LEFT_ARM_SHOULDER_HOME1, LEFT_ARM_ELBOW_ROTATE_HOME1, 
						LEFT_ARM_ELBOW_BEND_HOME1, LEFT_ARM_WRIST_ROTATE_HOME1, LEFT_ARM_CLAW_HOME1 );  // Explicitly close the claw
					m_pArmControlRight->ExecutePositionAndSpeed();

					// Point the head looking at the floor too (just for affect)
					gHeadIdle = FALSE;
					m_pHeadControl->SetHeadSpeed( HEAD_OWNER_BEHAVIOR_P1, SERVO_SPEED_MED, SERVO_SPEED_MED, SERVO_SPEED_MED );
					m_pHeadControl->SetHeadPosition( HEAD_OWNER_BEHAVIOR_P1, CAMERA_PAN_CENTER, CAMERA_TILT_CENTER-450, CAMERA_SIDETILT_CENTER );
					m_pHeadControl->ExecutePositionAndSpeed( HEAD_OWNER_BEHAVIOR_P1 );
					m_TaskState++;
				}
				case KINECT_FLOOR_SCAN_START_SEARCH_STATE:	// Request a scan by the Kinect Module
				{
					ROBOT_LOG( TRUE,"Behavior: KINECT_FLOOR_SCAN_START_SEARCH_STATE Sending WM_ROBOT_KINECT_SEARCH_FLOOR_CMD\n")
					m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
					nKinectObjectsDetected = 0;
					SendCommand( WM_ROBOT_KINECT_SEARCH_FLOOR_CMD, (DWORD)TRUE, (DWORD)0 ); // TRUE = Find Close Objects ONLY
					m_TaskState++;
					break;
				}
				case KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE:	// See if we are done waiting
				{	
					if( !m_KinectSearchComplete )
					{
						//ROBOT_LOG( TRUE,"KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE: Waiting for m_KinectSearchComplete 2\n")
						break;	// continue waiting
					}
					else
					{
						__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectSearchComplete, __itt_marker_scope_task);

						// Done waiting for the search to complete.
			
						#if DEBUG_KINECT_FLOOR_OBJECT_DETECTION	== 1
						// ********* TO DEBUG OBJECT DETECTION **********
							//*
							ROBOT_DISPLAY( TRUE, "*** LOOPING FOR DEBUG PURPOSES! ****")
							ROBOT_DISPLAY( TRUE, "********************************************\n" )
							// DEBUG DEBUG Loop on find object for debugging object detection
							ROBOT_LOG( TRUE,"DEBUG Behavior: KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE Looping - Sending WM_ROBOT_KINECT_SEARCH_FLOOR_CMD\n")
							m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
							nKinectObjectsDetected = 0;
							SendCommand( WM_ROBOT_KINECT_SEARCH_FLOOR_CMD, (DWORD)TRUE, (DWORD)0 ); // TRUE = Find Close Objects ONLY
							m_TaskState = KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE;
							break;
						#endif
						// **************************************



						if( nKinectObjectsDetected > 0 )
						{	
							// Object found.																								
							m_TaskState = KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE;
						}
						else
						{	// No object found.
							m_CurrentTask = OBJECT_TASK_DONE;
							m_TaskState = 1;
						}
					}
					break;
				}
				case KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE:	// Object found!
				{
					// Objects found.  Calculate path to the nearest one
				//	m_ObjectX = g_pKinectObjects3D->Object[0].CenterX;
				//	m_ObjectY = g_pKinectObjects3D->Object[0].CenterY;

					CString MsgString;
					MsgString.Format( "BEHAVIOR MODULE: Found Object to Pick up at Y =%4.1f, X =%4.1f\n",
						(double)m_ObjectY/10.0, (double)m_ObjectX/10.0 );
					ROBOT_DISPLAY( TRUE, (LPCTSTR)MsgString )
					if( (m_ObjectX > KINECT_RANGE_TENTH_INCHES_MAX) || (m_ObjectY > KINECT_RANGE_TENTH_INCHES_MAX) )
					{
						ROBOT_ASSERT(0);
					}

					if( m_pArmControlLeft->IsObjectInPickupZone(m_ObjectY, m_ObjectX) )
					{
						#if DEBUG_KINECT_FLOOR_OBJECT_DETECTION	== 1
							// ********* TO DEBUG OBJECT DETECTION **********
							ROBOT_DISPLAY( TRUE, "\n*** OBJECT FOUND AND IN RANGE, BUT PICKUP DISABLED FOR DEBUG PURPOSES! ****")
							ROBOT_DISPLAY( TRUE, "********************************************\n" )
							// DEBUG: Loop on find object for debugging object detection
							ROBOT_LOG( TRUE,"DEBUG Behavior: KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE Looping - Sending WM_ROBOT_KINECT_SEARCH_FLOOR_CMD\n")
							m_KinectSearchComplete = FALSE;	// Set flag so we know when the search is done.
							nKinectObjectsDetected = 0;
							SendCommand( WM_ROBOT_KINECT_SEARCH_FLOOR_CMD, (DWORD)TRUE, (DWORD)0 ); // TRUE = Find Close Objects ONLY
							m_TaskState = KINECT_FLOOR_SCAN_WAIT_SEARCH_STATE;
							break;
							// **************************************
						#endif

						// Object within range, pick it up.
						__itt_marker(pDomainControlThread, __itt_null, pshKinectArrivedAtObject, __itt_marker_scope_task);
						ROBOT_DISPLAY( TRUE, "KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE - Object found\n" )
						// Point head at object for realism.  Note Z assumed to be 1 inch above the floor (doesn't really matter)
						m_pHeadControl->SetHeadSpeed( HEAD_OWNER_BEHAVIOR_P1, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW );
						m_pHeadControl->LookAtXYZ( HEAD_OWNER_BEHAVIOR_P1, m_ObjectX,(m_ObjectY+100), 10 ); // Pad Y so camera does not bottom out
						m_CurrentTask = OBJECT_TASK_PICK_UP_OBJECT;
						m_TaskState = 1;
					}
					else
					{
						// Out of range, abort
						g_ClientTextToSend = _T( "The object is too far away to pick up" );
						SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
						ROBOT_DISPLAY( TRUE, "KINECT_FLOOR_SCAN_OBJECT_FOUND_STATE - Object Too far away to pickup without moving\n" )
						gArmTimerLeft = 30; // 10 second delay - Give time for Loki to speak this phrase!
						m_CurrentTask = OBJECT_TASK_DONE;
						m_TaskState = 0;	// Silence further talking
					}
					break;
				}
				default: ROBOT_ASSERT(0);
			}
			break;

		}	// OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT


		case OBJECT_TASK_PICK_UP_OBJECT:	// Now, Pick up the object
		{	
			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case 1:	
				{
					// Object found.  Tell Arm Behavior to pickup the nearest one
					m_ObjectPickupComplete = FALSE;
					//m_PutObjectInCarryBasket = TRUE; // And, tell the arm to toss it in the basket!!!
					CString MsgString;
					MsgString.Format( "OBJECT_TASK_PICK_UP_OBJECT: Found Object to Pick up at Y =%4.1f, X =%4.1f\n    Picking up... \n",
						(double)m_ObjectY/10.0, (double)m_ObjectX/10.0 );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					if( (m_ObjectX > KINECT_RANGE_TENTH_INCHES_MAX) || (m_ObjectY > KINECT_RANGE_TENTH_INCHES_MAX) ) 
					{
						ROBOT_ASSERT(0);
					}
					//__itt_marker(pDomainControlThread, __itt_null, pshKinectPickupObjectTrackingStart, __itt_marker_scope_task);

									
					// Tell Arm Behavior to pickup the object
					if( m_ObjectX >= 0 )
					{	// Object on Right side
						ROBOT_ASSERT(0); // TODO-MUST! not implemented!
						SendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_PICKUP_OBJECT_XYZ );
					}
					else
					{	// Object on Left side
						SendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_PICKUP_OBJECT_XYZ );
					}
					m_TaskState = 2;								
					break;
				}
				case 2:	
				{	
					// Wait for the Arm Behavior to pick up the object
					if( !m_ObjectPickupComplete )
					{
						break;	// continue waiting
					}

					// Pickup Done!  Now look for the next object.
					ROBOT_DISPLAY( TRUE, "OBJECT_TASK_PICK_UP_OBJECT: DONE PICKING UP OBJECT!" )
					EndActionMode();
					break;
				}
				default: ROBOT_ASSERT(0);
			}  // m_TaskState
			break;
		}	// OBJECT_TASK_PICK_UP_OBJECT
							
		default: 
			{
				ROBOT_LOG( TRUE,"ERROR - BAD STATE for m_CurrentTask = %d\n", m_CurrentTask )
				ROBOT_ASSERT(0);
			}
	}	// OBJECT_TASK_PICKUP_OBJECT


}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::ActionOpenDoor()
{
	switch( m_CurrentTask )
	{
		case 0:
		{
			break;	// Nothing to do
		}
		case 1: // Open the Door
		{
			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
					// Example: don't do anything if arm not yet in position
					//          if( m_pArmControlLeft->CheckArmPosition(FALSE) )	//	TRUE = Verbose
				}
				case 1: // Locate handle (visually) and drive to correct position
				{
					// TODO
					m_TaskState++;	// go to next task state
					break;
				}
				case 2: // Move body into position to reach the handle
				{
					{
						/*
						const UINT OPTIMAL_DISTANCE_FROM_DOOR = 12;
						UINT nDistanceToMove = g_pSensorSummary->nFrontObjectDistance - OPTIMAL_DISTANCE_FROM_DOOR;	// inches
						// Move body so that the claw is approx "N" inches from the door
						if ( nDistanceToMove > 0 )
						{
							SendCommand( WM_ROBOT_MOVE_SET_DISTANCE_CMD, (DWORD)nDistanceToMoveTenthInches, FORWARD );	// wParam = distance in inches, lParam = direction
						}
						else
						{
							SendCommand( WM_ROBOT_MOVE_SET_DISTANCE_CMD, (DWORD)nDistanceToMoveTenthInches, REVERSE );	// wParam = distance in inches, lParam = direction
						}
						*/

						m_TaskState++;	// go to next task state
					}
										
					break;
				}

				case 3: // Move arm into position to reach the handle
				{
					{
					/**
						UINT nDistanceToMove = g_pSensorSummary->nFrontObjectDistance - OPTIMAL_DISTANCE_FROM_DOOR;	// inches???
						// Move body so that the claw is approx "N" Tenth inches from the door
						if ( nDistanceToMove > 0 )
						{
							SendCommand( WM_ROBOT_MOVE_SET_DISTANCE_CMD, (DWORD)nDistanceToMoveTenthInches, FORWARD );	// wParam = distance in inches, lParam = direction
						}
						else
						{
							SendCommand( WM_ROBOT_MOVE_SET_DISTANCE_CMD, (DWORD)nDistanceToMoveTenthInches, REVERSE );	// wParam = distance in inches, lParam = direction
						}
											
					**/
						m_TaskState++;	// go to next task state
					}
										
					break;
				}
				case 4: // Find handle, then grasp and turn it
				{
					//m_CurrentTask++; 
					TaskOpenDoor();
					break;
				}
				case 5: // When done, move body back (open door) 
				{
					/***
					// Wait until handle turned
					if( m_TurnHandleState = TURN_HANDLE_STATE_SUCCESS )
					{
						m_TaskState++;	// go to next task state
					}
					else if( m_TurnHandleState = TURN_HANDLE_STATE_FAIL )
					{
						m_TaskState+=2;	// go to failed task state
					}
					**/
					m_TaskState++;
					break;
				}
				case 6:	// Done - Sucess!
				{
					m_TaskState = 0;	// back to Idle state
					m_CurrentActionMode = ACTION_MODE_NONE; // back to Idle state
					break;
				}
				case 7:	// Done - Failed!
				{
					m_TaskState = 0;	// back to Idle state
					m_CurrentActionMode = ACTION_MODE_NONE; // back to Idle state
					break;
				}
				default: ROBOT_ASSERT(0);
			}
			break;
		}
		case 2: // Say "Now what?"
		{
			break;
		}
		default: ROBOT_ASSERT(0);
	}

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CBehaviorModule::ActionGetBeer()
{	switch( m_CurrentTask )
	{
		case 0:
		{
			break;	// Nothing to do
		}
		case 1:	// Go to Fridge
		{
			//m_pArmControlRight->SetArmPosition( NOP, NOP, -7, 90, NOP ); // Shoulder, ElbowRotate, ElbowBend, Wrist, Grip
			//m_pArmControlRight->ExecutePosition();
			//m_ArmMovementStateRight++;
			//m_ArmWaitForMoveToCompleteRight = TRUE;
			m_CurrentTask++; // SKIP THIS STEP FOR NOW! TODO!
			break;
		}
		case 2: // Open Fridge Door
		{
			switch( m_TaskState )
			{
				case 0:
				{
					break;	// Nothing to do
				}
				case 1: // Locate handle
				{
					break;
				}
				case 2: // Move Arm into position
				{
					break;
				}
				case 3: // Move body into position
				{
					break;
				}
				case 4: // Grasp handle
				{
					break;
				}
				case 5: // Move body back (open door)
				{
					break;
				}
				default: ROBOT_ASSERT(0);
			}
			break;
		}
		case 3: // Get Beer
		{
			break;
		}
		case 4: // Close Fridge Door
		{
			break;
		}
		case 5: // Take Beer to human
		{
			break;
		}
		default: ROBOT_ASSERT(0);
	}
}

#endif // ROBOT_SERVER
