#ifndef __CAMERA_DETECTORS_MODULE_H__
#define __CAMERA_DETECTORS_MODULE_H__

//#include "Thread.h"
#include "RobotType.h"
#include "../Common/HardwareCmds.h"
#include "PathStruct.h"
#include "Globals.h"
//#include "MotorControl.h"

#include "cv.h"
//#include "cvcam.h"
#include "highgui.h"

using namespace std;

#include "cxmisc.h"
//#include "highgui.h"
//#include "cvaux.h"
#include <vector>
#include <string>
#include <algorithm>
#include <stdio.h>
#include <ctype.h>

//#include "ObjectKnowledge.h"

/////////////////////////////////////////////////////////////////////////////
class CCameraDetector
{

public:
	CCameraDetector( IplImage* pVideoFrame );	// May be Left or Right camera frame!
	~CCameraDetector(){};

	// Common Member valiables
	IplImage*	m_pVideoFrame;	// Pointer to the current Video Frame to process

	// Common Functions handled by base class
	CvSize	GetFrameSize() 
	{ return cvGetSize(m_pVideoFrame); }

	CvPoint	GetFrameCenter()
	{ return cvPoint( (m_pVideoFrame->width / 2), (m_pVideoFrame->height / 2) ); }

	CvPoint2D32f GetFrameCenterFloat()	// float version
	{ return cvPoint2D32f( ((double)m_pVideoFrame->width / 2.0), ((double)m_pVideoFrame->height / 2.0) ); }

	double	GetFrameAspectRatio()
	{ return ( (double)m_pVideoFrame->height/(double)m_pVideoFrame->width ); }

	// Overrides
	//virtual void Initialize();

};

/////////////////////////////////////////////////////////////////////////////
class CFaceDetector : public CCameraDetector
{

//#define	MIN_OBJECT_SIZE			100	// component size
//#define MOTION_RING_BUF_SIZE	  3	// number of cyclic frame buffer used for motion detection
									// (should, probably, depend on FPS)

#define CAMERA_FACE_DETECTOR_PATH ROBOT_DATA_PATH "\\OpenCV\\data\\haarcascades\\haarcascade_frontalface_default.xml"
// Not used (yet):#define CAMERA_UPPER_BODY_DETECTOR_PATH "C:/RobotData/OpenCV/data/haarcascades/haarcascade_upperbody.xml"


public:
	CFaceDetector(IplImage* pVideoFrame);
	~CFaceDetector();

protected:
	CvHaarClassifierCascade*	m_pFaceDetectCascade;	// the face detector
	CvMemStorage*				m_pFaceDetectStorage;	// memory for the face detector to use
//	CvSeq*						m_pFaceRectSeq;	// memory-access interface
	char*						m_FaceDetectorXMLPath;
	int 					m_FaceTrackingState;
	CvRect						m_FaceRect;

public:
	CvSeq*	DetectFaces();


};

/////////////////////////////////////////////////////////////////////////////
class CMotionDetector : public CCameraDetector
{

#define	MIN_OBJECT_SIZE			100	// component size
#define MOTION_RING_BUF_SIZE	  3	// (4) number of cyclic frame buffer used for motion detection
									// (should, probably, depend on FPS)
public:
	CMotionDetector(IplImage* pVideoFrame);
	~CMotionDetector();

//	void	Initialize();
//	void	InitMotionDetector();
//	void	ReleaseMotionDetector();

	CvPoint	DetectMotion( IplImage* pMotionDisplayFrame );


protected:

	// ring image buffer
	IplImage **ppMotionRingBuffer; 
	int m_Last;

	// temporary images
	IplImage*		mhi;		// MHI
	IplImage*		orient;		// orientation
	IplImage*		mask;		// valid orientation mask
	IplImage*		segmask;	// motion segmentation map
	CvMemStorage*	storage;	// temporary storage

};

/////////////////////////////////////////////////////////////////////////////
class CCamShiftDetector : public CCameraDetector
{

									// (should, probably, depend on FPS)
public:
	CCamShiftDetector(IplImage* pVideoFrame);
	~CCamShiftDetector();

protected:
	int					nHistBins;				// number of histogram bins
	float				rangesArr[2];			// histogram range
	CvRect				m_prevColorRect;		// location of color blob in previous frame
	int					vmin;
	int					vmax; 
	int					smin;				// limits for calculating hue
	CvRect				m_FooRect;			// location of color blob in previous frame

	IplImage*			pHSVImg;			// the input image converted to HSV color mode
	IplImage*			pHueImg;			// the Hue channel of the HSV image
	IplImage*			pMask;				// this image is used for masking pixels
	IplImage*			pProbImg;			// the face probability estimates for each pixel
	CvHistogram*		pHist;				// histogram of hue in the original face image

	CvPoint				m_CamShiftCenter;	// current face-location estimate from CamShift
	int 				m_State;			// State of the tracker
	//	int					nFrames;


public:
	void	startTracking( CvRect ColorRect );
	CvBox2D track();
	void	updateHueImage(const IplImage * pImg);
	void	setVmin(int _vmin);
	void	setSmin(int _smin);
	int 	GetState() { return m_State; }


protected:


};


/////////////////////////////////////////////////////////////////////////////
class CColorDetector : public CCameraDetector
{

public:
	CColorDetector(IplImage* pVideoFrame);
	~CColorDetector();

protected:

	int 				m_CrTarget;			// Red Target value
	int 				m_CbTarget;			// Blue Target value
	int					m_CrThreshold;
	int					m_CbThreshold; 

	IplImage*			pYCrCbImg;			// the YCrCb version of the image
//	IplImage*			pCrImg;				// the Cr channel of the YCrCb image
//	IplImage*			pCbImg;				// the Cb channel of the YCrCb image

	CvPoint				m_ColorCenter;	// current face-location estimate from Color
	BOOL				m_bInitialized;
	int 				m_CrMax;
	int 				m_CrMin;
	int 				m_CbMax;
	int 				m_CbMin;

public:
	int 	GetCrTarget() { return m_CrTarget; }
	int 	GetCbTarget() { return m_CbTarget; }

	void	SetColor( CvRect ColorRect );	// Auto set Cr, Cb from current image
	void	SetColor( int  Cr, int  Cb );	// Manually set Cr, Cb


	// Returns Center of Color Blob
	CvPoint FindColorBlob( IplImage* pColorBlobDisplayFrame, // debug display image
				CvPoint &BBpt1, CvPoint &BBpt2 );			 // bounding box of Color Blob

	void	setCrThreshold(int Threshold);
	void	setCbThreshold(int Threshold);
	BOOL	Initialized() { return m_bInitialized; }

	void	UpdateCrCbImages(const IplImage * pImg); // CrCb based

protected:


};

/////////////////////////////////////////////////////////////////////////////
class CLaserSpotDetector : public CCameraDetector
{

public:
	CLaserSpotDetector(IplImage* pVideoFrame);
	~CLaserSpotDetector();

protected:

	int					m_nPositiveFrames;	// Number of frames in a row where laser was detected
	CvPoint				m_LastSpotPosition; // Position of laser found in prior frame

//	IplImage*			pYCrCbImg;			// the YCrCb version of the image
//	IplImage*			pCrImg;				// the Cr channel of the YCrCb image
//	IplImage*			pCbImg;				// the Cb channel of the YCrCb image

//	CvPoint				m_ColorCenter;	// current face-location estimate from Color
//	BOOL				m_bInitialized;

public:

	// Returns position of Laser Spot. If not found, returns {0,0}
	CvPoint CLaserSpotDetector::FindLaserDot();

//	BOOL	Initialized() { return m_bInitialized; }
//	void	UpdateCrCbImages(const IplImage * pImg); // CrCb based

protected:


};

/////////////////////////////////////////////////////////////////////////////
typedef struct
{
	int  value;
	int  position;
} LINE_PIXEL_T;

class CLaserLineDetector : public CCameraDetector
{

public:
	CLaserLineDetector(IplImage* pVideoFrame);
	~CLaserLineDetector();

protected:
	int					m_nPositiveFrames;	// Number of frames in a row where laser was detected
	CvPoint				m_LastSpotPosition; // Position of laser found in prior frame
	int					m_width;			// Frame Width and Height
	int					m_height;

	LINE_PIXEL_T*		m_pLaserDotArray;	// Value and positon of laser points for each column in the frame

//	IplImage*			pYCrCbImg;			// the YCrCb version of the image
//	IplImage*			pCrImg;				// the Cr channel of the YCrCb image
//	IplImage*			pCbImg;				// the Cb channel of the YCrCb image

//	CvPoint				m_ColorCenter;	// current face-location estimate from Color
//	BOOL				m_bInitialized;

public:

	// Find Laser lines in the frame. If none found, returns FALSE
	BOOL CLaserLineDetector::FindLaserLines();
	int  CLaserLineDetector::GetPositionY(int  X);

protected:


};

/////////////////////////////////////////////////////////////////////////////
class CTrackObjectDetector : public CCameraDetector
{

public:
	CTrackObjectDetector(IplImage* pVideoFrame);
	~CTrackObjectDetector();

protected:

	IplImage			*m_GrayImg;
	IplImage			*m_PreviousGrayImg;
	IplImage			*m_Pyramid;
	IplImage			*m_PreviousPyramid;
	IplImage			*m_SwapTemp;

	int					m_width;
	int					m_height;
	int					m_WindowSize;
	CvPoint2D32f		*m_Points[2];
	CvPoint2D32f		*m_SwapPoints;
	char* 				m_Status;
	int 				m_FeatureCount;
	int 				m_Flags;
	int 				add_remove_pt;
	CvPoint 			m_Pt;
	CvPoint				m_BoxPt1;
	CvPoint				m_BoxPt2;
	CvPoint				m_CenterPt;
	CvPoint2D32f		m_ReferenceCloudCenter;


public:

	// Find Laser lines in the frame. If none found, returns FALSE
	void Init();
	BOOL LockOn( BOOL FindWallSocket = FALSE );
	BOOL GetDelta( CvPoint &ObjectDelta );

protected:


};


/////////////////////////////////////////////////////////////////////////////
class CMatchObjectDetector : public CCameraDetector
{

public:
	CMatchObjectDetector(IplImage* pVideoFrame);
	~CMatchObjectDetector();

protected:

	BOOL				m_bInitialized;

///	CvSeq				*m_imageKeypoints;		// Current Video Frame
///	CvSeq				*m_imageDescriptors;

//	ObjectKnowledge		*g_pObjectKnowledge;	// Database of Object info


public:

	BOOL				Initialize();
	POSITION			LookForMatchingObject();

	BOOL				Initialized() { return m_bInitialized; }
///	void				DeleteAllSURFInfoItems();


///	SURFInfoList*		GetSURFInfoList()  { return m_pSURFInfoList; }


	vector<int> ptpairs;

	// Utility functions

	double				compareSURFDescriptors( const double* d1, const double* d2, double best, int length );

	int					naiveNearestNeighbor( const double* vec, int laplacian, 
							const CvSeq* model_keypoints, const CvSeq* model_descriptors );

	void				findPairs( const CvSeq* objectKeypoints, const CvSeq* objectDescriptors, 
							const CvSeq* imageKeypoints, const CvSeq* imageDescriptors, vector<int>& ptpairs );

	/* a rough implementation for object location */
	int					locatePlanarObject( const CvSeq* objectKeypoints, const CvSeq* objectDescriptors,	
							const CvSeq* imageKeypoints, const CvSeq* imageDescriptors,	const CvPoint src_corners[4], CvPoint dst_corners[4] );



protected:

};




/////////////////////////////////////////////////////////////////////////////
class CStereoDetector : public CCameraDetector
{

public:
	CStereoDetector(IplImage* pVideoFrame);
	~CStereoDetector();


protected:


    CvSize imageSize;

	CvMat* mx1;
	CvMat* my1;
	CvMat* mx2;
	CvMat* my2;
	CvMat* img1r;
	CvMat* img2r;
	CvMat* disp;
	CvMat* disp3d;
	CvMat* vdisp;
	CvMat* pair;
//	double R1[3][3], R2[3][3], P1[3][4], P2[3][4];
//	CvMat _R1 = cvMat(3, 3, CV_64F, R1);
//	CvMat _R2 = cvMat(3, 3, CV_64F, R2);




//	int					m_nPositiveFrames;	// Number of frames in a row where laser was detected
//	CvPoint				m_LastSpotPosition; // Position of laser found in prior frame

//	IplImage*			pYCrCbImg;			// the YCrCb version of the image
//	IplImage*			pCrImg;				// the Cr channel of the YCrCb image
//	IplImage*			pCbImg;				// the Cb channel of the YCrCb image

//	CvPoint				m_ColorCenter;	// current face-location estimate from Color
//	BOOL				m_bInitialized;

public:

	//void CStereoDetector::Calibrate();
	void CStereoDetector::Calibrate( const char* imageList, int nx, int ny, int useUncalibrated );
	void CStereoDetector::ProcessFrames( IplImage* pVideoFrameLeft, IplImage* pVideoFrameRight );


//	BOOL	Initialized() { return m_bInitialized; }
//	void	UpdateCrCbImages(const IplImage * pImg); // CrCb based

protected:


};






#endif // __CAMERA_DETECTORS_MODULE_H__