// CameraModule.cpp: CCameraModule class implementation
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "ClientOrServer.h"
#if ( ROBOT_SERVER == 1 )	// This module used for Robot Server only

#include <math.h>
//#include <MMSystem.h>	// For Sound functions
#include "Globals.h"
#include "module.h"
#include "thread.h"
#include "videoInput.h" 
#include "cv.h"
//#include "cvcam.h"
#include "highgui.h"
#include "RobotSharedParams.h"
//#include <CLNUIDevice.h>	// Kinect Sensor

// For Kinect OpenNI library
//#include <XnOS.h>
//#include <math.h>
//#include <XnCppWrapper.h>
//using namespace xn;

///////////////////////////////////////////////////////////////////////////////
// OpenNI Stuff

//---------------------------------------------------------------------------
// Defines
//---------------------------------------------------------------------------
//#define SAMPLE_XML_PATH "../../../Data/SamplesConfig.xml"
#define SAMPLE_XML_PATH "C:\\Program Files (x86)\\OpenNI\\Data\\SamplesConfig.xml"

#define GL_WIN_SIZE_X 1280
#define GL_WIN_SIZE_Y 1024

#define DISPLAY_MODE_OVERLAY	1
#define DISPLAY_MODE_DEPTH		2
#define DISPLAY_MODE_IMAGE		3
#define DEFAULT_DISPLAY_MODE	DISPLAY_MODE_DEPTH

#define MAX_DEPTH 10000

//---------------------------------------------------------------------------
// Globals
//---------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
// Performance Tuning
#define FRAME_SKIP_COUNT			1	// how many frames to skip between each processing (Lower number increases CPU load) 
//#define MIN_FACE_SIZE			   10	// how small a face to find.  5 default. larger number = more CPU

#define CAMERA_BLUR_SETTLE_TIME	   10	// After moving head, allow motion blur to settle before processing
#define FACE_CHECK_INTERVAL		   10	// For use when using Face Tracking and CamShift color tracking
#define MOTION_DETECTOR_TIME	 1500	// Allow extra time for motion detector to settle
#define STEREO_DETECTOR_TIME	 2000	// Don't process stereo too often (processor intensive)

//#define CAPTURE_FRAMES_PER_SECOND	30	// FPS to sample incoming frames
#define SAMPLE_FRAME_RATE_MS			100 //65 = 15fps, 100 = 10fps, 200 = 5fps
#define CAPTURE_FRAME_RATE_FPS			10  // FPS at which Camera Captures!
///////////////////////////////////////////////////////////////////////////////

// Use for 320 x 240 window size
#define OPTIMAL_WINDOW_BOARDER	200 // Amount of space in the view around target window
#define WINDOW_BOARDER_FUDGE	40	// Hysterysis to avoid constant zoom in and out

// Use for 160 x 120 window size
//#define OPTIMAL_WINDOW_BOARDER	80 // Amount of space in the view around target window
//#define WINDOW_BOARDER_FUDGE	10	// Hysterysis to avoid constant zoom in and out

// FACE TRACKING STATE MACHINE
#define STATE_IDLE							0
#define STATE_LOOK_FOR_FACE					1
#define STATE_FACE_FOUND					2 
#define STATE_CAMSHIFT_TRACKING				3

#define CAMSHIFT_FACE_SIZE_MIN			   20	// if CamShift smaller then this, switch back to LOOK_FOR_FACE
#define CAMERA_PIXEL_FUDGE					3  // pixel error to allow before moving camera

// Video Record Modes:
#define VIDEO_RECORD_IDLE					0x00
#define VIDEO_RECORD_START					0x01
#define VIDEO_RECORD_RECORDING				0x02
#define VIDEO_RECORD_STOP					0x03


// Camera Color Blob Tracking modes
#define IDLE								0x00
//#define CAMERA_MODE_LOOK_AHEAD			0x01
#define CAMERA_MODE_TRACKING_COLOR			0x01
#define CAMERA_MODE_LOOKING_LEFT			0x02
#define CAMERA_MODE_LOOKING_RIGHT			0x03
#define CAMERA_MODE_LOOKING_CENTER			0x04

#define CAMERA_MODE_SCAN_PAN_LEFT			0x05
#define CAMERA_MODE_SCAN_PAN_RIGHT			0x06
//#define CAMERA_MODE_SCAN					0x03	// Numbers greater then this are only for SCANNING!???
//#define CAMERA_PROGRESSIVE_SCAN				0x06
//#define CAMERA_MODE_LOOK_LEFT				0x07	// quick look one frame to the left
//#define CAMERA_MODE_LOOK_RIGHT				0x08	// and to the right

// Set limit for Left and Right Scan distance
#define CAMERA_SCAN_PAN_LIMIT_LEFT	(CAMERA_PAN_CENTER - 200)	// Tenthdegrees
#define CAMERA_SCAN_PAN_LIMIT_RIGHT	(CAMERA_PAN_CENTER + 200)	// Tenthdegrees
#define CAMERA_SCAN_PAN_LIMIT_UP	(CAMERA_TILT_CENTER + 50)	// Tenthdegrees
#define CAMERA_SCAN_PAN_LIMIT_DOWN	 CAMERA_TILT_TENTHDEGREES_MAX_DOWN	// Look for cones close to bumber!

//#define CAMERA_SERVO_RATIO 10	// Zoom 0
#define CAMERA_SERVO_RATIO 10	// Zoom 1
//#define CAMERA_SERVO_RATIO 15	// Zoom 2
//#define CAMERA_SERVO_RATIO 20	// Zoom 3

#define OBJECT_DROP_RETRYS			5	// number of frames to retry before dropping back to scan mode
//#define P_SCAN_INCREMENT			2	// number of servo ticks moved each time
#define CAMERA_SCAN_PAN_STEP_SIZE	8	// number of servo ticks to move each Scan step

#define SERVO_DELAY_DISABLED		0
#define SERVO_DELAY_SHORT			(GetTickCount() + 200)	// ms to delay for servo movement
#define SERVO_DELAY_MEDIUM			(GetTickCount() + 400)	// ms to delay for servo movement
#define SERVO_DELAY_LONG			(GetTickCount() + 600)	// ms to delay for servo movement

// Window Positions

#define WINDOW_TOP_OFFSET					50	// move down for Loki
#define WINDOW_BOARDER_SIZE					14

#define RAW_WINDOW_POSITION_Y				  0+WINDOW_TOP_OFFSET

// Use these values if you want to see the Raw window.
//   I just cover it up with the processed window to save space
//#define VIEW_RAW_WINDOW


#ifdef VIEW_RAW_WINDOW
	#define PROCESSED_WINDOW_POSITION_Y_SMALL	158+WINDOW_TOP_OFFSET
	#define PROCESSED_WINDOW_POSITION_Y_MED		280+WINDOW_TOP_OFFSET
	#define PROCESSED_WINDOW_POSITION_Y_LARGE	480+WINDOW_TOP_OFFSET
#else
	#define PROCESSED_WINDOW_POSITION_Y_SMALL	30+WINDOW_TOP_OFFSET
	#define PROCESSED_WINDOW_POSITION_Y_MED		30+WINDOW_TOP_OFFSET
	#define PROCESSED_WINDOW_POSITION_Y_LARGE	30+WINDOW_TOP_OFFSET
#endif

	#define MOTION_WINDOW_POSITION_Y_SMALL		158+PROCESSED_WINDOW_POSITION_Y_SMALL
	#define MOTION_WINDOW_POSITION_Y_MED		280+PROCESSED_WINDOW_POSITION_Y_MED
	#define MOTION_WINDOW_POSITION_Y_LARGE		480+PROCESSED_WINDOW_POSITION_Y_LARGE

// Global variables to this module

//DWORD dwCameraTime = 0;

/**
void foo( CvSURFParams Myparams )
{
	ROBOT_LOG( TRUE,"Myparams = %d, %f, %d, %d\n", Myparams.extended, Myparams.hessianThreshold, Myparams.nOctaves, Myparams.nOctaveLayers)
}
**/

void displayCurrentSettings( videoInput *VI, int device )
{
	IGNORE_UNUSED_PARAM (device);

			long min, max, SteppingDelta, currentValue, flags, defaultValue;
//			long propBrightness;
//			long propContrast;
//			long propHue;
//			long propSaturation;
//			long propSharpness;
//			long propGamma;
//			long propColorEnable;
//			long propWhiteBalance;
//			long propBacklightCompensation;
//			long propGain;

//			long propPan;
//			long propTilt;
//			long propRoll;
//			long propZoom;
//			long propExposure;
//			long propIris;
//			long propFocus;

			// Show current values
			ROBOT_LOG( TRUE,"\nDEBUG - VideoSettings:\n")

			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propBrightness, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propBrightness = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propContrast, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propContrast = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propHue, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propHue = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propSaturation, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propSaturation = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propSharpness, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propSharpness = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propGamma, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propGamma = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propColorEnable, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propColorEnable = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propWhiteBalance, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propWhiteBalance = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propBacklightCompensation, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propBacklightCompensation = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propGain, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propGain = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propPan, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propPan = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propTilt, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propTilt = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propRoll, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propRoll = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propZoom, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propZoom = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propExposure, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propExposure = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propIris, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propIris = %d\n", currentValue)
			if( VI->getVideoSettingCamera(LEFT_CAMERA, VI->propFocus, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propFocus = %d\n", currentValue)

			ROBOT_LOG( TRUE,"======== FILTER ==============\n\n")
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propBrightness, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propBrightness = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propContrast, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propContrast = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propHue, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propHue = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propSaturation, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propSaturation = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propSharpness, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propSharpness = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propGamma, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propGamma = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propColorEnable, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propColorEnable = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propWhiteBalance, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propWhiteBalance = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propBacklightCompensation, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propBacklightCompensation = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propGain, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propGain = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propPan, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propPan = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propTilt, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propTilt = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propRoll, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propRoll = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propZoom, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propZoom = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propExposure, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propExposure = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propIris, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propIris = %d\n", currentValue)
			if( VI->getVideoSettingFilter(LEFT_CAMERA, VI->propFocus, min, max, SteppingDelta, currentValue, flags, defaultValue) )
				ROBOT_LOG( TRUE,"propFocus = %d\n", currentValue)

			ROBOT_LOG( TRUE,"=========================\n\n")


}

///////////////////////////////////////////////////////////////////////////////
//	Video Helper Functions
///////////////////////////////////////////////////////////////////////////////

int WindowPosX( IplImage* pInputFrame )
{
	int DisplayWidth = GetSystemMetrics(SM_CXSCREEN);
	int DisplayHeight = GetSystemMetrics(SM_CYSCREEN);


//	CvSize FrameSize = cvGetSize(pInputFrame);	// LAPTOP_DISPLAY_SIZE_WIDTH
	int PosX =  DisplayWidth - (pInputFrame->width + WINDOW_BOARDER_SIZE);
	return PosX;
}

///////////////////////////////////////////////////////////////////////////////
void CopyImage( IplImage* pInputFrame, IplImage* pOutputFrame )
{

	// Copy frame to the "working" image, inverting it if needed
	cvCopy( pInputFrame, pOutputFrame, 0 );

	if( 1 == pInputFrame->origin ) // 1 means the image is inverted
	{
		cvFlip( pOutputFrame, 0, 0 );
	}

}


///////////////////////////////////////////////////////////////////////////////
//	Video Callback Functions
///////////////////////////////////////////////////////////////////////////////

void OnMouseEvent( int event, int x, int y, int flags, void* param )
{
	IGNORE_UNUSED_PARAM (flags);

	CCameraModule* pCameraObject = (CCameraModule*)param;

    if( !pCameraObject )
        return;


    if( CV_EVENT_LBUTTONDOWN == event )
	{
		ROBOT_LOG( TRUE,"OnMouseEvent: MouseDown at %d, %d\n", x, y)
		pCameraObject->m_VideoMousePoint = cvPoint(x,y);
        pCameraObject->m_VideoMousePointSelected = TRUE;
	}

 /*   if( select_object )
    {
        selection.x = MIN(x,origin.x);
        selection.y = MIN(y,origin.y);
        selection.width = selection.x + CV_IABS(x - origin.x);
        selection.height = selection.y + CV_IABS(y - origin.y);
        
        selection.x = MAX( selection.x, 0 );
        selection.y = MAX( selection.y, 0 );
        selection.width = MIN( selection.width, m_pVideoFrame->width );
        selection.height = MIN( selection.height, m_pVideoFrame->height );
        selection.width -= selection.x;
        selection.height -= selection.y;
    }

    case CV_EVENT_LBUTTONUP:
    case CV_EVENT_LBUTTONDOWN:
        origin = cvPoint(x,y);
        selection = cvRect(x,y,0,0);
        select_object = 1;
        break;
    case CV_EVENT_LBUTTONUP:
        select_object = 0;
        if( selection.width > 0 && selection.height > 0 )
            track_object = -1;
        break;
    }
*/
}



//-----------------------------------------------------------------------------
// Name: VidCapThreadProc
// Desc: dedicated thread for grabbing video frames
// This thread created by the CCameraModule class
//-----------------------------------------------------------------------------
__itt_string_handle* pshVidCapThreadLoop = __itt_string_handle_create("VidCapThreadLoop");
__itt_string_handle* pshCamShutDown = __itt_string_handle_create("CameraShutDown");
__itt_string_handle* pshCamInit = __itt_string_handle_create("CameraInit");
__itt_string_handle* pshShowDialog = __itt_string_handle_create("ShowCameraDialog");
__itt_string_handle* pshGrabFrame = __itt_string_handle_create("GrabFrame");
__itt_string_handle* pshProcessFrame = __itt_string_handle_create("ProcessFrame");
__itt_string_handle* pshShowFrames = __itt_string_handle_create("ShowFrames");

__itt_string_handle* pshRecordVideo = __itt_string_handle_create("RecordVideo");
__itt_string_handle* pshHeadMoving = __itt_string_handle_create("HeadMoving");
__itt_string_handle* pshProcessingDisabled = __itt_string_handle_create("ProcessingDisabled");
__itt_string_handle* pshSURFDetector = __itt_string_handle_create("SURFDetector");
__itt_string_handle* pshStereoDetector = __itt_string_handle_create("StereoDetector");
__itt_string_handle* pshFaceDetector = __itt_string_handle_create("FaceDetector");
__itt_string_handle* pshCamShiftDetector = __itt_string_handle_create("CamShift");
__itt_string_handle* pshColorBlobDetector = __itt_string_handle_create("ColorBlobDetector");
__itt_string_handle* pshLaserSpotDetector = __itt_string_handle_create("LaserSpotDetector");
__itt_string_handle* pshObjectTrackingDetector = __itt_string_handle_create("ObjectTrackingDetector");
__itt_string_handle* pshMotionTrackingDetector = __itt_string_handle_create("MotionTrackingDetector");
__itt_string_handle* pshPIRTrackingDetector = __itt_string_handle_create("PIRTrackingDetector");
__itt_string_handle* pshPositionHead = __itt_string_handle_create("PositionHead");
__itt_string_handle* pshCameraIdle = __itt_string_handle_create("CameraIdle");
__itt_string_handle* pshTrackMotion = __itt_string_handle_create("TrackMotion");

__itt_string_handle* pshFaceFound = __itt_string_handle_create("FaceFound!");
__itt_string_handle* pshSURFFound = __itt_string_handle_create("SURFFound!");



DWORD WINAPI VidCapThreadProc( LPVOID lpParameter )
{
	IGNORE_UNUSED_PARAM (lpParameter);
	__itt_thread_set_name( "VidCap Thread" );

	//videoInput::listDevices();		//optional static function to list devices
	videoInput * VI  = new videoInput();    //required class to manage all the cameras
	//VI  ->setPhyCon(0, VI_COMPOSITE);	//optional (CAPTURE DEVICE TO SETUP, CAPTURE_CONNECTION) - use VI_COMPOSITE (defualt) for most

//	CvCapture* pCaptureLeft = 0;
//	CvCapture* pCaptureRight = 0;
	BOOL bFrameGrabSucessLeft = FALSE;
	BOOL bFrameGrabSucessRight = FALSE;
	IplImage* pInputFrameLeft = 0;
	IplImage* pInputFrameRight = 0;
	IplImage* pInputFrameKinectDepth = 0;	// Depth Image from Kinect sensor
	IplImage* pInputFrameKinectVideo = 0;	// Video Image from Kinect sensor
	CString MsgString;
	CCameraModule* pCamModule = 0;
	char c = '\0';	// used to get key input from video frame
	ROBOT_ASSERT(VI);
	DWORD dwFrameStartTime = 0;	// track how long each frame takes to process
	//CLNUICamera KinectCam = NULL;	// Kinect Sensor internal state for both Depth and Video

	Sleep(1500); // Let other modules init first

	while( g_bRunThread && g_bRunVidCapThread )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshVidCapThreadLoop);

		if( !g_pCameraModule )
		{
			__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
			Sleep( 200 ); // Camera module not yet initialized
			continue;
		}

		// Get a pointer to the CameraModule Object
		pCamModule = (CCameraModule*) g_pCameraModule;
		ROBOT_ASSERT( pCamModule );

		dwFrameStartTime = GetTickCount();	// track how long each frame takes to process

		///////////////////////////////////////////////////////////////////////////////////
		// SEE IF CAMERAS NEED TO BE SHUT DOWN
		///////////////////////////////////////////////////////////////////////////////////

		if( CAMERA_STATE_SHUTDOWN_REQUESTED == g_Camera[RIGHT_CAMERA].State )
		{
			__itt_marker(pDomainVidCapThread, __itt_null, pshCamShutDown, __itt_marker_scope_task);
			__itt_metadata_str_add(pDomainVidCapThread, __itt_null, pshCamShutDown, "RightCameraShutdown", 0);
			VI->stopDevice(RIGHT_CAMERA);
			pCamModule->ReleaseWorkingImagesRight();
			cvReleaseImage( &pInputFrameRight );	// Safe to call without checking pointer
			cvDestroyWindow( CAMERA_WINDOW_NAME_RIGHT );
			g_Camera[RIGHT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;
		}

		if( CAMERA_STATE_SHUTDOWN_REQUESTED == g_Camera[LEFT_CAMERA].State )
		{
			__itt_marker(pDomainVidCapThread, __itt_null, pshCamShutDown, __itt_marker_scope_task);
			__itt_metadata_str_add(pDomainVidCapThread, __itt_null, pshCamShutDown, "LeftCameraShutdown", 0);
			VI->stopDevice(LEFT_CAMERA);
			pCamModule->ReleaseWorkingImagesLeft();
			cvReleaseImage( &pInputFrameLeft );	// Safe to call without checking pointer
			cvDestroyWindow( CAMERA_WINDOW_NAME_LEFT );
			//	cvDestroyWindow( CAMERA_WINDOW_NAME_MOTION );
			//	cvDestroyWindow( CAMERA_WINDOW_NAME_COLOR_BLOB );
			g_Camera[LEFT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;
			g_CameraSubSystemStatus = SUBSYSTEM_DISABLED;

			if( pCamModule->m_MotionTrackingViewStarted && !pCamModule->m_MotionTrackingViewEnabled )
			{
				// Need to stop the Motion Tracking Window
				cvDestroyWindow( CAMERA_WINDOW_NAME_MOTION );
				pCamModule->m_MotionTrackingViewStarted = FALSE;
			}

			if( !pCamModule->m_VidCapProcessingEnabled )
			{
				pCamModule->ReleaseAllDetectors(); // It's safe (and fast) to call this all the time
			}
		}

		///////////////////////////////////////////////////////////////////////////////////
		// INITIALIZE NEW CAMERAS
		///////////////////////////////////////////////////////////////////////////////////
		if( CAMERA_STATE_INIT_REQUESTED == g_Camera[RIGHT_CAMERA].State )
		{
			__itt_marker(pDomainVidCapThread, __itt_null, pshCamInit, __itt_marker_scope_task);
			__itt_metadata_str_add(pDomainVidCapThread, __itt_null, pshCamInit, "RightCameraInit", 0);
			// INITIALIZE STEREO
			// Right Camera requires Left Camera running too.
			g_Camera[LEFT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;		// in case of error initializing
			g_Camera[RIGHT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;	// in case of error initializing

			int numDevices = VI->listDevices();	
			if( numDevices < 2 )
			{
				MsgString.Format( "**** ERROR CAN'T CREATE STEREO CAMERA CAPTURE! numDevices = %d", numDevices );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			// Initialize VideoInput for each camera

			// Must call before setupDevice. Directshow will try and get the closest possible framerate to what is requested
			VI->setIdealFramerate(LEFT_CAMERA, CAPTURE_FRAME_RATE_FPS );
			VI->setIdealFramerate(RIGHT_CAMERA, CAPTURE_FRAME_RATE_FPS );
			VI->setUseCallback(false);	// Capture in this thread

			if( !VI->setupDevice(LEFT_CAMERA, g_Camera[LEFT_CAMERA].FrameSize.cx, g_Camera[LEFT_CAMERA].FrameSize.cy) )
			{
				MsgString.Format( "**** ERROR CAN'T SETUP LEFT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
			if( !VI->setupDevice(RIGHT_CAMERA, g_Camera[RIGHT_CAMERA].FrameSize.cx, g_Camera[RIGHT_CAMERA].FrameSize.cy) )
			{
				MsgString.Format(  "**** ERROR CAN'T SETUP RIGHT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			pInputFrameLeft = cvCreateImage( cvSize(g_Camera[LEFT_CAMERA].FrameSize.cx,g_Camera[LEFT_CAMERA].FrameSize.cy), IPL_DEPTH_8U, 3 );
			pInputFrameRight = cvCreateImage( cvSize(g_Camera[RIGHT_CAMERA].FrameSize.cx,g_Camera[RIGHT_CAMERA].FrameSize.cy), IPL_DEPTH_8U, 3 );

			if( (VI->getWidth(LEFT_CAMERA) != g_Camera[LEFT_CAMERA].FrameSize.cx) ||
				(VI->getHeight(LEFT_CAMERA) != g_Camera[LEFT_CAMERA].FrameSize.cy) )
			{
				// Did not return same size as asked for!
				MsgString.Format(  "**** ERROR LEFT CAMERA WRONG SIZE!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
			if( (VI->getWidth(RIGHT_CAMERA) != g_Camera[RIGHT_CAMERA].FrameSize.cx) ||
				(VI->getHeight(RIGHT_CAMERA) != g_Camera[RIGHT_CAMERA].FrameSize.cy) )
			{
				// Did not return same size as asked for!
				MsgString.Format(  "**** ERROR RIGHT CAMERA WRONG SIZE!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			cvNamedWindow(CAMERA_WINDOW_NAME_LEFT, CV_WINDOW_AUTOSIZE);
			cvNamedWindow(CAMERA_WINDOW_NAME_RIGHT, CV_WINDOW_AUTOSIZE);

			// Grab first frame from each camera
			// if(VI->isFrameNew(device1))
			// deviceID, flipRedAndBlue, flipImage );
			if( !VI->getPixels(LEFT_CAMERA, (unsigned char *)pInputFrameLeft->imageData, false, true) )	//fills pixels as a BGR (for openCV) unsigned char array - no flipping
			{
				MsgString.Format(  "**** ERROR CAN'T GRAB FIRST FRAME FROM LEFT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			if( !VI->getPixels(LEFT_CAMERA, (unsigned char *)pInputFrameRight->imageData, false, true) )	//fills pixels as a BGR (for openCV) unsigned char array - no flipping
			{
				MsgString.Format( "**** ERROR CAN'T GRAB FIRST FRAME FROM RIGHT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			// Initialize working frames
			// Left Camera is used for all non-stereo stuff

			pCamModule->m_FrameNumber = 0;
			pCamModule->InitWorkingImagesLeft( pInputFrameLeft );	// Create m_pVideoFrame and m_pColorBlobFrame
			pCamModule->InitWorkingImagesRight( pInputFrameRight );

			// Setup Mouse input for Left window only
			cvSetMouseCallback( CAMERA_WINDOW_NAME_LEFT, OnMouseEvent, (void*)g_pCameraModule );
			
			pCamModule->MoveWindows();	// Move secondary windows	

			// Show first frame
			cvShowImage( CAMERA_WINDOW_NAME_LEFT, pInputFrameLeft );
			c = (char)cvWaitKey(1);

			cvShowImage( CAMERA_WINDOW_NAME_RIGHT, pInputFrameRight );
			c = (char)cvWaitKey(1);

			g_Camera[LEFT_CAMERA].State = CAMERA_STATE_INITIALIZED;
			g_Camera[RIGHT_CAMERA].State = CAMERA_STATE_INITIALIZED;
			g_CameraSubSystemStatus = SUBSYSTEM_CONNECTED;

			__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
			Sleep(SAMPLE_FRAME_RATE_MS);
			continue;
		}

		if( CAMERA_STATE_INIT_REQUESTED == g_Camera[LEFT_CAMERA].State )
		{
			__itt_marker(pDomainVidCapThread, __itt_null, pshCamInit, __itt_marker_scope_task);
			__itt_metadata_str_add(pDomainVidCapThread, __itt_null, pshCamInit, "LeftCameraInit", 0);

			// INITIALIZE LEFT CAMERA ONLY
			g_Camera[LEFT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;		// in case of error initializing

			int numDevices = VI->listDevices();	
			if( numDevices < 1 )
			{
				MsgString.Format( "**** ERROR CAN'T CREATE LEFT CAMERA CAPTURE! numDevices = %d", numDevices );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			// Initialize VideoInput for each camera

			// Must call before setupDevice. Directshow will try and get the closest possible framerate to what is requested
			VI->setIdealFramerate(LEFT_CAMERA, CAPTURE_FRAME_RATE_FPS );
			VI->setUseCallback(false);	// Capture in this thread

			if( !VI->setupDevice(LEFT_CAMERA, g_Camera[LEFT_CAMERA].FrameSize.cx, g_Camera[LEFT_CAMERA].FrameSize.cy) )
			{
				MsgString.Format( "**** ERROR CAN'T SETUP LEFT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
			pInputFrameLeft = cvCreateImage( cvSize(g_Camera[LEFT_CAMERA].FrameSize.cx,g_Camera[LEFT_CAMERA].FrameSize.cy), IPL_DEPTH_8U, 3 );

			if( (VI->getWidth(LEFT_CAMERA) != g_Camera[LEFT_CAMERA].FrameSize.cx) ||
				(VI->getHeight(LEFT_CAMERA) != g_Camera[LEFT_CAMERA].FrameSize.cy) )
			{
				// Did not return same size as asked for!
				MsgString.Format( "**** ERROR LEFT CAMERA WRONG SIZE!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
			cvNamedWindow(CAMERA_WINDOW_NAME_LEFT, CV_WINDOW_AUTOSIZE);

			// Grab first frame from left camera
			// if(VI->isFrameNew(device1))
			if( !VI->getPixels(LEFT_CAMERA, (unsigned char *)pInputFrameLeft->imageData, false, true) )	//fills pixels as a BGR (for openCV) unsigned char array - no flipping
			{
				MsgString.Format( "**** ERROR CAN'T GRAB FRAME FROM LEFT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}

			// Initialize working frames
			// Left Camera is used for all non-stereo stuff
			pCamModule->m_FrameNumber = 0;
			pCamModule->InitWorkingImagesLeft( pInputFrameLeft );	// Create m_pVideoFrame and m_pColorBlobFrame
			// Setup Mouse input for Left window only
			cvSetMouseCallback( CAMERA_WINDOW_NAME_LEFT, OnMouseEvent, (void*)g_pCameraModule );

			// Show first frame
			if( 640 == g_Camera[LEFT_CAMERA].FrameSize.cx )
			{
				// Scale down 640x480 to fit on display (easier on remote desktop too)
				cvResize( pCamModule->m_pVideoFrameLeft, pCamModule->m_pDisplayFrameLeft  ); // CV_INTER_CUBIC
			//	cvResizeWindow( CAMERA_WINDOW_NAME_LEFT, 320, 240 );
				cvShowImage( CAMERA_WINDOW_NAME_LEFT, pCamModule->m_pDisplayFrameLeft );
				pCamModule->MoveWindows();	// Move secondary windows	
			}
			else
			{
				cvShowImage( CAMERA_WINDOW_NAME_LEFT, pCamModule->m_pVideoFrameLeft );
				pCamModule->MoveWindows();	// Move secondary windows	
			}
			c = (char)cvWaitKey(1);

			g_Camera[LEFT_CAMERA].State = CAMERA_STATE_INITIALIZED;
			g_CameraSubSystemStatus = SUBSYSTEM_CONNECTED;

			__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
			Sleep(SAMPLE_FRAME_RATE_MS);
			continue;
		}

		///////////////////////////////////////////////////////////////////////////////////
		// Sleep if no cameras are enabled
		if(	(CAMERA_STATE_NOT_ENABLED == g_Camera[LEFT_CAMERA].State)  &&
			(CAMERA_STATE_NOT_ENABLED == g_Camera[RIGHT_CAMERA].State) )
		{
			__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
			Sleep( 200 ); // Camera module not yet initialized
			continue;
		}

		///////////////////////////////////////////////////////////////////////////////////
		// SET DEVICE PARAMETERS

		if( pCamModule->m_ShowFormatDialog )
		{
			__itt_marker(pDomainVidCapThread, __itt_null, pshShowDialog, __itt_marker_scope_task);
			ROBOT_LOG( TRUE,"ShowFormatDialog requested - LEFT CAMERA\n")
			VI->showSettingsWindow(LEFT_CAMERA);
			displayCurrentSettings(VI, LEFT_CAMERA);	// dump current setting for debug
			pCamModule->m_ShowFormatDialog = FALSE; // Reset
		}

		if( pCamModule->m_ShowPropDialog )
		{
			__itt_marker(pDomainVidCapThread, __itt_null, pshShowDialog, __itt_marker_scope_task);
			ROBOT_LOG( TRUE,"ShowPropDialog requested - RIGHT CAMERA\n")
			VI->showSettingsWindow(RIGHT_CAMERA);
			displayCurrentSettings(VI, RIGHT_CAMERA);	// dump current setting for debug
			pCamModule->m_ShowPropDialog = FALSE; // Reset
		}

		// Set Camera device parameters, if changed
		if( pCamModule->m_CurrentZoomLevel != g_CameraZoomPos )
		{
			ROBOT_LOG( TRUE,"zoom change requested\n")

			// convert units from 1-16 to 50 - 200 ( = 100 - 400% on Logitech)
			int ScaledZoomLevel = 50 + g_CameraZoomPos * 10;	// Note - useful scale is 0 - 15
			if( ScaledZoomLevel > 200 ) ScaledZoomLevel = 200;

			if( CAMERA_STATE_INITIALIZED == g_Camera[RIGHT_CAMERA].State )
				VI->setVideoSettingCamera(RIGHT_CAMERA, VI->propZoom, ScaledZoomLevel );
			if( CAMERA_STATE_INITIALIZED == g_Camera[LEFT_CAMERA].State )
				VI->setVideoSettingCamera(LEFT_CAMERA, VI->propZoom, ScaledZoomLevel );
			pCamModule->m_CurrentZoomLevel = g_CameraZoomPos;
			// TODO - other settings
			//			VI->setVideoSettingCamera(LEFT_CAMERA, VI->propBrightness, pCamModule->m_RequestedZoomLevel );
			//			VI->setVideoSettingCamera(LEFT_CAMERA, VI->propExposure, pCamModule->m_RequestedZoomLevel );
			//			VI->setVideoSettingCamera(LEFT_CAMERA, VI->propContrast, pCamModule->m_RequestedZoomLevel );
			//			VI->setVideoSettingCamera(LEFT_CAMERA, VI->propZoom, pCamModule->m_RequestedZoomLevel );

		}
	

		///////////////////////////////////////////////////////////////////////////////////
		// NORMAL OPERATION
		// GRAB CURRENT FRAMES FROM BOTH CAMERAS (if enabled)
		///////////////////////////////////////////////////////////////////////////////////
		bFrameGrabSucessLeft = FALSE;
		bFrameGrabSucessRight = FALSE;

		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshGrabFrame);

		// To assure Stereo, grab both left and right cameras close together
		if( CAMERA_STATE_INITIALIZED == g_Camera[RIGHT_CAMERA].State )
		{
			if(!VI->isFrameNew(RIGHT_CAMERA))
			{
				ROBOT_LOG( TRUE,"Skipping Frame - Right Camera\n")
				
				__itt_task_end(pDomainVidCapThread);  // pshGrabFrame
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
			bFrameGrabSucessRight = VI->getPixels(RIGHT_CAMERA, (unsigned char *)pInputFrameRight->imageData, false, true);	//fills pixels as a BGR (for openCV) unsigned char array - no flipping
			if( !bFrameGrabSucessRight )
			{
				MsgString.Format( "**** ERROR CAN'T GRAB FRAME FROM RIGHT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshGrabFrame
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
		}

		if( CAMERA_STATE_INITIALIZED == g_Camera[LEFT_CAMERA].State )
		{
			if(!VI->isFrameNew(LEFT_CAMERA))
			{
				ROBOT_LOG( TRUE,"Skipping Frame - Left Camera\n")
				
				__itt_task_end(pDomainVidCapThread);  // pshGrabFrame
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
			bFrameGrabSucessLeft = VI->getPixels(LEFT_CAMERA, (unsigned char *)pInputFrameLeft->imageData, false, true);	//fills pixels as a BGR (for openCV) unsigned char array - no flipping
			if( !bFrameGrabSucessLeft )
			{
				MsgString.Format( "**** ERROR CAN'T GRAB FRAME FROM LEFT CAMERA!" );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				
				__itt_task_end(pDomainVidCapThread);  // pshGrabFrame
				__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop
				continue;
			}
		}
		
		// Copy current frame to the m_pVideoFrame working image
		if( bFrameGrabSucessLeft )
		{
			CopyImage( pInputFrameLeft, pCamModule->m_pVideoFrameLeft );	// From, To
		}		
		if( bFrameGrabSucessRight )
		{
			CopyImage( pInputFrameRight, pCamModule->m_pVideoFrameRight );	// From, To
		}

		__itt_task_end(pDomainVidCapThread);  // pshGrabFrame

			

		///////////////////////////////////////////////////////////////////////////////////
		// PROCESS FRAMES FROM EITHER OR BOTH CAMERAS
		///////////////////////////////////////////////////////////////////////////////////

		if(	(CAMERA_STATE_INITIALIZED == g_Camera[LEFT_CAMERA].State)  ||
			(CAMERA_STATE_INITIALIZED == g_Camera[RIGHT_CAMERA].State) )
		{

			if( pCamModule->m_VidCapProcessingEnabled )
			{
				__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshProcessFrame);

				if( pCamModule->m_MotionTrackingViewEnabled && pCamModule->m_pVideoFrameLeft && !pCamModule->m_MotionTrackingViewStarted )
				{
					// Need to startup the Motion Tracking Window
					cvNamedWindow( CAMERA_WINDOW_NAME_MOTION, CV_WINDOW_AUTOSIZE );
					pCamModule->m_MotionTrackingViewStarted = TRUE;
					pCamModule->MoveWindows();	// Move secondary windows
				}

				// Initialize any new detectors
				pCamModule->InitNewDetectors();
				

				////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Process Left and Right Frames
				////////////////////////////////////////////////////////////////////////////////////////////////////////////
				pCamModule->ProcessVideoFrames();
				
				__itt_task_end(pDomainVidCapThread);  // pshProcessFrame
			}

			// Show the resulting frames
			__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshShowFrames);

			if( pCamModule->m_pVideoFrameLeft )
			{
				/**
				// DEBUG TEST ONLY!!!
				CvFont font;
				CvPoint pt1;
				pt1.x = 5;
				pt1.y = 50;
				int line_type = CV_AA; // change it to 8 to see non-antialiased graphics
				cvInitFont( &font, CV_FONT_HERSHEY_PLAIN, //CV_FONT_HERSHEY_COMPLEX, 
					1.0,		// Horz Scale
					1.0,		// Vert Scale
					0.0,		// Shear
					1,			// Thickness
					line_type ); 

				cvPutText( pCamModule->m_pVideoFrameLeft, "OpenCV forever!", pt1, &font, CV_RGB(255,255,255) );
				**/

				if( 640 == g_Camera[LEFT_CAMERA].FrameSize.cx )
				{
					// Scale down 640x480 to fit on display (easier on remote desktop too)
//					cvConvertScaleAbs( pCamModule->m_pVideoFrameLeft, pCamModule->m_pDisplayFrameLeft, 0.5, 0 );
//					cvShowImage( CAMERA_WINDOW_NAME_LEFT, pCamModule->m_pDisplayFrameLeft );
					cvResize( pCamModule->m_pVideoFrameLeft, pCamModule->m_pDisplayFrameLeft  ); // CV_INTER_CUBIC
				//	cvResizeWindow( CAMERA_WINDOW_NAME_LEFT, 320, 240 );
					cvShowImage( CAMERA_WINDOW_NAME_LEFT, pCamModule->m_pDisplayFrameLeft );
				}
				else
				{
					cvShowImage( CAMERA_WINDOW_NAME_LEFT, pCamModule->m_pVideoFrameLeft );
				}
			}
			if ( pCamModule->m_pVideoFrameRight )
			{
				if( 640 == g_Camera[RIGHT_CAMERA].FrameSize.cx )
				{
					// Scale down 640x480 to fit on display (easier on remote desktop too)
					//cvConvertScale( pCamModule->m_pVideoFrameRight, pCamModule->m_pDisplayFrameRight, 0.5, 0 );
					cvResize( pCamModule->m_pVideoFrameRight, pCamModule->m_pDisplayFrameRight  ); // CV_INTER_CUBIC
					cvShowImage( CAMERA_WINDOW_NAME_RIGHT, pCamModule->m_pDisplayFrameRight );
				}
				else
				{
					cvShowImage( CAMERA_WINDOW_NAME_RIGHT, pCamModule->m_pVideoFrameRight );
				}
			}
			if( pCamModule->m_MotionTrackingViewStarted )
			{
				cvShowImage( CAMERA_WINDOW_NAME_MOTION, pCamModule->m_pMotionFrame );
			}

			// See if we are taking a snapshot
			if( pCamModule->m_TakeSnapshot )
			{
				pCamModule->SaveSnapshot();
			}
			pCamModule->m_FrameNumber++;

			__itt_task_end(pDomainVidCapThread);  // pshShowFrames

		}

		// Calculate how long the frame took to process and display results
		DWORD dwFrameProcessingTime = GetTickCount() - dwFrameStartTime;

		//	Sleep( 1000/CAPTURE_FRAMES_PER_SECOND );
		// KLUDGE!  REQURED DUE TO BUG IN OPENCV.
		// NEED cvWaitKey() OR VIDEO DOES NOT DISPLAY!
		char c = (char)cvWaitKey(1);
		if( c == 27 )	ROBOT_LOG( TRUE,"ESC KEY Pressed!\n")
		

		__itt_task_end(pDomainVidCapThread);  // pshVidCapThreadLoop

		// Sleep for up to SAMPLE_FRAME_RATE_MS.  Subtract out processing time
		int SleepTime = SAMPLE_FRAME_RATE_MS - dwFrameProcessingTime;
		if( SleepTime > 0 )
		{
			// Only sleep if we are not busy processing.
			Sleep( SleepTime );
		}
		else
		{
			SleepTime = 0;
		}

		g_SensorStatus.VideoFPS = 1000.0 / (dwFrameProcessingTime+SleepTime);
		__itt_metadata_add(pDomainVidCapThread, __itt_null, pshVidCapThreadLoop, __itt_metadata_s32, 1, (void*)&g_SensorStatus.VideoFPS);

	}

	//////////////////////////////////////////////////////////////////////////////////////
	ROBOT_LOG( TRUE,"VidCap Thread exiting.\n")

	if( CAMERA_STATE_INITIALIZED == g_Camera[RIGHT_CAMERA].State )
	{
		pCamModule->ReleaseWorkingImagesRight();
	    cvReleaseImage( &pInputFrameRight );	// Safe to call without checking pointer
		cvDestroyWindow( CAMERA_WINDOW_NAME_RIGHT );
		g_Camera[RIGHT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;
	}

	if( CAMERA_STATE_INITIALIZED == g_Camera[LEFT_CAMERA].State )
	{
		pCamModule->ReleaseWorkingImagesLeft();
	    cvReleaseImage( &pInputFrameLeft );	// Safe to call without checking pointer
		cvDestroyWindow( CAMERA_WINDOW_NAME_LEFT );
		//	cvDestroyWindow( CAMERA_WINDOW_NAME_MOTION );
		//	cvDestroyWindow( CAMERA_WINDOW_NAME_COLOR_BLOB );

		g_Camera[LEFT_CAMERA].State = CAMERA_STATE_NOT_ENABLED;

		if( pCamModule->m_MotionTrackingViewStarted && !pCamModule->m_MotionTrackingViewEnabled )
		{
			// Need to stop the Motion Tracking Window
			cvDestroyWindow( CAMERA_WINDOW_NAME_MOTION );
			pCamModule->m_MotionTrackingViewStarted = FALSE;
		}

		if( !pCamModule->m_VidCapProcessingEnabled )
		{
			pCamModule->ReleaseAllDetectors(); // It's safe (and fast) to call this all the time
		}
	}

	SAFE_DELETE(VI);
	return 0;
}


///////////////////////////////////////////////////////////////////////////////
//	MODULE: CameraModule
///////////////////////////////////////////////////////////////////////////////


CCameraModule::CCameraModule( CDriveControlModule *pDriveControlModule )
{
	m_pDriveCtrl = pDriveControlModule;
	m_pHeadControl = new HeadControl();	// For controlling head servos

	m_FrameNumber = 0;
	m_pVideoFrameLeft = 0;		// Working copy of current LEFT video frame
	m_pVideoFrameRight = 0;		// Working copy of current RIGHT video frame
	m_pKinectDepthFrame = 0;
	m_pKinectVideoFrame = 0;
	m_pMotionFrame = 0;			// for displaying motion view
	m_pColorBlobFrame = 0;		// for displaying color blob view
	m_pDisplayFrameLeft = 0;	// Scaled Frame to display
	m_pDisplayFrameRight = 0;	// Scaled Frame to display
	m_pDisplayFrameKinectDepth = 0;
	m_pDisplayFrameKinectVideo = 0;


//	m_nCameraPanTiltCmd = 0;	// User commands
//	m_nUserCameraPanTiltSpeed = 0;
//	g_CameraPanPos = CAMERA_PAN_TENTHDEGREES_REST_POSITION;
//	g_CameraTiltPos = CAMERA_TILT_TENTHDEGREES_REST_POSITION;
//	g_CameraSideTiltPos = CAMERA_SIDETILT_TENTHDEGREES_REST_POSITION;
//	g_CameraNeckPos = CAMERA_NECK_TENTHDEGREES_REST_POSITION;
	g_CameraZoomPos	= MIN_ZOOM_LEVEL;
	g_CameraServoTorqueEnabled = FALSE;

	m_ShowPropDialog = FALSE;
	m_ShowFormatDialog = FALSE;

	// Detectors
	m_pFaceDetector			= NULL;
	m_pMotionDetector		= NULL;
	m_pCamShiftDetector		= NULL;
	m_pColorDetector		= NULL;
	m_pLaserSpotDetector	= NULL;
	m_pLaserLineDetector	= NULL;
	m_pStereoDetector		= NULL;
	m_pMatchObjectDetector	= NULL;
	m_pTrackObjectDetector	= NULL;


//	m_pConeDetector		= NULL;
//	m_pObjectDetector	= NULL;

	m_HeadMoving = FALSE;
	m_BlurSettleTime = 0;
	m_MotionDetectorTime = 0;
	m_StereoDetectorTime = 0;
	m_FaceLostTime = 0;
	m_FramesSinceMotionLost = 100;


	m_LastMotionCenter.x = 0;
	m_LastMotionCenter.y = 0;
	m_IdleBehavior = BEHAVIOR_FRIENDLY;

	// Detector Enabling flags
	m_VidCapProcessingEnabled = FALSE;
	m_FaceTrackingEnabled = FALSE;
	m_ColorTrackingEnabled = FALSE;		// Initialize the color tracking object
	m_ColorTrackingActive = FALSE;		// Actively track colors
	m_CamShiftTrackingEnabled = FALSE;
	m_ConeTrackingEnabled = FALSE;		// for tracking Cone SHAPE

	m_MotionTrackingEnabled = FALSE;
	m_IRTrackingEnabled = FALSE;
	m_MotionTrackingViewEnabled = FALSE;
	m_LaserSpotTrackingEnabled = FALSE;	
	m_LaserLineTrackingEnabled = FALSE;	

	m_ObjectTrackingEnabled = FALSE;
	m_ObjectTrackingActive = FALSE;
	m_ObjectToTrackFound = FALSE;

	m_StereoEnabled = FALSE;	
	m_HandTrackingEnabled = FALSE;
	m_TakeSnapshot = FALSE;
	m_VideoRecordMode = VIDEO_RECORD_IDLE;
	m_pVideoWriter = NULL;
	m_MotionTrackingViewStarted = FALSE;

	m_MatchObjectEnabled = FALSE;
	m_MatchObjectInFrame = FALSE;
	m_MatchObjectTracking = FALSE;


	// Processing variables
	m_FaceTrackingFrameCount = 0;
	m_CamShiftCenter.x = 0;
	m_CamShiftCenter.y = 0;
	m_FaceTrackCenter.x = 0;
	m_FaceTrackCenter.y = 0;
	m_FaceTrackSize.width = 0;
	m_FaceTrackSize.height = 0;
	m_MotionCenter.x = 0;
	m_MotionCenter.y = 0;
	m_ObjectTrackingCenter.x = 0;
	m_ObjectTrackingCenter.y = 0;
	m_ColorBlobCenter.x = 0;
	m_ColorBlobCenter.y = 0;
	m_LaserSpot.x = 0;
	m_LaserSpot.y = 0;
	m_IRTrackCenter.x = 0;
	m_IRTrackCenter.y = 0;
	m_PIR_Position.x = 0;
	m_PIR_Position.y = 0;
//	m_HandTrackPosition.x = 0;
//	m_HandTrackPosition.y = 0;

    memset( &m_CamShiftBox, 0, sizeof(CvBox2D) );

	m_FaceTrackingState = STATE_LOOK_FOR_FACE;
	ROBOT_LOG( TRUE,"m_FaceTrackingState = STATE_LOOK_FOR_FACE\n")
	m_CamShiftState = STATE_IDLE;
	ROBOT_LOG( TRUE,"m_CamShiftState = STATE_IDLE\n")


	// Color Blob Tracking
	m_ColorScanningEnabled = FALSE;
	m_ColorSearchingState = STATE_IDLE;	// If not Idle, looking for a color, but one not found yet
	m_FrameRetryCount = 0;
	m_ServoDelayTime = SERVO_DELAY_DISABLED;	// target system tic count, in ms. 0=disabled
	m_nPScanPosLeft = CAMERA_PAN_CENTER;
	m_nPScanPosRight = CAMERA_PAN_CENTER;

	m_VideoMousePoint.x = 0;
	m_VideoMousePoint.y = 0;
	m_VideoMousePointSelected = FALSE;
	//m_RandomHeadCounter = 0;
	g_LastCameraMoveTime = GetTickCount();

	// Ask the GUI to send the last saved values for Camera settings
//	SendResponse( WM_ROBOT_GET_CAMERA_SETTINGS, 0, 0 );

	// Create the video capture thread
	g_bRunVidCapThread = TRUE; // When FALSE, tells Vidcap thread to exit
	DWORD dwTempThreadId;
	g_hCameraVidCapThread = ::CreateThread( NULL, 0, VidCapThreadProc, (LPVOID)0, 0, &dwTempThreadId );
	ROBOT_LOG( TRUE, "Created VidCap Thread. ID = (0x%x)\n", dwTempThreadId )


}

///////////////////////////////////////////////////////////////////////////////
CCameraModule::~CCameraModule()
{
	ROBOT_LOG( TRUE,"SHUT DOWN: ~CCameraModule.  Waiting for VidCap Thread to exit...\n")
	if( INVALID_HANDLE_VALUE != g_hCameraVidCapThread) 
	{
		WaitForSingleObject( g_hCameraVidCapThread, INFINITE );
		CloseHandle( g_hCameraVidCapThread );
	}
	ROBOT_LOG( TRUE,"SHUT DOWN: ~CCameraModule.  VidCap Thread exit complete.\n")

	ReleaseAllDetectors();
	ReleaseWorkingImagesLeft();
	ReleaseWorkingImagesRight();
	SAFE_DELETE(m_pHeadControl);

	ROBOT_LOG( TRUE,"SHUT DOWN: ~CCameraModule done.\n")

}

///////////////////////////////////////////////////////////////////////////////
// Left Camera
void CCameraModule::InitWorkingImagesLeft( IplImage* pInputFrame )
{
	// On first frame, allocate "Working" frames of the correct size
	CvSize FrameSize = cvGetSize(pInputFrame);

	m_pVideoFrameLeft = cvCreateImage( FrameSize, 8, 3 );
	if( !m_pVideoFrameLeft ) ROBOT_ASSERT(0);
	
	m_pMotionFrame = cvCreateImage( FrameSize, 8, 3 );
	if( !m_pMotionFrame ) ROBOT_ASSERT(0);
	cvZero( m_pMotionFrame );

	m_pColorBlobFrame = cvCreateImage( FrameSize, 8, 3 );
	if( !m_pColorBlobFrame ) ROBOT_ASSERT(0);
	cvZero( m_pColorBlobFrame );

	m_pVideoFrameLeft->origin = 0;
	m_pMotionFrame->origin = 0;
	m_pColorBlobFrame->origin = 0;

	CvSize DisplaySize = {320,240};
	m_pDisplayFrameLeft = cvCreateImage( DisplaySize, 8, 3 );
	if( !m_pDisplayFrameLeft ) ROBOT_ASSERT(0);
	m_pDisplayFrameLeft->origin = 0;

}
void CCameraModule::ReleaseWorkingImagesLeft()
{
	// Make sure all detectors are shut down before releasing working images
	ReleaseAllDetectors();

	// These are safe to call without validating the pointers
    cvReleaseImage( &m_pVideoFrameLeft );
    cvReleaseImage( &m_pMotionFrame );
    cvReleaseImage( &m_pColorBlobFrame );
}

///////////////////////////////////////////////////////////////////////////////
// Right Camera
void CCameraModule::InitWorkingImagesRight( IplImage* pInputFrame )
{
	// On first frame, allocate "Working" frames of the correct size
	CvSize FrameSize = cvGetSize(pInputFrame);

	m_pVideoFrameRight = cvCreateImage( FrameSize, 8, 3 );
	if( !m_pVideoFrameRight ) ROBOT_ASSERT(0);

	m_pVideoFrameRight->origin = 0;

	CvSize DisplaySize = {320,240};
	m_pDisplayFrameRight = cvCreateImage( DisplaySize, 8, 3 );
	if( !m_pDisplayFrameRight ) ROBOT_ASSERT(0);
	m_pDisplayFrameRight->origin = 0;

}
void CCameraModule::ReleaseWorkingImagesRight()
{
	// These are safe to call without validating the pointers
    cvReleaseImage( &m_pVideoFrameRight );
}

///////////////////////////////////////////////////////////////////////////////
// Kinect Depth Camera
void CCameraModule::InitKinectDepthWorkingImages( IplImage* pInputFrame )
{
	// On first frame, allocate "Working" frames of the correct size
	CvSize FrameSize = cvGetSize(pInputFrame);
	m_pKinectDepthFrame = cvCreateImage( FrameSize, 8, 3 );
	if( !m_pKinectDepthFrame ) ROBOT_ASSERT(0);
	m_pKinectDepthFrame->origin = 0;

	//CvSize DisplaySize = {320,240};
	CvSize DisplaySize = {640,480};
	m_pDisplayFrameKinectDepth = cvCreateImage( DisplaySize, 8, 3 );
	if( !m_pDisplayFrameKinectDepth ) ROBOT_ASSERT(0);
	m_pDisplayFrameKinectDepth->origin = 0;
}

void CCameraModule::ReleaseKinectDepthWorkingImages()
{
	// These are safe to call without validating the pointers
    cvReleaseImage( &m_pDisplayFrameKinectDepth );
    cvReleaseImage( &m_pKinectDepthFrame );
}

///////////////////////////////////////////////////////////////////////////////
// Kinect Video Camera
void CCameraModule::InitKinectVideoWorkingImages( IplImage* pInputFrame )
{
	// On first frame, allocate "Working" frames of the correct size
	CvSize FrameSize = cvGetSize(pInputFrame);
	m_pKinectVideoFrame = cvCreateImage( FrameSize, 8, 3 );
	if( !m_pKinectVideoFrame ) ROBOT_ASSERT(0);
	m_pKinectVideoFrame->origin = 0;

	//CvSize DisplaySize = {320,240};
	CvSize DisplaySize = {640,480};
	m_pDisplayFrameKinectVideo = cvCreateImage( DisplaySize, 8, 3 );
	if( !m_pDisplayFrameKinectVideo ) ROBOT_ASSERT(0);
	m_pDisplayFrameKinectVideo->origin = 0;
}

void CCameraModule::ReleaseKinectVideoWorkingImages()
{
	// These are safe to call without validating the pointers
    cvReleaseImage( &m_pDisplayFrameKinectVideo );
    cvReleaseImage( &m_pKinectVideoFrame );
}


/////////////////////////////////////////////////////////////////////////////
void CCameraModule::InitNewDetectors( )
{
	// Create and initialize any detectors that have just been enabled for the LEFT camera
	// For each detector type:
	// If Enabled but not created, create the object
	// If Disabled, but existing, delete the object

	if( !m_VidCapProcessingEnabled )
	{
		return;
	}

	if( m_pVideoFrameLeft )
	{
		// Init detectors that use the left camera (most of them)


		// Face Detector
		if( (m_FaceTrackingEnabled) && (NULL == m_pFaceDetector) )
		{
			m_pFaceDetector = new CFaceDetector(m_pVideoFrameLeft);
		}
		else if( (!m_FaceTrackingEnabled) && (NULL != m_pFaceDetector) )
		{
			SAFE_DELETE( m_pFaceDetector );
		}

		// Motion Detector
		if( (m_MotionTrackingEnabled) && (NULL == m_pMotionDetector) )
		{
			m_pMotionDetector = new CMotionDetector(m_pVideoFrameLeft);
			// tell motion detector to allow settle time
			m_MotionDetectorTime = GetTickCount() + MOTION_DETECTOR_TIME;
		}
		else if( (!m_MotionTrackingEnabled) && (NULL != m_pMotionDetector) )
		{
			SAFE_DELETE( m_pMotionDetector );
		}

		// Color Detector #1 (Homebrew color detector)
		if( (m_ColorTrackingEnabled) && (NULL == m_pColorDetector) )
		{
			m_pColorDetector = new CColorDetector(m_pVideoFrameLeft);
			m_ColorTrackingActive = FALSE;	// until turned on
		}
		else if( (!m_ColorTrackingEnabled) && (NULL != m_pColorDetector) )
		{
			SAFE_DELETE(  m_pColorDetector );
			m_ColorTrackingActive = FALSE;
		}

		// Color Detector #2 (Camshift)
		if( (m_CamShiftTrackingEnabled) && (NULL == m_pCamShiftDetector) )
		{
			m_pCamShiftDetector = new CCamShiftDetector(m_pVideoFrameLeft);
		}
		else if( (!m_CamShiftTrackingEnabled) && (NULL != m_pCamShiftDetector) )
		{
			SAFE_DELETE(  m_pCamShiftDetector );
		}

		// Laser Spot Tracking
		if( (m_LaserSpotTrackingEnabled) && (NULL == m_pLaserSpotDetector) )
		{
			m_pLaserSpotDetector = new CLaserSpotDetector(m_pVideoFrameLeft);
			m_LaserSpotTrackingActive = FALSE;	// until turned on
		}
		else if( (!m_LaserSpotTrackingEnabled) && (NULL != m_pLaserSpotDetector) )
		{
			SAFE_DELETE(  m_pLaserSpotDetector );
			m_LaserSpotTrackingActive = FALSE;
		}

		// Laser Line Tracking
		if( (m_LaserLineTrackingEnabled) && (NULL == m_pLaserLineDetector) )
		{
			m_pLaserLineDetector = new CLaserLineDetector(m_pVideoFrameLeft);
			m_LaserLineTrackingActive = FALSE;	// until turned on
			m_LaserLineFound = FALSE;
		}
		else if( (!m_LaserLineTrackingEnabled) && (NULL != m_pLaserLineDetector) )
		{
			SAFE_DELETE(  m_pLaserLineDetector );
			m_LaserLineTrackingActive = FALSE;
			m_LaserLineFound = FALSE;
		}

		// Object Tracking
		if( (m_ObjectTrackingEnabled) && (NULL == m_pTrackObjectDetector) )
		{
			m_pTrackObjectDetector = new CTrackObjectDetector(m_pVideoFrameLeft);
			m_pTrackObjectDetector->Init();
			m_ObjectTrackingActive = FALSE;	// until turned on
			m_ObjectToTrackFound = FALSE;
		}
		else if( (!m_ObjectTrackingEnabled) && (NULL != m_pTrackObjectDetector) )
		{
			SAFE_DELETE(  m_pTrackObjectDetector );
			m_ObjectTrackingActive = FALSE;
			m_ObjectToTrackFound = FALSE;
		}

		// Match Object using SURF
		if( (m_MatchObjectEnabled) && (NULL == m_pMatchObjectDetector) )
		{
			m_pMatchObjectDetector = new CMatchObjectDetector(m_pVideoFrameLeft);
			m_pMatchObjectDetector->Initialize();
			//m_MatchObjectTime = 0;
			m_MatchObjectInFrame = TRUE;	// Debug Testing
//			m_MatchObjectInFrame = FALSE;	// until turned on
			m_MatchObjectTracking = FALSE;
		}
		else if( (!m_MatchObjectEnabled) && (NULL != m_pMatchObjectDetector) )
		{
			SAFE_DELETE(  m_pMatchObjectDetector );
			m_MatchObjectTracking = FALSE;
			m_MatchObjectInFrame = FALSE;
		}

		// Stereo
		if( (m_StereoEnabled) && (NULL == m_pStereoDetector) )
		{
			m_pStereoDetector = new CStereoDetector(m_pVideoFrameLeft);
			m_pStereoDetector->Calibrate(DEFAULT_STEREO_CALIB_FILE, 9, 6, 1);
			m_StereoDetectorTime = 0;
			m_StereoActive = FALSE;	// until turned on
		}
		else if( (!m_StereoEnabled) && (NULL != m_pStereoDetector) )
		{
			SAFE_DELETE(  m_pStereoDetector );
			m_StereoActive = FALSE;
		}

		// Cone SHAPE Detector (for RoboMagellan)
		/*** TODO
		if( (m_ConeTrackingEnabled) && (NULL == m_pConeDetector) )
		{
			m_pConeDetector = new CConeDetector(m_pVideoFrameLeft);
		}
		else if( (!m_ConeTrackingEnabled) && (NULL != m_pConeDetector) )
		{
			SAFE_DELETE(  m_pConeDetector );
		}
		***/

	}


}


/////////////////////////////////////////////////////////////////////////////
void CCameraModule::ReleaseAllDetectors()
{
	// Release all detectors when vid cap is shut down
	// (as indicated by m_VidCapProcessingEnabled == FALSE)

	if( NULL != m_pFaceDetector )
	{
		SAFE_DELETE( m_pFaceDetector );
	}

	if( NULL != m_pMotionDetector )
	{
		SAFE_DELETE( m_pMotionDetector );
	}

	// Color Detector #1 (Homebrew color detector)
	if( NULL != m_pColorDetector )
	{
		SAFE_DELETE( m_pColorDetector );
	}

	// Color Detector #2 (Camshift)
	if( NULL != m_pCamShiftDetector )
	{
		SAFE_DELETE( m_pCamShiftDetector );
	}

	/*** TODO
	if( NULL != m_pConeDetector )
	{
		SAFE_DELETE( m_pConeDetector );
	}

	if( NULL != m_pObjectDetector )
	{
		SAFE_DELETE( m_pObjectDetector );
	}
	***/

}


///////////////////////////////////////////////////////////////////////////////
void CCameraModule::ProcessMessage(
		UINT uMsg, WPARAM wParam, LPARAM lParam )
{

	CString MsgString;
	//BYTE nDistHigh, nDistLow;
	switch( uMsg )
	{

		// Camera VidCap Commands
		case WM_ROBOT_CAMERA_ENABLE_FEATURE:
		{
			g_bCmdRecognized = TRUE;
			BOOL EnableFeature = (BOOL)lParam;
			switch( wParam )
			{
				
				case CAMERA_ENABLE_VIDCAP_PROCESSING:
				{
					m_VidCapProcessingEnabled = EnableFeature;
				}
				break;

				// Detection Objects for each of these will be
				// created at the begining of the next video frame
				case CAMERA_ENABLE_TRACKING_FACE:
				{
					m_FaceTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_TRACKING_MOTION:
				{
					m_MotionTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_TRACKING_COLORS:
				{
					m_ColorTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_TRACKING_CAMSHIFT:
				{
					m_CamShiftTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_TRACKING_LASER_SPOT:
				{
					m_LaserSpotTrackingEnabled = EnableFeature;
					m_LaserLineTrackingEnabled = EnableFeature; // TODO - Fix this KLUDGE
				}
				break;

				case CAMERA_ENABLE_TRACKING_CONES:
				{
					m_ConeTrackingEnabled = EnableFeature; // For Cone SHAPE tracking
				}
				break;

				case CAMERA_ENABLE_TRACKING_OBJECTS:
				{
					m_ObjectTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_MATCHING_OBJECTS:
				{
					m_MatchObjectEnabled = EnableFeature;


/****

			const char* object_filename = "target.jpg";

			CvMemStorage* storage = cvCreateMemStorage(0);

			static CvScalar colors[] = 
			{
				{{0,0,255}},
				{{0,128,255}},
				{{0,255,255}},
				{{0,255,0}},
				{{255,128,0}},
				{{255,255,0}},
				{{255,0,0}},
				{{255,0,255}},
				{{255,255,255}}
			};

			IplImage* object = cvLoadImage( object_filename, CV_LOAD_IMAGE_GRAYSCALE );
			if( !object )
			{
				ROBOT_ASSERT(0);
			}
			IplImage* object_color = cvCreateImage(cvGetSize(object), 8, 3);
			cvCvtColor( object, object_color, CV_GRAY2BGR );

			CvSeq *objectKeypoints = 0, *objectDescriptors = 0;
			CvSeq *imageKeypoints = 0, *imageDescriptors = 0;
//			int i;

			////// BAD! //////
//			CvSURFParams params = cvSURFParams(500, 1); // 500

			CvSURFParams params;
			params.hessianThreshold = 500;
			params.extended = 1;
			params.nOctaves = 3;
			params.nOctaveLayers = 4;

//			foo( params );

			cvExtractSURF( object, 0, &objectKeypoints, &objectDescriptors, storage, 
				500, 1, 2, 4 ); // extended, hessianThreshold, nOctaves, nOctaveLayers
//				params ); // extended, hessianThreshold, nOctaves, nOctaveLayers




			printf("Object Descriptors: %d\n", objectDescriptors->total);

****/

				}
				break;

				case CAMERA_ENABLE_TRACKING_IR:
				{
					m_IRTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_SHOW_MOTION_VIEW:
				{
					m_MotionTrackingViewEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_TRACKING_HAND:
				{
					m_HandTrackingEnabled = EnableFeature;
				}
				break;

				case CAMERA_ENABLE_STEREO_VISION:
				{					
					m_StereoEnabled = EnableFeature;
				  //  StereoCalib("ch12_list.txt", 9, 6, 1);
				}
				break;

			}
		}
		break;


		// Process Camera commands from the GUI or command threads


		case WM_ROBOT_CAMERA_MATCH_OBJECT:
		{
			m_MatchObjectInFrame = TRUE; // One-shot.  Captures current frame then resets
		}


		case WM_ROBOT_CAMERA_BEHAVIOR_MODE:
		{
			g_bCmdRecognized = TRUE;
			m_IdleBehavior = wParam;
			if( m_IdleBehavior >= BEHAVIOR_MAX )
			{
				ROBOT_LOG( TRUE,"CAMERA_BEHAVIOR_MODE - ERROR! Illegal value! %d\n", wParam)
				m_IdleBehavior = BEHAVIOR_FRIENDLY;
				ROBOT_LOG( TRUE,"Behavior set to mode %d\n", m_IdleBehavior)
			}
			return;
		}

		case WM_ROBOT_CAMERA_MODE_CMD:
		{
			// wParam = Mode, lParam = Value
			g_bCmdRecognized = TRUE;

			#if( CAMERA_CONTROL_TYPE == SONY_SERIAL_CAMERA )
				// Sony Camera mode commands.  Send to Sony Camera thread.
				SendHardwareCmd( HW_SET_CAMERA_MODE, wParam, lParam );
				return;
			#else
				// Loki Camera Modes (using VideoInput via OpenCV )
				if( CAMERA_MODE_SHOW_FORMAT_DIALOG == wParam )
				{
					m_ShowFormatDialog = TRUE;
				}
				else if( CAMERA_MODE_SHOW_PROP_DIALOG == wParam )
				{
					m_ShowPropDialog = TRUE;
				}
				else
				{
					ROBOT_LOG( TRUE, "Unknown WM_ROBOT_CAMERA_MODE_CMD\n")
					ROBOT_ASSERT(0);
				}
			#endif

			break;
		}

		case WM_ROBOT_CAMERA_TAKE_SNAPSHOT:
		{
			g_bCmdRecognized = TRUE;
			m_TakeSnapshot = TRUE;	// take snapshot of next frame
			return;
		}
		case WM_ROBOT_CAMERA_RECORD_VIDEO:
		{
			g_bCmdRecognized = TRUE;
			// wParam: 1=Start / 0=Stop

			if( 1 == wParam )
			{
				m_VideoRecordMode = VIDEO_RECORD_START;
			}
			else
			{
				m_VideoRecordMode = VIDEO_RECORD_STOP;
			}
			return;
		}
		

		case WM_ROBOT_COLOR_BLOB_AUTO_CAL_CMD:
		{
			g_bCmdRecognized = TRUE;
			if( m_pColorDetector )
			{
				// Calibrate the Color tracker with object at center of frame
				CvRect	ColorBlobTargetRect;	// Region of color to sample for setting Color Target
				ColorBlobTargetRect.x = GetFrameCenter(m_pVideoFrameLeft).x -10;
				ColorBlobTargetRect.y = GetFrameCenter(m_pVideoFrameLeft).y -10;
				ColorBlobTargetRect.width = 20;
				ColorBlobTargetRect.height = 20;

				m_pColorDetector->SetColor(ColorBlobTargetRect);


				// On the GUI, display the target color found 
				SendResponse( WM_ROBOT_DISPLAY_SINGLE_ITEM, ROBOT_RESPONSE_COLOR_BLOB_CAL_RESULT,
					MAKELPARAM(	(WORD)m_pColorDetector->GetCrTarget(), 
								(WORD)m_pColorDetector->GetCbTarget() ) );

			}
			else
			{
				ROBOT_LOG( TRUE,"COLOR BLOB - Auto Calibrate ignored, ColorDetector not enabled\n")
			}
			break;
		}

		case WM_ROBOT_COLOR_BLOB_MANUAL_CAL_CMD:
		{
			g_bCmdRecognized = TRUE;
			if( m_pColorDetector )
			{
				// Calibrate the Color tracker with values passed from the GUI
				m_pColorDetector->SetColor(wParam, lParam);

				// Display the target color found on the GUI
				SendResponse( WM_ROBOT_DISPLAY_SINGLE_ITEM, ROBOT_RESPONSE_COLOR_BLOB_CAL_RESULT,
					MAKELPARAM(	(WORD)m_pColorDetector->GetCrTarget(), 
								(WORD)m_pColorDetector->GetCbTarget() ) );

				ROBOT_LOG( TRUE,"COLOR BLOB - Target Color Manually set to Cr=%d, Cb=%d\n", wParam, lParam)
			}
			else
			{
				ROBOT_LOG( TRUE,"COLOR BLOB - Manual Calibrate ignored, ColorDetector not enabled\n")
			}
			break;
		}

		case WM_ROBOT_SET_CAMERA_COLOR_THRESHOLD:
		{
			g_bCmdRecognized = TRUE;

			// Set threshold value for Cr or Cb value (amount of deviation from target)
			if( m_pColorDetector )
			{
				if(SET_COLOR_THRESHOLD_CR == wParam)
				{
					m_pColorDetector->setCrThreshold(lParam);
					ROBOT_LOG( TRUE,"COLOR BLOB - CrThreshold set to %d\n", lParam)
				}
				else if(SET_COLOR_THRESHOLD_CB == wParam)
				{
					m_pColorDetector->setCbThreshold(lParam);
					ROBOT_LOG( TRUE,"COLOR BLOB - CbThreshold set to %d\n", lParam)
				}
				else
				{
					ROBOT_LOG( TRUE,"ERROR! WM_ROBOT_SET_CAMERA_COLOR_THRESHOLD invalid param (%d)\n", lParam)
				}
			}
			else
			{
				ROBOT_LOG( TRUE,"COLOR BLOB - Threshold ignored, ColorDetector not enabled\n")
			}
			break;
		}

		case WM_ROBOT_CAMERA_INITIALIZE_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Reset the camera
			SendHardwareCmd( HW_CAMERA_INITIALIZE, 0, 0 );

			return;
		}

		case WM_ROBOT_CAMERA_POWER_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Power Camera on/off
			SendHardwareCmd( HW_SET_CAMERA_POWER, wParam, lParam );

			return;
		}

		case WM_ROBOT_USER_CAMERA_ZOOM_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Zoom in/out and speed of zoom
			SendHardwareCmd( (BYTE)HW_SET_CAMERA_ZOOM, wParam, lParam );
			return;
		}

		case WM_ROBOT_CAMERA_ZOOM_ABS_CMD:
		{
			// Set camera absolute zoom position.  Range is 0 - 16.
			g_bCmdRecognized = TRUE;
			int nZoomLevel = wParam;

			// Zooms camera to level specified
			if( nZoomLevel > 16 )
			{
				ROBOT_LOG( TRUE, "WM_ROBOT_CAMERA_ZOOM_ABS_CMD: BAD Zoom Level: %d \n", nZoomLevel )
				nZoomLevel = 16;
			}
			else if( nZoomLevel < 0 )
			{
				ROBOT_LOG( TRUE, "WM_ROBOT_CAMERA_ZOOM_ABS_CMD: BAD Zoom Level: %d \n", nZoomLevel )
				nZoomLevel = 0;
			}

			#if( CAMERA_CONTROL_TYPE == SONY_SERIAL_CAMERA )
				if( nZoomLevel != g_CameraZoomPos )
				{
					// Automatically handled by Video Capture loop, except for Sony Camera
					SendHardwareCmd( (BYTE)HW_SET_CAMERA_ZOOM_ABS, nZoomLevel, lParam );
				}
			#endif

			g_CameraZoomPos = nZoomLevel;	// NOTE!  used by Video Capture loop to detect when zoom change is needed
			return;
		}

		case WM_ROBOT_USER_CAMERA_FOWARD:
		{
			// Set camera neck Absolute up/forward position.
			// Note when we move the neck, we need to tilt the head too!
			g_bCmdRecognized = TRUE;

			// Special case "Rest" Position.
			/**
			if( CAMERA_NECK_TENTHDEGREES_REST_POSITION == wParam )
			{
				// Send  commands to move the Camera / Head to "Home" or "Rest" position
				SendHardwareCmd( HW_SET_CAMERA_POWER, POWER_ON, 0 );
				return;
			}
			**/
			// Otherwise, send the command to move the Camera Neck
			SendHardwareCmd( HW_SET_CAMERA_FORWARD_ABS, wParam, 0 ); //TODO - Servo Speed?
			return;
		}

		case WM_ROBOT_CAMERA_POINT_TO_XYZ_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Tell camera to go to ABSOLUTE PAN/TILT position pointing at X,Y,Z in space
			// wParam = X,Z in TenthInches
			// lParam = Y (distance from robot) in TenthInches

			// TODO REMOVE THIS COMMENT if( m_HandTrackingEnabled )
			{
				int Z = LOWORD(wParam);
				int X = HIWORD(wParam);
				int Y = lParam;
				m_pHeadControl->LookAtXYZ( HEAD_OWNER_BEHAVIOR_P1, X,Y,Z );
				m_pHeadControl->ExecutePosition( HEAD_OWNER_BEHAVIOR_P1 );
			}
			return;
		}
		

		case WM_ROBOT_CAMERA_PAN_ABS_CMD:
		{
			// Tell camera to go to ABSOLUTE PAN position specified in TENTHDEGREES
			// wParam = position to move to
			g_bCmdRecognized = TRUE;
			m_pHeadControl->SetHeadPosition( HEAD_OWNER_USER_CONTROL, wParam, NOP, NOP );	// Pan only
			m_pHeadControl->ExecutePosition( HEAD_OWNER_USER_CONTROL );

//			int CameraPanPos = (int)wParam;
//			SendHardwareCmd( HW_SET_CAMERA_PAN_ABS, CameraPanPos, 0 );
//			g_CameraPanPos = CameraPanPos;	// Keep track of position!
			return;
		}

		case WM_ROBOT_CAMERA_TILT_ABS_CMD:
		{
			// Tell camera to go to ABSOLUTE TILT position specified in TENTHDEGREES
			// wParam = position to move to

			g_bCmdRecognized = TRUE;
			m_pHeadControl->SetHeadPosition( HEAD_OWNER_USER_CONTROL, NOP, wParam, NOP );	// Tilt only
			m_pHeadControl->ExecutePosition( HEAD_OWNER_USER_CONTROL );

//			int CameraTiltPos = (int)wParam;
//			// Make sure Tilt command is within Camera limits
//			CheckPositionLimits( g_CameraPanPos, CameraTiltPos);

//			SendHardwareCmd( HW_SET_CAMERA_TILT_ABS, CameraTiltPos, 0 );
//			g_CameraTiltPos = CameraTiltPos;	// Keep track of position!
			return;
		}

		case WM_ROBOT_CAMERA_SIDETILT_ABS_CMD:
		{
			// Tell camera to go to ABSOLUTE SIDE TILT position specified
			// wParam = position to move to

			g_bCmdRecognized = TRUE;
			m_pHeadControl->SetHeadPosition( HEAD_OWNER_USER_CONTROL, NOP, NOP, wParam );	// SideTilt only
			m_pHeadControl->ExecutePosition( HEAD_OWNER_USER_CONTROL );

//			int CameraSideTiltPos = (int)wParam;
			// Make sure Tilt command is within Camera limits
//			CheckPositionLimits( g_CameraPanPos, CameraTiltPos);
//			SendHardwareCmd( HW_SET_CAMERA_SIDETILT_ABS, (UINT)CameraSideTiltPos, 0 );
			return;
		}

		case WM_ROBOT_CAMERA_PAN_REL_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Tell camera to go to position RELATIVE to current position
			m_pHeadControl->SetHeadPositionRelative( HEAD_OWNER_USER_CONTROL, wParam, NOP, NOP );	// Pan only
			m_pHeadControl->ExecutePosition( HEAD_OWNER_USER_CONTROL );

//			int CameraPanPos = g_CameraPanPos + (int)wParam;	// Keep track of position!
			// Make sure command is within Camera limits
//			CheckPositionLimits( CameraPanPos, g_CameraTiltPos);
//			SendHardwareCmd( HW_SET_CAMERA_PAN_ABS, CameraPanPos, 0 );
//			g_CameraPanPos = CameraPanPos;	// Keep track of position!
			return;
		}

		case WM_ROBOT_CAMERA_TILT_REL_CMD:
		{
			// wParam = Amount or positon to move to
			// lParam = Relative or Absolute
			// Tell camera to go to position RELATIVE to current position

			g_bCmdRecognized = TRUE;
			m_pHeadControl->SetHeadPositionRelative( HEAD_OWNER_USER_CONTROL, NOP, wParam, NOP );	// Tilt only
			m_pHeadControl->ExecutePosition( HEAD_OWNER_USER_CONTROL );

//			int CameraTiltPos = g_CameraTiltPos + (int)wParam;
			// Make sure Tilt command is within Camera limits
//			CheckPositionLimits( g_CameraPanPos, CameraTiltPos);
//			SendHardwareCmd( HW_SET_CAMERA_TILT_ABS, CameraTiltPos, 0 );
//			g_CameraTiltPos = CameraTiltPos;	// Keep track of position!
			return;
		}

/***
		case WM_ROBOT_USER_CAMERA_PAN_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to tell camera to move Pan and Tilt
			// This command controls both Pan and Tilt
			UINT m_nCameraPanTiltCmd = wParam;	// Tells info on movement direction


			// Set default to current position
			int NewPanPosition, NewTiltPosition, SideTilt;
			m_pHeadControl->GetHeadPosition( NewPanPosition, NewTiltPosition, SideTilt );


			// Convert command to servo movement
			
			UINT nPanTiltIncrement = m_nUserCameraPanTiltSpeed;	// in TenthDegrees - (example: speed 5 = 50 tenthdegrees = 5.0 degrees)
			
			switch( m_nCameraPanTiltCmd )  // Which direction to Pan/Tilt
			{

				case CAMERA_PAN_STOP:
				{
					break;
				}
				case CAMERA_PAN_UP:
				{
					NewTiltPosition += nPanTiltIncrement;
					break;
				}
				case CAMERA_PAN_DOWN:
				{
					NewTiltPosition -= nPanTiltIncrement;
					break;
				}

				case CAMERA_PAN_LEFT:
				{
					NewPanPosition -= nPanTiltIncrement;
					break;
				}

				case CAMERA_PAN_RIGHT:
				{
					NewPanPosition += nPanTiltIncrement;
					break;
				}

				// Combo Pan/Tilt commands
				case CAMERA_PAN_UP_LEFT:
				{
					NewTiltPosition += nPanTiltIncrement;
					NewPanPosition -= nPanTiltIncrement;
					break;
				}
				case CAMERA_PAN_UP_RIGHT:
				{
					NewTiltPosition += nPanTiltIncrement;
					NewPanPosition += nPanTiltIncrement;
					break;
				}
				case CAMERA_PAN_DOWN_LEFT:
				{
					NewTiltPosition -= nPanTiltIncrement;
					NewPanPosition -= nPanTiltIncrement;
					break;
				}
				case CAMERA_PAN_DOWN_RIGHT:
				{
					NewTiltPosition -= nPanTiltIncrement;
					NewPanPosition += nPanTiltIncrement;
					break;
				}
				case CAMERA_PAN_ABS_CENTER:
				{
					NewTiltPosition = 0;
					NewPanPosition = 0;
					break;
				}
				default:
				{
					ROBOT_LOG( TRUE, "ERROR! Invalid Camera Pan Direction, m_nCameraPanTiltCmd = 0x%08lX\n", m_nCameraPanTiltCmd )
				}
				break;
			}

			m_pHeadControl->SetHeadSpeed( HEAD_OWNER_USER_CONTROL, SERVO_SPEED_MED, SERVO_SPEED_MED, SERVO_SPEED_MED );
			m_pHeadControl->SetHeadPosition( HEAD_OWNER_USER_CONTROL, NewPanPosition, NewTiltPosition, NOP );
			m_pHeadControl->ExecutePositionAndSpeed( HEAD_OWNER_USER_CONTROL );

			return;
		}
		***/

		case WM_ROBOT_COLOR_BLOB_LOOK_AHEAD_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Command from user or NavModule to start looking for Color Blobs, straight ahead
			// wParam TRUE = Start, FALSE = Stop
			// lParam = not used
			if( wParam )
			{
				// Start Search
				if( (0 == m_pColorDetector->GetCrTarget()) && (0 == m_pColorDetector->GetCbTarget()) )
				{
					// Target Color not set!
					MsgString.Format( "ERRPOR!  Can't do Color Blob Search, Target Color not set!" );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					return;
				}

			
				m_ColorTrackingActive = TRUE;	// Enable active color tracking
				m_ColorScanningEnabled = FALSE;	// Just look for color blob ahead (and possibly one frame to left or right)
				CenterCamera();					// Point camera forward until a blob found
				m_ColorSearchingState = IDLE;	// Nothing to do unless a color gets spotted
				MsgString.Format( "Looking for ColorBlob Straight Ahead...." );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )

			}
			else
			{
				// Stop Search
				CenterCamera();					// Point camera forward
				m_ColorTrackingActive = FALSE;	// Stop tracking color
				m_ColorScanningEnabled = FALSE;
				m_ColorSearchingState = IDLE;
				MsgString.Format( "Stopped looking for ColorBlob." );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			}
			return;
		}


		case WM_ROBOT_COLOR_BLOB_SEARCH_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Command from user to start scanning for Color Blob
			// wParam TRUE = Start, FALSE = Stop
			// lParam = not used
			if( wParam )
			{
				// Start Search
				if( (0 == m_pColorDetector->GetCrTarget()) && (0 == m_pColorDetector->GetCbTarget()) )
				{
					// Target Color not set!
					MsgString.Format( "ERRPOR!  Can't do Color Blob Search, Target Color not set!" );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					return;
				}


				m_ColorTrackingActive = TRUE;	// Enable active color tracking
				m_ColorScanningEnabled = TRUE;	// Scan Left and right looking for the color blob
				m_ColorSearchingState = IDLE;	// Nothing to do unless a color gets spotted

				BeginScanningMode();			// Start scanning

				MsgString.Format( "Scanning for ColorBlob...." );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			}
			else
			{
				// Stop Search
				CenterCamera();					// Point camera forward
				m_ColorTrackingActive = FALSE;	// Stop tracking color
				m_ColorScanningEnabled = FALSE;
				m_ColorSearchingState = IDLE;
				MsgString.Format( "Stopped ColorBlob Search." );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			}

			return;
		}

		case WM_ROBOT_SENSOR_STATUS_READY:
		{
			//g_bCmdRecognized = TRUE;
			//CString MsgString;

			#if SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI // Head mounted IR is only on Loki

#ifdef IR_SENSORS_ADDED_BACK_TO_HEAD	// all this disabled for now

				if( m_IRTrackingEnabled && (!m_VidCapProcessingEnabled) )
				{
					// only do this if IR head tracking is enabled, but VidCapProcessing is NOT!
					// (if VidCapProcessingEnabled, the frame callback thread will prioritize IR tracking with other tracking)
					/*
						DWORD dwCurrTime = GetTickCount();
						if( (0 != m_CameraMoveTime)								&&
							((dwCurrTime - m_CameraMoveTime) > CAMERA_MOVE_TIME_IR) )
						{
							m_CameraMoveTime = 0; // Reset 
							//Beep(500, 100);	// Beep to indicate Motor done moving
						}
						else
					*/

					// Track closest object
					if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT]  <= IR_HEAD_TRACKING_RANGE ) ||		
						(g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] <= IR_HEAD_TRACKING_RANGE ) )		
					{
						// An object found within range of IR sensors.  Track it

						if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] )
						{
							m_IRTrackCenter.x = -30;	// Object is to the Left
						}
						else if( (g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] )
						{
							m_IRTrackCenter.x = 30;	// Object is to the Right
						}
						// Debug
						ROBOT_LOG( TRUE,"Camera IRTracking: Left = %d, Right = %d, Command = %d\n", 
							g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT], g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT], m_IRTrackCenter.x )

						if( (0 != m_IRTrackCenter.x ) )	// && (0 != m_IRTrackCenter.y)  && (0 == m_CameraMoveTime) )
						{
							//	m_IRTrackCenter.y = 20;	// Tilt head up a bit to look people in the eye :-)
							//	PositionCamera( m_IRTrackCenter );
							//	m_IRTrackCenter.x = 0;

							int CameraTiltPos = 200;	// Tilt up to look at person (TenthDegrees)
							int CameraPanPos = m_IRTrackCenter.x;

							m_pHeadControl->SetHeadSpeed(SERVO_SPEED_MED, SERVO_SPEED_MED SERVO_SPEED_MED )
							m_pHeadControl->SetHeadPosition( Owner, NOP, CameraTiltPos, NOP );				// Pan, Tilt, SideTilt
							m_pHeadControl->SetHeadPositionRelative( Owner, m_IRTrackCenter.x, NOP, NOP );	// Pan, Tilt, SideTilt
							m_pHeadControl->ExecutePositionAndSpeed( );
							ROBOT_LOG( TRUE,"CAMERA IR TRACK: Tracking Point: Pan = %d  Tilt = %d\n", CameraPanPos, CameraTiltPos )

							//m_pHeadControl->SetHeadPositionRelative( Owner, m_IRTrackCenter.x, m_IRTrackCenter.y, NOP, TRUE )
							// ROBOT_LOG( TRUE,"CAMERA IR TRACK: Tracking Point: Relative Pan = %d  Tilt = %d\n", m_IRTrackCenter.x, m_IRTrackCenter.y )
							// m_CameraMoveTime = GetTickCount();	// tell detectors that camera is moving
							// m_MotionDetectorTime = GetTickCount();	// tell motion detector to allow settle time
							m_IRTrackCenter.x = 0;
						}
					}
				}
#endif //IR_SENSORS_ADDED_BACK_TO_HEAD
			#endif // SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI
		}	// end case WM_ROBOT_SENSOR_STATUS_READY
	}	// end Switch
}


/***
					// Track closest object
					if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT]  <= IR_HEAD_TRACKING_RANGE ) ||		
						(g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] <= IR_HEAD_TRACKING_RANGE ) )		
					{
						// An object found within range of IR sensors.  Track it
						int CameraPanSpeed = 6;

						if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] )
						{
							m_IRTrackCenter.x = -20;
							SendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_LEFT, CameraPanSpeed );
							m_CameraMoveTime = GetTickCount();	// tell detectors that camera is moving
						}
						else if( (g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] )
						{
							m_IRTrackCenter.x = 20;
							SendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_RIGHT, CameraPanSpeed );
							m_CameraMoveTime = GetTickCount();	// tell detectors that camera is moving
						}
					}
					else if( 0 != m_IRTrackCenter.x )
					{
						// Head was moving, stop it.
						SendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_STOP, 0 );
						m_IRTrackCenter.x = 0;
					}
***/



/////////////////////////////////////////////////////////////////////////////
void CCameraModule::SaveSnapshot()
{
	// Save a snapshot of last left and right frames, with date and time stamp in the name
	SYSTEMTIME SystemTime;
	GetSystemTime( &SystemTime );
	static int savecount = 1;	// used for taking CALIBRATION pictures

	CString MsgString, SnapShotFileNameRoot, SnapShotFileNameLeft, SnapShotFileNameRight, SnapShotFileNameKinectDepth;
	SnapShotFileNameRoot.Format( ROBOT_DATA_PATH "\\Snapshots\\Loki_%u_%u_%u_%u_%u_%u", 
		SystemTime.wYear, SystemTime.wMonth, SystemTime.wDay, SystemTime.wHour, SystemTime.wMinute, SystemTime.wSecond );


	if( (CAMERA_STATE_INITIALIZED == g_Camera[LEFT_CAMERA].State) && m_pVideoFrameLeft )
	{
		// Save the Left image
		//SnapShotFileNameLeft = SnapShotFileNameRoot+"_Left.jpg";
		SnapShotFileNameLeft.Format(ROBOT_DATA_PATH "\\Snapshots\\left%d.jpg",savecount );

		BOOL fSucess = cvSaveImage( (LPCTSTR)SnapShotFileNameLeft, m_pVideoFrameLeft );
		if( fSucess )
		{
			MsgString.Format( "Snapshot (%s) saved", SnapShotFileNameLeft );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		}
		else
		{
			MsgString.Format( "Error saving snapshot (%s)", SnapShotFileNameLeft );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		}
	}

	if( (CAMERA_STATE_INITIALIZED == g_Camera[RIGHT_CAMERA].State) && m_pVideoFrameRight )
	{
		// Save the Right image
//		SnapShotFileNameRight = SnapShotFileNameRoot+"_Right.jpg";
		SnapShotFileNameRight.Format(ROBOT_DATA_PATH "\\Snapshots\\right%d.jpg",savecount );
		BOOL fSucess = cvSaveImage( (LPCTSTR)SnapShotFileNameRight, m_pVideoFrameRight );
		if( fSucess )
		{
			MsgString.Format( "Snapshot (%s) saved", SnapShotFileNameRight );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		}
		else
		{
			MsgString.Format( "Error saving snapshot (%s)", SnapShotFileNameRight );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		}
	}

	savecount++;

	m_TakeSnapshot = FALSE;
}


/////////////////////////////////////////////////////////////////////////////
void CCameraModule::ProcessVideoFrames()
{
	// Process Left and Right Frames
	// Give all clients of a video frame a chance to look at it


	if( !m_pVideoFrameLeft )
	{
		// Error, No frame to process
		ROBOT_DISPLAY( TRUE, "Error No Left Video Frame!" )
		return;
	}

	// See if we are recording video
	if( VIDEO_RECORD_IDLE != m_VideoRecordMode )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshRecordVideo);

		if( VIDEO_RECORD_START == m_VideoRecordMode )
		{
			// Open a new video file with date and time stamp in the name
			CString MsgString;
			CString VideoFileName;
			SYSTEMTIME SystemTime;
			GetSystemTime( &SystemTime );

			VideoFileName.Format( ROBOT_DATA_PATH "\\Video\\Loki_%u_%u_%u_%u_%u_%u.avi", 
				SystemTime.wYear, SystemTime.wMonth, SystemTime.wDay, SystemTime.wHour, SystemTime.wMinute, SystemTime.wSecond );

			m_pVideoWriter = cvCreateVideoWriter(
				(LPCTSTR)VideoFileName,			// Filename
				CV_FOURCC('D','I','V','X'),		// Fourcc code
				30,							// FPS
				cvGetSize(m_pVideoFrameLeft) );		// FrameSize
												// Optional: is_color (color or B&W)
			if( m_pVideoWriter )
			{
				MsgString.Format( "Video (%s) recording...", VideoFileName );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				m_VideoRecordMode = VIDEO_RECORD_RECORDING;
			}
			else
			{
				MsgString.Format( "Error creating video (%s)", VideoFileName );
				ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
				m_VideoRecordMode = VIDEO_RECORD_IDLE;
			}

		}
		else if( VIDEO_RECORD_RECORDING == m_VideoRecordMode )
		{
			if( NULL != m_pVideoWriter )
			{
				int Result = cvWriteFrame( m_pVideoWriter, m_pVideoFrameLeft );
				//ACE(".");
			}
		}
		else if( VIDEO_RECORD_STOP == m_VideoRecordMode )
		{
			if( NULL != m_pVideoWriter )
			{
				ROBOT_LOG( TRUE,"Done\n")
				cvReleaseVideoWriter( &m_pVideoWriter );
				m_VideoRecordMode = VIDEO_RECORD_IDLE;
			}
		}
		else
		{
			ROBOT_ASSERT(0); // bad state
		}	

		__itt_task_end(pDomainVidCapThread);		
	}


	CvRect	FaceRect;	// Square where face was found, for passing to CamShift (if enabled)
	CvRect	ColorBlobTargetRect;	// Region of color to sample for setting Color Target
	CvPoint	ColorBlobBBpt1 = {0,0};	// Bounding box of color blob found 
	CvPoint	ColorBlobBBpt2 = {0,0}; 

	////////////////////////////////////////////////////////////////////////////////////
	// Don't send move commands if current move still underway
	// avoids thrashing of face detector and movement detector
	DWORD dwCurrTime = GetTickCount();

	if( !m_pHeadControl->CheckHeadPosition(FALSE) )
	{
		// Head still moving
		m_HeadMoving = TRUE;
		__itt_marker(pDomainControlThread, __itt_null, pshHeadMoving, __itt_marker_scope_task);
		return;
	}

	if( m_HeadMoving )
	{
		// Head was moving but just reached it's destinstion
		// To allow frame blur to settle, set additional delays before detectors kick in
		m_HeadMoving = FALSE;
		g_LastCameraMoveTime = GetTickCount(); // delay IDLE behavior a bit longer
		m_BlurSettleTime = g_LastCameraMoveTime + CAMERA_BLUR_SETTLE_TIME;
		m_MotionDetectorTime = m_BlurSettleTime + MOTION_DETECTOR_TIME;
		m_StereoDetectorTime = m_BlurSettleTime + STEREO_DETECTOR_TIME;
		#ifndef _DEBUG
			m_BlurSettleTime += 1000;
		#endif
	}

	// Give additional time if NOT in debug mode, to slow the head down!
	#ifndef _DEBUG
	if( dwCurrTime < m_BlurSettleTime )
	{
		__itt_marker(pDomainControlThread, __itt_null, pshHeadMoving, __itt_marker_scope_task);
		return; // We don't to anyting until we have a good picture
	}

	#endif
	if( dwCurrTime > m_MotionDetectorTime )
	{
		m_MotionDetectorTime = 0; // Flag that motion detector is OK to run
	}

	if( dwCurrTime > m_StereoDetectorTime )
	{
		m_StereoDetectorTime = 0; // Flag that motion detector is OK to run
	}

	if( !m_VidCapProcessingEnabled )
	{
		// processing disabled
		__itt_marker(pDomainControlThread, __itt_null, pshProcessingDisabled, __itt_marker_scope_task);
		return;
	}


	//////////////////////////////////
	// Match Object using SURF 
	//if( m_MatchObjectEnabled && m_pMatchObjectDetector && (0 == m_StereoDetectorTime) )
	if( m_MatchObjectEnabled && m_pMatchObjectDetector )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshSURFDetector);

		ROBOT_ASSERT(m_pMatchObjectDetector);
		if( !m_VidCapProcessingEnabled )
		{
			ROBOT_ASSERT(0);
		}

		// See if it's time to look for objects in a frame, and reuse motion detector time to make sure the camera frame has settled
		if( m_MatchObjectInFrame && (0 == m_MotionDetectorTime) )
		{
			POSITION ObjectPos = m_pMatchObjectDetector->LookForMatchingObject();
			if( NULL != ObjectPos )
			{
				__itt_marker(pDomainVidCapThread, __itt_null, pshSURFFound, __itt_marker_scope_task);
				ROBOT_LOG( TRUE,"CameraModule:MatchObjectDetector:  Found object ID %04X\n", (UINT)ObjectPos)
			}
			else
			{
				ROBOT_LOG( TRUE,"CameraModule:MatchObjectDetector:  No Object match found\n")
			}
			// Send response with result of the Match attempt:
			SendCommand( WM_ROBOT_CAMERA_MATCH_RESPONSE, (DWORD)ObjectPos, 0 );
/// TODO for DEBUG - CONTINUOUS MONITORING, comment out the next line!			
			m_MatchObjectInFrame = FALSE; // reset until the next request
			g_LastCameraMoveTime = GetTickCount(); // delay IDLE behavior a bit longer
			__itt_task_end(pDomainVidCapThread); // pshSURFDetector
			return; // Don't do any further processing of this frame
		}
		//m_StereoDetectorTime = GetTickCount();	// wait for a while before next stereo frame

		__itt_task_end(pDomainVidCapThread);		
	}

	//////////////////////////////////
	// Stereo 
	if( m_StereoEnabled && m_pStereoDetector && (0 == m_StereoDetectorTime) )
	{
		ROBOT_ASSERT(m_pStereoDetector);
		if( !m_VidCapProcessingEnabled )
		{
			ROBOT_ASSERT(0);
		}

		// Time to handle a stereo frame
		if( m_pVideoFrameRight )
		{
			__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshStereoDetector);
			// Make sure the right camera is running too
			m_pStereoDetector->ProcessFrames( m_pVideoFrameLeft, m_pVideoFrameRight );
			m_StereoDetectorTime = GetTickCount();	// wait for a while before next stereo frame
			__itt_task_end(pDomainVidCapThread);
		}
		
	}


	//////////////////////////////////
	// Face Detector

	if( m_FaceTrackingEnabled && m_pFaceDetector && gHeadIdle)	// TODO KLUDGE! - Disable face tracking while arm behavior in motion
	{
		ROBOT_ASSERT(m_pFaceDetector);
		if( !m_VidCapProcessingEnabled )
		{
			ROBOT_ASSERT(0);
		}
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshFaceDetector);

		CvSeq* pFaceRectSeq = m_pFaceDetector->DetectFaces();
		if( 0 != pFaceRectSeq )
		{
			if( 0 != pFaceRectSeq->total )
			{
				// at least one face found
				__itt_marker(pDomainVidCapThread, __itt_null, pshFaceFound, __itt_marker_scope_task);

				// draw a thin green rectangular outline around each detection
				for(int i=0;i<(pFaceRectSeq? pFaceRectSeq->total:0); i++ )
				{
					CvRect* r = (CvRect*)cvGetSeqElem(pFaceRectSeq, i);
					CvPoint pt1 = { r->x, r->y };
					CvPoint pt2 = { r->x + r->width, r->y + r->height };
					if( 0 == i )
					{
						cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);
						FaceRect = *r;	// Save location for CamShift Tracking (if enabled)
						m_FaceTrackCenter.x = r->x + (r->width/2);
						m_FaceTrackCenter.y = r->y + (r->height/2);
						m_FaceTrackSize.width = r->width;
						m_FaceTrackSize.height = r->height;
						m_FaceLostTime = GetTickCount();

						ROBOT_LOG( TRUE,"+++++++>>> DEBUG m_FaceTrackCenter = %d, %d <<+++++++++++\n",
							m_FaceTrackCenter.x, m_FaceTrackCenter.y )
					}
					else
					{
						cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(0,255,0), 5, 4, 0);
					}
				}
			}
			else
			{
				// No faces found
				m_FaceTrackCenter.x = 0;
				m_FaceTrackCenter.y = 0;
				m_FaceTrackSize.height = 0;
				m_FaceTrackSize.width = 0;

			}
		}
		__itt_task_end(pDomainVidCapThread);

	}	// m_FaceTrackingEnabled
	else
	{
		m_FaceTrackCenter.x = 0;
		m_FaceTrackCenter.y = 0;
		m_FaceTrackSize.height = 0;
		m_FaceTrackSize.width = 0;
	}


	//////////////////////////////////
	// CamShift Histogram-based color Tracking
	if( m_CamShiftTrackingEnabled && m_pCamShiftDetector )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshCamShiftDetector);

		// Doing Non-Face color tracking
		// track the color blob in the center of the video frame
		if( m_pCamShiftDetector->GetState() == 0 )
		{
			// Start up (or restart) the CamShift tracker
			ColorBlobTargetRect.x = GetFrameCenter(m_pVideoFrameLeft).x;
			ColorBlobTargetRect.y = GetFrameCenter(m_pVideoFrameLeft).y;
			ColorBlobTargetRect.width = 20;
			ColorBlobTargetRect.height = 20;

			m_pCamShiftDetector->startTracking(ColorBlobTargetRect);
		}
		m_CamShiftBox = m_pCamShiftDetector->track();

		// outline with ellipse
		cvEllipseBox(m_pVideoFrameLeft, m_CamShiftBox, CV_RGB(0,0,255), 3, CV_AA, 0 );
		__itt_task_end(pDomainVidCapThread);		
	}


	//////////////////////////////////
	// Color Blob Tracker
	if( m_ColorTrackingEnabled && m_pColorDetector && 
		m_pColorDetector->Initialized() )	// Make sure the color to track was selected
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshColorBlobDetector);
		if( !m_pColorBlobFrame ) ROBOT_ASSERT(0);

		// track the color blob in the center of the video frame

		if( !m_pColorDetector->Initialized() )
		{
			// Start up (or restart) the Color tracker
			ColorBlobTargetRect.x = GetFrameCenter(m_pVideoFrameLeft).x;
			ColorBlobTargetRect.y = GetFrameCenter(m_pVideoFrameLeft).y;
			ColorBlobTargetRect.width = 20;
			ColorBlobTargetRect.height = 20;

			m_pColorDetector->SetColor(ColorBlobTargetRect);

			// Display the target color found on the GUI
			SendResponse( WM_ROBOT_DISPLAY_SINGLE_ITEM, ROBOT_RESPONSE_COLOR_BLOB_CAL_RESULT,
				MAKELPARAM(	(WORD)m_pColorDetector->GetCrTarget(), 
							(WORD)m_pColorDetector->GetCbTarget() ) );

		}

		CvRect ColorBlobBoundingBox = {0,0,0,0};
		m_ColorBlobCenter = m_pColorDetector->FindColorBlob( m_pColorBlobFrame, ColorBlobBBpt1, ColorBlobBBpt2  );

		if( (0 != m_ColorBlobCenter.x) || (0 != m_ColorBlobCenter.y) )
		{
			cvShowImage( CAMERA_WINDOW_NAME_COLOR_BLOB, m_pColorBlobFrame );  // KLUDGE - Use Motion Frame for debug!
		}
		ScanForColorBlob();	// Handle Color Blob Search state machine
		
		__itt_task_end(pDomainVidCapThread);
	}

	//////////////////////////////////
	// Laser Spot Tracker
	// TODO - Calculate camera FOV, and subtract vertical positon, so "real horizon" is used instead of this kludge!

	if( m_LaserSpotTrackingEnabled && m_pLaserSpotDetector  )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshLaserSpotDetector);
		CvRect LaserSpotBoundingBox = {0,0,0,0};

		m_LaserSpot = m_pLaserSpotDetector->FindLaserDot( );
		if( m_pHeadControl->GetTiltPosition() > CAMERA_TILT_TENTHDEGREES_TRACK_LASER_MAX_UP )
		{
			ROBOT_LOG( TRUE,"LASER - HEAD ABOVE TRACKING LEVEL AT %d\n", m_pHeadControl->GetTiltPosition() )
			// Don't track any laser spot in top of video frame if head higher then this
			if( m_LaserSpot.y < GetFrameCenter(m_pVideoFrameLeft).y )
			{
				ROBOT_LOG( TRUE,"LASER - Ignoring spot TOO HIGH at y=%d\n", m_LaserSpot.y )
				// Show it was found, but in the wrong color
				CvPoint pt1 = { m_LaserSpot.x - 6, m_LaserSpot.y - 6 };
				CvPoint pt2 = { m_LaserSpot.x + 6, m_LaserSpot.y + 6 };
				cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(255,255,0), 2, 2, 0);
				m_LaserSpot.x = 0;	// ignore the spot.  Probably just a light reflection
				m_LaserSpot.y = 0;
			}
		}
		__itt_task_end(pDomainVidCapThread);		
	}


	//////////////////////////////////
	// Laser Line Detector
	// TODO - Calculate camera FOV, and subtract vertical positon, so "real horizon" is used instead of this kludge!
/***
	if( m_LaserLineTrackingEnabled && m_pLaserLineDetector  )
	{
		//CvRect LaserSpotBoundingBox = {0,0,0,0};

		m_LaserLineFound = m_pLaserLineDetector->FindLaserLines( );
		if( m_pHeadControl->GetTiltPosition() > CAMERA_TILT_TENTHDEGREES_TRACK_LASER_MAX_UP )
		{
			ROBOT_LOG( TRUE,"LASER - HEAD ABOVE TRACKING LEVEL AT %d\n", m_pHeadControl->GetTiltPosition() )
			// Don't track any laser spot in top of video frame if head higher then this
		}
		else
		{
			for( UINT X = 0; X < (UINT)m_pVideoFrameLeft->width; X++ )
			{

				UINT Y = m_pLaserLineDetector->GetPositionY(X);
				if( Y != 0 )
				{
					// Show point was found
					CvPoint pt1 = { X - 1, Y - 1 };
					CvPoint pt2 = { Y + 1, Y + 1 };
					cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(255,0,0), 2, 2, 0);
				}
			}
		}
		
	}
***/

	//////////////////////////////////
	// Object Tracking Detector

	if( m_ObjectTrackingEnabled && m_pTrackObjectDetector  )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshObjectTrackingDetector);
		//CvRect LaserSpotBoundingBox = {0,0,0,0};
		if( !m_ObjectToTrackFound )
		{
			// Normal mode - get object to track from the first frame
			//m_ObjectToTrackFound = m_pTrackObjectDetector->LockOn( );

			// Special mode - get object to track from an image on disk
			m_ObjectToTrackFound = m_pTrackObjectDetector->LockOn( TRUE );
		}
		else
		{
			CvPoint ObjectTrackingFromFrameCenter;
			if( m_pTrackObjectDetector->GetDelta(ObjectTrackingFromFrameCenter) )
			{
				// Good features to track
				// Scale down to make tracking less agressive
				ObjectTrackingFromFrameCenter.x /= 2;
				ObjectTrackingFromFrameCenter.y /= 2;

				CvPoint	FrameCenter = GetFrameCenter(m_pVideoFrameLeft);
				m_ObjectTrackingCenter.x = FrameCenter.x + ObjectTrackingFromFrameCenter.x;
				m_ObjectTrackingCenter.y = FrameCenter.y + ObjectTrackingFromFrameCenter.y;


				if( (0 != m_ObjectTrackingCenter.x) || (0 != m_ObjectTrackingCenter.y) )
				{
					CvPoint pt1 = { (m_ObjectTrackingCenter.x - 8), (m_ObjectTrackingCenter.y - 8) };
					CvPoint pt2 = { (m_ObjectTrackingCenter.x + 8), (m_ObjectTrackingCenter.y + 8) };
					cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(128,0,255), 2 );
					ROBOT_LOG( TRUE,"+++++++>>> DEBUG m_ObjectTrackingCenter = %d, %d <<+++++++++++\n",
						m_ObjectTrackingCenter.x, m_ObjectTrackingCenter.y )
				}
			}
			else
			{
				m_ObjectTrackingCenter.x = 0;
				m_ObjectTrackingCenter.y = 0;
				// loosing trackable features, re-lock on
				//ROBOT_LOG( TRUE,"TrackObjectDetector: Lost Lock, re-sampling\n")
				//m_ObjectToTrackFound = m_pTrackObjectDetector->LockOn( );
			}
		}
		__itt_task_end(pDomainVidCapThread);		
	}



	//////////////////////////////////
	// Motion Tracking - note that motion detector disabled while camera moving
	if( m_MotionTrackingEnabled && m_pMotionDetector )
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshMotionTrackingDetector);
		if( !m_pMotionFrame ) ROBOT_ASSERT(0);
		if( !m_pMotionDetector ) ROBOT_ASSERT(0);

		m_MotionCenter = m_pMotionDetector->DetectMotion( m_pMotionFrame );

		if( 0 != m_MotionDetectorTime )
		{
			// Indicate that motion detection is disabled 
			// by putting an X on the screen
			CvPoint pt1 = { 10, 10 };
			CvPoint pt2 = { m_pVideoFrameLeft->width -10, m_pVideoFrameLeft->height -10 };
			cvLine(m_pColorBlobFrame, pt1, pt2, CV_RGB(255,0,0), 2, 4, 0);
			CvPoint pt3 = { 10, m_pVideoFrameLeft->height -10 };
			CvPoint pt4 = { m_pVideoFrameLeft->width -10, 10 };
			cvLine(m_pColorBlobFrame, pt3, pt4, CV_RGB(255,0,0), 2, 4, 0);
			m_MotionCenter.x =0;	// prevent camera from chasing noise
			m_MotionCenter.y =0;
		}
		else if( (0 != m_MotionCenter.x) || (0 != m_MotionCenter.y) )
		{
			CvPoint pt1 = { m_MotionCenter.x - 8, m_MotionCenter.y - 8 };
			CvPoint pt2 = { m_MotionCenter.x + 8, m_MotionCenter.y + 8 };
			cvRectangle(m_pColorBlobFrame, pt1, pt2, CV_RGB(0,255,0), 5, 4, 0);
			m_FramesSinceMotionLost = 0;
//			m_LastMotionCenter = m_MotionCenter;
		}

		cvShowImage( CAMERA_WINDOW_NAME_MOTION, m_pColorBlobFrame );
		__itt_task_end(pDomainVidCapThread);		
	}

	//////////////////////////////////
	// PIR Motion Tracking - Using Head-mounted PIR sensors
	// Track in the X axis only, no Y tracking.

	#if SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI // Only Loki has PIR sensors
		if( (m_IRTrackingEnabled) )
		{
			__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshPIRTrackingDetector);

			// KLUDGE! IR Sensors not enabled yet!, but we do have PIR (thermal), let's use them!

			if( 0 != g_SensorStatus.ThermalPosition )
			{
				// Turn toward Heat Thermal / Motion source
				//m_PIR_Position.y = CAMERA_TILT_FACE_POSITION;	// Look for face

				// Turn toward Heat Thermal / Motion source
//				m_PIR_Position.x = CAMERA_PAN_PIR_TENTHDEGREES * g_SensorStatus.ThermalPosition;
				m_PIR_Position.x = 20 * g_SensorStatus.ThermalPosition;

				// Debug
				ROBOT_LOG( TRUE,"Camera PIR Tracking: Position = %d degrees\n", (m_PIR_Position.x/10) )
			}



			/***
			// Track closest object
			if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT]  <= IR_HEAD_TRACKING_RANGE ) ||		
				(g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] <= IR_HEAD_TRACKING_RANGE ) )		
			{
				// An object found within range of IR sensors.  Track it

				if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] )
				{
					m_IRTrackCenter.x = -50;	// Object is to the Left
				}
				else if( (g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] )
				{
					m_IRTrackCenter.x = 50;	// Object is to the Right
				}
				// Debug
				ROBOT_LOG( TRUE,"Camera IRTracking: Left = %d, Right = %d, Command = %d\n", 
					g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT], g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT], m_IRTrackCenter.x );
			}
			***/
			__itt_task_end(pDomainVidCapThread);		
		}
	#endif // SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI

	////////////////////////////////////////////////////////////////////////////////////
	// Done with all frame processing.  
	// Highlight items found, then we will position the camera
	{
		__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshPositionHead);

		if( (0 != m_FaceTrackCenter.x ) && (0 != m_FaceTrackCenter.y) )
		{
			cvRectangle(m_pVideoFrameLeft, m_FaceTrackCenter, m_FaceTrackCenter, CV_RGB(0,255,0), 3, 4, 0);
		}

		// Put a fat dot at target center
		if( (0 != m_CamShiftCenter.x ) && (0 != m_CamShiftCenter.y) )
		{
			cvRectangle(m_pVideoFrameLeft, m_CamShiftCenter, m_CamShiftCenter, CV_RGB(0,0,255), 3, 4, 0);
		}

		if( (0 != m_ColorBlobCenter.x ) && (0 != m_ColorBlobCenter.y) )
		{
		// Put a fat dot at target center
			CvPoint CenterBox1, CenterBox2;
			CenterBox1.x = m_ColorBlobCenter.x - 20;
			CenterBox1.y = m_ColorBlobCenter.y - 20;
			CenterBox2.x = m_ColorBlobCenter.x + 20;
			CenterBox2.y = m_ColorBlobCenter.y + 20;

			cvRectangle(m_pVideoFrameLeft, CenterBox1, CenterBox2, CV_RGB(250,150,0), 3, 5, 0);
			// Mark the bounding box outline - does not work, outline too big!
			// cvRectangle(m_pColorBlobFrame, ColorBlobBBpt1, ColorBlobBBpt2, CV_RGB(0,255,0), 5, 4, 0);
		}


		if( (0 != m_MotionCenter.x) || (0 != m_MotionCenter.y) )
		{
			CvPoint pt1 = { m_MotionCenter.x - 8, m_MotionCenter.y - 8 };
			CvPoint pt2 = { m_MotionCenter.x + 8, m_MotionCenter.y + 8 };
			cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(0,255,0), 5, 4, 0);
		}

		if( (0 != m_LaserSpot.x) || (0 != m_LaserSpot.y) )
		{
			CvPoint pt1 = { m_LaserSpot.x - 6, m_LaserSpot.y - 6 };
			CvPoint pt2 = { m_LaserSpot.x + 6, m_LaserSpot.y + 6 };
			cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(255,0,0), 2, 3, 0);
		}


		////////////////////////////////////////////////////////////////////////////////////
		// Position the camera as needed
		// Priority is as follows:
		// Audio tracking (in case a new person is speaking (TODO!)
		// User selected point
		// FaceTracking
		// Motion 
		// CamShiftTracking
		// IR Tracking


		// See if user selected a point with the mouse
		if( m_VideoMousePointSelected )
		{
			if( (0 != m_VideoMousePoint.x) && (0 != m_VideoMousePoint.y) )
			{
				// User Selected a point!
				PositionCamera( HEAD_OWNER_USER_CONTROL, m_VideoMousePoint );
				m_VideoMousePointSelected = FALSE;
				m_VideoMousePoint.x = 0;
				m_VideoMousePoint.y = 0;
			}
		}
		else if( (0 != m_FaceTrackCenter.x ) && (0 != m_FaceTrackCenter.y) )
		{
			// DEBUG - Indicate the target to move the camera towards
				CvPoint pt1 = { m_FaceTrackCenter.x - 8, m_FaceTrackCenter.y - 8 };
				CvPoint pt2 = { m_FaceTrackCenter.x + 8, m_FaceTrackCenter.y + 8 };
				cvRectangle(m_pVideoFrameLeft, pt1, pt2, CV_RGB(0,255,0), 5, 4, 0);

			PositionCamera( HEAD_OWNER_FACE_TRACKER, m_FaceTrackCenter );
			Sleep(1);	// Let Camera serial thread run
			ZoomCameraToWindowSize( m_FaceTrackSize );
			m_LastFaceCenter = m_FaceTrackCenter;
			m_FaceTrackCenter.x = 0;
			m_FaceTrackCenter.y = 0;
			m_FaceTrackSize.height = 0;
			m_FaceTrackSize.width = 0;
		}	
		else if( (0 != m_ObjectTrackingCenter.x) && (0 != m_ObjectTrackingCenter.y) )
		{
			PositionCamera( HEAD_OWNER_BEHAVIOR_P2, m_ObjectTrackingCenter );
			Sleep(1);	// Let Camera serial thread run
			m_ObjectTrackingCenter.x = 0;
			m_ObjectTrackingCenter.y = 0;
		}
		else if( (0 != m_LaserSpot.x) && (0 != m_LaserSpot.y) )
		{
			PositionCamera( HEAD_OWNER_BEHAVIOR_P2, m_LaserSpot );
			m_LaserSpot.x = 0;
			m_LaserSpot.y = 0;
		}
		else if( (0 != m_MotionCenter.x ) && (0 != m_MotionCenter.y) && (0 == m_MotionDetectorTime) )
		{
			TrackMotion(); // try to find the person (again?)
			m_MotionCenter.x = 0;
			m_MotionCenter.y = 0;
		}
		else if( (0 != m_CamShiftCenter.x ) && (0 != m_CamShiftCenter.y) )
		{
			PositionCamera( HEAD_OWNER_BEHAVIOR_P2, m_CamShiftCenter );
			m_CamShiftCenter.x = 0;
			m_CamShiftCenter.y = 0;
		}
		else if( (0 != m_ColorBlobCenter.x ) && (0 != m_ColorBlobCenter.y) )
		{
			PositionCamera( HEAD_OWNER_BEHAVIOR_P2, m_ColorBlobCenter );
			m_ColorBlobCenter.x = 0;
			m_ColorBlobCenter.y = 0;
		}
		else if( (0 != m_IRTrackCenter.x ) )// && (0 != m_IRTrackCenter.y)  )
		{
			PositionCamera( HEAD_OWNER_BEHAVIOR_P2, m_IRTrackCenter );
			m_IRTrackCenter.x = 0;
		}
		else if( (0 != m_PIR_Position.x ) )
		{
			// Can't use PositionCamera, because that takes a relative PIXEL position based upon current view
			m_pHeadControl->SetHeadSpeed( HEAD_OWNER_PIR_TRACKER, SERVO_SPEED_MED, SERVO_SPEED_MED, SERVO_SPEED_MED );
			/*** TODO - Need to TUNE this Y code a bit before using it. Need to have delay since last face deteced so it does not jump around
			int CurrentPan;
			int CurrentTilt;
			int CurrentSideTilt;
			m_pHeadControl->GetHeadPosition( CurrentPan, CurrentTilt, CurrentSideTilt  );
			if( CurrentTilt < CAMERA_TILT_FACE_POSITION )
			{	// only tilt head higher than it alread is, not lower
				m_pHeadControl->SetHeadPosition( HEAD_OWNER_PIR_TRACKER, NOP, CAMERA_TILT_FACE_POSITION, NOP );	// Pan, Tilt, SideTilt
			}
			*/
			m_pHeadControl->SetHeadPositionRelative( HEAD_OWNER_PIR_TRACKER, m_PIR_Position.x, NOP, NOP );	// Pan, Tilt, SideTilt
			m_pHeadControl->ExecutePositionAndSpeed( HEAD_OWNER_PIR_TRACKER );
			m_PIR_Position.x = 0;
			m_PIR_Position.y = 0;
		}
	/**
		else if( (0 == m_MotionCenter.x ) && (0 == m_MotionCenter.y) )
		{
			// No motion detected
			if( m_FramesSinceMotionLost < 3 )
			{
				// Had motion, then lost it.  Assume target moved off camera in same direction
				ZoomAbs(0);	// go to wide mode (minimum zoom)
				CvPoint TargetPosition;
				TargetPosition.x = m_LastMotionCenter.x;
				TargetPosition.y = m_pVideoFrameLeft->height / 2;	// Leave tilt alone
				PositionCamera( TargetPosition );
				m_MotionCenter.x = 0;
				m_MotionCenter.y = 0;
				m_FramesSinceMotionLost++;
			}
			else
			{
				CameraIdleBehavior();
			}
		}
	**/
		else
		{
			///TAL_SCOPED_TASK_NAMED("Idle Behavior");
			CameraIdleBehavior();
			// TODO-CAR-MUST - Do scanning to look for Cone here?
			// Do Color Blob Search, if enabled
		}

		CvPoint pt1, pt2;
		CvPoint FrameCenter = GetFrameCenter(m_pVideoFrameLeft);
		//pVideoFrame->width

	#define CROSS_HAIR_SIZE	20

		// Vertical line
		// Full width of frame (for debugging)
		pt1.x = FrameCenter.x;
		pt1.y = 0;
		pt2.x = FrameCenter.x;
		pt2.y = m_pVideoFrameLeft->height;

		// Normal size
	/*	pt1.x = FrameCenter.x;
		pt1.y = FrameCenter.y - CROSS_HAIR_SIZE;
		pt2.x = FrameCenter.x;
		pt2.y = FrameCenter.y + CROSS_HAIR_SIZE;
	*/
		cvLine(m_pVideoFrameLeft, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);


		// Horizontal line
		// Full height of frame (for debugging)
		pt1.y = FrameCenter.y;
		pt1.x = 0;
		pt2.y = FrameCenter.y;
		pt2.x = m_pVideoFrameLeft->width;

		// Normal size
	/*	pt1.y = FrameCenter.y;
		pt1.x = FrameCenter.x - CROSS_HAIR_SIZE;
		pt2.y = FrameCenter.y;
		pt2.x = FrameCenter.x + CROSS_HAIR_SIZE;
	*/
		cvLine(m_pVideoFrameLeft, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);


		// Draw Laser Scanner target line
	#define LASER_LINE_SIZE	150	//40
		int LaserScannerLine_y = (int)( (double)m_pVideoFrameLeft->height * 0.83);

		pt1.y = LaserScannerLine_y;
		pt1.x = FrameCenter.x - LASER_LINE_SIZE;
		pt2.y = LaserScannerLine_y;
		pt2.x = FrameCenter.x + LASER_LINE_SIZE;
		cvLine(m_pVideoFrameLeft, pt1, pt2, CV_RGB(255,0,0), 1, 4, 0);


		// Draw Kinect target lines
		if( NULL != m_pKinectDepthFrame )
		{
			FrameCenter = GetFrameCenter(m_pKinectDepthFrame);

			// Vertical line
			pt1.x = FrameCenter.x;
			pt1.y = 0;
			pt2.x = FrameCenter.x;
			pt2.y = m_pKinectDepthFrame->height;
			cvLine(m_pKinectDepthFrame, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);
			cvLine(m_pKinectVideoFrame, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);

			// Horizontal line
			pt1.y = FrameCenter.y;
			pt1.x = 0;
			pt2.y = FrameCenter.y;
			pt2.x = m_pKinectDepthFrame->width;
			cvLine(m_pKinectDepthFrame, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);
			cvLine(m_pKinectVideoFrame, pt1, pt2, CV_RGB(0,255,0), 1, 4, 0);
		}
		__itt_task_end(pDomainVidCapThread);
	}

}

void CCameraModule::BeginScanningMode()
{
	// Start a slow pan of the area, looking for the Color Blob
	ROBOT_LOG( TRUE, "Beginning Scan for Color Blob...\n")
	if( rand() > (RAND_MAX/2) ) // 50/50 chance
	{
		// Start on left and scan right
		SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)CAMERA_SCAN_PAN_LIMIT_LEFT, 0 );
		SendCommand( WM_ROBOT_CAMERA_TILT_ABS_CMD, (DWORD)CAMERA_TILT_CENTER, 0 );
		m_ServoDelayTime = SERVO_DELAY_LONG; // Give Servo time to move all the way to the end
		m_ColorSearchingState = CAMERA_MODE_SCAN_PAN_RIGHT;	// Go to next state
	}
	else
	{
		// Start on right and scan left
		SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)CAMERA_SCAN_PAN_LIMIT_RIGHT, 0 );
		SendCommand( WM_ROBOT_CAMERA_TILT_ABS_CMD, (DWORD)CAMERA_TILT_CENTER, 0 );
		m_ServoDelayTime = SERVO_DELAY_LONG; // Give Servo time to move all the way to the end
		m_ColorSearchingState = CAMERA_MODE_SCAN_PAN_LEFT;	// Go to next state
	}
}

void CCameraModule::CenterCamera()
{
	SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)CAMERA_PAN_CENTER, 0 );
	SendCommand( WM_ROBOT_CAMERA_TILT_ABS_CMD, (DWORD)CAMERA_TILT_CENTER, 0 );
	m_ServoDelayTime = SERVO_DELAY_MEDIUM; // Give Servo time to move
	m_ColorSearchingState = IDLE;	// Go to idle state

}

/*void CCameraModule::CheckPositionLimits( int &NewPan, int &NewTilt)
{
	// Make sure Pan or Tilt command is within Camera limits

	if( NewPan > CAMERA_PAN_TENTHDEGREES_MAX_RIGHT )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Pan Limit Reached: CAMERA_PAN_TENTHDEGREES_MAX_RIGHT\n")
		NewPan = CAMERA_PAN_TENTHDEGREES_MAX_RIGHT;
	}
	else if( NewPan < CAMERA_PAN_TENTHDEGREES_MAX_LEFT )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Pan Limit Reached: CAMERA_PAN_TENTHDEGREES_MAX_LEFT\n")
		NewPan = CAMERA_PAN_TENTHDEGREES_MAX_LEFT;
	}

	// Make sure Tilt command is within Camera limits
	if( NewTilt > CAMERA_TILT_TENTHDEGREES_MAX_UP )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Pan Limit Reached: CAMERA_TILT_TENTHDEGREES_MAX_UP\n")
		NewTilt = CAMERA_TILT_TENTHDEGREES_MAX_UP;
	}
	else if( NewTilt < CAMERA_TILT_TENTHDEGREES_MAX_DOWN )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Pan Limit Reached: CAMERA_TILT_TENTHDEGREES_MAX_DOWN\n")
		NewTilt = CAMERA_TILT_TENTHDEGREES_MAX_DOWN;
	}
}

void CCameraModule::CheckTrackingPositionLimits( int &NewPan, int &NewTilt)
{
	// Try to keep camera toward front of robot, so it does not go off in la la land!

	if( NewPan > CAMERA_PAN_TENTHDEGREES_MAX_TRACK_RIGHT )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Pan Track Limit Reached: CAMERA_PAN_TENTHDEGREES_MAX_RIGHT\n")
		NewPan = CAMERA_PAN_TENTHDEGREES_MAX_TRACK_RIGHT;
	}
	else if( NewPan < CAMERA_PAN_TENTHDEGREES_MAX_TRACK_LEFT )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Pan Track Limit Reached: CAMERA_PAN_TENTHDEGREES_MAX_LEFT\n")
		NewPan = CAMERA_PAN_TENTHDEGREES_MAX_TRACK_LEFT;
	}

	// Make sure Tilt command is within Camera limits
	if( NewTilt > CAMERA_TILT_TENTHDEGREES_MAX_TRACK_UP )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Tilt Track Limit Reached: CAMERA_TILT_TENTHDEGREES_MAX_UP\n")
		NewTilt = CAMERA_TILT_TENTHDEGREES_MAX_TRACK_UP;
	}
	else if( NewTilt < CAMERA_TILT_TENTHDEGREES_MAX_TRACK_DOWN )
	{
		ROBOT_LOG( TRUE," CAMERAMODULE: Tilt Track Limit Reached: CAMERA_TILT_TENTHDEGREES_MAX_DOWN\n")
		//NewTilt = CAMERA_TILT_TENTHDEGREES_MAX_TRACK_DOWN;
	}
}
*/
void CCameraModule::ScanForColorBlob()
{
	// Used for Carbot, when looking for Orange cone on the course.

	if(	!m_ColorTrackingActive )
	{
		// The detector is enabled, but not don't track any colors at this time.
		return;
	}

	if( (0 == m_pColorDetector->GetCrTarget()) && (0 == m_pColorDetector->GetCbTarget()) )
	{
		// Target Color not set!
		CString MsgString;
		MsgString.Format( "ERRPOR!  Can't do Color Blob Search, Target Color not set!" );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString );
		m_ColorSearchingState = IDLE;
		return;
	}

	if( (0 != m_ColorBlobCenter.x) || (0 != m_ColorBlobCenter.y) )
	{
		// ColorBlob FOUND in current frame!
		if( !g_ColorBlobDetected )
		{
			// If last state did not detect a color blob, indicate we found one!
			// Beep(600, 100);	// Beep to indicate Color Blob found
			// Beep(900, 100);	
		}
		g_ColorBlobDetected = TRUE;	// Flag for Navigation control
		m_FrameRetryCount = OBJECT_DROP_RETRYS;	// Reset the bad frame counter
		//m_ServoDelayTime = SERVO_DELAY_DISABLED;		// Ignore any pending servo moves?
		return;
	}

	// Color blob not found in current frame.
	if( g_ColorBlobDetected )
	{
		// We were tracking a color blob, but now we lost it
		// Beep(900, 100);	// Beep to indicate Color Blob lost
		// Beep(600, 100);
	}

	g_ColorBlobDetected = FALSE;	// Flag for Navigation control

	if( --m_FrameRetryCount > 0 )
	{
		// We were tracking an object and lost it.
		// Retry for some number of frames to find the object, 
		// in case it's just not being picked up by the color blob detector
		return;
	}


	if( SERVO_DELAY_DISABLED != m_ServoDelayTime )
	{
		// check to see if servo done moving
		DWORD dwCurrTime = GetTickCount();
		int MoveTimeLeft = m_ServoDelayTime - dwCurrTime;  // m_ServoDelayTime is the target system tic count, in ms.
		if( MoveTimeLeft < 0 )
		{
			// Time expired, we're done waiting
			m_ServoDelayTime = SERVO_DELAY_DISABLED;
		}
		else
		{
			// Wait for servo to finish move before moving to next state.
			// Note that we do this AFTER checking for color blob, in case the
			// color blob happens to get spotted during a move!
			return;
		}
	}




	//////////////////////////////////////////////////////////////////////////
	// Color blob not found in current frame.  Decide how to search for it.
	switch( m_ColorSearchingState )
	{
		case IDLE:
		{
			// Idle mode - no need to move the camera
			//Sleep( 50 );
			break;
		}

		case CAMERA_MODE_TRACKING_COLOR:
		{

			// Last state, we were tracking a color, so we must have just lost it
			// Do a quick check to the left and right of the current position, in case
			// it moved just slightly out of the frame (ya gotta watch those sneaky cones!)
				// move left
				// set state looking left
				// if state looking left
				// move right
				// if state looking right
				// got to next state (check scanning mode?)


			// Look Left (from last position)
			SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(-CAMERA_SCAN_PAN_STEP_SIZE), 0 );
			m_ColorSearchingState = CAMERA_MODE_LOOKING_LEFT;
			m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
			break;
		}

		case CAMERA_MODE_LOOKING_LEFT:
		{
			// Was looking left, now go back to last known position and check again (step right once)
			SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(CAMERA_SCAN_PAN_STEP_SIZE), 0 );
			m_ColorSearchingState = CAMERA_MODE_LOOKING_CENTER;
			m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
			break;
		}

		case CAMERA_MODE_LOOKING_CENTER:
		{
			// Tried left and center, now look right
			SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(CAMERA_SCAN_PAN_STEP_SIZE), 0 );
			m_ColorSearchingState = CAMERA_MODE_LOOKING_RIGHT;
			m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
			break;
		}

		case CAMERA_MODE_LOOKING_RIGHT:
		{
			// We tried a quick peek left and right from last known position.  
			// Give up and try whatever scan mode we are in.

			if( m_ColorScanningEnabled )
			{
				// Scanning requested.  Start a new scan of the area
				BeginScanningMode();
			}
			else
			{
				// Otherwise, go to center
				ROBOT_LOG( TRUE, "Nothing Right or Left of last position.  Centering Camera\n" )
				CenterCamera();
			}

			break;
		}

/*		case CAMERA_MODE_SCAN:
		{
			// Start a slow pan of the area, looking for the Color Blob
			// Start on left and scan right
			ROBOT_LOG( TRUE, "Searching for Color Blob...\n"))
			SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)CAMERA_SCAN_PAN_LIMIT_LEFT, 0 );
			SendCommand( WM_ROBOT_CAMERA_TILT_ABS_CMD, (DWORD)CAMERA_TILT_CENTER, 0 );
			m_ServoDelayTime = SERVO_DELAY_LONG; // ms. Give Servo time to move all the way to the end
			m_ColorSearchingState = CAMERA_MODE_SCAN_PAN_RIGHT;	// Go to next state
			break;
		}
*/
		case CAMERA_MODE_SCAN_PAN_RIGHT:
		{
			// Pan Right
			if( m_pHeadControl->GetPanPosition() >= CAMERA_SCAN_PAN_LIMIT_RIGHT )
			{
				// reached end of Right Scan.  Quit scanning, or Pan Left?
				if( m_ColorScanningEnabled )
				{
					SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(-CAMERA_SCAN_PAN_STEP_SIZE), 0 );
					m_ColorSearchingState = CAMERA_MODE_SCAN_PAN_LEFT;
					m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
				}
				else
				{
					ROBOT_LOG( TRUE, "Centering Camera (after right pan)\n")
					CenterCamera();
				}
			}
			else
			{
				// Continue scanning Right
				SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(CAMERA_SCAN_PAN_STEP_SIZE), 0 );
				m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
			}
			break;
		}

		case CAMERA_MODE_SCAN_PAN_LEFT:
		{
			// Pan Left
			if( m_pHeadControl->GetPanPosition() >= CAMERA_SCAN_PAN_LIMIT_LEFT )
			{
				// reached end of Left Scan.  Quit scanning, or Pan Right?
				if( m_ColorScanningEnabled )
				{
					SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(CAMERA_SCAN_PAN_STEP_SIZE), 0 );
					m_ColorSearchingState = CAMERA_MODE_SCAN_PAN_RIGHT;
					m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
				}
				else
				{
					ROBOT_LOG( TRUE, "Centering Camera (after left pan)\n" )
					CenterCamera();
				}
			}
			else
			{
				// Continue scanning Left
				SendCommand( WM_ROBOT_CAMERA_PAN_REL_CMD, (DWORD)(-CAMERA_SCAN_PAN_STEP_SIZE), 0 );
				m_ServoDelayTime = SERVO_DELAY_SHORT; // Give Servo time to respond
			}
			break;
		}


/*		case CAMERA_PROGRESSIVE_SCAN:
*/


		default:
		{
			ROBOT_LOG( TRUE,">>>> ScanForColorBlob: Bad Camera Mode!\n" )
		}
	}	// end switch
}

void CCameraModule::CameraIdleBehavior()
{
	__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshCameraIdle);

	m_LastFaceCenter.x = 0;
	m_LastFaceCenter.y = 0;
	m_LastMotionCenter.x = 0;
	m_LastMotionCenter.y = 0;

	if(0 != m_MotionDetectorTime )
	{
		__itt_task_end(pDomainVidCapThread);
		return;	// camera moving
	}

	if( !gHeadIdle )
	{		
		__itt_task_end(pDomainVidCapThread);
		return; // Idle behavior disabled
	}

	// Haven't seen anyone in a while. 
	if( MIN_ZOOM_LEVEL != g_CameraZoomPos )
	{
		ZoomAbs(MIN_ZOOM_LEVEL);	// go to wide mode (minimum zoom)
		ROBOT_LOG( TRUE,"Camera Idle, so Zooming Out!\n")
	}

	__itt_task_end(pDomainVidCapThread);	
}

void CCameraModule::TrackMotion()
{
	static DWORD LastZoomTime = 0;
	__itt_task_begin(pDomainVidCapThread, __itt_null, __itt_null, pshTrackMotion);

	// Only called if motion detector found something, and camera not moving.
	DWORD TimeSinceFaceLost = GetTickCount() - m_FaceLostTime;

	if( TimeSinceFaceLost < 500 )
	{
		// Just lost the person... stay put and see if they come back
		ROBOT_LOG( TRUE,"MotionTrackDBG: TimeSinceFaceLost=%d, Waiting to see if FACE TRACKER Reaquires\n", TimeSinceFaceLost)
	}
	else if( (TimeSinceFaceLost < 2000) && (0 != m_LastFaceCenter.y) )
	{
		// Had someone, but lost them.  Use Pan only to try to reaquire
		CvPoint TargetPosition;
		TargetPosition.x = m_MotionCenter.x;
		TargetPosition.y = m_LastFaceCenter.y;
		PositionCamera( HEAD_OWNER_MOTION_TRACKER, TargetPosition );
		m_MotionCenter.x = 0;
		m_MotionCenter.y = 0;
		ROBOT_LOG( TRUE,"MotionTrackDBG: TimeSinceFaceLost=%d, Pan Only at LAST Face Level\n", TimeSinceFaceLost)
	}
	else 
	{
		// Start from scratch.  We see motion, so see if we can find a face
		DWORD TimeSinceLastZoom = GetTickCount() - LastZoomTime;

		if( (m_pHeadControl->GetTiltPosition() < (CAMERA_TILT_FACE_POSITION-20) ) ||  // +/- 2 degrees
			(m_pHeadControl->GetTiltPosition() > (CAMERA_TILT_FACE_POSITION+20) ) ) // Value in Tenthdegrees
		{
			// Don't keep sending over and over
			m_pHeadControl->SetHeadSpeed( HEAD_OWNER_MOTION_TRACKER, SERVO_SPEED_MED, SERVO_SPEED_MED, SERVO_SPEED_MED );
			m_pHeadControl->SetHeadPosition( HEAD_OWNER_MOTION_TRACKER, NOP, CAMERA_TILT_FACE_POSITION, 0 ); 
			m_pHeadControl->ExecutePositionAndSpeed( HEAD_OWNER_MOTION_TRACKER );
			ROBOT_LOG( TRUE,"MotionTrackDBG: Setting Motion Track tilt to CAMERA_TILT_FACE_POSITION\n")
			return; //260
		}

		if( TimeSinceLastZoom > 10000 )
		{
			// every n seconds try zooming in to find a face
			ZoomAbs(3);	// Zoom in a bit
			LastZoomTime = GetTickCount();
		}
		else
		{
			// Zoom out completely
			ZoomAbs(0);	// go to wide mode (minimum zoom)
		}

		CvPoint TargetPosition;
		TargetPosition.x = m_MotionCenter.x;
		TargetPosition.y = m_pVideoFrameLeft->height / 2; // center of frame (no Y movement)
		PositionCamera( HEAD_OWNER_MOTION_TRACKER, TargetPosition );
		m_MotionCenter.x = 0;
		m_MotionCenter.y = 0;
		ROBOT_LOG( TRUE,"MotionTrackDBG: TimeSinceFaceLost=%d, Pan at STANDARD Face Level\n", TimeSinceFaceLost)
	}
	__itt_task_end(pDomainVidCapThread);
}

/* void CCameraModule::WaitingForSomethingInteresting()
{
	// Only called if nothing detected for a while.

}
*/
void CCameraModule::ZoomAbs( int nZoomLevel )
{
	// Zooms camera to level specified
	if( nZoomLevel > 16 )
	{
		ROBOT_LOG( TRUE, "ZoomAbs - BAD Zoom Level: %d \n", nZoomLevel )
		nZoomLevel = 16;
	}
	else if( nZoomLevel < 0 )
	{
		ROBOT_LOG( TRUE, "ZoomAbs - BAD Zoom Level: %d \n", nZoomLevel )
		nZoomLevel = 0;
	}

	if( nZoomLevel != g_CameraZoomPos )
	{
		SendCommand( WM_ROBOT_CAMERA_ZOOM_ABS_CMD, (DWORD)nZoomLevel, 0 );
		// Since Zoom is slow, allow extra time before enabling motion detector
		g_LastCameraMoveTime = GetTickCount(); // And delay IDLE behavior a bit longer
		m_MotionDetectorTime = g_LastCameraMoveTime + MOTION_DETECTOR_TIME;
	}
}

void CCameraModule::ZoomIn( int ZoomDelta )
{
	// Zooms camera in by Delta specified
	int  nZoomLevel = g_CameraZoomPos + ZoomDelta;
	if( nZoomLevel > 16 )
	{
		ROBOT_LOG( TRUE, "ZoomIn - BAD Zoom Level: %d \n", nZoomLevel )
		nZoomLevel = 16;
	}

	if( g_CameraZoomPos < 16 )
	{
		ROBOT_LOG( TRUE, "CCamera Requesting Zoom Level: %d \n", nZoomLevel )
		SendCommand( WM_ROBOT_CAMERA_ZOOM_ABS_CMD, (DWORD)nZoomLevel, 0 );
		g_LastCameraMoveTime = GetTickCount(); // And delay IDLE behavior a bit longer
		m_MotionDetectorTime = g_LastCameraMoveTime + MOTION_DETECTOR_TIME;
	}
}

void CCameraModule::ZoomOut( int ZoomDelta )
{
	// Zooms camera in by Delta specified
	int  nZoomLevel = g_CameraZoomPos - ZoomDelta;
	if( nZoomLevel < MIN_ZOOM_LEVEL )
	{
		// Not an error, just hit max zoom out
		return;
	}

	if( g_CameraZoomPos > 0 )
	{
		ROBOT_LOG( TRUE, "CCamera Requesting Zoom Level: %d \n", nZoomLevel )
		SendCommand( WM_ROBOT_CAMERA_ZOOM_ABS_CMD, (DWORD)nZoomLevel, 0 );
		g_LastCameraMoveTime = GetTickCount(); // And delay IDLE behavior a bit longer
		m_MotionDetectorTime = g_LastCameraMoveTime + MOTION_DETECTOR_TIME;
	}
}

void CCameraModule::ZoomCameraToWindowSize( CvSize TargetWindowSize )
{
	// Given current window size and target size, find correct zoom amount

	if( (0 == TargetWindowSize.width) || (0 == TargetWindowSize.width) )
	{
		ROBOT_LOG( TRUE,"ERROR! ZoomCameraToWindowSize - invalid size!\n")
		ROBOT_ASSERT(0);
		return;
	}

	CvSize CurrentBoarderSize;
	CurrentBoarderSize.width = m_pVideoFrameLeft->width - TargetWindowSize.width;
	CurrentBoarderSize.height = m_pVideoFrameLeft->height - TargetWindowSize.height;

	if( (CurrentBoarderSize.width < 0) || (CurrentBoarderSize.height < 0) )
	{
		ROBOT_LOG( TRUE,"ERROR! ZoomCameraToWindowSize - invalid Current Boarder Size!\n")
		ROBOT_ASSERT(0);
		return;
	}

	int MinCurrentBoarder = min(CurrentBoarderSize.width, CurrentBoarderSize.height);
	if( MinCurrentBoarder > (OPTIMAL_WINDOW_BOARDER + WINDOW_BOARDER_FUDGE) )
	{
		// Current boarder too big (current window too small), so zoom in
		ZoomIn(1);
	}
	else if( MinCurrentBoarder < (OPTIMAL_WINDOW_BOARDER - WINDOW_BOARDER_FUDGE) )
	{
		// Current boarder too small (current window too big), so zoom out
		ZoomOut(1);
	}
}

void CCameraModule::PositionCamera( int nOwner, CvPoint CameraPanTiltPoint)
{
	// Adjust camera position to point at target

	ROBOT_LOG( TRUE,"DEBUG: PositionCamera: CameraPanTiltPoint = %d,%d\n",CameraPanTiltPoint.x, CameraPanTiltPoint.y )

	CString	MsgString;
	BOOL	MovingServo = FALSE;
	int		TenthDegreeMoveX = 0;
	int		TenthDegreeMoveY = 0;

	g_LastCameraMoveTime = GetTickCount(); // indicate when to execute IDLE behavior

	// Pan Camera to center on item
	// Global Zoom value g_CameraZoomPos ranges from 0 - 16,   

#if( CAMERA_CONTROL_TYPE == SONY_SERIAL_CAMERA )
	// This table is for SONY camera:
	// Table to map Zoom level to camera Field Of View
	// Camera values are non-linear, so table is used instead of formula.  See CameraZoom.xls spreadsheet for details
	const double FovDegreesTable[17] =
	{
		//0		  1		  2		  3		  4		  5		  6		  7		  8		// Zoom Level
// 		77.77,	74.33,	68.23,	64.34,	58.89,	52.39,	48.61,	42.32,	37.42,	//FovDegrees Calculated
 		79.0,	73.0,	66.0,	60.0,	54.0,	48.0,	42.0,	37.0,	32.0,	//FovDegrees Trimmed

		//9		 10		 11		 12		 13		 14		 15		 16				// Zoom Level
//		32.38,	28.94,	23.68,	19.22,	15.61,	12.88,	10.14,	8.30			//FovDegrees Calculated
		27.0,	24.0,	20.5,	16.5,	14.0,	11.5,	9.0,	7.5				//FovDegrees Trimmed
	};

	double DegreesPerFrameX = FovDegreesTable[g_CameraZoomPos];	// 8.3 - 78 ZOOM
	// Factor in Zoom level and ratio of Frame Height to Frame Width
	double DegreesPerFrameY = (FovDegreesTable[g_CameraZoomPos]);
	DegreesPerFrameY = DegreesPerFrameY * GetFrameAspectRatio();


#elif( CAMERA_CONTROL_TYPE == DYNA_SERVO_CONTROL )
	// This table is for LOKI, using "Logitech Quickcam Pro for Notebooks"
	// WARNING!!! ZOOM NOT SUPPORTED!
	// Calculation used: Robot at 52" from wall.  FOV width = 59", height = 43.8"
	// ArcTan = Opp/Adj. Opp = 1/2 width or height (to form right triangle)
	// Width  HALF FOV = ArcTan( (59/2) / 52 )   => HALF_FOV_X = 29.567.  FOV_X = 59.133 degrees
	// Height HALF FOV = ArcTan( (43.8/2) / 52 ) => HALF_FOV_Y = 22.839.  FOV_Y = 45.677 degrees
	// Now, adjust values as needed, since camera not mounted exactly over center of servos!
	#define FOV_X	64.0 	// 59 calculated.  Non-linear due to camera lens
	#define FOV_Y	45.0	// 45.677 degrees calculated

	double DegreesPerFrameX = FOV_X;
	double DegreesPerFrameY = FOV_Y;

//#elif( CAMERA_CONTROL_TYPE == PIC_SERVO_CONTROL )
//#elif( CAMERA_CONTROL_TYPE == EXTERN_SERVO_CONTROL )
	//TODO-CARBOT-MUST - Add FOV table here!
#else
	#error BAD CAMERA_CONTROL_TYPE
#endif


	// Camera Pan
	int PixelDeltaX = CameraPanTiltPoint.x - GetFrameCenter(m_pVideoFrameLeft).x;
	if( abs(PixelDeltaX) > CAMERA_PIXEL_FUDGE )
	{
		// Calculate Pixels per 1/10 Degree.
		double PixelsPerTenthDegreeX = m_pVideoFrameLeft->width / (DegreesPerFrameX * 10);
//		Old:		double ServoTicksPerPixelX = 5.0 / PixelsPerDegreeX;	// 5.0?  7.8?
//		double ServoTicksPerPixelX = CAMERA_PAN_PER_DEGREE / PixelsPerDegreeX;
		// KLUDGE FOR CAR TODO-CAR-MUST REMOVE / FIX THIS
//		ServoTicksPerPixelX =  0.3; // HARD CODED!!!
//		ServoMoveX = int(PixelDeltaX * ServoTicksPerPixelX);

		TenthDegreeMoveX = (int)((double)PixelDeltaX / PixelsPerTenthDegreeX );
		MovingServo = TRUE;
//		Sleep(10);

		// Limit how far from center Camera tracking will go (avoid la la land!)

	}

	// Camera Tilt
	int PixelDeltaY =  GetFrameCenter(m_pVideoFrameLeft).y - CameraPanTiltPoint.y;
	if( abs(PixelDeltaY) > CAMERA_PIXEL_FUDGE )
	{
		// Calculate Pixels per 1/10 Degree.
		double PixelsPerTenthDegreeY = m_pVideoFrameLeft->height / (DegreesPerFrameY *10);
//		double ServoTicksPerPixelY = 7.0 / PixelsPerDegreeY; 
//		double ServoTicksPerPixelY = CAMERA_TILT_PER_DEGREE / PixelsPerDegreeY; 
		// KLUDGE FOR CAR TODO-CAR-MUST REMOVE / FIX THIS
//		ServoTicksPerPixelY =  -0.3; // HARD CODED!!!
		
		TenthDegreeMoveY = (int)((double)PixelDeltaY / PixelsPerTenthDegreeY);
		MovingServo = TRUE;
	}

	if( MovingServo )
	{
		m_pHeadControl->SetHeadSpeed( nOwner, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW, SERVO_SPEED_MED_SLOW );
		m_pHeadControl->SetHeadPosition( nOwner, NOP, NOP, 0 ); // No Side-Tilt!
		m_pHeadControl->SetHeadPositionRelative( nOwner, TenthDegreeMoveX, TenthDegreeMoveY, NOP, TRUE ); // TRUE = Limit to front of robot
		m_pHeadControl->ExecutePositionAndSpeed( nOwner );
		ROBOT_LOG( TRUE,"CAMERA: Tracking Point: Relative Pan = %d  Tilt = %d\n", (int)(TenthDegreeMoveX/10), (int)(TenthDegreeMoveY/10) )
		ROBOT_LOG( TRUE,"***************> TRACKING SPEED FAST\n")
	}
}


/////////////////////////////////////////////////////////////////////////////
// VidCap - Video and Frame processing
//

/****
void CCameraModule::SetFrameSizeInfo( IplImage* m_pVideoFrameLeft )
{
	// Calculate size info about the frame
	m_FrameSize = cvGetSize(pVideoFrame);
	m_CameraFrameCenter.x = m_FrameSize.width /2;
	m_CameraFrameCenter.y = m_FrameSize.height /2;
	m_FrameAspectRatio = (double)m_FrameSize.height/(double)m_FrameSize.width;

}
***/

void CCameraModule::MoveWindows( )
{
#define LEFT_WINDOW_MARGIN		12

	UINT LeftWindowOffset = 0;
	// Move frames based upon frame size

	// Handle RIGHT CAMERA
	if( m_pVideoFrameRight )
	{
		// Right camera is enabled.  Left window will move over to make room
		CvSize FrameSizeRight = cvGetSize(m_pVideoFrameRight);
		if( 0 == FrameSizeRight.width )
		{
			ROBOT_ASSERT(0);
			return;
		}

		LeftWindowOffset = FrameSizeRight.width+LEFT_WINDOW_MARGIN;

		if( 120 == FrameSizeRight.height )
		{
			// 160x120
			cvMoveWindow( CAMERA_WINDOW_NAME_RIGHT, WindowPosX(m_pVideoFrameRight), 
				PROCESSED_WINDOW_POSITION_Y_SMALL );
		}
		else if( 240 == FrameSizeRight.height )
		{
			// 320x240
			cvMoveWindow( CAMERA_WINDOW_NAME_RIGHT, WindowPosX(m_pVideoFrameRight), 
				PROCESSED_WINDOW_POSITION_Y_MED );
		}
		else if( 480 == FrameSizeRight.height )
		{
			// 640x480
			cvMoveWindow( CAMERA_WINDOW_NAME_RIGHT, WindowPosX(m_pVideoFrameRight), 
				PROCESSED_WINDOW_POSITION_Y_LARGE );
		}
		else
		{
			ROBOT_ASSERT(0);
		}
	}


	// Handle LEFT CAMERA
	if( m_pVideoFrameLeft )
	{
		CvSize FrameSizeLeft = cvGetSize(m_pVideoFrameLeft);
		if( 0 == FrameSizeLeft.width )
		{
			ROBOT_ASSERT(0);
			return;
		}


		if( 120 == FrameSizeLeft.height )
		{
			// 160x120
			cvMoveWindow( CAMERA_WINDOW_NAME_LEFT, (WindowPosX(m_pVideoFrameLeft)-LeftWindowOffset), 
				PROCESSED_WINDOW_POSITION_Y_SMALL );
			if( m_MotionTrackingViewStarted )
			{
				cvMoveWindow( CAMERA_WINDOW_NAME_MOTION, WindowPosX(m_pVideoFrameLeft), 
					MOTION_WINDOW_POSITION_Y_SMALL );
			}
		}
		else if( 240 == FrameSizeLeft.height )
		{
			// 320x240
			cvMoveWindow( CAMERA_WINDOW_NAME_LEFT, (WindowPosX(m_pVideoFrameLeft)-LeftWindowOffset), 
				PROCESSED_WINDOW_POSITION_Y_MED );
			if( m_MotionTrackingViewStarted )
			{
				cvMoveWindow( CAMERA_WINDOW_NAME_MOTION, WindowPosX(m_pVideoFrameLeft), 
					MOTION_WINDOW_POSITION_Y_MED );
			}
		}
		else if( 480 == FrameSizeLeft.height )
		{
			// 640x480
			cvMoveWindow( CAMERA_WINDOW_NAME_LEFT, (WindowPosX(m_pVideoFrameLeft)-LeftWindowOffset), 
				PROCESSED_WINDOW_POSITION_Y_LARGE );
			if( m_MotionTrackingViewStarted )
			{
				cvMoveWindow( CAMERA_WINDOW_NAME_MOTION, WindowPosX(m_pVideoFrameLeft), 
					MOTION_WINDOW_POSITION_Y_LARGE );
			}
		}
		else
		{
			ROBOT_ASSERT(0);
		}
	}

}


#endif // #if ( ROBOT_SERVER == 1 )	// This module used for Robot Server only
