// Config.h - Configuration of debug and tracing switches
// 
#pragma once	// Only include this header once
#include "stdafx.h"
#include "RobotType.h"

//****************************************************************************
//                      SYSTEM DEBUG CONFIG SWITCHES
//****************************************************************************

// Bogus handle use for simulation
// Temp HACK - LONG_PTR is defined in DX9 SDK!
//#define SIMULATED_SIO_HANDLE	((HANDLE)(LONG_PTR)-2)
#define SIMULATED_SIO_HANDLE	((HANDLE)-2)
#define SIMULATED_SIO_SLEEP_TIME	5	// ms - time thread will sleep pretending to tx/rx data

// Simulation Override control - force robot to simulate SIO ports, etc.
#define FORCE_ROBOT_SIMULATION		0

// Force Kinect scan to be far from robot, and loop for debugging object detection
#define DEBUG_KINECT_FLOOR_OBJECT_DETECTION	0


// Enable / Disable break on Asserts
// Warning, if enabled, PC will freeze on ASSERT, but robot will continue driving!
#define ASSERT_ENABLED				1

// AI Engine
#define AI_ENGINE_ENABLED			0
#define SPEECH_ENGINE_ENABLED		1

// SENSOR DEBUG						1 = enable, 0 = disable
#define DEBUG_SENSOR_FUSION			0
#define DEBUG_IR					0
#define DEBUG_ULTRASONIC			0
#define DEBUG_PIR					0

#define REPORT_CLOSE_OBJECTS		0


//****************************************************************************

// Enable / Disable break on Asserts macro implementation
#if ASSERT_ENABLED == 1
 #define ROBOT_ASSERT(a) ASSERT(a)
#else
#define ROBOT_ASSERT(a) { if(a){} else { ROBOT_LOG( TRUE,  "*****> ASSERT DISABLED! FILE: "); ROBOT_LOG( TRUE, THIS_FILE); ROBOT_LOG( TRUE,  " <*****\n" ); } }
#endif

#if ( ROBOT_SERVER == 1 )
	// Enable or disable the AI engine here for debugging:

	#if ( ROBOT_TYPE == LOKI && AI_ENGINE_ENABLED == 1 )
		#define ENABLE_ROBOT_AI	1
	#else
		#define ENABLE_ROBOT_AI	0	// Always off for carbot
	#endif

#else
	#define ENABLE_ROBOT_AI	0	// Always off for client
#endif


// Delete data and set pointer to Null
#define SAFE_DELETE(p)  { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }


// Trace Macros
#define SERIAL_DBG			0 // enable this if PIC or servos don't work
#define CAMERA_DBG			0
#define MOTOR_DBG			0
#define NAV_DBG				1
#define MODULE_DBG			1
#define TEMP_DBG			1

 