// HardwareCmds.h
// Command interface between the PC and Microcontroller, and is shared by both compilers
// Make sure syntax works with all compilers needed
#ifndef __ROBOT_HW_COMMANDS_H__
#define __ROBOT_HW_COMMANDS_H__

#include "RobotType.h"

// Which servos are plugged in where
#define SERVO_SPEED						0
#define SERVO_TURN						1
#define SERVO_GEAR						2
#define SERVO_CAMERA_PAN				3
#define SERVO_CAMERA_TILT				4


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// NOTE - Also see HARDWARE CONFIG section in Globals.h

// General, applies to all robot types
	#define IR_SR_MAX_RANGE_TENTH_INCHES		240 // Tenth_Inches - Max range of Short Range IR sensors
	#define IR_SR_DETECT_RANGE_TENTH_INCHES		160 // Tenth_Inches - 4ft Max reliable range of Long range IR sensors for Object Finding
	#define IR_LR_MAX_RANGE_TENTH_INCHES		600 // Tenth_Inches - 5ft Max range of Long Range IR sensors
	#define IR_LR_DETECT_RANGE_TENTH_INCHES		480 // Tenth_Inches - 4ft Max reliable range of Long range IR sensors for Object Finding
	#define US_MAX_RANGE_TENTH_INCHES			960 // Tenth_Inches - 8ft Max range of US sensors
	#define US_MAX_DETECT_RANGE_TENTH_INCHES	960 // TODO Tenth_Inches - 8ft Max range of US sensors
	#define IR_RANGE_FUDGE_AMOUNT_TENTH_INCHES	 40	// Tenth_Inches - Amount of error in IR readings (for determining distance to objects)
	#define US_RANGE_FUDGE_AMOUNT_TENTH_INCHES	240 // Tenth_Inches - Amount of error in US readings (for determining distance to objects)


////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                            LOKI
////////////////////////////////////////////////////////////////////////////////////////////////////////
#if SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI

	/* Mr Roboto2 Sensor Configuration: (also see Globals.h)
	IR0: Left  Side Sensor angled almost 90 Degrees from forward
	IR1: Left  Forward, Low mounted, long range sensor
	IR2: Left  Head mounted, long range sensor.  Depends upon where the head is facing!
	IR3: Right Head mounted, long range sensor.  Depends upon where the head is facing!
	IR4: Right Forward, Low mounted, long range sensor
	IR5: Right Side Sensor angled almost 90 Degrees from forward

	US0: Head mounted UltraSonic Sensor
	US1: Left Forward mounted low UltraSonic Sensor
	US2: Right Forward mounted low UltraSonic Sensor

	IRB0-5: IR Bumpers
	I2C Expansion board: IRE2_0:
	*/

	#define OBJECT_EQUAL_DISTANCE		  0
	#define SIDE_SENSOR_ANGLE			 80	// degrees from forward TODO-LOKI-MUST is this right???
	#define ANGLE_SENSOR_ANGLE			 15 // Angled "IR Bumper" sensors - almost directly forward
	#define FORWARD_SENSOR_ANGLE		 10	// Hmm, use US angle, or IR angle?
	#define REAR_BUMPER_ANGLE			180
	#define REAR_IR_ANGLE				170
	#define VERTICAL_IR_DETECT_RANGE_TENTH_INCHES	 300	// Detect tables, etc that robot might run into

	#define REAR_RIGHT				(-REAR_IR_ANGLE)
	#define SIDE_RIGHT				 SIDE_SENSOR_ANGLE
	#define ANGLE_RIGHT				 ANGLE_SENSOR_ANGLE
	#define FORWARD_RIGHT			 FORWARD_SENSOR_ANGLE
	#define FORWARD_LEFT			(-FORWARD_SENSOR_ANGLE)	// Negative Numbers on Left side
	#define ANGLE_LEFT				(-ANGLE_SENSOR_ANGLE)
	#define SIDE_LEFT				(-SIDE_SENSOR_ANGLE)
	#define REAR_LEFT				(-REAR_IR_ANGLE)


	#define NUMBER_OF_WHEEL_ODOMETERS							  2		// LOKI has a Right and Left Odometer

	// WARNING! If you change these, you must update SensorOffsetDegrees in Globals.cpp!
	#define NUM_IR_SENSORS										  6	// 2 head mounted LR, 2 front facing LR, 2 side mounted SR
	#define NUM_IR3_SENSORS										  4 // 8	// I2C-IT IR Sensors - DEBUG!!!!
	#define NUM_IR2_SENSORS										  8	// I2C Sensors - DEBUG!!!!
	#define NUM_US_SENSORS										  3	// 1 head mounted, 2 front mounted

	// Sensor Field Of View for drawing sensor readings on the Map
	#define SENSOR_HALF_FOV_DEGREES_US							 150	// 1/2 of Sensor Field Of View in tenth degrees
	#define SENSOR_HALF_FOV_DEGREES_IR							 20	// This is actually bigger then sensor, but robot movement provides ambiguity

	// Sensor Positions in arrays
	#define	IR_SENSOR_SIDE_LEFT									  0
	#define	IR_SENSOR_FRONT_LEFT								  1
	#define	IR_SENSOR_HEAD_LEFT									  2
	#define	IR_SENSOR_HEAD_RIGHT								  3
	#define	IR_SENSOR_FRONT_RIGHT								  4
	#define	IR_SENSOR_SIDE_RIGHT								  5

	#define US_SENSOR_CAMERA									  0	// Mounted on Camera
	#define US_SENSOR_FRONT_LEFT								  1
	#define US_SENSOR_FRONT_RIGHT								  2


	#define IR_HEAD_TRACKING_RANGE_TENTH_INCHES					410 // Tenth_Inches - Head will track any object closer then this
	#define IR_TRACKING_FUDGE_TENTH_INCHES						 60 // Tenth_Inches
	#define IR_ELBOW_HIT_RANGE_TENTH_INCHES						 20 // Tenth_Inches - distance from front of claw tip (used by elbow sensor to detect impending arm collisions)	
	#define CLAW_DETECT_TRIGGER_DISTANCE_TENTH_INCHES			 60	// Tenth_Inches

	// Arm Length measurements defined in ArmControl.cpp
	// Arm Sensor measurements
	#define FOREARM_IR_TO_WRIST_OBSTRUCTION_TENTH_INCHES_L		 55.0	// Tenth_Inches - clear space even when wrist rotated
	#define FOREARM_IR_TO_WRIST_OBSTRUCTION_TENTH_INCHES_R		 80.0	// Tenth_Inches - clear space even when wrist rotated
	#define FOREARM_IR_TO_FINGER_TIP_TENTH_INCHES_L				130.0	// Tenth_Inches
	#define FOREARM_IR_TO_FINGER_TIP_TENTH_INCHES_R				102.5	// Tenth_Inches

	#define GRID_MAP_AVOID_OBJECT_DISTANCE						  60	// Tenth_Inches - Avoid objects closer then this distance to the robot


	////////////////////////////////////////////////////////////////
	#ifndef __PIC_TARGET	// Pic Compiler does not understand pack
	#pragma pack( 1 )
	#endif

	// Raw data from the PIC for Loki
	typedef struct
	{
		BYTE StatusFlags;		// 3
		BYTE LastError;			// 4
		BYTE DebugCode;			// 5
		BYTE HWBumper;			// 6
		BYTE IRBumper;			// 7
		BYTE ArmBumperL;		// 8
		BYTE ArmBumperR;		// 9
		BYTE Battery0;			//10
		BYTE CompassHigh;		//11
		BYTE CompassLow;		//12
		BYTE OdometerHighL;		//13
		BYTE OdometerLowL;		//14
		BYTE OdometerHighR;		//15
		BYTE OdometerLowR;		//16
		BYTE IR[NUM_IR_SENSORS];//17-22 	- UP TO N, may be less! - On main Pic board
		BYTE IR3[NUM_IR3_SENSORS];//23-26	- UP TO N, may be less! - I2C-IT IR Rangers
//		BYTE IR2[NUM_IR2_SENSORS];//		- UP TO N, may be less! - On external expansion board
//		BYTE ThermalArray[9];	//			- Ambient + 8 Pixels - Devantech TPA81 Thermal Array Sensor
//		BYTE US[NUM_US_SENSORS];//	

	} PIC_STATUS_T;
	#ifndef __PIC_TARGET
	#pragma pack( 4 )
	#endif

// I2C-IT Sensors
#define ARM_L_IR_SENSOR_CLAW		g_SensorStatus.IR3[0]
#define ARM_L_IR_SENSOR_ELBOW		g_SensorStatus.IR3[1]
#define ARM_R_IR_SENSOR_CLAW		g_SensorStatus.IR3[2]
#define ARM_R_IR_SENSOR_ELBOW		g_SensorStatus.IR3[3]


// HW Bumpers.  Pic uses: gStatus_HW_Bumper
#define HW_BUMPER_FRONT_MASK				0x01	// Bit 0 mask
#define HW_BUMPER_REAR_MASK					0x02	// Bit 1 mask
#define HW_BUMPER_SIDE_LEFT_MASK			0x04	// Bit 2 mask
#define HW_BUMPER_SIDE_RIGHT_MASK			0x08	// Bit 3 mask
#define HW_BUMPER_AVAILABLE_10				0x10	// Bit 4 mask
#define HW_BUMPER_RESERVED_20				0x20	// Bit reserved for relay control by PIC
#define HW_BUMPER_RESERVED_40				0x40	// Bit reserved for relay control by PIC
#define HW_BUMPER_RESERVED_80				0x80	// Bit reserved for relay control by PIC

// No longer used for Loki:
//#define FRONT_LEFT_BUMPER_BIT				   6	// Bit 6 - used by bit_set function in TimerISR
//#define FRONT_RIGHT_BUMPER_BIT				   7	// Bit 7


// IR Bumpers.  Pic uses: gStatus_IR_Bumper
#define IR_BUMPER_FRONT_LEFT_MASK			0x01	// Bit 0 mask
#define IR_BUMPER_FRONT_RIGHT_MASK			0x02	// Bit 1 mask
#define IR_BUMPER_CLIFF_LEFT_MASK			0x04	// Bit 2 mask
#define IR_BUMPER_CLIFF_RIGHT_MASK			0x08	// Bit 3 mask
#define IR_BUMPER_REAR_LEFT_MASK			0x10	// Bit 4 mask
#define IR_BUMPER_REAR_RIGHT_MASK			0x20	// Bit 5 mask
#define IR_BUMPER_PIR_LEFT_MASK				0x40	// Bit 6 mask
#define IR_BUMPER_PIR_RIGHT_MASK			0x80	// Bit 7 mask

// Left Arm Bumpers (IR and HW)
//#define ARM_L_BUMPER_NC_MASK				0x01	// Bit 0 mask
#define ARM_L_BUMPER_INSIDE_CLAW_MASK		0x02	// Bit 1 mask
//#define ARM_L_BUMPER_NC_MASK				0x04	// Bit 2 mask
//#define ARM_L_BUMPER_NC_MASK				0x08	// Bit 3 mask
//#define ARM_L_BUMPER_NC_MASK				0x10	// Bit 4 mask
#define ARM_L_HW_BUMPER_ELBOW_MASK			0x20	// Bit 5 mask	- HW Bumper switch on Elbow!
#define ARM_L_IR_BUMPER_FINGER_L_MASK		0x40	// Bit 6 mask
#define ARM_L_IR_BUMPER_FINGER_R_MASK		0x80	// Bit 7 mask

// Right Arm Bumpers (IR and HW)
//#define ARM_R_BUMPER_NC_MASK				0x01	// Bit 0 mask
//#define ARM_R_BUMPER_INSIDE_CLAW_MASK		0x02	// Bit 1 mask
//#define ARM_R_BUMPER_NC_MASK				0x04	// Bit 2 mask
//#define ARM_R_BUMPER_NC_MASK				0x08	// Bit 3 mask
//#define ARM_R_BUMPER_NC_MASK				0x10	// Bit 4 mask
#define ARM_R_HW_BUMPER_ELBOW_MASK			0x20	// Bit 5 mask	- HW Bumper switch on Elbow!
//#define ARM_R_IR_BUMPER_FINGER_L_MASK		0x40	// Bit 6 mask
//#define ARM_R_IR_BUMPER_FINGER_R_MASK		0x80	// Bit 7 mask


// Bumper Macros
#define IR_BUMPER_RANGE						 100	// Tenth_Inches - detection distance for IR bumpers
#define IR_BUMPER_OBJECT_FRONT_LEFT			(  g_SensorStatus.IRBumper & IR_BUMPER_FRONT_LEFT_MASK   )
#define IR_BUMPER_OBJECT_FRONT_RIGHT		(  g_SensorStatus.IRBumper & IR_BUMPER_FRONT_RIGHT_MASK  )
#define IR_BUMPER_CLIFF_LEFT				(!(g_SensorStatus.IRBumper & IR_BUMPER_CLIFF_LEFT_MASK ) )	// Lack of object = cliff
#define IR_BUMPER_CLIFF_RIGHT				(!(g_SensorStatus.IRBumper & IR_BUMPER_CLIFF_RIGHT_MASK ))
#define IR_BUMPER_OBJECT_REAR_LEFT			(  g_SensorStatus.IRBumper & IR_BUMPER_REAR_LEFT_MASK    )
#define IR_BUMPER_OBJECT_REAR_RIGHT			(  g_SensorStatus.IRBumper & IR_BUMPER_REAR_RIGHT_MASK   )
#define PIR_MOTION_DETECTED_LEFT			(!(g_SensorStatus.IRBumper & IR_BUMPER_PIR_LEFT_MASK )   )
#define PIR_MOTION_DETECTED_RIGHT			(!(g_SensorStatus.IRBumper & IR_BUMPER_PIR_RIGHT_MASK )  )

#define ARM_L_HW_BUMPER_OBJECT_ELBOW		(  g_SensorStatus.ArmBumperL & ARM_L_HW_BUMPER_ELBOW_MASK	 )
#define ARM_L_IR_BUMPER_OBJECT_FINGER_L		(  g_SensorStatus.ArmBumperL & ARM_L_IR_BUMPER_FINGER_L_MASK )
#define ARM_L_IR_BUMPER_OBJECT_FINGER_R		(  g_SensorStatus.ArmBumperL & ARM_L_IR_BUMPER_FINGER_R_MASK )
#define ARM_L_IR_BUMPER_INSIDE_CLAW			(  g_SensorStatus.ArmBumperL & ARM_L_BUMPER_INSIDE_CLAW_MASK )

#define ARM_R_HW_BUMPER_OBJECT_ELBOW		(  g_SensorStatus.ArmBumperR & ARM_R_HW_BUMPER_ELBOW_MASK	 )

#define HW_BUMPER_HIT_FRONT					(  g_SensorStatus.HWBumper & HW_BUMPER_FRONT_MASK  )
#define HW_BUMPER_HIT_REAR					(  g_SensorStatus.HWBumper & HW_BUMPER_REAR_MASK   )
#define HW_BUMPER_HIT_SIDE_LEFT				(  g_SensorStatus.HWBumper & HW_BUMPER_SIDE_LEFT_MASK   )
#define HW_BUMPER_HIT_SIDE_RIGHT			(  g_SensorStatus.HWBumper & HW_BUMPER_SIDE_RIGHT_MASK  )


// Analog Sensors used as bumper sensors
#define S_FOREARM_BONE_LEN_TENTH_INCHES_L							147.5	// Tenth_Inches
#define S_FOREARM_BONE_LEN_TENTH_INCHES_R							132.5	// Tenth_Inches

#define ARM_L_IR_BUMPER_OBJECT_ELBOW		(  ARM_L_IR_SENSOR_ELBOW < (S_FOREARM_BONE_LEN_TENTH_INCHES_L+IR_ELBOW_HIT_RANGE_TENTH_INCHES)  ) // I2C-IT Sensor: True if object detected within N Tenth_Inches
#define ARM_R_IR_BUMPER_OBJECT_ELBOW		(  ARM_R_IR_SENSOR_ELBOW < (S_FOREARM_BONE_LEN_TENTH_INCHES_R+IR_ELBOW_HIT_RANGE_TENTH_INCHES)  ) // I2C-IT Sensor: True if object detected within N Tenth_Inches




////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                           TURTLE
////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif SENSOR_CONFIG_TYPE == SENSOR_CONFIG_TURTLE
	// Turtle Sensor Configuration:


	#define OBJECT_EQUAL_DISTANCE		  0
	#define SIDE_SENSOR_ANGLE			 80	// degrees from forward TODO-TURTLE-MUST is this right???
	#define ANGLE_SENSOR_ANGLE			 15 // Angled "IR Bumper" sensors - almost directly forward
	#define FORWARD_SENSOR_ANGLE		 10	// Hmm, use US angle, or IR angle?
	#define REAR_BUMPER_ANGLE			180
	#define REAR_IR_ANGLE				170
	#define VERTICAL_IR_DETECT_RANGE_TENTH_INCHES	 300	// Detect tables, etc that robot might run into


	#define REAR_RIGHT				(-REAR_IR_ANGLE)
	#define SIDE_RIGHT				 SIDE_SENSOR_ANGLE
	#define ANGLE_RIGHT				 ANGLE_SENSOR_ANGLE
	#define FORWARD_RIGHT			 FORWARD_SENSOR_ANGLE
	#define FORWARD_LEFT			(-FORWARD_SENSOR_ANGLE)	// Negative Numbers on Left side
	#define ANGLE_LEFT				(-ANGLE_SENSOR_ANGLE)
	#define SIDE_LEFT				(-SIDE_SENSOR_ANGLE)
	#define REAR_LEFT				(-REAR_IR_ANGLE)


	#define NUMBER_OF_WHEEL_ODOMETERS							  2		// LOKI has a Right and Left Odometer

	// WARNING! If you change these, you must update SensorOffsetDegrees in Globals.cpp!
	#define NUM_IR_SENSORS										  6	// 2 head mounted LR, 2 front facing LR, 2 side mounted SR
	#define NUM_IR3_SENSORS										  4 // 8	// I2C-IT IR Sensors - DEBUG!!!!
	#define NUM_IR2_SENSORS										  8	// I2C Sensors - DEBUG!!!!
	#define NUM_US_SENSORS										  3	// 1 head mounted, 2 front mounted

	// Sensor Field Of View for drawing sensor readings on the Map
	#define SENSOR_HALF_FOV_DEGREES_US							 150	// 1/2 of Sensor Field Of View in tenth degrees
	#define SENSOR_HALF_FOV_DEGREES_IR							 20	// This is actually bigger then sensor, but robot movement provides ambiguity

	// Sensor Positions in arrays
	#define	IR_SENSOR_SIDE_LEFT									  0
	#define	IR_SENSOR_FRONT_LEFT								  1
	#define	IR_SENSOR_HEAD_LEFT									  2
	#define	IR_SENSOR_HEAD_RIGHT								  3
	#define	IR_SENSOR_FRONT_RIGHT								  4
	#define	IR_SENSOR_SIDE_RIGHT								  5

	#define US_SENSOR_CAMERA									  0	// Mounted on Camera
	#define US_SENSOR_FRONT_LEFT								  1
	#define US_SENSOR_FRONT_RIGHT								  2


	#define IR_HEAD_TRACKING_RANGE_TENTH_INCHES					410 // Tenth_Inches - Head will track any object closer then this
	#define IR_TRACKING_FUDGE_TENTH_INCHES						 60 // Tenth_Inches
	#define IR_ELBOW_HIT_RANGE_TENTH_INCHES						 20 // Tenth_Inches - distance from front of claw tip (used by elbow sensor to detect impending arm collisions)	
	#define CLAW_DETECT_TRIGGER_DISTANCE_TENTH_INCHES			 60	// Tenth_Inches

	// Arm Length measurements defined in ArmControl.cpp
	// Arm Sensor measurements
	#define FOREARM_IR_TO_WRIST_OBSTRUCTION_TENTH_INCHES_L		 55.0	// Tenth_Inches - clear space even when wrist rotated
	#define FOREARM_IR_TO_WRIST_OBSTRUCTION_TENTH_INCHES_R		 80.0	// Tenth_Inches - clear space even when wrist rotated
	#define FOREARM_IR_TO_FINGER_TIP_TENTH_INCHES_L				130.0	// Tenth_Inches
	#define FOREARM_IR_TO_FINGER_TIP_TENTH_INCHES_R				102.5	// Tenth_Inches

	#define GRID_MAP_AVOID_OBJECT_DISTANCE						  60	// Tenth_Inches - Avoid objects closer then this distance to the robot


	////////////////////////////////////////////////////////////////
	#ifndef __PIC_TARGET	// Pic Compiler does not understand pack
	#pragma pack( 1 )
	#endif

	// Raw data from the PIC for Loki
	typedef struct
	{
		BYTE StatusFlags;		// 3
		BYTE LastError;			// 4
		BYTE DebugCode;			// 5
		BYTE HWBumper;			// 6
		BYTE IRBumper;			// 7
		BYTE ArmBumperL;		// 8
		BYTE ArmBumperR;		// 9
		BYTE Battery0;			//10
		BYTE CompassHigh;		//11
		BYTE CompassLow;		//12
		BYTE OdometerHighL;		//13
		BYTE OdometerLowL;		//14
		BYTE OdometerHighR;		//15
		BYTE OdometerLowR;		//16
		BYTE IR[NUM_IR_SENSORS];//17-22 	- UP TO N, may be less! - On main Pic board
		BYTE IR3[NUM_IR3_SENSORS];//23-26	- UP TO N, may be less! - I2C-IT IR Rangers
//		BYTE IR2[NUM_IR2_SENSORS];//		- UP TO N, may be less! - On external expansion board
//		BYTE ThermalArray[9];	//			- Ambient + 8 Pixels - Devantech TPA81 Thermal Array Sensor
//		BYTE US[NUM_US_SENSORS];//	

	} PIC_STATUS_T;
	#ifndef __PIC_TARGET
	#pragma pack( 4 )
	#endif

// I2C-IT Sensors
#define ARM_L_IR_SENSOR_CLAW		g_SensorStatus.IR3[0]
#define ARM_L_IR_SENSOR_ELBOW		g_SensorStatus.IR3[1]
#define ARM_R_IR_SENSOR_CLAW		g_SensorStatus.IR3[2]
#define ARM_R_IR_SENSOR_ELBOW		g_SensorStatus.IR3[3]


// HW Bumpers.  Pic uses: gStatus_HW_Bumper
#define HW_BUMPER_FRONT_MASK				0x01	// Bit 0 mask
#define HW_BUMPER_REAR_MASK					0x02	// Bit 1 mask
#define HW_BUMPER_SIDE_LEFT_MASK			0x04	// Bit 2 mask
#define HW_BUMPER_SIDE_RIGHT_MASK			0x08	// Bit 3 mask
#define HW_BUMPER_AVAILABLE_10				0x10	// Bit 4 mask
#define HW_BUMPER_RESERVED_20				0x20	// Bit reserved for relay control by PIC
#define HW_BUMPER_RESERVED_40				0x40	// Bit reserved for relay control by PIC
#define HW_BUMPER_RESERVED_80				0x80	// Bit reserved for relay control by PIC

// No longer used for Loki:
//#define FRONT_LEFT_BUMPER_BIT				   6	// Bit 6 - used by bit_set function in TimerISR
//#define FRONT_RIGHT_BUMPER_BIT				   7	// Bit 7


// IR Bumpers.  Pic uses: gStatus_IR_Bumper
#define IR_BUMPER_FRONT_LEFT_MASK			0x01	// Bit 0 mask
#define IR_BUMPER_FRONT_RIGHT_MASK			0x02	// Bit 1 mask
#define IR_BUMPER_CLIFF_LEFT_MASK			0x04	// Bit 2 mask
#define IR_BUMPER_CLIFF_RIGHT_MASK			0x08	// Bit 3 mask
#define IR_BUMPER_REAR_LEFT_MASK			0x10	// Bit 4 mask
#define IR_BUMPER_REAR_RIGHT_MASK			0x20	// Bit 5 mask
#define IR_BUMPER_PIR_LEFT_MASK				0x40	// Bit 6 mask
#define IR_BUMPER_PIR_RIGHT_MASK			0x80	// Bit 7 mask

// Left Arm Bumpers (IR and HW)
//#define ARM_L_BUMPER_NC_MASK				0x01	// Bit 0 mask
#define ARM_L_BUMPER_INSIDE_CLAW_MASK		0x02	// Bit 1 mask
//#define ARM_L_BUMPER_NC_MASK				0x04	// Bit 2 mask
//#define ARM_L_BUMPER_NC_MASK				0x08	// Bit 3 mask
//#define ARM_L_BUMPER_NC_MASK				0x10	// Bit 4 mask
#define ARM_L_HW_BUMPER_ELBOW_MASK			0x20	// Bit 5 mask	- HW Bumper switch on Elbow!
#define ARM_L_IR_BUMPER_FINGER_L_MASK		0x40	// Bit 6 mask
#define ARM_L_IR_BUMPER_FINGER_R_MASK		0x80	// Bit 7 mask

// Right Arm Bumpers (IR and HW)
//#define ARM_R_BUMPER_NC_MASK				0x01	// Bit 0 mask
//#define ARM_R_BUMPER_INSIDE_CLAW_MASK		0x02	// Bit 1 mask
//#define ARM_R_BUMPER_NC_MASK				0x04	// Bit 2 mask
//#define ARM_R_BUMPER_NC_MASK				0x08	// Bit 3 mask
//#define ARM_R_BUMPER_NC_MASK				0x10	// Bit 4 mask
#define ARM_R_HW_BUMPER_ELBOW_MASK			0x20	// Bit 5 mask	- HW Bumper switch on Elbow!
//#define ARM_R_IR_BUMPER_FINGER_L_MASK		0x40	// Bit 6 mask
//#define ARM_R_IR_BUMPER_FINGER_R_MASK		0x80	// Bit 7 mask


// Bumper Macros
#define IR_BUMPER_RANGE						 100	// Tenth_Inches - detection distance for IR bumpers
#define IR_BUMPER_OBJECT_FRONT_LEFT			(  g_SensorStatus.IRBumper & IR_BUMPER_FRONT_LEFT_MASK   )
#define IR_BUMPER_OBJECT_FRONT_RIGHT		(  g_SensorStatus.IRBumper & IR_BUMPER_FRONT_RIGHT_MASK  )
#define IR_BUMPER_CLIFF_LEFT				(!(g_SensorStatus.IRBumper & IR_BUMPER_CLIFF_LEFT_MASK ) )	// Lack of object = cliff
#define IR_BUMPER_CLIFF_RIGHT				(!(g_SensorStatus.IRBumper & IR_BUMPER_CLIFF_RIGHT_MASK ))
#define IR_BUMPER_OBJECT_REAR_LEFT			(  g_SensorStatus.IRBumper & IR_BUMPER_REAR_LEFT_MASK    )
#define IR_BUMPER_OBJECT_REAR_RIGHT			(  g_SensorStatus.IRBumper & IR_BUMPER_REAR_RIGHT_MASK   )
#define PIR_MOTION_DETECTED_LEFT			(!(g_SensorStatus.IRBumper & IR_BUMPER_PIR_LEFT_MASK )   )
#define PIR_MOTION_DETECTED_RIGHT			(!(g_SensorStatus.IRBumper & IR_BUMPER_PIR_RIGHT_MASK )  )

#define ARM_L_HW_BUMPER_OBJECT_ELBOW		(  g_SensorStatus.ArmBumperL & ARM_L_HW_BUMPER_ELBOW_MASK	 )
#define ARM_L_IR_BUMPER_OBJECT_FINGER_L		(  g_SensorStatus.ArmBumperL & ARM_L_IR_BUMPER_FINGER_L_MASK )
#define ARM_L_IR_BUMPER_OBJECT_FINGER_R		(  g_SensorStatus.ArmBumperL & ARM_L_IR_BUMPER_FINGER_R_MASK )
#define ARM_L_IR_BUMPER_INSIDE_CLAW			(  g_SensorStatus.ArmBumperL & ARM_L_BUMPER_INSIDE_CLAW_MASK )

#define ARM_R_HW_BUMPER_OBJECT_ELBOW		(  g_SensorStatus.ArmBumperR & ARM_R_HW_BUMPER_ELBOW_MASK	 )

#define HW_BUMPER_HIT_FRONT					(  g_SensorStatus.HWBumper & HW_BUMPER_FRONT_MASK  )
#define HW_BUMPER_HIT_REAR					(  g_SensorStatus.HWBumper & HW_BUMPER_REAR_MASK   )
#define HW_BUMPER_HIT_SIDE_LEFT				(  g_SensorStatus.HWBumper & HW_BUMPER_SIDE_LEFT_MASK   )
#define HW_BUMPER_HIT_SIDE_RIGHT			(  g_SensorStatus.HWBumper & HW_BUMPER_SIDE_RIGHT_MASK  )


// Analog Sensors used as bumper sensors
#define S_FOREARM_BONE_LEN_TENTH_INCHES_L							147.5	// Tenth_Inches
#define S_FOREARM_BONE_LEN_TENTH_INCHES_R							132.5	// Tenth_Inches

#define ARM_L_IR_BUMPER_OBJECT_ELBOW		(  ARM_L_IR_SENSOR_ELBOW < (S_FOREARM_BONE_LEN_TENTH_INCHES_L+IR_ELBOW_HIT_RANGE_TENTH_INCHES)  ) // I2C-IT Sensor: True if object detected within N Tenth_Inches
#define ARM_R_IR_BUMPER_OBJECT_ELBOW		(  ARM_R_IR_SENSOR_ELBOW < (S_FOREARM_BONE_LEN_TENTH_INCHES_R+IR_ELBOW_HIT_RANGE_TENTH_INCHES)  ) // I2C-IT Sensor: True if object detected within N Tenth_Inches







////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                             SEEKER CAR-ROBOT
////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif SENSOR_CONFIG_TYPE == SENSOR_CONFIG_CARBOT
	// Seeker CarBot Sensor Configuration:

	// Angle from forward of objects
	#define OBJECT_EQUAL_DISTANCE	  0
	#define SIDE_SENSOR_ANGLE		 30	// degrees from forward TODO-CAR-MUST is this right???
	#define ANGLE_SENSOR_ANGLE		 20
	#define FORWARD_SENSOR_ANGLE	 10

	#define SIDE_RIGHT				 SIDE_SENSOR_ANGLE
	#define ANGLE_RIGHT				 ANGLE_SENSOR_ANGLE
	#define FORWARD_RIGHT			 FORWARD_SENSOR_ANGLE
	#define FORWARD_LEFT			(-FORWARD_SENSOR_ANGLE)	// Negative Numbers on Left side
	#define ANGLE_LEFT				(-ANGLE_SENSOR_ANGLE)
	#define SIDE_LEFT				(-SIDE_SENSOR_ANGLE)

	#define NUMBER_OF_WHEEL_ODOMETERS	1		// Carbot only has one odometer

	// WARNING! If you change these, you must update SensorOffsetDegrees in Globals.cpp!
	#define NUM_IR_SENSORS							  4
	#define NUM_US_SENSORS							  7

	// Sensor Field Of View for drawing sensor readings on the Map
	#define SENSOR_HALF_FOV_DEGREES_US		 15	// 1/2 of Sensor Field Of View in degrees
	#define SENSOR_HALF_FOV_DEGREES_IR		  2	// This is actually bigger then sensor, but robot movement provides ambiguity
	#define SIDE_THREAT_THRESHOLD			 120	// Tenth_Inches - avoid objects closer then this on the side

	// Sensor Positions in arrays.  Both IR and US numbered from left to right
	#define	IR_SENSOR_SIDE_LEFT				  0
	#define	IR_SENSOR_FRONT_LEFT			  1
	#define	IR_SENSOR_FRONT_RIGHT			  2
	#define	IR_SENSOR_SIDE_RIGHT			  3
	//#define	IR_SENSOR_ANGLE_RIGHT			  4 // No IR angle sensors on Carbot
	//#define	IR_SENSOR_SIDE_RIGHT			  5

	// US Sensors are arranged in a semi-circle, and
	// Numbered left to right (1-6)
	#define US_SENSOR_CAMERA				  0	// Mounted on Camera
	#define US_SENSOR_SIDE_LEFT				  1
	#define US_SENSOR_ANGLE_LEFT			  2
	#define US_SENSOR_FRONT_LEFT			  3
	#define US_SENSOR_FRONT_RIGHT			  4
	#define US_SENSOR_ANGLE_RIGHT			  5
	#define US_SENSOR_SIDE_RIGHT			  6

	/*
	#define	RESULT_FAILURE		0xFFFF
	#define	WAYPOINT_IN_RANGE		360	// with in Tenth_Inches, Look for Landmarks
	#define	WAYPOINT_STOP_DISTANCE	120	// Stop if within a foot of expected Waypoint

	#define OBJECT_COLLISION		  40	// Tenth_Inches - (IR Only) Anything closer than this is a potential collision
	#define OBJECT_AVOID_DEFAULT_FEET 4	// Feet! - used to initialize g_GlobalAvoidObjectRangeFeet, which is the actual value used in tests
	#define CAMERA_THRESHOLD_DEFAULT  5	// CrCb +/- Threshold value
	#define CAMERA_SCAN_DIST_DEFAULT 13;	// Feet where camera starts really looking hard for the cone
	*/

	/*
	#define OBJECT_URGENT			 180	// Tenth_Inches - 1.5 ft - Object really close, need to avoid!
	#define OBJECT_DANGER			 360	// Tenth_Inches - 3 ft - Object close, need to avoid
	#define OBJECT_WARNING			 480	// Tenth_Inches - 4 ft - Object getting close, need to avoid or slow down
	#define OBJECT_AHEAD			 600	// Tenth_Inches - 5 ft - Object in range, consider going around it
	#define OBJECT_FAR				 960	// Tenth_Inches - 8 ft - Object barely in range
	#define	IR_HALL_THRESHOLD		 180	// Tenth_Inches - Range of interest in hall way nav.  Greater than 1/2 width of average hall
	*/

	#define IR_RANGE_FUDGE_AMOUNT	 40	// Tenth_Inches - Amount of error in IR readings (for determining distance to objects)
	#define US_RANGE_FUDGE_AMOUNT	 240 // Tenth_Inches - Amount of error in US readings (for determining distance to objects)


	#define IR_SR_MAX_RANGE			 240 // Tenth_Inches - Max range of Short Range IR sensors
	#define IR_SR_DETECT_RANGE		 160 // Tenth_Inches - 4ft Max reliable range of Long range IR sensors for Object Finding
	#define IR_LR_MAX_RANGE			 600 // Tenth_Inches - 5ft Max range of Long Range IR sensors
	#define IR_LR_DETECT_RANGE		 480 // Tenth_Inches - 4ft Max reliable range of Long range IR sensors for Object Finding
	#define US_MAX_RANGE			 960 // Tenth_Inches - 8ft Max range of US sensors
	#define US_MAX_DETECT_RANGE		 960 // TODO Tenth_Inches - 8ft Max range of US sensors

	#define GRID_MAP_AVOID_OBJECT_DISTANCE	60	// Tenth_Inches - Avoid objects closer then this distance to the robot


	////////////////////////////////////////////////////////////////
	#ifndef __PIC_TARGET	// Pic Compiler does not understand pack
	#pragma pack( 1 )
	#endif

	// Raw data from the PIC for CARBOT
	typedef struct
	{
	BYTE StatusFlags;		// 3
	BYTE LastError;			// 4
	BYTE DebugCode;			// 5
	BYTE Bumper;			// 6
//	BYTE AckMotorCommand;	// xx
//	BYTE AckMotorSpeed;		// xx
//	BYTE AckMotorTurn;		// xx
	BYTE Battery0;			// 7
	BYTE Battery1;			// 8
	BYTE CompassHigh;		// 9
	BYTE CompassLow;		//10
	BYTE OdometerHigh;		//11
	BYTE OdometerLow;		//12
	BYTE Tachometer;		//13
	BYTE US[NUM_US_SENSORS];//14-20
	BYTE IR[NUM_IR_SENSORS];//21-26	- UP TO 6, may be less!
	BYTE AccelX;			//27
	BYTE AccelY;			//28
	} PIC_STATUS_T;

	#ifndef __PIC_TARGET
	#pragma pack( 4 )
	#endif


// TODO-MUST-CARBOT - FIX THESE TO MATCH REAL HARDWARE!!!!
#define HW_BUMPER_NOT_USED_MASK1			0x01	// Bit 4 mask
#define HW_BUMPER_NOT_USED_MASK2			0x02	// Bit 5 mask
#define HW_BUMPER_NOT_USED_MASK3			0x04	// Bit 6 mask
#define HW_BUMPER_REAR_MASK					0x08	// Bit 7 mask
#define HW_BUMPER_SIDE_LEFT_MASK			0x10	// Bit 2 mask
#define HW_BUMPER_SIDE_RIGHT_MASK			0x20	// Bit 3 mask
#define HW_BUMPER_FRONT_LEFT_MASK			0x40	// Bit 6 mask	// Set in TimerISR
#define HW_BUMPER_FRONT_RIGHT_MASK			0x80	// Bit 7 mask	// Set in TimerISR

// Used by PIC only:
#define FRONT_LEFT_BUMPER_BIT				   6	// Bit 6 - used by bit_set function in TimerISR
#define FRONT_RIGHT_BUMPER_BIT				   7	// Bit 7


// IR Bumpers.  Pic uses: gStatus_IR_Bumper
#define IR_BUMPER_FRONT_LEFT_MASK			0x01	// Bit 0 mask
#define IR_BUMPER_FRONT_RIGHT_MASK			0x02	// Bit 1 mask
#define IR_BUMPER_CLIFF_LEFT_MASK			0x04	// Bit 2 mask
#define IR_BUMPER_CLIFF_RIGHT_MASK			0x08	// Bit 3 mask
#define IR_BUMPER_REAR_LEFT_MASK			0x10	// Bit 4 mask
#define IR_BUMPER_REAR_RIGHT_MASK			0x20	// Bit 5 mask
#define IR_BUMPER_NOT_USED_MASK3			0x40	// Bit 6 mask
#define IR_BUMPER_NOT_USED_MASK4			0x80	// Bit 7 mask


// Bumper Macros
#define IR_BUMPER_OBJECT_FRONT_LEFT		(g_SensorStatus.IRBumper & IR_BUMPER_FRONT_LEFT_MASK)
#define IR_BUMPER_OBJECT_FRONT_RIGHT	(g_SensorStatus.IRBumper & IR_BUMPER_FRONT_RIGHT_MASK)
#define IR_BUMPER_CLIFF_LEFT			(!(g_SensorStatus.IRBumper & IR_BUMPER_CLIFF_LEFT_MASK))	// Lack of object = cliff
#define IR_BUMPER_CLIFF_RIGHT			(!(g_SensorStatus.IRBumper & IR_BUMPER_CLIFF_RIGHT_MASK))
#define IR_BUMPER_OBJECT_REAR_LEFT		(g_SensorStatus.IRBumper & IR_BUMPER_REAR_LEFT_MASK)
#define IR_BUMPER_OBJECT_REAR_RIGHT		(g_SensorStatus.IRBumper & IR_BUMPER_REAR_RIGHT_MASK)

#define HW_BUMPER_HIT_FRONT_LEFT		(g_SensorStatus.HWBumper & HW_BUMPER_FRONT_LEFT_MASK)
#define HW_BUMPER_HIT_FRONT_RIGHT		(g_SensorStatus.HWBumper & HW_BUMPER_FRONT_RIGHT_MASK)
#define HW_BUMPER_HIT_SIDE_LEFT			(g_SensorStatus.HWBumper & HW_BUMPER_SIDE_LEFT_MASK)
#define HW_BUMPER_HIT_SIDE_RIGHT		(g_SensorStatus.HWBumper & HW_BUMPER_SIDE_RIGHT_MASK)
#define HW_BUMPER_HIT_REAR				(g_SensorStatus.HWBumper & HW_BUMPER_REAR_MASK)

#define HW_BUMPER_HIT_FRONT				(HW_BUMPER_HIT_FRONT_RIGHT || HW_BUMPER_HIT_FRONT_LEFT)



////////////////////////////////////////////////////////////////
#else
	#error BAD SENSOR_CONFIG_TYPE!
#endif



////////////////////////////////////////////////////////////////
// Packet Structure for Comands to PIC
#ifndef __PIC_TARGET	// Pic Compiler does not understand pack
#pragma pack( 1 )
#endif

typedef struct
{
	//BYTE Sync0;
	//BYTE Sync1;
	BYTE Cmd;
	BYTE Param1;
	BYTE Param2;
	BYTE Param3;
	BYTE Param4;
	//BYTE TermChar;
} PIC_CMD_T;
#define PIC_CMD_SIZE 5	// 5 bytes

#ifndef __PIC_TARGET
#pragma pack( 4 )
#endif



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Generic Servo Constants
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#if( SERVO_CONTROL_TYPE == PIC_SERVO_CONTROL )

	#define SERVO_OFF					  0
	#define SERVO_CENTER				 63
	#define SERVO_MAX					127		// Servo values: 0=off, 1=bottom, 64=center 128=top
	#define SERVO_MIN					  1
	#define SERVO_MODE_180				  0		// Not used for PIC

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( SERVO_CONTROL_TYPE == EXTERN_SERVO_CONTROL )
// Use this if controlling servos from the Pololu Servo Controller

	#define SERVO_OFF					  0
	#define SERVO_CENTER				  0
	#define SERVO_MAX				    127
	#define SERVO_MIN				   -127
	#define SERVO_MODE_180				  8 // +8 Puts servo in 180 degree travel mode!

#elif( SERVO_CONTROL_TYPE == DYNA_SERVO_CONTROL )

	#define SERVO_OFF					  0
	#define SERVO_CENTER				  0
	#define SERVO_MAX					127		// Servo values: 0=off, 1=bottom, 64=center 128=top
	#define SERVO_MIN				   -127
	#define SERVO_MODE_180				  0		// Not used for DYNA

#else
	#error BAD SERVO_CONTROL_TYPE!
#endif



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Motor Control
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#if( MOTOR_CONTROL_TYPE == ER1_MOTOR_CONTROL )

	#define TICKS_PER_TENTH_INCH			    429.50	// MrRoboto 2 ER1 Maybe 409.6?

	// Turn Constants
	#define TURN_SERVO_CENTER			  0		// TRIM: + = LEFT - Use Only just before sending to the PIC!
	#define TURN_CENTER				 	  0
	#define TURN_LEFT_SLOW			   (-5)
	#define TURN_LEFT_MED_SLOW		   (-10)
	#define TURN_LEFT_MED			   (-20)
	#define TURN_LEFT_MED_FAST		   (-40)
	#define TURN_LEFT_FAST			  (-127)
	#define TURN_LEFT_MAX			  (-127)	// For LIMIT CHECKS

	#define TURN_RIGHT_SLOW			      5
	#define TURN_RIGHT_MED_SLOW		     10
	#define TURN_RIGHT_MED			     20
	#define TURN_RIGHT_MED_FAST		     40
	#define TURN_RIGHT_FAST			    127
	#define TURN_RIGHT_MAX			    127 	// For LIMIT CHECKS


/* OLD --------------
	#define TURN_5_DEG_LEFT			   (-20)
	#define TURN_10_DEG_LEFT		   (-40)
	#define TURN_20_DEG_LEFT		   (-60)
	TURN_30_DEG_LEFT
	#define TURN_FULL_LEFT			  (-120)	// 25 Degrees = max turn
	#define TURN_MAX_LEFT			  (-125)	// For LIMIT CHECKS

	#define TURN_5_DEG_RIGHT			 10
	#define TURN_10_DEG_RIGHT			 20
	#define TURN_20_DEG_RIGHT		  	 30
	#define TURN_30_DEG_RIGHT		  	 50
	#define TURN_FULL_RIGHT				127
	#define TURN_MAX_RIGHT				127		// For LIMIT CHECKS
	#define TURN_MAX_ABS				127		// Unsigned numberic limit for turns
OLD -----------------
*/


	// Turn constants.  WARNING! if you swap Left/Right, look for limit checks in code!!!
	// Absolute turn Servo values.  Don't use unless really needed!
	//#define TURN_SERVO_MAX_LEFT				(TURN_SERVO_CENTER + TURN_MAX_LEFT)
	//#define TURN_SERVO_MAX_RIGHT			(TURN_SERVO_CENTER + TURN_MAX_RIGHT)

	// Speed Constants
	#define SPEED_STOP				 	  0		// 60
	#define SPEED_FULL_FWD				127		// WARNING! if you swap these, update TempSpeedCmd in Module.cpp!!!
	#define SPEED_FULL_REV			   -127

		// ER1 TODO!  Set these to correct values!
	#define SPEED_SERVO_CENTER			  0		// Same as SPEED_STOP
	#define SPEED_FWD_SLOW				 20
	#define SPEED_FWD_MED_SLOW			 40
	#define SPEED_FWD_MED				 60
	#define SPEED_FWD_MED_FAST			 80
	#define SPEED_FWD_FAST				127

	#define SPEED_REV_SLOW			   (-20)
	#define SPEED_REV_MED_SLOW		   (-40)
	#define SPEED_REV_MED			   (-60)
	#define SPEED_REV_MED_FAST		   (-80)
	#define SPEED_REV_FAST			  (-127)

	#define LOW_GEAR				     35		// Absolute servo value (no need for dist from zero/center)
	#define HIGH_GEAR					 91

	#define TURN_MULTIPLIER				2.0		// Amount of wheel turn to apply to get back on heading
	#define RADAR_TURN_MULTIPLIER		  4		// Amount of wheel to apply to head for Radar Object
	#define TURN_ROTATION_COMPLETE_COMPENSATION 2.0	// degrees we assume the robot will wiggle after completing set turn amount

	// Braking and Automatic Speed Control
	// TODO - Are these correct for Mr Roboto2?
	#define SPEED_BRAKE_AMOUNT			 (-20)	// 8 if not using tachometer to know when wheel stopped
	#define SPEED_BRAKE_TIME			   30	// (* 100ms) Max amount of time to apply brake
	#define MOVE_DISTANCE_COASTING_COMPENSATION 20	// Tenth_Inches we assume the robot will coast after completing move distance
	#define MOVE_DISTANCE_COASTING_TIME		0	// 1/10 sec Delays to allow robot to coast after completing move distance


	#define NEW_SPEED_SET_TIME				 16	// number of 100ms intervals to wait for motor to come to speed after new command
	#define SPEED_CHANGE_TIME				  2	// number of 100ms intervals to wait for motor after speed correction
	#define MAX_SPEED_CORRECTION			  2	// max speed servo adjustment allowed

	#define MOVE_DISTANCE_SHORT				  40	// Tenth_Inches
	#define MOVE_DISTANCE_MED_SHORT			  60	// Tenth_Inches
	#define MOVE_DISTANCE_MED				 120	// Tenth_Inches
	#define MOVE_DISTANCE_LONG				 240	// Tenth_Inches
	#define MOVE_DISTANCE_EXTRA				  10 // extra distance to move while turning, so we don't run into the same object!


	#define SEGMENT_FOLLOW_DISTANCE_FUDGE	  60	//Tenth_Inches - tolerance when tracking wall/curb
	#define SEGMENT_FOLLOW_TURN_AMOUNT		  10	// amount of turn servo to apply to track wall/curb

	#define	WAYPOINT_IN_RANGE				 360	// within x Tenth_Inches, Look for Landmarks
	#define	WAYPOINT_STOP_DISTANCE			 120	// Stop if within a foot of expected Waypoint
	#define OBJECT_COLLISION				  10 // Tenth_Inches - (IR Only) Anything closer than this is a potential collision
	#define OBJECT_AVOID_DEFAULT_FEET		  2	// Feet! - used to initialize g_GlobalAvoidObjectRangeFeet, which is the actual value used in tests
	#define CAMERA_SCAN_DIST_DEFAULT		 13	// Feet where camera starts really looking hard for the cone
	#define CAMERA_TO_WHEEL_SCALE			  2	// multiplier between Camera Servo ticks and Wheel Servo ticks - more = more oversteer


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL )
	// Use this if controlling servos from the Pololu Servo Controller

	#define TICKS_PER_TENTH_INCH	   6.35	// Seeker CarBot: about 1.5 inchs per tick

	// Turn Constants
	#define TURN_SERVO_CENTER			127		// TRIM: + = LEFT - Use Only just before sending to the Servo Control!
	#define TURN_CENTER				 	  0

	#define TURN_LEFT_SLOW			   (-20)
	#define TURN_LEFT_MED_SLOW		   (-40)
	#define TURN_LEFT_MED			   (-60)
	#define TURN_LEFT_MED_FAST		   (-80)
	#define TURN_LEFT_FAST			  (-120)	// 25 Degrees = max turn
	#define TURN_LEFT_MAX			  (-125)	// For LIMIT CHECKS

	#define TURN_RIGHT_SLOW			     20
	#define TURN_RIGHT_MED_SLOW		     40
	#define TURN_RIGHT_MED			     60
	#define TURN_RIGHT_MED_FAST		     80
	#define TURN_RIGHT_FAST			    120 	// 25 Degrees = max turn
	#define TURN_RIGHT_MAX			    125 	// For LIMIT CHECKS


	// Turn constants.  WARNING! if you swap Left/Right, look for limit checks in code!!!
	// Absolute turn Servo values.  Don't use unless really needed!
	//#define TURN_SERVO_MAX_LEFT				(TURN_SERVO_CENTER + TURN_MAX_LEFT)
	//#define TURN_SERVO_MAX_RIGHT			(TURN_SERVO_CENTER + TURN_MAX_RIGHT)

	// Logical Speed constants
	#define SPEED_STOP				 	  0		//
	#define SPEED_FULL_FWD				125		// WARNING! if you swap these, update TempSpeedCmd in Module.cpp!!!
	#define SPEED_FULL_REV			  (-125)

	#define SPEED_FWD_SLOW				 13		// Can be below minimum!
	#define SPEED_FWD_MED_SLOW			 14		// 14 is Minimum that the NOVAK speed control will obay
	#define SPEED_FWD_MED				 16
	#define SPEED_FWD_MED_FAST			 40
	#define SPEED_FWD_FAST				100

	#define SPEED_REV_SLOW			   (-16)		// -16 is not reliable!
	#define SPEED_REV_MED_SLOW		   (-17)		// -17 is Minimum that the NOVAK speed control will obay
	#define SPEED_REV_MED			   (-20)
	#define SPEED_REV_MED_FAST		   (-30)
	#define SPEED_REV_FAST			  (-80)

	// Servo Speed Constants (actual servo settings)
	#define SPEED_SERVO_CENTER			105		// TRIM: + = ? - Use Only just before sending to the PIC!
	#define LOW_GEAR				    22
	#define HIGH_GEAR					250


	#define TURN_MULTIPLIER					0.5		// Amount of wheel turn to apply to get back on heading
	#define RADAR_TURN_MULTIPLIER			  4		// Amount of wheel to apply to head for Radar Object
	#define TURN_ROTATION_COMPLETE_COMPENSATION 0	// degrees we assume the robot will wiggle after completing set turn amount


	// Braking and Automatic Speed Control
	// TODO-CAR - Check that these values are correct for Carbot
	#define SPEED_BRAKE_AMOUNT			 (-20)	// 8 if not using tachometer to know when wheel stopped
	#define SPEED_BRAKE_TIME			   30	// (* 100ms) Max amount of time to apply brake
	#define MOVE_DISTANCE_COASTING_COMPENSATION 60	// Tenth_Inches we assume the robot will coast after completing move distance
	#define MOVE_DISTANCE_COASTING_TIME		0	// 1/10 sec Delays to allow robot to coast after completing move distance

	#define NEW_SPEED_SET_TIME			    16	// number of 100ms intervals to wait for motor to come to speed after new command
	#define SPEED_CHANGE_TIME			     2	// number of 100ms intervals to wait for motor after speed correction
	#define MAX_SPEED_CORRECTION		     2	// max speed servo adjustment allowed


	#define MOVE_DISTANCE_SHORT				 180	// Tenth_Inches
	#define MOVE_DISTANCE_MED_SHORT			 240	// Tenth_Inches
	#define MOVE_DISTANCE_MED				 360	// Tenth_Inches
	#define MOVE_DISTANCE_LONG				 480	// Tenth_Inches

	#define SEGMENT_FOLLOW_DISTANCE_FUDGE	  60	//Tenth_Inches - tolerance when tracking wall/curb
	#define SEGMENT_FOLLOW_TURN_AMOUNT		 10	// amount of turn servo to apply to track wall/curb

	#define	WAYPOINT_IN_RANGE				 360	// within x Tenth_Inches, Look for Landmarks
	#define	WAYPOINT_STOP_DISTANCE			 120	// Stop if within a foot of expected Waypoint
	#define OBJECT_COLLISION				  40 // Tenth_Inches - (IR Only) Anything closer than this is a potential collision
	#define OBJECT_AVOID_DEFAULT_FEET		  4	// Feet! - used to initialize g_GlobalAvoidObjectRangeFeet, which is the actual value used in tests
	#define CAMERA_SCAN_DIST_DEFAULT		 13	// Feet where camera starts really looking hard for the cone
	#define CAMERA_TO_WHEEL_SCALE			  2	// multiplier between Camera Servo ticks and Wheel Servo ticks - more = more oversteer


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( MOTOR_CONTROL_TYPE == POLOLU_TREX_MOTOR_CONTROL )
	// Use this if controlling the motors from the Pololu TReX Motor Controller

// TODO-TREX - Need to set all these values to correct numbers!!!

	#define TICKS_PER_TENTH_INCH		   3.14	// TODO-TREX Seeker (fix Simulator OdometerTickCount after adjusting this value!)

	// Turn Constants
	#define TURN_SERVO_CENTER			  0		// TRIM: + = LEFT - Use Only just before sending to the PIC!
	#define TURN_CENTER				 	  0
	#define TURN_LEFT_VERY_SLOW		   (-5)	// Only use if moving, otherwise will stall
	#define TURN_LEFT_SLOW			   (-15)
	#define TURN_LEFT_MED_SLOW		   (-25)
	#define TURN_LEFT_MED			   (-40)
	#define TURN_LEFT_MED_FAST		   (-60)
	#define TURN_LEFT_FAST			  (-127)
	#define TURN_LEFT_MAX			  (-127)	// For LIMIT CHECKS

	#define TURN_RIGHT_VERY_SLOW	      5	// Only use if moving, otherwise will stall
	#define TURN_RIGHT_SLOW			     15
	#define TURN_RIGHT_MED_SLOW		     25
	#define TURN_RIGHT_MED			     40
	#define TURN_RIGHT_MED_FAST		     60
	#define TURN_RIGHT_FAST			    127
	#define TURN_RIGHT_MAX			    127 	// For LIMIT CHECKS

	// Speed Constants
	#define SPEED_STOP				 	  0		// 60
	#define SPEED_FULL_FWD				127		// WARNING! if you swap these, update TempSpeedCmd in Module.cpp!!!
	#define SPEED_FULL_REV			   -127

	#define SPEED_SERVO_CENTER			  0		// Same as SPEED_STOP	
	#define SPEED_FWD_VERY_SLOW			 15		// Almost stop, but no jerking (used for object avoidance)
	#define SPEED_FWD_SLOW				 25
	#define SPEED_FWD_MED_SLOW			 40
	#define SPEED_FWD_MED				 60
	#define SPEED_FWD_MED_FAST			 80
	#define SPEED_FWD_FAST				127

	#define SPEED_REV_SLOW			   (-25)
	#define SPEED_REV_MED_SLOW		   (-40)
	#define SPEED_REV_MED			   (-60)
	#define SPEED_REV_MED_FAST		   (-80)
	#define SPEED_REV_FAST			  (-127)

	#define LOW_GEAR				     35		// Absolute servo value (no need for dist from zero/center)
	#define HIGH_GEAR					 91

	#define TURN_MULTIPLIER				2.0		// Amount of wheel turn to apply to get back on heading
	#define RADAR_TURN_MULTIPLIER		  4		// Amount of wheel to apply to head for Radar Object
	#define TURN_ROTATION_COMPLETE_COMPENSATION 2.0	// degrees we assume the robot will wiggle after completing set turn amount

	// Braking and Automatic Speed Control
	// TODO - Are these correct for Mr Roboto2?
	#define SPEED_BRAKE_AMOUNT			 (-20)	// 8 if not using tachometer to know when wheel stopped
	#define SPEED_BRAKE_TIME			   30	// (* 100ms) Max amount of time to apply brake
	#define MOVE_DISTANCE_COASTING_COMPENSATION 20	// Tenth_Inches we assume the robot will coast after completing move distance
	#define MOVE_DISTANCE_COASTING_TIME		0	// 1/10 sec Delays to allow robot to coast after completing move distance


	#define NEW_SPEED_SET_TIME				 16	// number of 100ms intervals to wait for motor to come to speed after new command
	#define SPEED_CHANGE_TIME				  2	// number of 100ms intervals to wait for motor after speed correction
	#define MAX_SPEED_CORRECTION			  2	// max speed servo adjustment allowed

	#define MOVE_DISTANCE_SHORT				  40	// Tenth_Inches
	#define MOVE_DISTANCE_MED_SHORT			  60	// Tenth_Inches
	#define MOVE_DISTANCE_MED				 120	// Tenth_Inches
	#define MOVE_DISTANCE_LONG				 240	// Tenth_Inches
	#define MOVE_DISTANCE_EXTRA				  10 // extra distance to move while turning, so we don't run into the same object!


	#define SEGMENT_FOLLOW_DISTANCE_FUDGE	  60	//Tenth_Inches - tolerance when tracking wall/curb
	#define SEGMENT_FOLLOW_TURN_AMOUNT		 10	// amount of turn servo to apply to track wall/curb

	#define	WAYPOINT_IN_RANGE				 360	// within x Tenth_Inches, Look for Landmarks
	#define	WAYPOINT_STOP_DISTANCE			 120	// Stop if within a foot of expected Waypoint
	#define OBJECT_COLLISION				  10 // Tenth_Inches - (IR Only) Anything closer than this is a potential collision
	#define OBJECT_AVOID_DEFAULT_FEET		  2	// Feet! - used to initialize g_GlobalAvoidObjectRangeFeet, which is the actual value used in tests
	#define CAMERA_SCAN_DIST_DEFAULT		 13	// Feet where camera starts really looking hard for the cone
	#define CAMERA_TO_WHEEL_SCALE			  2	// multiplier between Camera Servo ticks and Wheel Servo ticks - more = more oversteer


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( MOTOR_CONTROL_TYPE == IROBOT_MOTOR_CONTROL )
	// Use this if the motors are in the iRobot Create base

// TODO-TURTLE - Need to set all these values to correct numbers!!!

	#define TICKS_PER_TENTH_INCH		   3.14	// TODO-TURTLE  (fix Simulator OdometerTickCount after adjusting this value!)

	// Turn Constants
	#define TURN_SERVO_CENTER			  0		// TRIM: + = LEFT - Use Only just before sending to the PIC!
	#define TURN_CENTER				 	  0
	#define TURN_LEFT_VERY_SLOW		   (-5)	// Only use if moving, otherwise will stall
	#define TURN_LEFT_SLOW			   (-15)
	#define TURN_LEFT_MED_SLOW		   (-25)
	#define TURN_LEFT_MED			   (-40)
	#define TURN_LEFT_MED_FAST		   (-60)
	#define TURN_LEFT_FAST			  (-127)
	#define TURN_LEFT_MAX			  (-127)	// For LIMIT CHECKS

	#define TURN_RIGHT_VERY_SLOW	      5	// Only use if moving, otherwise will stall
	#define TURN_RIGHT_SLOW			     15
	#define TURN_RIGHT_MED_SLOW		     25
	#define TURN_RIGHT_MED			     40
	#define TURN_RIGHT_MED_FAST		     60
	#define TURN_RIGHT_FAST			    127
	#define TURN_RIGHT_MAX			    127 	// For LIMIT CHECKS

	// Speed Constants
	#define SPEED_STOP				 	  0		// 60
	#define SPEED_FULL_FWD				127		// WARNING! if you swap these, update TempSpeedCmd in Module.cpp!!!
	#define SPEED_FULL_REV			   -127

	#define SPEED_SERVO_CENTER			  0		// Same as SPEED_STOP	
	#define SPEED_FWD_VERY_SLOW			 15		// Almost stop, but no jerking (used for object avoidance)
	#define SPEED_FWD_SLOW				 25
	#define SPEED_FWD_MED_SLOW			 40
	#define SPEED_FWD_MED				 60
	#define SPEED_FWD_MED_FAST			 80
	#define SPEED_FWD_FAST				127

	#define SPEED_REV_SLOW			   (-25)
	#define SPEED_REV_MED_SLOW		   (-40)
	#define SPEED_REV_MED			   (-60)
	#define SPEED_REV_MED_FAST		   (-80)
	#define SPEED_REV_FAST			  (-127)

	#define LOW_GEAR				     35		// Absolute servo value (no need for dist from zero/center)
	#define HIGH_GEAR					 91

	#define TURN_MULTIPLIER				2.0		// Amount of wheel turn to apply to get back on heading
	#define RADAR_TURN_MULTIPLIER		  4		// Amount of wheel to apply to head for Radar Object
	#define TURN_ROTATION_COMPLETE_COMPENSATION 2.0	// degrees we assume the robot will wiggle after completing set turn amount

	// Braking and Automatic Speed Control
	// TODO - Are these correct?
	#define SPEED_BRAKE_AMOUNT			 (-20)	// 8 if not using tachometer to know when wheel stopped
	#define SPEED_BRAKE_TIME			   30	// (* 100ms) Max amount of time to apply brake
	#define MOVE_DISTANCE_COASTING_COMPENSATION 20	// Tenth_Inches we assume the robot will coast after completing move distance
	#define MOVE_DISTANCE_COASTING_TIME		0	// 1/10 sec Delays to allow robot to coast after completing move distance


	#define NEW_SPEED_SET_TIME				 16	// number of 100ms intervals to wait for motor to come to speed after new command
	#define SPEED_CHANGE_TIME				  2	// number of 100ms intervals to wait for motor after speed correction
	#define MAX_SPEED_CORRECTION			  2	// max speed servo adjustment allowed

	#define MOVE_DISTANCE_SHORT				  40	// Tenth_Inches
	#define MOVE_DISTANCE_MED_SHORT			  60	// Tenth_Inches
	#define MOVE_DISTANCE_MED				 120	// Tenth_Inches
	#define MOVE_DISTANCE_LONG				 240	// Tenth_Inches
	#define MOVE_DISTANCE_EXTRA				  10 // extra distance to move while turning, so we don't run into the same object!


	#define SEGMENT_FOLLOW_DISTANCE_FUDGE	  60	//Tenth_Inches - tolerance when tracking wall/curb
	#define SEGMENT_FOLLOW_TURN_AMOUNT		 10	// amount of turn servo to apply to track wall/curb

	#define	WAYPOINT_IN_RANGE				 360	// within x Tenth_Inches, Look for Landmarks
	#define	WAYPOINT_STOP_DISTANCE			 120	// Stop if within a foot of expected Waypoint
	#define OBJECT_COLLISION				  10 // Tenth_Inches - (IR Only) Anything closer than this is a potential collision
	#define OBJECT_AVOID_DEFAULT_FEET		  2	// Feet! - used to initialize g_GlobalAvoidObjectRangeFeet, which is the actual value used in tests
	#define CAMERA_SCAN_DIST_DEFAULT		 13	// Feet where camera starts really looking hard for the cone
	#define CAMERA_TO_WHEEL_SCALE			  2	// multiplier between Camera Servo ticks and Wheel Servo ticks - more = more oversteer


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL )

	#define TICKS_PER_TENTH_INCH		  6.35	// Seeker CarBot: about 1.5 inchs per tick

	// Turn Constants
	#define TURN_SERVO_CENTER			 61		// TRIM: + = LEFT - Use Only just before sending to the PIC!
	#define TURN_CENTER				 	  0

	#define TURN_LEFT_SLOW			    (-5)
	#define TURN_LEFT_MED_SLOW		   (-10)
	#define TURN_LEFT_MED			   (-20)
	#define TURN_LEFT_MED_FAST		   (-25)
	#define TURN_LEFT_FAST			   (-30)	// 25 Degrees = max turn
	#define TURN_LEFT_MAX			   (-32)	// For LIMIT CHECKS

	#define TURN_RIGHT_SLOW			      5
	#define TURN_RIGHT_MED_SLOW		     10
	#define TURN_RIGHT_MED			     20
	#define TURN_RIGHT_MED_FAST		     25
	#define TURN_RIGHT_FAST			     30	// 25 Degrees = max turn
	#define TURN_RIGHT_MAX			     32	// For LIMIT CHECKS


	// Turn constants.  WARNING! if you swap Left/Right, look for limit checks in code!!!
	// Absolute turn Servo values.  Don't use unless really needed!
	//#define TURN_SERVO_MAX_LEFT				(TURN_SERVO_CENTER + TURN_MAX_LEFT)
	//#define TURN_SERVO_MAX_RIGHT			(TURN_SERVO_CENTER + TURN_MAX_RIGHT)

	// Speed Constants
	#define SPEED_STOP				 	 60		//
	#define SPEED_FULL_FWD				 95		// WARNING! if you swap these, update TempSpeedCmd in Module.cpp!!!
	#define SPEED_FULL_REV				 31

	#define SPEED_SERVO_CENTER			 60		// Same as SPEED_STOP
	#define SPEED_FWD_SLOW				  3		// Can be below minimum!
	#define SPEED_FWD_MED_SLOW			  4		// 4 is Minimum that the NOVAK speed control will obay
	#define SPEED_FWD_MED				  5
	#define SPEED_FWD_MED_FAST			  8
	#define SPEED_FWD_FAST				 30

	#define SPEED_REV_SLOW			   (-3)		// -4 is not reliable!
	#define SPEED_REV_MED_SLOW		   (-4)		// -5 is Minimum that the NOVAK speed control will obay
	#define SPEED_REV_MED			   (-5)
	#define SPEED_REV_MED_FAST		   (-8)
	#define SPEED_REV_FAST			  (-20)

	#define LOW_GEAR				     35		// Absolute servo value (no need for dist from zero/center)
	#define HIGH_GEAR					 91

	#define TURN_MULTIPLIER					0.5		// Amount of wheel turn to apply to get back on heading
	#define RADAR_TURN_MULTIPLIER			  4		// Amount of wheel to apply to head for Radar Object

	// Braking and Automatic Speed Control
	// May not be correct for PIC, particularly BRAKE_TIME, which is in 20ms ticks for PIC.
	// Not used anyway for Carbot, which uses an external servo controller.
	#define SPEED_BRAKE_AMOUNT			 (-20)	// 8 if not using tachometer to know when wheel stopped
	#define SPEED_BRAKE_TIME			   30	// (* 100ms) Max amount of time to apply brake
	#define MOVE_DISTANCE_COASTING_COMPENSATION 60	// Tenth_Inches we assume the robot will coast after completing move distance
	#define MOVE_DISTANCE_COASTING_TIME	   10	// 100ms Delays to allow robot to coast after completing move distance

	#define NEW_SPEED_SET_TIME			   16	// number of 100ms intervals to wait for motor to come to speed after new command
	#define SPEED_CHANGE_TIME			    2	// number of 100ms intervals to wait for motor after speed correction
	#define MAX_SPEED_CORRECTION		    2	// max speed servo adjustment allowed

	#define MOVE_DISTANCE_SHORT			  180		// Tenth_Inches
	#define MOVE_DISTANCE_MED_SHORT		  240		// Tenth_Inches
	#define MOVE_DISTANCE_MED			  360		// Tenth_Inches
	#define MOVE_DISTANCE_LONG			  480		// Tenth_Inches

	#define SEGMENT_FOLLOW_DISTANCE_FUDGE	60	//Tenth_Inches - tolerance when tracking wall/curb
	#define SEGMENT_FOLLOW_TURN_AMOUNT		10	// amount of turn servo to apply to track wall/curb

	#define	WAYPOINT_IN_RANGE			360	// within x Tenth_Inches, Look for Landmarks
	#define	WAYPOINT_STOP_DISTANCE		120	// Stop if within a foot of expected Waypoint
	#define OBJECT_COLLISION			  40 // Tenth_Inches - (IR Only) Anything closer than this is a potential collision
	#define OBJECT_AVOID_DEFAULT_FEET	  4	// Feet! - used to initialize g_GlobalAvoidObjectRangeFeet, which is the actual value used in tests
	#define CAMERA_SCAN_DIST_DEFAULT	 13	// Feet where camera starts really looking hard for the cone
	#define CAMERA_TO_WHEEL_SCALE		  2	// multiplier between Camera Servo ticks and Wheel Servo ticks - more = more oversteer


#else
	#error BAD MOTOR_CONTROL_TYPE!
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CAMERA CONTROL TYPE
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#if( ROBOT_TYPE == CARBOT )
	#if( CAMERA_CONTROL_TYPE == SONY_SERIAL_CAMERA )

		// Camera motion range in Tenth Degrees
		// Sony camera can do 0 to 300 degrees (or +/- 149 degrees to be safe) max
		#define CAMERA_PAN_CENTER					0		// TenthDegrees from center
		#define CAMERA_TILT_CENTER					0		// TenthDegrees from center

		#define CAMERA_PAN_TENTHDEGREES_MAX_RIGHT	( 89 * 10)	// +89 degrees
		#define CAMERA_PAN_TENTHDEGREES_MAX_LEFT	(-89 * 10)	// -89 degrees
		#define CAMERA_TILT_TENTHDEGREES_MAX_UP		( 19 * 10)	// +19 degrees
		#define CAMERA_TILT_TENTHDEGREES_MAX_DOWN	(-19 * 10)	// -19 degrees

		#define CAMERA_SENSE_RANGE				240		// Max distance Camera can spot a cone reliably
		#define SONY_CAMERA_MAX_RIGHT			870
		#define SONY_CAMERA_MAX_LEFT		  (-870)
		#define SONY_CAMERA_MAX_UP				290
		#define SONY_CAMERA_MAX_DOWN		  (-270)
		#define SONY_CAMERA_PAN_PER_DEGREE		7.8		// 870/90 degrees then experiment
		#define SONY_CAMERA_TILT_PER_DEGREE	    7.0		// 280/20 degrees then experiment
		#define SONY_CAMERA_MIN_ZOOM_ABS	 0x0000
		#define SONY_CAMERA_MAX_ZOOM_ABS	 0x03FF		// As specified in Sony docs


		#define CAMERA_PAN_PER_DEGREE			SONY_CAMERA_PAN_PER_DEGREE
		#define CAMERA_TILT_PER_DEGREE			SONY_CAMERA_TILT_PER_DEGREE

		#define CAMERA_PAN_SERVO_RATIO			10	// Pixel value to camera movement divisor
		#define CAMERA_TILT_SERVO_RATIO			10


		#define CAMERA_PAN_SERVO_INCREMENT		 10		// ticks per reading
		#define CAMERA_PAN_SERVO_CENTER			  0		// Trim Right = larger number
		#define CAMERA_PAN_SCAN_START_RIGHT		(CAMERA_PAN_SERVO_CENTER + (8*CAMERA_PAN_SERVO_INCREMENT))
		#define CAMERA_PAN_SCAN_START_LEFT		(CAMERA_PAN_SERVO_CENTER - (8*CAMERA_PAN_SERVO_INCREMENT))
		#define CAMERA_PAN_SERVO_MAX_RIGHT		SONY_CAMERA_MAX_RIGHT		// Max Right
		#define CAMERA_PAN_SERVO_MAX_LEFT		SONY_CAMERA_MAX_LEFT		// Max Left
		#define CAMERA_PAN_TILT_ABS_UNDEFINED	0xFE	// Servo positions are in an unknown state

		#define CAMERA_TILT_SERVO_CENTER		  0 	// Trim Up = larger number
		#define CAMERA_TILT_SERVO_MAX_UP		SONY_CAMERA_MAX_UP		// Max Up
		#define CAMERA_TILT_SERVO_MAX_DOWN		SONY_CAMERA_MAX_DOWN	// Max Down

		#define CAMERA_TILT_CURB				(CAMERA_TILT_SERVO_CENTER - 15)	// Point slightly down to see curbs
		#define CAMERA_PAN_CURB_RIGHT			(CAMERA_PAN_SERVO_MAX_RIGHT - 15)
		#define CAMERA_PAN_CURB_LEFT			(CAMERA_PAN_SERVO_MAX_LEFT + 15)

		#define CAMERA_TILT_FACE				(CAMERA_TILT_SERVO_CENTER + 200)	// Point upward to see faces. Max up is 290

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	#elif( CAMERA_CONTROL_TYPE == PIC_SERVO_CONTROL )

		#define CAMERA_PAN_CENTER					0		// TenthDegrees from center
		#define CAMERA_TILT_CENTER					0		// TenthDegrees from center

		// Camera motion range in Tenth Degrees
		// TODO - Need to set these to correct values!!!
		// Carbot camera can do ??? degrees (or +/- ??? degrees to be safe) max
		#define CAMERA_PAN_TENTHDEGREES_MAX_RIGHT	( 59 * 10)	// +59 degrees
		#define CAMERA_PAN_TENTHDEGREES_MAX_LEFT	(-59 * 10)	// -59 degrees
		#define CAMERA_TILT_TENTHDEGREES_MAX_UP		( 19 * 10)	// +19 degrees
		#define CAMERA_TILT_TENTHDEGREES_MAX_DOWN	(-10 * 10)	// -10 degrees

		#define CAMERA_SENSE_RANGE				240		// Max distance Camera can spot a cone reliably
		// Camera Pan/Tilt Constants
		#define CAMERA_PAN_SERVO_INCREMENT		  3		// ticks per reading
		#define CAMERA_PAN_SERVO_CENTER			 73		// Trim Right = larger number (63 default)
		#define CAMERA_PAN_SCAN_START_RIGHT	(CAMERA_PAN_SERVO_CENTER + (8*CAMERA_PAN_SERVO_INCREMENT))
		#define CAMERA_PAN_SCAN_START_LEFT	(CAMERA_PAN_SERVO_CENTER - (8*CAMERA_PAN_SERVO_INCREMENT))
		#define CAMERA_PAN_SERVO_MAX_RIGHT		120		// Max Right
		#define CAMERA_PAN_SERVO_MAX_LEFT		 30		// Max Left
		#define CAMERA_PAN_TILT_ABS_UNDEFINED	0xFE	// Servo positions are in an unknown state

		#define CAMERA_TILT_SERVO_CENTER		 63		// Trim Up = larger number (63 default)
		#define CAMERA_TILT_SERVO_MAX_UP		 72		// Max Up
		#define CAMERA_TILT_SERVO_MAX_DOWN		 48		// Max Down

		#define CAMERA_TILT_CURB				(CAMERA_TILT_SERVO_CENTER - 15)	// Point slightly down to see curbs
		#define CAMERA_PAN_CURB_RIGHT			(CAMERA_PAN_SERVO_MAX_RIGHT - 15)
		#define CAMERA_PAN_CURB_LEFT			(CAMERA_PAN_SERVO_MAX_LEFT + 15)

		#define CAMERA_TILT_FACE				(CAMERA_TILT_SERVO_MAX_UP)	// Point upward to see faces.  Not really used for Carbot

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	#elif( CAMERA_CONTROL_TYPE == EXTERN_SERVO_CONTROL )

		// Camera Pan/Tilt Constants
		#define CAMERA_PAN_TILT_ABS_UNDEFINED	0xFE	// Servo positions are in an unknown state

		#define CAMERA_PAN_SERVO_MAX_LEFT		 55		// Max Left
		#define CAMERA_PAN_SERVO_CENTER			150		// Trim Right = larger number
		#define CAMERA_PAN_SERVO_MAX_RIGHT		245		// Max Right
		#define CAMERA_PAN_SERVO_INCREMENT		 10		// ticks per reading
		#define CAMERA_PAN_SCAN_START_RIGHT	(CAMERA_PAN_SERVO_CENTER + (8*CAMERA_PAN_SERVO_INCREMENT))
		#define CAMERA_PAN_SCAN_START_LEFT	(CAMERA_PAN_SERVO_CENTER - (8*CAMERA_PAN_SERVO_INCREMENT))

		#define CAMERA_TILT_SERVO_MAX_UP		240 	// Max Up
		#define CAMERA_TILT_SERVO_CENTER		110		// Trim Up = larger number
		#define CAMERA_TILT_SERVO_MAX_DOWN		 48 	// Max Down

		#define CAMERA_TILT_CURB				(CAMERA_TILT_SERVO_CENTER - 30)	// Point slightly down to see curbs
		#define CAMERA_PAN_CURB_RIGHT			(CAMERA_PAN_SERVO_MAX_RIGHT + 30)
		#define CAMERA_PAN_CURB_LEFT			(CAMERA_PAN_SERVO_MAX_LEFT - 30)

		#define CAMERA_TILT_FACE				(CAMERA_TILT_SERVO_MAX_UP - 30)	// Point upward to see faces.  Not really used for Carbot

	#else
		#error BAD CAMERA_CONTROL_TYPE
	#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( ROBOT_TYPE == LOKI )

	// Camera motion range in Tenth Degrees
	// Dynamixel servos can do 0 to 300 degrees (or +/- 149 degrees to be safe) max

	//	1023 = 300degrees
	//	TicksPerDegree = 1023ticks / 300 degrees = 3.41 ticks
	//	DegreesPerTick = 300degrees / 1023 ticks = 0.293 degrees

	#define DYNA_SERVO_TICKS_PER_TENTHDEGREE	  0.341
	#define DYNA_SERVO_TENTHDEGREES_PER_TICK	  2.932
	//#define KERR_SERVO_TICKS_PER_TENTHDEGREE	  0.341 // not used - see KerrControl.h
	//#define KERR_SERVO_TENTHDEGREES_PER_TICK	  2.932 //

	//////////////////////////////////////////////////////////////
	#define DYNA_MULTI_SERVO_ID						 0	// Identify Broadcast servo messages
	#define DYNA_CAMERA_PAN_SERVO_ID				 1
	//#define DYNA_CAMERA_NECK_SERVO_ID				 2	// NOT USED!!!
	#define DYNA_CAMERA_TILT_SERVO_ID				 3
	#define DYNA_CAMERA_SIDETILT_SERVO_ID			 4

	#define NUMBER_OF_DYNA_SERVOS_IN_HEAD			 3

	//////////////////////////////////////////////////////////////
	// ARM SERVOS - Includes Shoulder Joint (even though it's not a Dyna Servo)

	#define DYNA_RIGHT_ARM_ELBOW_ROTATE_SERVO_ID	   5
	#define DYNA_RIGHT_ARM_ELBOW_BEND_SERVO_ID		   6
	#define DYNA_KINECT_SCANNER_SERVO_ID			   7	// Replaced arm servo with Kinect servo
	#define DYNA_RIGHT_ARM_WRIST_SERVO_ID			   8
	#define DYNA_RIGHT_ARM_CLAW_SERVO_ID			   9

	#define DYNA_LEFT_ARM_ELBOW_ROTATE_SERVO_ID		  10
	#define DYNA_LEFT_ARM_ELBOW_BEND_SERVO_ID		  11
	//#define DYNA_LEFT_ARM_ELBOW_BEND_SERVO2_ID		  12 -- Replaced by RX64 (servo 11) only!
	#define DYNA_LEFT_ARM_WRIST_SERVO_ID			  13
	#define DYNA_LEFT_ARM_CLAW_SERVO_ID				  14

	#define KERR_RIGHT_ARM_SHOULDER_SERVO_ID	 	  15	// Used for Command but not Status!
	#define KERR_LEFT_ARM_SHOULDER_SERVO_ID			  16	// Used for Command but not Status!

	#define NUMBER_OF_SMART_SERVOS					  16	// Total number of Smart Servos (Dyna+Kerr) + 1. Used for Bulk servo array, etc.
	#define NUMBER_OF_DYNA_AX12_SERVOS_PER_ARM		   3

	#define KERR_ARM_MOTOR_ID_RIGHT					   1	// Right Arm motor.  Use in KerrControl Only!
	#define KERR_ARM_MOTOR_ID_LEFT					   2	// Left Arm motor.  Use in KerrControl Only!


	//////////////////////////////////////////////////////////////
	// Common Values in TICKS
	// Right Arm
	#define DYNA_RIGHT_ARM_ELBOW_ROTATE_MAX			 930	// Rotate Max Clockwise (140 degrees)
	#define DYNA_RIGHT_ARM_ELBOW_ROTATE_MIN			   0	// Rotate Max Counter-clockwise (-140 degrees)

	#define DYNA_RIGHT_ARM_ELBOW_BEND_MAX			 984	// Max Curl - Same as Home2 position
	#define DYNA_RIGHT_ARM_ELBOW_BEND_MIN			 150	// Back-Bend (ouch!)

	#define DYNA_RIGHT_ARM_WRIST_ROTATE_MAX			1023	// Rotate Max Clockwise (150 degrees)
	#define DYNA_RIGHT_ARM_WRIST_ROTATE_MIN			   0	// Rotoate Max Counter Clockwise (-150 degrees)

	#define DYNA_RIGHT_ARM_CLAW_OPEN_MIN			 205	// Fully Open (90 degrees)
	#define DYNA_RIGHT_ARM_CLAW_CLOSED_MAX			 512	// Closed Tight! (0 degrees)

	// Left Arm
	#define DYNA_LEFT_ARM_ELBOW_ROTATE_MAX			 930	// Rotate Max Clockwise (140 degrees)
	#define DYNA_LEFT_ARM_ELBOW_ROTATE_MIN			   0	// Rotate Max Counter-clockwise (-140 degrees)

	#define DYNA_LEFT_ARM_ELBOW_BEND_MIN			 95		// Min = Curl - Same as Home position 2
	#define DYNA_LEFT_ARM_ELBOW_BEND_MAX			793		// Back-Bend (ouch!) (-40 degrees)

	#define DYNA_LEFT_ARM_WRIST_ROTATE_MAX			1023	// Rotate Max Clockwise (150 degrees)
	#define DYNA_LEFT_ARM_WRIST_ROTATE_MIN			   0	// Rotoate Max Counter Clockwise (-150 degrees)

	#define DYNA_LEFT_ARM_CLAW_OPEN_MAX				955		// Fully Open (130 degrees)
	#define DYNA_LEFT_ARM_CLAW_CLOSED_MIN			 409	// Closed Tight!

	#define DYNA_TORQUE_LIMIT_MAX				  0x03FF	// 1023

	// compliance = 0.3degrees per step
	#define DYNA_COMPLIANCE_LIMIT_MIN			     1	//  1 ( 0.3deg) - VERY Tight compliance!
	#define DYNA_COMPLIANCE_VERY_TIGHT			     2	//  2 ( 0.6deg) - Tight Compliance, Head/Kinect servos only?
	#define DYNA_COMPLIANCE_TIGHT				     4	//  4 ( 1.2deg) - Tight Compliance, Head/Kinect servos only?
	#define DYNA_COMPLIANCE_MED_TIGHT			     8	//  8 ( 2.4deg) - Med Tight Compliance, Head/Kinect servos only?
	#define DYNA_COMPLIANCE_MED					    16	// 16 ( 4.8deg) - Medium Compliance
	#define DYNA_COMPLIANCE_NORMAL				    32	// 32 ( 9.6deg) - Default Compliance
	#define DYNA_COMPLIANCE_LIMIT_MAX			    64	// 64 (19.2deg) - Loose Compliance


	// Tolerance for Joint positioning.  Joint within this will be considered at commanded position
	#define DYNA_SERVO_DELTA_DEFAULT_TENTHDEGREES				60	// = 6 degrees. 
	#define ARM_JOINT_DELTA_MAX_TENTHDEGREES					60	// = 6 degrees. 
	#define HEAD_JOINT_DELTA_MAX_TENTHDEGREES					30	// = 3 degrees. 
	#define HEAD_JOINT_DELTA_HIGH_PRECISION_TENTHDEGREES		15	// = 1.5 degrees. 
	#define KINECT_JOINT_DELTA_MAX_TENTHDEGREES					30	// = 3.0 degrees. 
	#define KINECT_SERVO_TOLERANCE_NORMAL_TENTHDEGREES			30	// = 3 degrees. 
	#define KINECT_SERVO_TOLERANCE_HIGH_PRECISION_TENTHDEGREES	 8	// = 0.8 degrees. 


	// Laser Scanner
	// Position for scanning with laser for objects on the floor
	#define LASER_FLOOR_SCAN_THRESHOLD_TENTH_DEGREES		(-200)	// Don't detect objects if laser pointed higher than this.  
																		// Obsolete?: If camera pointing down more then 20 degrees, compensate for small mechanical alignment changes
	#define LASER_FLOOR_SCAN_FAR_TENTH_DEGREES			(-160)	// 16.0 degrees down for looking for objects on the floor.  (~7 feet from robot)
	//        ^---- Warning!  too far to scan - max 99 scan lines!
	#define LASER_FLOOR_SCAN_5_FEET_TENTH_DEGREES		(-240)	// 24.0 degrees down for looking for objects on the floor.  (~5 feet from robot)
	#define LASER_FLOOR_SCAN_MEDIUM_TENTH_DEGREES		(-350)	// 35.0 degrees down for looking for objects on the floor.  (~? feet from robot)
	#define LASER_FLOOR_SCAN_NEAR_TENTH_DEGREES			(-600)	// 60.0 degrees down for objects near grasping range of robot (~ 12" from robot)
	#define LASER_FLOOR_SCAN_MIN_TENTH_DEGREES			(-760)	// Closest object detection (~ 3.5" from robot)

	// Position for Laser Scan Positioning (level laser) for SLAM
	#define LASER_TILT_TENTHDEGREES_SLAM_POSITION			0	// Head tilted up, so laser is level

	// Values in Servo Ticks:
	#define DYNA_CAMERA_SERVO_MAX_RIGHT			  	  0
	#define DYNA_CAMERA_SERVO_MAX_LEFT			   1021
	#define DYNA_CAMERA_SERVO_MAX_UP				348
	#define DYNA_CAMERA_SERVO_MAX_DOWN				750
//	#define DYNA_CAMERA_PAN_PER_TENTHDEGREE		  3.41		// 1023/300 degrees then experiment
//	#define DYNA_CAMERA_TILT_PER_TENTHDEGREE	  3.41		// Same?


//	#define CAMERA_PAN_PER_DEGREE			DYNA_CAMERA_PAN_PER_DEGREE
//	#define CAMERA_TILT_PER_DEGREE			DYNA_CAMERA_TILT_PER_DEGREE

//	#define CAMERA_PAN_SERVO_RATIO			10	// TODO-MUST Pixel value to camera movement divisor
//	#define CAMERA_TILT_SERVO_RATIO			10


	#define CAMERA_PAN_SERVO_INCREMENT		 10		// ticks per reading
	#define CAMERA_PAN_SERVO_CENTER			511		// Trim: Left = larger number
	#define CAMERA_PAN_SCAN_START_RIGHT		(CAMERA_PAN_SERVO_CENTER + (8*CAMERA_PAN_SERVO_INCREMENT))
	#define CAMERA_PAN_SCAN_START_LEFT		(CAMERA_PAN_SERVO_CENTER - (8*CAMERA_PAN_SERVO_INCREMENT))

	#define CAMERA_PAN_SERVO_MAX_RIGHT		DYNA_CAMERA_SERVO_MAX_RIGHT		// Max Right
	#define CAMERA_PAN_SERVO_MAX_LEFT		DYNA_CAMERA_SERVO_MAX_LEFT		// Max Left
	#define CAMERA_PAN_TILT_ABS_UNDEFINED	0xFE	// Servo positions are in an unknown state

	#define CAMERA_TILT_SERVO_CENTER		511 	// Trim: Down = larger number
	#define CAMERA_TILT_SERVO_MAX_UP		DYNA_CAMERA_SERVO_MAX_UP	// Max Up
	#define CAMERA_TILT_SERVO_MAX_DOWN		DYNA_CAMERA_SERVO_MAX_DOWN	// Max Down

	#define CAMERA_TILT_CURB				(CAMERA_TILT_SERVO_CENTER - 15)	// Point slightly down to see curbs
	#define CAMERA_PAN_CURB_RIGHT			(CAMERA_PAN_SERVO_MAX_RIGHT - 15)
	#define CAMERA_PAN_CURB_LEFT			(CAMERA_PAN_SERVO_MAX_LEFT + 15)


	// Loki Neck:
	#define CAMERA_NECK_SERVO_MAX_BACK				823		// Resting position
	#define CAMERA_NECK_SERVO_MAX_FORWARD			256		// Full forward
//	#define CAMERA_NECK_SERVO_PAN_ENABLE_POSITION	700		// Don't allow PAN if head further back then this!

	#define CAMERA_NECK_SERVO_BACK_POSITION			658		// Move back when startled
	#define CAMERA_NECK_SERVO_UP_POSITION			558		// Standard Up Position
	#define CAMERA_NECK_SERVO_INTEREST_POSITION		453		// Looking at something Interesting
	#define CAMERA_NECK_SERVO_FORWARD_POSITION		(CAMERA_NECK_SERVO_MAX_FORWARD - 20)	// Full Forward Position

	// Loki Curious Tilt:
	#define CAMERA_SIDETILT_SERVO_CENTER			511
	#define CAMERA_SIDETILT_SERVO_MAX_LEFT			389
	#define CAMERA_SIDETILT_SERVO_MAX_RIGHT			645

	#define CAMERA_SIDETILT_SERVO_TILT_LEFT			(CAMERA_SIDETILT_SERVO_CENTER - 100)
	#define CAMERA_SIDETILT_SERVO_TILT_RIGHT		(CAMERA_SIDETILT_SERVO_CENTER + 100)

	// "Rest" Servo Positions:
	#define CAMERA_NECK_SERVO_REST_POSITION			(CAMERA_NECK_SERVO_MAX_BACK - 5)
	#define CAMERA_PAN_SERVO_REST_POSITION			CAMERA_PAN_SERVO_CENTER
	#define CAMERA_TILT_SERVO_REST_POSITION			CAMERA_TILT_SERVO_CENTER		// Standard tilt when in "Rest" position
	#define CAMERA_SIDETILT_SERVO_REST_POSITION		CAMERA_SIDETILT_SERVO_CENTER	// 3 degree offset to Compensate for HW

//	#define CAMERA_TILT_SERVO_REST_LIMIT			350	// Max tilt when in Rest position

	// Laser Scanner:
	#define LASER_TILT_SERVO_CENTER					511
	#define LASER_TILT_SERVO_MAX_UP					0
	#define LASER_TILT_SERVO_MAX_DOWN				1021

	//////////////////////////////////////////////////////////////
	// Common Values in TENTH DEGREES
	#define CAMERA_PAN_CENTER									0		// TenthDegrees from center  See CAMERA_PAN_TENTH_DEGREES_ZERO
	#define CAMERA_TILT_CENTER									0		// TenthDegrees from center See CAMERA_TILT_TENTH_DEGREES_ZERO
	#define CAMERA_SIDETILT_CENTER								0		// 3 deg offset to compensate for HW?
	#define KINECT_TILT_CENTER									0		// TenthDegrees from center See LASER_TILT_TENTH_DEGREES_ZERO
	#define KINECT_HUMAN_DETECT_START_POSITION				(  25 * 10)	// +25 degrees - normal position for tracking humans
	#define KINECT_OBJECT_AVOIDANCE_POSITION				( -35 * 10)	// degrees - normal position for avoiding objects
	#define KINECT_SLEEP_POSITION							( -45 * 10)	// -45 degrees - keep dust off the lense

	#define KINECT_TILT_TENTHDEGREES_MAX_UP					(  65 * 10)	//  +65 degrees
	#define KINECT_TILT_TENTHDEGREES_MAX_DOWN				( -65 * 10)	//  -65 degrees
	//#define KINECT_TILT_TENTHDEGREES_NAVIGATE				( 20 * 10)	//  degrees - when looking for obstacles while navigating
	//#define KINECT_TILT_TENTHDEGREES_PEOPLE				( 20 * 10)	//  degrees - when looking for People

	#define CAMERA_PAN_TENTHDEGREES_MAX_RIGHT				( 149 * 10)	// +149 degrees
	#define CAMERA_PAN_TENTHDEGREES_MAX_LEFT				(-149 * 10)	// -149 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_UP					(  34 * 10)	//  +34 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_DOWN				( -72 * 10)	//  -72 degrees

	// Limits for how far head will move while tracking people
	#define CAMERA_PAN_TENTHDEGREES_MAX_TRACK_RIGHT			(  80 * 10)	// +80 degrees
	#define CAMERA_PAN_TENTHDEGREES_MAX_TRACK_LEFT			( -80 * 10)	// -80 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_TRACK_UP			(  38 * 10)	//  +40 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_TRACK_DOWN			( -20 * 10)	//  -20 degrees

	#define CAMERA_TILT_TENTHDEGREES_TRACK_LASER_MAX_UP		( -33 * 10)	// While tracking laser spot, keep head lower then this

	#define CAMERA_TILT_SLEEP_POSITION						( -68 * 10)	// Point down to "sleep"
	#define CAMERA_TILT_FACE_POSITION						(  15 * 10)	// Point upward to see faces.
	#define CAMERA_TILT_LASER_LEVEL							(  13 * 10)	// Laser is level - used for finding walls
	#define CAMERA_TILT_ROBOT_MOVING						( -25 * 10)	// Point down to look for obstacles while moving (37" out)

	#define CAMERA_PAN_PIR_RIGHT							(  30 * 10)	// Look to the right if PIR sensor detects motion
	#define CAMERA_PAN_PIR_LEFT								( -30 * 10)	// Look to the left if PIR sensor detects motion
	#define CAMERA_PAN_PIR_TENTHDEGREES						( 5 * 10)	// amount to move if PIR sensor detects thermal object



	// Loki Curious Tilt:
	#define CAMERA_SIDETILT_TENTHDEGREES_MAX_LEFT			(  21 * 10)	// +19 degrees
	#define CAMERA_SIDETILT_TENTHDEGREES_MAX_RIGHT			( -21 * 10)	// -19 degrees
	#define CAMERA_SIDETILT_TENTHDEGREES_TILT_LEFT			(  17 * 10)	// +15 degrees
	#define CAMERA_SIDETILT_TENTHDEGREES_TILT_RIGHT			( -17 * 10)	// -15 degrees

	#define CAMERA_NECK_TENTHDEGREES_MAX_FORWARD			(  69 * 10)	// +69 degrees
	#define CAMERA_NECK_TENTHDEGREES_MAX_BACK				( -89 * 10)	// -89 degrees

	#define CAMERA_NECK_TENTHDEGREES_BACK_POSITION			( -30 * 10)	// Move back when startled
	#define CAMERA_NECK_TENTHDEGREES_UP_POSITION			(  -8 * 10)	// Standard Up Position
	#define CAMERA_NECK_TENTHDEGREES_INTEREST_POSITION		(  20 * 10)	// Looking at something Interesting
	#define CAMERA_NECK_TENTHDEGREES_FORWARD_POSITION		(CAMERA_NECK_TENTHDEGREES_MAX_FORWARD - 20)	// Full Forward Position

	// "Rest" Positions:
	#define CAMERA_NECK_TENTHDEGREES_REST_POSITION			( -85 * 10)
	#define CAMERA_PAN_TENTHDEGREES_REST_POSITION			CAMERA_PAN_CENTER
	#define CAMERA_TILT_TENTHDEGREES_REST_POSITION			CAMERA_TILT_CENTER
	#define CAMERA_SIDETILT_TENTHDEGREES_REST_POSITION		CAMERA_SIDETILT_CENTER

//	#define CAMERA_TILT_TENTHDEGREES_REST_LIMIT				(  30 * 10)		// Max tilt when in Rest position
//	#define CAMERA_NECK_TENTHDEGREES_PAN_ENABLE_POSITION	( -60 * 10)		// Don't allow PAN if head further back then this!


// Misc stuff

	#define DYNA_CAMERA_MIN_ZOOM_ABS			0x0000
	#define DYNA_CAMERA_MAX_ZOOM_ABS			0x0001		// TODO - NOT SUPPORTED

	#define CAMERA_SENSE_RANGE				240		// Max distance Camera can spot a cone reliably

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif( ROBOT_TYPE == TURTLE )

	// Camera motion range in Tenth Degrees
	// Dynamixel servos can do 0 to 300 degrees (or +/- 149 degrees to be safe) max

	//	1023 = 300degrees
	//	TicksPerDegree = 1023ticks / 300 degrees = 3.41 ticks
	//	DegreesPerTick = 300degrees / 1023 ticks = 0.293 degrees

	#define DYNA_SERVO_TICKS_PER_TENTHDEGREE	  0.341
	#define DYNA_SERVO_TENTHDEGREES_PER_TICK	  2.932

	//////////////////////////////////////////////////////////////
	#define DYNA_MULTI_SERVO_ID						 0	// Identify Broadcast servo messages
	#define DYNA_CAMERA_PAN_SERVO_ID				 1
	//#define DYNA_CAMERA_NECK_SERVO_ID				 2	// NOT USED!!!
	#define DYNA_CAMERA_TILT_SERVO_ID				 3
	#define DYNA_CAMERA_SIDETILT_SERVO_ID			 4

	#define NUMBER_OF_DYNA_SERVOS_IN_HEAD			 3

	//////////////////////////////////////////////////////////////
	// ARM SERVOS - Includes Shoulder Joint (even though it's not a Dyna Servo)

	#define DYNA_RIGHT_ARM_ELBOW_ROTATE_SERVO_ID	   5
	#define DYNA_RIGHT_ARM_ELBOW_BEND_SERVO_ID		   6

	#define DYNA_KINECT_SCANNER_SERVO_ID			   7	// Replaced arm servo with Kinect servo

	#define DYNA_RIGHT_ARM_WRIST_SERVO_ID			   8
	#define DYNA_RIGHT_ARM_CLAW_SERVO_ID			   9

	#define DYNA_LEFT_ARM_ELBOW_ROTATE_SERVO_ID		  10
	#define DYNA_LEFT_ARM_ELBOW_BEND_SERVO_ID		  11
	//#define DYNA_LEFT_ARM_ELBOW_BEND_SERVO2_ID		  12 -- Replaced by RX64 (servo 11) only!
	#define DYNA_LEFT_ARM_WRIST_SERVO_ID			  13
	#define DYNA_LEFT_ARM_CLAW_SERVO_ID				  14

	#define KERR_RIGHT_ARM_SHOULDER_SERVO_ID	 	  15	// Used for Command but not Status!
	#define KERR_LEFT_ARM_SHOULDER_SERVO_ID			  16	// Used for Command but not Status!

	#define NUMBER_OF_DYNA_AX12_SERVOS_PER_ARM		   3

	#define KERR_ARM_MOTOR_ID_RIGHT					   1	// Right Arm motor.  Use in KerrControl Only!
	#define KERR_ARM_MOTOR_ID_LEFT					   2	// Left Arm motor.  Use in KerrControl Only!


	#define NUMBER_OF_SMART_SERVOS					  16	// Total number of Smart Servos (Dyna+Kerr) + 1. Used for Bulk servo array, etc.


	//////////////////////////////////////////////////////////////
	// Common Values in TICKS
	// Right Arm
	#define DYNA_RIGHT_ARM_ELBOW_ROTATE_MAX			 930	// Rotate Max Clockwise (140 degrees)
	#define DYNA_RIGHT_ARM_ELBOW_ROTATE_MIN			   0	// Rotate Max Counter-clockwise (-140 degrees)

	#define DYNA_RIGHT_ARM_ELBOW_BEND_MAX			 984	// Max Curl - Same as Home2 position
	#define DYNA_RIGHT_ARM_ELBOW_BEND_MIN			 150	// Back-Bend (ouch!)

	#define DYNA_RIGHT_ARM_WRIST_ROTATE_MAX			1023	// Rotate Max Clockwise (150 degrees)
	#define DYNA_RIGHT_ARM_WRIST_ROTATE_MIN			   0	// Rotoate Max Counter Clockwise (-150 degrees)

	#define DYNA_RIGHT_ARM_CLAW_OPEN_MIN			 205	// Fully Open (90 degrees)
	#define DYNA_RIGHT_ARM_CLAW_CLOSED_MAX			 512	// Closed Tight! (0 degrees)

	// Left Arm
	#define DYNA_LEFT_ARM_ELBOW_ROTATE_MAX			 930	// Rotate Max Clockwise (140 degrees)
	#define DYNA_LEFT_ARM_ELBOW_ROTATE_MIN			   0	// Rotate Max Counter-clockwise (-140 degrees)

	#define DYNA_LEFT_ARM_ELBOW_BEND_MIN			 95		// Min = Curl - Same as Home position 2
	#define DYNA_LEFT_ARM_ELBOW_BEND_MAX			793		// Back-Bend (ouch!) (-40 degrees)

	#define DYNA_LEFT_ARM_WRIST_ROTATE_MAX			1023	// Rotate Max Clockwise (150 degrees)
	#define DYNA_LEFT_ARM_WRIST_ROTATE_MIN			   0	// Rotoate Max Counter Clockwise (-150 degrees)

	#define DYNA_LEFT_ARM_CLAW_OPEN_MAX				955		// Fully Open (130 degrees)
	#define DYNA_LEFT_ARM_CLAW_CLOSED_MIN			 409	// Closed Tight!

	#define DYNA_TORQUE_LIMIT_MAX				  0x03FF	// 1023

	// compliance = 0.3degrees per step
	#define DYNA_COMPLIANCE_LIMIT_MIN			     1	//  1 ( 0.3deg) - VERY Tight compliance!
	#define DYNA_COMPLIANCE_VERY_TIGHT			     2	//  2 ( 0.6deg) - Tight Compliance, Head/Kinect servos only?
	#define DYNA_COMPLIANCE_TIGHT				     4	//  4 ( 1.2deg) - Tight Compliance, Head/Kinect servos only?
	#define DYNA_COMPLIANCE_MED_TIGHT			     8	//  8 ( 2.4deg) - Med Tight Compliance, Head/Kinect servos only?
	#define DYNA_COMPLIANCE_MED					    16	// 16 ( 4.8deg) - Medium Compliance
	#define DYNA_COMPLIANCE_NORMAL				    32	// 32 ( 9.6deg) - Default Compliance
	#define DYNA_COMPLIANCE_LIMIT_MAX			    64	// 64 (19.2deg) - Loose Compliance


	// Tolerance for Joint positioning.  Joint within this will be considered at commanded position
	#define DYNA_SERVO_DELTA_DEFAULT_TENTHDEGREES				60	// = 6 degrees. 
	#define ARM_JOINT_DELTA_MAX_TENTHDEGREES					60	// = 6 degrees. 
	#define HEAD_JOINT_DELTA_MAX_TENTHDEGREES					30	// = 3 degrees. 
	#define HEAD_JOINT_DELTA_HIGH_PRECISION_TENTHDEGREES		15	// = 1.5 degrees. 
	#define KINECT_JOINT_DELTA_MAX_TENTHDEGREES					30	// = 3.0 degrees. 
	#define KINECT_SERVO_TOLERANCE_NORMAL_TENTHDEGREES			30	// = 3 degrees. 
	#define KINECT_SERVO_TOLERANCE_HIGH_PRECISION_TENTHDEGREES	 8	// = 0.8 degrees. 


	// Laser Scanner
	// Position for scanning with laser for objects on the floor
	#define LASER_FLOOR_SCAN_THRESHOLD_TENTH_DEGREES		(-200)	// Don't detect objects if laser pointed higher than this.  
																		// Obsolete?: If camera pointing down more then 20 degrees, compensate for small mechanical alignment changes
	#define LASER_FLOOR_SCAN_FAR_TENTH_DEGREES			(-160)	// 16.0 degrees down for looking for objects on the floor.  (~7 feet from robot)
	//        ^---- Warning!  too far to scan - max 99 scan lines!
	#define LASER_FLOOR_SCAN_5_FEET_TENTH_DEGREES		(-240)	// 24.0 degrees down for looking for objects on the floor.  (~5 feet from robot)
	#define LASER_FLOOR_SCAN_MEDIUM_TENTH_DEGREES		(-350)	// 35.0 degrees down for looking for objects on the floor.  (~? feet from robot)
	#define LASER_FLOOR_SCAN_NEAR_TENTH_DEGREES			(-600)	// 60.0 degrees down for objects near grasping range of robot (~ 12" from robot)
	#define LASER_FLOOR_SCAN_MIN_TENTH_DEGREES			(-760)	// Closest object detection (~ 3.5" from robot)

	// Position for Laser Scan Positioning (level laser) for SLAM
	#define LASER_TILT_TENTHDEGREES_SLAM_POSITION			0	// Head tilted up, so laser is level

	// Values in Servo Ticks:
	#define DYNA_CAMERA_SERVO_MAX_RIGHT			  	  0
	#define DYNA_CAMERA_SERVO_MAX_LEFT			   1021
	#define DYNA_CAMERA_SERVO_MAX_UP				348
	#define DYNA_CAMERA_SERVO_MAX_DOWN				750
//	#define DYNA_CAMERA_PAN_PER_TENTHDEGREE		  3.41		// 1023/300 degrees then experiment
//	#define DYNA_CAMERA_TILT_PER_TENTHDEGREE	  3.41		// Same?


//	#define CAMERA_PAN_PER_DEGREE			DYNA_CAMERA_PAN_PER_DEGREE
//	#define CAMERA_TILT_PER_DEGREE			DYNA_CAMERA_TILT_PER_DEGREE

//	#define CAMERA_PAN_SERVO_RATIO			10	// TODO-MUST Pixel value to camera movement divisor
//	#define CAMERA_TILT_SERVO_RATIO			10


	#define CAMERA_PAN_SERVO_INCREMENT		 10		// ticks per reading
	#define CAMERA_PAN_SERVO_CENTER			511		// Trim: Left = larger number
	#define CAMERA_PAN_SCAN_START_RIGHT		(CAMERA_PAN_SERVO_CENTER + (8*CAMERA_PAN_SERVO_INCREMENT))
	#define CAMERA_PAN_SCAN_START_LEFT		(CAMERA_PAN_SERVO_CENTER - (8*CAMERA_PAN_SERVO_INCREMENT))

	#define CAMERA_PAN_SERVO_MAX_RIGHT		DYNA_CAMERA_SERVO_MAX_RIGHT		// Max Right
	#define CAMERA_PAN_SERVO_MAX_LEFT		DYNA_CAMERA_SERVO_MAX_LEFT		// Max Left
	#define CAMERA_PAN_TILT_ABS_UNDEFINED	0xFE	// Servo positions are in an unknown state

	#define CAMERA_TILT_SERVO_CENTER		511 	// Trim: Down = larger number
	#define CAMERA_TILT_SERVO_MAX_UP		DYNA_CAMERA_SERVO_MAX_UP	// Max Up
	#define CAMERA_TILT_SERVO_MAX_DOWN		DYNA_CAMERA_SERVO_MAX_DOWN	// Max Down

	#define CAMERA_TILT_CURB				(CAMERA_TILT_SERVO_CENTER - 15)	// Point slightly down to see curbs
	#define CAMERA_PAN_CURB_RIGHT			(CAMERA_PAN_SERVO_MAX_RIGHT - 15)
	#define CAMERA_PAN_CURB_LEFT			(CAMERA_PAN_SERVO_MAX_LEFT + 15)


	// Loki Neck:
	#define CAMERA_NECK_SERVO_MAX_BACK				823		// Resting position
	#define CAMERA_NECK_SERVO_MAX_FORWARD			256		// Full forward
//	#define CAMERA_NECK_SERVO_PAN_ENABLE_POSITION	700		// Don't allow PAN if head further back then this!

	#define CAMERA_NECK_SERVO_BACK_POSITION			658		// Move back when startled
	#define CAMERA_NECK_SERVO_UP_POSITION			558		// Standard Up Position
	#define CAMERA_NECK_SERVO_INTEREST_POSITION		453		// Looking at something Interesting
	#define CAMERA_NECK_SERVO_FORWARD_POSITION		(CAMERA_NECK_SERVO_MAX_FORWARD - 20)	// Full Forward Position

	// Loki Curious Tilt:
	#define CAMERA_SIDETILT_SERVO_CENTER			511
	#define CAMERA_SIDETILT_SERVO_MAX_LEFT			389
	#define CAMERA_SIDETILT_SERVO_MAX_RIGHT			645

	#define CAMERA_SIDETILT_SERVO_TILT_LEFT			(CAMERA_SIDETILT_SERVO_CENTER - 100)
	#define CAMERA_SIDETILT_SERVO_TILT_RIGHT		(CAMERA_SIDETILT_SERVO_CENTER + 100)

	// "Rest" Servo Positions:
	#define CAMERA_NECK_SERVO_REST_POSITION			(CAMERA_NECK_SERVO_MAX_BACK - 5)
	#define CAMERA_PAN_SERVO_REST_POSITION			CAMERA_PAN_SERVO_CENTER
	#define CAMERA_TILT_SERVO_REST_POSITION			CAMERA_TILT_SERVO_CENTER		// Standard tilt when in "Rest" position
	#define CAMERA_SIDETILT_SERVO_REST_POSITION		CAMERA_SIDETILT_SERVO_CENTER	// 3 degree offset to Compensate for HW

//	#define CAMERA_TILT_SERVO_REST_LIMIT			350	// Max tilt when in Rest position

	// Laser Scanner:
	#define LASER_TILT_SERVO_CENTER					511
	#define LASER_TILT_SERVO_MAX_UP					0
	#define LASER_TILT_SERVO_MAX_DOWN				1021

	//////////////////////////////////////////////////////////////
	// Common Values in TENTH DEGREES
	#define CAMERA_PAN_CENTER									0		// TenthDegrees from center  See CAMERA_PAN_TENTH_DEGREES_ZERO
	#define CAMERA_TILT_CENTER									0		// TenthDegrees from center See CAMERA_TILT_TENTH_DEGREES_ZERO
	#define CAMERA_SIDETILT_CENTER								0		// 3 deg offset to compensate for HW?
	#define KINECT_TILT_CENTER									0		// TenthDegrees from center See LASER_TILT_TENTH_DEGREES_ZERO
	#define KINECT_HUMAN_DETECT_START_POSITION				( 25 * 10)	// +25 degrees - normal position for tracking humans
	#define KINECT_OBJECT_AVOIDANCE_POSITION				(-45 * 10)	// degrees - normal position for avoiding objects
	#define KINECT_SLEEP_POSITION							(-75 * 10)	// degrees - keep dust off the lense

	#define KINECT_TILT_TENTHDEGREES_MAX_UP					(  65 * 10)	//  +65 degrees
	#define KINECT_TILT_TENTHDEGREES_MAX_DOWN				( -65 * 10)	//  -65 degrees
	#define KINECT_TILT_TENTHDEGREES_LONG_RANGE				( -25 * 10)	//  24" - Max range (~20 feet?)
	#define KINECT_TILT_TENTHDEGREES_MID_RANGE				( -40 * 10)	//  6" - 60" (almost 6 feet) 
	#define KINECT_TILT_TENTHDEGREES_CLOSE_RANGE			( -55 * 10)	//  6" - 28" (including 6 in on front-sides)
	//#define KINECT_TILT_TENTHDEGREES_NAVIGATE				( -50 * 10)	//  degrees - when looking for obstacles while navigating
	//#define KINECT_TILT_TENTHDEGREES_PEOPLE				( 20 * 10)	//  degrees - when looking for People

	#define CAMERA_PAN_TENTHDEGREES_MAX_RIGHT				( 149 * 10)	// +149 degrees
	#define CAMERA_PAN_TENTHDEGREES_MAX_LEFT				(-149 * 10)	// -149 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_UP					(  34 * 10)	//  +34 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_DOWN				( -72 * 10)	//  -72 degrees

	// Limits for how far head will move while tracking people
	#define CAMERA_PAN_TENTHDEGREES_MAX_TRACK_RIGHT			(  80 * 10)	// +80 degrees
	#define CAMERA_PAN_TENTHDEGREES_MAX_TRACK_LEFT			( -80 * 10)	// -80 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_TRACK_UP			(  38 * 10)	//  +40 degrees
	#define CAMERA_TILT_TENTHDEGREES_MAX_TRACK_DOWN			( -20 * 10)	//  -20 degrees

	#define CAMERA_TILT_TENTHDEGREES_TRACK_LASER_MAX_UP		( -33 * 10)	// While tracking laser spot, keep head lower then this

	#define CAMERA_TILT_SLEEP_POSITION						( -68 * 10)	// Point down to "sleep"
	#define CAMERA_TILT_FACE_POSITION						(  15 * 10)	// Point upward to see faces.
	#define CAMERA_TILT_LASER_LEVEL							(  13 * 10)	// Laser is level - used for finding walls
	#define CAMERA_TILT_ROBOT_MOVING						( -50 * 10)	// Point down to look for obstacles while moving

	#define CAMERA_PAN_PIR_RIGHT							(  30 * 10)	// Look to the right if PIR sensor detects motion
	#define CAMERA_PAN_PIR_LEFT								( -30 * 10)	// Look to the left if PIR sensor detects motion
	#define CAMERA_PAN_PIR_TENTHDEGREES						( 5 * 10)	// amount to move if PIR sensor detects thermal object



	// Loki Curious Tilt:
	#define CAMERA_SIDETILT_TENTHDEGREES_MAX_LEFT			(  21 * 10)	// +19 degrees
	#define CAMERA_SIDETILT_TENTHDEGREES_MAX_RIGHT			( -21 * 10)	// -19 degrees
	#define CAMERA_SIDETILT_TENTHDEGREES_TILT_LEFT			(  17 * 10)	// +15 degrees
	#define CAMERA_SIDETILT_TENTHDEGREES_TILT_RIGHT			( -17 * 10)	// -15 degrees

	#define CAMERA_NECK_TENTHDEGREES_MAX_FORWARD			(  69 * 10)	// +69 degrees
	#define CAMERA_NECK_TENTHDEGREES_MAX_BACK				( -89 * 10)	// -89 degrees

	#define CAMERA_NECK_TENTHDEGREES_BACK_POSITION			( -30 * 10)	// Move back when startled
	#define CAMERA_NECK_TENTHDEGREES_UP_POSITION			(  -8 * 10)	// Standard Up Position
	#define CAMERA_NECK_TENTHDEGREES_INTEREST_POSITION		(  20 * 10)	// Looking at something Interesting
	#define CAMERA_NECK_TENTHDEGREES_FORWARD_POSITION		(CAMERA_NECK_TENTHDEGREES_MAX_FORWARD - 20)	// Full Forward Position

	// "Rest" Positions:
	#define CAMERA_NECK_TENTHDEGREES_REST_POSITION			( -85 * 10)
	#define CAMERA_PAN_TENTHDEGREES_REST_POSITION			CAMERA_PAN_CENTER
	#define CAMERA_TILT_TENTHDEGREES_REST_POSITION			CAMERA_TILT_CENTER
	#define CAMERA_SIDETILT_TENTHDEGREES_REST_POSITION		CAMERA_SIDETILT_CENTER

//	#define CAMERA_TILT_TENTHDEGREES_REST_LIMIT				(  30 * 10)		// Max tilt when in Rest position
//	#define CAMERA_NECK_TENTHDEGREES_PAN_ENABLE_POSITION	( -60 * 10)		// Don't allow PAN if head further back then this!


// Misc stuff

	#define DYNA_CAMERA_MIN_ZOOM_ABS			0x0000
	#define DYNA_CAMERA_MAX_ZOOM_ABS			0x0001		// TODO - NOT SUPPORTED

	#define CAMERA_SENSE_RANGE				240		// Max distance Camera can spot a cone reliably





#else
	#error BAD ROBOT_TYPE
#endif



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Non-Specific settings
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

#define WAYPOINT_SENSOR_RANGE			 960		// Distance (Tenth_Inches) from waypoint to start looking for landmarks
#define WAYPOINT_STOP_RANGE				 480		// Distance (inches) from waypoint landmark to stop, and head to next landmark
#define WAYPOINT_BACKUP_RANGE			 360		// If closer than this, backup before heading to next landmark

// Camera Pan/Tilt control
#define US_SCAN_MAX_SAMPLES			     16		// See "US1_SAMPLES"

// Camera Control parameters (used in WM_ROBOT_CAMERA_PAN_CMD, and by Pic)
#define CAMERA_PAN_STOP						0x00
#define CAMERA_PAN_UP						0x01
#define CAMERA_PAN_DOWN						0x02
#define CAMERA_PAN_LEFT						0x03
#define CAMERA_PAN_RIGHT					0x04
#define CAMERA_PAN_UP_LEFT					0x05
#define CAMERA_PAN_UP_RIGHT					0x06
#define CAMERA_PAN_DOWN_LEFT				0x07
#define CAMERA_PAN_DOWN_RIGHT				0x08
#define CAMERA_PAN_ABS_CENTER				0x09	// Absolute Center Command

// ABS Camera Speed settings
//#define CAMERA_SPEED_VERY_SLOW				1		// Range 1 to SERVO_SPEED_MAX (15)
//#define CAMERA_SPEED_SLOW					4
//#define CAMERA_SPEED_MED					8
//#define CAMERA_SPEED_FAST					12
//#define CAMERA_SPEED_MAX					SERVO_SPEED_MAX


#define SIO_SYNC_0							0xE5	// Serial Command Sync characters
#define SIO_SYNC_1							0x5F
#define CMD_TERM_CHAR						0xC4	// Serial Command termination character (use for check)
#define SERIAL_CMD_SIZE 					   8	// Fixed length of a command from the Host to the PIC

#define LED_EYES_OFF						0x00
#define LED_EYES_ON							0x01
#define LED_EYES_CLOSE						0x02
#define LED_EYES_OPEN						0x03
#define LED_EYES_BLINK						0x04
#define LED_EYE_BRIGHTNESS_MAX				  19	// On time in 20ms cycle


//////////////////////////////////////////////////////////////////////////////////////////
// Hardware Command IDs to the PIC			CMD_ID	PARAMETER1		PARAMETER2
#define HW_NO_CMD_PENDING					0x00 // Used host code to track flag that no commands are pending
#define HW_GET_STATUS						0x01 //	-				-
#define HW_SET_MOTOR_STOP					0x02 //	-				-
//#define HW_SET_MOVE_DISTANCE				0x03 // -				Speed   + High Byte	+ Low Byte Distance to Move in "Ticks"

#define HW_GET_VERSION						0x05 //	-				-
#define HW_RESET_CPU						0x06 //	-				-
#define HW_KEEP_ALIVE						0x07 //	-				-
#define HW_GET_RADAR_SCAN_DATA				0x08 //	-				-
#define HW_INITIALIZE						0x09 //	-				-
#define HW_SET_SPEED_AND_TURN				0x0A //	Speed			Turn	(in servo values)
#define HW_SET_TURN							0x0B //					Speed (servo val)
#define HW_SET_SPEED						0x0C //					Speed (servo val)
#define HW_CLEAR_ERROR						0x0D //	-				-

#define HW_SET_POWER_MODE					0x10 //	Servo Power		Laptop Power -	Sets power of servos, laptop and PIC!
#define HW_SET_MOTOR_POWER					0x11 //	-				On/Off
#define HW_SET_LIGHT_POWER					0x12 //	Camera Lights	On/Off
#define HW_SET_AUX_LIGHT_POWER				0x13 // Aux/Running Lts On/Off
#define HW_RESET_ODOMETER					0x14 //	-				-
#define HW_SET_LED_EYES						0x15 // Left Eye Off/On/Blink		Right Eye Off/On/Blink
#define HW_RESET_WATCHDOG					0x16 //	-				-
#define HW_SET_MOTOR_BRAKE					0x17 //	Brake Value		Brake Time			-
#define HW_COMPASS_CAL_MODE					0x18 // -				-
#define HW_COMPASS_CAL_POINT				0x19 // -				-
#define HW_INITIALIZE_SERVO					0x1A

#define HW_ENABLE_RADAR_SCAN				0x1B //	-				Off/On
#define HW_ENABLE_SPEED_CONTROL				0x1C // -				Off/On
#define HW_SET_DYNA_USB						0x1D // -				Off/On
#define HW_SET_IR_SENSOR_POWER				0x1E // L/Right (TODO)	Off/On
#define HW_SET_KINECT_POWER					0x1F // -				On/Off

#define HW_CAMERA_MIN_COMMAND				0x20 // For sending to the right thread!
#define HW_CAMERA_INITIALIZE				0x20 //	Pan/Tilt Dir	Pan/Tilt Speed
#define HW_SET_CAMERA_PAN_TILT				0x21 //	Pan/Tilt Dir	Pan/Tilt Speed
#define HW_SET_CAMERA_PAN_ABS				0x22 //	Servo Position	-
#define HW_SET_CAMERA_TILT_ABS				0x23 //	Servo Position	-
#define HW_SET_CAMERA_PAN_TILT_ABS			0x24 // Pan Position	Tilt Position
#define HW_SET_CAMERA_ZOOM					0x25 //	Zoom In/Out		Zoom Speed
#define HW_SET_CAMERA_POWER					0x26 //	On/Off			-
#define HW_SET_CAMERA_ZOOM_ABS				0x27 // Zoom Level (range 0 to 64)
#define HW_SET_CAMERA_ABS_PAN_TILT_SPEED	0x28 // Speed (0-22)
#define HW_SET_CAMERA_MODE					0x29 // Mode			Value
#define HW_SET_CAMERA_NECK_ABS				0x2A // Position		-
#define HW_SET_CAMERA_SIDETILT_ABS			0x2B // Position 		-
#define HW_SET_CAMERA_FORWARD_ABS			0x2C
#define HW_GET_CAMERA_BULK_SERVO_STATUS		0x2D //	Just dumps status to the log
#define HW_SET_BULK_HEAD_POSITION			0x2E // Uses global g_DynaServoCmd[] buffer
#define HW_SET_BULK_KINECT_POSITION			0x2F // Uses global g_DynaServoCmd[] buffer

#define HW_CAMERA_MAX_COMMAND				0x2F // For sending to the right thread!

#define HW_OTHER_SERVO_MIN_COMMAND			0x30 // For sending to the right thread!
#define HW_SET_GEAR							0x31 //	-				Servo Position (LOW_GEAR or HIGH_GEAR)
#define HW_SET_SERVO_POS_TICKS				0x32 //	Servo			Positon (SERVO_MIN to SERVO_MAX)
#define HW_SET_SERVO_POS_TENTHDEGREES		0x33 //	Servo			Positon (SERVO_MIN to SERVO_MAX)
#define HW_GET_SERVO_STATUS					0x34 //	Servo			Current Status
#define HW_READ_SERVO_REGISTER				0x35 //	Servo			Current Status
#define HW_WRITE_SERVO_REGISTER_BYTE		0x36 //	Servo			Current Status
#define HW_WRITE_SERVO_REGISTER_WORD		0x37 //	Servo			Current Status

#define HW_SET_SERVO_TORQUE_ENABLE			0x38 //	Which Arm.  Uses global g_DynaServoCmd[] buffer
#define HW_SET_SERVO_TORQUE_LIMIT			0x39 //	Arm/Joint		Value (0 - 1023) see DYNA_TORQUE_LIMIT_MAX
#define HW_SET_BULK_ARM_POSITION			0x3A //	Which Arm.  Uses global g_DynaServoCmd[] buffer
#define HW_GET_SMART_SERVO_STATUS			0x3B //	Servo Group.		Return uses global g_DynaServoCmd[] buffer
#define HW_SET_SERVO_COMPLIANCE				0x3C // Servo			Compliance Slope see DYNA_COMPLIANCE_TIGHT

#define HW_OTHER_SERVO_MAX_COMMAND			0x3D // For sending to the right thread!


#define HW_PIC_MAX_MESSAGE					0x3D// Max message for the PIC
// Other Hardware messages (such as serial Port messages) go here!

#define HW_SET_LASER_SCANNER_POWER			0x3E //	N/A.			1=On/0=Off
#define HW_LASER_REQUEST_SCAN				0x3F // N/A.			Number of Scans to do

#define HW_IROBOT_UNDOCK					0x40
#define HW_IROBOT_DOCK						0x41

#define HW_MAX_MESSAGE						0x42	// Maximum number a message can be (for error trapping)
//////////////////////////////////////////////////////////////////////////////////////////

// Status Flags from the PIC (StatusFlags)
#define HW_STATUS_WATCHDOG_EXPIRED			0x01
#define HW_STATUS_POWER_ON_INDICATOR		0x02
#define HW_STATUS_MOVE_DISTANCE_COMPLETE	0x04
#define HW_STATUS_BRAKE_COMPLETE			0x08
#define HW_STATUS_RADAR_SCAN_READY			0x10
#define HW_STATUS_RC_BUTTON_PWR_ENABLE		0x20
#define HW_STATUS_RC_BUTTON_2				0x40
//#define HW_STATUS_REAR_BUMPER				0x80	// KLUDGE- put here since 9 bumpers!


// Error Codes from PIC (LastError)
#define HW_PIC_NO_ERROR						0x00
#define HW_SERIAL_CMD_BUFFER_OVERFLOW		0x01
#define HW_SERIAL_CMD_NOT_READY				0x02
#define HW_SERIAL_CMD_INVALID				0x03
#define HW_SERIAL_CMD_CHECKSUM_ERROR		0x04
#define HW_BAD_STATE						0x05
#define HW_UNKNOWN_CMD						0x06
#define HW_CMD_NOT_IMPLEMENTED				0x07
#define HW_SERIAL_EXTRA_CHARS				0x08
#define HW_SERIAL_BAD_SYNC					0x09
#define HW_SERIAL_WAIT_PRIOR_CMD			0x0A
#define HW_SERVO_VALUE_ERROR				0x0B
#define HW_RADAR_SCAN_BAD_STATE				0x0C
#define HW_BAD_ISR							0x0D	// This is a CRITICAL ERROR!

// Sensor Error values
#define HW_SENSOR_ERROR						0xFE	// Bad Sensor or other error
#define HW_SENSOR_NOT_ATTACHED				0xFD	// Sensor never started
#define HW_SENSOR_NO_OBJECT					0xFC	// Max Range (no object detected)

// Generic Power commands
#define POWER_OFF							0x00
#define POWER_ON							0x01


// Messages - Messages from the PIC, with Data
#define HW_MSG_DEBUG_STRING					0x00
//#define HW_MSG_IR_RANGE1					0x01
//#define HW_MSG_IR_RANGE2					0x02

// Other defines
#define RESPONSE_STRING_LEN					52	// Max length of response from the PIC

// HW_SET_POWER_MODE Modes
// Servo will press Laptop Power button.  Typically, puts laptop in standby.
#define LAPTOP_POWER_MODE_DEFAULT				0x00	// Leave laptop alone.  Do nothing
#define LAPTOP_POWER_OFF_NO_RESTART				0x01	// Shut down Laptop (using servo).  Requires manual restart.
#define LAPTOP_POWER_OFF_RESTART_ON_ANY_SENSOR	0x02	// Shut down, but restart when any sensor triggers (preset thresholds)
#define LAPTOP_POWER_OFF_RESTART_ON_PIR_SENSOR	0x03	// Shut down, restart only with PIR sensor
#define LAPTOP_POWER_OFF_RESTART_NO_PIR_SENSOR	0x04	// Shut down, restart with any sensor BUT PIR
#define LAPTOP_POWER_OFF_RESTART_ON_TIMER		0x05	// TODO - not implemented!



#endif	//__ROBOT_PIC_COMMANDS_H__
