// HardwareConfig.h
// Physical hardware configuration / dimensions of robot
// Note: Make sure to include HardwareCmds.h if needed

#ifndef __ROBOT_HARDWARE_CONFIG_DEFINED_H__
#define __ROBOT_HARDWARE_CONFIG_DEFINED_H__

#include "RobotType.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                            LOKI
////////////////////////////////////////////////////////////////////////////////////////////////////////
#if SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI

	// Note: Arm Length measurements defined in ArmControl.cpp

	#define DISTANCE_BETWEEN_WHEELS_TENTH_INCHES 142.0	// 14.0 inches -Used for calculating amount of turn based upon difference in wheel odometers

	#define ROBOT_BODY_SENSOR_RADIUS_TENTH_INCHES		80.0	// Sensor is 8 inches from center of robot body
	#define ROBOT_BODY_WIDTH_TENTH_INCHES				160	// Width of robot main body
	#define ROBOT_BODY_LENGTH_TENTH_INCHES				160	// Length of robot main body up to tail wheel (actual body is a little longer)
	#define ROBOT_ARM_WIDTH_TENTH_INCHES			30	// Amount that each Arm sticks out from the sides of the robot main body
	#define ROBOT_BODY_WITH_ARMS_WIDTH_TENTH_INCHES		(ROBOT_BODY_WIDTH_TENTH_INCHES + (ROBOT_ARM_WIDTH_TENTH_INCHES * 2))	// Size of robot, for calculating if it can fit between objects!

	#define HALF_ROBOT_BODY_WIDTH_TENTH_INCHES				(ROBOT_BODY_WIDTH_TENTH_INCHES/2)
	#define HALF_ROBOT_BODY_LENGTH_TENTH_INCHES				(ROBOT_BODY_LENGTH_TENTH_INCHES/2)
	#define HALF_ROBOT_BODY_WITH_ARMS_WIDTH_TENTH_INCHES	(HALF_ROBOT_BODY_WIDTH_TENTH_INCHES + ROBOT_ARM_WIDTH_TENTH_INCHES)
	#define HALF_ROBOT_CLAW_WIDTH_TENTH_INCHES				20

	// Sensor Zones for object avoidance
	#define FRONT_ZONE_THREAT_MIN_THRESHOLD		 120	// Tenth_Inches - Always avoid objects closer then this on the front
	#define ARM_ZONE_THREAT_MIN_THRESHOLD		  30	// Tenth_Inches - Always avoid objects closer then this on the front
	#define SIDE_THREAT_MIN_THRESHOLD			  40	// Tenth_Inches - avoid objects closer then this on the side
	#define SIDE_THREAT_NORM_THRESHOLD			 100	// Tenth_Inches - avoid objects closer then this on the side
	#define THREAT_PERSISTANCE_TIME				 50	// (1/10 sec) - how long to persist until allowing wider side avoidance behavior

	#define FRONT_CENTER_ZONE_EDGE_TENTH_INCHES				HALF_ROBOT_BODY_WIDTH_TENTH_INCHES // 2 zones in front center of robot
	#define FRONT_ARM_ZONE_EDGE_TENTH_INCHES				(FRONT_CENTER_ZONE_EDGE_TENTH_INCHES+ROBOT_ARM_WIDTH_TENTH_INCHES) // zone in front of each arm
	#define FRONT_SIDE_ZONE_WIDTH_TENTH_INCHES				90 // look for objects to avoid this far to either side of the robot
	#define FRONT_SIDE_ZONE_EDGE_TENTH_INCHES				(FRONT_ARM_ZONE_EDGE_TENTH_INCHES+FRONT_SIDE_ZONE_WIDTH_TENTH_INCHES)

	#define AVOIDANCE_SHARP_TURN_RANGE			  40 // Tenth_Inches - if front object closer then this, turn sharply

	// Head measurements (for Camera positioning)
	#define HEAD_IR_OFFSET_FROM_FRONT_TENTH_INCHES	 70	// Distance from Head IR sensors to front of robot
	#define LASER_MS_COMMAND_BASE_LEN			13 // lengh of command NOT including "Number of Scans"

	#define CAMERA_CENTER_HEIGHT_ABOVE_GROUND_TENTH_INCHES		 370.0	// Tenth_Inches (not really correct - was fudged.. todo
	#define CAMERA_CENTER_DIST_FROM_ROBOT_FRONT_TENTH_INCHES	 100.0	// Tenth_Inches - distance to SERVO, not CAMERA! (8.5?)

	#define CAMERA_SERVO_HEIGHT_ABOVE_GROUND_TENTH_INCHES		 345.0	// Tenth_Inches 34.5
	#define CAMERA_SERVO_DISTANCE_FROM_FRONT_TENTH_INCHES		  92.0	// Tenth_Inches 9.2
	#define CAMERA_HEIGHT_ABOVE_CAMERA_SERVO_TENTH_INCHES		  76.0	// Tenth_Inches 7.6
	#define CAMERA_OFFSET_IN_FRONT_OF_SERVO_TENTH_INCHES		  25.0	// Tenth_Inches - Camera lense sits in front of neck servo

	// Laser position measurements
	#define LASER_MIN_OBJECT_HEIGHT_TENTH_INCHES		  		   8	// Tenth_Inches!
	#define LASER_HEIGHT_ABOVE_GROUND_TENTH_INCHES				 274.0	// Tenth_Inches - distance to laser beam
//	#define LASER_HEIGHT_ABOVE_SERVO_TENTH_INCHES				  10.0	// Tenth_Inches 
	#define LASER_DISTANCE_FROM_FRONT_TENTH_INCHES				  34.0	// Tenth_Inches - distance to laser front

	// Kinect position measurements
	#define KINECT_SLICE_MIN_OBJECT_HEIGHT_TENTH_INCHES	  		   4.0	// Tenth_Inches! - min height to be included in a slice (but 3D check is tighter)
	#define KINECT_HEIGHT_ABOVE_GROUND_TENTH_INCHES				 265.0	// Tenth_Inches - distance to Kinect sensor center
	#define KINECT_DISTANCE_FROM_FRONT_TENTH_INCHES				  40.0	// Tenth_Inches - distance to Kinect internal sensor

	// Degrees from Forward of each sensor
	// TODO! IR2,IR3 and US0 - Depends upon where the head is facing!  ASSUME forward for now!

	static int SensorOffsetDegrees_IR[NUM_IR_SENSORS] =
		//  0    1    2    3    4    5 		// IR Sensor Number
		{ -70, -10,  -5,   5,  10,  70 };	// Degrees from Forward

	static int SensorOffsetDegrees_US[NUM_US_SENSORS] = 
		//  0    1    2  					// UltraSonic Sensor Number
		{   0, -20,   20 };					// Degrees from Forward

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                           TURTLE
////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif SENSOR_CONFIG_TYPE == SENSOR_CONFIG_TURTLE
	// Turtle Sensor Configuration:


	#define DISTANCE_BETWEEN_WHEELS_TENTH_INCHES 142.0	// 14.0 inches -Used for calculating amount of turn based upon difference in wheel odometers

	#define ROBOT_BODY_SENSOR_RADIUS_TENTH_INCHES		75.0	// Sensor is 7.5 inches from center of robot body
	#define ROBOT_BODY_DRAW_RADIUS_TENTH_INCHES			75.0	// Size of robot to draw on the map
	#define ROBOT_BODY_WIDTH_TENTH_INCHES				75		// Width of robot main body
	#define ROBOT_BODY_LENGTH_TENTH_INCHES				75		// Length of robot main body
	#define ROBOT_ARM_WIDTH_TENTH_INCHES				 1		// Amount that each Arm sticks out from the sides of the robot main body
	#define ROBOT_BODY_WITH_ARMS_WIDTH_TENTH_INCHES		(ROBOT_BODY_WIDTH_TENTH_INCHES + (ROBOT_ARM_WIDTH_TENTH_INCHES * 2))	// Size of robot, for calculating if it can fit between objects!

	#define HALF_ROBOT_BODY_WIDTH_TENTH_INCHES				(ROBOT_BODY_WIDTH_TENTH_INCHES/2)
	#define HALF_ROBOT_BODY_LENGTH_TENTH_INCHES				(ROBOT_BODY_LENGTH_TENTH_INCHES/2)
	#define HALF_ROBOT_BODY_WITH_ARMS_WIDTH_TENTH_INCHES	(HALF_ROBOT_BODY_WIDTH_TENTH_INCHES + ROBOT_ARM_WIDTH_TENTH_INCHES)
	#define HALF_ROBOT_CLAW_WIDTH_TENTH_INCHES				20

	// Sensor Zones for object avoidance
	#define FRONT_ZONE_THREAT_MIN_THRESHOLD		 120	// Tenth_Inches - Always avoid objects closer then this on the front
	#define ARM_ZONE_THREAT_MIN_THRESHOLD		  30	// Tenth_Inches - Always avoid objects closer then this on the front
	#define SIDE_THREAT_MIN_THRESHOLD			  40	// Tenth_Inches - avoid objects closer then this on the side
	#define SIDE_THREAT_NORM_THRESHOLD			 100	// Tenth_Inches - avoid objects closer then this on the side
	#define THREAT_PERSISTANCE_TIME				 50	// (1/10 sec) - how long to persist until allowing wider side avoidance behavior

	#define FRONT_CENTER_ZONE_EDGE_TENTH_INCHES				HALF_ROBOT_BODY_WIDTH_TENTH_INCHES // 2 zones in front center of robot
	#define FRONT_ARM_ZONE_EDGE_TENTH_INCHES				(FRONT_CENTER_ZONE_EDGE_TENTH_INCHES+ROBOT_ARM_WIDTH_TENTH_INCHES) // zone in front of each arm
	#define FRONT_SIDE_ZONE_WIDTH_TENTH_INCHES				90 // look for objects to avoid this far to either side of the robot
	#define FRONT_SIDE_ZONE_EDGE_TENTH_INCHES				(FRONT_ARM_ZONE_EDGE_TENTH_INCHES+FRONT_SIDE_ZONE_WIDTH_TENTH_INCHES)

	#define AVOIDANCE_SHARP_TURN_RANGE			  40 // Tenth_Inches - if front object closer then this, turn sharply

	// Head measurements (for Camera positioning)
	#define HEAD_IR_OFFSET_FROM_FRONT_TENTH_INCHES	 70	// Distance from Head IR sensors to front of robot
	#define LASER_MS_COMMAND_BASE_LEN			13 // lengh of command NOT including "Number of Scans"

	#define CAMERA_CENTER_HEIGHT_ABOVE_GROUND_TENTH_INCHES		 370.0	// Tenth_Inches (not really correct - was fudged.. todo
	#define CAMERA_CENTER_DIST_FROM_ROBOT_FRONT_TENTH_INCHES	  80.0	// Tenth_Inches - distance to SERVO, not CAMERA! (8.5?)

	#define CAMERA_SERVO_HEIGHT_ABOVE_GROUND_TENTH_INCHES		 300.0	// Tenth_Inches 30.0
	#define CAMERA_SERVO_DISTANCE_FROM_FRONT_TENTH_INCHES		  80.0	// Tenth_Inches 9.2
	#define CAMERA_HEIGHT_ABOVE_CAMERA_SERVO_TENTH_INCHES		  20.0	// Tenth_Inches 7.6
	#define CAMERA_OFFSET_IN_FRONT_OF_SERVO_TENTH_INCHES		  10.0	// Tenth_Inches - Camera lense sits in front of servo

	// Laser position measurements
	#define LASER_MIN_OBJECT_HEIGHT_TENTH_INCHES		  		   8	// Tenth_Inches!
	#define LASER_HEIGHT_ABOVE_GROUND_TENTH_INCHES				 274.0	// Tenth_Inches - distance to laser beam
//	#define LASER_HEIGHT_ABOVE_SERVO_TENTH_INCHES				  10.0	// Tenth_Inches 
	#define LASER_DISTANCE_FROM_FRONT_TENTH_INCHES				  34.0	// Tenth_Inches - distance to laser front

	// Kinect position measurements
	#define KINECT_SLICE_MIN_OBJECT_HEIGHT_TENTH_INCHES	  		   4.0	// Tenth_Inches! - min height to be included in a slice (but 3D check is tighter)
	#define KINECT_HEIGHT_ABOVE_GROUND_TENTH_INCHES				 320.0	// Tenth_Inches - distance to Kinect sensor center
	#define KINECT_DISTANCE_FROM_FRONT_TENTH_INCHES				  75.0	// Tenth_Inches - distance to Kinect internal sensor

	// Degrees from Forward of each sensor
	// TODO! IR2,IR3 and US0 - Depends upon where the head is facing!  ASSUME forward for now!

	static int SensorOffsetDegrees_IR[NUM_IR_SENSORS] =
		//  0    1    2    3    4    5 		// IR Sensor Number
		{ -70, -10,  -5,   5,  10,  70 };	// Degrees from Forward

	static int SensorOffsetDegrees_US[NUM_US_SENSORS] = 
		//  0    1    2  					// UltraSonic Sensor Number
		{   0, -20,   20 };					// Degrees from Forward


////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                             SEEKER CAR-ROBOT
////////////////////////////////////////////////////////////////////////////////////////////////////////
#elif SENSOR_CONFIG_TYPE == SENSOR_CONFIG_CARBOT
	// Seeker CarBot Sensor Configuration:

	#define ROBOT_BODY_SENSOR_RADIUS_TENTH_INCHES	  40.0	// Tenth Inches. Most sensors are about 3-4 inches from center of robot body
	#define ROBOT_BODY_DRAW_RADIUS_TENTH_INCHES		  80.0	// Radius of robot to draw on the map


	// TODO-CAR-MUST!
	static int SensorOffsetDegrees_IR[NUM_IR_SENSORS] =
		//  0    1    2    3        		// IR Sensor Number
		{ -80,  -5,   5,  80};				// Degrees from Forward

	static int SensorOffsetDegrees_US[NUM_US_SENSORS] = 
		//  0      1    2    3    4    5    6		// UltraSonic Sensor Number (0=Camera mount)
		{   0,   -70, -42, -14,  14,  42,  70 };	// Degrees from Forward


#else
	#error SENSOR_CONFIG_TYPE
#endif // SENSOR_CONFIG_TYPE


#endif // __ROBOT_HARDWARE_CONFIG_DEFINED_H__