// Module.cpp
// This version modified for Loki.
// Removed most Seeker/Car specific functionality, such as speed control


#include "stdafx.h"
#include <math.h>
#include <MMSystem.h>	// For Sound functions
#include "Globals.h"
#include "module.h"
#include "thread.h"
#include "HardwareConfig.h"

// Global to this module (defined as extern in Module.h)
//int					gAutoNavigateMode = MODE_MANUAL_OVERRIDE;

///__itt_string_handle* pshMotorCommand = __itt_string_handle_createA("Motor Command, Speed, Turn = ");
__itt_string_handle* pshCase1 = __itt_string_handle_create("1 Case");
__itt_string_handle* pshCase2 = __itt_string_handle_create("2 Case");
__itt_string_handle* pshCase3 = __itt_string_handle_create("3 Case");
__itt_string_handle* pshCase4 = __itt_string_handle_create("4 Case");
__itt_string_handle* pshCase5 = __itt_string_handle_create("5 Case");
__itt_string_handle* pshCase6 = __itt_string_handle_create("6 Case");
__itt_string_handle* pshCase7 = __itt_string_handle_create("7 Case");
__itt_string_handle* pshCase8 = __itt_string_handle_create("8 Case");
__itt_string_handle* pshCase9 = __itt_string_handle_create("9 Case");


///////////////////////////////////////////////////////////////////////////////
// Base Class
CRobotModule::CRobotModule()
{
	m_pCurrentSegment = NULL;
	m_pCurrentWaypoint = NULL;

}

void CRobotModule::DisplayStatus( LPCTSTR lpszError )
{

	CString strStatus;
		strStatus.Format( "Module: %s" , lpszError );
	ROBOT_DISPLAY( TRUE, (LPCTSTR)strStatus )

	ROBOT_LOG( TRUE, "Module: DisplayStatus: %s\n",lpszError )
}

void CRobotModule::SendHardwareCmd(WORD Request, DWORD wParam, DWORD lParam)
{
	// Send a command to the right control hardware:
	//   ER1 Motor Control, Servo controller, or PIC

	// See if this is a Motor command
	if( (HW_SET_MOTOR_STOP == Request)		||
		(HW_SET_MOTOR_BRAKE == Request)		||
		(HW_SET_SPEED_AND_TURN == Request)	||
		(HW_SET_TURN == Request)			||
		(HW_SET_SPEED == Request) )	
	{
		// Motor command.

		if( SIMULATED_SIO_HANDLE == g_hPicCommPort )
		{
			// Pic is not connected, simulate as if it were
			// This allows offline debug, regardless of actual hardware type
			#if MOTOR_CONTROL_TYPE == ER1_MOTOR_CONTROL
				// Special case ER1 to allow motor control testing, even with no PIC!
				if( INVALID_HANDLE_VALUE == g_hMotorCommPort )
				{
					// Pic is not connected, simulate as if it were
					PIC_CMD_T	SimulatedPicCmd;
					SimulatedPicCmd.Cmd = Request;
					SimulatedPicCmd.Param1 = wParam;
					SimulatedPicCmd.Param2 = lParam;
					SimulatePic( SimulatedPicCmd );
				}
			#else
				// all other hardware configurations
					// Pic is not connected, simulate as if it were
					PIC_CMD_T	SimulatedPicCmd;
					SimulatedPicCmd.Cmd = (BYTE)Request;
					SimulatedPicCmd.Param1 = (BYTE)wParam;
					SimulatedPicCmd.Param2 = (BYTE)lParam;
					SimulatePic( SimulatedPicCmd );
			#endif
			return;
		}



		// Not Simulated.  Send to the hardware that is controlling the motors
		#if MOTOR_CONTROL_TYPE == PIC_CONTROLLER
		
			// Convert from Logical Speed/Turn to Hardware Speed/Turn values
			HWSpeed = SPEED_SERVO_CENTER + m_Speed;	 
			HWTurn = TURN_SERVO_CENTER - m_Turn;	// servo is reversed

			// Send the command to the PIC Comm Write Thread
			PostThreadMessage( g_dwPicCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			return;

		#elif MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL	

			if( HW_SET_MOTOR_STOP == Request )
			{
				// Send the STOP command.
				PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
				return;
			}
			else if( HW_SET_MOTOR_BRAKE == Request )
			{
				// Handle the BRAKE command.
				// wParam = How hard to brake
				// lParam = How long to brake
				// Set a timer to time how long to Brake for
				gBrakeTimer = lParam;

				// Send the Brake command to the Servo controller.
				wParam += SPEED_SERVO_CENTER;
				PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+HW_SET_SPEED), wParam, 0 );
				return;
			}
			else if( HW_SET_SPEED_AND_TURN == Request )
			{
				ROBOT_ASSERT(0); // HW_SET_SPEED_AND_TURN NOT USED!
				// Break into two separate commands:
				// Handle Turn - Convert from Logical Turn to Hardware Turn value
				wParam -= TURN_SERVO_CENTER;	// Turn servo is reverse of Speed servo
				// Send this command to an external servo controller via the ServoComm Write Thread
				PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+HW_SET_TURN), wParam, 0 );

				// Handle Speed - Convert from Logical Speed to Hardware Speed value
				lParam += SPEED_SERVO_CENTER;
				// Send this command to an external servo controller via the ServoComm Write Thread
				PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+HW_SET_SPEED), lParam, 0 ); // lParam is correct in this case
			}
			else if( HW_SET_TURN == Request )
			{
				// Handle Turn - Convert from Logical Turn to Hardware Turn value
				wParam -= TURN_SERVO_CENTER;	// Turn servo is reverse of Speed servo
				ROBOT_LOG( MOTOR_DBG, "Execute: New Turn = %d, Ack Turn = %d\n", m_Turn, m_LastMotorTurn )
				// Send this command to an external servo controller via the ServoComm Write Thread
				PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			}
			else if( HW_SET_SPEED == Request )
			{
				// Handle Speed - Convert from Logical Speed to Hardware Speed value
				wParam += SPEED_SERVO_CENTER;
				// Send this command to an external servo controller via the ServoComm Write Thread
				PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			}
			else
			{
				ROBOT_ASSERT(0); // BAD MOTOR COMMAND
			}
			return;

		#elif MOTOR_CONTROL_TYPE == ER1_MOTOR_CONTROL

			// Send this command to an ER1 controller via the MotorCommThreadFunc
			if( HW_SET_MOTOR_BRAKE == Request )
			{
				// ER1 does not support the BRAKE command
				PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+HW_SET_MOTOR_STOP), 0, 0 );
			}
			else
			{
				PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			}
			return;

		#elif MOTOR_CONTROL_TYPE == POLOLU_TREX_MOTOR_CONTROL
			// Send the command to the Motor Comm Write Thread
			PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			return;

		///////////////////////////////////////////////////////////////////////////////
		#elif MOTOR_CONTROL_TYPE == IROBOT_MOTOR_CONTROL

			PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			return;

		#else
			#error BAD MOTOR_CONTROL_TYPE
		#endif

		return;
	}  // (if Motor command)


	// See if this is a Camera command
	if( (Request >= HW_CAMERA_MIN_COMMAND) && (Request <= HW_CAMERA_MAX_COMMAND) )
	{
		// Camera command.
		#if CAMERA_CONTROL_TYPE == PIC_SERVO_CONTROL	
			// Send the command to the PIC Comm Write Thread
			PostThreadMessage( g_dwPicCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );

		#elif CAMERA_CONTROL_TYPE == EXTERN_SERVO_CONTROL	
			// Send this command to an external servo controller via the ServoComm Write Thread
			PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );

		#elif CAMERA_CONTROL_TYPE == DYNA_SERVO_CONTROL	
			// Send this command to Dynamixel Servos via the SmartServoCommThreadFunc Thread
			PostThreadMessage( g_dwSmartServoCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );

		#elif CAMERA_CONTROL_TYPE == SONY_SERIAL_CAMERA
			// Send this command to a Sony serial interface camera via the CameraCommThreadFunc
			// NOTE - this will change if using SONY PRO camera!
			PostThreadMessage( g_dwCameraCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
		#else
			#error BAD CAMERA_CONTROL_TYPE
		#endif	

		return;
	}

	// See if this is a Power command
	if( HW_SET_POWER_MODE == Request )
	{
		if( LAPTOP_POWER_MODE_DEFAULT == lParam )
		{
			// Leave laptop alone.  Do nothing
			// (Ignore POWER_MODE commands to Dynamixel servos, passed in wParam)
			return;
		}
	}

	if( HW_SET_KINECT_POWER == Request )
	{
		#if MOTOR_CONTROL_TYPE == IROBOT_MOTOR_CONTROL
		// On iRobot Create Base, Kinect power is controled through the iRobot
			PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
		#else
			ROBOT_LOG( TRUE, "ERROR!! : HW_SET_KINECT_POWER not supported on this Robot Type!\n" )
		#endif
		return;
	}

	if( HW_SET_DYNA_USB == Request )
	{
		#if MOTOR_CONTROL_TYPE == IROBOT_MOTOR_CONTROL
		// On iRobot Create Base, Dynamixel power is controled through the iRobot
			PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
		#endif
		return;
	}

	#if ROBOT_TYPE == TURTLE
		// On Turtle, LED Lights are controled through the iRobot base
		if( HW_SET_AUX_LIGHT_POWER == Request )
		{
			PostThreadMessage( g_dwMotorCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
			return;
		}
	#endif

	// See if this is some other SERVO command
	if( (Request >= HW_OTHER_SERVO_MIN_COMMAND) && (Request <= HW_OTHER_SERVO_MAX_COMMAND) )
	{
		// Some other Servo command (not motor or camera related)
		// Camera command.
		#if OTHER_SERVO_CONTROL_TYPE == PIC_CONTROLLER	
			// Send the command to the PIC Comm Write Thread
			PostThreadMessage( g_dwPicCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );

		#elif OTHER_SERVO_CONTROL_TYPE == EXTERN_SERVO_CONTROL	
			// Send this command to an external servo controller via the ServoComm Write Thread
			PostThreadMessage( g_dwServoCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );

		#elif OTHER_SERVO_CONTROL_TYPE == DYNA_SERVO_CONTROL	
			// Send this command to an Dynamixel and Kerr servos via the SmartServoComm Thread
			PostThreadMessage( g_dwSmartServoCommThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );
		#else
			#error BAD OTHER_SERVO_CONTROL_TYPE
		#endif

		return;
	}

	// Other command.  Send the command to the PIC Comm Write Thread
	// See if the command needs to be simulated
	if( SIMULATED_SIO_HANDLE == g_hPicCommPort )
	{
		// Pic is not connected, simulate as if it were
		PIC_CMD_T	SimulatedPicCmd;
		SimulatedPicCmd.Cmd = (BYTE)Request;
		SimulatedPicCmd.Param1 = (BYTE)wParam;
		SimulatedPicCmd.Param2 = (BYTE)lParam;
		SimulatePic( SimulatedPicCmd );
		return;
	}

	// Nope, real command to send to the PIC!
	PostThreadMessage( g_dwPicCommWriteThreadId, (WM_ROBOT_MESSAGE_BASE+Request), wParam, lParam );

}



/****
signed int CRobotModule::CalculateThreatDirection()
{
	// Calculate "average potential" of threats

	// Bumper collisions override other sensors
	if( CENTER_HIT == g_SensorStatus.Bumper  )
	{		
		return( 0 );	// Center Hit on Both bumpers
	}
	else if( LEFT_HIT == g_SensorStatus.Bumper )
	{		
		return( -10 );	// Hit on Left bumper (mostly in front, but slightly to one side)
	}
	else if( RIGHT_HIT == g_SensorStatus.Bumper )
	{		
		return( 10 );	// Hit on Right bumper
	}

	// New version, attempts to weight sensor based upon direction of robot
	// Front Sensors get priority

	int nClosestObjectLeft = 255;
	if( g_pSensorSummary->nClosestObjectFrontLeft < nClosestObjectLeft )
		nClosestObjectLeft = g_pSensorSummary->nClosestObjectFrontLeft;
	if( g_pSensorSummary->nClosestObjectSideLeft < nClosestObjectLeft )
		nClosestObjectLeft = g_pSensorSummary->nClosestObjectSideLeft;

	int nClosestObjectRight = 255;
	if( g_pSensorSummary->nClosestObjectFrontRight < nClosestObjectRight )
		nClosestObjectRight = g_pSensorSummary->nClosestObjectFrontRight;
	if( g_pSensorSummary->nClosestObjectSideRight < nClosestObjectRight )
		nClosestObjectRight = g_pSensorSummary->nClosestObjectSideRight;

	// Return the result.  Negative means *threat* to the Left
	return( nClosestObjectLeft - nClosestObjectRight );

}
****/

void CRobotModule::SpeakCannedPhrase( int PhraseToSpeak )
{

/**	// Play a wav file
	if( SOUND_STOP == Sound )	
	{
		// Stop playing
		DWORD dwOptions = 0;
		BOOL bSucess = PlaySound(
			NULL,			// Filename  
			NULL,			// No device Handle needed   
			dwOptions);		// Control Flags
		if( bSucess )
		{
			ROBOT_LOG( TRUE, "ERROR: Could not stop play sound!\n")
		}
		else
		{
			ROBOT_LOG( TRUE, "SpeakCannedPhrase: Stopped sound.\n")
		}
	}
	else
	{
	**/
//		PostThreadMessage( g_dwSoundThreadId, (WM_ROBOT_PLAY_SOUND), Sound, 0); 
		PostMessage( g_RobotSetupViewHWND, WM_ROBOT_SPEAK_TEXT, PhraseToSpeak, 0 );
	
}


///////////////////////////////////////////////////////////////////////////////
//	MODULES
///////////////////////////////////////////////////////////////////////////////
	// Message are sent to each module in order, highest priority first

///////////////////////////////////////////////////////////////////////////////
// Drive Control Arbitrator
///////////////////////////////////////////////////////////////////////////////
CDriveControlModule::CDriveControlModule()
{
	m_Command			= HW_SET_MOTOR_STOP;
	m_LastMotorCommand	= HW_SET_MOTOR_STOP;
	m_Speed				= SPEED_STOP;
	m_LastMotorSpeed	= SPEED_STOP;
	m_Turn				= TURN_CENTER;
	m_LastMotorTurn		= TURN_CENTER;
	m_BrakeLastTachometer = 0;

	m_SpeedRequested	= SPEED_STOP; 
	m_SpeedCurrentSetting = SPEED_STOP;
	m_TachometerTarget	= 0;
	m_MotorForward		= TRUE;
	m_UserOverrideMode	= FALSE;

	m_DistHigh = 0;
	m_DistLow = 0;
	m_DriveOwner = NO_MODULE;	// Which Module is currently in charge

	m_MoveDistanceRemaining = 0;
	m_TurnRotationRemaining = 0;
	m_StopAfterMoveDistance = TRUE;
	m_StopAfterTurnRotation = TRUE;
	m_ModulesSuppressed = NO_MODULE;	// All modules enabled

	gMotorSpeedTimer = 0;
	gBrakeTimer = 0;

}

///////////////////////////////////////////////////////////////////////////////
// Functions related to Pic Status
///////////////////////////////////////////////////////////////////////////////

/*
void CDriveControlModule::SetPicAck(int  Command, int  SpeedServoVal, int  TurnServoVal)
{
	// Called by Sensor Module. Provides lastest PIC status info
	// Pic returns absolute Servo values. Need to convert to logical values

	m_LastMotorCommand = Command;

	//TODO-SERVO fix this to match direction of servo travel
	m_LastMotorSpeed = SpeedServoVal - SPEED_SERVO_CENTER;
	m_LastMotorTurn = TurnServoVal  - TURN_SERVO_CENTER;
}
*/


void CDriveControlModule::UpdateTurnRotation(double RotationAngleAmount)
{
	// See if previously TurnRotation (rotate in place) has been completed
	// RotationAngleAmount is negative if turning left
	// m_TurnRotationRemaining is always positive (direction independent)
	if( m_TurnRotationRemaining > 361.0 )
	{
		ROBOT_ASSERT(0);
	}

	double PosRotationAngleAmount = fabs(RotationAngleAmount);

	if( 0 != m_TurnRotationRemaining )
	{
		// A SetTurnRotation was/is in progress
		m_TurnRotationRemaining = m_TurnRotationRemaining - PosRotationAngleAmount;
			ROBOT_LOG( TRUE, "DEBUG UpdateTurnRotation: Turn Amt = %f, Remaining = %f\n", 
				PosRotationAngleAmount, m_TurnRotationRemaining )

		if( m_TurnRotationRemaining <= TURN_ROTATION_COMPLETE_COMPENSATION )
		{
			// Turn completed!
			m_TurnRotationRemaining = 0;
			ROBOT_DISPLAY( TRUE, "SetTurnRotation Completed" )

			if( m_StopAfterTurnRotation )
			{
				m_Command = HW_SET_MOTOR_STOP;
				m_Speed = SPEED_STOP;
				m_Turn = TURN_CENTER;
				ExecuteCommand();
			}
		}
	}
}

void CDriveControlModule::UpdateMoveDistance(double OdometerUpdateTenthInches)
{
	// See if previously request move has been completed
	if( m_MoveDistanceRemaining > 0 )
	{
		// A SetMoveDistance was/is in progress
		m_MoveDistanceRemaining = m_MoveDistanceRemaining - fabs(OdometerUpdateTenthInches);
//		ROBOT_LOG( TRUE, "DEBUG UpdateMoveDistance: Move Amt = %f, Remaining = %d\n", 
//			fabs(OdometerUpdate), m_MoveDistanceRemaining )

		if( m_MoveDistanceRemaining <= MOVE_DISTANCE_COASTING_COMPENSATION )
		{
			// Move completed!
			m_MoveDistanceRemaining = 0;
			ROBOT_DISPLAY( TRUE, "SetMoveDistance Completed" )
			if( m_StopAfterMoveDistance )
			{
				m_Command = HW_SET_MOTOR_STOP;
				m_Speed = SPEED_STOP;
				ExecuteCommand();
			}
		}
	}
}

BOOL CDriveControlModule::RobotStopped()
{
	// See if the robot is moving or stopped

	#if MOTOR_CONTROL_TYPE == ER1_MOTOR_CONTROL		// For ER1 Robot
		if( g_SensorStatus.OdometerUpdateTenthInches  < 5.0 )	// TODO-MUST What value should be here?
		{
			ROBOT_LOG( TRUE, "DEBUG STOPPED???==========> %f, TRUE!\n", g_SensorStatus.OdometerUpdateTenthInches )
			return TRUE;
		}
		else
		{
			ROBOT_LOG( TRUE, "DEBUG STOPPED???==========> %f, FALSE!\n", g_SensorStatus.OdometerUpdateTenthInches )
			return FALSE;
		}
	#else
		if( (g_SensorStatus.Tachometer  < 2) && (0 == m_MoveDistanceRemaining) && (0 == m_TurnRotationRemaining) )
			return TRUE;
		else
			return FALSE;

	#endif

}




///////////////////////////////////////////////////////////////////////////////
// Commands to Motors
///////////////////////////////////////////////////////////////////////////////

void CDriveControlModule::Stop(int  Module)
{
	if( CheckAndSetOwner(Module) )
	{
		m_DriveOwner = Module;	
		m_Command = HW_SET_MOTOR_STOP;
		m_Speed = SPEED_STOP;
		m_MoveDistanceRemaining = 0;
		m_TurnRotationRemaining = 0;
		CString MsgString, ModuleName;
		ModuleNumberToName( Module, ModuleName );
		MsgString.Format( "%s: New Stop Command Requested" ,ModuleName );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		ExecuteCommand();
	}
}

void CDriveControlModule::Brake(int  Module)
{
	// Note: Once a Brake operation starts, the Pic won't abort until done.
	if( CheckAndSetOwner(Module) )
	{
		m_DriveOwner = Module;	
		m_Command = HW_SET_MOTOR_BRAKE;
		m_Speed = SPEED_STOP;			// After braking, speed set by PIC to Stop
		m_Turn = TURN_CENTER;			// And wheel centered
//		m_MoveDistanceRemaining = 0;
//		m_TurnRotationRemaining = 0;
		CString MsgString, ModuleName;
		ModuleNumberToName( Module, ModuleName );
		MsgString.Format( "%s: New Brake Command Requested",ModuleName );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		ExecuteCommand();
	}
}


void CDriveControlModule::SetTurnRotation( int  Module, int Speed, int Turn, int TurnAmountDegrees, BOOL StopAfterTurn )
{
	// Note: A SetTurnRotation can be over ridden by another command, which will abort the turn.
	int  AbsTurnAmountDegrees = abs(TurnAmountDegrees);
	
	if( CheckAndSetOwner(Module) )
	{
		CString MsgString, ModuleName;
		m_DriveOwner = Module;	
		m_Command = HW_SET_SPEED_AND_TURN;
		/*
		// TODO - Speed not currently used, assumes rotate in place.  add calc to allow complex turns?
		if( Speed >= 0 ) 
			m_Speed = Speed + g_SpeedIncrease;	// Forward
		else 
			m_Speed = Speed - g_SpeedIncrease;	// Reverse
		*/

		if( AbsTurnAmountDegrees > 360 )
		{
			ROBOT_ASSERT(0);
		}
		m_StopAfterTurnRotation = StopAfterTurn;
		m_Speed = 0;
		m_Turn = Turn;	// This sets the speed/tightness of the turn
		m_TurnRotationRemaining = AbsTurnAmountDegrees; // Non-Zero tells other modules that turn not completed.
		ModuleNumberToName( Module, ModuleName );
		MsgString.Format( "%s: New SetTurnRotation Requested: Speed:%d Turn %d RotationAmountDegrees %d",
			ModuleName, m_Speed, m_Turn, AbsTurnAmountDegrees );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		ExecuteCommand();
	}
}


void CDriveControlModule::SetMoveDistance( int  Module, int Speed, int Turn, int  DistanceTenthInches, BOOL StopAfterMove )
{
	// Note: A SetMoveDistance can be over ridden by another command, which will abort the move dist.
	if( CheckAndSetOwner(Module) )
	{
		if( Speed != 0 )
		{
			CString MsgString, ModuleName;
			m_DriveOwner = Module;	
			m_Command = HW_SET_SPEED_AND_TURN;
			if( Speed >= 0 ) 
				m_Speed = Speed + g_SpeedIncrease;	// Forward
			else 
				m_Speed = Speed - g_SpeedIncrease;	// Reverse

			m_StopAfterMoveDistance = StopAfterMove;
			m_Turn = Turn;
			m_MoveDistanceRemaining = DistanceTenthInches; // Non-Zero tells other modules that move not completed.
			ModuleNumberToName( Module, ModuleName );
			MsgString.Format( "%s: New SetMoveDistance Requested: Speed:%d Turn %d DistanceTenthInches %d",ModuleName, m_Speed, m_Turn, m_MoveDistanceRemaining );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			ExecuteCommand();
		}
		else
		{
			m_MoveDistanceRemaining = 0;
		}
	}
}

void CDriveControlModule::SetSpeed(int  Module, int Speed)
{
	if( CheckAndSetOwner(Module) )
	{
		CString MsgString, ModuleName;
		m_DriveOwner = Module;	
		m_Command = HW_SET_SPEED_AND_TURN;	
		// Set the speed.  Drive control keeps track of the last turn
		if( Speed >= 0 ) 
			m_Speed = Speed + g_SpeedIncrease;	// Forward
		else 
			m_Speed = Speed - g_SpeedIncrease;	// Reverse
		m_MoveDistanceRemaining = 0;
		m_TurnRotationRemaining = 0;
		ModuleNumberToName( Module, ModuleName );
		MsgString.Format( "%s: New Speed Requested: %d", ModuleName, Speed );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		ExecuteCommand();
	}
}
void CDriveControlModule::SetTurn(int  Module, int Turn)
{
	CString MsgString, ModuleName;
	if( CheckAndSetOwner(Module) )
	{
		m_DriveOwner = Module;
		m_Command = HW_SET_SPEED_AND_TURN;
		if( Turn < TURN_LEFT_MAX ) Turn = TURN_LEFT_MAX;	// Negative = Left
		if( Turn > TURN_RIGHT_MAX ) Turn = TURN_RIGHT_MAX;	// Positive = Right
		m_Turn = Turn ;	// Set the turn.  Drive control keeps track of the last speed
		m_MoveDistanceRemaining = 0;
		m_TurnRotationRemaining = 0;
		ModuleNumberToName( Module, ModuleName );
		MsgString.Format( "%s: New Turn Requested: %d",ModuleName, Turn );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		ExecuteCommand();
	}
}
void CDriveControlModule::SetSpeedAndTurn(int  Module, int Speed, int Turn)
{
	if( CheckAndSetOwner(Module) )
	{
		m_DriveOwner = Module;	
		m_Command = HW_SET_SPEED_AND_TURN;
		if( 0 == Speed ) 
			m_Speed = Speed; // Stop!
		else if( Speed > 0 ) 
			m_Speed = Speed + g_SpeedIncrease;	// Forward
		else if( Speed < 0 ) 
			m_Speed = Speed - g_SpeedIncrease;	// Reverse
		m_Turn = Turn;
		m_MoveDistanceRemaining = 0;
		m_TurnRotationRemaining = 0;
		CString MsgString, ModuleName;
		ModuleNumberToName( Module, ModuleName );
		MsgString.Format( "%s: New Speed: %d and Turn %d Requested",
			ModuleName, Speed, Turn );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		ExecuteCommand();
	}
}


BOOL CDriveControlModule::CheckAndSetOwner( int  Module )
{
	CString MsgString, ModuleName, OwnerName;

	if( (0 != (m_ModulesSuppressed & Module)) )// &&
//		(OVERRIDE_MODULE != Module) )				// Override Module can not be Suppressed!
	{
		// Module is suppressed
		ModuleNumberToName( Module, ModuleName );
		ModuleNumberToName( m_DriveOwner, OwnerName );
		MsgString.Format("Request by %s Rejected: Suppressed by %s",ModuleName, OwnerName );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		return FALSE;
	}

	if( Module < m_DriveOwner )
	{
		// Higher priority module has control
		ModuleNumberToName( Module, ModuleName );
		ModuleNumberToName( m_DriveOwner, OwnerName );
//		MsgString.Format( "Request by %s Rejected: %s has control",ModuleName, OwnerName );
//		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
		return FALSE;
	}

	// No one with higher priority has control.
	if( Module > m_DriveOwner )
	{
		ROBOT_DISPLAY( TRUE, "==================================================================")
		ModuleNumberToName( Module, ModuleName );
		ModuleNumberToName( m_DriveOwner, OwnerName );
		MsgString.Format( "Module %s has taken control from %s",ModuleName, OwnerName );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )

		m_DriveOwner = Module;

//		Beep(1000, 200);	// Beep to indicate Module change
	}

	// Requesting module has control now
	// Indicate on the GUI who is in control
	SendResponse( WM_ROBOT_DISPLAY_SINGLE_ITEM, ROBOT_RESPONSE_DRIVE_MODULE_OWNER, Module );
	return TRUE;

}

BOOL CDriveControlModule::IsOwner( int  Module )
{
	if( (0 != (m_ModulesSuppressed & Module)) ||
		(Module != m_DriveOwner ) )
	{
		// Not current owner
		return FALSE;
	}
	return TRUE;
}

BOOL CDriveControlModule::ReleaseOwner( int  Module )
{
	if( Module < m_DriveOwner )
	{
		// Higher priority module has control already so just ignore.
/*		CString MsgString, ModuleName, OwnerName;
		ModuleNumberToName( Module, ModuleName );
		ModuleNumberToName( m_DriveOwner, OwnerName );
		MsgString.Format( "Module %s releasing control, but %s already has it",ModuleName, OwnerName );
		ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
*/
		return FALSE;
	}

	// No one with higher priority has control.  Release Drive owner.
	CString MsgString, ModuleName;
	ModuleNumberToName( Module, ModuleName );
	MsgString.Format( "Module %s releasing control",ModuleName );
	ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
	ROBOT_DISPLAY( TRUE, "==================================================================")
	m_DriveOwner = NO_MODULE;
	return TRUE;

}

void CDriveControlModule::SuppressModule( int  Module )
{
	if( m_DriveOwner == Module )
	{
		// This module is currently the drive owner.  Release control then suppress it.
		ReleaseOwner( Module );
	}
	m_ModulesSuppressed |= Module;	// Set the flag
}

void CDriveControlModule::EnableModule( int  Module )
{
	m_ModulesSuppressed &= !Module;	// Clear the flag
}


///////////////////////////////////////////////////////////////////////////////
// ExecuteCommand - sends the actual command to the Servo controller as needed
//
void CDriveControlModule::ExecuteCommand()
{
	CString MsgString, OwnerName;


//////////////////////////////////////////////////////////////////////////////////////////
#if MOTOR_CONTROL_TYPE == PIC_CONTROLLER		// Use this if controlling servos from the PIC

	if( (m_LastMotorCommand != m_Command )	||
		(m_LastMotorSpeed != m_Speed )		||
		(m_LastMotorTurn != m_Turn )	)
	{
		// new command is pending, and needs to be sent
		if( m_LastMotorCommand != m_Command )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Cmd = %02X, Ack Cmd = %02X\n", m_Command, m_LastMotorCommand)
		if( m_LastMotorSpeed != m_Speed )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Speed = %d, Ack Speed = %d\n", m_Speed, m_LastMotorSpeed)
		if( m_LastMotorTurn != m_Turn )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Turn = %d, Ack Turn = %d\n", m_Turn, m_LastMotorTurn)
	
		if( HW_SET_MOTOR_BRAKE == m_Command )
		{
			// Send the BRAKE command.
			ROBOT_DISPLAY( TRUE, "Sending BRAKE command")
			SendHardwareCmd(HW_SET_MOTOR_BRAKE, SPEED_BRAKE_AMOUNT, SPEED_BRAKE_TIME);
			m_Turn = TURN_CENTER;	// Brake automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else if( HW_SET_MOTOR_STOP == m_Command )
		{
			// Send the STOP command.
			ROBOT_DISPLAY( TRUE, "Sending STOP command")
			SendHardwareCmd(HW_SET_MOTOR_STOP,0,0);
			m_Speed = SPEED_STOP;
		}
		else
		{
			// Regular Motor commands
			ModuleNumberToName( m_DriveOwner, OwnerName );
			MsgString.Format( "From %s Sending command: %d Speed = %d Turn = %d", 
				OwnerName, m_Command, (m_Speed), (m_Turn));
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			SendHardwareCmd( HW_SET_SPEED_AND_TURN, m_Speed, m_Turn );		
			//	SendHardwareCmd( HW_SET_SPEED, HWSpeed,0 );		
			//	SendHardwareCmd( HW_SET_TURN, HWTurn,0 );		
		}

		m_LastMotorCommand = m_Command;
		m_LastMotorSpeed = m_Speed;
		m_LastMotorTurn = m_Turn;

	}

//////////////////////////////////////////////////////////////////////////////////////////
#elif MOTOR_CONTROL_TYPE == POLOLU_TREX_MOTOR_CONTROL

	if( (m_LastMotorCommand != m_Command )	||
		(m_LastMotorSpeed != m_Speed )		||
		(m_LastMotorTurn != m_Turn )	)
	{
		int commandparams[3];
		commandparams[0] =  m_Command;
		commandparams[1] =  m_Speed;
		commandparams[2] =  m_Turn;
		///__itt_marker(pDomainMotorControl, __itt_null, pshMotorCommand, __itt_marker_scope_task);
		///__itt_metadata_add(pDomainMotorControl, __itt_null, pshMotorCommand, __itt_metadata_s32, 3, (void*)&commandparams);

		// new command is pending, and needs to be sent
		if( m_LastMotorCommand != m_Command )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Cmd = %02X, Ack Cmd = %02X\n", m_Command, m_LastMotorCommand )
		if( m_LastMotorSpeed != m_Speed )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Speed = %d, Ack Speed = %d\n", m_Speed, m_LastMotorSpeed )
		if( m_LastMotorTurn != m_Turn )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Turn = %d, Ack Turn = %d\n", m_Turn, m_LastMotorTurn )

		if( HW_SET_MOTOR_BRAKE == m_Command )
		{
			// Send the BRAKE command.
			ROBOT_DISPLAY( TRUE, "Sending STOP/BRAKE command")
			SendHardwareCmd(HW_SET_MOTOR_BRAKE, 0, 0);
			m_Turn = TURN_CENTER;	// Brake or Stop automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else if( HW_SET_MOTOR_STOP == m_Command )
		{
			// Send the STOP command.
			ROBOT_DISPLAY( TRUE, "Sending STOP command")
			SendHardwareCmd(HW_SET_MOTOR_STOP, 0, 0);
			m_Turn = TURN_CENTER;	// Brake or Stop automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else
		{
			// Regular Motor commands
			ModuleNumberToName( m_DriveOwner, OwnerName );
			MsgString.Format( "From %s Sending command: %d Speed = %d Turn = %d", 
				OwnerName, m_Command, (m_Speed), (m_Turn));
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			SendHardwareCmd( HW_SET_SPEED_AND_TURN, m_Speed, m_Turn );		
		}

		m_LastMotorCommand = m_Command;
		m_LastMotorSpeed = m_Speed;
		m_LastMotorTurn = m_Turn;

	}

//////////////////////////////////////////////////////////////////////////////////////////
#elif MOTOR_CONTROL_TYPE == IROBOT_MOTOR_CONTROL

	if( (m_LastMotorCommand != m_Command )	||
		(m_LastMotorSpeed != m_Speed )		||
		(m_LastMotorTurn != m_Turn )	)
	{
		int commandparams[3];
		commandparams[0] =  m_Command;
		commandparams[1] =  m_Speed;
		commandparams[2] =  m_Turn;
		///__itt_marker(pDomainMotorControl, __itt_null, pshMotorCommand, __itt_marker_scope_task);
		///__itt_metadata_add(pDomainMotorControl, __itt_null, pshMotorCommand, __itt_metadata_s32, 3, (void*)&commandparams);

		// new command is pending, and needs to be sent
		if( m_LastMotorCommand != m_Command )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Cmd = %02X, Ack Cmd = %02X\n", m_Command, m_LastMotorCommand )
		if( m_LastMotorSpeed != m_Speed )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Speed = %d, Ack Speed = %d\n", m_Speed, m_LastMotorSpeed )
		if( m_LastMotorTurn != m_Turn )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Turn = %d, Ack Turn = %d\n", m_Turn, m_LastMotorTurn )

		if( HW_SET_MOTOR_BRAKE == m_Command )
		{
			// Send the BRAKE command.
			ROBOT_DISPLAY( TRUE, "Sending STOP/BRAKE command")
			SendHardwareCmd(HW_SET_MOTOR_BRAKE, 0, 0);
			m_Turn = TURN_CENTER;	// Brake or Stop automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else if( HW_SET_MOTOR_STOP == m_Command )
		{
			// Send the STOP command.
			ROBOT_DISPLAY( TRUE, "Sending STOP command")
			SendHardwareCmd(HW_SET_MOTOR_STOP, 0, 0);
			m_Turn = TURN_CENTER;	// Brake or Stop automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else
		{
			// Regular Motor commands
			ModuleNumberToName( m_DriveOwner, OwnerName );
			MsgString.Format( "From %s Sending command: %d Speed = %d Turn = %d", 
				OwnerName, m_Command, (m_Speed), (m_Turn));
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			SendHardwareCmd( HW_SET_SPEED_AND_TURN, m_Speed, m_Turn );		
		}

		m_LastMotorCommand = m_Command;
		m_LastMotorSpeed = m_Speed;
		m_LastMotorTurn = m_Turn;

	}

//////////////////////////////////////////////////////////////////////////////////////////
#elif MOTOR_CONTROL_TYPE == ER1_MOTOR_CONTROL		// For ER1 Robot

	if( (m_LastMotorCommand != m_Command )	||
		(m_LastMotorSpeed != m_Speed )		||
		(m_LastMotorTurn != m_Turn )	)
	{
		// new command is pending, and needs to be sent
		if( m_LastMotorCommand != m_Command )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Cmd = %02X, Ack Cmd = %02X\n", m_Command, m_LastMotorCommand )
		if( m_LastMotorSpeed != m_Speed )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Speed = %d, Ack Speed = %d\n", m_Speed, m_LastMotorSpeed )
		if( m_LastMotorTurn != m_Turn )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Turn = %d, Ack Turn = %d\n", m_Turn, m_LastMotorTurn )

		if( HW_SET_MOTOR_BRAKE == m_Command )
		{
			// TODO-ER1: Send the BRAKE command. - For now, this is just a stop command!!
			ROBOT_DISPLAY( TRUE, "Sending STOP/BRAKE command")
			SendHardwareCmd(HW_SET_MOTOR_STOP, 0, 0);
			m_Turn = TURN_CENTER;	// Brake or Stop automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else if( HW_SET_MOTOR_STOP == m_Command )
		{
			// Send the STOP command.
			ROBOT_DISPLAY( TRUE, "Sending STOP command")
			SendHardwareCmd(HW_SET_MOTOR_STOP, 0, 0);
			m_Turn = TURN_CENTER;	// Brake or Stop automaticaly sets the turn to center
			m_Speed = SPEED_STOP;	// and the speed to Stop when done		
		}
		else
		{
			// Regular Motor commands
			ModuleNumberToName( m_DriveOwner, OwnerName );
			MsgString.Format( "From %s Sending command: %d Speed = %d Turn = %d", 
				OwnerName, m_Command, (m_Speed), (m_Turn));
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			SendHardwareCmd( HW_SET_SPEED_AND_TURN, m_Speed, m_Turn );		
		}

		m_LastMotorCommand = m_Command;
		m_LastMotorSpeed = m_Speed;
		m_LastMotorTurn = m_Turn;

	}
//////////////////////////////////////////////////////////////////////////////////////////
#elif MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL		// For CarBot

	static BOOL bFirstSpeedCommand = TRUE;

	// TODO!!! TEST Brake, Speed Control, and Move set distance commands!

	// Handle Turn
	if( m_LastMotorTurn != m_Turn )
	{
		// new turn command is pending, and needs to be sent
		ROBOT_LOG( MOTOR_DBG, "Execute: New Turn = %d, Ack Turn = %d\n", m_Turn, m_LastMotorTurn )
		SendHardwareCmd( HW_SET_TURN, m_Turn,0 );		
		m_LastMotorTurn = m_Turn;
	}

	if( 0 != gBrakeTimer )
	{
		// Currently Braking.  Ignore any pending speed command (it will get handled 
		// next time, since it stays in m_Speed until overridden)
		BrakeControl();
		return;
	}

	// Handle Speed
	if( (m_LastMotorCommand != m_Command )	||
		(m_LastMotorSpeed != m_Speed )		)
	{
		// New Speed command

		if( m_LastMotorCommand != m_Command )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Cmd = %02X, Ack Cmd = %02X\n", m_Command, m_LastMotorCommand )
		if( m_LastMotorSpeed != m_Speed )
			ROBOT_LOG( MOTOR_DBG, "Execute: New Speed = %d, Ack Speed = %d\n", m_Speed, m_LastMotorSpeed )

		if( HW_SET_MOTOR_STOP == m_Command )
		{
			// Send the STOP command.
			ModuleNumberToName( m_DriveOwner, OwnerName );
			MsgString.Format( "From %s Sending STOP Command", OwnerName);
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			m_Speed = SPEED_STOP;
			m_SpeedRequested = SPEED_STOP;	 
			m_SpeedCurrentSetting = SPEED_STOP;
			SendHardwareCmd(HW_SET_MOTOR_STOP, 0, 0);
		}
		else if( HW_SET_MOTOR_BRAKE == m_Command )
		{
			// Send the Brake command first
			SendHardwareCmd(HW_SET_MOTOR_BRAKE, SPEED_BRAKE_AMOUNT, SPEED_BRAKE_TIME);
			m_Speed = SPEED_STOP;			// The speed will be Stop when done		
			m_SpeedRequested = SPEED_STOP;	 
			m_SpeedCurrentSetting = SPEED_STOP;	// for Speed control (disabled while braking)
			m_BrakeLastTachometer = MAX_INT;
			m_TachometerTarget = 0;

			// Then automatically set turn to center
			ROBOT_DISPLAY( TRUE, "Sending BRAKE command")
			m_Turn = TURN_CENTER;	
			m_LastMotorTurn = m_Turn;
			SendHardwareCmd( HW_SET_TURN, m_Turn,0 );		
		}
		else
		{
			// Regular Motor commands - could be a Joystick type Stop, though

			// on first real motor command, make sure the car is in gear!
			if( bFirstSpeedCommand && (0 != m_Speed) )		
			{
				bFirstSpeedCommand = FALSE;
				SendHardwareCmd( HW_SET_GEAR, 0, LOW_GEAR );	// Servo position for Low
				Sleep(500);	// give time for the servo thread to put in gear
			}

			ModuleNumberToName( m_DriveOwner, OwnerName );
			MsgString.Format( "From %s Sending command: %d Speed = %d", OwnerName, m_Command, (m_Speed) );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )

			m_SpeedRequested = m_Speed;	 
			m_SpeedCurrentSetting = m_SpeedRequested;	
			// TODO-LOKI?? m_SpeedCurrentSetting will be modified as needed by speed control
			SendHardwareCmd( HW_SET_SPEED, m_SpeedRequested,0 );	

			// Set direction flag used by Odometer, 
			// Assumes controling module will first stop before changing direction
			if( m_Speed < 0 )
				m_MotorForward = FALSE;
			else
				m_MotorForward = TRUE;

			// Set the target Tachometer reading for the speed control.
			m_TachometerTarget = SetTachometerTarget( m_Speed );

			gMotorSpeedTimer = NEW_SPEED_SET_TIME;		// give time for motors to respond before checking again

		}

		m_LastMotorCommand = m_Command;
		m_LastMotorSpeed = m_Speed;
	}
	else
	{
		// No new Speed command.  Call the SpeedControl function to adjust speed servo as needed 
		//TODO-CAR-MUST! ENABLE THIS, after filling in the values for  SpeedToTachTable
		//				SpeedControl();
	}
	
#else
	#error BAD MOTOR_CONTROL_TYPE
#endif
//////////////////////////////////////////////////////////////////////////////////////////

}

///////////////////////////////////////////////////////////////////////////////
// Speed Control
// Only used on PC when using an external servo controller!
// Handles Speed control, Brake, and Move Set Distance.

///////////////////////////////////////////////////////////////////////////////


// TODO-CAR-MUST - fill in these values from observed Tach values at various speeds!
const int SpeedToTachTable[] =
//  0    1    2    3    4    5    6    7    8    9   10   11   12   13  14  15		// Target Speed
{   0,   0,   0,   10, 15,  17,  19,  30,  34,  36,  38,  40,  42,  44, 46, 48 };	// Target Tach value


int  CDriveControlModule::SetTachometerTarget( int TargetSpeed )
{
	// given a target Speed, return the corresponding target Tachometer value

	int  Tach = 0;
	int  Speed = abs( TargetSpeed );

	for( int index=0; index <= 15; index++ )	// 
	{
		if( Speed >= SpeedToTachTable[index]  )
		{
			Tach = index * 3;	// TODO-CAR-MUST supply the right multiplier here!
			return Tach;
		}
	}
	return Tach;
}


void CDriveControlModule::BrakeControl()
{
	if( 0 != gBrakeTimer )	// 100ms timer count down in globals.cpp
	{
		// Braking in progress!
		if( (g_SensorStatus.Tachometer < 4)							||	// Stopped, or nearly so
			(g_SensorStatus.Tachometer > (m_BrakeLastTachometer + 2)) )	// No longer slowing down!  Probably going in reverse!
		{
			// Note the fudge factor applied to avoid thinking braking is done too early
			// Stop the motor and indicate that Braking is done.

			ROBOT_DISPLAY( TRUE, "BrakeControl: Braking Complete." )
			m_Speed = SPEED_STOP;
			m_SpeedRequested = SPEED_STOP;	 
			m_SpeedCurrentSetting = SPEED_STOP;
			m_BrakeLastTachometer = 0;
			m_TachometerTarget = 0;
			gBrakeTimer = 0;
			SendHardwareCmd(HW_SET_MOTOR_STOP, 0, 0);
		}
		else if(g_SensorStatus.Tachometer < m_BrakeLastTachometer)
		{
			// Still slowing down, continue to monitor motor speed
			m_BrakeLastTachometer = g_SensorStatus.Tachometer;
		}
	}
}


void CDriveControlModule::SpeedControl()
{
	// This function called from ExecuteCommand(), which is called every time 
	// a WM_ROBOT_SENSOR_STATUS_READY update is processed.
	// However, it is NOT called when a new command get issued.

	// Member functtions used: m_Command, m_Speed, m_Turn?
	//		m_LastMotorCommand = m_Command;
	//		m_LastMotorSpeed = m_Speed;
	//		m_LastMotorTurn = m_Turn;
#if MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL
// TODO-LOKI ---> MOTOR_CONTROL_TYPE == POLOLU_TREX_MOTOR_CONTROL

	// None of this is used if ER1 Pilot control or PIC is being used for motor control.
	
	// #define SPEED_STOP				 	  0		//
	// #define SPEED_FULL_FWD				125		// WARNING! if you swap these, update m_CurrentSpeed in Module.cpp!!!
	// #define SPEED_FULL_REV			  (-125)

	///////////////////////////////////////////////////////////////////////////////
	// Calculate Speed Control feedback for the motors, and adjust speed
	// (Moved from PIC code)

	// Check if Power is enabled with the RC "dead man switch" control
	// Note: to debug speed control, disable the this test
	if( !(g_SensorStatus.StatusFlags & HW_STATUS_RC_BUTTON_PWR_ENABLE) )
	{
		// RC Power Button not enabled.  Delay and disable speed control.
		// Otherwise, causes a speed surge when the power is restored again
		gMotorSpeedTimer = NEW_SPEED_SET_TIME * 2;// give time for motors to respond after power is restored
		return;
	}

	// See if we need to wait for a delay to complete
	if( gBrakeTimer != 0 ) 
	{
		return;
	}

	if( gMotorSpeedTimer != 0 ) 
	{
		return;
	}



	if( (0 != m_TachometerTarget) && (0 != m_SpeedRequested)	&& 
		(HW_SET_MOTOR_STOP != m_Command) &&	(HW_SET_MOTOR_BRAKE != m_Command) )
	{
		// Requested speed not zero, and not braking, so perform motor speed control.

		if( g_SensorStatus.Tachometer < m_TachometerTarget-1 )
		{
			// Going too slow
			if( m_MotorForward )
			{
				if( m_SpeedCurrentSetting < (m_SpeedRequested + MAX_SPEED_CORRECTION) )
				{
					m_SpeedCurrentSetting++;	// Speed up forward
				}
			}
			else
			{
				if( m_SpeedCurrentSetting > (m_SpeedRequested - MAX_SPEED_CORRECTION) )
				{
					m_SpeedCurrentSetting--;	// Speed up backward
				}
			}
			SendHardwareCmd( HW_SET_SPEED, m_SpeedCurrentSetting,0 );	
			gMotorSpeedTimer = SPEED_CHANGE_TIME;		// give time for motors to respond before checking again
		}
		else if( g_SensorStatus.Tachometer > m_TachometerTarget+1 )
		{
			// Going too fast
			if( m_MotorForward )
			{
//						if( SpeedServo > HW_SPEED_STOP )	// don't overshoot center!
				if( ( m_SpeedRequested >= (SPEED_SERVO_CENTER+SPEED_FWD_SLOW) ) &&
					( m_SpeedCurrentSetting > (SPEED_SERVO_CENTER+SPEED_FWD_SLOW) ) )	// ??don't cycle on and off!
				{
					m_SpeedCurrentSetting--;	// Slow down forward
				}
			}
			else
			{
//						if( SpeedServo < HW_SPEED_STOP )	// don't overshoot center!
				if( (m_SpeedRequested <= (SPEED_SERVO_CENTER-SPEED_REV_SLOW)) &&
					(m_SpeedCurrentSetting < (SPEED_SERVO_CENTER-SPEED_REV_SLOW)) )	// ??don't cycle on and off!
				{
					m_SpeedCurrentSetting++;	// Slow down backward
				}
			}
			SendHardwareCmd( HW_SET_SPEED, m_SpeedCurrentSetting,0 );	
			gMotorSpeedTimer = SPEED_CHANGE_TIME;		// give time for motors to respond before checking again
		}
	}

	// Note for Set Move distance, see UpdateMoveDistance()

#endif //MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL
}


///////////////////////////////////////////////////////////////////////////////
//	MODULE: SystemModule

CSystemModule::CSystemModule( CDriveControlModule *pDriveControlModule )
{
	m_pDriveCtrl = pDriveControlModule;
}

CSystemModule::~CSystemModule()
{
		ROBOT_LOG( TRUE, "~CSystemModule()\n" )
}

void CSystemModule::ProcessMessage(
		UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	IGNORE_UNUSED_PARAM (wParam);

	switch( uMsg )  
	{
		case WM_ROBOT_SENSOR_STATUS_READY:
		{
			g_bCmdRecognized = TRUE;

			// Check for any errors
			switch( g_SensorStatus.LastError )
			{
				case HW_PIC_NO_ERROR:
				{
					// Nothing to do
				}
				break;

				case HW_SERIAL_CMD_BUFFER_OVERFLOW:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERIAL_CMD_BUFFER_OVERFLOW\n" )
				}
				break;

				case HW_SERIAL_CMD_NOT_READY:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERIAL_CMD_NOT_READY\n" )
				}
				break;

				case HW_SERIAL_CMD_INVALID:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERIAL_CMD_INVALID\n" )
				}
				break;

				case HW_SERIAL_CMD_CHECKSUM_ERROR:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERIAL_CMD_CHECKSUM_ERROR\n" )
				}
				break;

				case HW_SERIAL_EXTRA_CHARS:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERIAL_EXTRA_CHARS\n" )
				}
				break;

				case HW_BAD_STATE:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_BAD_STATE\n" )
				}
				break;

				case HW_UNKNOWN_CMD:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_UNKNOWN_CMD\n" )
				}
				break;

				case HW_CMD_NOT_IMPLEMENTED:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_CMD_NOT_IMPLEMENTED\n" )
				}
				break;

				case HW_SERIAL_BAD_SYNC:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERIAL_BAD_SYNC\n" )
				}
				break;

				case HW_SERIAL_WAIT_PRIOR_CMD:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Warning: HW_SERIAL_WAIT_PRIOR_CMD\n" )
				}
				break;

				case HW_SERVO_VALUE_ERROR:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: HW_SERVO_VALUE_ERROR\n" )
				}
				break;

				case HW_RADAR_SCAN_BAD_STATE:
				{
					ROBOT_DISPLAY( TRUE, ">>>  PIC Error: IR_SCAN_BAD_STATE\n" )
				}
				break;

				case HW_BAD_ISR:
				{
					// This is a CRITICAL ERROR!
					ROBOT_DISPLAY( TRUE, ">>>  CRITICAL PIC Error!! : HW_BAD_ISR\n" )
				}
				break;

				default:
				{
					CString MsgString;
					MsgString.Format( ">>>  PIC Error: UNKNOWN ERROR: %02Xh", g_SensorStatus.LastError );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString );
				}
			}	// end switch( g_SensorStatus.LastError )

 

			////////////////////////////////////////////////////////////////////////////
			// All status sent as one big bulk structure
			// Post message to the GUI display (local or remote)
			SendResponse( WM_ROBOT_DISPLAY_BULK_ITEMS,	// command
				ROBOT_RESPONSE_PIC_STATUS,				// Param1 = Bulk data command
				0 );									// Param2 = not used
			return;
		}
		case WM_ROBOT_REQUEST_VERSION_CMD:
		{
			g_bCmdRecognized = TRUE;
			SendHardwareCmd(HW_GET_VERSION, 0, 0);
			return;
		}

		// Used only in Simulation.  During real run, this command is sent
		// directly to the hardware by the TimerThreadProc in Globls.cpp
		case WM_ROBOT_REQUEST_STATUS_CMD:
		{
			g_bCmdRecognized = TRUE;
			SendHardwareCmd(HW_GET_STATUS, 0, 0);
			return;
		}

		case WM_ROBOT_PIC_VERSION_READY:
		{
			g_bCmdRecognized = TRUE;
			CString VersionInfo;
			if( lParam != PIC_CODE_VERSION )
			{
				if( 99 == lParam )
				{
					VersionInfo.Format( "PIC Version 99 - SIMULATION" );
				}
				else
				{
					ROBOT_LOG( TRUE, "\n===========================================\n")
					ROBOT_LOG( TRUE, "               VERSION ERROR!\n")
					ROBOT_LOG( TRUE, "===========================================\n")
					VersionInfo.Format( "PIC VERSION ERROR!  Expected: %d, Received: %d", PIC_CODE_VERSION, lParam );
					SpeakCannedPhrase( SPEAK_PIC_VERSION_ERROR );
					AfxMessageBox( _T("PIC VERSION ERROR!") );
				}
			}
			else
			{
				VersionInfo.Format( "PIC S/W Version: %d", lParam );
				SpeakCannedPhrase( SPEAK_PIC_CONNECTED );
			}
			ROBOT_DISPLAY( TRUE, (LPCTSTR)VersionInfo )
			SendResponse( WM_ROBOT_DISPLAY_SINGLE_ITEM, ROBOT_RESPONSE_PIC_VERSION, lParam );
			SendCommand( WM_ROBOT_SET_LED_EYES_CMD, (DWORD)LED_EYES_BLINK, 0 ); // Turn on eyes after version received. (gives time for pic sync)

			return;
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
//	MODULE: UserCmdModule

CUserCmdModule::CUserCmdModule( CDriveControlModule *pDriveControlModule )
{
	m_pDriveCtrl = pDriveControlModule;
	m_CurrentSpeed = SPEED_STOP;
	m_CurrentTurn = TURN_CENTER;
	m_VidCapProcessingEnabled = FALSE;
	m_IRTrackingEnabled = FALSE;
	m_UserOwnerRequested = FALSE;
	#if ( ROBOT_SERVER == 1 ) //////////////// SERVER ONLY //////////////////
		m_pArmControlRight = new ArmControl( RIGHT_ARM );
		m_pArmControlLeft = new ArmControl( LEFT_ARM );
	#endif
	m_UserOwner = USER_CMD_MODULE;	// Normal mode.  Can be switched to OVERRIDE_MODULE if needed

}

CUserCmdModule::~CUserCmdModule()
{
	ROBOT_LOG( TRUE, "~CUserCmdModule()\n" )
	#if ( ROBOT_SERVER == 1 ) //////////////// SERVER ONLY //////////////////
		SAFE_DELETE( m_pArmControlRight );
		SAFE_DELETE( m_pArmControlLeft );
	#endif
}

void CUserCmdModule::ProcessMessage(
		UINT uMsg, WPARAM wParam, LPARAM lParam )
{


/***		
	if( NO_HANDLER == gDriveMotorOwner )
	{
		// Someone took drive control away from this module, but 
		// now control has been made available again
		// Restore last motor drive command, and take control
		SendHardwareCmd(HW_SET_SPEED_AND_TURN, (BYTE)gCurrentSpeed, (BYTE)gCurrentTurn );
		gDriveMotorOwner = MANUAL_CONTROL_HANDLER;
	}
***/

	switch( uMsg )  
	{

#ifdef THIS_CODE_DOES_NOTHING_AND_CAUSES_PROBLEMS_WITH_MODULE_OWNERSHIP
		case WM_ROBOT_SERVO_STATUS_READY:
		case WM_ROBOT_SENSOR_STATUS_READY:
		{
			g_bCmdRecognized = TRUE;
			//BYTE nDistHigh, nDistLow;
			CString MsgString;

/*			if( m_pDriveCtrl->IsOwner(OVERRIDE_MODULE) )
			{
				// Last time through, we took over control via OVERRIDE
				// This would have caused other modules to go to IDLE
				// Now, reset back to normal operation
				m_pDriveCtrl->ReleaseOwner( OVERRIDE_MODULE );
			}

*/
			if( USER_CMD_MODULE == m_UserOwner )
			{
				if( !m_pDriveCtrl->IsOwner(USER_CMD_MODULE) )
				{
					// This module lost control of the drive wheels (such as to avoid an object).  
					// Try to reassert last User speed and turn (if still avoiding, arbitrator will ignore this command)
					// Note that this does not work for SetMoveDistance or SetTurnAmount commands.
					// They are aborted by collision or avoidance.
	//				if( (SPEED_STOP != m_CurrentSpeed) || (TURN_CENTER != m_CurrentTurn) )
					{
						m_pDriveCtrl->SetSpeedAndTurn( USER_CMD_MODULE, m_CurrentSpeed, m_CurrentTurn );
					}
				}
			}
/****
			if( m_IRTrackingEnabled && (!m_VidCapProcessingEnabled) )
			{
				// only do this if IR head tracking is enabled, but 
				// VidCapProcessing is NOT enabled!
				// (if VidCapProcessingEnabled, the camera thread will prioritise IR tracking with other tracking)

				// Track closest object
				if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT]  <= IR_HEAD_TRACKING_RANGE ) ||		
					(g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] <= IR_HEAD_TRACKING_RANGE ) )		
				{
					// An object found within range of IR sensors.  Track it

					if( (g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] )
					{
						m_IRTrackCenter.x = -20;	// Object is to the Left
					}
					else if( (g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT] + IR_TRACKING_FUDGE) < g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT] )
					{
						m_IRTrackCenter.x = 20;	// Object is to the Right
					}
					// Debug
					ROBOT_LOG( TRUE, "UserModule IRTracking: Left = %d, Right = %d, Command = %d\n", 
						g_SensorStatus.IR[IR_SENSOR_HEAD_LEFT], g_SensorStatus.IR[IR_SENSOR_HEAD_RIGHT], m_IRTrackCenter.x )
				}

			}
***/
		}
		break;
#endif
		case WM_ROBOT_OPEN_DATA_FILE:
		{
			g_bCmdRecognized = TRUE;
			// Post a Windows message to the Server GUI thread to open the Path file.
			// wParam = Path Type (Outdoor, Gridmap, etc.)
			// lParam = not used
			PostMessage( g_RobotCmdViewHWND, (WM_ROBOT_DISPLAY_OPEN_DATA_FILE), wParam, lParam );
			//m_NavPathStarted = FALSE;
		}		
		break;

		case WM_ROBOT_MOVE_SET_DISTANCE_CMD:
		{
			g_bCmdRecognized = TRUE;
			// wParam = Distance in Tenth Inches, lParam = direction
			
			// DEBUG
			int nTempTicks = (int)(wParam * TICKS_PER_TENTH_INCH);

			CString MsgString;
			MsgString.Format( "MOVE_SET_DISTANCE: %d Inches", 
				wParam/10 );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )

			// VALIDATE PIC CODE HERE - THEN REMOVE THIS!  TODO-CAR
			//WORD gDesiredMoveTicks = (((BYTE) nDistHigh) << 8);	// Get High Byte
			//gDesiredMoveTicks += ((BYTE) nDistLow);		// Add in Low Byte	
			//ROBOT_LOG( TRUE, "DEBUG: gDesiredMoveTicks = %u\n", gDesiredMoveTicks)

			if( FORWARD == lParam )
			{
				m_pDriveCtrl->SetMoveDistance( m_UserOwner, SPEED_FWD_MED_SLOW, TURN_CENTER, wParam);
			}
			else
			{
				m_pDriveCtrl->SetMoveDistance( m_UserOwner, SPEED_REV_SLOW, TURN_CENTER, wParam);
			}
		}
		break;


		case WM_ROBOT_TURN_SET_DISTANCE_CMD:
		{
			g_bCmdRecognized = TRUE;
			// wParam = Turn Amount in degrees, 
			// lParam = direction (Left/Right) and speed
			
			CString MsgString;
			MsgString.Format( "TURN_SET_DISTANCE: %d Degrees", wParam );
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )

			if( LEFT == lParam )
			{
				m_pDriveCtrl->SetTurnRotation( m_UserOwner, SPEED_STOP, (int)lParam, wParam);
			}
			else
			{
				m_pDriveCtrl->SetTurnRotation( m_UserOwner, SPEED_STOP, (int)lParam, wParam);
			}
		}
		break;

		case WM_ROBOT_SET_PATH_SPEED_INCREASE:
		{
			g_bCmdRecognized = TRUE;
			CString MsgString;
			MsgString.Format("Setting Path Speed Increase to %u", lParam);
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			g_SpeedIncrease = (int )lParam;
		}
		break;

		case WM_ROBOT_SET_AVOID_OBJ_RANGE:
		{
			g_bCmdRecognized = TRUE;
			CString MsgString;
			MsgString.Format("Setting Avoid Object Range to %u feet", lParam);
			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
			g_GlobalMaxAvoidObjectDetectionFeet = (int )lParam;
		}
		break;

		case WM_ROBOT_BRAKE_CMD:
		{
			g_bCmdRecognized = TRUE;
			if( 0 != lParam )
			{
				ROBOT_DISPLAY( TRUE, "TEST: SENDING BRAKE COMMAND!")
				SendHardwareCmd(HW_SET_MOTOR_BRAKE, (DWORD)SPEED_BRAKE_AMOUNT, (DWORD)SPEED_BRAKE_TIME);
			}
			else
			{
				ROBOT_DISPLAY( TRUE, "SENDING BRAKE COMMAND!")
				m_pDriveCtrl->Brake( m_UserOwner );
				// m_pDriveCtrl->ExecuteCommand();	// Don't wait for next status update
			}
		}
		break;

		case WM_ROBOT_USER_OVERRIDE_CMD:
		{
			g_bCmdRecognized = TRUE;
			// This command only comes from a user doing manual control
			// Allows user to override higher priority modules, or release control when done.
			// wParam = action to take

			/*if( SET_USER_RELEASED == wParam )
			{
				// Release user control
				m_pDriveCtrl->ReleaseOwner( USER_CMD_MODULE );
				m_UserOwner = USER_CMD_MODULE;	// back to normal user mode
				m_pDriveCtrl->EnableModule( COLLISION_MODULE );
				m_pDriveCtrl->EnableModule( AVOID_OBJECT_MODULE );
				m_pDriveCtrl->EnableModule( WAY_POINT_NAV_MODULE );
				m_pDriveCtrl->EnableModule( GRID_NAV_MODULE );
				m_pDriveCtrl->ReleaseOwner( OVERRIDE_MODULE );
			}
			else
			*/
			if( SET_USER_NORMAL == wParam )
			{
				// Normal mode.  User can override lesser modules, but Avoid and Collision will act
				ROBOT_DISPLAY( TRUE, " USER_OVERRIDE_CMD: Mode set to USER_NORMAL\n" )

					
				m_pDriveCtrl->ReleaseOwner( OVERRIDE_MODULE );
				m_UserOwner = USER_CMD_MODULE;	// back to normal user mode
				//m_pDriveCtrl->EnableModule( COLLISION_MODULE );
				//m_pDriveCtrl->EnableModule( AVOID_OBJECT_MODULE );
				//m_pDriveCtrl->SuppressModule( WAY_POINT_NAV_MODULE );
				//m_pDriveCtrl->SuppressModule( GRID_NAV_MODULE );
			}
			else if( SET_USER_OVERRIDE == wParam )
			{
				// furture user commands will override all other modules
				ROBOT_DISPLAY( TRUE, " USER_OVERRIDE_CMD: Mode set to USER_OVERRIDE MODE!\n" )
				m_UserOwner = OVERRIDE_MODULE;	// Super user mode

				// ??? this will force the other modules to abort and go back to IDLE state:
				//m_pDriveCtrl->SuppressModule( COLLISION_MODULE );
				//m_pDriveCtrl->SuppressModule( AVOID_OBJECT_MODULE );
				//m_pDriveCtrl->SuppressModule( WAY_POINT_NAV_MODULE );
				//m_pDriveCtrl->SuppressModule( GRID_NAV_MODULE );
			}
			else if( SET_USER_OVERRIDE_AND_STOP == wParam )
			{
				// Take control from all other modules and Stop (User already higher priority then Nav modules, so no need to supress them)
				ROBOT_LOG( TRUE, " USER_OVERRIDE_CMD: Mode set to USER_OVERRIDE_AND_STOP!\n" )
				// ?? this will force the other modules to abort and go back to IDLE state:

				m_UserOwner = OVERRIDE_MODULE;	// Super user mode
				//m_pDriveCtrl->SuppressModule( COLLISION_MODULE );
				//m_pDriveCtrl->SuppressModule( AVOID_OBJECT_MODULE );
				//m_pDriveCtrl->SuppressModule( WAY_POINT_NAV_MODULE );
				//m_pDriveCtrl->SuppressModule( GRID_NAV_MODULE );
				m_CurrentSpeed = SPEED_STOP;
				m_CurrentTurn = TURN_CENTER;
				m_pDriveCtrl->SetSpeedAndTurn( m_UserOwner, m_CurrentSpeed, m_CurrentTurn );
			}
			else
			{
				ROBOT_ASSERT(0);	// Logic Error
			}
		}
		break;

		case WM_ROBOT_JOYSTICK_DRIVE_CMD:
		{
			// This command only comes from a user doing manual control.
			// lParam is Speed, wParam is Turn
			// ROBOT_DISPLAY( TRUE, "Server ACK: JOYSTICK DRIVE CMD" )
			g_bCmdRecognized = TRUE;
			int NewSpeed = wParam;
			int NewTurn = lParam;


			// Speed and Turn commands from GUI are +/- 127, with zero = Center/Stop 
			// Convert to HW command values, for reverse, we add a negative number

			//-------------------------------------------------------------------------------
			#if( MOTOR_CONTROL_TYPE == SERVO_MOTOR_CONTROL )
				// RC Car with external servo control: Servo max values are 0-255

				m_CurrentSpeed = SPEED_STOP + (NewSpeed);	 // convert to servo range
				if( m_CurrentSpeed > SPEED_FULL_FWD )			// Limit to valid range
				{
					m_CurrentSpeed = SPEED_FULL_FWD;	
				}
				else if( m_CurrentSpeed < SPEED_FULL_REV )
				{
					m_CurrentSpeed = SPEED_FULL_REV;	
				}

				m_CurrentTurn = (NewTurn);	// convert to servo range
				if( m_CurrentTurn < TURN_LEFT_MAX )			// Limit to valid range
				{
					m_CurrentTurn = TURN_LEFT_MAX;	
				}
				else if( m_CurrentTurn > TURN_RIGHT_MAX )
				{
					m_CurrentTurn = TURN_RIGHT_MAX;	
				}

				//int test = (TURN_CENTER - m_CurrentTurn);
				m_pDriveCtrl->SetSpeedAndTurn( m_UserOwner, m_CurrentSpeed, m_CurrentTurn );

			#elif( MOTOR_CONTROL_TYPE == POLOLU_TREX_MOTOR_CONTROL )
				// Robot using Pololu TReX Motor control
				m_CurrentSpeed = NewSpeed;
				m_CurrentTurn = NewTurn;
				m_pDriveCtrl->SetSpeedAndTurn( m_UserOwner, m_CurrentSpeed, m_CurrentTurn );


			#elif MOTOR_CONTROL_TYPE == ER1_MOTOR_CONTROL
				// Robot using ER1 "Pilot" Motor control: Max values are 0-255
				m_CurrentSpeed = NewSpeed;
				m_CurrentTurn = NewTurn;
				m_pDriveCtrl->SetSpeedAndTurn( m_UserOwner, m_CurrentSpeed, m_CurrentTurn );


			#elif MOTOR_CONTROL_TYPE == IROBOT_MOTOR_CONTROL
				// Robot using iRobot Create base
				m_CurrentSpeed = NewSpeed;
				m_CurrentTurn = NewTurn;
				m_pDriveCtrl->SetSpeedAndTurn( m_UserOwner, m_CurrentSpeed, m_CurrentTurn );

			#else
				#error BAD ROBOT_TYPE
			#endif

			return;
		}

		case WM_ROBOT_RESET_WATCHDOG_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Process Watchdog Reset command to PIC
			// 12v Power ON also does similar things, but this function will not turn power on/of
			// Tell the PC to resume sending Keep Alive pings
			g_bResetWatchdog = TRUE;
			// Send Command to reset the watchdog timer, and clear Watchdog error state
			// BOOL bResult = VendorCommandWrite( HW_RESET_WATCHDOG, 0, 0 );
			return;
		}

		case WM_ROBOT_RESET_ODOMETER:
		{
			g_bCmdRecognized = TRUE;
			// Process Odometer Reset command to PIC
			g_SensorStatus.OdometerTenthInches = 0;
			g_SensorStatus.OdometerUpdateTenthInches = 0;
			SendHardwareCmd( HW_RESET_ODOMETER, 0, (BYTE)lParam );	// 0=off, non-zero = on
			return;
		}


		case WM_ROBOT_12V_POWER_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Process 12v Power On/Off command from the GUI
			// (Allows user to manually turn 12v On/Off.  12v turned off automatically
			// if the Keep Alives fail to reset the timer in time.

			// Send Command to turn the power on or off
			// Note: Power On automatically resets the PIC Watchdog Timer
			// Send Command to turn pwr on or off
			SendHardwareCmd( HW_SET_MOTOR_POWER, 0, (BYTE)lParam );	// 0=off, non-zero = on
			return;
		}

		case WM_ROBOT_ENABLE_RADAR_SCAN_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Process Command to enable a Radar Scanning
			// The PIC will indicate when each scan is complete, and run until disabled

			// First, set the Tilt Angle (adjust for minimum height object detected)
			// Note: CENTER=63, MAX_UP=75, MAX_DOWN=45
			SendHardwareCmd( HW_SET_CAMERA_TILT_ABS, (CAMERA_TILT_SERVO_CENTER-2), 0 );

			// Now, tell the Radar Scan to start
			SendHardwareCmd( HW_ENABLE_RADAR_SCAN, (BYTE)wParam, (BYTE)lParam );
			return;
		}

		case WM_ROBOT_SERVO_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Process Servo position command from the GUI
			// Servos numbered above 127 are controlled via PC Serial Port (handled by CServoModule)
			// Servos below 127 are controlled via PIC (handled here)
			//ROBOT_DISPLAY( TRUE, "Servo Command received" )

			WORD	nServoNumber = (WORD)wParam;
			WORD	nPosition = (WORD)lParam;
			if( nServoNumber <= 127 )
			{
				SendHardwareCmd( HW_SET_SERVO_POS_TICKS, nServoNumber, nPosition );
			}
			return;
		}

/*		case WM_ROBOT_SET_ARM_JOINT_POSITION:
		{
			g_bCmdRecognized = TRUE;
			// Process Arm Joint position command from the GUI
			ROBOT_DISPLAY( TRUE, "Arm Servo Single Joint Command received" )
			SendHardwareCmd( HW_SET_ARM_JOINT_POSITION, wParam, lParam );
			return;
		}
*/
		case WM_ROBOT_SET_ARM_POSITION:
		{
			g_bCmdRecognized = TRUE;
			// Process command to move all joints in the arm.  Uses global g_BulkServoCmd to pass the data
			//ROBOT_DISPLAY( TRUE, "Arm Servo Bulk Positon Command received" )


			// Get current X,Y,Z position for debug
			#if ( ROBOT_SERVER == 1 ) //////////////// SERVER ONLY //////////////////

				FPOINT3D_T ArmXYZ;
				if( LEFT_ARM == wParam )
				{
					m_pArmControlLeft->GetTargetArmXYZ( ArmXYZ );
					//ROBOT_LOG( TRUE, "DEBUG: RIGHT ARM X = %4.1f, Y = %4.1f, Z = %4.1f\n", ArmX, ArmY, ArmZ )
				}
				else
				{
					m_pArmControlRight->GetTargetArmXYZ( ArmXYZ );
					//ROBOT_LOG( TRUE, "DEBUG: LEFT ARM X = %4.1f, Y = %4.1f, Z = %4.1f\n", ArmX, ArmY, ArmZ )
				}

				//ROBOT_DISPLAY( TRUE, "MODULE: Sending HW_SET_BULK_ARM_POSITION\n" )
				SendHardwareCmd( HW_SET_BULK_ARM_POSITION, wParam, lParam );
			#endif
			return;
		}

		case WM_ROBOT_SET_HEAD_POSITION:
		{
			g_bCmdRecognized = TRUE;
			// Process command to move all joints in the head.  Uses global g_BulkServoCmd to pass the data
			//ROBOT_DISPLAY( TRUE, "Head Servo Bulk Positon Command received" )
			SendHardwareCmd( HW_SET_BULK_HEAD_POSITION, wParam, lParam );
			return;
		}

		case WM_ROBOT_SET_KINECT_POSITION:
		{
			g_bCmdRecognized = TRUE;
			// Process command to move Kinect servo.  Uses global g_BulkServoCmd to pass the data
			//ROBOT_DISPLAY( TRUE, "Kinect Bulk Positon Command received" )
			SendHardwareCmd( HW_SET_BULK_KINECT_POSITION, wParam, lParam );
			return;
		}

		case WM_ROBOT_SET_SERVO_TORQUE_ENABLE:
		{
			g_bCmdRecognized = TRUE;
			// Process command to enable/disable servo torque.
			// Uses bitfields: wParam = Operation, lParam = Enable/Disable
			//ROBOT_DISPLAY( TRUE, "Arm Servo Bulk Torque Enable Command received" )
			SendHardwareCmd( HW_SET_SERVO_TORQUE_ENABLE, wParam, lParam );
			return;
		}

		case WM_ROBOT_SET_SERVO_TORQUE_LIMIT:
		{
			g_bCmdRecognized = TRUE;
			// Process command to set servo torque.
			//ROBOT_DISPLAY( TRUE, "Servo Set Torque Command received" )
			SendHardwareCmd( HW_SET_SERVO_TORQUE_LIMIT, wParam, lParam );
			return;
		}

		case WM_ROBOT_GET_SMART_SERVO_STATUS:
		{
			g_bCmdRecognized = TRUE;
			// Process command to read current status of all joints in the arm.  Uses global g_BulkServoCmd to receive the data
			//ROBOT_DISPLAY( TRUE, "Arm Get Bulk Status received" )
			SendHardwareCmd( HW_GET_SMART_SERVO_STATUS, wParam, lParam );
			return;
		}

		case WM_ROBOT_GET_SERVO_STATUS:
		{
			g_bCmdRecognized = TRUE;
			// Process Servo status request from the GUI
			//ROBOT_DISPLAY( TRUE, "Servo Status Request received" )

			BYTE	nServoNumber = (BYTE)wParam;
			if( nServoNumber <= 127 )
			{
				SendHardwareCmd( HW_GET_SERVO_STATUS, nServoNumber, 0 );
			}
			return;
		}


		case WM_ROBOT_SET_GEAR_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Set the Car Gear speed.
			int Gear = lParam;		// 0 = Low Gear, 1 = High Gear
			if( 0 == lParam )
			{
				// Low Gear
				SendHardwareCmd( HW_SET_GEAR, 0, LOW_GEAR );	// Servo position for Low
			}
			else
			{
				// High Gear
				SendHardwareCmd( HW_SET_GEAR, 0, HIGH_GEAR );	// Servo position for High
			}
			return;
		}

		/* Handled by Behavioral Module
		case WM_ROBOT_SET_LED_EYES_CMD:
		*/

		case WM_ROBOT_LIGHT_POWER_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to set the driving Lights On/Off
			SendHardwareCmd( HW_SET_LIGHT_POWER, 0, (BYTE)lParam );	// 0=off, 1=on
			return;
		}

		case WM_ROBOT_AUX_LIGHT_POWER_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to set the Aux Lights On/Off
			SendHardwareCmd( HW_SET_AUX_LIGHT_POWER, 0, (BYTE)lParam );	// 0=off, 1=on
			return;
		}

		case WM_ROBOT_ENABLE_DYNA_CONTROLLER_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to enable Dyna Servos.  For Turtle, this enables power to the servos
			SendHardwareCmd( HW_SET_DYNA_USB, 0, (BYTE)lParam );	// 0=off, 1=on
			return;
		}
		
		case WM_ROBOT_KINECT_POWER_ENABLE_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to set the Kinect On/Off
			SendHardwareCmd( HW_SET_KINECT_POWER, 0, (BYTE)lParam );	// 0=off, 1=on
			return;
		}

		case WM_ROBOT_POWER_MODE_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to turn servos and laptop on/off
			//                                   SERVO PWR,    LAPTOP PWR
			SendHardwareCmd( HW_SET_POWER_MODE, (BYTE)wParam, (BYTE)lParam );	// see LAPTOP_POWER_MODE_DEFAULT
			return;
		}


		case WM_ROBOT_IR_SENSOR_POWER_CMD:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to enable/disable power to the IR sensor in the RIGHT claw
			SendHardwareCmd( HW_SET_IR_SENSOR_POWER, /*TODO - Right/L*/ 0, (BYTE)lParam );	// 0=off, 1=on
			return;
		}


		case WM_ROBOT_TEXT_TO_AI:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to play a sound
			BOOL bResult = PostThreadMessage( g_dwAIThreadId, (WM_ROBOT_TEXT_TO_AI), wParam, lParam);
			return;
		}

		case WM_ROBOT_SPEAK_TEXT:
		{
			// send command to speak some text.
			PostMessage( g_RobotSetupViewHWND, WM_ROBOT_SPEAK_TEXT, wParam, lParam );
		}

		case WM_ROBOT_PLAY_SOUND:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to play a sound wav file (NOT Text to speach!)
			//	DONT DO THIS:	PostMessage( g_RobotSetupViewHWND, WM_ROBOT_SPEAK_TEXT, wParam, lParam );
			BOOL bResult = PostThreadMessage( g_dwSoundThreadId, (WM_ROBOT_PLAY_SOUND), wParam, lParam);
			return;
		}

		case WM_ROBOT_PLAY_MUSIC:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to play a music file
			BOOL bResult = PostThreadMessage( g_dwSoundThreadId, (WM_ROBOT_PLAY_MUSIC), wParam, lParam);
			return;
		}

		case WM_ROBOT_SET_COMPASS_CAL_MODE:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to enter Compass Calibration Mode
			SendHardwareCmd( HW_COMPASS_CAL_MODE, 0, (BYTE)lParam );	// 0=off, 1=on
			return;
		}

		case WM_ROBOT_SET_COMPASS_CAL_POINT:
		{
			g_bCmdRecognized = TRUE;
			// Send Command to enter Compass Calibration Mode
			SendHardwareCmd( HW_COMPASS_CAL_POINT, 0, 0 );
			return;
		}

		case WM_ROBOT_CAMERA_ENABLE_FEATURE:
		{
			// This command is normally handled by the Camera Module.
			// Just done here too so we can handle special case of IR Head Tracking
			//g_bCmdRecognized = TRUE;
			BOOL EnableFeature = (BOOL)lParam;
			switch( wParam )
			{
				
				case CAMERA_ENABLE_VIDCAP_PROCESSING:
				{
					m_VidCapProcessingEnabled = EnableFeature;
					return;
				}

				case CAMERA_ENABLE_TRACKING_IR:
				{
					m_IRTrackingEnabled = EnableFeature;
					return;
				}

			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//	MODULE: CollisionModule

CCollisionModule::CCollisionModule( CDriveControlModule *pDriveControlModule )
{
	m_pDriveCtrl = pDriveControlModule;
	m_CollisionState = DISABLED;
	m_CollisionDirection = 0;
	gCollisionTimer = 0;
	m_RandomTurn = 0;
}

CCollisionModule::~CCollisionModule()
{
		ROBOT_LOG( TRUE, "~CCollisionModule()\n" )
}



void CCollisionModule::ProcessMessage( 
		UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	IGNORE_UNUSED_PARAM (wParam);
	IGNORE_UNUSED_PARAM (lParam);

	switch( uMsg )
	{
		case WM_ROBOT_ENABLE_COLLISION_MODULE:		// COLLISION_MODULE
		{

			// Command from user to enable/disable the Collision Module
			g_bCmdRecognized = TRUE;
			// wParam = Enabled/Disabled
			// lParam = not used

			if( wParam )
			{
				// module enabled
				m_CollisionState = IDLE;
				ROBOT_DISPLAY( TRUE, "Collision Module Enabled" )
			}
			else
			{
				m_CollisionState = DISABLED;
				m_pDriveCtrl->ReleaseOwner( COLLISION_MODULE );
				ROBOT_DISPLAY( TRUE, "Collision Module Disabled" )
			}
			return;
		}

		case WM_ROBOT_SENSOR_STATUS_READY:
		{
			g_bCmdRecognized = TRUE;
			//BYTE nDistHigh, nDistLow;
			CString MsgString;

			if( DISABLED == m_CollisionState )
			{
				// This module is disabled
				return;
			}

			if( !m_pDriveCtrl->IsOwner(COLLISION_MODULE) )
			{
				// This module lost control of the drive wheels.  Go to a good state.
				m_CollisionState = IDLE;
			}
			
			if( (IDLE == m_CollisionState ) && 
				(0 == m_pDriveCtrl->GetCurrentSpeed()) && 
				(0 == m_pDriveCtrl->GetCurrentTurn())  )
			{
				// Not currently moving.  
				// Ignore sensors so we don't suddenly start moving if someone obscures a sensor!
				return;
			}

			// See if we need to wait for a delay to complete (CARBOT ONLY!)
			if( gCollisionTimer != 0 ) 
			{
				ROBOT_LOG( TRUE, "Waiting for Collision Timer (%d)\n", gCollisionTimer )
				return;

			}

			///////////////////////////////////////////////////////////////////
			// Process Collision State Machine

			////////////////////////////////////////////////////////////////
			#if ( (ROBOT_TYPE == LOKI) || ( ROBOT_TYPE == TURTLE) )
			////////////////////////////////////////////////////////////////
			#define COLLISION_BACKUP_SPEED				SPEED_REV_MED_SLOW
			#define COLLISION_BACKUP_DISTANCE			MOVE_DISTANCE_MED
			#define COLLISION_INITIAL_ROTATION_AMOUNT	15 // degrees
			#define COLLISION_FINAL_ROTATION_AMOUNT		25 // degrees	 - don't keep driving into the wall


//			MsgString.Format("DEBUG FRONT SENSORS! Range=%d Tenth Inches, Angle=%d",  
//				g_pSensorSummary->nFrontObjectDistance, g_pSensorSummary->nFrontObjectDirection);
//			ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString );

			if( BRAKING != m_CollisionState )
			{
				// unless we are already Braking due to a collision, handle new collision
				// even if currently handling another collision.
/** TODO - is this needed?
				// Check for object obstructing elbow (the most common collision, since they stick out!
				if( g_pSensorSummary->bObjectElbowLeft || g_pSensorSummary->bObjectElbowRight )
				{
					if( g_pSensorSummary->bObjectElbowLeft && g_pSensorSummary->bObjectElbowRight )
					{
						// collision on both sides  Oh no!  what to do?
					ROBOT_DISPLAY( TRUE, "COLLISION MODULE: ERROR! Elbow collision on Both Sides!!!\n" )
						g_ClientTextToSend =  "Error! Arm hits on both sides" );	
						SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
						Turn = TURN_CENTER;	// NO turn, just stop!
						Speed = SPEED_STOP;	
					}
					else if( g_pSensorSummary->bObjectElbowLeft )
					{		
						ROBOT_DISPLAY( TRUE, "COLLISION MODULE: Arm Left Side!\n" )
						//g_ClientTextToSend =  "Elbow hit Left" );	
						//SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
						Turn = TURN_RIGHT_MED_SLOW;
						Speed = SPEED_REV_SLOW;	// force a backward pivot turn on opposite wheel
					}
					else if( g_pSensorSummary->bObjectElbowRight )
					{		
						ROBOT_DISPLAY( TRUE, "COLLISION MODULE: Arm Right Side!\n" )
						//g_ClientTextToSend =  "Elbow hit Right" );	
						//SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
						Turn = TURN_LEFT_MED_SLOW;
						Speed = SPEED_REV_SLOW;	// force a backward pivot turn
					}
					else
					{
						// Logic Error
						ROBOT_ASSERT(0);
					}
					// ER1 Controller does not have "Breaking" but stop does a pretty fast stop!
					// Seems to ignore the deceleration profile... just stops fast
					m_pDriveCtrl->Stop( COLLISION_MODULE );
					//SpeakCannedPhrase( SPEAK_COLLISION );

					m_CollisionState = BRAKING;
					ROBOT_DISPLAY( TRUE, "Collision State: BRAKING")
					return;	// Let the motor control issue the stop command!

				}

***/
				if( (g_pSensorSummary->nFrontObjectDistance <= OBJECT_COLLISION) ||
					(g_pSensorSummary->nSideObjectDistance  <= OBJECT_COLLISION) ||					
					((g_pSensorSummary->nRearObjectDistance <= OBJECT_COLLISION)&& (m_pDriveCtrl->GetCurrentSpeed() < SPEED_STOP))	) 	// Only look at rear sensor if backing up!
				{
					// Collision
					ROBOT_LOG( TRUE, "COLLISION MODULE: Collision detected\n" )

					CString MsgString, SensorString;
					if( g_pSensorSummary->nFrontObjectDistance <= OBJECT_COLLISION )
					{
						// Collision was in front
						m_CollisionDirection = g_pSensorSummary->nFrontObjectDirection; // Save so later we know where the threat came from.
						if( HW_BUMPER_HIT_FRONT )
							SensorString.Format( "Front Bumper: ");
						else
							SensorString.Format( "Front Sensor: ");
						MsgString.Format("COLLISION! %s Range=%d inches, Angle=%d", SensorString, 
							g_pSensorSummary->nFrontObjectDistance/10, g_pSensorSummary->nFrontObjectDirection);
						ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					}
					else if( g_pSensorSummary->nSideObjectDistance <= OBJECT_COLLISION )
					{
						// Collision was on a side
						m_CollisionDirection = g_pSensorSummary->nSideObjectDirection; // Save so later we know where the threat came from.
						if( HW_BUMPER_HIT_SIDE_LEFT || HW_BUMPER_HIT_SIDE_RIGHT )
							SensorString.Format( "Side Bumper: ");
						else
							SensorString.Format( "Side Sensor: ");
						SensorString.Format("COLLISION! Side Sensor: Range=%d inches, Angle=%d", 
							g_pSensorSummary->nSideObjectDistance/10, g_pSensorSummary->nSideObjectDirection);
						ROBOT_DISPLAY( TRUE, (LPCTSTR)SensorString )
					}
					else if( g_pSensorSummary->nRearObjectDistance <= OBJECT_COLLISION )
					{
						// Collision was at the rear!
						m_CollisionDirection = g_pSensorSummary->nRearObjectDirection; // Save so later we know where the threat came from.
						MsgString.Format("COLLISION! Rear Bumper: Range=%d inches, Angle=%d", 0, m_CollisionDirection);
						ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					}
					else
					{
						// Should never get here!
						ROBOT_DISPLAY( TRUE, "ERROR! Module.cpp Logic Error - Collision State Machine" )
						ROBOT_ASSERT(0);
					}

					// Loki Controller does not have "Breaking" but stop does a pretty fast stop!
					m_pDriveCtrl->Stop( COLLISION_MODULE );
					//SpeakCannedPhrase( SPEAK_COLLISION );
					m_CollisionState = BRAKING;
					ROBOT_DISPLAY( TRUE, "Collision State: BRAKING" )
					return;	// Let the motor control issue the stop command!
				}
			}	
						
			
			switch (m_CollisionState) 
			{
				case IDLE:	// LOKI
				{
					// Handled by the default collision detection above
				}
				break;

				case BRAKING:	// LOKI
				{
					// Wait for PIC to report that we are done braking
					if( m_pDriveCtrl->RobotStopped() )
					{
						// Done Braking.  Now move away from the object

						// Pick a random direction to use in case the collision was straight ahead
						if( rand() > (RAND_MAX/2) )
							m_RandomTurn = TURN_LEFT_SLOW;
						else
							m_RandomTurn = TURN_RIGHT_SLOW;

						if( REAR_BUMPER_ANGLE == m_CollisionDirection )
						{
							// Robot was backing up when we hit object on bumper, so just move slightly to remove pressure from the bumper
							// we were probably just avoiding something else anyway...
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_FWD_MED_SLOW, 
								TURN_CENTER, 30, STOP_AFTER );	// TenthInches
							ROBOT_LOG( TRUE, "Collision Module - Moving Forward\n" )
							m_CollisionState = FINAL_MOVE;
							ROBOT_DISPLAY( TRUE, "Collision State: FINAL_MOVE" )
							return;
						}
						else if( REAR_LEFT == m_CollisionDirection )
						{
							// Robot was backing up when we hit on Left side, so move forward Right, then Left
							m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, SPEED_FWD_MED_SLOW, // pivot on one wheel
								TURN_RIGHT_MED_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module - Moving Forward Right\n" )
							m_CollisionState = TURN_FORWARD1;
							ROBOT_DISPLAY( TRUE, "Collision State: TURN_FORWARD1" )
							return;
						}
						else if( REAR_RIGHT == m_CollisionDirection )
						{
							// Robot was backing up when we hit on Right side, so move forward Left, then Right
							m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, SPEED_FWD_MED_SLOW, // pivot on one wheel
								TURN_LEFT_MED_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module - Moving Forward Left\n" )
							m_CollisionState = TURN_FORWARD1;
							ROBOT_DISPLAY( TRUE, "Collision State: TURN_FORWARD1" )
							return;
						}
						// ROBOT WAS MOVING FORWARD
						else if( m_CollisionDirection < 0 )
						{
							// Object on the Left, so pivot right, and then left
							m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, COLLISION_BACKUP_SPEED, // pivot on wheel
								TURN_RIGHT_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module 1- Rotate Right\n" )
						}
						else if( m_CollisionDirection > 0 )
						{
							// Object on the Right, so pivot left, and then right
							m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, COLLISION_BACKUP_SPEED, // pivot on wheel
								TURN_LEFT_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module 1- Rotate Right\n" )
						}
						else
						{
							// Object straight ahead.  Back away curve in random direction
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, COLLISION_BACKUP_SPEED, 
								m_RandomTurn, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module 1- Obj ahead, Backing Straight\n" )
						}								
						m_CollisionState = TURN_BACKUP1;
						ROBOT_DISPLAY( TRUE, "Collision State: TURN_BACKUP1" )
						return;
					}
				}
				break;

				case TURN_BACKUP1:	// LOKI
				{
					// Wait for drive control to report that we are done rotating
					if( m_pDriveCtrl->TurnRotationCompleted() )
					{
						// Done with slight rotation. Now back up straight
						m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, COLLISION_BACKUP_SPEED, 
							TURN_CENTER, COLLISION_BACKUP_DISTANCE, DONT_STOP_AFTER);
						ROBOT_LOG( TRUE, "Collision Module - Backing Straight\n" )
							
						m_CollisionState = BACKING1;
						ROBOT_DISPLAY( TRUE, "Collision State: BACKING1" )
						return;
					}						
				}	
				break;

				case BACKING1:	// LOKI
				{
					// Wait for drive control to report that we are done backing
					if( m_pDriveCtrl->MoveDistanceCompleted() )
					{
						// Done with first part of backing up. Turn the other way
						if( m_CollisionDirection < 0 )
						{
							// Object on the Left, so rotate the other direction (swing the back-end to the right)
							m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, COLLISION_BACKUP_SPEED, // pivot on wheel
								TURN_LEFT_MED_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module 1- Rotate Right\n" )
						}
						else if( m_CollisionDirection > 0 )
						{
							// Object on the Right, so rotate the other way
							m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, COLLISION_BACKUP_SPEED, // pivot on wheel
								TURN_RIGHT_MED_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module 1- Rotate Left\n" )
						}
						else
						{
							// Object straight ahead.  Random turn the other way
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, COLLISION_BACKUP_SPEED, 
								m_RandomTurn, COLLISION_BACKUP_DISTANCE, DONT_STOP_AFTER);
							ROBOT_LOG( TRUE, "Collision Module 1- Obj ahead, Backing Straight\n" )
						}								
						m_CollisionState = TURN_BACKUP2;
						ROBOT_DISPLAY( TRUE, "Collision State: TURN_BACKUP2" )
						return;
					}						
				}	
				break;

				case TURN_BACKUP2:	// LOKI
				{
					// Wait for drive control to report that we are done rotating after backing
					if( m_pDriveCtrl->TurnRotationCompleted() )
					{
						// Done turning.  Ready to go foward again.
						m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_FWD_MED_SLOW, 
							TURN_CENTER, 60); // TenthInches
						ROBOT_LOG( TRUE, "Collision Module - Moving forward slightly\n" )
							
						m_CollisionState = FINAL_MOVE;
						ROBOT_DISPLAY( TRUE, "Collision State: FINAL_MOVE" )
						return;
					}						
				}	
				break;

				// Colision while backing up
				case TURN_FORWARD1:	// LOKI
				{
					// Moving forward after a rear collision.
					// Done with first part of S turn. Turn the other way
					if( m_CollisionDirection < 0 )
					{
						// Object on the Left, so swing back left again
						m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, SPEED_FWD_MED_SLOW, // pivot on wheel
							TURN_LEFT_MED_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, STOP_AFTER);
						ROBOT_LOG( TRUE, "Collision Module 1- Rotate Right\n" )
					}
					else if( m_CollisionDirection > 0 )
					{
						// Object on the Right, so swing back right again
						m_pDriveCtrl->SetTurnRotation( COLLISION_MODULE, SPEED_FWD_MED_SLOW, // pivot on wheel
							TURN_RIGHT_MED_SLOW, COLLISION_INITIAL_ROTATION_AMOUNT, STOP_AFTER);
						ROBOT_LOG( TRUE, "Collision Module 1- Rotate Left\n" )
					}
					else
					{
						// Object straight ahead.  Random turn the other way
						m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_FWD_MED_SLOW, 
							-m_RandomTurn, COLLISION_BACKUP_DISTANCE, DONT_STOP_AFTER);
						ROBOT_LOG( TRUE, "Collision Module 1- Obj ahead, Backing Straight\n" ) 
					}								
					m_CollisionState = FINAL_MOVE;
					ROBOT_DISPLAY( TRUE, "Collision State: FINAL_MOVE" )
					return;
				}	
				break;

				case FINAL_MOVE:	// LOKI
				{
					// Wait for drive control to report that we are done moving forward
					if( m_pDriveCtrl->MoveDistanceCompleted() )
					{
						// Done moving forward, 
						// Done recovering from collision. Ready to resume where we left off.
						m_CollisionDirection = 0;
						m_CollisionState = IDLE;
						ROBOT_DISPLAY( TRUE, "Collision State: Done --> IDLE" )
						m_pDriveCtrl->ReleaseOwner( COLLISION_MODULE );
					}
				}
				break;

				default:
				{
					MsgString.Format( "ERROR! Illegal m_CollisionState = %d", m_CollisionState );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					m_CollisionState = IDLE;
					m_pDriveCtrl->ReleaseOwner( COLLISION_MODULE );
				}
			}	// Switch Collision State


			////////////////////////////////////////////////////////////////
			#elif ROBOT_TYPE == CARBOT
			////////////////////////////////////////////////////////////////
			switch (m_CollisionState) 
			{
				case IDLE:	// CARBOT
				{
					if( (g_pSensorSummary->nFrontObjectDistance < OBJECT_COLLISION) ||
						(g_pSensorSummary->nSideObjectDistance < OBJECT_COLLISION) )
					{
						// object very close, or bumper hit
						ROBOT_LOG( TRUE, "COLLISION MODULE: Collision detected\n" )

						CString MsgString, SensorString;
						if( g_pSensorSummary->nFrontObjectDistance < g_pSensorSummary->nSideObjectDistance )
						{
							// Collision was in front
							m_CollisionDirection = g_pSensorSummary->nFrontObjectDirection; // Save so later we know where the threat came from.

							if( HW_BUMPER_HIT_FRONT )
							{
								SensorString.Format( "Front Bumper: ");
							}
							else
							{
								SensorString.Format( "Front Sensor: ");
							}
						}
						else
						{
							// Collision was on a side
							m_CollisionDirection = g_pSensorSummary->nSideObjectDirection; // Save so later we know where the threat came from.

							SensorString.Format("COLLISION! Side Sensor: Range=%d inches, Angle=%d", 
								g_pSensorSummary->nSideObjectDistance/10, g_pSensorSummary->nSideObjectDirection);
							ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
						}

						MsgString.Format("COLLISION! %s Range=%d inches, Angle=%d", SensorString, 
							g_pSensorSummary->nFrontObjectDistance/10, g_pSensorSummary->nFrontObjectDirection);
						ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )


						m_pDriveCtrl->Brake( COLLISION_MODULE );
						//SpeakCannedPhrase( SPEAK_COLLISION );

						m_CollisionState = BRAKING;
						ROBOT_DISPLAY( TRUE, "Collision State: BRAKING" )
					}
				}
				break;

				case BRAKING:	// CARBOT
				{
					// Wait for PIC to report that we are done braking
					if( m_pDriveCtrl->RobotStopped() )
					{
						// Done Braking.  Now back away from the object
						if( m_CollisionDirection < 0 )
						{
							// Object on the Left, turn wheels Right while backing
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_REV_MED_SLOW, 
								TURN_RIGHT_MED_SLOW, MOVE_DISTANCE_MED);
						}
						else if( m_CollisionDirection > 0 )
						{
							// Object on the Right, turn wheels Left while backing
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_REV_MED_SLOW, 
								TURN_LEFT_MED_SLOW, MOVE_DISTANCE_MED);
						}
						else
						{
							// Object straight ahead.  Back away straight
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_REV_MED_SLOW, 
								TURN_CENTER, MOVE_DISTANCE_MED);
						}	
							
						m_CollisionState = BACKING1;
						ROBOT_DISPLAY( TRUE, "Collision State: BACKING1" )
						return;
					}						
				}	
				break;
			

				case BACKING1:	// CARBOT
				{
					// Wait for drive control to report that we are done backing
					// Ignore reports of collision in front, while we were backing up
					if( m_pDriveCtrl->MoveDistanceCompleted() )
					{
						// Done with first part of backing up.
						// If curving turn wheels the other way to prepare robot for
						// going around the object.
						if( m_CollisionDirection < 0 )
						{
							// Object on the Left, now turn wheels Left while backing
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_REV_MED_SLOW, 
								TURN_LEFT_MED, MOVE_DISTANCE_MED_SHORT);
						}
						else if( m_CollisionDirection > 0 )
						{
							// Object on the Right, now turn wheels Right while backing
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_REV_MED_SLOW, 
								TURN_RIGHT_MED, MOVE_DISTANCE_MED_SHORT);
						}
						else
						{
							// Object straight ahead.  Back away straight
							m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_REV_MED_SLOW, 
								TURN_CENTER, MOVE_DISTANCE_MED_SHORT);
						}	
							
						m_CollisionState = BACKING2;
						ROBOT_DISPLAY( TRUE, "Collision State: BACKING2" )
						return;
					}						
				}	
				break;

				case BACKING2:	// CARBOT
				{
					// Wait for drive control to report that we are done backing
					// TODO: Monitor collision in back, while we were backing up
					if( m_pDriveCtrl->MoveDistanceCompleted() )
					{
						// Done backing up, now wait for timer to allow for coasting, and for wheels to center
						m_pDriveCtrl->SetTurn(COLLISION_MODULE, TURN_CENTER); // center wheels while coasting
						gCollisionTimer = MOVE_DISTANCE_COASTING_TIME;
						m_CollisionState = FORWARD1;
						ROBOT_DISPLAY( TRUE, "Collision State: FORWARD1" )
					}
				}
				break;

				case FORWARD1:	// CARBOT
				{
					// Timer completed
					// Done backing up, now turn wheels and go forward
					if( m_CollisionDirection > 0 )
					{
						// Turn Left (point away from objects to the right).
						m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_FWD_MED_SLOW, 
							TURN_LEFT_FAST, MOVE_DISTANCE_MED_SHORT);
					}
					else
					{
						// Turn Right
						m_pDriveCtrl->SetMoveDistance( COLLISION_MODULE, SPEED_FWD_MED_SLOW, 
							TURN_RIGHT_FAST, MOVE_DISTANCE_MED_SHORT);
					}
					m_CollisionState = FORWARD2;
					ROBOT_DISPLAY( TRUE, "Collision State: FORWARD2" )
				}
				break;

				case FORWARD2:	// CARBOT
				{
					// Wait until we are done going forward
					if( m_pDriveCtrl->MoveDistanceCompleted() )
					{
						// Done recovering from collision. Ready to resume where we left off.
						m_CollisionDirection = 0;
						m_CollisionState = IDLE;
						ROBOT_DISPLAY( TRUE, "Collision State: Done --> IDLE" )
						m_pDriveCtrl->ReleaseOwner( COLLISION_MODULE );
						return;
					}
				}
				break;

				default:
				{
					MsgString.Format( "ERROR! Illegal m_CollisionState = %d", m_CollisionState );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )
					m_CollisionState = IDLE;
					m_pDriveCtrl->ReleaseOwner( COLLISION_MODULE );
				}
			}	// Switch Collision State


			////////////////////////////////////////////////////////////////
			#else
				#error BAD ROBOT_TYPE
			#endif
			////////////////////////////////////////////////////////////////


		}	// Switch Status Ready
	}	// Switch UMsg

}
// End of CollisionModule



///////////////////////////////////////////////////////////////////////////////
//	MODULE: AvoidObjectModule
#define AVOIDANCE_FORWARD_SPEED		SPEED_FWD_MED_SLOW
#define AVOIDANCE_FORWARD_DISTANCE	MOVE_DISTANCE_SHORT
#define AVOIDANCE_TURN_SPEED		SPEED_FWD_MED_SLOW
#define DOOR_SPOTTING_DISTANCE_TENTH_INCHES				480
#define DOORWAY_MIN_CLEAR_AREA_DEPTH_TENTH_INCHES		300
#define DOORWAY_MIN_CLEAR_AREA_WIDTH_TENTH_INCHES		230	// 23 inches.  Robot is 22 inches wide, including arms!


CAvoidObjectModule::CAvoidObjectModule( CDriveControlModule *pDriveControlModule )
{
	m_pDriveCtrl = pDriveControlModule;
	m_AvoidanceState = DISABLED;
	m_WarningDirection = 0;
	m_PriorSpeed = SPEED_STOP;
	m_SideAvoidDistanceTenthInches = SIDE_THREAT_NORM_THRESHOLD;
	gAvoidanceTimer = 0;
	m_LastObjectDirection = 0;
	m_pKinectServoControl = new KinectServoControl;

}

CAvoidObjectModule::~CAvoidObjectModule()
{
	ROBOT_LOG( TRUE, "~CAvoidObjectModule()\n" )
	SAFE_DELETE( m_pKinectServoControl );
}


void CAvoidObjectModule::ProcessMessage( UINT uMsg, WPARAM wParam, LPARAM lParam ) // AVOID_OBJECT_MODULE
{

	int  TurnAmount = 0;	// absolute amount to turn
	int	Turn = 0;			// Amount * direction
	int TurnLeft = 0;
	int TurnRight = 0;
	int Speed = 0;


	switch( uMsg )
	{
		case WM_ROBOT_ENABLE_AVOIDANCE_MODULE:
		{
			// Command from user to enable/disable the Avoidance Module
			g_bCmdRecognized = TRUE;
			// wParam = Enabled/Disabled
			// lParam = not used

			if( wParam )
			{
				// module enabled
				m_AvoidanceState = IDLE;
				ROBOT_DISPLAY( TRUE, "Avoidance Module Enabled" )
			}
			else
			{
				m_AvoidanceState = DISABLED;
				m_pDriveCtrl->ReleaseOwner( AVOID_OBJECT_MODULE );
				m_pKinectServoControl->SetTiltPosition( KINECT_TILT_OWNER_COLLISION_AVOIDANCE, KINECT_HUMAN_DETECT_START_POSITION );	// By default, leave Kinect in a position where it can find humans
				m_pKinectServoControl->ReleaseOwner(KINECT_TILT_OWNER_COLLISION_AVOIDANCE);

				ROBOT_DISPLAY( TRUE, "Avoidance Module Disabled" )
			}
			return;
		}
		break;

		case WM_ROBOT_SERVO_STATUS_READY:
		case WM_ROBOT_SENSOR_STATUS_READY:
		{
			g_bCmdRecognized = TRUE;
			//BYTE nDistHigh, nDistLow;
			CString MsgString;


			if( DISABLED == m_AvoidanceState )
			{
				// This module is disabled
				return;
			}

			if( !m_pDriveCtrl->IsOwner(AVOID_OBJECT_MODULE) )
			{
				// This module lost control of the drive wheels.  Go to a good state.
				m_AvoidanceState = IDLE;
			}

			if( (IDLE == m_AvoidanceState) &&
				(0 == m_pDriveCtrl->GetCurrentSpeed()) && 
				(0 == m_pDriveCtrl->GetCurrentTurn())  )
			{
				// Not currently moving.  
				// Ignore sensors so we don't suddenly start moving if someone obscures a sensor!
				// Also, leave Kinect in default position until robot starts moving

				// Move Kinect into position to detect people, if no other owner has control
				m_pKinectServoControl->SetTiltPosition( KINECT_TILT_OWNER_NONE, KINECT_OBJECT_AVOIDANCE_POSITION );

				return;
			}

			// Move Kinect into position to detect objects
			m_pKinectServoControl->SetTiltPosition( KINECT_TILT_OWNER_COLLISION_AVOIDANCE, KINECT_OBJECT_AVOIDANCE_POSITION );

			if( IDLE == m_AvoidanceState )
			{
				// This module was not in control at last status update
				// Save the last commanded speed
				m_PriorSpeed = m_pDriveCtrl->GetCurrentSpeed();
				Speed = m_PriorSpeed;	// Use this speed as default while avoiding.  May be overriden below.
			}
			else
			{
				// This module was in control at last status update
				// Restore the prior speed for use in calculations below
				Speed = m_PriorSpeed;
			}

			// Side threshold is controlled by a persistance to avoid thrashing
			// also used for narrow corridors and doorways
			if( gAvoidanceTimer != 0 ) 
			{
				m_SideAvoidDistanceTenthInches = SIDE_THREAT_MIN_THRESHOLD;
			}
			else
			{
				m_SideAvoidDistanceTenthInches = SIDE_THREAT_NORM_THRESHOLD;
				m_LastObjectDirection = 0;	// reset after timer expires  TODO REMOVE THIS?
			}


			///////////////////////////////////////////////////////////////////
			// See if an object has been found that is closer then BOTH
			// the Global limit set in the GUI, AND the limit set for the current path segment

			// Avoid objects.  Turn while looking for a clear path

			int  AvoidObjectDistanceTenthInches = __min( (g_GlobalMaxAvoidObjectDetectionFeet * 120), g_SegmentAvoidObjectRangeTenthInches );

			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			#if SENSOR_CONFIG_TYPE == SENSOR_CONFIG_CARBOT

				if(	(g_pSensorSummary->nFrontObjectDistance <= AvoidObjectDistanceTenthInches)		||
					(g_pSensorSummary->nLeftSideZone < m_SideAvoidDistanceTenthInches)			||
					(g_pSensorSummary->nRightSideZone < m_SideAvoidDistanceTenthInches)		)
				{
					// Object in front or on the side within threshold area to avoid
					// Ignore current state, we always "reset" when an object to avoid is seen

					ROBOT_ASSERT(0); // TODO!  NEED TO IMPLEMENT THIS FOR CARBOT!
					/*****
					// TODO-CAR-MUST - 
					// This is old Carbot code.  currently using code below for both Carbot and Loki
					// need to determine if Carbot needs different code for it's 6 US sensors!

					// First, see if there is a clear path anywhere ahead.  
					// Order of these is important!  Must check forward first!
					if( g_pSensorSummary->nLeftFrontZone > AvoidObjectDistanceTenthInches )
					{
						return -10; // Clear path ahead Left!
					}
					if( g_pSensorSummary->nRightFrontZone > AvoidObjectDistanceTenthInches )
					{
						return 10; // Clear path ahead Right!
					}
					if( g_pSensorSummary->nLeftArmZone > AvoidObjectDistanceTenthInches )
					{
						return -20; // Clear path ahead Hard Left!
					}
					if( g_pSensorSummary->nRightArmZone > AvoidObjectDistanceTenthInches )
					{
						return 20; // Clear path ahead Hard Right!
					}

					// Ok, not so easy.  Try calculating potential threats
					int ThreatsLeft = 
						( (NO_OBJECT_IN_RANGE - g_pSensorSummary->nLeftFrontZone)*3 ) + 
						( (NO_OBJECT_IN_RANGE - g_pSensorSummary->nLeftArmZone)*2 ) + 
						( (NO_OBJECT_IN_RANGE - g_pSensorSummary->nLeftSideZone)    );

					int ThreatsRight = 
						( (NO_OBJECT_IN_RANGE - g_pSensorSummary->nRightFrontZone)*3 ) + 
						( (NO_OBJECT_IN_RANGE - g_pSensorSummary->nRightArmZone)*2 ) + 
						( (NO_OBJECT_IN_RANGE - g_pSensorSummary->nRightSideZone)    );

					return( ThreatsRight - ThreatsLeft );
					****/
				}

			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			#elif ( (SENSOR_CONFIG_TYPE == SENSOR_CONFIG_LOKI) || (SENSOR_CONFIG_TYPE == SENSOR_CONFIG_TURTLE) )

				Turn = 0;

				if(	(g_pSensorSummary->nFrontObjectDistance <= AvoidObjectDistanceTenthInches) ||
					(g_pSensorSummary->nSideObjectDistance < m_SideAvoidDistanceTenthInches) )
				{
					// Something to avoid

					// Tighen up the side distance; focus on avoiding current obstacle
					gAvoidanceTimer = THREAT_PERSISTANCE_TIME;
					// Slow down while manuvering
/**
					Speed = Speed - SPEED_FWD_SLOW;	// subtract one "speed level" from current speed
					if( Speed < SPEED_FWD_SLOW ) Speed = SPEED_FWD_SLOW;
					if( g_pSensorSummary->nFrontObjectDistance <= AVOIDANCE_SHARP_TURN_RANGE )
					{
						// Object is close!  A sharp turn is needed!
						ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Forcing sharp turn to avoid close object\n" )
						Speed = SPEED_FWD_SLOW;	// force almost a pivot turn
					}
**/
				}

				// Start Avoidance Behavior

				if( DetectAndHandleCliff(Turn, Speed) )
				{
					ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Avoiding Cliff\n" )
				}
				// Avoid very close objects in key zones
				else if( (g_pSensorSummary->nRightArmZone <= ARM_ZONE_THREAT_MIN_THRESHOLD) ||
						 (g_pSensorSummary->nLeftArmZone <= ARM_ZONE_THREAT_MIN_THRESHOLD) )
				{
					// close object about to hit an arm
					if( g_pSensorSummary->nRightArmZone < g_pSensorSummary->nLeftArmZone )
					{
						// closest object is on the right
						ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Turning Left to avoid object on Front Right Arm Zone\n" )
						Turn = TURN_LEFT_MED;	// hard turn
					}
					else
					{
						ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Turning Right to avoid object on Front Left Arm Zone\n" )
						Turn = TURN_RIGHT_MED;	// hard turn
					}
					Speed = SPEED_FWD_SLOW;	// force a pivot turn

				}
				else if( DetectAndHandleDoorway(Turn, Speed) )
				{
					ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: AVOID_OBJECT_MODULE: Heading for Doorway\n" )

				}				
				else if( g_pSensorSummary->nFrontObjectDistance <= AvoidObjectDistanceTenthInches )
				{
					// There is an object somewhere in front of us. 
					ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: FrontObjectDistance = %d, direction = %d\n",
						g_pSensorSummary->nFrontObjectDistance, g_pSensorSummary->nFrontObjectDirection )

					if( g_pSensorSummary->nFrontObjectDirection > 0 )
					{
						// Object to the Right, Turn Left
						ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Turning Left to avoid object on Front Right\n" )
						Turn = TURN_LEFT_MED_SLOW;
					}
					else if( g_pSensorSummary->nFrontObjectDirection < 0 )
					{
						// Object to the Left, Turn Right
						ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Turning Right to avoid object on Front Left\n" )
						Turn = TURN_RIGHT_MED_SLOW;
					}
					else
					{
						// Object Dead Ahead. Check side sensors for a hint
						if( 0 != g_pSensorSummary->nSideObjectDirection )
						{
							// In range of one of the side sensors ( and both not reading identical)
							if( g_pSensorSummary->nSideObjectDirection > 0 )
							{
								// Object to the Right, Turn Left
								ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Object Dead Ahead, turning Left (other object on right side)\n" )
								Turn = TURN_LEFT_MED_SLOW;
							}
							if( g_pSensorSummary->nSideObjectDirection < 0 )
							{
								// Object to the Left, Turn Right
								ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Object Dead Ahead, turning Right (other object on left side)\n" )
								Turn = TURN_RIGHT_MED_SLOW;
							}
							else
							{
								ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Object Dead Ahead!  What should I do?\n" )
								ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Slowing Down in case I crash!\n" )
								Speed = SPEED_FWD_MED_SLOW;
								//Turn = TURN_RIGHT_MED_SLOW; // when in doubt, turn right
							}
						}
					}	// End of object dead ahead
				}	// End object somewhere in front of us. 
				else if( g_pSensorSummary->nSideObjectDistance < m_SideAvoidDistanceTenthInches )
				{
					// No object ahead, but object on the side within threshold area to avoid
					if( g_pSensorSummary->nSideObjectDirection > 0 )
					{
						// Object to the Right, Turn Left
						Turn = TURN_LEFT_SLOW; // Leave speed, just a gentle turn
					}
					else
					{
						// Object is closer to the Left
						Turn = TURN_RIGHT_SLOW; 
					}
					ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Turn %d to avoid object on side\n", Turn )
				}


				////////////////////////////////////////////////////////////////////////
				// Now, do the speed and turn changes!
				if( 0 != Turn )
				{
					// Object found.  Avoidance is needed
					ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: Turn = %d, Speed = %d\n", Turn, Speed )
					m_pDriveCtrl->SetSpeedAndTurn( AVOID_OBJECT_MODULE, Speed, Turn );
					m_AvoidanceState = TURNING1;
//					ROBOT_DISPLAY( TRUE, "Avoidance State: TURNING1" )

				} 
				else if( TURNING1 == m_AvoidanceState )
				{
					// We had been turning to avoid an object, and the way is clear now.
					// Move forward a little before releasing control, so we don't just drive right back into the same object!
					m_pDriveCtrl->SetMoveDistance( AVOID_OBJECT_MODULE, Speed, 
						TURN_CENTER, AVOIDANCE_FORWARD_DISTANCE, DONT_STOP_AFTER);	// Don't stop when done, smoothly move to normal speed
					ROBOT_LOG( TRUE, "Avoidance Module - Avoided, now going Forward Straight for a bit\n" )
					m_AvoidanceState = FORWARD1;
					ROBOT_DISPLAY( TRUE, "Avoidance State: FORWARD1" )
				}
				else if( FORWARD1 == m_AvoidanceState )
				{
					// Wait for drive control to report that we are done moving forward
					if( m_pDriveCtrl->MoveDistanceCompleted() )
					{
						// Done moving forward, 
						// Done recovering from object Avoidance. Ready to resume where we left off.
						m_AvoidanceState = IDLE;
						ROBOT_DISPLAY( TRUE, "Avoidance State: Done --> IDLE" )
						m_pDriveCtrl->ReleaseOwner( AVOID_OBJECT_MODULE );
					}
				}
				else if( m_pDriveCtrl->IsOwner(AVOID_OBJECT_MODULE) )
				{
					// No objects to avoid, but this module has control from last time.  Release control now
					ROBOT_DISPLAY( TRUE, "Avoidance State: Done Avoiding Object" )
					m_AvoidanceState = IDLE;
					m_pDriveCtrl->ReleaseOwner( AVOID_OBJECT_MODULE );
				}
			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			#else
				#error BAD SENSOR_CONFIG_TYPE!  
			#endif
			///////////////////////////////////////////////////////////////////////////////////////////////////////////
		}	// end case WM_ROBOT_SENSOR_STATUS_READY
	}
}


BOOL CAvoidObjectModule::DetectAndHandleCliff( int &Turn, int &Speed )
{
	if( g_pSensorSummary->bLeftCliff || g_pSensorSummary->bRightCliff )
	{
		// Cliff detected!
		Speed = SPEED_STOP;	// force a pivot turn

		if( g_pSensorSummary->bLeftCliff && g_pSensorSummary->bRightCliff )
		{
			// Cliff on both sides  Oh no!  what to do?
			ROBOT_DISPLAY( TRUE, "AVOID_OBJECT_MODULE: ERROR! Cliff on Both Sides!!!\n" )
			//g_ClientTextToSend = _T( "Error! Cliff on both sides") );	
			//SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
			Turn = TURN_CENTER;	// NO turn, just stop!
		}
		else if( g_pSensorSummary->bLeftCliff )
		{		
			ROBOT_DISPLAY( TRUE, "AVOID_OBJECT_MODULE: Cliff Left Side!\n" )
			//g_ClientTextToSend = "Cliff Left" );	
			//SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
			Turn = TURN_RIGHT_MED_SLOW;
		}
		else if( g_pSensorSummary->bRightCliff )
		{		
			ROBOT_DISPLAY( TRUE, "AVOID_OBJECT_MODULE: Cliff Right Side!\n" )
			//g_ClientTextToSend = "Cliff Right" );	
			//SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
			Turn = TURN_LEFT_MED_SLOW;
		}
		else
		{
			// Logic Error
			ROBOT_ASSERT(0);
		}
		return TRUE; // Cliff detected
	}
	else
	{
		return FALSE; // no Cliff
	}
}

/////////////////////////////////////////////////////////////////
BOOL CAvoidObjectModule::DetectAndHandleDoorway( int &Turn, int &Speed )
{
	// This function only gets called if there is an object in the RightFrontSideZone and LeftFrontSideZone
	// Since both are obstructed, look for a gap between them big enough for the robot to fit.
	// Returns TRUE if a doorway was found.  Note, also works for corridors.

	int  NumberOfSamples = 0;
	int RightEdgeX = 0;
	int LeftEdgeX = 0;
	int X, Y;
	int PreviousX = 0;

	if( (g_pSensorSummary->nLeftFrontSideZone  > DOOR_SPOTTING_DISTANCE_TENTH_INCHES) ||
		(g_pSensorSummary->nRightFrontSideZone > DOOR_SPOTTING_DISTANCE_TENTH_INCHES) ) 
	{
		return FALSE; // no door frame found in range
	}
	int TargetClearDistance = (__max(g_pSensorSummary->nRightFrontSideZone, g_pSensorSummary->nLeftFrontSideZone) + DOORWAY_MIN_CLEAR_AREA_DEPTH_TENTH_INCHES) ; // tenth inches

	NumberOfSamples = g_pLaserScannerData->NumberOfSamples;
	if( NumberOfSamples <= 0 )
	{
		return FALSE; // Bad data
	}

	for( int i = 0; i < (int)NumberOfSamples; i++ ) // Scanner goes Right to Left
	{
		X = g_pLaserScannerData->ScanPoints[i].X ; // tenth inches
		Y = g_pLaserScannerData->ScanPoints[i].Y ;

		if( (X <= (HALF_ROBOT_BODY_WITH_ARMS_WIDTH_TENTH_INCHES+FRONT_SIDE_ZONE_WIDTH_TENTH_INCHES))  &&
			(X >= -(HALF_ROBOT_BODY_WITH_ARMS_WIDTH_TENTH_INCHES+FRONT_SIDE_ZONE_WIDTH_TENTH_INCHES)) )
		{
			// point is inside RightFrontSideZone to LeftFrontSideZone
			if( 0 == RightEdgeX )
			{ 
				// looking for right edge
				if( Y >= TargetClearDistance )
				{	// Found inside of door edge!
					RightEdgeX = PreviousX;	// Use the last valid data
				}
			}
			else
			{
				// Right edge found, looking for left edge
				if( Y < TargetClearDistance )
				{
					if( (RightEdgeX - X) >= DOORWAY_MIN_CLEAR_AREA_WIDTH_TENTH_INCHES )
					{
						// Found it!
						LeftEdgeX = X;
						break;
					}
					else
					{
						// too narrow, restart search
						RightEdgeX = 0;
					}
				}
			}
			PreviousX = X;

		}	// inside zone if interest
	}	// for loop

	if( 0 != LeftEdgeX )
	{
		// Found Doorway
		int DoorwayWidth = RightEdgeX - LeftEdgeX;
		int DoorwayCenter = RightEdgeX - (DoorwayWidth/2);
		Turn = (DoorwayCenter / 5);		// /10 tenth inches  * 2 for more turn 
/*		if( DoorwayCenter > 50 )		// tenth inches off course
			Turn = TURN_RIGHT_MED_SLOW;
		else if( DoorwayCenter > 10 )
			Turn = TURN_RIGHT_SLOW;	
		else if( DoorwayCenter < -10 )
			Turn = TURN_LEFT_SLOW;	
		else if( DoorwayCenter < -50 )
			Turn = TURN_LEFT_MED_SLOW;	
		else
			Turn = TURN_CENTER;
*/
		ROBOT_LOG( TRUE, "**********************************\n" )
		ROBOT_LOG( TRUE, "AVOID_OBJECT_MODULE: DOORWAY Found - Center = %d, Width = %d Inches\n", DoorwayCenter/10, DoorwayWidth/10 )
		ROBOT_LOG( TRUE, "**********************************\n" )

		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


// End of AvoidObjectModule

