// ObjectKnowledge.h: ObjectKnowledge class implementation
// Stores information about known physical objects in the world 
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ClientOrServer.h"
#if ( ROBOT_SERVER == 1 )	// This module used for Robot Server only

#include <math.h>
#include "Globals.h"
//#include "module.h"
//#include "thread.h"
#include "ObjectKnowledge.h"

#define OBJECT_PATH_PREFIX ROBOT_DATA_PATH "\\KnownObjects\\"

///////////////////////////////////////////////////////////////////////////////
// ObjectKnowledge


ObjectKnowledge::ObjectKnowledge()
{
	m_initialized =	FALSE;


	ROBOT_LOG( TRUE,  "Constructor complete for ObjectKnowledge\n" )
}

ObjectKnowledge::~ObjectKnowledge()
{
	// Release resources
	DeleteAllObjects();

	ROBOT_LOG( TRUE,  "~ObjectKnowledge done\n" )
}

void ObjectKnowledge::Init()
{
	if( m_initialized )
	{
		ROBOT_LOG( TRUE,  "ObjectKnowledge::Init: ERROR! - Already initialized!\n" )
		return;
	}

	ROBOT_LOG( TRUE,  "ObjectKnowledge: Init: Loading Known Objects\n" )
	LoadKnownObjects( DEFAULT_KNOWN_OBJECTS_FILE );
	m_initialized = TRUE;

	ROBOT_LOG( TRUE,  "ObjectKnowledge: Init complete\n" )
}

ObjectInfo* ObjectKnowledge::GetObjectAt( POSITION Pos ) 
{
	// Return the ObjectInfo from the CTypedPtrList
	ObjectInfo *pObjectInfo = m_ObjectInfoList.GetAt( Pos );
	return pObjectInfo;
}


void ObjectKnowledge::AddObjectToList( ObjectInfo* pObjectInfo ) 
{
	// Add new ObjectInfo structure to the CTypedPtrList
	m_ObjectInfoList.AddTail(pObjectInfo);
}

/***
// Walk the list

	ObjectInfo* pObjectInfo;

	POSITION pos = m_ObjectInfoList.GetHeadPosition();
	while (pos != NULL)
	{
		pObjectInfo = m_ObjectInfoList.GetNext(pos);
		CString foo = pObjectInfo->foo;
		CString bar = pObjectInfo->bar;
	}


***/


void ObjectKnowledge::DeleteAllObjects() 
{
	// Delete all of the objects pointed to
	// by the CTypedPtrList. Then remove all of the
	// CSegmentStruct pointers from the CTypedPtrList, which
	// is faster than removing each individually.
	POSITION pos = m_ObjectInfoList.GetHeadPosition();
	while (pos != NULL)
	{
		delete m_ObjectInfoList.GetNext(pos);
	}

	m_ObjectInfoList.RemoveAll();

}


// Get Known Objects from the text file
//-----------------------------------------------------------------------------
// Name: LoadKnownObjects
// Desc: Load Known Objects from a text file
// NOTES: Expects following format of file:
// :F File Name for image
// :O Name of object
// :G Generic type of object
// :C Comment
// :P Prompt to ask when object found
// :N Response to "No"
// :Y Response to "Yes"
// :I ID <-- Must be the last parameter!
// Note that ID is at the *end* of each data block.  ID is the trigger to save the info.
//-----------------------------------------------------------------------------
void ObjectKnowledge::LoadKnownObjects( char* FileName )
{
	FILE		*File, *TempFile;
	char		buffer[255];
	char		FieldType;
	char		*tmpStr = 0;

	ROBOT_LOG( TRUE,  "\n\n =================================================================\n" )
	ROBOT_LOG( TRUE,  "ObjectKnowledge::LoadKnownObjects: Loading from File (%s)\n", FileName)

	// Open the file for reading
	errno_t nError;
	nError = fopen_s( &File, FileName, "r" );
	if( 0 != nError )
	{
		ROBOT_LOG( TRUE,  "ObjectKnowledge::LoadKnownObjects: ERROR!! Could Not open Import File (%s)!\n", FileName )
		return;
	}

	ROBOT_LOG( TRUE,  "ObjectKnowledge::LoadKnownObjects: Processing Import File (%s)!\n", FileName )

	// Ok, now get data
	ObjectInfo* pObjectInfo = new ObjectInfo();	// ObjectInfo holds one object

	int nIndex = 0;
	while( TRUE )
	{
		tmpStr = fgets(buffer, sizeof(buffer), File);

		// KLUDGE NEEDED  TO MAKE WINDOWS LAZY READ WORK!  SHEESH!!!!
		Sleep(2);

		if( NULL == tmpStr )
		{
			// EOF?
			int Eof = feof( File );
			if( 0 == Eof )
			{
				ROBOT_LOG( TRUE,  "CMatchObjectDetector: ERROR - Aborted before End of File.\n" )
			}
			int FileError = ferror( File );
			if( 0 != FileError )
			{
				ROBOT_LOG( TRUE,  "CMatchObjectDetector: FILE ERROR = %d\n\n", FileError)
				ROBOT_ASSERT(0);
			}
			break;
		}

//		ROBOT_LOG( TRUE,  "     DEBUG READ: (%s)\n", tmpStr)

		if( 10 == tmpStr[0] )	// Carriage Return
		{			
			//ROBOT_LOG( TRUE,  "BLANK LINE\n", tmpStr)
			continue;	// Blank line
		}


		// Remove the carriage return for all other tests
		tmpStr[strlen(tmpStr)-1] = '\0';


		if( 0 == strncmp(tmpStr, "//", 2) )
		{			
			ROBOT_LOG( TRUE,  "%s\n", tmpStr)	// Print the comment
			continue;
		}

		if( ':' == tmpStr[0] )
		{
			// Begin Field
			FieldType = tmpStr[1];
//			if( ('X' != FieldType) &&	// Not receiving XY data
//				(0 != nStrokes)		)	// But not all strokes were read
//			{
//				ROBOT_LOG( TRUE,  "ERROR! Not all expected Strokes were found!\n" )
//				ROBOT_ASSERT(0);
//			}

			switch( FieldType )
			{
				case'X':	// EXecute - Marks the end of the object descripton.  Triggers save of the data read so far 
				{	
					pObjectInfo->nIndex = nIndex++;	// Use order loaded for the Index
					ROBOT_LOG( TRUE,  "Object Index: %d\n\n", pObjectInfo->nIndex)

					// Save the prior object in the list					
					AddObjectToList( pObjectInfo );
					// create the next one
					pObjectInfo = new ObjectInfo();
					pObjectInfo->Descriptors = 0;
					pObjectInfo->Keypoints = 0;
					pObjectInfo->Storage = 0;
					pObjectInfo->nIndex = -1;
					break;
				}
				case 'F':	// File name of image to compare 
				{	
					pObjectInfo->strImageFileName = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "Filename    : %s\n", pObjectInfo->strImageFileName)
					CString TempFileName;
					TempFileName = OBJECT_PATH_PREFIX+pObjectInfo->strImageFileName;
					//FILE *TempFile;
					errno_t nError;
					nError = fopen_s( &TempFile, TempFileName, "r" );
					if( 0 != nError )
					{
						ROBOT_LOG( TRUE,  "\n\n==================================================================================\n" )
						ROBOT_LOG( TRUE,  "LoadKnownObjects: ERROR!! BAD IMAGE FILE (%s)!\n", TempFileName )
						ROBOT_LOG( TRUE,  "==================================================================================\n\n" )
					}
					else
					{
						// TODO - Do something here?
						fclose( TempFile );
					}
					break;
				}
				case 'O':	// Object Name (ex: styx CD)
				{	
					pObjectInfo->strObjectName = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "Object Name : %s\n", pObjectInfo->strObjectName)
					break;
				}
				case 'G':	// Generic Name of object (ex: Music CD)
				{	
					pObjectInfo->strGenericObjectName = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "Generic Name: %s\n", pObjectInfo->strGenericObjectName)
					break;
				}
				case 'C':	// Comment about the object TODO:  allow multiple comments, and pick one???
				{	
					pObjectInfo->strComment = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "Comment     : %s\n", pObjectInfo->strComment)
					break;
				}
				case 'P':	// Prompt to ask user about the object.  If none, use NOP
				{	
					pObjectInfo->strPrompt = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "Prompt      : %s\n", pObjectInfo->strPrompt)
					break;
				}
				case 'N':	// Response to user saying "No" 
				{	
					pObjectInfo->strUserResponseNo = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "No Response : %s\n", pObjectInfo->strUserResponseNo)
					break;
				}
				case 'Y':	// Response to user saying "Yes"
				{	
					pObjectInfo->strUserResponseYes = tmpStr+3;	// trim out the FieldType
					ROBOT_LOG( TRUE,  "Yes Response: %s\n", pObjectInfo->strUserResponseYes)
					break;
				}
				default:
				{
					ROBOT_LOG( TRUE,  "ERROR! ImportFromTextFile: Unknown Field type (%s)\n", tmpStr )
//					GridmapObjectType = GRIDMAP_OBJECT_NONE;
//					bGridMapItem = FALSE;
					ROBOT_ASSERT(0);
				}
			}
			continue;
		}
		else
		{
			ROBOT_LOG( TRUE,  "BAD LINE: {%s}\n", tmpStr)
		}
	}

//	SetModifiedFlag( );    // Mark document as modified to confirm File Close.
//	UpdateAllViews(NULL, 0L, NULL);	// Update the GUI View
	if( fclose( File ) )
	{
		ROBOT_LOG( TRUE,  "Error Closing Export File (%s)!\n", FileName )
	}

	// Delete the last Object, since no data got added to it
	if( pObjectInfo )
		delete pObjectInfo;

	ROBOT_LOG( TRUE,  "\n\n =================================================================\n" )

}








//-----------------------------------------------------------------------------
// Name: SaveToTextFile
// Desc: Save all Known Objects to a text file for persistance
//-----------------------------------------------------------------------------
void ObjectKnowledge::SaveToTextFile( char* FileName )
{
	IGNORE_UNUSED_PARAM (FileName);

/*****

	FILE *File;
	CTypedPtrList<CObList, CStroke*> *strokeList;
	CRect	StrokeBB;	// smallest rect that surrounds all of the points in the stroke
	UINT	StrokePenWidth;	// One width applies to entire stroke
	char	ObjectTypeChar;
	int		DebugStrokeNumber = 1;

	int LinesWritten = 0;

	// Open the file for writing
    FILE *File;
	errno_t nError;
	nError = fopen_s( &File, FileName, "w" );
	if( 0 != nError )
	{
		ROBOT_LOG( TRUE,  "Could Not open Export File (%s)!\n", FileName )
		return;
	}

	// Ok, now dump the map data
	// We only care about User Strokes, Walls, Objects, etc.

	// ":" = Field Start (Example, :Walls)

	fprintf(File, "\n// Exported Map Data\n" );
	ObjectTypeChar = 'Z';	// error flag

	for(int i=0; i<=5; i++)
	{
		// Note - i does not need to map to the stroke type, it's just an iterator
		// in general draw from solid objects to less solid
		if( 0 == i)	// Draw Walls 
		{
			fprintf(File, "// Wall Strokes\n" );
			strokeList = &m_WallList;
			ObjectTypeChar = 'W';
		}
		else if( 1 == i)	// Cliffs
		{	
			fprintf(File, "// Cliff Strokes\n" );
			strokeList = &m_CliffList;
			ObjectTypeChar = 'C';
		}
		else if( 2 == i)	// Obstacles
		{	
			fprintf(File, "// Obstacle Strokes\n" );
			strokeList = &m_ObstacleList;
			ObjectTypeChar = 'O';
		}
		else if( 3 == i)	// Low Obstacles
		{	
			fprintf(File, "// Low Obstacle Strokes\n" );
			strokeList = &m_LowObstacleList;
			ObjectTypeChar = 'L';
		}
		else if( 4 == i)	// Doors
		{	
			fprintf(File, "// Door Strokes\n" );
			strokeList = &m_DoorList;
			ObjectTypeChar = 'D';
		}
		else if( 5 == i)	// User scribbles
		{	
			fprintf(File, "// User Strokes\n" );
			strokeList = &m_strokeList;
			ObjectTypeChar = 'U';
		}
		else
		{
			ROBOT_ASSERT(0);
		}

		POSITION pos = strokeList->GetHeadPosition( );
		while (pos != NULL)
		{
			CStroke* pStroke = strokeList->GetNext(pos);
			StrokeBB = pStroke->GetBoundingRect();
			StrokePenWidth = pStroke->GetPenWidth();

			// Note: No need to store BB!

			// Stroke Info -  Pen Width, number of lines is appended to the object type
			fprintf(File, "// Item %d\n", DebugStrokeNumber++ );
			fprintf(File, ":%c %d,%d\n", ObjectTypeChar, StrokePenWidth, pStroke->m_pointArray.GetSize() );
			LinesWritten++;

			// Export the series of line segments in a stroke
			//fprintf(File, "// Stroke Lines - x,y, x,y, x,y, ...\n" );
			//LinesWritten++;
			for( int i=0; i < pStroke->m_pointArray.GetSize(); i++ )
			{
				fprintf(File, ":XY %d,%d\n", pStroke->m_pointArray[i].x, pStroke->m_pointArray[i].y );
				LinesWritten++;
			}
			fprintf(File, "\n" );
		}
	}

	fprintf(File, "// End of File\n" );
	fprintf(File, "///////////////////////////////\r\n" );
	ROBOT_LOG( TRUE,  "Export to file (%s):  %d Lines Written!\n", FileName, LinesWritten )
	CString strStatus;
	strStatus.Format("Export to file (%s):  %d Lines Written!\n", FileName, LinesWritten );
	ROBOT_DISPLAY( TRUE, (strStatus)

	if( fclose( File ) )
	{
		ROBOT_LOG( TRUE,  "Error Closing Export File (%s)!\n", FileName )
	}

***/
}






#endif // ROBOT_SERVER - This module used for Robot Server only


