#ifndef __ROBOT_SHARED_PARAMS_H__
#define __ROBOT_SHARED_PARAMS_H__

#include "RobotType.h"

// Common defines used by Robot Client and Server
// But not required by specific hardware, such as PIC

#define SERVER_PORT    50000
#define CLIENT_PORT    50001

#define DEFAULT_PATH_FILE			ROBOT_DATA_PATH "\\DefaultPath.pat"
#define DEFAULT_MAP_FILE			ROBOT_DATA_PATH "\\BlankMap.rmp"
#define DEFAULT_UPSTAIRS_MAP_FILE	ROBOT_DATA_PATH "\\Upstairs.rmp"
#define DEFAULT_DOWNSTAIRS_MAP_FILE	ROBOT_DATA_PATH "\\Downstairs.rmp"
#define DEFAULT_KNOWN_OBJECTS_FILE	ROBOT_DATA_PATH "\\KnownObjects\\KnownObjects.txt"
#define DEFAULT_KNOWN_OBJECTS_DIR	ROBOT_DATA_PATH "\\KnownObjects\\"
#define DEFAULT_STEREO_CALIB_FILE	ROBOT_DATA_PATH "\\StereoData\\CalibrationList.txt"

#define GPS_TENTH_INCHES_PER_DEGREE_LAT		43859650	// West Coast

//#define GPS_INCHES_PER_DEGREE_LONG		29505780	// Seattle, WA:
#define GPS_TENTH_INCHES_PER_DEGREE_LONG	30673280	// Portland, OR:
//#define GPS_INCHES_PER_DEGREE_LONG		34592100	// San Francisco:

// General State defines
#define OFF									0x00
#define ON									0x01
#define AUTO								0x02

#define REVERSE								0x00
#define FORWARD								0x01
#define LEFT								0x00
#define RIGHT								0x01

#define NORTH								0x00
#define SOUTH								0x01
#define EAST								0x02
#define WEST								0x03

#define SUPPRESS							0x00
#define UNSUPPRESS							0x01

// Parameter for WM_ROBOT_USER_OVERRIDE_CMD
//#define SET_USER_RELEASED				0x00						
#define SET_USER_NORMAL					0x01
#define SET_USER_OVERRIDE				0x02
#define SET_USER_OVERRIDE_AND_STOP		0x03
//#define SET_USER_OVERRIDE_STATE_SENT	0x04	// Special state just for keypress tracking


/////////////////////////////////////////////////////////////////////////////
enum WM_ROBOT_MESSAGES { 

	// Commands To Module (included PIC responses)
	// No ACK Required
		WM_ROBOT_MESSAGE_BASE = WM_APP,				// Start at App Base message
		WM_ROBOT_REQUEST_STATUS_CMD,				// WM_ROBOT_MESSAGE_BASE + 0x01	// Usually Sent directly to HW thread, except simulation
		WM_ROBOT_REQUEST_VERSION_CMD,				// Request version from PIC
		WM_ROBOT_JOYSTICK_DRIVE_CMD,
		WM_ROBOT_DRIVE_LOCAL_CMD,					// FUTURE: User has local, direct control of the robot, 
		WM_ROBOT_DRIVE_REMOTE_CMD,					// FUTURE: or is remote (can't see robot directly)
		WM_ROBOT_USER_OVERRIDE_CMD,					// Override command for User control of the robot (PANIC stop)
		WM_ROBOT_GET_SERVO_STATUS,				

		// HW Responses
		WM_ROBOT_SENSOR_STATUS_READY,				
		WM_ROBOT_PIC_VERSION_READY,				
		WM_ROBOT_GPS_DATA_READY,					
		WM_ROBOT_ER1_ODOMETER_READY,				
		WM_ROBOT_SERVO_STATUS_READY,				

		WM_ROBOT_MOVE_SET_DISTANCE_CMD,				// wParam = distance in tenth inches, lParam = direction
		WM_ROBOT_TURN_SET_DISTANCE_CMD,				// wParam = distance in degrees, lParam = Direction and speed
		WM_ROBOT_RESET_WATCHDOG_CMD,			
		WM_ROBOT_12V_POWER_CMD,				
		WM_ROBOT_CLIENT_KEEP_ALIVE_CMD,		

		WM_ROBOT_SERVO_CMD,					
		WM_ROBOT_LIGHT_POWER_CMD,			
		WM_ROBOT_AUX_LIGHT_POWER_CMD,		
		WM_ROBOT_ENABLE_OBJECT_NAV_UPDATE,	
		WM_ROBOT_GOTO_WAYPOINT_CMD,			
		WM_ROBOT_SPEAK_TEXT,					
		WM_ROBOT_EXECUTE_PATH,				
		WM_ROBOT_RESET_ODOMETER,				
		WM_ROBOT_BRAKE_CMD,					
		WM_ROBOT_OPEN_DATA_FILE,					// wParem = Path type, LParam = Map type.  Request Server to open the files
		WM_ROBOT_SET_GEAR_CMD,				
		WM_ROBOT_SENSOR_DATA,				
		WM_ROBOT_PLAY_SOUND,				
		WM_ROBOT_PLAY_MUSIC,				
		WM_ROBOT_TEXT_TO_AI,			
		WM_ROBOT_ENABLE_DYNA_CONTROLLER_CMD,	

		WM_ROBOT_THREAD_EXIT_CMD,					// Tell all threads to exit
		WM_ROBOT_IR_SENSOR_POWER_CMD,	
		WM_ROBOT_POWER_MODE_CMD,					// wParam = Arm/Head Power, lParam = Laptop mode. See LAPTOP_POWER_MODE_DEFAULT
		WM_ROBOT_SET_KINECT_POSITION,				// BULK servo command.  wParam not used. lParam = Set Speed too (True/False) 
		WM_ROBOT_KINECT_POWER_ENABLE_CMD,			// wParam not used, lParam = Enable/Disable
		// add more here...

		WM_ROBOT_CAMERA_POWER_CMD,					// User Pan/Tilt commands
		WM_ROBOT_CAMERA_INITIALIZE_CMD,			
		WM_ROBOT_USER_CAMERA_PAN_CMD,		
		WM_ROBOT_USER_CAMERA_ZOOM_CMD,		
		WM_ROBOT_SET_HEAD_POSITION,					// BULK Head command.  wParam not used. lParam = Set Speed too (True/False) 
		WM_ROBOT_USER_CAMERA_PAN_TILT_SPEED_CMD,	// wParam = speed

		// Absolute Pan/Tilt Commands
		WM_ROBOT_CAMERA_PAN_ABS_CMD,				// wParam = value
		WM_ROBOT_CAMERA_TILT_ABS_CMD,				// wParam = value
		// Relative Pan/Tilt Commands
		WM_ROBOT_CAMERA_PAN_REL_CMD,				// wParam = value
		WM_ROBOT_CAMERA_TILT_REL_CMD,				// wParam = value
		// Other Camera commands
		WM_ROBOT_CAMERA_ZOOM_ABS_CMD,				// wParam = zoom level.  Range 0 to 16.
		WM_ROBOT_CAMERA_MODE_CMD,					// wParam = mode, lParam = value
		WM_ROBOT_CAMERA_SIDETILT_ABS_CMD,			// wParam = value
		WM_ROBOT_CAMERA_POINT_TO_XYZ_CMD,			// wParam = X,Z, lParam = Y -- point to XYZ position in space (Y is dist from robot)

		WM_ROBOT_KINECT_SEARCH_FLOOR_CMD,			// wParam:  TRUE = Find Close Objects Only
		WM_ROBOT_KINECT_TRACK_OBJECT_CMD,			// 


		WM_ROBOT_FIND_OBJECT_AT_XYZ_CMD,			// wParam = X,Z, lParam = Y -- point to XYZ position in space (Y is dist from robot)
		WM_ROBOT_SLAM_CHECK_CMD,					// wParam n/a, lParam n/a
		WM_ROBOT_DO_LASER_SCANS,					// wParam = number of scans
		WM_ROBOT_ENABLE_COLLISION_MODULE,	
		WM_ROBOT_ENABLE_AVOIDANCE_MODULE,	
		WM_ROBOT_SET_AVOID_OBJ_RANGE,		
		WM_ROBOT_SET_SCAN_DISTANCE_CMD,		
		WM_ROBOT_SET_PATH_SPEED_INCREASE,	
		WM_ROBOT_SET_COMPASS_CAL_MODE,		
		WM_ROBOT_SET_COMPASS_CAL_POINT,		
		WM_ROBOT_ENABLE_RADAR_SCAN_CMD,		
		WM_ROBOT_ENABLE_CLIFF_SENSORS,		
		WM_ROBOT_ENABLE_GPS_PATH,			

		WM_ROBOT_GOTO_GRID_LOCATION_CMD,				
		WM_ROBOT_SET_BEHAVIOR_CMD,					// Select behavior mode for robot, lparam = behavior
		WM_ROBOT_SET_ACTION_CMD,					// Select Action mode.  lparam = mode
		WM_ROBOT_SET_LED_EYES_CMD,					// wParam = mode, lParam = brightness
		WM_ROBOT_SET_CURRENT_LOCATION,				// wParem = X, lParam = Y

		WM_ROBOT_SPEECH_RECO_SET_CMD_MODE,			// wParam = bEnable.  Commands to enable Speech Reco Command mode (vs AI)

		WM_ROBOT_SET_ARM_POSITION,					// wParam = Right/Left Arm, lParam = Set Speed too (True/False)
		WM_ROBOT_SET_ARM_DEFAULT_SPEED,				// wParam = Arm, lParam = Value
		WM_ROBOT_GET_SMART_SERVO_STATUS,				// wParam = not used, lParam = Get Speed too (True/False)
		WM_ROBOT_SET_SERVO_TORQUE_ENABLE,			// Uses bitfields: wParam = Operation, lParam = Enable/Disable
		WM_ROBOT_SET_SERVO_TORQUE_LIMIT,				// wParam = ServoID, lParam = Torque Limit
		WM_ROBOT_SET_ARM_MOVEMENT,					// Complex Arm movements.  wParam = Operation, lParam = Left/Right

		WM_ROBOT_ENABLE_RECO,					
		WM_ROBOT_DICTATION_RECO_EVENT,			
		WM_ROBOT_COMMAND_RECO_EVENT,				
		WM_ROBOT_SPEAKING_COMPLETE_EVENT,		

		WM_ROBOT_CAMERA_ENABLE_FEATURE,			
		WM_ROBOT_COLOR_BLOB_AUTO_CAL_CMD,		
		WM_ROBOT_COLOR_BLOB_MANUAL_CAL_CMD,		
		WM_ROBOT_COLOR_BLOB_LOOK_AHEAD_CMD,		
		WM_ROBOT_COLOR_BLOB_SEARCH_CMD,			
		WM_ROBOT_SET_CAMERA_COLOR_THRESHOLD,		
		WM_ROBOT_CAMERA_LOOK_AHEAD_CMD,			
		WM_ROBOT_CAMERA_BEHAVIOR_MODE,			

		WM_ROBOT_USER_CAMERA_FOWARD,				
		WM_ROBOT_USER_CAMERA_SIDETILT,			
		WM_ROBOT_CAMERA_TAKE_SNAPSHOT,				// wParam, lParam not used
		WM_ROBOT_CAMERA_RECORD_VIDEO,				// wParam: True = start, False = Stop.  lParam not used

		// Bulk Commands from Remote GUI
		WM_ROBOT_TEXT_MESSAGE_TO_SERVER,			
		WM_ROBOT_BULK_DATA_TO_SERVER,			

		WM_ROBOT_CAMERA_MATCH_OBJECT,				// Match objects on file to the current frame.  Respond with WM_ROBOT_CAMERA_MATCH_RESPONSE
		WM_ROBOT_CAMERA_MATCH_RESPONSE,				// wParam: ObjectID, lParam: Location in Frame.  
		WM_ROBOT_LASER_SCAN_DATA_READY,				// from LaserScannerParser
		WM_ROBOT_KINECT_SEARCH_COMPLETE,				// wParam = ObjectFound (T/F)

		//////// GUI Display messages Only! Commands >= to this are ignored by Module!!!
		// See HW_MAX_MESSAGE in HardwareCmds.h!
		WM_GUI_MESSAGES,								
		WM_ROBOT_SEND_TEXT_MESSAGES,				
		WM_ROBOT_DISPLAY_STATUS_MESSAGES,		
		WM_ROBOT_DISPLAY_SINGLE_ITEM,				// Display a single item on the remote client GUI
		WM_ROBOT_DISPLAY_BULK_ITEMS,				// Display lots of data on the remote client GUI
		WM_ROBOT_REMOTE_GUI_CMD,					// Commands from Remote GUI, for camera or other things that need a window
		WM_ROBOT_DISPLAY_TCP_TIME,				
		WM_ROBOT_DISPLAY_OPEN_DATA_FILE,			// Does the REAL file open, in the display thread!
		WM_ROBOT_CLIENT_CONNECT,				
		WM_ROBOT_CLIENT_DISCONNECT,			
		WM_ROBOT_OPEN_MAP_FILE,						// wParam: 0=Create New file, lParam =
		WM_ROBOT_ADD_WAYPOINT_LOCATION,				// Message from MapView to PathView to add a new Waypoint at given location
		WM_ROBOT_UPDATE_VIEW,						// Tell view to update ( server changed some data)
		WM_ROBOT_GET_CAMERA_SETTINGS,				// Tell GUI to send current camera settings to the Camera Module

	////////////////////////////////////////////////
	// Commands from Remote GUI to Server GUI
		ROBOT_REMOTE_ENABLE_CAMERA,						
		ROBOT_REMOTE_ENABLE_MOTION_VIEW,					
		WM_ROBOT_MAX_ROBOT_MESSAGE					// Max message ID used by this program (for noise filter on send messages)
};   // WM_ROBOT_MESSAGES


	// Server Response Messages
enum SERVER_RESPONSE_MESSAGES {
		ROBOT_RESPONSE_TEXT_MESSAGE = 0,
		ROBOT_RESPONSE_CONNECTED,	
		ROBOT_RESPONSE_RADAR_SAMPLE,	
		ROBOT_RESPONSE_IR_SAMPLE,		
		ROBOT_RESPONSE_PIC_DEBUG1,					// For debugging PIC code
		ROBOT_RESPONSE_PIC_DEBUG2,					// used for debugging new message code
		ROBOT_RESPONSE_PIC_WATCHDOG,				// Indicates watchdog expired or reset
		ROBOT_RESPONSE_PIC_VERSION,			
		ROBOT_RESPONSE_PIC_12V_POWER,		
		ROBOT_RESPONSE_PIC_BUMPER,					// state of bumper switches
		ROBOT_RESPONSE_PIC_LAST_ERROR,		
		ROBOT_RESPONSE_PIC_STATUS,
		ROBOT_RESPONSE_GPS_DATA,			
		ROBOT_RESPONSE_COLOR_BLOB_CAL_RESULT,
		ROBOT_RESPONSE_DRIVE_MODULE_OWNER,	
		ROBOT_RESPONSE_MOTOR_SPEED,					// From the ER1 Pilot controller
		ROBOT_RESPONSE_LASER_SCANNER_DATA,		
		ROBOT_RESPONSE_KINECT_DATA		
}; // SERVER_RESPONSE_MESSAGES


/////////////////////////////////////////////////////////////////////////////

#ifdef ASDFKASELKNVALK
/////////////////////////////////////////////////////////////////////////////
// Commands To Module (included PIC responses)
// No ACK Required
#define WM_ROBOT_MESSAGE_BASE				WM_APP
#define WM_ROBOT_REQUEST_STATUS_CMD			(WM_ROBOT_MESSAGE_BASE+0x01)	// Usually Sent directly to HW thread, except simulation
#define WM_ROBOT_REQUEST_VERSION_CMD		(WM_ROBOT_MESSAGE_BASE+0x02)	// Request version from PIC
#define WM_ROBOT_JOYSTICK_DRIVE_CMD			(WM_ROBOT_MESSAGE_BASE+0x03)
#define WM_ROBOT_USER_OVERRIDE_CMD			(WM_ROBOT_MESSAGE_BASE+0x04)	// Override command for User control of the robot
#define WM_ROBOT_GET_SERVO_STATUS			(WM_ROBOT_MESSAGE_BASE+0x05)

// HW Responses
#define WM_ROBOT_SENSOR_STATUS_READY			(WM_ROBOT_MESSAGE_BASE+0x06)
#define WM_ROBOT_PIC_VERSION_READY			(WM_ROBOT_MESSAGE_BASE+0x07)
#define WM_ROBOT_GPS_DATA_READY				(WM_ROBOT_MESSAGE_BASE+0x08)
#define WM_ROBOT_ER1_ODOMETER_READY			(WM_ROBOT_MESSAGE_BASE+0x09)
#define WM_ROBOT_SERVO_STATUS_READY			(WM_ROBOT_MESSAGE_BASE+0x0A)

#define WM_ROBOT_MOVE_SET_DISTANCE_CMD		(WM_ROBOT_MESSAGE_BASE+0x0B)	// wParam = distance in tenth inches, lParam = direction
#define WM_ROBOT_TURN_SET_DISTANCE_CMD		(WM_ROBOT_MESSAGE_BASE+0x0C)	// wParam = distance in degrees, lParam = direction and speed
#define WM_ROBOT_RESET_WATCHDOG_CMD			(WM_ROBOT_MESSAGE_BASE+0x0D)
#define WM_ROBOT_12V_POWER_CMD				(WM_ROBOT_MESSAGE_BASE+0x0E)
#define WM_ROBOT_CLIENT_KEEP_ALIVE_CMD		(WM_ROBOT_MESSAGE_BASE+0x0F)

#define WM_ROBOT_SERVO_CMD					(WM_ROBOT_MESSAGE_BASE+0x10)
#define WM_ROBOT_LIGHT_POWER_CMD			(WM_ROBOT_MESSAGE_BASE+0x11)
#define WM_ROBOT_AUX_LIGHT_POWER_CMD		(WM_ROBOT_MESSAGE_BASE+0x12)
#define WM_ROBOT_ENABLE_OBJECT_NAV_UPDATE	(WM_ROBOT_MESSAGE_BASE+0x13)
#define WM_ROBOT_GOTO_WAYPOINT_CMD			(WM_ROBOT_MESSAGE_BASE+0x14)
#define WM_ROBOT_SPEAK_TEXT					(WM_ROBOT_MESSAGE_BASE+0x15)
#define WM_ROBOT_EXECUTE_PATH				(WM_ROBOT_MESSAGE_BASE+0x16)
#define WM_ROBOT_RESET_ODOMETER				(WM_ROBOT_MESSAGE_BASE+0x17)
#define WM_ROBOT_BRAKE_CMD					(WM_ROBOT_MESSAGE_BASE+0x18)
#define WM_ROBOT_OPEN_DATA_FILE				(WM_ROBOT_MESSAGE_BASE+0x19)	// wParem = Path type, LParam = Map type.  Request Server to open the files
#define WM_ROBOT_SET_GEAR_CMD				(WM_ROBOT_MESSAGE_BASE+0x1A)
#define WM_ROBOT_SENSOR_DATA				(WM_ROBOT_MESSAGE_BASE+0x1B)
#define WM_ROBOT_PLAY_SOUND					(WM_ROBOT_MESSAGE_BASE+0x1C)
#define WM_ROBOT_PLAY_MUSIC					(WM_ROBOT_MESSAGE_BASE+0x1D)
#define WM_ROBOT_TEXT_TO_AI					(WM_ROBOT_MESSAGE_BASE+0x1E)
#define WM_ROBOT_ENABLE_DYNA_CONTROLLER_CMD	(WM_ROBOT_MESSAGE_BASE+0x1F)

#define WM_ROBOT_THREAD_EXIT_CMD			(WM_ROBOT_MESSAGE_BASE+0x20)	// Tell all threads to exit
#define WM_ROBOT_IR_SENSOR_POWER_CMD		(WM_ROBOT_MESSAGE_BASE+0x21)
#define WM_ROBOT_POWER_MODE_CMD				(WM_ROBOT_MESSAGE_BASE+0x22)	// wParam = Arm/Head Power, lParam = Laptop mode. See LAPTOP_POWER_MODE_DEFAULT
#define WM_ROBOT_SET_KINECT_POSITION		(WM_ROBOT_MESSAGE_BASE+0x23)	// BULK servo command.  wParam not used. lParam = Set Speed too (True/False) 
#define WM_ROBOT_KINECT_POWER_ENABLE_CMD	(WM_ROBOT_MESSAGE_BASE+0x24)	// wParam not used, lParam = Enable/Disable
// add more here...

#define WM_ROBOT_CAMERA_POWER_CMD			(WM_ROBOT_MESSAGE_BASE+0x2A)	// User Pan/Tilt commands
#define WM_ROBOT_CAMERA_INITIALIZE_CMD		(WM_ROBOT_MESSAGE_BASE+0x2B)
#define WM_ROBOT_USER_CAMERA_PAN_CMD		(WM_ROBOT_MESSAGE_BASE+0x2C)
#define WM_ROBOT_USER_CAMERA_ZOOM_CMD		(WM_ROBOT_MESSAGE_BASE+0x2D)
#define WM_ROBOT_SET_HEAD_POSITION			(WM_ROBOT_MESSAGE_BASE+0x2E)	// BULK Head command.  wParam not used. lParam = Set Speed too (True/False) 
#define WM_ROBOT_USER_CAMERA_PAN_TILT_SPEED_CMD	(WM_ROBOT_MESSAGE_BASE+0x2F)	// wParam = speed

// Absolute Pan/Tilt Commands
#define WM_ROBOT_CAMERA_PAN_ABS_CMD			(WM_ROBOT_MESSAGE_BASE+0x30)	// wParam = value
#define WM_ROBOT_CAMERA_TILT_ABS_CMD		(WM_ROBOT_MESSAGE_BASE+0x31)	// wParam = value
// Relative Pan/Tilt Commands
#define WM_ROBOT_CAMERA_PAN_REL_CMD			(WM_ROBOT_MESSAGE_BASE+0x32)	// wParam = value
#define WM_ROBOT_CAMERA_TILT_REL_CMD		(WM_ROBOT_MESSAGE_BASE+0x33)	// wParam = value
// Other Camera commands
#define WM_ROBOT_CAMERA_ZOOM_ABS_CMD		(WM_ROBOT_MESSAGE_BASE+0x34)	// wParam = zoom level.  Range 0 to 16.
#define WM_ROBOT_CAMERA_MODE_CMD			(WM_ROBOT_MESSAGE_BASE+0x35)	// wParam = mode, lParam = value
#define WM_ROBOT_CAMERA_SIDETILT_ABS_CMD	(WM_ROBOT_MESSAGE_BASE+0x36)	// wParam = value
#define WM_ROBOT_CAMERA_POINT_TO_XYZ_CMD	(WM_ROBOT_MESSAGE_BASE+0x37)	// wParam = X,Z, lParam = Y -- point to XYZ position in space (Y is dist from robot)

#define WM_ROBOT_KINECT_SEARCH_FLOOR_CMD	(WM_ROBOT_MESSAGE_BASE+0x38)	// wParam:  TRUE = Find Close Objects Only
#define WM_ROBOT_KINECT_TRACK_OBJECT_CMD	(WM_ROBOT_MESSAGE_BASE+0x39)	// 


#define WM_ROBOT_FIND_OBJECT_AT_XYZ_CMD		(WM_ROBOT_MESSAGE_BASE+0x3A)	// wParam = X,Z, lParam = Y -- point to XYZ position in space (Y is dist from robot)
#define WM_ROBOT_SLAM_CHECK_CMD				(WM_ROBOT_MESSAGE_BASE+0x3B)	// wParam n/a, lParam n/a
#define WM_ROBOT_DO_LASER_SCANS				(WM_ROBOT_MESSAGE_BASE+0x3C)	// wParam = number of scans
#define WM_ROBOT_ENABLE_COLLISION_MODULE	(WM_ROBOT_MESSAGE_BASE+0x40)
#define WM_ROBOT_ENABLE_AVOIDANCE_MODULE	(WM_ROBOT_MESSAGE_BASE+0x41)
#define WM_ROBOT_SET_AVOID_OBJ_RANGE		(WM_ROBOT_MESSAGE_BASE+0x42)
#define WM_ROBOT_SET_SCAN_DISTANCE_CMD		(WM_ROBOT_MESSAGE_BASE+0x43)
#define WM_ROBOT_SET_PATH_SPEED_INCREASE	(WM_ROBOT_MESSAGE_BASE+0x44)
#define WM_ROBOT_SET_COMPASS_CAL_MODE		(WM_ROBOT_MESSAGE_BASE+0x45)
#define WM_ROBOT_SET_COMPASS_CAL_POINT		(WM_ROBOT_MESSAGE_BASE+0x46)
#define WM_ROBOT_ENABLE_RADAR_SCAN_CMD		(WM_ROBOT_MESSAGE_BASE+0x47)
#define WM_ROBOT_ENABLE_CLIFF_SENSORS		(WM_ROBOT_MESSAGE_BASE+0x48)
#define WM_ROBOT_ENABLE_GPS_PATH			(WM_ROBOT_MESSAGE_BASE+0x49)

#define WM_ROBOT_GOTO_GRID_LOCATION_CMD		(WM_ROBOT_MESSAGE_BASE+0x4A)
#define WM_ROBOT_SET_BEHAVIOR_CMD			(WM_ROBOT_MESSAGE_BASE+0x4B)	// Select behavior mode for robot, lparam = behavior
#define WM_ROBOT_SET_ACTION_CMD				(WM_ROBOT_MESSAGE_BASE+0x4C)	// Select Action mode.  lparam = mode
#define WM_ROBOT_SET_LED_EYES_CMD			(WM_ROBOT_MESSAGE_BASE+0x4D)	// wParam = mode, lParam = brightness
#define WM_ROBOT_SET_CURRENT_LOCATION		(WM_ROBOT_MESSAGE_BASE+0x4E)	// wParem = X, lParam = Y

#define WM_ROBOT_SPEECH_RECO_SET_CMD_MODE	(WM_ROBOT_MESSAGE_BASE+0x4F)	// wParam = bEnable.  Commands to enable Speech Reco Command mode (vs AI)

#define WM_ROBOT_SET_ARM_POSITION			(WM_ROBOT_MESSAGE_BASE+0x50)	// wParam = Right/Left Arm, lParam = Set Speed too (True/False)
#define WM_ROBOT_SET_ARM_DEFAULT_SPEED		(WM_ROBOT_MESSAGE_BASE+0x51)	// wParam = Arm, lParam = Value
#define WM_ROBOT_GET_SMART_SERVO_STATUS		(WM_ROBOT_MESSAGE_BASE+0x52)	// wParam = not used, lParam = Get Speed too (True/False)
#define WM_ROBOT_SET_SERVO_TORQUE_ENABLE	(WM_ROBOT_MESSAGE_BASE+0x53)	// Uses bitfields: wParam = Operation, lParam = Enable/Disable
#define WM_ROBOT_SET_SERVO_TORQUE_LIMIT		(WM_ROBOT_MESSAGE_BASE+0x54)	// wParam = ServoID, lParam = Torque Limit
#define WM_ROBOT_SET_ARM_MOVEMENT			(WM_ROBOT_MESSAGE_BASE+0x55)	// Complex Arm movements.  wParam = Operation, lParam = Left/Right

#define WM_ROBOT_ENABLE_RECO				(WM_ROBOT_MESSAGE_BASE+0x56)
#define WM_ROBOT_DICTATION_RECO_EVENT		(WM_ROBOT_MESSAGE_BASE+0x57)
#define WM_ROBOT_COMMAND_RECO_EVENT			(WM_ROBOT_MESSAGE_BASE+0x58)
#define WM_ROBOT_SPEAKING_COMPLETE_EVENT	(WM_ROBOT_MESSAGE_BASE+0x59)


#define WM_ROBOT_CAMERA_CMDS				(WM_ROBOT_MESSAGE_BASE+0x60)	// Commands above this are for Camera Thread only
#define WM_ROBOT_CAMERA_ENABLE_FEATURE		(WM_ROBOT_MESSAGE_BASE+0x61)

#define WM_ROBOT_COLOR_BLOB_AUTO_CAL_CMD	(WM_ROBOT_MESSAGE_BASE+0x63)
#define WM_ROBOT_COLOR_BLOB_MANUAL_CAL_CMD	(WM_ROBOT_MESSAGE_BASE+0x64)
#define WM_ROBOT_COLOR_BLOB_LOOK_AHEAD_CMD	(WM_ROBOT_MESSAGE_BASE+0x65)
#define WM_ROBOT_COLOR_BLOB_SEARCH_CMD		(WM_ROBOT_MESSAGE_BASE+0x66)
#define WM_ROBOT_SET_CAMERA_COLOR_THRESHOLD	(WM_ROBOT_MESSAGE_BASE+0x67)
#define WM_ROBOT_CAMERA_LOOK_AHEAD_CMD		(WM_ROBOT_MESSAGE_BASE+0x68)
#define WM_ROBOT_CAMERA_BEHAVIOR_MODE		(WM_ROBOT_MESSAGE_BASE+0x69)

#define WM_ROBOT_USER_CAMERA_FOWARD			(WM_ROBOT_MESSAGE_BASE+0x6A)
#define WM_ROBOT_USER_CAMERA_SIDETILT		(WM_ROBOT_MESSAGE_BASE+0x6B)

#define WM_ROBOT_CAMERA_TAKE_SNAPSHOT		(WM_ROBOT_MESSAGE_BASE+0x6C)	// wParam, lParam not used
#define WM_ROBOT_CAMERA_RECORD_VIDEO		(WM_ROBOT_MESSAGE_BASE+0x6D)	// wParam: True = start, False = Stop.  lParam not used

// Bulk Commands from Remote GUI
#define WM_ROBOT_TEXT_MESSAGE_TO_SERVER		(WM_ROBOT_MESSAGE_BASE+0x80)
#define WM_ROBOT_BULK_DATA_TO_SERVER		(WM_ROBOT_MESSAGE_BASE+0x81)

#define WM_ROBOT_CAMERA_MATCH_OBJECT		(WM_ROBOT_MESSAGE_BASE+0x82)	// Match objects on file to the current frame.  Respond with WM_ROBOT_CAMERA_MATCH_RESPONSE
#define WM_ROBOT_CAMERA_MATCH_RESPONSE		(WM_ROBOT_MESSAGE_BASE+0x83)	// wParam: ObjectID, lParam: Location in Frame.  

#define WM_ROBOT_LASER_SCAN_DATA_READY		(WM_ROBOT_MESSAGE_BASE+0x84)	// from LaserScannerParser
#define WM_ROBOT_KINECT_SEARCH_COMPLETE		(WM_ROBOT_MESSAGE_BASE+0x84)	// wParam = ObjectFound (T/F)

//////// GUI Display messages Only! Commands >= to this are ignored by Module!!!
// See HW_MAX_MESSAGE in HardwareCmds.h!
#define WM_GUI_MESSAGES						(WM_ROBOT_MESSAGE_BASE+0x91)	
#define WM_ROBOT_SEND_TEXT_MESSAGES			(WM_ROBOT_MESSAGE_BASE+0x91)
#define WM_ROBOT_DISPLAY_STATUS_MESSAGES	(WM_ROBOT_MESSAGE_BASE+0x92)
#define WM_ROBOT_DISPLAY_SINGLE_ITEM		(WM_ROBOT_MESSAGE_BASE+0x93)	// Display a single item on the remote client GUI
#define WM_ROBOT_DISPLAY_BULK_ITEMS			(WM_ROBOT_MESSAGE_BASE+0x94)	// Display lots of data on the remote client GUI
#define WM_ROBOT_REMOTE_GUI_CMD				(WM_ROBOT_MESSAGE_BASE+0x95)	// Commands from Remote GUI, for camera or other things that need a window
#define WM_ROBOT_DISPLAY_TCP_TIME			(WM_ROBOT_MESSAGE_BASE+0x96)
#define WM_ROBOT_DISPLAY_OPEN_DATA_FILE		(WM_ROBOT_MESSAGE_BASE+0x97)	// Does the REAL file open, in the display thread!
#define WM_ROBOT_CLIENT_CONNECT				(WM_ROBOT_MESSAGE_BASE+0x98)
#define WM_ROBOT_CLIENT_DISCONNECT			(WM_ROBOT_MESSAGE_BASE+0x99)
#define WM_ROBOT_OPEN_MAP_FILE				(WM_ROBOT_MESSAGE_BASE+0x9A)	// wParam: 0=Create New file, lParam =
#define WM_ROBOT_ADD_WAYPOINT_LOCATION		(WM_ROBOT_MESSAGE_BASE+0x9B)	// Message from MapView to PathView to add a new Waypoint at given location
#define WM_ROBOT_UPDATE_VIEW				(WM_ROBOT_MESSAGE_BASE+0x9C)	// Tell view to update ( server changed some data)
#define WM_ROBOT_GET_CAMERA_SETTINGS		(WM_ROBOT_MESSAGE_BASE+0x9D)	// Tell GUI to send current camera settings to the Camera Module
////////////////////////////////////////////////

#define WM_ROBOT_MAX_MESSAGE				(WM_ROBOT_MESSAGE_BASE+0x9F)	// Max message ID used by WM_ROBOT (for error checking)

// Commands from Remote GUI to Server GUI
#define ROBOT_REMOTE_ENABLE_CAMERA				(WM_ROBOT_MESSAGE_BASE+0xC0)		
//#define ROBOT_REMOTE_START_CAMERA				(WM_ROBOT_MESSAGE_BASE+0xC1)		
#define ROBOT_REMOTE_ENABLE_MOTION_VIEW			(WM_ROBOT_MESSAGE_BASE+0xC2)		

#define WM_ROBOT_MAX_ROBOT_MESSAGE				(WM_ROBOT_MESSAGE_BASE+0xFF)	// Max message ID used by this program (for noise filter on send messages)
/////////////////////////////////////////////////////////////////////////////


// Server Response Messages

#define ROBOT_RESPONSE_TEXT_MESSAGE				0x00
#define ROBOT_RESPONSE_CONNECTED				0x01
#define ROBOT_RESPONSE_RADAR_SAMPLE				0x02
#define ROBOT_RESPONSE_IR_SAMPLE				0x03
#define ROBOT_RESPONSE_PIC_DEBUG1				0x04		// For debugging PIC code
#define ROBOT_RESPONSE_PIC_DEBUG2				0x05		// used for debugging new message code
#define ROBOT_RESPONSE_PIC_WATCHDOG				0x06		// Indicates watchdog expired or reset
#define ROBOT_RESPONSE_PIC_VERSION				0x07
#define ROBOT_RESPONSE_PIC_12V_POWER			0x08
#define ROBOT_RESPONSE_PIC_BUMPER				0x09		// state of bumper switches
#define ROBOT_RESPONSE_PIC_LAST_ERROR			0x0A
#define ROBOT_RESPONSE_PIC_STATUS				0x0B

#define ROBOT_RESPONSE_GPS_DATA					0x0C
#define ROBOT_RESPONSE_COLOR_BLOB_CAL_RESULT	0x0D
#define ROBOT_RESPONSE_DRIVE_MODULE_OWNER		0x0E
#define ROBOT_RESPONSE_MOTOR_SPEED				0x0F		// From the ER1 Pilot controller
#define ROBOT_RESPONSE_LASER_SCANNER_DATA		0x10
#define ROBOT_RESPONSE_KINECT_MAP_DATA			0x11

#endif 


	
// AutoNavigateMode for WM_ROBOT_AUTO_NAVIGATE_CMD
//#define MODE_NAVIGATE_PATH					0x00	// Fully autonomous mode
//#define MODE_MANUAL_AVOID_OBJECTS			0x01	// Manual control, but avoid objects in path
//#define MODE_MANUAL_NO_COLLISION			0x02	// Stop on collisions, but ignore anyting else
//#define MODE_MANUAL_FULL_OVERRIDE			0x03	// Direct control - ignore sensors

/////////////////////////////////////////////////////////////////////////////
// Behavior modes - enabled by WM_ROBOT_SET_BEHAVIOR_CMD


#define BEHAVIOR_FRIENDLY					0x00	// Random movements, tracks motion and faces, listening for commands
#define BEHAVIOR_MANUAL_CONTROL				0x01	// Don't do any automatic behaviors, voice recog turned off
#define BEHAVIOR_WAIT_FOR_VERBAL_CMD		0x02	// No motion, waiting for verbal command
#define BEHAVIOR_GUARD						0x03	// No motion, monitoring for movement, "Intruder Alert!"
#define BEHAVIOR_PERSONAL_SPACE				0x04	// Move if someone violated your personal space
#define BEHAVIOR_MAX						0x05	// Last valid mode

// Action Modes - Random ideas for now, of things robot could do
// Set by WM_ROBOT_SET_ACTION_CMD
// WARNING - order of these must match the CMD GUI!
enum SET_ACTION_CMD { 
		ACTION_MODE_NONE = 0,					// No Action mode pending
		ACTION_MODE_FOLLOW_PERSON,				//
		ACTION_MODE_KARATE_DEMO,				//	
		ACTION_MODE_PICKUP_OBJECTS,				// Look for and pickup object anywhere in front of robot
		ACTION_MODE_PICKUP_CLOSE_OBJECT,		// Look for and pickup nearest object that is within reach of the robot
		ACTION_MODE_GO_TO_LOCATION,				// Go to named location on map (example "master bedroom")?
		ACTION_MODE_FOLLOW_OBJECT,				//
		ACTION_MODE_FIND_OBJECT, 				//
		ACTION_MODE_DELIVER_RECORDED_MSG,		//
		ACTION_MODE_RETREIVE_INTERNET_DATA,		//
		ACTION_MODE_START_PHONE_CALL,			//	Call someone using Skype?
		ACTION_MODE_GET_BEER,					//	Every robot should be able to get Beer from the Fridge!
		ACTION_MODE_OPEN_DOOR,					//	Just to demo door opening ability
		ACTION_MODE_DARTH_VADER					//  Do a Darth impression
};

// Tasks needed to accomplish actions
#define TASK_NONE							0x00	// No Tasks Pending
#define TASK_GO_TO_LOCATION					0x01	//
#define TASK_OPEN_DOOR						0x02	//
#define TASK_CLOSE_DOOR						0x03	//
#define TASK_OPEN_FRIDGE_DOOR				0x04	//
#define TASK_CLOSE_FRIDGE_DOOR				0x05	//
#define TASK_GET_BEER						0x06	//
#define TASK_RETURN_TO_START				0x07	//
#define TASK_FIND_HUMAN						0x08	//
#define TASK_PERFORM_KARATE					0x09	//

// Tasks for picking up objects
#define OBJECT_TASK_NONE					0x00	// No Tasks Pending
#define OBJECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT	0x01	//
#define OBJECT_TASK_PICK_UP_OBJECT			0x02
#define OBJECT_TASK_LOOK_WHILE_MOVING		0x03	//
#define OBJECT_TASK_CLOSE_SCAN_FOR_OBJECT	0x04	//
#define OBJECT_TASK_MOVE_TO_OBJECT			0x05	//
#define OBJECT_TASK_DONE					0x06	//

// Kinect Module Task states
#define OBJECT_TASK_NONE					0x00	// No Tasks Pending
#define KINECT_TASK_SCAN_FLOOR_FOR_CLOSEST_OBJECT	0x01	//
#define KINECT_TASK_TRACK_CLOSEST_OBJECT	0x02	//
#define KINECT_TASK_HUMAN_DETECTION			0x03	// Kinect can be in either Object or Human search/track mode


// Turn Handle States
#define TURN_HANDLE_STATE_NONE				0x00
#define ARM_TURN_HANDLE_STATE_IN_PROGRESS	0x01
#define TURN_HANDLE_STATE_SUCCESS			0x02
#define TURN_HANDLE_STATE_FAIL				0x03



// Arm Movements - may be complex movements.  wParam for WM_ROBOT_SET_ARM_MOVEMENT
#define ARM_MOVEMENT_NONE					0x00	// No Action mode pending
#define ARM_MOVEMENT_HOME1					0x01	// lparam - Which Arm or BOTH_ARMS (Applies for all ARM commands)
#define ARM_MOVEMENT_HOME2					0x02	// 
#define ARM_MOVEMENT_EXTEND_ARM_AUTO_CLAW	0x03	// Code figures out if claw open or closed
#define ARM_MOVEMENT_OPEN_CLAW				0x04	// 
#define ARM_MOVEMENT_CLOSE_CLAW_FULL		0x05	// 
#define ARM_MOVEMENT_EXTEND_FULL			0x06	// 
#define ARM_MOVEMENT_SHAKE_READY			0x07	// 
#define ARM_MOVEMENT_SHAKE_MOVE				0x08	// 

#define ARM_MOVEMENT_PICKUP_OBJECT_BLIND	0x09	// Blindly try to pick up object at set position
#define ARM_MOVEMENT_PUT_DOWN_OBJECT		0x0A	// 
#define ARM_MOVEMENT_THROW_OBJECT_FRONT		0x0B	// 
#define ARM_MOVEMENT_PUT_IN_BASKET			0x0C	// 
#define ARM_MOVEMENT_LOOK_AT_HAND			0x0D	// 
#define ARM_MOVEMENT_SCRATCH_HEAD			0x0E	// 
#define ARM_MOVEMENT_ARM_UP_FULL			0x0F	// 
#define ARM_MOVEMENT_SCRATCH_BACK			0x10	// 
#define ARM_MOVEMENT_WAVE					0x11	// 
#define ARM_MOVEMENT_GRAB_COKE				0x12	// 
#define ARM_MOVEMENT_LIFT_OBJECT			0x13	// 
#define ARM_MOVEMENT_PICKUP_OBJECT_XYZ		0x14	// Try to pickup object at XYZ found by Kinect

// Complex moves, and moves for two hands
#define ARM_MOVEMENT_IDENTIFY_OBJECT		0x15	//
#define ARM_MOVEMENT_KARATE					0x16	//

#define ARM_MOVEMENT_TURN_DOOR_HANDLE		0x17	// Assumes horizontal door knob, used in our home
#define ARM_MOVEMENT_OPEN_HATCH				0x18	// 
#define ARM_MOVEMENT_CLOSE_HATCH			0x19	// 
#define ARM_MOVEMENT_SALUTE					0x1A	// 
#define ARM_MOVEMENT_RANDOM					0x1B	// 
#define ARM_MOVEMENT_TURN_DOOR_KNOB			0x1C	// Round Door knobs
#define ARM_MOVEMENT_GRAB_FRIDGE_HANDLE		0x1D	// Refridgerator doors!
#define ARM_MOVEMENT_ROBOT_MOVING			0x1E	//
#define ARM_MOVEMENT_EXTEND_ARM_OPEN_CLAW	0x1F	//
#define ARM_MOVEMENT_EXTEND_ARM_CLOSE_CLAW	0x20	//
#define ARM_MOVEMENT_ENABLE_IDLE_MOVEMENT	0x21	// lparam = True/False
#define ARM_MOVEMENT_90_DEGREE				0x22	// Mechanical Calibration Mode

// Head Movements - may be complex (multi-step) movements.  wParam for WM_ROBOT_SET_HEAD_MOVEMENT
#define HEAD_MOVEMENT_NONE					0x00	// No Action mode pending
#define HEAD_MOVEMENT_HOME					0x01	// Look straight ahead
#define HEAD_MOVEMENT_SLEEP					0x02	// Power down position

// Vision System updates.  wParam for WM_ROBOT_VISION_STATUS_READY
#define CAMERA_MATCH_OBJECT_UPDATE			0x00	// lparam = object ID.  Response to WM_ROBOT_CAMERA_MATCH_OBJECT command


// Sound Commands
#define SOUND_STOP					 0x00
#define SOUND_DARTH_VADER			 0x10
// put sound effects here...

// Text To Speech Commands
#define SPEAK_PIC_CONNECTED			 0x01 
#define SPEAK_PIC_VERSION_ERROR		 0x02 
#define SPEAK_SIMULATION_MODE		 0x03
#define SPEAK_COLLISION				 0x04
#define SPEAK_AVOID					 0x05
#define SPEAK_TRAPPED				 0x06
#define SPEAK_EXECUTING_PATH		 0x07
#define SPEAK_WAYPOINT_REACHED		 0x08
#define SPEAK_LOOKING_FOR_LANDMARK	 0x09
#define SPEAK_HEADING_FOR_OBJECT	 0x0A


#define SPEAK_TEXT_FROM_BUFFER		 0xFF	// get text from golbal buffer "g_ClientTextToSend", and speak it


// Music Files
// #define MUSIC_STOP					 0
#define MUSIC_SILENCE_STR				_T( ROBOT_DATA_PATH "\\RobotSounds\\silence.wav" )
#define MUSIC_MR_ROBOTO				 1 
#define MUSIC_MR_ROBOTO_STR				_T( ROBOT_DATA_PATH "\\RobotSounds\\MrRoboto.wav" )
#define MUSIC_HAWAII50				 2
#define MUSIC_HAWAII50_STR				_T( ROBOT_DATA_PATH "\\RobotSounds\\Hawaii50.wav" )


//////////// HEAD / CAMERA /////////////////////////////////

// Used to group servos for commands and status updates
enum SERVO_GROUP {
	MOVING_SERVOS = 0,		// only applies to servos that are moving. See IsDynaServoMoving()
	RIGHT_ARM,
	LEFT_ARM,
	BOTH_ARMS,
	HEAD_SERVOS,
	KINECT_SERVO,
	ALL_SERVOS
};

// Generic Servo Speeds
#define SERVO_SPEED_STOP					0x00	// Steps by 16 each, to allow for variable speeds if needed
#define SERVO_SPEED_EXTREMELY_SLOW			0x03	// 1/2 step for this one
#define SERVO_SPEED_VERY_SLOW				0x10	
#define SERVO_SPEED_SLOW					0x20
#define SERVO_SPEED_MED_SLOW				0x30
#define SERVO_SPEED_MED						0x40
#define SERVO_SPEED_MED_FAST				0x50
#define SERVO_SPEED_FAST					0x60
#define SERVO_SPEED_MAX						0x70

#define CLAW_TORQUE_DETECT_OBJECT			  100	// Min torque reported to decide robot is holding something

//////////// RIGHT ARM /////////////////////////////////

// Arm Limits - In TENTH DEGREES!
#define RIGHT_ARM_ELBOW_ROTATE_MAX				 (140*10)	// Rotate Max Clockwise (140 degrees)
#define RIGHT_ARM_ELBOW_ROTATE_MIN				(-140*10)	// Rotate Max Counter-clockwise (-140 degrees)

#define RIGHT_ARM_ELBOW_BEND_MAX				 (186*10)	// Max Curl - Same as Home position
#define RIGHT_ARM_ELBOW_BEND_MIN				 (-55*10)//(-40*10)	// Back-Bend (ouch!) (-55 degrees)

#define RIGHT_ARM_WRIST_ROTATE_MAX				 (150*10)	// Rotate Max Clockwise (150 degrees)
#define RIGHT_ARM_WRIST_ROTATE_MIN				(-150*10)	// Rotoate Max Counter Clockwise (-150 degrees)

#define RIGHT_ARM_CLAW_OPEN_MAX					 (90*10)	// Fully Open (90 degrees)
#define RIGHT_ARM_CLAW_OPEN_NORMAL				 (50*10)	// Mostly Open (100 degrees)
#define RIGHT_ARM_CLAW_CLOSED_MIN				     (0)	// Closed Tight!

#define RIGHT_SHOULDER_ROTATE_MAX				 (330*10)	// Rotoate forward
#define RIGHT_SHOULDER_ROTATE_MIN				(-330*10)	// Rotate Backward

//Standard Arm Positions - in DEGREES!
#define RIGHT_ARM_CLAW_OPEN_FULL					 105	// Degrees
#define RIGHT_ARM_CLAW_OPEN_HALF					  50	// Degrees
#define RIGHT_ARM_CLAW_CLOSED_COKE					  30	// Degrees
#define RIGHT_ARM_CLAW_CLOSED_LOOSE					  15	// Degrees
#define RIGHT_ARM_CLAW_CLOSED_TIGHT					   0	// Degrees

// Home Position1 - Normal
#define RIGHT_ARM_SHOULDER_HOME1					 -15	// Degrees
#define RIGHT_ARM_ELBOW_ROTATE_HOME1				   0    // Degrees
#define RIGHT_ARM_ELBOW_BEND_HOME1					 116	// Degrees - Moved to avoid side IR sensor!
#define RIGHT_ARM_WRIST_ROTATE_HOME1				   0	// Degrees
#define RIGHT_ARM_CLAW_HOME1						 RIGHT_ARM_CLAW_CLOSED_LOOSE

// Home Position2 - Claws up
#define RIGHT_ARM_SHOULDER_HOME2					   5	// Degrees - so elbow stays up when powered off
#define RIGHT_ARM_ELBOW_ROTATE_HOME2				   0	// Degrees
#define RIGHT_ARM_ELBOW_BEND_HOME2					 185	// Degrees
#define RIGHT_ARM_WRIST_ROTATE_HOME2				   0	// Degrees
#define RIGHT_ARM_CLAW_HOME2						 RIGHT_ARM_CLAW_CLOSED_LOOSE

// 90 Degree Calibration Position (for adjusting hardware)
#define RIGHT_ARM_SHOULDER_90_DEGREE				   0	// Degrees - so elbow stays up when powered off
#define RIGHT_ARM_ELBOW_ROTATE_90_DEGREE			   0	// Degrees
#define RIGHT_ARM_ELBOW_BEND_90_DEGREE				  90	// Degrees
#define RIGHT_ARM_WRIST_ROTATE_90_DEGREE			   0	// Degrees
#define RIGHT_ARM_CLAW_90_DEGREE					 RIGHT_ARM_CLAW_CLOSED_LOOSE

// Elbows In for moving


// Shake Hands Position
#define RIGHT_ARM_SHOULDER_ARM_SHAKE				  RIGHT_ARM_SHOULDER_ARM_SHAKE_UP	// Degrees
#define RIGHT_ARM_ELBOW_ROTATE_ARM_SHAKE			  -5	// Degrees
#define RIGHT_ARM_ELBOW_BEND_ARM_SHAKE				  65	// Degrees
#define RIGHT_ARM_WRIST_ROTATE_ARM_SHAKE			RIGHT_ARM_WRIST_ROTATE_HOME1
#define RIGHT_ARM_CLAW_ARM_SHAKE					RIGHT_ARM_CLAW_OPEN_HALF

// Shake UP
#define RIGHT_ARM_SHOULDER_ARM_SHAKE_UP				  80	// Degrees
#define RIGHT_ARM_ELBOW_BEND_ARM_SHAKE_UP			  60	// Degrees

// Shake Down
#define RIGHT_ARM_SHOULDER_ARM_SHAKE_DOWN			  70	// Degrees
#define RIGHT_ARM_ELBOW_BEND_ARM_SHAKE_DOWN			  60	// Degrees

#define RIGHT_ARM_SHOULDER_ARM_STRAIGHT_UP			 180	// Degrees


//////////// LEFT ARM /////////////////////////////////

// Arm Limits - In TENTH DEGREES!
#define LEFT_ARM_ELBOW_ROTATE_MAX	  				  (140*10)	// Rotate Max Clockwise (140 degrees)
#define LEFT_ARM_ELBOW_ROTATE_MIN					  (-140*10)	// Rotate Max Counter-clockwise (-140 degrees)

#define LEFT_ARM_ELBOW_BEND_MAX						  (164*10)	// Max Curl - Same as Home position
#define LEFT_ARM_ELBOW_BEND_MIN						  (-40*10)	// Back-Bend (ouch!) (-40 degrees)

#define LEFT_ARM_WRIST_ROTATE_MAX					  (150*10)	// Rotate Max Clockwise (150 degrees)
#define LEFT_ARM_WRIST_ROTATE_MIN					  (-150*10)	// Rotoate Max Counter Clockwise (-150 degrees)

#define LEFT_ARM_CLAW_OPEN_MAX						  (130*10)	// Fully Open (130 degrees)
#define LEFT_ARM_CLAW_OPEN_NORMAL					  (100*10)	// Mostly Open (100 degrees)
#define LEFT_ARM_CLAW_CLOSED_MIN					  (-15*10)		// Closed Tight!

#define LEFT_SHOULDER_ROTATE_MAX				  (330*10)	// Rotoate forward
#define LEFT_SHOULDER_ROTATE_MIN				  (-330*10)	// Rotate Backward

//Standard Arm Positions - in DEGREES!
#define LEFT_ARM_CLAW_OPEN_FULL							120		// Degrees
#define LEFT_ARM_CLAW_OPEN_HALF							 60		// Degrees
#define LEFT_ARM_CLAW_CLOSED_COKE						 40		// Degrees
#define LEFT_ARM_CLAW_CLOSED_LOOSE						  0		// Degrees
#define LEFT_ARM_CLAW_CLOSED_SNUG						-10		// Degrees  Fingertips touch at zero torque (not good for paper, but fine for other objects)
#define LEFT_ARM_CLAW_CLOSED_TIGHT						-15		// Degrees  MUST NOT EXCEED LEFT_ARM_CLAW_CLOSED_MIN

// Home Position 1
#define LEFT_ARM_SHOULDER_HOME1						-15	// Degrees
#define LEFT_ARM_ELBOW_ROTATE_HOME1					 -4	// Degrees // Negative means to the Robot's Left (away from the body)
#define LEFT_ARM_ELBOW_BEND_HOME1					109 // Degrees - Moved to avoid side IR sensor!
#define LEFT_ARM_WRIST_ROTATE_HOME1					  0	// Degrees
#define LEFT_ARM_CLAW_HOME1							LEFT_ARM_CLAW_CLOSED_LOOSE

// Home Position 2 - Claws Up
#define LEFT_ARM_SHOULDER_HOME2						  5	// Degrees - so elbow stays up when powered off
#define LEFT_ARM_ELBOW_ROTATE_HOME2					  0	// Degrees
#define LEFT_ARM_ELBOW_BEND_HOME2					163	// Degrees
#define LEFT_ARM_WRIST_ROTATE_HOME2					  0 // Degrees
#define LEFT_ARM_CLAW_HOME2							LEFT_ARM_CLAW_CLOSED_LOOSE

// 90 Degree Calibration Position (for adjusting hardware)
#define LEFT_ARM_SHOULDER_90_DEGREE					  0	// Degrees - so elbow stays up when powered off
#define LEFT_ARM_ELBOW_ROTATE_90_DEGREE				  0	// Degrees
#define LEFT_ARM_ELBOW_BEND_90_DEGREE				 90	// Degrees
#define LEFT_ARM_WRIST_ROTATE_90_DEGREE				  0 // Degrees
#define LEFT_ARM_CLAW_90_DEGREE						LEFT_ARM_CLAW_CLOSED_LOOSE

// Shake Hands Position
#define LEFT_ARM_SHOULDER_ARM_SHAKE					 65	// Degrees
#define LEFT_ARM_ELBOW_ROTATE_ARM_SHAKE				 -5	// Degrees
#define LEFT_ARM_ELBOW_BEND_ARM_SHAKE				 65	// Degrees
#define LEFT_ARM_WRIST_ROTATE_ARM_SHAKE				LEFT_ARM_WRIST_ROTATE_HOME2
#define LEFT_ARM_CLAW_ARM_SHAKE						LEFT_ARM_CLAW_CLOSED_LOOSE

// Shake UP
#define LEFT_ARM_SHOULDER_ARM_SHAKE_UP				 70	// Degrees
#define LEFT_ARM_ELBOW_BEND_ARM_SHAKE_UP			 60	// Degrees

// Shake Down
#define LEFT_ARM_SHOULDER_ARM_SHAKE_DOWN			 50	// Degrees
#define LEFT_ARM_ELBOW_BEND_ARM_SHAKE_DOWN			 60	// Degrees

#define LEFT_ARM_SHOULDER_ARM_STRAIGHT_UP			180	// Degrees

#define STRONG_TORQUE								500
#define MED_TORQUE									400
#define GENTLE_TORQUE								250



// TODO - for outdoor, look at fixed high-speed Shutter speed, to avoid blur

/////////////////////////////////////////////////////////////////////////////
// Camera Absolute or Relative parameters (used in WM_ROBOT_CAMERA_PAN_ABS_CMD and TILT)
//#define CAMERA_VALUE_ABSOLUTE				0x00
//#define CAMERA_VALUE_RELATIVE				0x01	// Value is relative to current position

// Camera Control parameters (used in WM_ROBOT_USER_CAMERA_PAN_CMD)
#define CAMERA_ZOOM_STOP					0x00
#define CAMERA_ZOOM_IN						0x01
#define CAMERA_ZOOM_OUT						0x02

#define CAMERA_PAN_SLOW						0x00
#define CAMERA_PAN_MEDIUM					0x01
#define CAMERA_PAN_FAST						0x03

// Sony Camera White Balance
#define CAMERA_WB_AUTO						0x00
#define CAMERA_WB_INDOOR					0x01
#define CAMERA_WB_OUTDOOR					0x02

// Camera Arm parameters (not used)
#define CAMERA_ARM_UP						0x00
#define CAMERA_ARM_DOWN						0x01
#define CAMERA_ARM_OFF						0x01
#define CAMERA_ARM_ON						0x00


// Camera modes passed in WM_ROBOT_CAMERA_MODE_CMD
// Loki Camera Modes (using VideoInput via OpenCV )
#define CAMERA_MODE_SHOW_FORMAT_DIALOG		0x00	// lParam = TRUE/FALSE
#define CAMERA_MODE_SHOW_PROP_DIALOG		0x01	// lParam = TRUE/FALSE
// Sony Camera mode commands
#define CAMERA_MODE_DISPLAY_ENABLE			0x02	// lParam = TRUE/FALSE
#define CAMERA_MODE_TRACK_FACE				0x03


/////////////////////////////////////////////////////////////////////////////
// Loki Head Positions

// WM_ROBOT_USER_CAMERA_FOWARD common positions
//#define HEAD_REST_POSITION					0x00
//#define HEAD_BACK_POSITION					0x01
//#define HEAD_UP_POSITION					0x02
//#define HEAD_INTEREST_POSITION				0x03
//#define HEAD_FORWARD_POSITION				0x04

// Head tilt parameters
//#define HEAD_SIDETILT_CENTER				0x00
//#define HEAD_SIDETILT_LEFT					0x01
//#define HEAD_SIDETILT_RIGHT					0x02


/////////////////////////////////////////////////////////////////////////////
// Features controlled by WM_ROBOT_CAMERA_ENABLE_FEATURE
#define CAMERA_ENABLE_VIDCAP_PROCESSING		0x00	// Used to disable ALL vidcap processing!
#define CAMERA_ENABLE_TRACKING_FACE			0x01
#define CAMERA_ENABLE_TRACKING_COLORS		0x02
#define CAMERA_ENABLE_TRACKING_CAMSHIFT		0x03
#define CAMERA_ENABLE_TRACKING_OBJECTS		0x04
#define CAMERA_ENABLE_TRACKING_CONES		0x05
#define CAMERA_ENABLE_TRACKING_MOTION		0x06
#define CAMERA_ENABLE_TRACKING_IR			0x07
#define CAMERA_ENABLE_SHOW_MOTION_VIEW		0x08
#define CAMERA_ENABLE_TRACKING_LASER_SPOT	0x09
#define CAMERA_ENABLE_TRACKING_HAND			0x0A
#define CAMERA_ENABLE_STEREO_VISION			0x0B
#define CAMERA_ENABLE_MATCHING_OBJECTS		0x0C


#define SET_COLOR_THRESHOLD_CR				0x01
#define SET_COLOR_THRESHOLD_CB				0x02
#define NO_MATCH							(-1)	// No match found by MatchObjectDetector 

// 12V and Sony Camera Power parameters
#define POWER_OFF							0x00
#define POWER_ON							0x01
#define CAMERA_RESET						0x02
#define SYSTEM_SHUT_DOWN					0x04

/*
// LED Eyes parameters
#define LED_EYE_OFF							0x00
#define LED_EYE_ON							0x01
#define LED_EYE_CLOSE						0x02
#define LED_EYE_OPEN						0x03
#define	EYES_BLINK							0x04
#define LED_EYE_BRIGHTNESS_MAX				  19	// On time in 20ms cycle
*/

// Misc parameters
#define MUSIC_STOP							0x00
#define MUSIC_START							0x01

#define PATH_EXECUTE_CANCEL					0x00
#define NAV_PATH_EXECUTE_START				0x01
#define GRID_PATH_EXECUTE_START				0x02
#define PATH_EXECUTE_PAUSE					0x03
#define PATH_EXECUTE_RESUME					0x04


// File Open Parameters for Path and Map types
#define PATH_TYPE_NONE						0x00	// Don't load a Path
#define PATH_TYPE_DEFAULT					0x01	// Load the default Path file
#define PATH_TYPE_BLANK						0x02	// Load a blank Path file


#define MAP_TYPE_NONE						0x00	// Load a blank Map file
#define MAP_TYPE_BLANK						0x01	// Load a blank Map file
#define MAP_TYPE_ROBOTHON					0x02	// Not used at this time
#define MAP_TYPE_UPSTAIRS					0x03	// Load the Upstairs Map
#define MAP_TYPE_DOWNSTAIRS					0x04	// Load the Downstairs Map


// Sensor IDs for enable/disable via WM_ROBOT_ENABLE_SENSOR
#define SENSOR_IR0							0x00
#define SENSOR_IR1							0x01
#define SENSOR_IR2							0x02
#define SENSOR_IR3							0x03
#define SENSOR_IR4							0x04
#define SENSOR_IR5							0x05

#define SENSOR_US0							0x10
#define SENSOR_US1							0x11
#define SENSOR_US2							0x12

#define BULK_DATA_TYPE_UNKNOWN				0x00	// Parameter 1 for WM_ROBOT_BULK_DATA_TO_SERVER
#define BULK_DATA_TYPE_MAP_STROKE			0x01


// Radar parameters - see globals.h for buffer declaration
#define MAX_SENSORS					4	// Total number of sensor scans tracked (not counting Laser Scanner)
#define FIXED_IR_ARRAY				0	// Array of fixed (non moving) IR sensors
#define FIXED_US_ARRAY				1	// Array of fixed (non moving) Ultrasonic sensors
#define IR_ARRAY1					2
#define US_ARRAY1					3

#define FIXED_IR_SAMPLES			 6
#define FIXED_US_SAMPLES			 3
#define ULTRASONIC1_SAMPLES			16	// and number of samples for each type
#define ULTRASONIC2_SAMPLES			32


#define US1_SAMPLES					16	// number of samples.
/**
#define US_1_HARD_LEFT				 1	//  0/1
#define US_1_MID_LEFT				 4	//  2-4
#define US_1_SLIGHT_LEFT			 6	//  5/6
#define US_1_CENTER					 8	//  7/8 CHANGE if number of samples change!
#define US_1_SLIGHT_RIGHT			10	//  9/10
#define US_1_MID_RIGHT				13	// 11-13
#define US_1_HARD_RIGHT				15	// 14/15

#define US_1_CENTER_LEFT	 		 7	//  7/8 CHANGE if number of samples change!
#define US_1_CENTER_RIGHT	 		 8	//  7/8 CHANGE if number of samples change!
**/

#define IR1_SAMPLES					16
#define IR2_SAMPLES					16


#define ULTRASONIC_RAW_MAX						  255		// Max value for each type
//#define ULTRASONIC_TENTH_INCHES_MAX			  840		// 1080	// For Carbot, we use 9ft(108 inches)
#define ULTRASONIC_TENTH_INCHES_MAX				 1440		// Max EZ1 works pretty good to 12 feet! // For Carbot, we use 9ft(108 inches)
#define LASER_RANGEFINDER_MM_MAX				 4096		// URG-04LX max range = 4096mm = 13.4 feet 
//#define LASER_RANGEFINDER_TENTH_INCHES_MAX	1560.0		// = 13.0 feet, just under sensor limit
//#define LASER_RANGEFINDER_TENTH_INCHES_ERROR	1570.0		// Bad Reading, ignore
#define LASER_RANGEFINDER_TENTH_INCHES_MAX		 1560		// = 13.0 feet, just under sensor limit
#define LASER_RANGEFINDER_TENTH_INCHES_ERROR	 1570		// Bad Reading, ignore
#define LASER_RANGEFINDER_DEGREES_PER_STEP	0.3515625
#define LASER_RANGEFINDER_MAX_SAMPLES			 2047		// Max number of samples per scan from URG-04LX Laser Scanner
#define LASER_RANGEFINDER_MAX_SCAN_LINES		    4		// Max number of scan lines to store for analysis
#define LASER_RANGEFINDER_MAX_OBJECT_LINES		  100		// Max number of object scan lines to store for analysis (up to one sample per 1/2 inch for 4 feet)


#define KINECT_MM_MAX							60960 		// Kinect max range = 60960 = 20 feet 
#define KINECT_RANGE_TENTH_INCHES_MAX			 2400		// = 20.0 feet, just under sensor limit
#define KINECT_RANGE_TENTH_INCHES_ERROR			 2401		// Bad Reading, ignore
#define KINECT_DEGREES_PER_STEP_X			0.3515625		// TODO Horizontal degrees per pixel step calculation: ????
#define KINECT_DEGREES_PER_STEP_Y			0.3515625		// TODO Horizontal degrees per pixel step calculation: ????
#define KINECT_MAX_SAMPLES						 2047		// Max number of samples per scan from URG-04LX Laser Scanner
#define KINECT_MAX_SCAN_LINES					   16		// Max number of scan lines to store for analysis
#define KINECT_MAX_OBJECT_LINES					  100		// Max number of object scan lines to store for analysis (up to one sample per 1/2 inch for 4 feet)


//#define IR_RAW_MAX							 255
#define IR_GUI_TENTH_INCHES_MAX					 840				// 7 feet - Just used to draw circle in Radar GUI
#define IR_MAX_TENTH_INCHES						 240
#define IR_LR_TENTH_INCHES_MAX					 360				// Long Range Sensors
#define IR_SR_TENTH_INCHES_MAX					 240				// Short Range Sensors
#define PIC_NO_OBJECT_IN_RANGE					0xFC			// Same value as used in PIC
#define PIC_OBJECT_DETECTED_RANGE_UNKNOWN		0xFE
#define NO_OBJECT_IN_RANGE						LASER_RANGEFINDER_TENTH_INCHES_MAX		// used to be 0xFC, Same value as used in PIC
#define OBJECT_DETECTED_RANGE_UNKNOWN			LASER_RANGEFINDER_TENTH_INCHES_ERROR	// used to be 0xFE
#define SENSOR_DISABLED							2048	// disable any computations that use this sensor

#define STATUS_BUF_LEN							4096
#define STATUS_TEXT_BUF_LEN					(STATUS_BUF_LEN-6)
#define STATUS_DATA_BUF_LEN					(STATUS_BUF_LEN-16)	// 12 plus padding for paranoia
#define BULK_DATA_SIZE						STATUS_DATA_BUF_LEN	// Size of transfer buffer
#define	WIDE_BULK_DATA_SIZE					(BULK_DATA_SIZE*2)	// Room to store Wide Bytes for sound routines


typedef struct
{
	DWORD	Cmd;
	DWORD	Param1;
	DWORD	Param2;
} SOCK_DATA_T;

typedef struct
{
	DWORD	MessageType;	// WM_ROBOT_RESPONSE_...
	DWORD	Length;
	CHAR	Message[STATUS_TEXT_BUF_LEN];
} SOCKET_STRING_MESSAGE_T;

typedef struct
{
	DWORD	MessageType;	// WM_ROBOT_RESPONSE_...
	DWORD	Param1;
	DWORD	Param2;
} SOCKET_STATUS_MESSAGE_T;


typedef struct
{
	DWORD	MessageType;	// WM_ROBOT_RESPONSE_...
	DWORD	Param1;
	DWORD	Param2;
	WORD	Length;
} SOCKET_DATABUF_HEADER_T;

typedef struct
{
	DWORD	MessageType;	// Same as HEADER_T, plus the Data buffer
	DWORD	Param1;
	DWORD	Param2;
	WORD	Length;
	CHAR	Data[STATUS_DATA_BUF_LEN];
} SOCKET_DATABUF_MESSAGE_T;


#define GPS_MAX_SAT	12	// 12 Satellites max
typedef struct
{
	WORD	wPRN;
	WORD	wSignalQuality;
	WORD	wAzimuth;
	WORD	wElevation;
	BOOL	bUsedInSolution;
} SAT_INFO_T;


typedef struct
{
	DWORD		dwCommandCount;				// number of NMEA commands received (processed or not processed)
	double		dGGALatitude;				// < 0 = South, > 0 = North
	double		dGGALongitude;				// < 0 = West, > 0 = East
	double		dGGAAltitude;				// Altitude: mean-sea-level (geoid) meters
	double		dGSAVDOP;					//
	double		dGSAHDOP;					//
	double		dGSAPDOP;					//
	//double	dRMCLatitude;				// current latitude
	//double	dRMCLongitude;				// current longitude
	//double	dRMCGroundSpeed;			// speed over ground, knots
	//double	dRMCCourse;				// course over ground, degrees true
	//double	dRMCMagVar;				// magnitic variation, degrees East(+)/West(-)
	WORD		wGSVTotalNumSatsInView;		//
	WORD		wGSASatsInSolution[GPS_MAX_SAT]; // ID of sats in solution
	SAT_INFO_T	GSVSatInfo[GPS_MAX_SAT];	//
	//BYTE		btRMCDataValid;				// A = Data valid, V = navigation rx warning
	BYTE		btGGAGPSQuality;				// 0 = fix not available, 1 = GPS sps mode, 2 = Differential GPS, SPS mode, fix valid, 3 = GPS PPS mode, fix valid
	BYTE		btGSAMode;					// M = manual, A = automatic 2D/3D
	BYTE		btGSAFixMode;				// 1 = fix not available, 2 = 2D, 3 = 3D
} GPS_MESSAGE_T;
	





#endif