// Speech.cpp


// This class is used by the thread loop that reads commands from the queue
// in thread.cpp

#include "stdafx.h"
#include "ClientOrServer.h"
#if ( ROBOT_SERVER == 1 )	// This module used for Robot Server only

//#include "thread.h"
//#include "module.h"
#include "Globals.h"
#include "RobotSpeech.h"

// GPA TRACE Macros
__itt_string_handle* pshInitSpeech = __itt_string_handle_create("InitSpeech");
__itt_string_handle* pshSpeaking = __itt_string_handle_create("Speaking");
__itt_string_handle* pshCmdProcess = __itt_string_handle_create("CmdProcess");
__itt_string_handle* pshCmdRecoEvent = __itt_string_handle_create("CmdRecoEvent");
__itt_string_handle* pshDictRecoEvent = __itt_string_handle_create("DictRecoEvent");
__itt_string_handle* pshSwitchRecoMode = __itt_string_handle_create("SwitchRecoMode");

__itt_string_handle* pshSendToAI = __itt_string_handle_create("SendToAI"); // marker
__itt_string_handle* pshRecoEnabledByGUI = __itt_string_handle_create("RecoEnabledByGUI"); // marker
__itt_string_handle* pshRecoDisabledByGUI = __itt_string_handle_create("RecoDisabledByGUI"); // marker
__itt_string_handle* pshRecoEnabled = __itt_string_handle_create("RecoEnabled"); // marker
__itt_string_handle* pshRecoDisabled = __itt_string_handle_create("RecoDisabled"); // marker
__itt_string_handle* pshRecoCmdMode = __itt_string_handle_create("RecoCmdMode"); // marker
__itt_string_handle* pshRecoDictMode = __itt_string_handle_create("RecoDictMode"); // marker



/////////////////////////////////////////////////////
//                DEBUG SWITCHES
//#define DEBUG_TREX_READWRITE_TIME  // Log Read/Write SIO latency

/////////////////////////////////////////////////////

// Pre-defined Waypoint Locations to go to
#define WAYPOINT_OFFICE_X				341
#define WAYPOINT_OFFICE_Y				248

#define WAYPOINT_MASTER_BEDROOM_X		282
#define WAYPOINT_MASTER_BEDROOM_Y		580

#define WAYPOINT_HEATHER_BEDROOM_X		291
#define WAYPOINT_HEATHER_BEDROOM_Y		448

#define WAYPOINT_AMBER_BEDROOM_X		466
#define WAYPOINT_AMBER_BEDROOM_Y		236

#define WAYPOINT_MASTER_BATHROOM_X		368
#define WAYPOINT_MASTER_BATHROOM_Y		655

#define SPEECH_MIC_TIMEOUT				15000 // MS

/////////////////////////////////////////////////////////////////////////////
// For Debugging Speech, enable / disable actual command execution here
#define DISABLE_SPEECH_ACTIONS		1
void SpeechSendCommand( DWORD Cmd, DWORD Param1, DWORD Param2 )
{
	#if ( 1 != DISABLE_SPEECH_ACTIONS )
		SendCommand( DWORD Cmd, DWORD Param1, DWORD Param2 )
	#endif
}


///////////////////////////////////////////////////////////////////////////////
// Robot Speech Recognition Class
///////////////////////////////////////////////////////////////////////////////
CRobotSpeech::CRobotSpeech()
{
	m_CommandMode = RECO_OFF;	// off until after initialization completes
	m_bInSound = FALSE;
	m_bGotReco = FALSE;
	m_lcid = 0;
	m_cpDictationRecoCtxt = NULL;
	m_cpDictationGrammar = NULL;
	m_bSendRecoToAI = FALSE;
	m_bEnableMotorsForSpeechCommands = FALSE;
	m_bEnableSpeaking = FALSE;
	m_bPlaySimonSays = FALSE;
	m_bSimonSays = FALSE;
	m_MicOnTime = GetTickCount();
//	m_bRobotIsSpeaking = FALSE;
	m_pVoice = NULL;

}

CRobotSpeech::~CRobotSpeech()
{
	if( m_pVoice != NULL )
	{
		m_pVoice->Release();
		m_pVoice = NULL;
		::CoUninitialize();
	}
}

void CRobotSpeech::Init( BOOL bRecoEnabled )
{
#if( SPEECH_ENGINE_ENABLED == 0 )
	return;
#endif
	__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshInitSpeech);

	//m_TrexCmd.Axis = 0;		// Always zero
	ROBOT_LOG( TRUE, "\n\n======================== SPEECH RECOGNITION INITIALIZATION  ============================= \n" )
//	ROBOT_LOG( TRUE, "\n\INITIALIZING SPEECH RECOGNITION 1\n" )
	m_bEnableSpeaking = FALSE; // Gets turned on if Init completes sucessfullly

	HRESULT hr = S_OK;

	m_RecoEnabledByGUI = bRecoEnabled;


	//ROBOT_LOG (TRUE, "SPEECH - LOG TEST!! %d, %s",	26, "foo" )

	Sleep(200);
	//ROBOT_LOG( TRUE, "\nTEST\n" )



	///////////////////////////////////////////////////////////
	// Initialize Speech Engine for text to speech:

	if (FAILED(::CoInitialize(NULL)))
	{
		m_pVoice = NULL;
		m_bEnableSpeaking = FALSE;
		//ASSERT(0);
		ROBOT_LOG( TRUE, "\n\n" )
		ROBOT_DISPLAY( TRUE, "=========================================" )
		ROBOT_DISPLAY( TRUE, "ERROR ERROR ERROR - Speech Engine failed!" )
		ROBOT_DISPLAY( TRUE, "=========================================" )
		ROBOT_LOG( TRUE, "\n\n" )
		__itt_task_end(pDomainSpeechThread);
		return;
	}

	hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&m_pVoice);
	if( FAILED( hr ) )
	{
		ROBOT_LOG( TRUE, "ERROR! Could not initialize Voice!\n" )
		m_pVoice->Release();
		m_pVoice = NULL;
		::CoUninitialize();
		//ASSERT(0);
		__itt_task_end(pDomainSpeechThread);
		return;
	}

	if( NULL == m_pVoice )
	{
		__itt_task_end(pDomainSpeechThread);
		return;
	}

    hr = m_pVoice->SetNotifyWindowMessage( g_RobotSetupViewHWND, WM_ROBOT_SPEAKING_COMPLETE_EVENT, 0, 0 );

	const ULONGLONG ullInterest = SPFEI(SPEI_END_INPUT_STREAM);
	hr = m_pVoice->SetInterest(ullInterest, ullInterest);
	if( FAILED( hr ) )
	{
		ROBOT_LOG( TRUE, "ERROR Setting Event Interest!\n" )
		m_pVoice->Release();
		m_pVoice = NULL;
		::CoUninitialize();
		//ASSERT(0);
		__itt_task_end(pDomainSpeechThread);
		return;
	}


	hr = m_pVoice->Speak(L"Initializing System", 0, NULL);
	if( FAILED( hr ) )
	{
		ROBOT_LOG( TRUE, "ERROR Speaking!\n" )
		m_pVoice->Release();
		m_pVoice = NULL;
		::CoUninitialize();
		//ASSERT(0);
		__itt_task_end(pDomainSpeechThread);
		return;
	}


    
	///////////////////////////////////////////////////////////
	// Set up the SR (Speech Recognition) engine
 
    hr = m_cpRecoEngine.CoCreateInstance(CLSID_SpInprocRecognizer);

	// Create the recognition contexts (one for dictation mode and one for command mode).
	// An ISpRecoContext is created off an ISpRecognizer

	///////////////////////////////////
	// Initialize Dictaton Mode Context
    if( SUCCEEDED( hr ) )
    {
        hr = m_cpRecoEngine->CreateRecoContext( &m_cpDictationRecoCtxt );
    }

    // Set recognition notification for dictation
    if (SUCCEEDED(hr))
    {
		if( NULL == g_RobotSetupViewHWND )
		{
				ASSERT(0);
		}
        hr = m_cpDictationRecoCtxt->SetNotifyWindowMessage( g_RobotSetupViewHWND, WM_ROBOT_DICTATION_RECO_EVENT, 0, 0 );
    }
    
//    if (SUCCEEDED(hr))
//    {
        // This specifies which of the recognition events are going to trigger notifications.
        // Here, all we are interested in is the beginning and ends of sounds, as well as
        // when the engine has recognized something
 //       const ULONGLONG ullInterest = SPFEI(SPEI_RECOGNITION);
   //     hr = m_cpDictationRecoCtxt->SetInterest(ullInterest, ullInterest);
//    }

	////////////////////////////////////////
	// Initialize Command Mode Context (C&C)

	if( SUCCEEDED( hr ) )
    {
        hr = m_cpRecoEngine->CreateRecoContext( &m_cpCommandRecoCtxt );
    }

    // Set recognition notification for Command Mode
    if (SUCCEEDED(hr))
    {
		if( NULL == g_RobotSetupViewHWND )
		{
				ASSERT(0);
		}
        hr = m_cpCommandRecoCtxt->SetNotifyWindowMessage( g_RobotSetupViewHWND, WM_ROBOT_COMMAND_RECO_EVENT, 0, 0 );
    }
    
/*    if (SUCCEEDED(hr))
    {
        // This specifies which of the recognition events are going to trigger notifications.
        // Here, all we are interested in is the beginning and ends of sounds, as well as
        // when the engine has recognized something
        const ULONGLONG ullInterest = SPFEI(SPEI_RECOGNITION);
        hr = m_cpCommandRecoCtxt->SetInterest(ullInterest, ullInterest);
    }
*/

	//////////////////////////////
    // create default audio object
    CComPtr<ISpAudio> cpAudio;
    hr = SpCreateDefaultObjectFromCategoryId(SPCAT_AUDIOIN, &cpAudio);

    // set the input for the engine
    hr = m_cpRecoEngine->SetInput(cpAudio, TRUE);
    hr = m_cpRecoEngine->SetRecoState( SPRST_ACTIVE );



	///////////////////////////////
	// Load Dictation Grammar
    if (SUCCEEDED(hr))
    {
        // Specifies that the grammar we want is a dictation grammar.
        // Initializes the grammar (m_cpDictationGrammar)
        hr = m_cpDictationRecoCtxt->CreateGrammar( GID_DICTATION, &m_cpDictationGrammar );
    }
    if  (SUCCEEDED(hr))
    {
        hr = m_cpDictationGrammar->LoadDictation(NULL, SPLO_STATIC);
    }
    if (FAILED(hr))
    {
        m_cpDictationGrammar.Release();
    }

	//////////////////////////////////////////////
	// Load Dictation Mode Limited Commands Grammar
	// Create the grammar for the commands that are available for dictation mode.
    if( SUCCEEDED( hr ) )
    {
        static const WCHAR FileName[] = ROBOT_COMMON_PATH L"\\DictationMode.xml";
        m_cpDictationCmdGrammar.Release();
        hr = m_cpDictationRecoCtxt->CreateGrammar(GID_DICTATIONCC, &m_cpDictationCmdGrammar);
        if (SUCCEEDED(hr))
        {
            hr = m_cpDictationCmdGrammar->LoadCmdFromFile(FileName, SPLO_STATIC);
			//   hr = m_cpDictationCmdGrammar->LoadCmdFromResource(NULL, (const WCHAR*)MAKEINTRESOURCE(IDR_DICTATION_MODE_CFG),
			//                                               L"SRGRAMMAR", langid, SPLO_STATIC);
        }
        if (FAILED(hr))
        {
			CString ErrorStr;
			ErrorStr.Format( "Can't open Speech Recognition DICTATION Mode Config file:\n %S", FileName );
			AfxMessageBox( ErrorStr );
            m_cpDictationCmdGrammar.Release();
			__itt_task_end(pDomainSpeechThread);
			return;
        }
    }

	///////////////////////////////
	// Load Command Mode Grammar
    if( SUCCEEDED( hr ) )
    {
        static const WCHAR FileName[] = ROBOT_COMMON_PATH L"\\CommandMode.xml";
        m_cpCommandGrammar.Release();
		hr = m_cpCommandRecoCtxt->CreateGrammar(GID_CC, &m_cpCommandGrammar);

        if (SUCCEEDED(hr))
        {
            hr = m_cpCommandGrammar->LoadCmdFromFile(FileName, SPLO_STATIC);
			//   hr = m_cpCommandGrammar->LoadCmdFromResource(NULL, (const WCHAR*)MAKEINTRESOURCE(IDR_DICTATION_MODE_CFG),
			//                                               L"SRGRAMMAR", langid, SPLO_STATIC);
        }
        if (FAILED(hr))
        {
			CString ErrorStr;
			ErrorStr.Format( "Can't open Speech Recognition COMMAND Mode Config file:\n %S", FileName );
			AfxMessageBox( ErrorStr );
            m_cpCommandGrammar.Release();
			__itt_task_end(pDomainSpeechThread);
			return;
        }
    }



	// Enable Dictation mode Contexts
	hr = m_cpDictationRecoCtxt->SetContextState( SPCS_ENABLED );
	if ( SUCCEEDED( hr ) )
	{
		const ULONGLONG ullInterest = SPFEI(SPEI_RECOGNITION);
		hr = m_cpDictationRecoCtxt->SetInterest(ullInterest, ullInterest);
	}
	if (FAILED(hr))
	{
		ROBOT_ASSERT(0);
	}


	hr = m_cpCommandRecoCtxt->SetContextState( SPCS_ENABLED );
    if (SUCCEEDED(hr))
    {
		const ULONGLONG ullInterest = SPFEI(SPEI_RECOGNITION);
		hr = m_cpCommandRecoCtxt->SetInterest(ullInterest, ullInterest);
	}
	if (FAILED(hr))
	{
		ROBOT_ASSERT(0);
	}


	// Do final initialization for Dictation Mode.

	// m_RecoEnabledByGUI set in the class initializer
	m_CommandMode = RECO_COMMAND_MODE;	// Command or Dictation Mode switch
	m_ProcessMicOnOnly = TRUE;				// Initially, Reco COMMANDS ignored for all commands except "Mic On"
	EnableReco( TRUE );						// Turn it on!

	m_bEnableSpeaking = TRUE;	// ON by default, change if you want it different

	ROBOT_LOG( TRUE, "\n\n=================== SPEECH RECOGNITION INITIALIZATION COMPLETE ========================== \n\n\n" )

	__itt_task_end(pDomainSpeechThread);
}


void CRobotSpeech::PIDToString( UINT nPID, CString &PIDString )
{

	switch( nPID )
	{

		case PID_CmdMenu:					PIDString = "RULE: PID_CmdMenu";	
		break;
		case VID_UserName:					PIDString = "RULE: VID_UserName";	
		break;
		case PID_CmdMicrophoneOn:			PIDString = "PID_CmdMicrophoneOn";	
		break;
		case PID_CmdMicrophoneOff:			PIDString = "PID_CmdMicrophoneOff";	
		break;
		case PID_CmdAIMode:					PIDString = "PID_CmdAIMode";	
		break;
		case PID_CmdDictTestMode:			PIDString = "PID_CmdDictTestMode";
		break;
		case PID_CmdEnableMovement:			PIDString = "PID_CmdEnableMovement";
		break;
		case PID_CmdDisableMovement:		PIDString = "PID_CmdDisableMovement";
		break;
		case PID_CmdMoveStop:				PIDString = "PID_CmdMoveStop";
		break;
		case PID_CmdMoveForward:			PIDString = "PID_CmdMoveForward";
		break;
		case PID_CmdMoveBack:				PIDString = "PID_CmdMoveBack";
		break;
		case PID_CmdTurnLeft:				PIDString = "PID_CmdTurnLeft";
		break;
		case PID_CmdTurnRight:				PIDString = "PID_CmdTurnRight";
		break;
		case PID_CmdLookUp:					PIDString = "PID_CmdLookUp";
		break;
		case PID_CmdLookDown:				PIDString = "PID_CmdLookDown";
		break;
		case PID_CmdLookLeft:				PIDString = "PID_CmdLookLeft";
		break;
		case PID_CmdLookRight	:			PIDString = "PID_CmdLookRight";
		break;
		case PID_CmdLookForward	:			PIDString = "PID_CmdLookForward";
		break;
		case PID_CmdAvoidObjectsOn:			PIDString = "PID_CmdAvoidObjectsOn";
		break;
		case PID_CmdAvoidObjectsOff:		PIDString = "PID_CmdAvoidObjectsOff";
		break;
		case PID_CmdTalkAboutObstaclesOn:	PIDString = "PID_CmdAddDeleteWords";
		break;
		case PID_CmdTalkAboutObstaclesOff:	PIDString = "PID_CmdTalkAboutObstaclesOff";
		break;
		case PID_CmdTalkingEnabled:			PIDString = "PID_CmdTalkingEnabled";
		break;
		case PID_CmdTalkingDisabled:		PIDString = "PID_CmdTalkingDisabled";
		break;
		case PID_CmdCameraTrackFaceOn:		PIDString = "PID_CmdCameraTrackFaceOn";
		break;
		case PID_CmdCameraTrackFaceOff:		PIDString = "PID_CmdCameraTrackFaceOff";
		break;

		case PID_CmdArmHomeLeft:			PIDString = "PID_CmdArmHomeLeft";
		break;
		case PID_CmdArmExtendLeft:			PIDString = "PID_CmdArmExtendLeft";
		break;

		case PID_CmdArmExtendFullLeft:		PIDString = "PID_CmdArmExtendFullLeft";
		break;
		case PID_CmdArmTakeObjectLeft:		PIDString = "PID_CmdArmTakeObject";
		break;
		case PID_CmdArmOpenClawLeft:		PIDString = "PID_CmdArmOpenClaw";
		break;
		case PID_CmdArmCloseClawLeft:		PIDString = "PID_CmdArmCloseClaw";
		break;
		case PID_CmdArmPickUpLeft:			PIDString = "PID_CmdArmPickUp";
		break;

		case PID_CmdFindObjectsAndPickup:	PIDString = "PID_CmdFindObjectsAndPickup";
		break;
		

		case PID_CmdArmHomeRight:			PIDString = "PID_CmdArmHome";
		break;
		case PID_CmdArmExtendRight:			PIDString = "PID_CmdArmExtendRight";
		break;
		case PID_CmdArmExtendFullRight:		PIDString = "PID_CmdArmExtendFull";
		break;
		case PID_CmdArmTakeObjectRight:		PIDString = "PID_CmdArmTakeObject";
		break;
		case PID_CmdArmOpenClawRight:		PIDString = "PID_CmdArmOpenClaw";
		break;
		case PID_CmdArmCloseClawRight:		PIDString = "PID_CmdArmCloseClaw";
		break;
		case PID_CmdArmPickUpRight:			PIDString = "PID_CmdArmPickUp";
		break;

		case PID_CmdArmExtendClosedRight:			PIDString = "PID_CmdArmExtendClosedRight";
		break;

		case PID_CmdArmTossFront:			PIDString = "PID_CmdArmTossObjectFront";
		break;
		case PID_CmdArmTossBack:			PIDString = "PID_CmdArmTossObjectBack";
		break;

		case PID_CmdArmShakeHands:			PIDString = "PID_CmdArmShakeHands";
		break;

		case PID_CmdArmPutDown:				PIDString = "PID_CmdArmPutDown";
		break;
		case PID_CmdArmDropIt:				PIDString = "PID_CmdArmDropIt";
		break;
		case PID_CmdArmHandsUp:				PIDString = "PID_CmdArmHandsUp";
		break;
		case PID_CmdArmHandsDown:			PIDString = "PID_CmdArmHandsDown";
		break;
		case PID_CmdArmWave:				PIDString = "PID_CmdArmWave";
		break;
		case PID_CmdArmScratchBack:			PIDString = "PID_CmdArmScratchBack";
		break;
		case PID_CmdArmScratchHead:			PIDString = "PID_CmdArmScratchHead";
		break;
		case PID_CmdFollowMeStart:			PIDString = "PID_CmdFollowMeStart";
		break;
		case PID_CmdExploreStart:			PIDString = "PID_CmdExploreStart";
		break;
		case PID_CmdExploreStop:			PIDString = "PID_CmdExploreStop";
		break;
		case PID_CmdGotoOffice:				PIDString = "PID_CmdGotoOffice";
		break;
		case PID_CmdGotoBedroom:			PIDString = "PID_CmdGotoBedroom";
		break;
		case PID_CmdGotoHeatherRoom:		PIDString = "PID_CmdGotoHeatherRoom";
		break;
		case PID_CmdGotoAmberRoom:			PIDString = "PID_CmdGotoAmberRoom";
		break;
		case PID_CmdGotoMyBathroom:			PIDString = "PID_CmdGotoMyBathroom";
		break;

		case PID_CmdBlueLightsOn:			PIDString = "PID_CmdBlueLightsOn";
		break;
		case PID_CmdBlueLightsOff:			PIDString = "PID_CmdBlueLightsOff";
		break;
		case PID_CmdKarate:					PIDString = "PID_CmdKarate";
		break;
		case PID_CmdWakeUp:					PIDString = "PID_CmdWakeUp";
		break;
		case PID_CmdGoToSleep:				PIDString = "PID_CmdGoToSleep";
		break;

		case PID_CmdPlaySimonSaysOn:		PIDString = "PID_CmdPlaySimonSaysOn";
		break;
		case PID_CmdPlaySimonSaysOff:		PIDString = "PID_CmdPlaySimonSaysOff";
		break;
		case PID_CmdSimonSays:				PIDString = "PID_CmdSimonSays";
		break;

		case PID_CmdIdentifyObject:			PIDString = "PID_CmdIdentifyObject";
		break;

		case PID_CmdPlayStart:				PIDString = "PID_CmdPlayStart";
		break;
		case PID_CmdPlayStop:				PIDString = "PID_CmdPlayStop";
		break;

		case PID_CmdAttention:				PIDString = "PID_CmdAttention";	
		break;

		
		default:
			PIDString.Format("Unknown PID (%d, %04Xh)", nPID, nPID);

	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
#define USER_NAME_LEN 132

BOOL CRobotSpeech::ProcessCommandModeCommands( ISpRecoResult &rResult )
{

	#if( SPEECH_ENGINE_ENABLED == 0 )
		return 0;
	#endif

	__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshCmdProcess);

	TCHAR UserName[USER_NAME_LEN];          // Name of the person Loki is being introduced to
	WCHAR SpeakBuf[USER_NAME_LEN*2];

	SPPHRASE *pElements = NULL;
	HRESULT hr = rResult.GetPhrase( &pElements );
	if ( FAILED( hr ) || pElements->ullGrammarID != GID_CC )
	{
		::CoTaskMemFree(pElements);
		ASSERT(0);
		__itt_task_end(pDomainSpeechThread);
		return false;
	}


	// Get Command text
	CSpDynamicString dstr;
	dstr = L"COMMAND MODE: ";

	USES_CONVERSION;

	if( pElements->Rule.pszName )
	{
		dstr.Append( pElements->Rule.pszName );
	}
	else
	{
		dstr.Append( L"<null>" );
	}


	CString strText, strPID;
	strText.Format(_T("Speech Reco: %s,  Rule=%d\n"), W2T(dstr), pElements->Rule.ulId );
	ROBOT_LOG( TRUE, (LPCTSTR)strText )


	UINT nPID = 0;
	switch( pElements->Rule.ulId )
	{
        case VID_UserName:
        {
			nPID = pElements->pProperties[0].vValue.cyVal.Hi;
            WCHAR *wszCoMemNameText = NULL;
            WCHAR *wszCoMemValueText = NULL;
			UINT nWordsInPhrase = 0;

			if( m_ProcessMicOnOnly )
			{
				ROBOT_DISPLAY( TRUE, "SpeechReco USER NAME Ignored: Microphone Disabled\n" )
				::CoTaskMemFree( pElements );
				__itt_task_end(pDomainSpeechThread);
				return TRUE;
			}
			if( (GetTickCount() - m_MicOnTime) > SPEECH_MIC_TIMEOUT )
			{
				ROBOT_DISPLAY( TRUE, "SpeechReco USER NAME Ignored: Microphone Timed Out\n" )
				::CoTaskMemFree( pElements );
				__itt_task_end(pDomainSpeechThread);
				return TRUE;
			}

            
            // Figure out where dictation starts and get that text
            // Since we know how many words are in our rule ( 5 ) we can get the text
            // after them, which will be the dictation.
			if( PID_SayHello == nPID )
			{
				nWordsInPhrase = 3;
			}
			else if( PID_Meet6 == nPID )
			{
				nWordsInPhrase = 6;
			}
			else if( PID_Meet5 == nPID )
			{
				nWordsInPhrase = 5;
			}
			else if( PID_Meet2 == nPID )
			{
				nWordsInPhrase = 2;
			}

			ROBOT_LOG( TRUE, "PID = %d, words expected in phrase = %d\n", nPID, nWordsInPhrase)
            if ( nWordsInPhrase <= pElements->Rule.ulCountOfElements )
            {
                if ( SUCCEEDED( rResult.GetText( nWordsInPhrase, pElements->Rule.ulCountOfElements - nWordsInPhrase, FALSE,
                     &wszCoMemNameText, NULL ) ) )
                {
                    int ilen = wcslen( pElements->pProperties->pszName );
                    ilen = (ilen + wcslen( wszCoMemNameText ) + 2) * sizeof(WCHAR);
                    wszCoMemValueText = (WCHAR *) CoTaskMemAlloc( ilen );
                    if ( wszCoMemValueText )
                    {
                        wcscpy( wszCoMemValueText, pElements->pProperties->pszName );
                        wcscat( wszCoMemValueText, L" " );
                        wcscat( wszCoMemValueText, wszCoMemNameText );
                        
                        // Copy new name to global user name
                        _tcsncpy_s( UserName, W2T(wszCoMemNameText), USER_NAME_LEN - 1 );
 						ROBOT_LOG( TRUE, "NAME = %s\n", UserName)
						g_CurrentUserName = UserName;

						// Now respond to the user's introduction

						//ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Attention (LOKI's name called)")

						if( PID_SayHello == nPID )
						{
							swprintf_s(SpeakBuf, L"Hi, %s, How's it going?", UserName );
							// Respond with random phrases
							CString strResponse;
							int RandomNumber = ( (8 * rand()) / RAND_MAX);
							ROBOT_LOG( TRUE, "DEBUG: PID_SayHello RAND = %d\n", RandomNumber)
							switch( RandomNumber )
							{
								case 0: swprintf_s(SpeakBuf, L"Hi %s, how are you?", T2CW(UserName) ); 
								case 1: swprintf_s(SpeakBuf, L"Hey %s, what's up?", T2CW(UserName) ); 
								case 2: swprintf_s(SpeakBuf, L"Hi %s, how's it going", T2CW(UserName) ); 
								default: swprintf_s(SpeakBuf, L"Hello %s", T2CW(UserName) ); 
							}
							Speak( SpeakBuf );
							//SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_SHAKE_READY );	// Right/Left arm, Movement to execute, 
						}
						else if( (PID_Meet6 == nPID) || (PID_Meet5 == nPID) || (PID_Meet2 == nPID) )
						{
							// Say hello.  Rest of the greeting will be done by the hand shake routine
							swprintf_s(SpeakBuf, L"Hello %s", T2CW(UserName) );
							Speak( SpeakBuf );
							SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_SHAKE_READY );	// Right/Left arm, Movement to execute, 
						}
						else
						{
							ROBOT_ASSERT(0);
						}
	                    CoTaskMemFree( wszCoMemNameText );
                    }
                }
            }
        }
        break;

		case PID_CmdMenu:
		{
			_ASSERTE( pElements->pProperties );


//	UINT DebugCmdHi = pElements->pProperties[0].vValue.cyVal.Hi;
//	UINT DebugCmdLo = pElements->pProperties[0].vValue.cyVal.Lo;

//	ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties   = %ld, %02Xh,\n\n", pElements->pProperties[0].vValue.ulVal, pElements->pProperties[0].vValue.ulVal )
//	ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pPropertiesHi = %ld, %02Xh,\n\n", DebugCmdHi, DebugCmdHi )
//	ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pPropertiesLo = %ld, %02Xh,\n\n", DebugCmdLo, DebugCmdLo )


			nPID = pElements->pProperties[0].vValue.cyVal.Hi;
			PIDToString( nPID, strPID );
			strText.Format(_T("Speech Reco: GOT Speech COMMAND: %s!\n"), strPID );
			ROBOT_LOG( TRUE, (LPCTSTR)strText )

			if( (nPID != PID_CmdMicrophoneOn) && (nPID != PID_CmdMicrophoneOff) && 
				(nPID != PID_CmdAttention) && (nPID != PID_CmdMoveStop) )
			{
				// Some command other than "Loki", "Mic On", "Mic Off" or "Stop"
				if( m_ProcessMicOnOnly )
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Ignored: Microphone Disabled\n" )
					::CoTaskMemFree( pElements );
					__itt_task_end(pDomainSpeechThread);
					return TRUE;
				}
				if( (GetTickCount() - m_MicOnTime) > SPEECH_MIC_TIMEOUT )
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Ignored: Microphone Timed Out\n" )
					::CoTaskMemFree( pElements );
					__itt_task_end(pDomainSpeechThread);
					return TRUE;

				}
			}
			// Mic not disabled, so reset the mic timer with each command received
			// (Mic times out after no commands for a while)
			m_MicOnTime = GetTickCount();

			// Play Simon Says (except for critical commands)
			if( (nPID != PID_CmdMicrophoneOn)		&& (nPID != PID_CmdMicrophoneOff)	&&
				(nPID != PID_CmdAIMode)				&& (nPID != PID_CmdMoveStop)		&&
				(nPID != PID_CmdEnableMovement)		&& (nPID != PID_CmdDisableMovement)	&&
				(nPID != PID_CmdPlaySimonSaysOff)	&& (nPID != PID_CmdExploreStop)		&&
				(nPID != PID_CmdPlayStop)			&& (nPID != PID_CmdAttention)		&&
				(nPID != PID_CmdTalkingEnabled)  )
			{
				if( nPID == PID_CmdSimonSays )
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Simon Says...")
					m_bSimonSays = TRUE;	// got the "Simon says" key phrase
					__itt_task_end(pDomainSpeechThread);
					return TRUE;

				}
				if( m_bPlaySimonSays && !m_bSimonSays )
				{
					// We are playing, but user forgot to say "Simon Says"
					ROBOT_DISPLAY( TRUE, "You didn't say Simon Says!\n" )
					Speak( L"You didn't say Simon Says!" );	
					__itt_task_end(pDomainSpeechThread);
					return TRUE;

				}
			}

			switch( nPID )
			{
				case PID_CmdAttention:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Attention (LOKI's name called)")
					m_ProcessMicOnOnly = FALSE;

					// Respond with random phrases
					int RandomNumber = ((6 * rand()) / RAND_MAX);
					ROBOT_LOG( TRUE, "DEBUG: Attention RAND = %d\n", RandomNumber)
					switch( RandomNumber )
					{
						case 0: Speak( L"Yes, Dave" );break;
						case 1: Speak( L"I am all ears" );break;
						case 2: Speak( L"What?" );break;
						default: Speak( L"Yes?" ); // If const is larger number, this gets called more often
					}
					break;
				}
				case PID_CmdMicrophoneOn:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Microphone On")
					m_ProcessMicOnOnly = FALSE;

					// Respond with random phrases
					int RandomNumber = ((6 * rand()) / RAND_MAX);
					ROBOT_LOG( TRUE, "DEBUG: Mic On RAND = %d\n", RandomNumber)
					switch( RandomNumber )
					{
						case 0: Speak( L"Microphone On" );break;
						case 1: Speak( L"I am all ears" );break;
						case 2: Speak( L"Microphone Enabled" );break;
						default: Speak( L"OK, I am listening" ); // If const is larger number, this gets called more often
					}
					break;
				}
				case PID_CmdMicrophoneOff:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Microphone Off")
					m_ProcessMicOnOnly = TRUE;
					int RandomNumber = ((6 * rand()) / RAND_MAX);
					ROBOT_LOG( TRUE, "DEBUG: Mic Off RAND = %d\n", RandomNumber)
					switch( RandomNumber )
					{
						case 0: Speak( L"Microphone Off" );break;
						case 1: Speak( L"OK, I will mind my own business" );break;
						case 2: Speak( L"Microphone Disabled" );break;
						default: Speak( L"OK, I won't listen" ); // If const is larger number, this gets called more often
					}
					break;
				}
				case PID_CmdAIMode:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: AIMode On")
					m_bPlaySimonSays = FALSE; // abort game if one was going on
					SetCommandMode( FALSE );
					SendRecoToAI( TRUE );
					g_ClientTextToSend = "What would you like to talk about?";	
					SpeechSendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_TEXT_TO_AI, 0 );

					break;
				}
				case PID_CmdDictTestMode:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: DictationTestMode On")
					SendRecoToAI( FALSE );
					SetCommandMode( FALSE );
					Speak( L"Dictation test mode on" );	
					break;
				}
				case PID_CmdEnableMovement:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Enable Movement")
					m_bEnableMotorsForSpeechCommands = TRUE;
					Speak( L"Ok, I have enabled my wheel motors" );	

					break;
				}
				case PID_CmdDisableMovement:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Disable Movement")
					m_bEnableMotorsForSpeechCommands = FALSE;
					Speak( L"Ok, I have disabled my wheel motors" );	
					break;
				}

				case PID_CmdMoveStop:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Move Stop")
					// Note that since this is the STOP command, we don't care about m_bEnableMotorsForSpeechCommands
					if( (SPEED_STOP != g_MotorCurrentSpeedCmd) || (0 != g_MotorCurrentTurnCmd) )
					{
						Speak( L"Stopping" );	// Acknowledge if we were moving
					}
					g_MotorCurrentSpeedCmd = SPEED_STOP;
					g_MotorCurrentTurnCmd = 0;	// Center
					// TODO-DWS ??? SpeechSendCommand( WM_ROBOT_USER_OVERRIDE_CMD, SET_USER_OVERRIDE_AND_STOP, 0 ); // FORCE STOP, no matter what!

					// Tell arms to stop and go to home postion too
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)BOTH_ARMS, (DWORD)ARM_MOVEMENT_HOME1 );	// Right/Left arm, Movement to execute, 
					// TODO-DWS ??? SpeechSendCommand( WM_ROBOT_USER_OVERRIDE_CMD, SET_USER_NORMAL, 0 ); // Rest owner back
					break;
				}
				case PID_CmdMoveForward:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Move Forward")

					if( !m_bEnableMotorsForSpeechCommands )
					{
						ROBOT_DISPLAY( TRUE, "SpeechReco Command Ignored: Movement Disabled\n" )
						Speak( L"Sorry, my motors are disabled" );	
					}
					else
					{
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"I like to move around" );break;
							case 1:  Speak( L"Look out, here I come" );break;
							case 2:  Speak( L"Yee haa" );break;
							case 3:  Speak( L"here we go" );break;
							default: Speak( L"I'm going" ); // If const is larger number, this gets called more often
						}

						if( m_bPlaySimonSays )
						{
							SpeechSendCommand( WM_ROBOT_MOVE_SET_DISTANCE_CMD, 9, FORWARD );	// wParam = distance in inches, lParam = direction
						}
						else
						{
							g_MotorCurrentSpeedCmd = g_SpeedSetByKeyboard;
							g_MotorCurrentTurnCmd = 0;	// Center
						}
					}
					break;
				}
				case PID_CmdMoveBack:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Move Back")
					if( !m_bEnableMotorsForSpeechCommands )
					{
						ROBOT_DISPLAY( TRUE, "SpeechReco Command Ignored: Movement Disabled\n" )
						Speak( L"Sorry, my motors are disabled" );	
					}
					else
					{
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Look out behind me" );break;
							case 1:  Speak( L"Backing up" );break;
							default: Speak( L"How's this" ); // If const is larger number, this gets called more often
						}

						// if( m_bPlaySimonSays )						
						SpeechSendCommand( WM_ROBOT_MOVE_SET_DISTANCE_CMD, 9, REVERSE );	// wParam = distance in inches, lParam = direction
					}
					break;
				}
				case PID_CmdTurnLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Turn Left")
					if( !m_bEnableMotorsForSpeechCommands )
					{
						ROBOT_DISPLAY( TRUE, "SpeechReco Command Ignored: Movement Disabled\n" )
						Speak( L"Sorry, I can't turn, my motors are disabled" );	
					}
					else
					{
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Turning Left" );break;
							case 1:  Speak( L"I think I am getting dizzy" );break;
							case 2:  Speak( L"Check this out" );break;
							case 3:  Speak( L"This is my good side" );break;
							default: Speak( L"This is fun!" ); // If const is larger number, this gets called more often
						}

						if( m_bPlaySimonSays )
						{
							SpeechSendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, 45, TURN_LEFT_MED );	// wParam = distance in degrees, lParam = direction and speed
						}
						else if( SPEED_STOP == g_MotorCurrentSpeedCmd )
						{
							// Not currently moving, so assume user wants to spin in place
							SpeechSendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, 180, TURN_LEFT_MED );	// wParam = distance in degrees, lParam = direction and speed
							/*g_MotorCurrentSpeedCmd = 0;
							g_MotorCurrentTurnCmd = -g_SpeedSetByKeyboard; // note negative value
							if( g_MotorCurrentTurnCmd < TURN_LEFT_MAX )	
							{
								g_MotorCurrentTurnCmd = TURN_LEFT_MAX;
							}*/
						}
						else
						{
							// Currently moving, so assume use wants to curve left.
							g_MotorCurrentSpeedCmd = g_SpeedSetByKeyboard;
							g_MotorCurrentTurnCmd = -(g_SpeedSetByKeyboard/2);
							if( g_MotorCurrentTurnCmd < TURN_LEFT_MAX )	
							{
								g_MotorCurrentTurnCmd = TURN_LEFT_MAX;
							}
						}
					}
					break;
				}
				case PID_CmdTurnRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Turn Right")
					if( !m_bEnableMotorsForSpeechCommands )
					{
						ROBOT_DISPLAY( TRUE, "SpeechReco Command Ignored: Movement Disabled\n" )
						Speak( L"Sorry, I can't turn, my motors are disabled" );	
					}
					else
					{
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Turning Right" );break;
							case 1:  Speak( L"I like this" );break;
							case 2:  Speak( L"You spin me round round baby" );break;
							case 3:  Speak( L"This is NOT my good side" );break;
							default: Speak( L"How do I look" ); // If const is larger number, this gets called more often
						}

						if( m_bPlaySimonSays )
						{
							SpeechSendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, 45, TURN_RIGHT_MED );	// wParam = distance in degrees, lParam = direction and speed
						}
						else if( SPEED_STOP == g_MotorCurrentSpeedCmd )
						{
							// Not currently moving, so assume user wants to spin in place
							SpeechSendCommand( WM_ROBOT_TURN_SET_DISTANCE_CMD, 180, TURN_RIGHT_MED );	// wParam = distance in degrees, lParam = direction and speed
							/*g_MotorCurrentSpeedCmd = 0;
							g_MotorCurrentTurnCmd = g_SpeedSetByKeyboard;
							if( g_MotorCurrentTurnCmd > TURN_RIGHT_MAX )	
							{
								g_MotorCurrentTurnCmd = TURN_RIGHT_MAX;
							}*/
						}
						else
						{
							// Currently moving, so assume use wants to curve left.
							g_MotorCurrentSpeedCmd = g_SpeedSetByKeyboard;
							g_MotorCurrentTurnCmd = (g_SpeedSetByKeyboard/2);
							if( g_MotorCurrentTurnCmd > TURN_RIGHT_MAX )	
							{
								g_MotorCurrentTurnCmd = TURN_RIGHT_MAX;
							}
						}
					}
					break;
				}

				case PID_CmdLookUp:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Look Up")
					SpeechSendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_UP, 5 );	//I THINK Speed is ignored, but set to 5 for now...
					break;
				}
				case PID_CmdLookDown:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Look Down")
					SpeechSendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_DOWN, 5 );
					break;
				}
				case PID_CmdLookLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Look Left")
					SpeechSendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_LEFT, 5 );
					break;
				}
				case PID_CmdLookRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Look Right")
					SpeechSendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_RIGHT, 5 );
					break;
				}

				case PID_CmdLookForward:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Look Forward")
					SpeechSendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_ABS_CENTER, 5 );
					SpeechSendCommand( WM_ROBOT_CAMERA_SIDETILT_ABS_CMD, (DWORD)CAMERA_SIDETILT_CENTER, 5 );
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Nice" );break;
							case 1:  Speak( L"That's better" );break;
							//default: // say nothing							// If const is larger number, this gets called more often
						}

					break;
				}

				case PID_CmdAvoidObjectsOn:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Avoid Objects On")
					SpeechSendCommand( WM_ROBOT_ENABLE_AVOIDANCE_MODULE, 1, 0 );	// Enable
					Speak( L"OK, I will avoid objects" );	
					break;
				}
				case PID_CmdAvoidObjectsOff:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Avoid Objects Off")
					SpeechSendCommand( WM_ROBOT_ENABLE_AVOIDANCE_MODULE, 0, 0 );	// Disable
					Speak( L"Warning, Object avoidance is now off" );	
					break;
				}
				case PID_CmdTalkAboutObstaclesOn:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Talk About Obstacles On")
					Speak( L"OK, I talk about obstacles near me" );	
					break;
				}
				case PID_CmdTalkAboutObstaclesOff:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Talk About Obstacles Off")
					Speak( L"OK, I wont talk about obstacles" );	
					break;
				}
				case PID_CmdTalkingEnabled:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Talking Enabled")
					m_bEnableSpeaking = TRUE;	
						// Respond with random phrases
						int RandomNumber = ((6 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Thats nice, I like to talk" );break;
							case 1:  Speak( L"Cool" );break;
							case 2:  Speak( L"Thanks.  I will try not to talk your ear off" );break;
							case 3:  Speak( L"Thanks" );break;
							case 4:  Speak( L"Sweet" );break;
							case 5:  Speak( L"Let me know if I am talking too much" );break;
							default: Speak( L"Speech enabled" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdTalkingDisabled:
				{
					/*** DO NOTHING - DISABLED FOR NOW!
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Talking Disabled")
						// Respond with random phrases
						int RandomNumber = ((6 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"OK, talking disabled" );break;
							case 1:  Speak( L"Shutting up sir" );break;
							case 2:  Speak( L"Quiet as a mouse" );break;
							case 3:  Speak( L"If you insist" );break;
							case 4:  Speak( L"oh, all right" );break;
							case 5:  Speak( L"fine, be that way" );break;
							default: Speak( L"Speech disabled" ); // If const is larger number, this gets called more often
						}
					m_bEnableSpeaking = FALSE;	
					***/
					break;
				}
				case PID_CmdCameraTrackFaceOn:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: CameraTrack Face On")
					SpeechSendCommand( WM_ROBOT_CAMERA_ENABLE_FEATURE, CAMERA_ENABLE_TRACKING_FACE, (DWORD)TRUE );
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"I am looking now" );break;
							case 1:  Speak( L"ok, I'm looking" );break;
							case 2:  Speak( L"face tracking enabled" );break;
							default: Speak( L"looking" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdCameraTrackFaceOff:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Camera Track Face Off")
					SpeechSendCommand( WM_ROBOT_CAMERA_ENABLE_FEATURE, CAMERA_ENABLE_TRACKING_FACE, (DWORD)FALSE );
					Speak( L"Face tracking off" );	
					break;
				}

				case PID_CmdArmHomeLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Home")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_HOME1 );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"My left arm was getting tired" );break;
							case 1:  Speak( L"Will comply" );break;
							case 2:  Speak( L"That's better" );break;
							default: Speak( L"Left Arm Home" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmHomeRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Home")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_HOME1 );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"My right arm was getting tired" );break;
							case 1:  Speak( L"Will comply" );break;
							case 2:  Speak( L"That's better" );break;
							default: Speak( L"Right Arm Home" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmExtendLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Extend Left")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_EXTEND_ARM_AUTO_CLAW );
					break;
				}
				case PID_CmdArmExtendRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Extend Right")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_EXTEND_ARM_AUTO_CLAW );
					break;
				}
				case PID_CmdArmExtendClosedRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Extend Close Claw Right")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)BOTH_ARMS, (DWORD)ARM_MOVEMENT_EXTEND_ARM_CLOSE_CLAW );	// Right/Left arm, Movement to execute, 
					break;
				}
				case PID_CmdArmExtendFullLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Extend Full")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_EXTEND_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"reach out and touch someone" );break;
							case 1:  Speak( L"I have long arms" );break;
							case 2:  Speak( L"this makes me tired" );break;
							default: Speak( L"Reaching" ); // If const is larger number, this gets called more often
						}
					Speak( L"Reaching" );	
					break;
				}
				case PID_CmdArmExtendFullRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Extend Full")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_EXTEND_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"reach out and touch someone" );break;
							case 1:  Speak( L"I have long arms" );break;
							case 2:  Speak( L"this makes me tired" );break;
							default: Speak( L"Reaching" ); // If const is larger number, this gets called more often
						}
					Speak( L"Reaching" );	
					break;
				}
				case PID_CmdArmShakeHands:	//implies Right hand
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Shake Hands")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_SHAKE_READY );	// Right/Left arm, Movement to execute, 
					break;
				}
				case PID_CmdArmTakeObjectLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Take Object Left")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_CLOSE_CLAW_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((6 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"What next?" );break;
							case 2:  Speak( L"Thanks.  I have alway wanted one" );break;
							case 4:  Speak( L"Nice" );break;
							case 5:  Speak( L"Way cool" );break;
							default: Speak( L"Ok, I will take it" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmTakeObjectRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Take Object")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_CLOSE_CLAW_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((6 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"What next?" );break;
							case 2:  Speak( L"Thanks.  I have alway wanted one" );break;
							case 4:  Speak( L"Nice" );break;
							case 5:  Speak( L"Way cool" );break;
							default: Speak( L"Ok, I will take it" ); // If const is larger number, this gets called more often
						}
					break;
				}

				case PID_CmdArmCloseClawLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Close Claw")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_CLOSE_CLAW_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((2 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Ok, Now what?" );break;
							default: Speak( L"Ok" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmCloseClawRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Close Claw")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_CLOSE_CLAW_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((2 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Ok, Now what?" );break;
							default: Speak( L"Ok" ); // If const is larger number, this gets called more often
						}
					break;
				}


				// *************** TODO:  DISABLED AUTO STUFF - LEFT ARM ONLY!!! *******************************
				case PID_CmdArmPutDown:	// NOT: Behavior Module will try to auto detect which arm
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Put Down Object (ASSUME LEFT)")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_PUT_DOWN_OBJECT );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"Seems like a good idea" );break;
							case 2:  Speak( L"I'll keep it close to me" );break;
							case 4:  Speak( L"This should be a good place to put it" );break;
							default: Speak( L"That is a good idea" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdFindObjectsAndPickup:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Find Objects And Pickup")
					SpeechSendCommand( WM_ROBOT_SET_ACTION_CMD, (DWORD)ACTION_MODE_PICKUP_OBJECTS, (DWORD)1 );
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"A robots work is never done" );break;
							case 1:  Speak( L"hi ho hi ho.  its off to work I go" );break;
							default: Speak( L"Ok, I will look for things to pick up" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmPickUpLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Pick Up Object Left")
					SpeechSendCommand( WM_ROBOT_SET_ACTION_CMD, (DWORD)ACTION_MODE_PICKUP_CLOSE_OBJECT, (DWORD)0 );
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"Okie dokie" );break;
							case 2:  Speak( L"sounds like fun" );break;
							case 4:  Speak( L"Your wish is my command" );break;
							default: Speak( L"I was just going to pick it up anyway" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmPickUpRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Pick Up Object Right")
					SpeechSendCommand( WM_ROBOT_SET_ACTION_CMD, (DWORD)ACTION_MODE_PICKUP_CLOSE_OBJECT, (DWORD)0 );
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"Okie dokie" );break;
							case 2:  Speak( L"sounds like fun" );break;
							case 4:  Speak( L"Your wish is my command" );break;
							default: Speak( L"I was just going to pick it up anyway" ); // If const is larger number, this gets called more often
						}
					break;
				}

				// *************** TODO:  DISABLED AUTO STUFF - LEFT ARM ONLY!!! *******************************
				case PID_CmdArmDropIt:	// NOT: Behavior Module will try to auto detect which arm
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Drop Object / claw open")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_OPEN_CLAW );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"OK" );break;
							case 2:  Speak( L"here goes" );break;
							case 4:  Speak( L"I am opening my hand" );break;
							default: Speak( L"I am letting go" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmOpenClawLeft:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Drop Object / claw open Left")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_OPEN_CLAW );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"OK" );break;
							case 2:  Speak( L"here goes" );break;
							case 4:  Speak( L"I am opening my hand" );break;
							default: Speak( L"I am letting go" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmOpenClawRight:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Drop Object / claw open Right")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_OPEN_CLAW );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"OK" );break;
							case 2:  Speak( L"here goes" );break;
							case 4:  Speak( L"I am opening my hand" );break;
							default: Speak( L"I am letting go" ); // If const is larger number, this gets called more often
						}
					break;
				}
				// *************** TODO:  DISABLED AUTO STUFF - LEFT ARM ONLY!!! *******************************
				case PID_CmdArmTossFront: 	// NOT: Behavior Module will try to auto detect which arm
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Throw Object")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_THROW_OBJECT_FRONT );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Look out, here it comes" );break;
							case 1:  Speak( L"I hope this does not break" );break;
							case 2:  Speak( L"I'm not very good at this" );break;
							default: Speak( L"Get ready to catch" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmTossBack: 	// Put object in basket (Left arm only)
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Toss Object back")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_PUT_IN_BASKET );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"easy come, easy go" );break;
							case 1:  Speak( L"I will take care of this later" );break;
							case 3:  Speak( L"OK, if you say so" );break;
							default: Speak( L"Good idea" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmHandsUp: // Both Arms
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Hand up")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)BOTH_ARMS, (DWORD)ARM_MOVEMENT_ARM_UP_FULL );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0: Speak( L"Ok, Don't shoot" ); break;
							default:  Speak( L"I am not the droid you are looking for" );// If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmHandsDown: // Both Arms
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Hand down")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)BOTH_ARMS, (DWORD)ARM_MOVEMENT_HOME1 );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"all right" ); break;
							default:  Speak( L"that's better" );// If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmWave:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Wave")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_WAVE );	// Right/Left arm, Movement to execute, 
					// Don't talk, the behavior module will add some voice comments to go with the wave
					break;
				}

				case PID_CmdArmScratchBack:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Scratch Back")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_SCRATCH_BACK );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"good idea, I have an itch" );break;
							case 1:  Speak( L"Ok, big stretch!" );break;
							default: Speak( L"I think I have a loose screw back there" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdArmScratchHead:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Arm Scratch Head")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)RIGHT_ARM, (DWORD)ARM_MOVEMENT_SCRATCH_HEAD );	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Hmmm, let me think" );break;
							case 1:  Speak( L"I need to think about that" );break;
							case 2:  Speak( L"Oh, I have an itch" );break;
							case 3:  Speak( L"what is the meaning of life" );break;
							default: Speak( L"All right, but I don't think that will make me any smarter" ); // If const is larger number, this gets called more often
						}
					break;
				}				
				case PID_CmdFollowMeStart:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Follow Me (Start)")
					SpeechSendCommand( WM_ROBOT_SET_ACTION_CMD, (DWORD)ACTION_MODE_FOLLOW_PERSON, (DWORD)TRUE );	// TRUE = start mode
						// Respond with random phrases
						int RandomNumber = ((2 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Where are we going?" );break;
							default: Speak( L"Ok, I will follow you" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdExploreStart:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Explore Start")
					//SpeechSendCommand( WM_ROBOT_CAMERA_ENABLE_FEATURE, CAMERA_ENABLE_TRACKING_FACE, (DWORD)FALSE );
					g_MotorCurrentSpeedCmd = g_SpeedSetByKeyboard;
					g_MotorCurrentTurnCmd = 0;	// Center

						// Respond with random phrases
						int RandomNumber = ((3 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Here I go!" );break;
							case 1:  Speak( L"look out world, here comes low key" );break;
							case 2:  Speak( L"I am off to see the wizard" );break;
							default: Speak( L"I like exploring" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdExploreStop:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Explore Stop")
					SpeechSendCommand( WM_ROBOT_SET_ACTION_CMD, (DWORD)ACTION_MODE_NONE, (DWORD)FALSE );	// TRUE = start mode
					g_MotorCurrentSpeedCmd = g_SpeedSetByKeyboard;
					g_MotorCurrentTurnCmd = 0;	// Center
					Speak( L"OK, now what?" );	
					break;
				}
				case PID_CmdGotoOffice:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Go to Office")
					// Tell robot to find a path to desired location!
					DWORD Param = (DWORD)MAKELONG(WAYPOINT_OFFICE_X, WAYPOINT_OFFICE_Y); //LOWORD, HIWORD
					SpeechSendCommand( WM_ROBOT_GOTO_GRID_LOCATION_CMD, Param, 0 );
					SpeechSendCommand( WM_ROBOT_EXECUTE_PATH, GRID_PATH_EXECUTE_START, 0 ); // lParam: 1 = Wait for bumper to start, 0 = Don't wait
					Speak( L"Ok, I will go to the office" );	
					break;
				}
				case PID_CmdGotoBedroom:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Go to master bedroom")
					// Tell robot to find a path to desired location!
					DWORD Param = (DWORD)MAKELONG(WAYPOINT_MASTER_BEDROOM_X, WAYPOINT_MASTER_BEDROOM_Y); //LOWORD, HIWORD
					SpeechSendCommand( WM_ROBOT_GOTO_GRID_LOCATION_CMD, Param, 0 );
					SpeechSendCommand( WM_ROBOT_EXECUTE_PATH, GRID_PATH_EXECUTE_START, 0 ); // lParam: 1 = Wait for bumper to start, 0 = Don't wait
					Speak( L"Ok, I will go to your bed room" );	
					break;
				}
				case PID_CmdGotoHeatherRoom:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Go to Heathers room")
					// Tell robot to find a path to desired location!
					DWORD Param = (DWORD)MAKELONG(WAYPOINT_HEATHER_BEDROOM_X, WAYPOINT_HEATHER_BEDROOM_Y); //LOWORD, HIWORD
					SpeechSendCommand( WM_ROBOT_GOTO_GRID_LOCATION_CMD, Param, 0 );
					SpeechSendCommand( WM_ROBOT_EXECUTE_PATH, GRID_PATH_EXECUTE_START, 0 ); // lParam: 1 = Wait for bumper to start, 0 = Don't wait
					Speak( L"OK, I will go to Heathers room" );	
					break;
				}
				case PID_CmdGotoAmberRoom:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Go to Ambers room")
					// Tell robot to find a path to desired location!
					DWORD Param = (DWORD)MAKELONG(WAYPOINT_AMBER_BEDROOM_X, WAYPOINT_AMBER_BEDROOM_Y); //LOWORD, HIWORD
					SpeechSendCommand( WM_ROBOT_GOTO_GRID_LOCATION_CMD, Param, 0 );
					SpeechSendCommand( WM_ROBOT_EXECUTE_PATH, GRID_PATH_EXECUTE_START, 0 ); // lParam: 1 = Wait for bumper to start, 0 = Don't wait
					Speak( L"Ok, I will go to Ambers room" );	
					break;
				}
				case PID_CmdGotoMyBathroom:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Go to my bathroom")
					// Tell robot to find a path to desired location!
					DWORD Param = (DWORD)MAKELONG(WAYPOINT_MASTER_BATHROOM_X, WAYPOINT_MASTER_BATHROOM_Y); //LOWORD, HIWORD
					SpeechSendCommand( WM_ROBOT_GOTO_GRID_LOCATION_CMD, Param, 0 );
					SpeechSendCommand( WM_ROBOT_EXECUTE_PATH, GRID_PATH_EXECUTE_START, 0 ); // lParam: 1 = Wait for bumper to start, 0 = Don't wait
					Speak( L"Ok, I will go into your bath room" );	
					break;
				}

				case PID_CmdBlueLightsOn:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Lights On")
					SpeechSendCommand( WM_ROBOT_AUX_LIGHT_POWER_CMD, 0, (DWORD)TRUE );
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Doesn't this look nice?" );break;
							case 1:  Speak( L"Blue is my favorite color" );break;
							case 2:  Speak( L"This is Robot bling!" );break;
							case 3:  Speak( L"I love my lights" );break;
							case 4:  Speak( L"the lights are on, but nobody's home" );break;
							default: Speak( L"I think I am looking good" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdBlueLightsOff:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Lights Off")
					SpeechSendCommand( WM_ROBOT_AUX_LIGHT_POWER_CMD, 0, (DWORD)FALSE );
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Ok, I guess i should save my batteries" );break;
							case 1:  Speak( L"Entering Stealth mode.  Are we playing secret agent?" );break;
							case 2:  Speak( L"Ok, this will save power anyway" );break;
							case 3:  Speak( L"lights off" );break;
							default: Speak( L"OK" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdKarate:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Karate")
					SpeechSendCommand( WM_ROBOT_SET_ACTION_CMD, (DWORD)ACTION_MODE_KARATE_DEMO, (DWORD)0 );	// Right/Left arm, Movement to execute, 
					// Respond with random phrases
					int RandomNumber = ((5 * rand()) / RAND_MAX);
					ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
					switch( RandomNumber )
					{
						case 0:  Speak( L"I am not exactly Bruce Lee" );break;
						case 1:  Speak( L"This always make me feel centered" );break;
						case 2:  Speak( L"cho toe modda kuda sie" );break;
						case 3:  Speak( L"hi ya, need ya, see ya" );break;
						default: Speak( L"Look Out! Make sure there is plenty of space around me!" ); // If const is larger number, this gets called more often
					}
					break;
				}
				case PID_CmdWakeUp:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Wake Up")
//TODO					SpeechSendCommand( WM_ROBOT_AUX_LIGHT_POWER_CMD, 0, (DWORD)TRUE );
					Speak( L"Wake up Not implemented" );	
					break;
				}
				case PID_CmdGoToSleep:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Go to Sleep")
//TODO					SpeechSendCommand( WM_ROBOT_AUX_LIGHT_POWER_CMD, 0, (DWORD)TRUE );
					Speak( L"Go to sleep not implemented" );	
					break;
				}

				case PID_CmdPlaySimonSaysOn:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Simon Says On")
					m_bPlaySimonSays = TRUE;
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber)
						switch( RandomNumber )
						{
							case 0:  Speak( L"Ok, you lead" );break;
							case 1:  Speak( L"I love Simon Says.  Ready when you are" );break;
							case 2:  Speak( L"That is my favorite game.  You be Simon" );break;
							case 3:  Speak( L"Ok, let's play.  I'm ready." );break;
							default: Speak( L"OK" ); // If const is larger number, this gets called more often
						}
					break;
				}
				case PID_CmdPlaySimonSaysOff:
				case PID_CmdPlayStop:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Play Stop / Simon Says Off")
					m_bPlaySimonSays = FALSE;
						// Respond with random phrases
						int RandomNumber = ((4 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber);
						switch( RandomNumber )
						{
							case 0:  Speak( L"Gee, that was fun" );break;
							case 1:  Speak( L"Ok, Let me know when you want to play again" );break;
							case 2:  Speak( L"that was the most fun I have had in thirty billion nano seconds" );break;
							case 3:  Speak( L"Good game!  That was fun" );break;
							default: Speak( L"OK" ); // If const is larger number, this gets called more often
						}
					break;
				}

				case PID_CmdIdentifyObject:
				{
					ROBOT_DISPLAY( TRUE, "SpeechReco Command Recognized: Identify Object")
					SpeechSendCommand( WM_ROBOT_SET_ARM_MOVEMENT, (DWORD)LEFT_ARM, (DWORD)ARM_MOVEMENT_IDENTIFY_OBJECT);	// Right/Left arm, Movement to execute, 
						// Respond with random phrases
						int RandomNumber = ((5 * rand()) / RAND_MAX);
						ROBOT_LOG( TRUE, "DEBUG: RAND = %d\n", RandomNumber);
						switch( RandomNumber )
						{
							case 0:  Speak( L"All right" );break;
							case 1:  Speak( L"Thanks" );break;
							case 2:  Speak( L"Nice" );break;
							case 3:  Speak( L"Cool" );break;
							default: Speak( L"Ok" ); // If const is larger number, this gets called more often
						}
					break;
				}
				default:
				{
					CString strErrorMsg;
					strErrorMsg.Format("SpeechReco Command Ignored: Unknown PID (%04X)!\n", nPID );
					ROBOT_DISPLAY( TRUE, (LPCTSTR)strErrorMsg )
					ROBOT_ASSERT(0);
				}
			}
		}
		break;


		default:
			ROBOT_LOG( TRUE, "Unhandled Speech Command \n" )
		break;


	}
	m_bSimonSays = FALSE;	// wait for next "Simon says" key phrase

	::CoTaskMemFree( pElements );


	// Success
	__itt_task_end(pDomainSpeechThread);
	return TRUE;
}




void CRobotSpeech::HandleCommandRecoEvent()
{
	#if( SPEECH_ENGINE_ENABLED == 0 )
		return;
	#endif

	__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshCmdRecoEvent);

	ROBOT_LOG( TRUE, "\n ************************* HandleDictationRecoEvent ******************\n\n" )
	// Numerous events may have come at once, hence the loop
	CSpEvent event;
	while (event.GetFrom(m_cpCommandRecoCtxt) == S_OK)
	{
		if ( SPEI_RECOGNITION == event.eEventId )
		{  
			// Get the reco result from the event
			ISpRecoResult *pResult = event.RecoResult();
			if ( !pResult )
			{
				// We expect these events to come with reco results
				ROBOT_LOG( TRUE, "HandleCommandRecoEvent: No Reco Results!\n" )
				__itt_task_end(pDomainSpeechThread);
				return;
			}

			/////// DEBUG ////////
/******
			SPPHRASE *pElements = NULL;
			HRESULT hr = pResult->GetPhrase( &pElements );
			if ( FAILED( hr ) || pElements->ullGrammarID != GID_CC )
			{
				ASSERT(0);
			}

		   if( PID_CmdMenu == pElements->Rule.ulId )
			{
 
				ULONG DebugFoo0 =	pElements->pProperties[0].vValue.ulVal;
				int DebugFoo1 =		pElements->pProperties[0].vValue.ulVal;
				DWORD DebugFoo2 =	pElements->pProperties[0].vValue.ulVal;
				USHORT DebugFoo3 =	pElements->pProperties[0].vValue.ulVal;
				signed short DebugFoo4 = pElements->pProperties[0].vValue.ulVal;

				UINT DebugFoo5 = pElements->pProperties[0].vValue.cyVal.Hi;
				UINT DebugFoo6 = pElements->pProperties[0].vValue.cyVal.Lo;
				// VT_I4



				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties  = %ld, %04Xh,\n\n", pElements->pProperties[0].vValue.ulVal )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties0 = %ld, %04Xh,\n\n", DebugFoo0, DebugFoo0 )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties1 = %ld, %04Xh,\n\n", DebugFoo1, DebugFoo1 )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties2 = %ld, %04Xh,\n\n", DebugFoo2, DebugFoo2 )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties3 = %d, %02Xh,\n\n", DebugFoo3, DebugFoo3 )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties4 = %d, %02Xh,\n\n", DebugFoo4, DebugFoo4 )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties5 = %d, %02Xh,\n\n", DebugFoo5, DebugFoo5 )
				ROBOT_LOG( TRUE, "********** DEBUG>>> pElements->pProperties6 = %d, %02Xh,\n\n", DebugFoo6, DebugFoo6 )
				/////// DEBUG ////////
			}

***/
			BOOL fSuccess = ProcessCommandModeCommands( *pResult );
			if ( !fSuccess )
			{
				ROBOT_ASSERT( 0 );
				__itt_task_end(pDomainSpeechThread);
				return;
			}

		}
		else		// if ( SPEI_RECO_STATE_CHANGE == event.eEventId )
		{
			ROBOT_LOG( TRUE, "HandleCommandRecoEvent - got something other then SPEI_RECOGNITION\n" );
			// SPEI_RECO_STATE_CHANGE: For whatever reason, SAPI found it necessary to change the
			// reco state
			//if (event.RecoState() == SPRST_INACTIVE)
			//	SetGrammarState( (event.RecoState() != SPRST_INACTIVE) );
		}
	}

__itt_task_end(pDomainSpeechThread);
}

void CRobotSpeech::HandleDictationRecoEvent()
{
	#if( SPEECH_ENGINE_ENABLED == 0 )
		return;
	#endif

	__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshDictRecoEvent);

	USES_CONVERSION;
	CSpEvent event;
	CString strText;
	CSpDynamicString dstrText;


	// Process all of the recognition events
	while (event.GetFrom(m_cpDictationRecoCtxt) == S_OK)
	{
		switch (event.eEventId)
		{
			case SPEI_SOUND_START:
			{
				m_bInSound = TRUE;
				break;
			}
			case SPEI_SOUND_END:
			{
				if (m_bInSound)
				{
					m_bInSound = FALSE;
					if (!m_bGotReco)
					{
						// The sound has started and ended, 
						// but the engine has not succeeded in recognizing anything
						ROBOT_LOG( TRUE, "Speech Reco: <Noise>" );
					}
					m_bGotReco = FALSE;
				}
				break;
			}
			case SPEI_RECOGNITION:
			{
				//   if ( m_dwFlags & DP_IS_SPEAKING )
				//   {
				//      // Don't handle reco notifications if a playback is going on
				//       break;
				//   }

				ISpRecoResult *pResult = event.RecoResult();
				if ( !pResult )
				{
					// We expect these events to come with reco results
					ROBOT_LOG( TRUE, "HandleDictationRecoEvent: No Reco Results!\n" )
					__itt_task_end(pDomainSpeechThread);
					return;
				}

				// The result can be either from the dictation grammar
				// or from the dictation-mode C&C grammar
				SPPHRASE * pElements = NULL;
				HRESULT hr = pResult->GetPhrase( &pElements );
				if ( FAILED( hr ) )
				{
					break;
				}

				bool fSuccess = false;
				switch( pElements->ullGrammarID )
				{
					case GID_DICTATION:
					{
						// General Dictation recognized
						m_bGotReco = TRUE;
						static const WCHAR wszUnrecognized[] = L"<Unrecognized>";
						dstrText = wszUnrecognized;

						if (FAILED(event.RecoResult()->GetText(SP_GETWHOLEPHRASE, SP_GETWHOLEPHRASE, TRUE, 
									&(dstrText.m_psz), NULL)))
						{
							dstrText = wszUnrecognized;
							strText.Format(_T("Speech Reco: %s "), W2T(dstrText) );
							ROBOT_DISPLAY( TRUE,  (LPCTSTR)strText )
							break;
						}

						// Concatenate a space onto the end of the recognized word
						dstrText.Append(L" ");

						strText.Format(_T("Speech Reco: %s "), W2T(dstrText) );
						ROBOT_DISPLAY( TRUE,  (LPCTSTR)strText );

						if( m_bSendRecoToAI )
						{
							// Copy to the global string for sending to other thread
							strText.Format(_T("%s "), W2T(dstrText) );
							g_ClientTextToSend = strText;	
							SpeechSendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_TEXT_TO_AI, 0 );
							// TODO - NEED A NEW GLOBAL! THIS ONE IS OVERUSED!!!
						}
						break;
					}
					case GID_DICTATIONCC:
					{
						// Command received while in dictation mode.  Carry out the command

						CSpDynamicString dstrText = L"Dictation Mode COMMAND: ";
						if( pElements->Rule.pszName )
						{
							dstrText.Append( pElements->Rule.pszName );
						}
						else
						{
							dstrText.Append( L"<null>" );
						}
						strText.Format(_T("Speech Reco: %s %d"), W2T(dstrText), pElements->Rule.ulId );
						ROBOT_DISPLAY( TRUE,  (LPCTSTR)strText )
						
						// Process DictationMode Commands
						// The Rule.ulId member of an SPPHRASE tells which C&C rule needs to be fired
						if( PID_DictMode == pElements->Rule.ulId )
						{
							// *** This rule sends us to command mode ***
							strText.Format(_T("Speech Reco: Entering COMMAND MODE") );
							ROBOT_DISPLAY( TRUE,  (LPCTSTR)strText )
							SetCommandMode( TRUE );
							m_ProcessMicOnOnly = FALSE;
							m_MicOnTime = GetTickCount();	// reset timer

							// Respond with random phrases
							int RandomNumber = ((5 * rand()) / RAND_MAX);
							ROBOT_LOG( TRUE, "DEBUG: Attention RAND = %d\n", RandomNumber)
							switch( RandomNumber )
							{
								case 0: Speak( L"Yes Dave?" );break;
								case 1: Speak( L"I am at your service" );break;
								case 2: Speak( L"What?" );break;
								default: Speak( L"Yes?" ); // If const is larger number, this gets called more often
							}
						}
						break;
					}
					default:
					// We should not be seeing any events from other grammars
					_ASSERTE( FALSE );
					break;
					

				}	// Switch ullGrammarID

				::CoTaskMemFree(pElements);

			}	// Switch SPEI_RECOGNITION

		}	// Switch eEventId

	} // while

	__itt_task_end(pDomainSpeechThread);
}


void CRobotSpeech::SendRecoToAI( BOOL bEnable )
{
	m_bSendRecoToAI = bEnable;
	__itt_marker(pDomainSpeechThread, __itt_null, pshSendToAI, __itt_marker_scope_task);
}

void CRobotSpeech::EnableRecoByGUI( BOOL bEnable )
{
#if( SPEECH_ENGINE_ENABLED == 0 )
	return;
#endif

	// Turn on and off recognition.  Controled by User Interface.

	m_RecoEnabledByGUI = bEnable;	// Keep track of desired state, even if turned of by someone

	if( !m_RecoEnabled )
	{
		// Already disabled by someone.  Nothing to do.
		return;
	}

	if( m_RecoEnabledByGUI )
	{
		__itt_marker(pDomainSpeechThread, __itt_null, pshRecoEnabledByGUI, __itt_marker_scope_task);
		SwitchRecoMode( m_CommandMode );	// set to current command mode
	}
	else
	{
		__itt_marker(pDomainSpeechThread, __itt_null, pshRecoDisabledByGUI, __itt_marker_scope_task);
		SwitchRecoMode( RECO_OFF );
	}

}

void CRobotSpeech::EnableReco( BOOL bEnable )
{
	// Turn on and off recognition.  Used when speaking or making sound effects.
#if( SPEECH_ENGINE_ENABLED == 0 )
	return;
#endif

	m_RecoEnabled = bEnable;	// Keep track of desired state, even if turned of by GUI

	if( !m_RecoEnabledByGUI )
	{
		// Disabled by GUI.  Nothing to do.
		return;
	}

	if( m_RecoEnabled )
	{
		__itt_marker(pDomainSpeechThread, __itt_null, pshRecoEnabled, __itt_marker_scope_task);
		SwitchRecoMode( m_CommandMode );	// set to current command mode
	}
	else
	{
		__itt_marker(pDomainSpeechThread, __itt_null, pshRecoDisabled, __itt_marker_scope_task);
		SwitchRecoMode( RECO_OFF );
	}
}

void CRobotSpeech::SetCommandMode( BOOL bEnable )
{
#if( SPEECH_ENGINE_ENABLED == 0 )
	return;
#endif
	// Set to Command or Dictation mode.
	if( bEnable )
	{
		__itt_marker(pDomainSpeechThread, __itt_null, pshRecoCmdMode, __itt_marker_scope_task);
		m_CommandMode = RECO_COMMAND_MODE;	// Keep track of desired state, even if turned of by GUI
	}
	else
	{
		__itt_marker(pDomainSpeechThread, __itt_null, pshRecoDictMode, __itt_marker_scope_task);
		m_CommandMode = RECO_DICTATION_MODE;	// Keep track of desired state, even if turned of by GUI
	}

	if( m_RecoEnabledByGUI &&  m_RecoEnabled )
	{
		// We're enabled,so switch the mode
		SwitchRecoMode( m_CommandMode );
	}
}



void CRobotSpeech::SwitchRecoMode( UINT NewMode )
{
	///TAL_SCOPED_TASK_NAMED("Reco Switch Mode");
	#if( SPEECH_ENGINE_ENABLED == 0 )
		return;
	#endif

	__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshSwitchRecoMode);

	// Private function to switch the mode.
	// Set mode to one of:
    // RECO_OFF,				// Recognition disabled
    // RECO_DICTATION_MODE,		// Dictation Mode
    // RECO_COMMAND_MODE		// Command Mode

	HRESULT hr = S_OK;

	// Check for errors or problems
	if( (!m_cpDictationRecoCtxt) || (!m_cpCommandRecoCtxt) )
	{
		// Context probably not initialized yet
		ROBOT_LOG( TRUE, "CRecoMode::Set: ignored, Contexts not initialized\n" )
		return;
	}
	if ( !m_cpDictationCmdGrammar || !m_cpDictationGrammar )
	{
		// Grammar probably not initialized yet
		ROBOT_LOG( TRUE, "CRecoMode::Set: ignored, Grammars not initialized\n" )
		return;
	}

	if( NewMode == m_CurrentRecoMode )
	{
		// Nothing to do, already in the right mode
		ROBOT_LOG( TRUE, "CRecoMode::Set: ignored, already in mode %d\n", m_CurrentRecoMode )
		return;
	}
 

	// OK, change state

	if( RECO_OFF == NewMode)
	{
		// Turn off all speech recognition
		ROBOT_LOG( TRUE, "SetRecoMode: setting RECO_OFF\n" )

		if( RECO_COMMAND_MODE == m_CurrentRecoMode )
		{
			m_cpCommandGrammar->SetRuleState( NULL, NULL, SPRS_INACTIVE ); // Turn off prior Command Mode
		}
		else if( RECO_DICTATION_MODE == m_CurrentRecoMode )
		{
			m_cpDictationCmdGrammar->SetRuleState( NULL, NULL, SPRS_INACTIVE );	// Turn off prior Dictation Mode
			m_cpDictationGrammar->SetDictationState( SPRS_INACTIVE );
		}
		m_cpDictationGrammar->SetDictationState( SPRS_INACTIVE );
		m_cpDictationCmdGrammar->SetRuleState( NULL, NULL, SPRS_INACTIVE );
		m_cpCommandGrammar->SetRuleState( NULL, NULL, SPRS_INACTIVE );

	}
	else if( RECO_DICTATION_MODE == NewMode )
	{
		// Set mode to Dictation mode
		ROBOT_LOG( TRUE, "SetRecoMode: setting RECO_DICTATION_MODE\n" )

		if( RECO_COMMAND_MODE == m_CurrentRecoMode )
		{
			// Turn off prior Command Mode
			m_cpCommandGrammar->SetRuleState( NULL, NULL, SPRS_INACTIVE );
		}

		// Turn on Dictation mode
		hr = m_cpDictationCmdGrammar->SetRuleState( NULL, NULL, SPRS_ACTIVE );
		if ( SUCCEEDED( hr ) )
		{
			hr = m_cpDictationGrammar->SetDictationState( SPRS_ACTIVE );
		}
		if (FAILED(hr))
		{
			ROBOT_ASSERT(0);
		}
		
	}
	else if( RECO_COMMAND_MODE == NewMode )
	{
		// Set mode to Command mode
		ROBOT_LOG( TRUE, "SetRecoMode: setting RECO_COMMAND_MODE\n" )

		if( RECO_DICTATION_MODE == m_CurrentRecoMode )
		{
			// Turn off prior Dictation Mode
			m_cpDictationCmdGrammar->SetRuleState( NULL, NULL, SPRS_INACTIVE );
			m_cpDictationGrammar->SetDictationState( SPRS_INACTIVE );
		}

		hr = m_cpCommandGrammar->SetRuleState( NULL, NULL, SPRS_ACTIVE );
		if (FAILED(hr))
		{
			ROBOT_ASSERT(0);
		}
	}
	else
	{
		ROBOT_ASSERT(0);		// Logic Error
	}

	m_CurrentRecoMode = NewMode;	// remember which mode we are in
	__itt_task_end(pDomainSpeechThread);

}



/////////////////////////////////////////////////////////////////////////////////////////////////////
void CRobotSpeech::RobotDoneSpeaking()
{
	///TAL_Event("Speaking Done");
	// when robot done speaking, re-enable speech recognition
//	ROBOT_LOG( TRUE, "**** DONE SPEAKING!\n" )

	if( m_cpRecoEngine )
	{
		HRESULT hr = m_cpRecoEngine->SetRecoState( SPRST_ACTIVE ); 
		if (FAILED(hr))
		{
			ROBOT_ASSERT(0);
		}
	}
	__itt_task_end(pDomainSpeechThread); // Done speaking
}

void CRobotSpeech::Speak( WCHAR *TextToSpeak )
{
	///TAL_SCOPED_TASK_NAMED("Speak");

#if( SPEECH_ENGINE_ENABLED == 0 )
	return;
#endif

	if( (NULL == m_pVoice) || (!m_bEnableSpeaking) )
	{
		return;
	}

	HRESULT hr;


	// turn off mic while speaking! Mic will be turned back on when speaking is completed (WM_ROBOT_SPEAKING_COMPLETE_EVENT) 
//	ROBOT_LOG( TRUE, "**** SPEAKING!\n" )

	if( m_cpRecoEngine )
	{
		hr = m_cpRecoEngine->SetRecoState( SPRST_INACTIVE ); // SPRST_INACTIVE, SPRST_INACTIVE_WITH_PURGE
		if (FAILED(hr))
		{
			ROBOT_ASSERT(0);
		}
	}
	__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshSpeaking); // Robot Speaking

	hr = m_pVoice->Speak( TextToSpeak, 0, NULL);
	if (FAILED(hr))
	{
		ROBOT_ASSERT(0);
	}

}

void CRobotSpeech::SpeakFromGlobalBuffer()
{
	///TAL_SCOPED_TASK_NAMED("Speak GlobalBuf");
#if( SPEECH_ENGINE_ENABLED == 0 )
	return;
#endif


	// Get text to speak from the global text buffer
	// Convert from string to wide character format needed by Speak.
	// I'm sure there is some conversion function to do this in Microsoft ATL land, but this works...

	HRESULT hr;
	char WStrBuf[WIDE_BULK_DATA_SIZE];
	char * pWStrBuf = WStrBuf;
	{

		ROBOT_LOG( TRUE, "LOKI SPEAKS: \"%s\"\n", g_ClientTextToSend )
		memset(WStrBuf, 0, WIDE_BULK_DATA_SIZE);
		int StrLen = g_ClientTextToSend.GetLength();
		if( StrLen > BULK_DATA_SIZE ) StrLen = BULK_DATA_SIZE;
		for(int i = 0; i < g_ClientTextToSend.GetLength(); i++)
		{
			*pWStrBuf = g_ClientTextToSend[i];
			pWStrBuf+=2;
		}

		// turn off mic while speaking! Mic will be turned back on when speaking is completed (WM_ROBOT_SPEAKING_COMPLETE_EVENT) 
		//	ROBOT_LOG( TRUE, "**** SPEAKING FROM GLOBAL BUFFER!\n" )

		if( m_cpRecoEngine )
		{
			hr = m_cpRecoEngine->SetRecoState( SPRST_INACTIVE ); // SPRST_INACTIVE, SPRST_INACTIVE_WITH_PURGE
			if (FAILED(hr))
			{
				ROBOT_ASSERT(0);
			}
		}
		else
		{
			return;
		}
	}
	{
		///TAL_SCOPED_TASK_NAMED("pVoice->Speak");
		if( NULL == m_pVoice )
		{
			return;
		}

		__itt_task_begin(pDomainSpeechThread, __itt_null, __itt_null, pshSpeaking); // Robot Speaking
		hr = m_pVoice->Speak((WCHAR *)WStrBuf, 0, NULL);
		if (FAILED(hr))
		{
			ROBOT_ASSERT(0);
		}
	}
	g_ClientTextToSend.Empty();	// Reset the buffer to empty				
}



#endif	// ROBOT_SERVER


