#include "stdafx.h"
#include "ClientOrServer.h"
#if ( ROBOT_SERVER == 1 )	// This module used for Robot Server only

#include <math.h>
#include "thread.h"
#include "module.h"
#include <MMSystem.h>	// For Sound functions
#include "Globals.h"
#include <sapi.h>
//#include <stdio.h>
//#include <string.h>
//#include <atlbase.h>
#include <sphelper.h>
#include "Kinect.h"

#if ENABLE_ROBOT_AI == 1
	#include "RobotAI.h"
#endif

// ITT
//const wchar_t threadLoopsCounterName[] = L"Thread loops";
//const wchar_t taskOpsCounterName[] = L"Task ops";




//-----------------------------------------------------------------------------
//	Name: ControlThreadProc
//	This thread is the heart of the robot control system.
//	The tread gets status and requests from the message loop
//	and sends them to various modules for processing.
//	The order of dispatch is important!  Highest priority modules
//	are processed first.  When a module wishes to suppress futher
//	processing, it sets the CmdDisabled flag.
//-----------------------------------------------------------------------------
///
__itt_string_handle* pshControlThreadLoop = __itt_string_handle_create("ControlThreadLoop");
__itt_string_handle* pshSensorModule = __itt_string_handle_create("SensorModule");
__itt_string_handle* pshCameraModule = __itt_string_handle_create("CameraModule");
__itt_string_handle* pshKinectModule = __itt_string_handle_create("KinectModule");
__itt_string_handle* pshSystemModule = __itt_string_handle_create("SystemModule");
__itt_string_handle* pshCollisionModule = __itt_string_handle_create("CollisionModule");
__itt_string_handle* pshAvoidModule = __itt_string_handle_create("AvoidModule");
__itt_string_handle* pshUserCmdModule = __itt_string_handle_create("UserCmdModule");
__itt_string_handle* pshWayPointModule = __itt_string_handle_create("WayPointModuleX");
__itt_string_handle* pshGridNavModule = __itt_string_handle_create("GridNavModule");
__itt_string_handle* pshBehaviorModule = __itt_string_handle_create("BehaviorModule");
__itt_string_handle* pshControlThreadLoopTop = __itt_string_handle_create("Top of Loop"); //marker

__itt_string_handle* pSH2 = __itt_string_handle_create("index");


DWORD WINAPI ControlThreadProc( LPVOID NotUsed )
{
	IGNORE_UNUSED_PARAM (NotUsed);
	__itt_thread_set_name( "Control Thread" );


	MSG	msg;

	// Allocate robot modules
	CDriveControlModule	*pDriveControlModule = new CDriveControlModule();
	CSensorModule		*pSensorModule = new CSensorModule(pDriveControlModule);
	CCameraModule		*pCameraModule = new CCameraModule(pDriveControlModule);  
						 g_pCameraModule = (void*)pCameraModule; // Make pointer to class available to video callback
	CKinectModule		*pKinectModule = new CKinectModule(pDriveControlModule); // Kinect Module creates its own video thread 
						 g_pKinectModule = (void*)pKinectModule; // Make pointer to class available to video callback
	CSystemModule		*pSystemModule = new CSystemModule(pDriveControlModule);
	CCollisionModule	*pCollisionModule = new CCollisionModule(pDriveControlModule);
	CAvoidObjectModule	*pAvoidObjectModule = new CAvoidObjectModule(pDriveControlModule);
	CUserCmdModule		*pUserCmdModule = new CUserCmdModule(pDriveControlModule);
	CWayPointNavModule	*pWayPointNavModule = new CWayPointNavModule(pDriveControlModule, pSensorModule);
	CGridNavModule		*pGridNavModule = new CGridNavModule(pDriveControlModule, pSensorModule);
	CBehaviorModule		*pBehaviorModule = new CBehaviorModule(pDriveControlModule);


	Sleep(1500);	// Let other init run
	
	UINT UserMessage = 0;
	UINT LoopIndex = 0;

	while( 0 != GetMessage( &msg, NULL, WM_ROBOT_MESSAGE_BASE, (WM_ROBOT_MAX_ROBOT_MESSAGE) ) )
		// OK to filter out Windows messages, since we don't use WM_QUIT
	{
		{
			if( WM_ROBOT_THREAD_EXIT_CMD == (msg.message) )
			{
				break;	// Quit command received!
			}

			// Get message from the message queue and process it.
			if( (msg.message < WM_ROBOT_MESSAGE_BASE) || 
				((msg.message) >= WM_GUI_MESSAGES) ||		// Ignore messages intended for the GUI
				((msg.message) > WM_ROBOT_MAX_ROBOT_MESSAGE) )	// Or messages out of range
			{
				// Ignore any messages not intended for user (where do these come from?)
				//ROBOT_LOG( TRUE,  "CONTROL THREAD: message out of Range, WM_ROBOT_MESSAGE_BASE = 0x%08X, Msg = 0x%08lX, wParam = 0x%08lX, lParam = 0x%08lX\n", 
				//	temp, msg.message, msg.wParam, msg.lParam )
				continue;
			}
			UserMessage = msg.message;
			g_bCmdRecognized = FALSE;

			__itt_marker(pDomainControlThread, __itt_null, pshControlThreadLoopTop, __itt_marker_scope_task);
			__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshControlThreadLoop);
			//__itt_metadata_add(pDomainControlThread, __itt_null, pshControlThreadLoop, __itt_metadata_s32, 1, (void*)&LoopIndex);

			__itt_metadata_add(pDomainControlThread, __itt_null, pSH2, __itt_metadata_s32, 1, (void*)&LoopIndex);
			LoopIndex++;

			// Dispatch the message to each module, highest priority first
			// Sensor module is called first, so it's sensor data it gets scaled
			// to appropriate units.  Processed data is returned in SensorData.
			//
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshSensorModule);
				pSensorModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshCameraModule);
				pCameraModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshKinectModule);
				pKinectModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshSystemModule);
				pSystemModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshCollisionModule);
				pCollisionModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshAvoidModule);
				pAvoidObjectModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshUserCmdModule);
				pUserCmdModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshWayPointModule);
				pWayPointNavModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshGridNavModule);
				pGridNavModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}
			{
				__itt_task_begin(pDomainControlThread, __itt_null, __itt_null, pshBehaviorModule);
				pBehaviorModule->ProcessMessage( UserMessage, msg.wParam, msg.lParam );
				__itt_task_end(pDomainControlThread);
			}

			if( !g_bCmdRecognized )
			{
				ROBOT_LOG( TRUE, _T( "Thread.cpp: ERROR unknown message (probably new) Msg = 0x%08lX, wParam = 0x%08lX, lParam = 0x%08lX\n"), 
					UserMessage, msg.wParam, msg.lParam )
			}
			else
			{
				// DriveControlModule arbitrates requests from other modules and issues
				// the command to the PIC.  Send motor commands after each status update
				if( WM_ROBOT_SENSOR_STATUS_READY == (msg.message))
				{
					///TAL_SCOPED_TASK_NAMED("Execute Drive Control");
	//				ROBOT_LOG( TRUE,  "DBG: Thread Execute Cmd %02X\n",(msg.message) )
					// 
					pDriveControlModule->ExecuteCommand();
				}
			}
			__itt_task_end(pDomainControlThread);  // pshControlThreadLoop
		}
		Sleep(0); // force thread switch after each command
	}

	// We received a WM_ROBOT_THREAD_EXIT_CMD or WM_QUIT message, exit
	ROBOT_LOG( TRUE,  "ControlThread exiting.\n" )
	// Release modules (in reverse order)
	SAFE_DELETE( pBehaviorModule );
	SAFE_DELETE( pGridNavModule );
	SAFE_DELETE( pWayPointNavModule );
	SAFE_DELETE( pUserCmdModule );
	SAFE_DELETE( pAvoidObjectModule );
	SAFE_DELETE( pCollisionModule );
	SAFE_DELETE( pSystemModule );
	SAFE_DELETE( pCameraModule );
	SAFE_DELETE( pKinectModule );
	SAFE_DELETE( pSensorModule );
	SAFE_DELETE( pDriveControlModule );

	return 0;
}

//-----------------------------------------------------------------------------
// Name: SoundThreadProc
// This thread plays sounds to avoid the latency of startup for ASYNC sounds
// messages are sent to g_dwSoundThreadId
//-----------------------------------------------------------------------------
DWORD WINAPI SoundThreadProc( LPVOID NotUsed )
{
	IGNORE_UNUSED_PARAM (NotUsed);
	__itt_thread_set_name( "Sound Thread" );

	MSG	msg;
    HRESULT hr = E_FAIL;

	Sleep(5000); // allow other threads to start first


// TODO!  Does this thread still do speach, or just music?  Is it needed?  Can we play iTunes?

	////////////////////////////////////////////
	////////////////////////////////////////////
	// WHEN NEXT LINE IS UNCOMMENTED, ALL VOICE DISABLED!!!
	//	return 0;
	////////////////////////////////////////////
	////////////////////////////////////////////


	// for Wav playback:
	DWORD dwOptions = SND_ASYNC | SND_FILENAME | SND_NODEFAULT ;	// SND_NOWAIT

	///////////////////////////////////////////////////////////////////////////////////////
	// Initialize Speech Engine For Text to speech:
/****
	ISpVoice * pVoice = NULL;


//	IStream   *pStream, 
//	DWORD      dwFlags, 
//	ULONG     *pulStreamNumber


	if (FAILED(::CoInitialize(NULL)))
	{
		pVoice = NULL;
		ASSERT(0);
		return 0;
	}

	hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
	if( FAILED( hr ) )
	{
		ROBOT_LOG( TRUE,  "ERROR! Could not initialize Voice!\n" )
		pVoice->Release();
		pVoice = NULL;
		::CoUninitialize();
		ASSERT(0);
		return 0;
	}

	// Disable Speech recogntion while talking
	PostMessage( g_RobotSetupViewHWND, WM_ROBOT_ENABLE_RECO, FALSE, 0 );
	Sleep(10);	// Allow time to turn off reco engine

	hr = pVoice->Speak(L"Initializing System", 0, NULL);
	if( FAILED( hr ) )
	{
		ROBOT_LOG( TRUE,  "ERROR Speaking!\n" )
	}
	PostMessage( g_RobotSetupViewHWND, WM_ROBOT_ENABLE_RECO, TRUE, 0 );
***/
	// test wave file
/*** TODO
   hr = cpSpStream->BindToFile(WAVE_FILENAME, SPFM_OPEN_READONLY, NULL, NULL, NULL);
	hr = SpeakStream(
	pStream, 
	dwFlags, 
	pulStreamNumber );
***/

	///////////////////////////////////////////////////////////////////////////////////////
	// Loop forever


	while( 0 != GetMessage( &msg, NULL, WM_ROBOT_MESSAGE_BASE, (WM_ROBOT_MAX_ROBOT_MESSAGE) ) )
		// OK to filter out Windows messages, since we don't use WM_QUIT
	{
		if( WM_ROBOT_THREAD_EXIT_CMD == (msg.message) )
		{
			break;	// Quit command received!
		}

		// 

		// Get message from the message queue and process it.
		if( (WM_ROBOT_PLAY_MUSIC) == msg.message )
		{
			// Start playing Music
			// for Wav files, options are: 	DWORD dwOptions = SND_SYNC | SND_FILENAME | SND_NODEFAULT ;	// SND_NOWAIT

			LPCTSTR pFileName;
			switch( msg.wParam )  // wParam is the number of the music requested
			{
				case MUSIC_STOP:
				{
					pFileName = MUSIC_SILENCE_STR; // HACK - play silent wav to kill prior.
					break;
				}
				case MUSIC_MR_ROBOTO:
				{
					pFileName = MUSIC_MR_ROBOTO_STR;
					break;
				}
				case MUSIC_HAWAII50:
				{
					pFileName = MUSIC_HAWAII50_STR;
					break;
				}
				default:
				{
					ROBOT_LOG( TRUE,  ">>>> ERROR: Unmapped Music Request!\n" )
					continue;
				}
			}
			
			// Disable Speech recogntion while playing music - TODO - change this to allow STOP voice command!  Instead, just stop text from getting to the AI...
			PostMessage( g_RobotSetupViewHWND, WM_ROBOT_ENABLE_RECO, FALSE, FALSE );
			Sleep(10);	// Allow time to turn off reco engine

			BOOL bSucess = PlaySound(
				pFileName,		// Filename  
				NULL,			// No device Handle needed   
				dwOptions);		// Control Flags
			if( !bSucess )
			{
				ROBOT_LOG( TRUE,  ">>>> ERROR: Could not play sound %s!\n", pFileName)
			}
			PostMessage( g_RobotSetupViewHWND, WM_ROBOT_ENABLE_RECO, TRUE, FALSE );
		}
		else if( (WM_ROBOT_PLAY_SOUND) == msg.message )
		{
/***
			// Play "sound bytes" from Wav files

  // TODO-LOKI!!!

			// Disable Speech recogntion while talking
			PostMessage( g_RobotSetupViewHWND, WM_ROBOT_ENABLE_RECO, FALSE, FALSE );
			Sleep(10);	// Allow time to turn off reco engine

			LPCTSTR pFileName;

			switch( msg.wParam )  // wParam is the Sound requested
			{
				case SPEAK_TEXT_FROM_BUFFER:
				{
					ROBOT_ASSERT(0);

				// Convert from string to wide character format needed by Speak.
					// I'm sure there is somen conversion function to do this in Microsoft ATL land, but this works...
					char WStrBuf[WIDE_BULK_DATA_SIZE];
					char * pWStrBuf = WStrBuf;
					memset(WStrBuf, 0, WIDE_BULK_DATA_SIZE);
					int StrLen = g_ClientTextToSend.GetLength();
					if( StrLen > BULK_DATA_SIZE ) StrLen = BULK_DATA_SIZE;
					for(int i = 0; i < g_ClientTextToSend.GetLength(); i++)
					{
						*pWStrBuf = g_ClientTextToSend[i];
						pWStrBuf+=2;
					}
					hr = pVoice->Speak((const unsigned short *)WStrBuf, 0, NULL);
					g_ClientTextToSend.Empty();	// Reset the buffer to empty				

				}
				break;
				case SOUND_PIC_CONNECTED:
				{
					hr = pVoice->Speak(L"PIC Connected!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\PicConnectedSound.wav");
				}
				break;
				case SOUND_PIC_VERSION_ERROR:
				{
					Beep(1000,500);
					Beep(200,500);
					Beep(1000,500);
					Beep(200,500);
					hr = pVoice->Speak(L"WARNING! WARNING! PIC Version Error!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\PicConnectedSound.wav");
				}
				break;				
				case SOUND_SIMULATION_MODE:
				{
					hr = pVoice->Speak(L"Simulation Mode!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\PicSimulationMode.wav");
				}
				break;
				case SOUND_COLLISION:
				{
					hr = pVoice->Speak(L"Ouch", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\CollisionSound.wav");
				}
				break;
				case SOUND_AVOID:
				{
					hr = pVoice->Speak(L"Avoiding!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\AvoidSound.wav");
				}
				break;
				case SOUND_TRAPPED:
				{
					hr = pVoice->Speak(L"Help, I am trapped!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\TrappedSound.wav");
				}
				break;
				case SOUND_EXECUTING_PATH:
				{
					hr = pVoice->Speak(L"Executing Path!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\ExecutingPath.wav");
				}
				break;
				case SOUND_WAYPOINT_REACHED:
				{
					hr = pVoice->Speak(L"Waypoint Reached!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\WayPointReached.wav");
				}
				break;
				case SOUND_LOOKING_FOR_LANDMARK:
				{
					hr = pVoice->Speak(L"Looking for Landmark!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\LookingForLandmark.wav");
				}
				break;
				case SOUND_HEADING_FOR_OBJECT:
				{
					hr = pVoice->Speak(L"Heading For Object!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\HeadingForObject.wav");
				}
				case SOUND_DARTH_VADER:
				{
					ROBOT_LOG( TRUE,  "===============  Doing Darth!=================\n" )

					hr = pVoice->Speak(L"Would you like to hear my impression of Darth Vaider?", 0, NULL);
					Sleep(1000);

					// Position Camera slightly to left

					SendCommand( WM_ROBOT_CAMERA_PAN_TILT_SPEED_CMD, (DWORD)16, (DWORD)16 );	// Fairly fast
					SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)(CAMERA_PAN_TENTHDEGREES_MAX_LEFT/2), 0 );
					SendCommand( WM_ROBOT_CAMERA_TILT_ABS_CMD, (DWORD)(CAMERA_PAN_CENTER), 0 );

					//Sleep(500);
					hr = pVoice->Speak(L"OK, here goes,", 0, NULL);

					Sleep(1000);

					// Slow Pan camera while talking like Darth
					SendCommand( WM_ROBOT_CAMERA_PAN_TILT_SPEED_CMD, (DWORD)6, (DWORD)6 );	// Slow pan
					SendCommand( WM_ROBOT_LIGHT_POWER_CMD, 0, (DWORD)TRUE );	// Lights On
					Sleep(300);
					SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)(CAMERA_PAN_TENTHDEGREES_MAX_RIGHT/2), 0 );

					Sleep(300);
					pFileName = _T("..\\RobotData\\RobotSounds\\vader_bidding.wav");
					BOOL bSucess = PlaySound(
						pFileName,		// Filename  
						NULL,			// No device Handle needed   
						dwOptions);		// Control Flags
					if( !bSucess )
					{
						ROBOT_LOG( TRUE,  ">>>> ERROR: Could not play sound vader_bidding.wav!\n" )
					}
					Sleep(500);
					SendCommand( WM_ROBOT_LIGHT_POWER_CMD, 0, (DWORD)FALSE );	// Lights Off
					Sleep(1000);
					SendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_ABS_CENTER, 7 );
					Sleep(2000);
					hr = pVoice->Speak(L"So, Did you like my Darth Vaider impression?", 0, NULL);
					Sleep(1000);
					hr = pVoice->Speak(L"Good.. Well, that's all I can do so far.", 0, NULL);
					SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)(CAMERA_PAN_CENTER+20), 0 );
					hr = pVoice->Speak(L"Dave, Do really need to make me smarter when you get time! ", 0, NULL);
					hr = pVoice->Speak(L"Perhaps more memory would help?", 0, NULL);
					Sleep(500);
					SendCommand( WM_ROBOT_CAMERA_PAN_ABS_CMD, (DWORD)(CAMERA_PAN_CENTER-20), 0 );
					hr = pVoice->Speak(L"Well, I am tired, I think I will power down for a while", 0, NULL);
					SendCommand( WM_ROBOT_USER_CAMERA_PAN_CMD, (DWORD)CAMERA_PAN_ABS_CENTER, 7 );
					SendCommand( WM_ROBOT_CAMERA_TILT_ABS_CMD, (DWORD)(CAMERA_TILT_CENTER+20), 0 );
					SendCommand( WM_ROBOT_SET_LED_EYES_CMD, (DWORD)FALSE, (DWORD)FALSE );	// Eyes Off

					ROBOT_LOG( TRUE,  "===============  Darth Done!=================\n" )
				}
				break;
				default:
				{
					hr = pVoice->Speak(L"Unknown Speak Command!", 0, NULL);
					//pFileName = _T("..\\RobotData\\RobotSounds\\UndefinedSound.wav");
					ROBOT_LOG( TRUE,  ">>>> ERROR: Unknown sound!\n" )
				}
			}	// end switch
			// ReEnable Speech recogntion when done
			PostMessage( g_RobotSetupViewHWND, WM_ROBOT_ENABLE_RECO, TRUE, FALSE );
***/
		}
		else
		{
			// Ignore any messages not intended for this function
			UINT temp = WM_ROBOT_MESSAGE_BASE;	// for debug
			ROBOT_LOG( TRUE,  "SOUND THREAD: message out of Range, WM_ROBOT_MESSAGE_BASE = 0x%08X, Msg = 0x%08lX, wParam = 0x%08lX, lParam = 0x%08lX\n", 
				temp, msg.message, msg.wParam, msg.lParam )
			continue;
		}
		Sleep(0); // force thread switch after each command
	}

	// We received a WM_ROBOT_THREAD_EXIT_CMD or WM_QUIT message, exit
	ROBOT_LOG( TRUE,  "SoundThread exiting.\n" )

	// Release voice resources
/*	if( pVoice != NULL )
	{
		pVoice->Release();
		pVoice = NULL;
		::CoUninitialize();
	}
	*/
	return 0;
}


//-----------------------------------------------------------------------------
// Name: AIThreadProc
// This thread processes AI input and generates responses
//-----------------------------------------------------------------------------

#if ENABLE_ROBOT_AI == 1

DWORD WINAPI AIThreadProc( LPVOID NotUsed )
{
	__itt_thread_set_name( "AI Thread" );

	MSG	msg;
    HRESULT hr = E_FAIL;
	CString input, ResponseString, MsgString;


	// Give a chance for the rest of the Robot threads to start up
	Sleep(2000);
	Sleep(2000);
	CRobotAI *pRobotAI = new CRobotAI;

	///////////////////////////////////////////////////////////////////////////////////////
	// Initialize AI Engine
	ROBOT_LOG( TRUE,  "\n=============================================================\n" )
	ROBOT_LOG( TRUE,  "              INITIALIZING AI\n" )
	ROBOT_LOG( TRUE,  "=============================================================\n\n" )

	if( !pRobotAI->Initialize() )
	{
		ROBOT_DISPLAY( TRUE, "Error Initializing IA Engine.  AIThreadProc Exiting!")
		return 0;
	}
	

//	pRobotAI->Test();

	///////////////////////////////////////////////////////////////////////////////////////
	// Loop forever


	while( 0 != GetMessage( &msg, NULL, WM_ROBOT_MESSAGE_BASE, (WM_ROBOT_MAX_ROBOT_MESSAGE) ) )
		// OK to filter out Windows messages, since we don't use WM_QUIT
	{
		{
			///TAL_SCOPED_TASK_NAMED("AI MSG Loop");

			if( WM_ROBOT_THREAD_EXIT_CMD == msg.message )
			{
				break;	// Quit command received!
			}

			if( (msg.message < WM_ROBOT_MESSAGE_BASE) || 
				((msg.message) >= WM_GUI_MESSAGES) ||		// Ignore messages intended for the GUI
				((msg.message) > WM_ROBOT_MAX_ROBOT_MESSAGE) )	// Or messages out of range
			{
				// Ignore any messages not intended for this thread
				continue;
			}
			// 


			// Get message from the message queue and process it.
			if( (WM_ROBOT_TEXT_TO_AI) == msg.message )
			{
				// Text for the AI to process

	//				int StrLen = g_ClientTextToSend.GetLength();
	//				if( StrLen > BULK_DATA_SIZE ) StrLen = BULK_DATA_SIZE;
	//				g_ClientTextToSend[i];
	//					hr = pVoice->Speak((const unsigned short *)WStrBuf, 0, NULL);
					CString input = g_ClientTextToSend;	// Get text from global buffer
					g_ClientTextToSend.Empty();			// Reset the buffer to empty, so next text can come in
					
					// Talk to the AI
					pRobotAI->GetAIResponse( input, ResponseString );

					// Trap Special Responses from the AI!
					// All start with ##
					if( ResponseString.Left(2) == "##" )
					{
						ROBOT_LOG( TRUE,  "Special Command String found: %s\n", ResponseString )
						// TODO - Process commands here!!
						// TODO TODO TODO
						ResponseString = "Acknowledged";	// Acknowledge the command
					}

					// Copy to global string for playing
					g_ClientTextToSend = _T(ResponseString);	
					//SendCommand( WM_ROBOT_TEXT_MESSAGE_TO_SERVER, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER );
					PostMessage( g_RobotSetupViewHWND, WM_ROBOT_SPEAK_TEXT, SPEAK_TEXT_FROM_BUFFER, 0 );

					MsgString.Format( "Loki Says: %s", ResponseString );
					ROBOT_DISPLAY( TRUE,  (LPCTSTR)MsgString )

			}
			else
			{
				// Ignore any messages not intended for this function
				UINT temp = WM_ROBOT_MESSAGE_BASE;	// for debug
				ROBOT_LOG( TRUE,  "AI THREAD: message out of Range, WM_ROBOT_MESSAGE_BASE = %08X, Msg = 0x%08lX, wParam = %08lX, lParam = %08lX\n", 
					temp, msg.message, msg.wParam, msg.lParam )
				continue;
			}
		}
		Sleep(0); // force thread switch after each command
	}

	// We received a WM_ROBOT_THREAD_EXIT_CMD or WM_QUIT message, exit
	ROBOT_LOG( TRUE,  "AIThread exiting.\n" )

	// Release AI resources
	return 0;

}	// End of AI Thread

#endif //if (ENABLE_AI == 1)
#endif	// ROBOT_SERVER