// RobotType.h
// Top level selection of what kind of robot this is!
// This file included by Microcontroller code, and is shared by both compilers
// Make sure syntax works with all compilers needed

#ifndef __ROBOT_SELECTION_H__
#define __ROBOT_SELECTION_H__

#include "ClientOrServer.h"

// Hardware Configuration for various types of robots
// Robot type
#define LOKI							0
#define TURTLE							1
#define CARBOT							2

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CONFIGURE HARDWARE HERE!

#define ROBOT_TYPE						LOKI
//#define ROBOT_TYPE					TURTLE
//#define ROBOT_TYPE						CARBOT

// TEMP KLUDGE
#define KINECT_SDK_TYPE_MICROSOFT		0

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Motor control types
#define ER1_MOTOR_CONTROL				0
#define POLOLU_TREX_MOTOR_CONTROL		1
#define SERVO_MOTOR_CONTROL				2
#define IROBOT_MOTOR_CONTROL			3

// Servo control types
#define PIC_SERVO_CONTROL				4
#define EXTERN_SERVO_CONTROL			5
#define DYNA_SERVO_CONTROL				6
#define SONY_SERIAL_CAMERA				7

// Sensor Configuraitons (varies by robot)
#define SENSOR_CONFIG_LOKI				0
#define SENSOR_CONFIG_CARBOT			1
#define SENSOR_CONFIG_TURTLE			2
//#define SENSOR_CONFIG_HELMET_BOT		3
//#define SENSOR_CONFIG_BUG_BOT			4


#if ROBOT_TYPE == CARBOT
	#define PIC_CODE_VERSION			22
	#define MOTOR_CONTROL_TYPE			EXTERN_SERVO_CONTROL
	#define SERVO_CONTROL_TYPE			EXTERN_SERVO_CONTROL
	#define CAMERA_CONTROL_TYPE			EXTERN_SERVO_CONTROL
	#define OTHER_SERVO_CONTROL_TYPE	EXTERN_SERVO_CONTROL
	#define SENSOR_CONFIG_TYPE			SENSOR_CONFIG_CARBOT
	#define ROBOT_DATA_PATH	"C:\\Dev\\_Robot\\SeekerData"
	#define ROBOT_COMMON_PATH L"C:\\Dev\\_Robot\\Common"

#elif ROBOT_TYPE == LOKI
	#define PIC_CODE_VERSION			30
	#define MOTOR_CONTROL_TYPE			POLOLU_TREX_MOTOR_CONTROL
	#define CAMERA_CONTROL_TYPE			DYNA_SERVO_CONTROL
	#define SERVO_CONTROL_TYPE			PIC_SERVO_CONTROL
	#define OTHER_SERVO_CONTROL_TYPE	DYNA_SERVO_CONTROL
	#define SENSOR_CONFIG_TYPE			SENSOR_CONFIG_LOKI
	#define ROBOT_DATA_PATH	"C:\\Dev\\_Robot\\LokiData"
	#define ROBOT_COMMON_PATH L"C:\\Dev\\_Robot\\Common"

#elif ROBOT_TYPE == TURTLE
	#define PIC_CODE_VERSION			0	// No PIC used
	#define MOTOR_CONTROL_TYPE			IROBOT_MOTOR_CONTROL
	#define CAMERA_CONTROL_TYPE			DYNA_SERVO_CONTROL
	#define SERVO_CONTROL_TYPE			DYNA_SERVO_CONTROL
	#define OTHER_SERVO_CONTROL_TYPE	DYNA_SERVO_CONTROL
	#define SENSOR_CONFIG_TYPE			SENSOR_CONFIG_TURTLE
	#define ROBOT_DATA_PATH	"C:\\Dev\\_Robot\\TurtleData"
	#define ROBOT_COMMON_PATH L"C:\\Dev\\_Robot\\Common"

#else
	#error BAD ROBOT_TYPE
#endif


#endif // __ROBOT_SELECTION_H__
