/****************************************************************************
*
*  RobotPic.c
*
*  Pic Timer Interrupt-driven PIC Robot control code for Robot
*
*  This version for use with ER1 controller (most control functions removed)!
*
*  Written the CCSC C Compiler 2003-2006 Dave Shinsel
*
****************************************************************************/

////////////////////////////////////////////
// For Debugging, Turn on debug code here

//#define DEBUG_SCOPE_20MS_TIMING	// Show where we are in the 20ms cycle
//#define DEBUG_TESTS_ENABLED	// WARNING! Disables normal operation!
//#define DBG_PRINT_ENABLED     // Turns on dbg print

// Turn this on to enable use of Compass!
#define COMPASS_ATTACHED


////////////////////////////////////////////

// Other constants
//#define ULTRASONIC_SRF04	 // or ULTRASONIC_SRF04_SRF10_I2C
//#define RADAR_SCAN_ENABLED		// Disabled if not needed, to save code space

//***************************************************************************
// Include the routines that make the magic happen :-)

#include <16F877a.H>	// Setup for correct part
//#include <string.h>
#define WORD int16	// Must define here for compatability with PC compiler
#define __PIC_TARGET
#include "../Common/PicCommands.h"
#include "RobotPic.h"

// I2C **************************************************************************
//#ifndef EEPROM_SDA
//#define EEPROM_SDA  PIN_C4
//#define EEPROM_SCL  PIN_C3
//#endif
#use i2c(master, sda=PIN_C4, scl=PIN_C3, FORCE_HW)
// TODO: Try these options: SLOW,FAST, FORCE_HW
// I2C **************************************************************************

#include "RobotPic_Sub.c"
#include "RobotPic_TimerISR.c"


#ifdef DEBUG_TESTS_ENABLED
	#include "DebugTests.c"
#endif


/*************** TIMING NOTES ********************
 Sharp 5-beam sensor : 20ms per beam x 5 = 100ms
 MaxSonar UltraSonic sensor: 50ms each x 3 = 150ms
 Sharp Long Range IR: 55ms per reading
**************************************************/

#define HEARTBEAT_COUNT_TIME  50 // heartbeat every 2 seconds

//***************************************************************************
//  Main
//***************************************************************************

void main( void )
{

//---------------------------------------------------------------------------
// Initialization
//   int TestPort;

int   HeartbeatCount;
int	  BlinkTime;

	// Initialize variables and servo positions, setup A/D, etc.
	Initialize();

#ifdef DEBUG_TESTS_ENABLED
//	DebugTests();	// WARNING! Disables normal operation!
	// Note, this must follow Initialize, so A/D gets setup!
#endif


	// Setup 1ms timer interrupt ( assumes 20MHz clock )
	g20msTimer = 0;			// Servo timer
	set_timer0( INTR_SET_TIME_1_MS );		// RTCC = TIMER_0
	setup_timer_0( RTCC_INTERNAL | RTCC_DIV_128 );
	setup_timer_1(T1_INTERNAL | T1_DIV_BY_8); // Timer for Ultrasonic Echo Timing


	// Ok, enable interrups and start!
	enable_interrupts( INT_RTCC );	// Timer 0 Interrupt enable
	enable_interrupts( INT_RDA );	// RS-232 Receive Data Interrupt enable
//	enable_interrupts( INT_TBE );	// DONT ENABLE THIS!  Handled in RobotPic_Sub.c!!!
	enable_interrupts( GLOBAL );	// Ok, turn them on!

/* This does not work - Need to check ports B3,6,7 are all free and short lines!
   #use rs232(DEBUGGER)
   printf("Init Complete\n");
   #use rs232(RS232_CONFIG)
*/

   //---------------------------------------------------------------------------
   // Control Loop
   //
   // The Loop runs like this. It syncs with 20ms Timer loop, starting
   // at 3 ms when not much else going on.  Call this Start.
   // Start+0 ms: Read Sensors, every 20ms.
   // Start+10 ms: Read Compass, every 40ms
   //
	// Green LED shows heartbeat, Yellow shows Serial activity
	output_low( PIN_LED_G );
	//LedState_Green = 0;

   //   TestPort = 1;
   HeartbeatCount = HEARTBEAT_COUNT_TIME; // count down from here

	while ( TRUE )	// Loop Forever
	{
		// No LED?  may stop if I2C Compass not connected!

StartOfLoop:	// Useful for breaking out of any nested code and starting this loop over

/*
   #use rs232(DEBUGGER)
   printf(".");
   #use rs232(RS232_CONFIG)
*/
	//-------------------------------------------------------------------
		// Check for command from controlling computer (iPAQ or PC)
 		if( gSerialState == SIO_BAD_TERM_CHAR )
		{
			// Error! Dropped a byte during serial transfer!
			// Ignore the command, and return an error!
			gStatus_LastError = HW_SERIAL_CMD_CHECKSUM_ERROR;
			output_low( PIN_LED_R );	// Turn On ERROR LED
			//LedState_Red = 0;

			// Force an immediate report of the error
			sprintf(gResponseMsg, "AK1:ERR CheckSum!");
			SendResponse(0);
			gSerialState = SIO_WAIT_FOR_SYNC_0;	// Start over looking for a new command
		}

		if( gSerialCmdReady )
		{
			// Command received
			//printf("COMMAND RECEIVED: %d", gPicCmd.Cmd );
			output_low( PIN_LED_Y );	// LED on at start of command, off upon response sent
			//LedState_Yellow = 0;
			// Indicate we have received at least one command from the PC
			// So It's OK to start sending status data, etc.

			// Any command from the PC resets the Watchdog timer
			//gWatchDogTimer = WATCHDOG_TIME;
			//gStatus_StatusFlags &= (0xFF^HW_STATUS_WATCHDOG_EXPIRED); // Clear Flag

			// Process the command received
			ProcessSerialCmd();
			gSerialCmdReady = FALSE;	// Allow processing next command
         // HeartbeatCount = HEARTBEAT_COUNT_TIME; // no need for Heartbeat if cmds being responded to
		}	// If Cmd received

#ifndef ULTRASONIC_SRF04	// if using SRF10 instead of SRF04
//gUS_Range[0] = 1; // DEBUG!!!

		if( IDLE == gI2C_USState )
		{
			// Tell I2C US sensor it can ping
			I2C_USPingStart();
			gI2C_USState = I2C_US_PINGING;
			gI2C_USTimer = 65;	//ms to wait before reading result.  The SRF10 needs 65MS by default! (6 meter range!)
			//gSensorTimingState = SENSOR_US_DISABLED;
//         gUS_Range[0] = 2; // DEBUG!!!
		}
		else if( (I2C_US_PINGING == gI2C_USState) && (0 == gI2C_USTimer) )
		{
			// Read the result from the US sensor
			I2C_USRead();
			gI2C_USState = IDLE;
			// TODO - tell other US sensors they can read now
			// Not used for now.  SRF04 -or- SRF10 only. See if gI2C_USState and gSensorTimingState can be combined
			//gTimedSensorNumber = 0;
			//gSensorTimingState = IDLE;
//         gUS_Range[0] = 4; // DEBUG!!!
		}
#endif
		if( gTimeToReadSensors )	// Sync up with 20ms timer
 		{
			gTimeToReadSensors = FALSE;		// reset until next time
         
         HeartbeatCount--;
         if( (0 != gSerialState) || (0 == HeartbeatCount) )
         {
            SendHeartbeat();
            HeartbeatCount = HEARTBEAT_COUNT_TIME; // reset
         }          

         

			// Read most sensors every 20ms

			// Read IR proximity sensor switches
			ReadIRBumpers();

			// Read HW switches
			ReadHWBumpers();

			// Read Right Arm HW and IR bumper sensors
//			ReadRightArmBumpers();

			// Read Left Arm HW and IR bumper sensors
			ReadLeftArmBumpers();

			// Read Analog IR sensors and Battery
			ReadA2DSensors();


         BlinkTime--;
         if( 0 == BlinkTime )
         {
			


				// Turn LED EYES on or off ( in 20ms steps )
				if( LED_EYES_OPEN == gLedEyeMode )
				{
					// Open Eye in stages ( start shift at 5, count down to 0 )
					SetLedEyes( (0xFC << gEyeState) );	// SetLedEyes will handle the bottom 3 bits
					if( 0 == gEyeState )
					{
						gLedEyeMode = LED_EYES_ON;	// fully on now
					}
					else
					{
						gEyeState--;
					}
				}
				else if( LED_EYES_CLOSE == gLedEyeMode )
				{
					// Close Eye in stages ( start shift at 0, count up to 5 )
					SetLedEyes( (0xFC << gEyeState) );	// SetLedEyes will handle the bottom 3 bits
					if( 6 == gEyeState )
					{
						gLedEyeMode = LED_EYES_OFF;	// fully off now
					}
					else
					{
						gEyeState++;
					}
				}
				else if( LED_EYES_BLINK == gLedEyeMode )
				{
					// Close Eye in stages ( start shift at 0, count up to 5 ), and then open again
					SetLedEyes( (0xFC << gEyeState) );	// SetLedEyes will handle the bottom 3 bits
					if( 6 == gEyeState )
					{
						gLedEyeMode = LED_EYES_OPEN;	// fully off now, turn back on in stages
					}
					else
					{
						gEyeState++;
					}
				}
		 }

			// READ COMPASS
			if( gTimeToReadCompass )	// Every 40 ms
			{
				// Read the compass no more than once every 40ms
				gTimeToReadCompass = FALSE;	// skip next time
            
         	output_high( PIN_LED_R ); // Off - blink on (every 40ms) if we are receive an RX char
            

			// WARNING!  Will HANG PIC if Compass not connected. Also NEED I2C JUMPERS installed!
            #ifdef COMPASS_ATTACHED
				  ReadCompass();				// Get latest Compass info from the I2C device
            #endif
				output_low( PIN_LED_G );	// Green LED shows heartbeat, Yellow shows Serial activity
				//LedState_Green = 0;

				// DEBUG - LED EYES
			//	SetLedEyes( 0x00 );	// LED Eyes On


				// DEBUG - I2C test!!!
				//WriteI2C_Port(I2C_PCF8574_1_WRITE, 0xFF);  // Green LED is bit 7
			}
			else
			{
				// Flip-flop
				gTimeToReadCompass = TRUE;
				output_high( PIN_LED_G );	// Green LED shows heartbeat, Yellow shows Serial activity
				//LedState_Green = 1;

				// Read Thermal Array sensor
				ReadThermalArray();

				//WriteI2C_Port(I2C_PCF8574_1_WRITE, 0xBF); // Bring bit 6 low - Green LED
				//WriteI2C_Port(I2C_PCF8574_1_WRITE, 0x7F); // Bring bit 7 low - Red LED
			}
		}

		delay_ms(5);

	}	// Loop Forever

}	// main


