/****************************************************************************
*
* RobotPic.h
*
*  Timer Interrupt-driven PIC Robot control code for CarBot Robot
*
*  Written the CCSC C Compiler 2003-2006 Dave Shinsel
*
****************************************************************************/


// Setup for 16F series part
#device *=16
#fuses HS,NOWDT,NOPROTECT
////////////////////////////////////////////////////////////////
// You can either use the ICD (In Circuit Debugger) OR (not both)
// serial upload (requires bootloader flashed into the chip)
// Only enable this line if DEBUGGING!:
//#device ICD=TRUE

// Configure for 20MHz crystal, 16F877 PIC.  Change this if different
#use delay( clock=20000000 )

//OR protect bootloader code for the 8k 16F876/7
//#org 0x1F00, 0x1FFF void loader16F876(void) {}
////////////////////////////////////////////////////////////////


//#use rs232( baud=9600, xmit=PIN_C6, rcv=PIN_C7, ERRORS )
// Cause the compiler to keep receive errors in the variable RS232_ERRORS
// and to reset errors when they occur.

#define RS232_CONFIG baud=9600, xmit=PIN_C6, rcv=PIN_C7, ERRORS
#use rs232( RS232_CONFIG )

/*
#ifdef DBG_PRINT_ENABLED   // Print to ICD Console
   #define DBG_PRINT(mytext) #use rs232(DEBUGGER) printf(mytext); #use rs232(RS232_CONFIG)
#else
   #define DBG_PRINT(mytext) ()
#endif
*/

/*
Debug example (macros don;t work right)
   #use rs232(DEBUGGER)
   printf("mytext");
   #use rs232(RS232_CONFIG)
*/

// Debug Definitions
#define DEBUG_VAR8(x)	printf("DEBUG %s = %u \(%02X Hex\) \r\n",#x,x,x)


//***************************************************************************
// I2C Address Mappings

// Compass
#define I2C_COMPASS_WRITE	0xC0
#define I2C_COMPASS_READ	0xC1


// SRF10 UltraSonic Ranger
#define I2C_US_SONAR_WRITE	0xE0
#define I2C_US_SONAR_READ	0xE1



// First PCF8574A I/O expander chip I2C Address: 0x4E (On Pic Board)
// (all 3 address lines pulled high)
#define   I2C_PCF8574_1_WRITE  0b01001110	// 1110 = Address + R/W bit, 0=Write (0x4E)
#define   I2C_PCF8574_1_READ   0b01001111	// 1111 = Address + R/W bit, 1=Read  (0x4F)
//                                   ^^^	selectable address bits

// Second PCF8574A I/O expander chip I2C Address: 0x4C (On Loki Expansion Board)
// (Address line 3 high, 2 high, 1 low)
#define   I2C_PCF8574_2_WRITE  0b01001100	// 1100 = Address + R/W bit, 0=Write (0x4C)
#define   I2C_PCF8574_2_READ   0b01001101	// 1101 = Address + R/W bit, 1=Read  (0x4D)
//                                   ^^^	selectable address bits

// Third PCF8574A I/O expander chip I2C Address: 0x44 (In Loki's Head)
// (Address line 3 low, 2 high, 1 low)
#define   I2C_PCF8574_3_WRITE  0b01000100	// 1100 = Address + R/W bit, 0=Write (0x44) - TODO change 4A to allow I2C-IT!
#define   I2C_PCF8574_3_READ   0b01000101	// 1101 = Address + R/W bit, 1=Read  (0x45)
//                                   ^^^	selectable address bits

// Fourth PCF8574A I/O expander chip I2C Address: 0x42 (In Loki's Left Arm)
// (Address line 3 low, 2 low, 1 high)
#define   I2C_PCF8574_4_WRITE  0b01000010	// 1100 = Address + R/W bit, 0=Write (0x42)
#define   I2C_PCF8574_4_READ   0b01000011	// 1101 = Address + R/W bit, 1=Read  (0x43)
//                                   ^^^	selectable address bits

// Fifth PCF8574A I/O expander chip I2C Address: 0x46 (In Loki's Right Arm)
// (Address line 3 low, 2 high, 1 high)
#define   I2C_PCF8574_5_WRITE  0b01000110	// 1100 = Address + R/W bit, 0=Write (0x46)
#define   I2C_PCF8574_5_READ   0b01000111	// 1101 = Address + R/W bit, 1=Read  (0x47)
//                                   ^^^	selectable address bits

// MAX127 A2D chip.  MAX127 A2D use 0101 + A2,A1,A0 + R/W - I2C Address: 0x50 (I/O Expansion Board)
#define I2C_A2D_ADDRESS_W	0b01010000	// 0101, dip switch, R/W bit, 1=Read, 0=Write
#define I2C_A2D_ADDRESS_R	0b01010001
//				   Starts at 0x50 ^^^ selectable address bits (000 for Loki)


// I2C A/D Channels
// ControlByte = 0b1xxx0000, where xxx is Channel.  See Page 10 of MAX127 Spec!
	// bit 7 = Conrol Byte Flag. Must be 1
	// bit 6,5,4 = Channel select
	// bit 3,2 = voltage range; 0,0 = set to 0-5v
	// bit 1,0 = power down modes; 0,0 = set to always on
#define IC2_A2D_CHANNEL_0	0b10000000
#define IC2_A2D_CHANNEL_1	0b10010000
#define IC2_A2D_CHANNEL_2	0b10100000
#define IC2_A2D_CHANNEL_3	0b10110000
#define IC2_A2D_CHANNEL_4	0b11000000
#define IC2_A2D_CHANNEL_5	0b11010000
#define IC2_A2D_CHANNEL_6	0b11100000
#define IC2_A2D_CHANNEL_7	0b11110000
//                             ^^^ Channel Select


// I2C-IT IR Rangers
// I2C-IT Address range from 0x20 to 0x2C or 0x40 to 0x4C (every 4 bits)
// I2C Write = base address.  I2C Read = base address + 1
#define I2C_IT_SENSOR_ADDR_0	0x20
#define I2C_IT_SENSOR_ADDR_1	0x24
#define I2C_IT_SENSOR_ADDR_2	0x28
#define I2C_IT_SENSOR_ADDR_3	0x2C
#define I2C_IT_SENSOR_ADDR_4	0x40
//#define I2C_IT_SENSOR_ADDR_5	0x44 - In Use!  Change chip in Loki's head!
#define I2C_IT_SENSOR_ADDR_6	0x48
//#define I2C_IT_SENSOR_ADDR_7	0x4C - In Use (blocked)




//---------------------------------------------------------------------------
// Port Definitions ( with MicroChip Names )				16F877 Pin/Header Pin

//#use fast_io( A )		// TODO-CAR Requires that we set the I/O Correctly!
//#use fast_io( B )
//#use fast_io( C )
//#use fast_io( D )
//#use fast_io( E )

#define PIN_IR_NC		PIN_A0	// IN	RA0/AN0		*> Spare IR!		 2/7
#define PIN_IR_0		PIN_A1	// IN	RA1/AN1		* AN1=IR1			 3/9
#define PIN_IR_1		PIN_A2	// IN	RA2/AN2		* AN2=RightFwdIR	 4/11
#define PIN_IR_2		PIN_A3	// IN	RA3/AN3		* AN3=ScanningIR	 5/13
#define DEBUG_SCOPE		PIN_A4	// OUT	RA4/TOCKI	* SCOPE	<NOT A/D!>	 6/1 <** A4 Requires Pullup Resistor!
#define PIN_BATTERY0	PIN_A5	// IN	RA5/AN4/SS	* PIC BATTERY		 7/2

#define PORT_A_DIR		0b11101111	// 1=Input 0=Output

#define EXTERN_INT		PIN_B0	// IN	RB0/INT		> INTERRUPT   (INT)	33/20 // Used by External devices such as I2C Port expansion
#define PIN_UNUSED_B1	PIN_B1	// IN	RB1			* Rear Bumper  (PWM1)	34/18 <<IDEA: PIN_RC_BUTTON2 = REAR BUMPER?
#define PIN_LED_G		PIN_B2	// IN	RB2			* Green LED	 	35/16 <<IDEA: PIN_RC_KILL_SW
#define PIN_PWR_RELAY	PIN_B3	// IN/OUT RB3/PGM	* Available?  (PWM3)	36/14
#define PIN_LED_Y		PIN_B4	// IN/OUT RB4		* Yellow LED	(PWM4)	37/12
#define PIN_LED_R		PIN_B5	// IN/OUT RB5		* Red LED		38/10
#define PIN_PROGRAM_CLK	PIN_B6	// IN				* Programmer	39/6 <<IDEA: CTS
#define PIN_PROGRAM_DTA	PIN_B7	// IN				* Programmer	40/8 <<IDEA: RTS
#define PORT_B_DIR		0b11111111	// 1=Input 0=Output

#define PIN_US_TRIGGER0	PIN_C0	// OUT	RC0/T1CLKI	* Free				15/25
#define PIN_US_TRIGGER2	PIN_C1	// OUT	RC1/CCP2	> Accel-use RB6/7?	16/27
#define PIN_US_RETURN	PIN_C2	// IN	RC2/CCP1	* UltraSonic		17/29
#define PIN_I2C_CLK		PIN_C3	// IN	RC3/SCK/SCL	* I2C Clk In/Out	18/31
#define PIN_I2C_IN		PIN_C4	// IN	RC4/SDI/SDA	* I2C Data In/Out	23/33
#define PIN_US_TRIGGER1	PIN_C5	// OUT	RC5/SDO		* Start Max US			24/35
#define PIN_TX			PIN_C6	// OUT	RC6/TX/CK	* RS232 TX	( TX )	25/37
#define PIN_RX			PIN_C7	// IN	RC7/RX/DT	* RS232 RX	( RX )	26/39
#define PORT_C_DIR		0b10011100	// 1=Input 0=Output

#define PIN_SERVO_0		PIN_D0	// OUT	RD0/PSP0	*					19/40	// Laptop Power Servo
#define PIN_SERVO_1		PIN_D1	// OUT	RD1/PSP1	*					20/38
#define PIN_SERVO_2		PIN_D2	// OUT	RD2/PSP2	*					21/36
#define PIN_Unused_D3	PIN_D3	// OUT	RD3/PSP3	*					22/34
#define PIN_MOTOR_DIR_R	PIN_D4	// OUT 	RD4/PSP4	* Front Bumber		27/32
#define PIN_MOTOR_DIR_L	PIN_D5	// OUT 	RD5/PSP5	* Front Bumper		28/30
#define PIN_SPEED_R		PIN_D6	// OUT	RD6/PSP6	* LED Eye			29/28
#define PIN_SPEED_L		PIN_D7	// IN	RD7/PSP7	* LED Eye			30/26
#define PORT_D_DIR		0b11110000	// 1=Input 0=Output

#define PIN_IR_3		PIN_E0	// IN	RE0/RD/AN5	* AN5=			 8/15
#define PIN_IR_4		PIN_E1	// IN	RE1/WR/AN6	> AN6=		 9/17
#define PIN_IR_5		PIN_E2	// IN	RE2/CS/AN7	> AN7=		10/19
#define PORT_E_DIR		0b11111111	// 1=Input 0=Output

// ADC channel selection codes
#define	A2D_CHAN_IR0		1	// AN1=IR0	- LeftSideIR
#define A2D_CHAN_IR1		2	// AN2=IR1	- LeftFwdIR
#define A2D_CHAN_IR2		3	// AN3=IR2	- LeftFwdLongRangeIR

//#define	A2D_NOT_USED	0	// AN0=Sharp 5 Beam IR Ranger (IR3-7)

#define	A2D_CHAN_IR3		5	// AN5=IR8	- RightFwdLongRangeIR
#define A2D_CHAN_IR4		6	// AN6=IR9	- RightFwdIR
#define A2D_CHAN_IR5		7	// AN7=IR10	- RightSideIR

#define	A2D_CHAN_BATT0		4	// AN4=Bat0 - First Battery

//---------------------------------------------------------------------------
// Constants
#define	TIME_20_MS				   (19-1)		// Loop every 20ms.  Subtract long pulse times (INTR_SET_TIME_)
#define INTR_TIME_1MS				   38		// Note: for 38 ticks/ms = 26.3us per tick ( use for UltraSonic ranging )
#define INTR_SET_TIME_1_MS		( 256-INTR_TIME_1MS )	// Ticks per timer0 loop ( counts up from this value! )
#define INTR_SET_TIME_2_MS		( 256-(INTR_TIME_1MS*2) )
#define INTR_SET_TIME_3_MS		( 256-(115) )
#define INTR_SET_TIME_400US		( 256-15 )		// short cycle for start of servo position
#define INTR_SET_TIME_600US		( 256-23 )		// compensate for short cycle - get back to 1MS ticks
#define ONE_SECOND						 1000	// number of 1ms ticks for g1msTimer to reach zero in one second
#define FULL_SPEED					       31	// Max Command speed from iPaq
#define TABLE_VOLTAGE_OFFSET			   20	// Compensate for 6 or 7 cell Battery pack
#define STATUS_SEND_INTERVAL			   16	// Must be Even Number!  10 = Every 200 ms
#define RADAR_SEND_INTERVAL				    8	// send on half status boundries
#define WATCHDOG_TIME					  100	// 20ms * 100 = 2 second timeout
#define TX_BUFFER_SIZE					   70

#define POWER_BUTTON_SERVO_MOVE_TIME	   50	// 20ms * 50 = 1 second button "push" time
#define LAPTOP_POWER_SERVO_UP_POSITION	   65	// Position to move servo to when Idle
#define LAPTOP_POWER_SERVO_DOWN_POSITION   40	// Position to move servo to when pushing the laptop power button


// Timed Sensor Reading sequence
#define SENSOR_US_0						0
#define SENSOR_US_1						1
#define SENSOR_US_2						2
//#define SENSOR_ACCEL_X					3
//#define SENSOR_ACCEL_Y					4
#define NUMBER_OF_US_SENSORS			3
#define NUMBER_OF_TIMED_SENSORS			3	// 3 Ultrasonic, 0 for Accelerometer (X&Y)


// For Sharp GP2Y3A003K0F 5-beam IR sensor
//#define IR_BEAM_LED_1	0b00000011	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_2	0b00000101	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_3	0b00001001	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_4	0b00010001	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_5	0b00100001	// bit 0 = "Vin", always on

// Servo Limits are defined in PicCommands.h, including:
// SERVO_MAX SERVO_MIN SERVO_CENTER SERVO_OFF, etc.

// Servo Movement Speed
#define SERVO_RATE_SLOW				    1
#define SERVO_RATE_MED				    2
#define SERVO_RATE_FAST			 	    4
#define SERVO_RATE_MAX				   50

#define DEBUG_STRING_LEN			   20
#define SERIAL_BUFFER_SIZE			   10	// Buffer for incoming serial data

// Serial Cmd Processing State Machine
#define SIO_WAIT_FOR_SYNC_0			 0x00
#define SIO_WAIT_FOR_SYNC_1			 0x01
#define SIO_GET_CMD					 0x02
#define SIO_GET_PARAM1				 0x03
#define SIO_GET_PARAM2				 0x04
#define SIO_GET_PARAM3				 0x05
#define SIO_GET_PARAM4				 0x06
#define SIO_GET_TERM_CHAR			 0x07
#define SIO_CMD_READY				 0x08
#define SIO_WAIT_FOR_CMD_COMPLETE	 0x09
#define SIO_BAD_TERM_CHAR			 0x0A

// State Machines
#define IDLE						0x00	// Used in all state machines

// Sensor Timing State Machine
#define SENSOR_WAITING_FOR_US_ECHO			1
#define SENSOR_WAITING_FOR_ALL_US_ECHOS		2
#define SENSOR_WAITING_FOR_ACCEL_RE			3
#define SENSOR_WAITING_FOR_ACCEL_FE			4
#define SENSOR_ALL_READINGS_DONE			5	// Until next status request
#define SENSOR_US_DISABLED					6	// Disable while I2C US ranger running

//#define SENSOR_WAITING_FOR_US_I2C			3
//#define SENSOR_WAITING_FOR_US_RE			0
//#define SENSOR_WAITING_FOR_US_FE			1

// Camera mounted I2C US Sensor State machine
#define I2C_US_SERVO_MOVING					1
//#define I2C_US_SERVO_IN_POSITION			2
#define I2C_US_OK_TO_PING					3
#define I2C_US_PINGING						4


// Robot Power State Machine
#define POWER_STATE_PRESSING_POWER_BUTTON	1
#define POWER_STATE_RELEASING_POWER_BUTTON	2
#define POWER_STATE_MONITORING_SENSORS		3


// Radar Scan State Machine
#define RADAR_SCAN_BEGIN_READ				1
#define RADAR_SCAN_READ_AND_MOVE			2
#define RADAR_SCAN_READY					3
#define RADAR_SCAN_NEXT_SCAN				4

#define RADAR_SCAN_BAD_STATE				9
//#define RADAR_SCAN_READ_AND_MOVE

#define RADAR_DELAY_SERVO_MOVE				10	// 20 ms ticks = 200ms
#define RADAR_DELAY_SEEK_END				40	// 20 ms ticks = 800ms

#define FWD									1	// Motor Directon Forward or reverse
#define REV									0

#define TRIGGER_THRESHOLD					4	// NOT Inches! Fudge factor before triggering sensors for human presence detection


//---------------------------------------------------------------------------
// Globals
int8			g20msTimer = 0;			// Master counter for 20ms Servo loop
int16			g1msTimer = 0;
			// Counts down from max FFFF (65,535) to zero on demand (65.5 seconds max)
int8			gUSPingTimer = 0;		// Counts up from zero to 5, every 10ms (50ms cycle)
int8			gI2C_USTimer = 0;		// Counts down from 65 to zero, to count off 65ms delay
//int8			gWatchDogTimer = 0;
boolean			gTimeToReadSensors = FALSE;
boolean			gTimeToReadCompass = FALSE;
int8			gSerialState = SIO_WAIT_FOR_SYNC_0;
boolean			gSerialCmdReady = FALSE;
int8			gSensorTimingState = IDLE;
//int8			gUS_Timer = 0;
//int16			gMaxSonarRisingEdgeTime = 0;
//int8			gTachometer = 0;		// Holds last reading from wheel counters
char        gSerialRxChar = 0;
//boolean			gRadarScanEnabled = FALSE;
//int8			gRadarScanState = 0;
int8			gI2C_USState = IDLE;
//boolean			gRadarSampleReady = FALSE;
// //boolean			gI2C_USServoReady = TRUE;
// //boolean			gI2C_USOKToPing = FALSE;
//signed int8		gUSSampleNumber = 0;
//int8			gUSRadarSample = 0;
//boolean			LedState_Green = 1;		// 1 = Off (active low LEDs)
//boolean			LedState_Yellow = 1;
//boolean			LedState_Red = 1;
int8			gLedEyeTimer_L = 0;
int8			gLedEyeTimer_R = 0;
//int8			gLedEyeBrightness = 0;
int8			gLedEyeMode = 0;
int8			gEyeState = 0;

int8			gRobotPowerMode = 0;		// overall state of the robot, both Laptop and PIC
int8			gRobotPowerState = IDLE;	// Where we are in the state machine
int8			gPowerStateTimer = 0;

// State of relays controlled by the 8574 I2C Expander:
boolean			gCameraLightRelayOn = FALSE;
boolean			gAuxRelayOn = FALSE;
boolean			gDynaUSBEnable = FALSE;
boolean			gClawIRSensorOn = FALSE;

int8			gTimedSensorNumber = 0;	// Which timed sensor to use  WARNING - don't exceed NUMBER_OF_US_SENSORS

// PIC A/D
unsigned int8	gIR_0 = 0;	// Tracks highest value (closest object) of Raw value
unsigned int8	gIR_1 = 0;
unsigned int8	gIR_2 = 0;
unsigned int8	gIR_3 = 0;
unsigned int8	gIR_4 = 0;
unsigned int8	gIR_5 = 0;
unsigned int8	gBattery0 = 0xFF;	// Tracks lowest value

// I2C A/D:
unsigned int8	gIR2_0 = 0;	// Tracks highest value (closest object) of Raw value
unsigned int8	gIR2_1 = 0;	// so start low
unsigned int8	gIR2_2 = 0;
unsigned int8	gIR2_3 = 0;
unsigned int8	gIR2_4 = 0;
unsigned int8	gIR2_5 = 0;
unsigned int8	gIR2_6 = 0;
unsigned int8	gIR2_7 = 0;

// I2C-IT IR Sensors
unsigned int8	gIR3_0 = HW_SENSOR_NOT_ATTACHED;	// Tracks lowest value (closest Object)
unsigned int8	gIR3_1 = HW_SENSOR_NOT_ATTACHED;	// in inches or cm, so start high, and
unsigned int8	gIR3_2 = HW_SENSOR_NOT_ATTACHED;	// First read each cycle will apply
unsigned int8	gIR3_3 = HW_SENSOR_NOT_ATTACHED;	// a "real" value
unsigned int8	gIR3_4 = HW_SENSOR_NOT_ATTACHED;	// Unless the sensor is not attached
unsigned int8	gIR3_5 = HW_SENSOR_NOT_ATTACHED;
unsigned int8	gIR3_6 = HW_SENSOR_NOT_ATTACHED;
unsigned int8	gIR3_7 = HW_SENSOR_NOT_ATTACHED;

unsigned int8	gThermalArray[9];

unsigned int16	gCompass = 0;
//int8			gAccelX = 0;
//int8			gAccelY = 0;

unsigned int8	gUS_Range[NUMBER_OF_US_SENSORS];


// Trigger Levels for sensors
// Used during Laptop Power Down
unsigned int8	Trigger_IR_0 = 0; // Tracks highest value (closest object) of Raw value
unsigned int8	Trigger_IR_1 = 0;
//unsigned int8	Trigger_IR_2 = 0;
//unsigned int8	Trigger_IR_3 = 0;
//unsigned int8	Trigger_IR_4 = 0;
//unsigned int8	Trigger_IR_5 = 0;

//unsigned int8	Trigger_IR2_0 = 0;
//unsigned int8	Trigger_IR2_1 = 0;
//unsigned int8	Trigger_IR2_2 = 0;
//unsigned int8	Trigger_IR2_3 = 0;
//unsigned int8	Trigger_IR2_4 = 0;
//unsigned int8	Trigger_IR2_5 = 0;
//unsigned int8	Trigger_IR2_6 = 0;
//unsigned int8	Trigger_IR2_7 = 0;

//unsigned int8	Trigger_IR3_0 = HW_SENSOR_NOT_ATTACHED;	// I2C-IT sensors report in CM or Inches
//unsigned int8	Trigger_IR3_1 = HW_SENSOR_NOT_ATTACHED;
//unsigned int8	Trigger_IR3_2 = HW_SENSOR_NOT_ATTACHED;

//unsigned int8	Trigger_HW_Bumper = 0;
//unsigned int8	Trigger_IR_Bumper = 0;
//unsigned int8	Trigger_RightArm_Bumper = 0;



// Common data returned as Status to the PC
// Must keep in sync with PIC_STATUS_T
int8			gStatus_StatusFlags = 0;
int8			gStatus_LastError = 0;
int8			gStatus_DebugCode = 0;
int8			gStatus_HW_Bumper = 0;
int8			gStatus_IR_Bumper = 0;
int8			gStatus_RightArm_Bumper = 0;
int8			gStatus_LeftArm_Bumper = 0;

signed int16	gStatus_OdometerTickCount_L = 0;	// Negative indicates backwards travel
signed int16	gStatus_OdometerTickCount_R = 0;	// Negative indicates backwards travel
//int8			gStatus_Tachometer = 0;
//int8			gStatus_UltraSonic0 = 0;
//int8			gStatus_UltraSonic1 = 0;
//int8			gStatus_UltraSonic2 = 0;
Boolean			gStatus_MotorDir_L = FWD;
Boolean			gStatus_MotorDir_R = FWD;


PIC_CMD_T		gPicCmdBuffer;	// Used for incoming characters
PIC_CMD_T		gPicCmd;		// Used for processing the command


Boolean			gCmdReady = FALSE;		// When True, cmd from serial port ready to process
Boolean			gLastOdomState_L = 0;
Boolean			gLastOdomState_R = 0;
int8			TempMotorTickCount_L = 0;
int8			TempMotorTickCount_R = 0;
//int8			gTachometer_L = 0;
//int8			gTachometer_R = 0;


/**
#if( CAMERA_CONTROL_TYPE == PIC_CONTROLLER )
//int8			gCameraPanTilt_Timer = 0;
unsigned int8	gCameraPanTiltSpeed = 0;
boolean			gCameraPanRight = FALSE;
boolean			gCameraPanLeft = FALSE;
boolean			gCameraTiltUp = FALSE;
boolean			gCameraTiltDown = FALSE;
#endif
**/

boolean			gCompassCalMode = FALSE;	// Sets Compass Calibration mode; disables other compass activity
int8			gTxNextIn = 0;
int8			gTxNextOut = 0;


// Cmd for desired Servo Location
int8	LaptopPowerServo	= LAPTOP_POWER_SERVO_UP_POSITION;	// Servo 0
int8	ServoCmd_1			= 0;	// Servo 1
int8	ServoCmd_2			= 0;	// Servo 2
int8	ServoCmd_3			= 0;	// Servo 3
//int8	ServoCmd_4			= 0;	// Servo 4
//int8	ServoCmd_5		= 0;	// Servo 5
//int8	ServoCmd_6			= 0;	// Servo 6
//int8	ServoCmd_7			= 0;	// Servo 7

// Internal variable for servo movement used by Timer0 ISR.
// Don't change directly!
int8	ServoPosition_0		= 0;
int8	ServoPosition_1		= 0;
int8	ServoPosition_2		= 0;
int8	ServoPosition_3		= 0;
//int8	ServoPosition_4		= 0;
//int8	ServoPosition_5		= 0;
//int8	ServoPosition_6		= 0;
//int8	ServoPosition_7		= 0;


// Serial Command buffer (Input)
char	gSerialBuffer[SERIAL_BUFFER_SIZE+2];// Buffer for serial characters coming in RS-232 port (tack on one for end Null)

// Serial Response buffer (Output)
char	gResponseBuf[RESPONSE_STRING_LEN+4];
char	*gResponseMsg = &gResponseBuf[4];

byte gTxBuffer[TX_BUFFER_SIZE];


// End of file
