/****************************************************************************
*
*  RobotPic_Sub.c
*
*  Pic Timer Interrupt-driven PIC Robot control code for the Robot
*
*  Written for the CCSC C Compiler 2003-2006 Dave Shinsel
*
****************************************************************************/



//***************************************************************************
// Subroutine forward declarations

int8	a2d( int8 ChannelNumber );
void	ReadIRBumpers();
void	ReadRightArmBumpers();
void	ReadLeftArmBumpers();
void	SetRelayState();
boolean	GetSerialCmd();
void	SendResponse(int SetMessageLen);
void	SetServoPosition(int8 ServoCmd, int8 &ServoPosition, int8 ServoRate);
void	CheckForSerialData();
void	InitSensorValues();


//***************************************************************************
//	RS232 Receive Interrupt handler - Called when RS232 receive data available
//
#INT_RDA
void rda_isr( )
{
	CheckForSerialData();

}

//---------------------------------------------------------------------------
// Function to handle Char in HW buffer
// Throw bytes away until sync received
void CheckForSerialData()
{
	//char cTemp;

	if( (gSerialCmdReady) && (SIO_WAIT_FOR_CMD_COMPLETE == gSerialState) )
	{
		// Prior command has not finished being handled.  We must wait.
		return;
	}

	while( kbhit() )	// Check for characters in the hardware buffer
	{

		gSerialRxChar = getc();	// Get next character
     	output_low( PIN_LED_R ); // On - show we got a character

		//output_low( PIN_DBG2_LED );	// Turn On

		switch( gSerialState )
		{
			case SIO_WAIT_FOR_SYNC_0:
			{
				// Looking for first sync byte
				if( gSerialRxChar == SIO_SYNC_0 )
				{
					// First Sync byte found, go to next state
					gSerialState++;
				}
				break;
			}
			case SIO_WAIT_FOR_SYNC_1:
			{
				// Looking for second sync byte
				if( gSerialRxChar == SIO_SYNC_1 )
				{
					// Second Sync byte found
					gSerialState++;
				}
				break;
			}
			case SIO_GET_CMD:
			{
				// Get CMD byte
				gPicCmdBuffer.Cmd = gSerialRxChar;
				gSerialState++;
				break;
			}
			case SIO_GET_PARAM1:
			{
				// Get first Parameter
				gPicCmdBuffer.Param1 = gSerialRxChar;
				gSerialState++;
				break;
			}
			case SIO_GET_PARAM2:
			{
				gPicCmdBuffer.Param2 = gSerialRxChar;
				gSerialState++;
				break;
			}
			case SIO_GET_PARAM3:
			{
				gPicCmdBuffer.Param3 = gSerialRxChar;
				gSerialState++;
				break;
			}
			case SIO_GET_PARAM4:
			{
				gPicCmdBuffer.Param4 = gSerialRxChar;
				gSerialState++;
				break;
			}
			case SIO_GET_TERM_CHAR:
			{
				// Check for terminator char
				if( gSerialRxChar == CMD_TERM_CHAR )
				{
					// Got the full command!  Copy to the processing structure, so we
					// can start filling the input buffer with the next command.
					if( gSerialCmdReady )
					{
						// Prior command has not finished being handled.  We must wait.
						gSerialState = SIO_WAIT_FOR_CMD_COMPLETE;
						// Indicate that we are waiting
						return;
					}
					else
					{
						memcpy( &gPicCmd, &gPicCmdBuffer, PIC_CMD_SIZE );
						gSerialState = SIO_WAIT_FOR_SYNC_0;	// Ready to get the next command
						gSerialCmdReady = TRUE;
						return;	// Stop looping for now, so this routine does not take too long.
					}
				}
				else
				{
					gSerialState = SIO_BAD_TERM_CHAR;
				}
				break;
			}
			case SIO_WAIT_FOR_CMD_COMPLETE:
			{
				// Prior command completed.  Ok to copy the command now.
				memcpy( &gPicCmd, &gPicCmdBuffer, PIC_CMD_SIZE );
				gSerialState = SIO_WAIT_FOR_SYNC_0;	// Ready to get the next command
				gSerialCmdReady = TRUE;

				return;	// Stop looping for now, so this routine does not take too long.
			}
			default:
			{
				gStatus_LastError = HW_BAD_STATE;
				gSerialState = SIO_WAIT_FOR_SYNC_0;	// Abort and get next command
			}
		}
	}
		//output_high( PIN_DBG2_LED );	// Turn Off

}

//***************************************************************************
// String helper functions imported from <string.h> :
int *strlen(char *s)
{
   char *sc;

   for (sc = s; *sc != 0; sc++);
   return(sc - s);
}

char *strcopy(char *s1, char *s2)
{
  char *s;

  for (s = s1; *s2 != 0; s++, s2++)
	  *s = *s2;
  return(s1);
}
void SerialError( int ErrNo)
{
	int foo;
  	foo = RS232_ERRORS;
    output_high( PIN_LED_Y );  // Turn Off LED
    //LedState_Yellow = 1;
    output_high( PIN_LED_G );  // Turn Off LED
    //LedState_Green = 1;
}
//***************************************************************************
//	RS232 Transmit Interrupt handler - Called when RS232 transmit buffer empty
//

#int_tbe
void tbe_isr()
{
	putc(gTxBuffer[gTxNextOut]);
	gTxNextOut=(gTxNextOut+1) % TX_BUFFER_SIZE;
	if( gTxNextIn==gTxNextOut )
		disable_interrupts(INT_TBE);


/*
	if( gTxNextIn==gTxNextOut )
	{
		// No data to send
		return;
	}
	putc(gTxBuffer[gTxNextOut]);
	gTxNextOut=(gTxNextOut+1) % TX_BUFFER_SIZE;
	//disable_interrupts(INT_TBE);
*/
}

// Buffered putc function
void bputc(char c)
{
	short restart;
	int temp;

   restart=gTxNextIn==gTxNextOut;
   gTxBuffer[gTxNextIn]=c;
   temp=(gTxNextIn+1) % TX_BUFFER_SIZE;
   while(temp==gTxNextOut)
   {
   	// wait if buffer full
    enable_interrupts(int_tbe);	// kudge!
   }
   gTxNextIn=temp;
   if(restart)
    enable_interrupts(int_tbe);

/*
	gTxBuffer[gTxNextIn]=c;
	temp=(gTxNextIn+1) % TX_BUFFER_SIZE;
	while( temp==gTxNextOut )
	{
		// buffer full
		delay_us(10);
	}
	//disable_interrupts(INT_TBE);
	gTxNextIn=temp;	// Tell the ISR that there is a new char now
	//enable_interrupts(INT_TBE);
*/

}

//***************************************************************************
//	SendResponse
//	Sends string via RS-232 to controlling computer, in response to a command sent down.

void SendResponse(int SetMessageLen)
{
	int8 nSendMsgLen;
	int8 i;
	// Note gResponseMsg is a pointer to the 4th char in gResponseBuf.


	if( SetMessageLen == 0 )
	{
		nSendMsgLen = strlen( gResponseMsg );
	}
	else
	{
		nSendMsgLen = SetMessageLen;
	}
	if( nSendMsgLen >= RESPONSE_STRING_LEN )
	{
		// Don't allow message to exceed buffer size!
		nSendMsgLen = RESPONSE_STRING_LEN-1;
		output_low( PIN_LED_R );	// Turn On Error LED
		//LedState_Red = 0;
		gStatus_LastError = HW_SERIAL_CMD_BUFFER_OVERFLOW;
	}

	gResponseBuf[0] = 'Z';
	gResponseBuf[1] = 'Z';
	gResponseBuf[2] = nSendMsgLen+1;		// Add one for the CMD Number
	gResponseBuf[3] = gPicCmd.Cmd;		// Return the CMD number being responded to
	// Note: gResponseBuf[4] ==> start of gResponseMsg

	for( i=0; i < (nSendMsgLen+4); i++ )
	{
		// Removed for Serial Cable
		//delay_ms(1);	// Give time for the IRDA to assert CTS
		// while ( input(CTS) ); // Wait for CTS to go Low to transmit
		//delay_ms(2);
		// Use Red LED to show status of Xmit buffer - stays on if hung
		//output_low( PIN_LED_R );	// Turn On LED
		//LedState_Red = 0;
		if( RS232_ERRORS != 0x90 )
		{
			SerialError(1);
		}
		bputc( gResponseBuf[i] );
		if( RS232_ERRORS != 0x90 )
		{
			SerialError(2);
		}
	}
	output_High( PIN_LED_Y );	// Turn LED off after sending each response
	//LedState_Yellow = 1;
}


//***************************************************************************
void SetAllServosOff()
{
	// Move Servo to "up" position, and then turn off servo power
	LaptopPowerServo = LAPTOP_POWER_SERVO_UP_POSITION;
	gPowerStateTimer = POWER_BUTTON_SERVO_MOVE_TIME;
	gRobotPowerState = IDLE;

//	LaptopPowerServo	= SERVO_OFF;	// Servo 0
	ServoCmd_1			= SERVO_OFF;	// Servo 1
	ServoCmd_2			= SERVO_OFF;	// Servo 2
	ServoCmd_3			= SERVO_OFF;	// Servo 3
//	ServoCmd_4			= SERVO_OFF;	// Servo 4
//	ServoCmd_5			= SERVO_OFF;	// Servo 5
//	ServoCmd_6			= SERVO_OFF;	// Servo 6
//	ServoCmd_7			= SERVO_OFF;	// Servo 7

} // SetAllServosOff




//***************************************************************************
//  I2C_USInit
//	Initialize the Devantech SRF10 Ultrasonic ranger via I2C.
//  Set up custom parameters, such as Gain and max Range
#ifndef ULTRASONIC_SRF04	// if using SRF10 instead of SRF04
void I2C_USInit()
{

	i2c_start();
	i2c_write(I2C_US_SONAR_WRITE);	// US Ranger is at I2C address E0 hex. Bit0:  Low=Write, High=Read
	i2c_write(0x01);	// Register 1 = Gain Register
	i2c_write(0x07);	// Lower Gain = Shorter Range, less side lobes
//	i2c_write(0x18);	// Auto increment to Register 2 = Range Register, set to 3 feet
	i2c_stop();

}
#endif

//***************************************************************************
//  I2C_USPingStart
//	Starts a ping on the Devantech SRF10 Ultrasonic ranger via I2C.
#ifndef ULTRASONIC_SRF04	// if using SRF10 instead of SRF04
void I2C_USPingStart()
{

	// Write Ping Start Byte
	i2c_start();
	i2c_write(I2C_US_SONAR_WRITE);	// US Ranger is at I2C address E0 hex. Bit0:  Low=Write, High=Read
	i2c_write(0x50);	// Start the Ping.  0x50 = return value in Inches.
	i2c_stop();

}
#endif
//***************************************************************************
//  I2C_USRead
//	Reads Devantech SRF10 Ultrasonic ranger via I2C.
//	Returns 16 bit value representing range
#ifndef ULTRASONIC_SRF04	// if using SRF10 instead of SRF04
void I2C_USRead()
{

//	int16	USReading = 0;
	int8	TempUSHigh = 0;
	int8	TempUSLow = 0;

gUS_Range[0] = 5;

	// Read the results
	i2c_start();
	i2c_write(I2C_US_SONAR_WRITE);	// US Ranger is at I2C address E0 hex. Bit0:  Low=Write, High=Read
	i2c_write(0x02);	// Register 2 = High Byte of 16 bit register for US reading
	i2c_start();
	i2c_write(I2C_US_SONAR_READ);	// Bit0 High indicates doing a read from this device
	TempUSHigh = i2c_read();
	TempUSLow = i2c_read(0);	// 0 indicates last read.
	i2c_stop();

	if( (TempUSHigh != 0) || (TempUSLow > HW_SENSOR_NO_OBJECT)  )
	{
		// TempUSHigh only used if over 21.25 feet, or for cm!
		gUS_Range[0] = HW_SENSOR_NO_OBJECT;
	}
	else
	{
		gUS_Range[0] = TempUSLow;
	}

	// TODO: Do a running average to smooth out readings

	// TODO: remove this?
	//USReading = TempUSHigh <<8;	// Shift the high byte over
	//USReading = ( USReading + TempUSLow );



//TODO  REMOVE THIS DEBUG
//		printf("I2C US Reading=%04lu HEX HI=%02X LOW=%02X, DEC HI=%03u LOW=%03u\r",
//			USReading, TempUSHigh, TempUSLow,
//			TempUSHigh, TempUSLow );

}
#endif

//***************************************************************************
//  ReadI2C_A2D
//	Reads A2D chip on expansion card via I2C.

unsigned int8 ReadI2C_A2D( unsigned int8 ChannelSelect)
{

	// Note: MAX 127/128 supports 12 bits of precision.
	// low byte has 4 bits of data, and 4 bytes of zeros, giving more granularity / accuracy.
  	// We drop the low byte because its are not needed for IR range (IR Sensor not that accurate)
	// For ChannelSelect ControlByte, see definition of IC2_A2D_CHANNEL_0, and MAX127 Datasheet, page 10

	unsigned int8 A2DResultLow;	 // Data not kept, just thrown away.  Add in if precision needed
	unsigned int8 A2DResultHigh; // Real data that is returned

	// Begin the Write cycle (send the A/D command request)
	i2c_start();
	// Send the Address of the chip.
	i2c_write(I2C_A2D_ADDRESS_W);
	// Send the Control Byte
	i2c_write(ChannelSelect);
	//Send stop bit - stop writing (starts A/D conversion)
	i2c_stop();		// TODO - can this be removed?
	delay_us(10);


	// Now begin the read cycle
	i2c_start();
	// Send the Chip Address
	i2c_write(I2C_A2D_ADDRESS_R);
	// Read the high value
	A2DResultHigh = i2c_read();	// with Ack
	// Read the low value (this is discarded, but does not work if we don't read it)
	A2DResultLow = i2c_read(0);	// Send NAK to signal end of read
	// Stop reading
	i2c_stop();

	Return A2DResultHigh;

}


//***************************************************************************
//
//	Reads HVW "I2C-IT"IR Range Sensor via I2C.
// Tell the I2C-It Sensor what Units to use for return values
#define I2C_IT_UNITS_INCHES			1
#define I2C_IT_UNITS_CENTEMETERS	2
#define I2C_IT_UNITS_RAW			3

unsigned int8 ReadI2C_IT( unsigned int8 SensorAddress)
{
	// I2C-IT Address range from 0x20 to 0x2E (base address is alway even)
	// I2C Write - base address.  I2C Read = base address + 1

//#use i2c(master, sda=PIN_C4, scl=PIN_C3, SMBUS, SLOW)
// TODO: Try these options: SLOW,FAST, FORCE_HW

	unsigned int8 RangeValue; // data that is returned


	i2c_start();
	i2c_write(SensorAddress);		// Send the Address of the chip
	i2c_write(I2C_IT_UNITS_INCHES);	// Tell the I2C-It Sensor what Units to use for return values
	i2c_start();					// Re-start
	i2c_write(SensorAddress+1);		// Add 1 to the Address for all Read Commands
	RangeValue = i2c_read(0);		// Get the value
	i2c_stop();


//#use i2c(master, sda=PIN_C4, scl=PIN_C3)
// TODO: Try these options: SLOW,FAST, FORCE_HW

	Return RangeValue;

}

//***************************************************************************
//  ReadI2C_Port
//	Reads PCF8574 I/O Expansion chip on expansion card via I2C.
//  NOTE: For repeated reads, need to separate into the write followed
//        by "N" number of reads.

int8 ReadI2C_Port(int Address)
{
	int8	Result;
	int8 boo;


	//printf("i2c start... ");
	i2c_start();

	// Send the Address of the chip.
	//printf("i2c write... ");
	i2c_write(Address);


	// TODO!  Get rid of this and just do the next read (replace boo with Result)???

	// For now, we just do a single read.  In the future, if we want to
	// monitor a port, do this as many times as you want to...
	Result = i2c_read();	// ACK means we are going to do another read after this


	//printf("i2c read... ");
	boo = i2c_read(0);	// NAK means we are done reading the port

	//printf("i2c stop\r\n");
	i2c_stop();

	return Result;

}

//***************************************************************************
//  WriteI2C_Port
//	Writes PCF8574 I/O Expansion chip on expansion card via I2C.

void WriteI2C_Port( int8 Address, int8 Value )

{


	// Begin the Write cycle (send the A/D command request)
	//printf("i2c start... ");
	i2c_start();

	// Send the Address of the chip.
	//printf("i2c write Command... ");
	i2c_write(Address);

	// Write the values - do this as many times as you like
	//printf("i2c write Value... ");
	i2c_write(Value);

	//printf("i2c write Value Again... ");
//	i2c_write(Value);

	//printf("i2c stop\r\n");
	i2c_stop();

}

//***************************************************************************
//  CalCompassPt
//	Used to Calibrate the Devantech CMPS03 Compass via I2C.
//	Writes 255 to register 15.  Need to do this once for each of the 4 compass points.
void CalCompassPt()
{
	i2c_start();
	i2c_write(I2C_COMPASS_WRITE);	// Compass is at I2C address C0 hex. Bit0:  Low=Write, High=Read
	i2c_write(0x15);	// Register 15 = Calibrate
	i2c_write(0xFF);	// Calibrate command
	i2c_stop();
}


//***************************************************************************
//  ReadCompass
//	Reads Devantech CMPS03 Compass via I2C.
//	Returns 16 bit value representing 0-360 degrees from North
//	Uses the more complex read of values 0.0->360.0, rather then the simple read of 0-255

void ReadCompass()
{
	int16	CompassReading = 0;
	int8	TempCompassHigh = 0;
	int8	TempCompassLow = 0;

	if( gCompassCalMode )
	{
		// Calibration Mode active.  Don't disturb the compass during calibration.
		return;
	}

	// Read High Byte
	i2c_start();
	i2c_write(I2C_COMPASS_WRITE);	// Compass is at I2C address C0 hex. Bit0:  Low=Write, High=Read
	i2c_write(0x02);	// Register 2 = High Byte of 16 bit register for compass reading
	i2c_start();
	i2c_write(I2C_COMPASS_READ);	// Bit0 High indicates doing a read next
	TempCompassHigh = i2c_read();
	TempCompassLow = i2c_read(0);
	i2c_stop();

	CompassReading = TempCompassHigh <<8;	// Shift the high byte over
	CompassReading = ( CompassReading + TempCompassLow );	// 3600--> 360.0

	gCompass = CompassReading;	// Save most recent sample
//	gCompass = 3580;

}




//***************************************************************************
//  ReadThermalArray
//	Reads Devantech TPA81 Thermal Array Sensor via I2C.
//	Sets 8 Bytes, one for each element in the array, corresponding to temperature
//  read by each pixel of the sensor

#define I2C_TPA_ADDRESS_W	 0xD0	// I2C Address of Devantech TPA81 Thermal Array Sensor
#define I2C_TPA_ADDRESS_R	 0xD1	// I2C Address of Devantech TPA81 Thermal Array Sensor


// Read Thermal Array - Reads 9  bytes from the Devantec TPA,
// Register 1 = Ambient temperature, Register 2-9 =
void ReadThermalArray()
{
	int8 i;

	// Setup the read
	i2c_start();
	i2c_write(I2C_TPA_ADDRESS_W);	// TPA is at I2C address D0 hex. Bit0:  Low=Write, High=Read
	i2c_write(1);					// Start read at register 1 (Ambient)
	i2c_start();
	i2c_write(I2C_TPA_ADDRESS_R);	// Bit0 High indicates doing a read from this device
	//Value = i2c_read();

	// Read Ambient and each of the 8 value registers
	for(i=1; i<=9; i++)
	{
		if( i == 9 )
		{
			gThermalArray[i] = i2c_read(0);   // 0 indicates last read
			//value = i2c_read();
		}
		else
		{
			gThermalArray[i] = i2c_read();
			//value = i2c_read();
		}
	}

	i2c_stop();
}



//***************************************************************************
// ReadA2DSensors
// Reads the A/D sensors each time called from the main loop. (about every 20ms)
ReadA2DSensors()
{
	// Read every time, but only saves the Max or Min value (closest object, or lowest voltage)
	// Note: for IR sensors, Larger numbers mean closer object, so save Largest value!
	// For Battery, save lowest value

	int8 temp;

	//	Read I2C A2D Channels
	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_0 );
	if( temp > gIR2_0 ) gIR2_0 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_1 );
	if( temp > gIR2_1 ) gIR2_1 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_2 );
	if( temp > gIR2_2 ) gIR2_2 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_3 );
	if( temp > gIR2_3 ) gIR2_3 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_4 );
	if( temp > gIR2_4 ) gIR2_4 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_5 );
	if( temp > gIR2_5 ) gIR2_5 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_6 );
	if( temp > gIR2_6 ) gIR2_6 = temp;	// save highest value (closest object)

	temp = ReadI2C_A2D( IC2_A2D_CHANNEL_7 );
	if( temp > gIR2_7 ) gIR2_7 = temp;	// save highest value (closest object)


	// Read PIC built-in A/D channels
	temp = a2d( A2D_CHAN_BATT0 );
	if( temp < gBattery0 ) gBattery0 = temp;	// save lowest value

	temp  = a2d( A2D_CHAN_IR0 );		// LeftSideIR
	if( temp > gIR_0 ) gIR_0 = temp;	// save highest value (closest object)

	temp  = a2d( A2D_CHAN_IR1 );		// LeftFwdIR
	if( temp > gIR_1 ) gIR_1 = temp;	// save highest value (closest object)

	temp  = a2d( A2D_CHAN_IR2 );		// LeftFwdLongRangeIR
	if( temp > gIR_2 ) gIR_2 = temp;	// save highest value (closest object)

	temp  = a2d( A2D_CHAN_IR3 );		// RightFwdLongRangeIR
	if( temp > gIR_3 ) gIR_3 = temp;	// save highest value (closest object)

	temp  = a2d( A2D_CHAN_IR4 );		// RightFwdIR
	if( temp > gIR_4 ) gIR_4 = temp;	// save highest value (closest object)

	temp = a2d( A2D_CHAN_IR5 );			// RightSideIR
	if( temp > gIR_5 ) gIR_5 = temp;	// save highest value (closest object)

	// Read I2C-IT IR Rangers
	gIR3_0 = ReadI2C_IT( I2C_IT_SENSOR_ADDR_0 );
	gIR3_1 = ReadI2C_IT( I2C_IT_SENSOR_ADDR_1 );
	gIR3_2 = ReadI2C_IT( I2C_IT_SENSOR_ADDR_2 );
	gIR3_3 = ReadI2C_IT( I2C_IT_SENSOR_ADDR_3 );
	//if( temp < gIR3_0 )
		//gIR3_0 = temp;	// save lowest value (closest object)
/*
	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_1 );
	if( temp < gIR3_1 ) gIR3_1 = temp;	// save lowest value (closest object)

	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_2 );
	if( temp < gIR3_2 ) gIR3_2 = temp;	// save lowest value (closest object)

	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_3 );
	if( temp < gIR3_3 ) gIR3_3 = temp;	// save lowest value (closest object)

	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_4 );
	if( temp < gIR3_4 ) gIR3_4 = temp;	// save lowest value (closest object)

	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_5 );
	if( temp < gIR3_5 ) gIR3_5 = temp;	// save lowest value (closest object)

	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_6 );
	if( temp < gIR3_6 ) gIR3_6 = temp;	// save lowest value (closest object)

	temp = ReadI2C_IT( I2C_IT_SENSOR_ADDR_7 );
	if( temp < gIR3_7 ) gIR3_7 = temp;	// save lowest value (closest object)
*/


}


//***************************************************************************
// Read HW Bumper and other switches, and control relays
// Uses 8574 I2C Expanders

// I2C device #1 (on main PIC board)
void SetRelayState()
{
	//   P0-5 = INPUT from IR Bumpers
	//   P6,7 = OUTPUT for Relay Control (independent of IR sensor)
	int8 PortCmd;
	PortCmd = 0b00011111;		// All "read" bits must be high
	if( gCameraLightRelayOn )
		PortCmd |= 0b10000000;	// Turn on the Light relay bit
	if( gAuxRelayOn )
		PortCmd |= 0b01000000;	// Turn on the Aux relay bit
	if( !gDynaUSBEnable )		// Reverse logic - Active Low to turn on
		PortCmd |= 0b00100000;	// Turn on the Dyna USB Controller bit
	WriteI2C_Port(I2C_PCF8574_1_WRITE, PortCmd); // Set the Relay state
}
void ReadHWBumpers()
{
	// Top 3 bits in the I2C port are reserved for Relay control
	int PortValues;
	PortValues = ReadI2C_Port(I2C_PCF8574_1_READ);
	PortValues ^= 0b11111111;	// invert all bits (goes low when HW bumper hit)
	PortValues &= 0b00011111;	// mask out relay bits

	gStatus_HW_Bumper |= PortValues;	// Any High bit in status over time indicates bumper hit!
}


// I2C device #2 (On Loki Expansion Board)
void ReadIRBumpers()
{
	int PortValues;
	PortValues = ReadI2C_Port(I2C_PCF8574_2_READ);
	PortValues ^= 0b11111111;	// invert all bits (goes low when IR bumper hit)

	gStatus_IR_Bumper |= PortValues;	// Any High bit over time indicates object (or cliff) detected!
}


// I2C device #3 (in Loki's Head)
void SetLedEyes( int8 EyeState )
{
	int8 PortCmd;
	PortCmd = EyeState | 0b00000001; // P0 = US Read enable.  High = Enabled (TODO - Toggle as needed to coordinate with other US sensors)
	PortCmd = PortCmd  | 0b00000010; // P1 = N/C I/O port.  Keep high (floating) by default
	PortCmd = PortCmd  & 0b11111011; // P2 = N/C Transistor.  Keep low (transistor off) by default

	WriteI2C_Port(I2C_PCF8574_3_WRITE, PortCmd);
}


// I2C device #4 (Left Arm)
void SetClawIRStateLeft()
{
	// Uses 8574 I2C Expander:
	//   P0 = OUTPUT to turn on the IR LED (turned off normally to save power)

/*** TODO!!!

	int8 PortCmd;

	PortCmd = 0b11111010;		// All "read" bits must be high
	if( gClawIRSensorOn )
		PortCmd |= 0b00000101;	// Turn on the IR LED bit, and the LED to show it's on
	WriteI2C_Port(I2C_PCF8574_4_WRITE, PortCmd); // Set the IR LED state
***/

}
void ReadLeftArmBumpers()
{
	int PortValues;
	PortValues = ReadI2C_Port(I2C_PCF8574_4_READ); // Read 4th I2C expansion chip
	PortValues ^= 0b11111111;	// invert all bits (goes low when HW or IR bumper hit)
//	PortValues ^= 0b11111101;	// invert all bits (goes low when HW or IR bumper hit), except #2 (Inside Claw)
//	PortValues &= 0b11110010;	// mask out IR LED and other LED bits
	gStatus_LeftArm_Bumper |= PortValues;	// Any High bit over time indicates object detected!
}

void ReadRightArmBumpers()
{
	int PortValues;
	PortValues = ReadI2C_Port(I2C_PCF8574_5_READ); // Read 5th I2C expansion chip
	PortValues ^= 0b11111111;	// invert all bits (goes low when HW or IR bumper hit)
//	PortValues ^= 0b11111101;	// invert all bits (goes low when HW or IR bumper hit), except #2 (Inside Claw)
//	PortValues &= 0b11110010;	// mask out IR LED and other LED bits
	gStatus_RightArm_Bumper |= PortValues;	// Any High bit over time indicates object detected!
}





/***************************************************************************
// OLD SelectIOExpander for use with Sharp 5-beam IR - NOT USED!
// Uses 8574 I2C Expander:
//   P0 = Vin
//   P1-5 = LED1-5 (5-beam IR) - NOW AVAILABLE!
//   P6,7 = I2C expander Relay Control (independent of IR sensor)
//   	gCameraLightRelayState tracks requested state of the relay

//#define IR_BEAM_LED_1	0b00000011	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_2	0b00000101	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_3	0b00001001	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_4	0b00010001	// bit 0 = "Vin", always on
//#define IR_BEAM_LED_5	0b00100001	// bit 0 = "Vin", always on

void SelectIOExpander()
{
	int8 PortCmd;
	int8 BeamSelect;
	int8 ExampleBeamNumber = 1;

	PortCmd = 0b00000000;		// P0 "Vin" Low = Reset for the next read (see Sharp doc)
	if( gCameraLightRelayOn )
		PortCmd |= 0b10000000;	// Turn on the Light relay bit
	if( gAuxRelayOn )
		PortCmd |= 0b01000000;	// Turn on the Aux relay bit

	WriteI2C_Port(I2C_PCF8574_1_WRITE, PortCmd);		// Apply the "Reset" pulse for the next read
	delay_ms(5);				// Reset pulse width = 5 ms

	BeamSelect = 0b00000001 << ExampleBeamNumber;	// select the right beam (1-5)
	BeamSelect |= 0b00000001;	// P0 "Vin" High = Start the next read
	PortCmd |= BeamSelect;		// Select the LED, and pull Reset high
	WriteI2C_Port(I2C_PCF8574_1_WRITE, PortCmd);		// Write to the port


	// in 20ms the Analog voltage will be ready to be read
	// reading = a2d( A2D_CHAN_5BEAM_IR );

}
*****/

//***************************************************************************
//	a2d
//	Reads an Analog to Digital Input
//
int8 a2d( int8 ChannelNumber )
{
	int8 A2dValue;
	set_adc_channel( ChannelNumber );	// There are 8 A/D inputs, pick which one
	delay_us( 10 );						// Need to delay before a valid read
	A2dValue = read_adc( );
	return A2dValue;

	return 0;
} // a2d

void InitSensorValues()
{
	// Called after each status sent to the host
	// sets all values to Max, then the sensor returns the lowest value (closest object) found

	// Ready the sensor state machine
	// Disable these to allow continuous pinging:
// TODO!	gTimedSensorNumber = 0;
// TODO!	gSensorTimingState = IDLE;	// Enable getting next US Ping
	int8 i;

/*	gIRBeamNumber = 0;
	for(i=0; i<5; i++)
	{
		gIRBeam[i] = 0;
	}
*/
	for(i=0; i<NUMBER_OF_US_SENSORS; i++)
	{
		gUS_Range[i] = 0;
	}

	gIR_0 =  0;	// Tracks highest value (closest object) of Raw value
	gIR_1 =  0;
	gIR_2 =  0;
	gIR_3 =  0;
	gIR_4 =  0;
	gIR_5 =  0;


// I2C A/D:
	gIR2_0 = 0;	// Tracks highest value (closest object) of Raw value
	gIR2_1 = 0;
	gIR2_2 = 0;
	gIR2_3 = 0;
	gIR2_4 = 0;
	gIR2_5 = 0;
	gIR2_6 = 0;
	gIR2_7 = 0;

// I2C-IT IR Sensors
	gIR3_0 = 50;//HW_SENSOR_NOT_ATTACHED;	// Tracks lowest value (closest Object)
	gIR3_1 = 51; //HW_SENSOR_NOT_ATTACHED;	// in inches or cm, so start high, and
	gIR3_2 = 52; //HW_SENSOR_NOT_ATTACHED;	// First read each cycle will apply
	gIR3_3 = HW_SENSOR_NOT_ATTACHED;	// a "real" value
	gIR3_4 = HW_SENSOR_NOT_ATTACHED;	// Unless the sensor is not attached
	gIR3_5 = HW_SENSOR_NOT_ATTACHED;
	gIR3_6 = HW_SENSOR_NOT_ATTACHED;
	gIR3_7 = HW_SENSOR_NOT_ATTACHED;

	for( i=0; i<9; i++ )
	{
		gThermalArray[i] = 0; // track highest value
	}

	gBattery0 = 0xFF;		// Tracks lowest value
//	gBattery1 = 0;
//	gStatus_CompassHigh = 0;
//	gStatus_CompassLow = 0;


}

//***************************************************************************
void CameraPanTiltStop()
{
#if( CAMERA_CONTROL_TYPE == PIC_CONTROLLER )

	// Stop any current Pan or Tilt operations
	gCameraPanRight = FALSE;
	gCameraPanLeft = FALSE;
	gCameraTiltUp = FALSE;
	gCameraTiltDown = FALSE;
	gCameraPanTiltSpeed = 0;	// Put timer loop in base state
#endif
}

//***************************************************************************
void ProcessPanTiltCmd()
{
#if( CAMERA_CONTROL_TYPE == PIC_CONTROLLER )
	// Handle Camera pan/tilt if this robot needs the PIC to do it

	sprintf(gResponseMsg, "AK1:CAM Pan=%02X Speed=%02X", gPicCmd.Param1, gPicCmd.Param2);

	// gPicCmd.Param1 = Camera Pan/Tilt Direction
	// gPicCmd.Param2 = Camera Pan/Tilt Speed

	// Camera Pan speed is 1-14, 1 is slowest, so need to invert
	if( gPicCmd.Param2 > 14) gPicCmd.Param2 = 14;
	gCameraPanTiltSpeed = (14 - gPicCmd.Param2) * 10;	// 0 = fastest speed (least delay)

	// Now do Pan/Tilt.  Initialize all to off.
	CameraPanTiltStop();

	switch( gPicCmd.Param1 )  // Pan/Tilt Command
	{
		case CAMERA_PAN_STOP:
		{
			// Nothing to do, already handled above by CameraPanTiltStop();
		}
		break;

		case CAMERA_PAN_ABS_CENTER:
		{
			// Center the camera as quickly as possible
			CameraPanServo = CAMERA_PAN_SERVO_CENTER;
			CameraTiltServo = CAMERA_TILT_SERVO_CENTER;
		}
		break;

		// Pan or Tilt Commands

		case CAMERA_PAN_UP:
		{
			gCameraTiltUp = TRUE;
		}
		break;

		case CAMERA_PAN_DOWN:
		{
			gCameraTiltDown = TRUE;
		}
		break;

		case CAMERA_PAN_LEFT:
		{
			gCameraPanLeft = TRUE;
		}
		break;

		case CAMERA_PAN_RIGHT:
		{
			gCameraPanRight = TRUE;
		}
		break;

	// Combo Pan/Tilt commands

		case CAMERA_PAN_UP_LEFT:
		{
			gCameraPanLeft = TRUE;
			gCameraTiltUp = TRUE;
		}
		break;

		case CAMERA_PAN_UP_RIGHT:
		{
			gCameraPanRight = TRUE;
			gCameraTiltUp = TRUE;
		}
		break;

		case CAMERA_PAN_DOWN_LEFT:
		{
			gCameraPanLeft = TRUE;
			gCameraTiltDown = TRUE;
		}
		break;

		case CAMERA_PAN_DOWN_RIGHT:
		{
			gCameraPanRight = TRUE;
			gCameraTiltDown = TRUE;
		}
		break;

		default:
		{
			sprintf(gResponseMsg, "AK1:BAD CAM Pan=%02X Speed=%02X", gPicCmd.Param1, gPicCmd.Param2);
		}
	}
#endif
}

void Initialize()
{

//	set_tris_a( PORT_A_DIR );
//	set_tris_b( PORT_B_DIR );
//	set_tris_c( PORT_C_DIR );
//	set_tris_d( PORT_D_DIR );
//	set_tris_e( PORT_E_DIR );
//	delay_ms(1);
	//port_b_pullups(0);

	int8 i;
	// Initialize all variables
	gCmdReady = FALSE;
	gResponseBuf[0] = 0;
	gI2C_USState = IDLE;
	gCompassCalMode = FALSE;
	gTimedSensorNumber = 0;
	gUSPingTimer = 0;


	gPicCmd.Cmd = 0;
	gPicCmd.Param1 = 0;
	gPicCmd.Param2 = 0;
	gPicCmd.Param3 = 0;
	gPicCmd.Param4 = 0;

	InitSensorValues();

	gStatus_StatusFlags = 0;
	gStatus_LastError = HW_PIC_NO_ERROR;
	gStatus_DebugCode = 0;
	gStatus_HW_Bumper = 0;
	gStatus_IR_Bumper = 0;
	gStatus_LeftArm_Bumper = 0;
	gStatus_RightArm_Bumper = 0;

	gStatus_OdometerTickCount_L = 0;
	gStatus_OdometerTickCount_R = 0;
	gStatus_MotorDir_L = 0;
	gStatus_MotorDir_R = 0;

	gLedEyeMode = LED_EYES_BLINK;
	SetLedEyes( 0 );	// Off.  SetLedEyes will handle the bottom 3 bits

	gSerialState = SIO_WAIT_FOR_SYNC_0;
	gSerialCmdReady = FALSE;

	// Setup the I/O Ports

	// Max Sonar EZ1 requires that trigger (Rx) line be held low for first 250ms
//	output_low( PIN_US_TRIGGER0 );
//	output_low( PIN_US_TRIGGER1 );
//	output_low( PIN_US_TRIGGER2 );

	// NOTE! Setup pin values before setting direction to avoid "glitch" at startup!
	SetAllServosOff();				// Keep timer routine from starting any servo just yet
	output_low( PIN_SERVO_0 );		// Turn Off here so the servo does not "glitch"
	output_low( PIN_SERVO_1 );
	output_low( PIN_SERVO_2 );
//	output_low( PIN_SERVO_3 );


	// set Camera lights off.
	gCameraLightRelayOn = FALSE;
	gAuxRelayOn = FALSE;
	gDynaUSBEnable = TRUE; // Enable Dyna USB controller
	gClawIRSensorOn = FALSE;
	SetRelayState();

//	output_high( PIN_ACCEL_AXIS );	//TODO-ACC shoudl this be a LOW?

	//	output_high( PIN_LIGHT_PWR );	// Turn Off
	output_high( PIN_LED_R );	// Turn Off
	//LedState_Red = 1;
	output_high( PIN_LED_G );	// Turn Off
	//LedState_Green = 1;
	output_high( PIN_LED_Y );	// Turn Off
	//LedState_Yellow = 1;

//	output_low( PIN_BLUE_EYE_L);	// Turn Off (backward from other LEDs)
//	output_low( PIN_BLUE_EYE_R );	// Turn Off

 	setup_adc_ports( ALL_ANALOG ); // Set A0 A1 A2 A3 A5 E0 E1 E2 as Analog inputs
	setup_adc( ADC_CLOCK_INTERNAL );
	set_adc_channel( 0 );


	// Max Sonar EZ1 requires that trigger (Rx) line be held high after 250ms start up,
	// for the first 49ms calibration +  do a range cycle, so it can calibrate properly
/*
	delay_ms(250);	// Wait for EZ1 to startup before calibration
	output_high( PIN_US_TRIGGER0 );
	output_low( PIN_LED_G ); // On
	delay_ms(50);	// wait for 49ms cal cycle + 20us min ping
	output_low( PIN_US_TRIGGER0 );	// Start the sonic burst!
	delay_ms(60);	// bogus reading
	output_high( PIN_LED_G ); // Off

	output_low( PIN_LED_Y ); // On
	output_high( PIN_US_TRIGGER1 );
	delay_ms(50);	// wait for 49ms cal cycle + 20us min ping
	output_low( PIN_US_TRIGGER1 );	// Start the sonic burst!
	delay_ms(60);	// bogus reading
	output_high( PIN_LED_Y ); // Off

	output_low( PIN_LED_R ); // On
	output_high( PIN_US_TRIGGER2 );
	delay_ms(50);	// wait for 49ms cal cycle + 20us min ping
	output_low( PIN_US_TRIGGER2 );	// Start the sonic burst!
	delay_ms(60);	// bogus reading
	output_high( PIN_LED_R ); // Off
*/

	// Indicate Startup by blinking the LEDs
	for( i=0; i<3; i++ )
	{
		//printf("DEBUG LEDs\r\n");

		WriteI2C_Port(I2C_PCF8574_4_WRITE, 0xFE);		// Right Arm Led On
		WriteI2C_Port(I2C_PCF8574_5_WRITE, 0xFE);		// Left Arm Led On
		output_low( PIN_LED_G );		delay_ms(70);	// PIC Led On
		WriteI2C_Port(I2C_PCF8574_4_WRITE, 0xFF);		// Right Arm Led Off
		WriteI2C_Port(I2C_PCF8574_5_WRITE, 0xFF);		// Left Arm Led Off
		output_high( PIN_LED_G );		delay_ms(70);	// Off

		WriteI2C_Port(I2C_PCF8574_4_WRITE, 0xFE);		// Right Arm Led On
		WriteI2C_Port(I2C_PCF8574_5_WRITE, 0xFE);		// Left Arm Led On
		output_low( PIN_LED_Y );		delay_ms(70);	// On
		WriteI2C_Port(I2C_PCF8574_4_WRITE, 0xFF);		// Right Arm Led Off
		WriteI2C_Port(I2C_PCF8574_5_WRITE, 0xFF);		// Left Arm Led Off
		output_high( PIN_LED_Y );		delay_ms(70);	// Off

		WriteI2C_Port(I2C_PCF8574_4_WRITE, 0xFE);		// Right Arm Led On
		WriteI2C_Port(I2C_PCF8574_5_WRITE, 0xFE);		// Left Arm Led On
		output_low( PIN_LED_R );		delay_ms(70);	// On
		WriteI2C_Port(I2C_PCF8574_4_WRITE, 0xFF);		// Right Arm Led Off
		WriteI2C_Port(I2C_PCF8574_5_WRITE, 0xFF);		// Left Arm Led Off
		output_high( PIN_LED_R );		delay_ms(70);	// Off

		delay_ms(70);
		delay_ms(70);
	}


}




// **************************************************************************
// GetSensorStateForHumans()
// Check Sensors readings for current objects.  Will be used to detect changes
//

void SetTriggerStateForHumanDetect()
{

	Trigger_IR_0 = gIR_0 + TRIGGER_THRESHOLD;	// Larger number = closer
	Trigger_IR_1 = gIR_1 + TRIGGER_THRESHOLD;
//	Trigger_IR_2 = gIR_2 + TRIGGER_THRESHOLD;
//	Trigger_IR_3 = gIR_3 + TRIGGER_THRESHOLD;
//	Trigger_IR_4 = gIR_4 + TRIGGER_THRESHOLD;
//	Trigger_IR_5 = gIR_5 + TRIGGER_THRESHOLD;

//	Trigger_IR2_0 = gIR2_0 + TRIGGER_THRESHOLD;
//	Trigger_IR2_1 = gIR2_1 + TRIGGER_THRESHOLD;
//	Trigger_IR2_2 = gIR2_2 + TRIGGER_THRESHOLD;
//	Trigger_IR2_3 = gIR2_3 + TRIGGER_THRESHOLD;
//	Trigger_IR2_4 = gIR2_4 + TRIGGER_THRESHOLD;
//	Trigger_IR2_5 = gIR2_5 + TRIGGER_THRESHOLD;
//	Trigger_IR2_6 = gIR2_6 + TRIGGER_THRESHOLD;
//	Trigger_IR2_7 = gIR2_7 + TRIGGER_THRESHOLD;

//	Trigger_IR3_0 = gIR3_0 - TRIGGER_THRESHOLD;	// I2C-IT Sensor!
//	Trigger_IR3_1 = gIR3_1 - TRIGGER_THRESHOLD;
//	Trigger_IR3_2 = gIR3_2 - TRIGGER_THRESHOLD;


//	Trigger_HW_Bumper = gStatus_HW_Bumper;		// Not used???
//	Trigger_IR_Bumper = gStatus_IR_Bumper;
//	Trigger_LeftArm_Bumper = gStatus_LeftArm_Bumper;
//	Trigger_RightArm_Bumper = gStatus_RightArm_Bumper;

//		(gUS_Range[0] < HUMAN_DETECT_RANGE)	||
//		(gUS_Range[1] < HUMAN_DETECT_RANGE)	||
//		(gUS_Range[2] < HUMAN_DETECT_RANGE)	||

}




// **************************************************************************
// CheckSensorsForHumans()
// Check Sensors during power save mode, to see if any humans detected
//
boolean CheckSensorsForHumans()
{
	boolean HumanDetected = FALSE;

	if(
		(gIR_0 > Trigger_IR_0)		||		// Larger number = closer
		(gIR_1 < Trigger_IR_1)		||
//		(gIR_2 < Trigger_IR_2)		||
//		(gIR_3 < Trigger_IR_3)		||
//		(gIR_4 < Trigger_IR_4)		||
//		(gIR_5 < Trigger_IR_5)		||

//		(gIR2_0 < Trigger_IR2_0)	||
//		(gIR2_1 < Trigger_IR2_1)	||
//		(gIR2_2 < Trigger_IR2_2)	||
//		(gIR2_3 < Trigger_IR2_3)	||
//		(gIR2_4 < Trigger_IR2_4)	||
//		(gIR2_5 < Trigger_IR2_5)	||
//		(gIR2_6 < Trigger_IR2_6)	||
//		(gIR2_7 < Trigger_IR2_7)	||

//		(gIR3_0 < Trigger_IR3_0)	||
//		(gIR3_1 < Trigger_IR3_1)	||
//		(gIR3_2 < Trigger_IR3_2)	||

//		(gUS_Range[0] < HUMAN_DETECT_RANGE)	||
//		(gUS_Range[1] < HUMAN_DETECT_RANGE)	||
//		(gUS_Range[2] < HUMAN_DETECT_RANGE)	||

		(0 != gStatus_HW_Bumper)		||
		(!(gStatus_IR_Bumper & IR_BUMPER_PIR_LEFT_MASK )   ) ||
		(!(gStatus_IR_Bumper & IR_BUMPER_PIR_LEFT_MASK )   ) ||
		(0 != gStatus_RightArm_Bumper)	||
		(0 != gStatus_LeftArm_Bumper)	 )
	{
		return 1;	// Human detected!
	}



}



// **************************************************************************
// ProcessStatusCmd()
// Send current status to the PC
//
void ProcessStatusCmd()
{
	int8 n;
//	int16	TempCompass;
	sprintf(gResponseMsg, "AK2");

	gStatus_DebugCode = 0;

	// Fill out the block of data to send to the PC
	// This MUST match PIC_STATUS_T
	gResponseMsg[3]  = gStatus_StatusFlags;
	gResponseMsg[4]  = gStatus_LastError;
	gResponseMsg[5]  = gStatus_DebugCode;
	gResponseMsg[6]  = gStatus_HW_Bumper;
	gResponseMsg[7]  = gStatus_IR_Bumper;
	gResponseMsg[8]  = gStatus_LeftArm_Bumper;
	gResponseMsg[9]  = gStatus_RightArm_Bumper;
	gResponseMsg[10] = gBattery0;

	// gCompass is in tenth degrees: 0 - 3599 (359.9 degrees)
	gResponseMsg[11] = (int8)(gCompass >>8);		// Hi Byte
	gResponseMsg[12] = (int8)(gCompass & 0xFF);		// Low Byte

	gResponseMsg[13] = (unsigned int8)((unsigned int16)gStatus_OdometerTickCount_L >>8);		// Hi Byte
	gResponseMsg[14] = (unsigned int8)((unsigned int16)gStatus_OdometerTickCount_L & 0xFF);		// Low Byte
	gResponseMsg[15] = (unsigned int8)((unsigned int16)gStatus_OdometerTickCount_R >>8);		// Hi Byte
	gResponseMsg[16] = (unsigned int8)((unsigned int16)gStatus_OdometerTickCount_R & 0xFF);		// Low Byte

	gResponseMsg[17] = 10;//gThermalArray[0]; // Thermal Array Ambient
	gResponseMsg[18] = 11;//gThermalArray[1]; // Thermal Array Pixel 1
	gResponseMsg[19] = 12;//gThermalArray[2]; // Thermal Array Pixel 2
	gResponseMsg[20] = 13;//gThermalArray[3]; // Thermal Array Pixel 3
	gResponseMsg[21] = 14;//gThermalArray[4]; // Thermal Array Pixel 4
	gResponseMsg[22] = 15;//gThermalArray[5]; // Thermal Array Pixel 5
	gResponseMsg[23] = 16;//gThermalArray[6]; // Thermal Array Pixel 6
	gResponseMsg[24] = 17;//gThermalArray[7]; // Thermal Array Pixel 7
	gResponseMsg[25] = 18;//gThermalArray[8]; // Thermal Array Pixel 8

	// A/D on main PIC board
	gResponseMsg[26] = gIR_0;	// IR0
	gResponseMsg[27] = gIR_1;	// IR1
	gResponseMsg[28] = gIR_2;	// IR2
	gResponseMsg[29] = gIR_3;	// IR3
	gResponseMsg[30] = gIR_4;	// IR4
	gResponseMsg[31] = gIR_5;	// IR5

	// I2C A/D on expansion board
	gResponseMsg[32] = gIR2_0;	// IR0
	gResponseMsg[33] = gIR2_1;	// IR1
	gResponseMsg[34] = gIR2_2;	// IR2
	gResponseMsg[35] = gIR2_3;	// IR3
	gResponseMsg[36] = gIR2_4;	// IR4
	gResponseMsg[37] = gIR2_5;	// IR5
	gResponseMsg[38] = gIR2_6;	// IR6
	gResponseMsg[39] = gIR2_7;	// IR7

	// I2C-IT IR Rangers
	gResponseMsg[40] = gIR3_0;	// IR0
	gResponseMsg[41] = gIR3_1;	// IR1
	gResponseMsg[42] = gIR3_2;	// IR2
	gResponseMsg[43] = gIR3_3;	// IR3
	gResponseMsg[44] = 54;//gIR3_4;	// IR4
	gResponseMsg[45] = 55;//gIR3_5;	// IR5
	gResponseMsg[46] = 56;//gIR3_6;	// IR6
	gResponseMsg[47] = 57;//gIR3_7;	// IR7

	gResponseMsg[48] = gUS_Range[0];	// Center - WARNING, Do not exceed NUMBER_OF_US_SENSORS
	gResponseMsg[49] = gUS_Range[1];	// Left
	gResponseMsg[50] = gUS_Range[2];	// Right

	// Send the Status!
	SendResponse(51);	// Must be gResponseMsg[xx] +1
						// WARNING!  Do not exceed RESPONSE_STRING_LEN!


	// Now, Reset values as needed

	gStatus_LastError = 0;
	output_high( PIN_LED_R );	// Turn Off Error LED
	//LedState_Red = 1;
	gStatus_HW_Bumper = 0;				// Reset after reporting the hit
	gStatus_IR_Bumper = 0;				// Reset after reporting the hit
	gStatus_LeftArm_Bumper = 0;			// Reset after reporting the hit
	gStatus_RightArm_Bumper = 0;		// Reset after reporting the hit

	gStatus_StatusFlags = 0;
	gStatus_OdometerTickCount_L = 0;
	gStatus_OdometerTickCount_R = 0;

//	gStatus_DebugCode = 0;
//	gStatus_UltraSonic0 = 0xFF;	// Initialize to MAX, sensor sends MIN value found
//	gStatus_UltraSonic1 = 0xFF;
//	gStatus_UltraSonic2 = 0xFF;	DONT reset this one, it's used by the scanner
//gUS_Range[0] = 0; // WARNING - don't exceed NUMBER_OF_US_SENSORS
//gUS_Range[1] = 0;
//gUS_Range[2] = 0;


	// Reset sensor to prepare to get next reading
	InitSensorValues();
}


void SendHeartbeat()
{
   sprintf(gResponseMsg, "AK5");
   gResponseMsg[3] = gSerialState; // SIO state for debugging SIO issues
   gResponseMsg[4] = gSerialRxChar; // last SIO char received for debugging SIO issues
   SendResponse(5);
}

//***************************************************************************
void ProcessSerialCmd()
{
	int8 i;
	switch( gPicCmd.Cmd )
	{

		case HW_RESET_CPU:
		{
			// Reset!
			//StopMotor();	// Before we do anything, make sure the motor is stopped.
			SetLedEyes( 0x00 );	// LED Eyes Off
			//Initialize();
			delay_ms(500);
			reset_cpu();
			break;
		}

		case HW_INITIALIZE:
		{
			// Reset servos to neutral positions, etc.
			sprintf(gResponseMsg, "AK1:INIT");
			Initialize();
			SetAllServosOff();
			break;
		}

		case HW_COMPASS_CAL_MODE:
		{
			if( 1 == gPicCmd.Param2 )
			{
				sprintf(gResponseMsg, "AK1:CAL Start");
				gCompassCalMode = TRUE;
			}
			else
			{
				sprintf(gResponseMsg, "AK1:CAL End");
				gCompassCalMode = FALSE;
			}
			SendResponse(0);
			break;
		}

		case HW_COMPASS_CAL_POINT:
		{
			if( gCompassCalMode )
			{
				CalCompassPt();
				sprintf(gResponseMsg, "AK1:COMPASS PT");
			}
			else
			{
				sprintf(gResponseMsg, "AK1:Not in CalMode");
			}
			SendResponse(0);
			break;
		}

		case HW_GET_STATUS:
		{
			ProcessStatusCmd();
			break;
		}

		case HW_CLEAR_ERROR:
		{
			sprintf(gResponseMsg, "AK1:CLR_ERR");
			gStatus_LastError = HW_PIC_NO_ERROR;
			output_high( PIN_LED_R );	// Turn Off Error LED
			//LedState_Red = 1;
			gStatus_DebugCode = 0;
			SendResponse(0);
			break;
		}

		case HW_GET_VERSION:
		{
			sprintf(gResponseMsg, "AK3");
			gResponseMsg[3] = PIC_CODE_VERSION;
			SendResponse(4);
			break;
		}

		case HW_RESET_ODOMETER:
		{
			sprintf(gResponseMsg, "AK1:ODOM NA!");
			// Not Implemented: gStatus_OdometerTickCount = 0;
			SendResponse(0);
			break;
		}

		case HW_SET_LIGHT_POWER:
		{
         // Relays are controlled via I2C expanded ports
			if( 0 != gPicCmd.Param2 )
			{
				gCameraLightRelayOn = TRUE;
				SetRelayState();
				sprintf(gResponseMsg, "AK1:SptLts On");
			}
			else
			{
				gCameraLightRelayOn = FALSE;
				SetRelayState();
				sprintf(gResponseMsg, "AK1:SptLts Off");
			}
			SendResponse(0);
			break;
		}

		case HW_SET_AUX_LIGHT_POWER:
		{
         // Relays are controlled via I2C expanded ports
			if( 0 != gPicCmd.Param2 )
			{
				gAuxRelayOn = TRUE;
				SetRelayState();
				sprintf(gResponseMsg, "AK1:AuxLts On");
			}
			else
			{
				gAuxRelayOn = FALSE;
				SetRelayState();
				sprintf(gResponseMsg, "AK1:AuxLts Off");
			}
			SendResponse(0);
			break;
		}

		case HW_SET_IR_SENSOR_POWER:
		{
         // IR LED is controlled via I2C expanded ports
			if( 0 != gPicCmd.Param2 )
			{
				gClawIRSensorOn = TRUE;
				SetClawIRStateLeft();
				sprintf(gResponseMsg, "AK1:LClawIR On");
			}
			else
			{
				gClawIRSensorOn = FALSE;
				SetClawIRStateLeft();
				sprintf(gResponseMsg, "AK1:LClawIR Off");
			}
			SendResponse(0);
			break;
		}

		case HW_SET_DYNA_USB:
		{
         // Reset the Dynamixel USB Controller via I2C expansion port
			if( 0 == gPicCmd.Param2 )
			{
				gDynaUSBEnable = FALSE;
				SetRelayState();
				sprintf(gResponseMsg, "AK1:Dyna USB Off");
			}
			else	// if( 1 == gPicCmd.Param2 )
			{
				gDynaUSBEnable = TRUE;
				SetRelayState();
				sprintf(gResponseMsg, "AK1:Dyna USB On");
			}
			SendResponse(0);
			break;
		}

		case HW_SET_LED_EYES:
		{
			// Set brightness of LED "Eyes".
			// 0 = off, 1-19 = brightness level
			gLedEyeMode = gPicCmd.Param1;		// for open and close of eyes
//			gLedEyeBrightness = gPicCmd.Param2;	// NOT IMPLEMENTED!


			if( LED_EYES_OPEN == gLedEyeMode )
			{
				// Open Eye in stages
				gEyeState = 6;	// Start closed
			}
			else if( LED_EYES_CLOSE == gLedEyeMode )
			{
				// Close Eye in stages
				gEyeState = 0;	// Start opened
			}
			else if( LED_EYES_ON == gLedEyeMode )
			{
				// Open Eye instantly
				SetLedEyes( 0xFF  );	// SetLedEyes will handle the bottom 3 bits
			}
			else if( LED_EYES_OFF == gLedEyeMode )
			{
				// Close Eye instantly
				SetLedEyes( 0x00  );	// SetLedEyes will handle the bottom 3 bits
			}
			else if( LED_EYES_BLINK == gLedEyeMode )
			{
				// Close Eye in stages, then open again
				gEyeState = 0;	// Start opened
			}


//			if( gLedEyeBrightness > LED_EYE_BRIGHTNESS_MAX )	// On time in 20ms cycle
//			{
//				gLedEyeBrightness = LED_EYE_BRIGHTNESS_MAX;
//			}
			sprintf(gResponseMsg, "AK1:EYES %02X", gPicCmd.Param1 );
			SendResponse(0);
			break;
		}


		case HW_SET_MOTOR_POWER:
		{
			if( 0 == (int8) gPicCmd.Param2 )
			{
//				output_low( PIN_PWR_RELAY );	// Turn Off
				sprintf(gResponseMsg, "AK1:TODO:POWER OFF");
			}
			else
			{
//				output_high( PIN_PWR_RELAY );	// Turn On
				sprintf(gResponseMsg, "AK1:TODO:POWER ON");
			}
			SendResponse(0);
			break;
		}

		/*
		#define LAPTOP_POWER_MODE_DEFAULT				0x00	// Leave laptop alone.  Do nothing
		#define LAPTOP_POWER_OFF_NO_RESTART				0x01	// Shut down Laptop (using servo).  Requires manual restart.
		#define LAPTOP_POWER_OFF_RESTART_ON_ANY_SENSOR	0x02	// Shut down, but restart when any sensor triggers (preset thresholds)
		#define LAPTOP_POWER_OFF_RESTART_ON_PIR_SENSOR	0x03	// Shut down, restart only with PIR sensor
		#define LAPTOP_POWER_OFF_RESTART_NO_PIR_SENSOR	0x04	// Shut down, restart with any sensor BUT PIR
		#define LAPTOP_POWER_OFF_RESTART_ON_TIMER		0x05	// TODO - not implemented!

		#define POWER_STATE_PRESSING_POWER_BUTTON
		#define POWER_STATE_RELEASING_POWER_BUTTON
		#define POWER_STATE_MONITORING_SENSORS
		*/
		case HW_SET_POWER_MODE:
		{
			//output_low( PIN_PWR_RELAY );	// Turn Off
			gRobotPowerMode = gPicCmd.Param2;
			if( LAPTOP_POWER_MODE_DEFAULT == gRobotPowerMode )
			{
   			sprintf(gResponseMsg, "AK1:PWR_DEFAULT");
				// Leave Servo in "up" position, and turn off servo power
				LaptopPowerServo = LAPTOP_POWER_SERVO_UP_POSITION;
				gPowerStateTimer = POWER_BUTTON_SERVO_MOVE_TIME;
				gRobotPowerState = IDLE;
			}
			else
			{
				// Must be one of the Laptop Power Off commands
				// Push the power button the go to next state
   			sprintf(gResponseMsg, "AK1:PWR_STBY");
				LaptopPowerServo = LAPTOP_POWER_SERVO_DOWN_POSITION;
				gRobotPowerState = POWER_STATE_PRESSING_POWER_BUTTON;
				gPowerStateTimer = POWER_BUTTON_SERVO_MOVE_TIME;
			}
			SendResponse(0);
			break;
		}

#if( CAMERA_CONTROL_TYPE == PIC_CONTROLLER )

		case HW_SET_CAMERA_PAN_ABS:
		{
			//CameraPanTiltStop();	// Stop any current pan/tilt operation
			//CameraPanServo = gPicCmd.Param1;
			// Happens often, so for performance resons, don't ACK.
			//sprintf(gResponseMsg, "AK1:CAM ABSPan POS=%u", gPicCmd.Param1);
			sprintf(gResponseMsg, "AK1:ERR CAM PAN NI");	// Not Implemented for Loki
			SendResponse(0);
			break;
		}
		case HW_SET_CAMERA_TILT_ABS:
		{
			//CameraPanTiltStop();	// Stop any current pan/tilt operation
			//CameraTiltServo = gPicCmd.Param1;
			// Happens often, so for performance resons, don't ACK.
			//sprintf(gResponseMsg, "AK1:CAM ABS Tilt POS=%u", gPicCmd.Param1);
			sprintf(gResponseMsg, "AK1:ERR CAM TILT NI"); // Not Implemented for Loki
			SendResponse(0);
			break;
		}

		case HW_SET_CAMERA_PAN_TILT:
		{
			// For slow pan or tilt.  Runs until stopped or Servo end reached.
			//ProcessPanTiltCmd();
			sprintf(gResponseMsg, "AK1:ERR CAM P/T NI"); // Not Implemented for Loki
			SendResponse(0);
			break;
		}
#endif

		default:
		{
//			output_low( PIN_LED_R );	// Turn On Error LED
//			LedState_Red = 0;
			gStatus_LastError = HW_CMD_NOT_IMPLEMENTED;
			sprintf(gResponseMsg, "NAK:BadCmd:%02X %02X %02X\n", gPicCmd.Cmd, gPicCmd.Param1, gPicCmd.Param2);
			SendResponse(0);
		}
	}

}

