/****************************************************************************
*
*  RobotPic_TimerISR.c
*
*  PIC Timer Interrupt-driven PIC Robot control code for Robot
*
*  Written for the CCSC C Compiler 2003-2006 Dave Shinsel
*
****************************************************************************/
//#define DEBUG_SCOPE_20MS_TIMING


//***************************************************************************
// SetServoPosition
// Get Servo Commands, and copy to the variables used by the timer ISR
void SetServoPosition(int8 ServoCmd, int8 &ServoPosition, int8 ServoRate)
{
	if( (ServoRate == SERVO_RATE_MAX) || (ServoRate == SERVO_OFF) )
	{
		ServoPosition = ServoCmd;		// Move servo a fast as possible to new position
		return;
	}

	if( ServoPosition < ServoCmd )		// Need to move servo forward
	{
		ServoPosition += ServoRate;		// Move servo, at specified rates (slow to fast)
		if( ServoPosition > ServoCmd )
		{
			ServoPosition = ServoCmd;	// Overshot value.  Stop at cmd position
		}
	}
	else if( ServoPosition > ServoCmd )	// Need to move servo backward
	{
		ServoPosition -= ServoRate;		// Move servo, at specified rates (slow to fast)
		if( ServoPosition < ServoCmd )
		{
			ServoPosition = ServoCmd;	// Overshot value.  Stop at cmd position
		}
	}
	// else, already at Commanded position.  Do Nothing.
}

//***************************************************************************
//	Timer 0 Interrupt handler - Triggers every 1ms (except for T=0-3)
//	Everything syncs around a 20ms cycle
//

#INT_RTCC // AKA Timer0
void rtcc_isr( )					// Interrupt Service Routine for Timer 0
{

	int8 ServoPulseCount;
	int i;
	int gMotorSpeed_Timer;


	if( 0 == g20msTimer )
	{
		set_timer0( (INTR_SET_TIME_400US) );	// Short Time for this period
	}
	else if( 1 == g20msTimer )
	{
		set_timer0( (INTR_SET_TIME_3_MS) );	// Longer Time for this period
		if( g1msTimer != 0 ) g1msTimer--;	// Make up for fact that this will use 2 MS.
		if( gI2C_USTimer != 0) gI2C_USTimer--;
	}
	else if( 2 == g20msTimer )
	{
		set_timer0( (INTR_SET_TIME_600US) );	// Compensate for first short time to get on even 20MS cycle
	}
	else
	{
		set_timer0( INTR_SET_TIME_1_MS );	// Assumes clock is 20MHz. Interrupts every 1ms
	}

	// Timers used for delays in main program.
	// Each counts down to zero at 1ms ticks, then stops.
	if( g1msTimer != 0 )
	{
		g1msTimer--;
	}

	if( gI2C_USTimer != 0 )
	{
		gI2C_USTimer--;
	}


	// Check Odometer Wheels (or other position sensors, such as arms?)
	// for White/Black transitions every 1ms
	// Note: This code duplicated in the first time period, as it is 3ms long
		if( input(PIN_SPEED_L) != gLastOdomState_L)
		{
			if( TempMotorTickCount_L < 255 ) TempMotorTickCount_L++;
			gLastOdomState_L = !(gLastOdomState_L);
		}

		if( input(PIN_SPEED_R) != gLastOdomState_R)
		{
			if( TempMotorTickCount_R < 255 ) TempMotorTickCount_R++;
			gLastOdomState_R = !(gLastOdomState_R);
		}

	// Check Front HW bumper switches (any press will trigger the bumper)
	// I2C Bumpers are checked elsewhere
//	if( !input(PIN_FWD_BMPR_R) )	// Low = bumper pressed, but set status bit HIGH
//		bit_set(gStatus_HW_Bumper, FRONT_RIGHT_BUMPER_BIT);

//	if( !input(PIN_FWD_BMPR_L) )
//		bit_set(gStatus_HW_Bumper, FRONT_LEFT_BUMPER_BIT);


	// Adjust brightness of LED Eyes
	if( 1 == gLedEyeTimer_L )
	{
		// Count down complete.  Turn off LED
//		output_low( PIN_BLUE_EYE_L );	// Left Eye Off
		gLedEyeTimer_L = 0;
	}
	else if( gLedEyeTimer_L != 0 )
	{
		gLedEyeTimer_L--;
	}

	if( 1 == gLedEyeTimer_R )
	{
		// Count down complete.  Turn off LED
//		output_low( PIN_BLUE_EYE_R );	// Right Eye Off
		gLedEyeTimer_R = 0;
	}
	else if( gLedEyeTimer_R != 0 )
	{
		gLedEyeTimer_R--;
	}



	///////////////////////////////////////////////////////////////////////////////////
	// Check the time-sensitive Sensors State Machine
	// used for Ultrasonic echos and (if attached) the accelerometer pulse widths (X&Y)
	// Sensors are numbered as follows:
	// 0: SENSOR_US_0
	// 1: SENSOR_US_1
	// 2: SENSOR_US_2

#ifdef ULTRASONIC_SRF04

	if( SENSOR_WAITING_FOR_US_ECHO == gSensorTimingState )
	{
		// Waiting for UltraSonic ping to be received.
		// Check for return echo
		if( 0 != CCP_1 )
		{
			// Return received!
			if( CCP_1 >= 0x5500 )	// max range 8 feet
			{
				gUS_Range[gTimedSensorNumber] = HW_SENSOR_NO_OBJECT;	// 252 
			}
			else
			{
				gUS_Range[gTimedSensorNumber] = ( CCP_1 / 0x80 );	// round to 1 byte value
			}
	        //gUS_Timer = 0; // Reset the timer, so it does not overflow
			if( (++gTimedSensorNumber) >= NUMBER_OF_TIMED_SENSORS)	// next sensor -- WARNING - don't exceed NUMBER_OF_US_SENSORS
			{
				gTimedSensorNumber = 0;
			}
			gSensorTimingState = IDLE;	// Done with range until next ping sent (every 50ms)
		}
	}
#endif //ifdef ULTRASONIC_SRF04

	//---------------------------------------------------------------------------
	// Update sensors and servos based upon where we are in the 20ms cycle
	//---------------------------------------------------------------------------
	if( 0 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_high( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin)
		#endif
/*
		// Unless disabled, Start Pulse width for LED Eyes
		if( 0 != gLedEyeBrightness_L )
		{
//			output_high( PIN_BLUE_EYE_L );			// Left Eye On
			gLedEyeTimer_L = gLedEyeBrightness_L;	// Start count down timer
		}
		if( 0 != gLedEyeBrightness_R )
		{
//			output_high( PIN_BLUE_EYE_R );			// Right Eye On
			gLedEyeTimer_R = gLedEyeBrightness_R;	// Start count down timer
		}
*/


		// Note that the next Servo section will take 3ms, so must compensate for anything
		// that is normally driven by the 1ms interrupt.
		// Update Servo values from Servo Cmd.
		SetServoPosition( LaptopPowerServo,	ServoPosition_0,	SERVO_RATE_MAX );
		//SetServoPosition( ServoCmd_1,	ServoPosition_1,	SERVO_RATE_MAX );
		//SetServoPosition( ServoCmd_2,	ServoPosition_2,	SERVO_RATE_MAX );
		//SetServoPosition( ServoCmd_3,	ServoPosition_3,	SERVO_RATE_MAX );

		// Output the Servo pulse Start by raising the line high
		// Note: a command of 0 = SERVO_OFF - no pulse!
		Output_Bit( PIN_SERVO_0, (ServoPosition_0 != SERVO_OFF) );
		//Output_Bit( PIN_SERVO_1, (ServoPosition_1 != SERVO_OFF) );
		//Output_Bit( PIN_SERVO_2, (ServoPosition_2 != SERVO_OFF) );
		//Output_Bit( PIN_SERVO_3, (ServoPosition_3 != SERVO_OFF) );


		///////////////////////////////////////////////////////////////////////////////////
		// time-sensitive Sensors State Machine
		// This is where we Start sensor reads
		// *** NOTE: Not used in this version!  All sensor readings tied to Status updates! ***
		// if( IDLE == gSensorTimingState )
		// CheckSensorNumber:

	}
	//---------------------------------------------------------------------------
	else if( 1 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_low( DEBUG_SCOPE );
		#endif

		// Go low when appropriate for each servo
		// Servo values are in the range of 0-SERVO_MAX ( a command of 0 = servo off - no pulse! )
		for( ServoPulseCount = 1; ServoPulseCount <= SERVO_MAX; ServoPulseCount++ )
		{
			if( ServoPulseCount == ServoPosition_0 )		output_low( PIN_SERVO_0 );
			//if( ServoPulseCount == ServoPosition_1 )		output_low( PIN_SERVO_1 );
			//if( ServoPulseCount == ServoPosition_2 )		output_low( PIN_SERVO_2 );
			//if( ServoPulseCount == ServoPosition_3 )		output_low( PIN_SERVO_3 );

			if( (31 == ServoPulseCount) || (90 == ServoPulseCount) )
			{
				// Wheel sensors are checked every 1ms for White/Black transitions
				// since this is a long interrupt cycle, we need to check here
				if( input(PIN_SPEED_L) != gLastOdomState_L)
				{
					if( TempMotorTickCount_L < 255 ) TempMotorTickCount_L++;
					gLastOdomState_L = !(gLastOdomState_L);
				}

				if( input(PIN_SPEED_R) != gLastOdomState_R)
				{
					if( TempMotorTickCount_R < 255 ) TempMotorTickCount_R++;
					gLastOdomState_R = !(gLastOdomState_R);
				}

				//TODO !!!
				CheckForSerialData();
				// The code above takes 8us, so just need to add one more
				//delay_us(1);	// pad to make the full "tick" time
			}
			else
			{
				delay_us( 5 );	// pad to make the full "tick" time
			}
		}
		// Since we spent all this time, we might have missed SIO Interrupt, so check
		CheckForSerialData();

	}
	//---------------------------------------------------------------------------
	else if( 2 == g20msTimer )
	{
		delay_us( 5 );	// pad to make the full "tick" time = 1/50ms ( 20us )
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_high( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin)
		#endif

	}
	//---------------------------------------------------------------------------
	else if( (3 == g20msTimer)	||
         (13 == g20msTimer) )
	{
		// Output_high( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin

#ifdef ULTRASONIC_SRF04	// if using SRF04 instead of SRF10
		// US Radar.  A Ping is sent every 50ms (to allow all echos to die down)
		// So this is checked at interval 3 and 13 of the 20ms loop (every 10ms)
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_low( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin)
		#endif

		if( SENSOR_US_DISABLED == gSensorTimingState )
		{
			// Do nothing.  US Sensors disabled
		}
		// Update the US count timer.
		else if( gUSPingTimer++ >= 4 )		// Counts up from zero to 5, every 10ms (50ms cycle)
		{
			// Time to ping next US sensor!
			gUSPingTimer = 0;	// Reset the timer: 5*10ms = 50ms
//output_high( DEBUG_SCOPE );

			// Make sure prior ping completed
			if( SENSOR_WAITING_FOR_US_ECHO == gSensorTimingState )
			{
				// Error!  Last reading not complete!  Bad Sensor?
				gUS_Range[gTimedSensorNumber] = HW_SENSOR_ERROR;	// Flag error = 254
				gSensorTimingState = IDLE;
				if( (++gTimedSensorNumber) >= NUMBER_OF_TIMED_SENSORS)
				{
					gTimedSensorNumber = 0;
				}
			}


//gTimedSensorNumber = 0;

			// Start the next UltraSonic ping
			switch( gTimedSensorNumber )
			{
				case SENSOR_US_0:
				{
					output_high( PIN_US_TRIGGER0 );
					delay_us(20);
					output_low( PIN_US_TRIGGER0 );	// Start the sonic burst!
					break;
				}
				case SENSOR_US_1:
				{
					output_high( PIN_US_TRIGGER1 );
					delay_us(20);
					output_low( PIN_US_TRIGGER1 );	// Start the sonic burst!
					break;
				}
				case SENSOR_US_2:
				{
					output_high( PIN_US_TRIGGER2 );
					delay_us(20);
					output_low( PIN_US_TRIGGER2 );	// Start the sonic burst!
					break;
				}
				default:
				{
					// Error - reset to first sensor
					gTimedSensorNumber = 0;
					gSensorTimingState = IDLE;
					return;
				}
			}


			// For Ultrasonic Sensors, wait for the Initial PING to be sent
			// This loops about 800us max if US module not plugged in.
			// Note: SRF04 triggers within 300us, but SRF05 triggers can take up to 800us!
			gSensorTimingState = IDLE;
			for(i=0; i<100; i++) // Calibrated at 850us, just under 1ms limit!
			{
				// don't start measuring until the echo pulse goes high
				if( 1 == input(PIN_US_RETURN) )
				{
					gSensorTimingState = SENSOR_WAITING_FOR_US_ECHO; // Only moves to this state if ranger is working
					//gUS_Timer = 0;	// Start the counter

					// Ok, the Pulse went high, look for the falling edge
					setup_ccp1(CCP_CAPTURE_FE);	// Look for Falling Edge
					CCP_1=0;
					set_timer1(0);	// Resets the timer to zero!
					break;
				}
				else
				{
					delay_us(4);
				}
			}

			if( gSensorTimingState != SENSOR_WAITING_FOR_US_ECHO )
			{
				// Indicate Bad sensor! (probably not attached)
				gUS_Range[gTimedSensorNumber] = HW_SENSOR_NOT_ATTACHED; //253

				// Go to the next one!
				gSensorTimingState = IDLE;
				if( (++gTimedSensorNumber) >= NUMBER_OF_TIMED_SENSORS)
				{
					gTimedSensorNumber = 0;
				}
			}
		}
#endif  // ifdef ULTRASONIC_SRF04

//	delay_us(10);
//	output_low( DEBUG_SCOPE );
	}
	//---------------------------------------------------------------------------
	else if( 4 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_high( DEBUG_SCOPE );
		#endif
/***
		// 20ms Timers used for delays in main program. Each counts down to zero then stops.
		//if( g20msTimer != 0 )	// left here as an example
		//{
		//	g20msTimer--;
		//}

***/
	}
	//---------------------------------------------------------------------------
	else if( 5 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_low( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin)
		#endif

		// Get status of Remote Control
		/***
		if( input( PIN_RC_KILL_SW ) )	// Indicates Motor Power Enabled
		{
			gStatus_StatusFlags |= HW_STATUS_RC_BUTTON_PWR_ENABLE;	// Set Flag
		}
		else
		{
			gStatus_StatusFlags &= (0xFF^HW_STATUS_RC_BUTTON_PWR_ENABLE); // Clear Flag
		}
		if( input( PIN_RC_BUTTON2 ) )
		{
			gStatus_StatusFlags |= HW_STATUS_RC_BUTTON_2;	// Set Flag
		}
		else
		{
			gStatus_StatusFlags &= (0xFF^HW_STATUS_RC_BUTTON_2); // Clear Flag
		}
		***/

	}
	//---------------------------------------------------------------------------
	else if( 6 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_high( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin)
		#endif

		// Calculate speed and distance every so often (once every "N" * 20ms)
		// See bottom of this fuction for value
		if( gMotorSpeed_Timer != 0 )
		{
			gMotorSpeed_Timer--;
		}
		else
		{
			// Timer has expired!  Calculate Tach, Odometer, and move distance
			// Or Arm position!

			// Update Odometer readings
			gStatus_MotorDir_L	= input(PIN_MOTOR_DIR_L);	// TODO-LOKI - one of these is reversed!
			gStatus_MotorDir_R	= input(PIN_MOTOR_DIR_R);

			// Left Motor
			if( 1 == gStatus_MotorDir_L )
			{
				// Motor moving Forward
				gStatus_OdometerTickCount_L += TempMotorTickCount_L;
			}
			else
			{
				// Motor moving Backward
				gStatus_OdometerTickCount_L -= TempMotorTickCount_L;	// ok to go negative
			}
			TempMotorTickCount_L = 0;	// restart counter for next measurement cycle

			// Right Motor
			if( 0 == gStatus_MotorDir_R ) // Reversed Motor!
			{
				// Motor moving Forward
				gStatus_OdometerTickCount_R += TempMotorTickCount_R;
			}
			else
			{
				// Motor moving Backward
				gStatus_OdometerTickCount_R -= TempMotorTickCount_R;	// ok to go negative
			}
			TempMotorTickCount_R = 0;	// restart counter for next measurement cycle


			// Now, reset the timer for the next measurement
			gMotorSpeed_Timer = 10;	// 10 * 20ms = 200ms = 5 times / sec
		}
	}
	//---------------------------------------------------------------------------
	else if( 7 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_low( DEBUG_SCOPE );
		#endif

		// Handle Slow Servo Movements Here
         /***
		if( gCameraPanTilt_Timer != 0 )	// Speed of Pan and Tilt
		{
			gCameraPanTilt_Timer--;
		}
		else
		{
			// Timer has expired!  Move servos one tick, if requested
			if( gCameraPanRight && (CameraPanServo < CAMERA_PAN_SERVO_MAX_RIGHT) )
			{
				CameraPanServo++;
			}
			else if( gCameraPanLeft && (CameraPanServo > CAMERA_PAN_SERVO_MAX_LEFT) )
			{
				CameraPanServo--;
			}

			if( gCameraTiltUp  && (CameraTiltServo < CAMERA_TILT_SERVO_MAX_UP) )
			{
				CameraTiltServo++;
			}
			else if( gCameraTiltDown  && (CameraTiltServo > CAMERA_TILT_SERVO_MAX_DOWN) )
			{
				CameraTiltServo--;
			}

			gCameraPanTilt_Timer = gCameraPanTiltSpeed;	// Reset timer
		}
         ***/

	}
	//---------------------------------------------------------------------------
	else if( 8 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_high( DEBUG_SCOPE );
		#endif

		// Do slow timers in this time slot

		// Check Power State Timer
		// Used for moving the Laptop Servo (and other stuff?)
		if( gPowerStateTimer != 0 )
		{
			gPowerStateTimer--;
		}
		else
		{
			// Timer has expired!
			if( (IDLE == gRobotPowerState) || (POWER_STATE_MONITORING_SENSORS == gRobotPowerState) )
			{
				// Done with whatever it was doing, so turn servo off
				LaptopPowerServo = SERVO_OFF;
			}
			else if( POWER_STATE_PRESSING_POWER_BUTTON == gRobotPowerState)
			{
				// Was Pressing the Power Button.  Move servo back to up position
				LaptopPowerServo = LAPTOP_POWER_SERVO_UP_POSITION;
				gRobotPowerState = POWER_STATE_RELEASING_POWER_BUTTON;
				gPowerStateTimer = POWER_BUTTON_SERVO_MOVE_TIME;
			}
			else if( POWER_STATE_RELEASING_POWER_BUTTON == gRobotPowerState)
			{
				// Was Releasing the Power Button.  Go to next state
				if( (LAPTOP_POWER_MODE_DEFAULT == gRobotPowerMode) ||
					(LAPTOP_POWER_OFF_NO_RESTART == gRobotPowerMode) )
				{
					gRobotPowerState = IDLE;
					LaptopPowerServo = SERVO_OFF;
				}
				else
				{
					//case LAPTOP_POWER_OFF_RESTART_ON_ANY_SENSOR:
					//case LAPTOP_POWER_OFF_RESTART_ON_PIR_SENSOR:
					//case LAPTOP_POWER_OFF_RESTART_NO_PIR_SENSOR:
					//case LAPTOP_POWER_OFF_RESTART_ON_TIMER:
					gRobotPowerState = POWER_STATE_MONITORING_SENSORS;
				}
			}
		} // gPowerStateTimer



		// Count down the "Brake" timer.
		// this is how long reverse is applied for Brake action
		/***
		if( gBrakeTimer != 0 )
		{
			gBrakeTimer--;
			if( gBrakeTimer == 0 )
			{
				// Timer has expired
				SpeedServo = PIC_SPEED_STOP;
				DesiredSpeed = SpeedServo;
				gTargetSpeedTickCount = 0;
				gStatus_StatusFlags |= HW_STATUS_BRAKE_COMPLETE; // Set Flag
			}
		}
		***/

		// Count down the "Watchdog" timer.
		// this is max time between messages from the PC
/*** TODO-CAR		if( gWatchDogTimer != 0 )
		{
			gWatchDogTimer--;
			if( gWatchDogTimer == 0 )
			{
				// Timer has expired
				StopMotor();
				gStatus_StatusFlags |=  HW_STATUS_WATCHDOG_EXPIRED; // Set Flag
				output_low( PIN_LED_R );	// Turn On Error LED
				LedState_Red = 0;				// Light the Error LED
			}
		}
***/
	}
	//---------------------------------------------------------------------------
	else if( 9 == g20msTimer )
	{

		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_low( DEBUG_SCOPE );	// Show the cycle we are in by pulsing Scope Probe pin)
		#endif
		// Do other timing stuff here
	}

	//---------------------------------------------------------------------------
	else if( 14 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_high( DEBUG_SCOPE );
		#endif

		// This is a good time for the main loop to read the Compass or IR sensors
		// MS 13 is used by Ultrasonics, but MS 14-19 are "dead time"
		gTimeToReadSensors = TRUE;

	}
	//---------------------------------------------------------------------------
	else if( 15 == g20msTimer )
	{
		#ifdef DEBUG_SCOPE_20MS_TIMING
			output_low( DEBUG_SCOPE );
		#endif
	}


	//---------------------------------------------------------------------------
	// End of 20ms Cycle check
	//---------------------------------------------------------------------------

	// Since we were in an ISR, we might have missed SIO Interrupt, so check for data
	CheckForSerialData();

	// Update the 20ms count timer.
	if ( g20msTimer < TIME_20_MS )
	{
		g20msTimer++;			// Increment the 20ms clock
	}
	else
	{
		g20msTimer = 0;			// Reset 20ms timer
	}

}

//***************************************************************************
//	Default Interrupt handler - Called for interrupts that there is no handler for!
//
#int_default
default_isr()
{
	//printf("BAD ISR!\r\n");
	gStatus_LastError = HW_BAD_ISR;	// Report Critical Error!
//	output_low( PIN_LED_R );	// Turn On Error LED
//	LedState_Red = 0;
}
